@@ -1,5 +1,5 @@
 /*
- * $Id: SND_OpenALDevice.cpp,v 1.12 2004/04/17 01:14:35 kester Exp $
+ * $Id: SND_OpenALDevice.cpp,v 1.25 2006/01/29 21:29:14 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,10 +31,6 @@
  * SND_OpenALDevice derived from SND_IAudioDevice
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 #pragma warning (disable:4786) // get rid of stupid stl-visual compiler debug warning
 #endif //WIN32
@@ -60,6 +56,8 @@
 #endif
 #include <fcntl.h>
 
+#include <signal.h>
+
 /* untill openal gets unified we need this hack for non-windows systems */
 #if !defined(WIN32) && !defined(ALC_MAJOR_VERSION)
 
@@ -202,9 +200,15 @@ ALvoid alutUnloadWAV(ALenum format,ALvoid *data,ALsizei size,ALsizei freq)
 
 #endif /* WIN32 */
 
+#ifdef __APPLE__
+#define OUDE_OPENAL 1
+#endif
 
 
 SND_OpenALDevice::SND_OpenALDevice()
+	: SND_AudioDevice(),
+	  m_context(NULL),
+	  m_device(NULL)
 {
     /* Removed the functionality for checking if noaudio was provided on */
     /* the commandline. */
@@ -227,8 +231,27 @@ SND_OpenALDevice::SND_OpenALDevice()
 			m_context = alcCreateContext(dev, NULL);
 
 			if (m_context) {
-				alcMakeContextCurrent(m_context);
-				m_audio = true;
+#ifdef AL_VERSION_1_1
+		alcMakeContextCurrent((ALCcontext*)m_context);
+#else
+			alcMakeContextCurrent(m_context);
+#endif
+			m_audio = true;
+				m_device = dev;
+#ifdef __linux__
+				/*
+				*   SIGHUP Hack:
+				*
+				*   On Linux, alcDestroyContext generates a SIGHUP (Hangup) when killing the OpenAL
+				*   mixer thread, which kills Blender.
+				*  
+				*   So we set the signal to ignore....
+				*
+				*   TODO: check if this applies to other platforms.
+				*
+				*/
+				signal(SIGHUP, SIG_IGN);
+#endif
 			}
 		}
 
@@ -301,16 +324,37 @@ void SND_OpenALDevice::MakeCurrent() const
 
 SND_OpenALDevice::~SND_OpenALDevice()
 {
-	if (m_context) {
-		alcMakeContextCurrent(m_context);
-
-		if (m_buffersinitialized)
-			alDeleteBuffers(NUM_BUFFERS, m_buffers);
-
-		if (m_sourcesinitialized)
-			alDeleteSources(NUM_SOURCES, m_sources);
+	MakeCurrent();
+	
+	if (m_buffersinitialized)
+	{
+		alDeleteBuffers(NUM_BUFFERS, m_buffers);
+		m_buffersinitialized = false;
 	}
 	
+	if (m_sourcesinitialized)
+	{
+		for (int i = 0; i < NUM_SOURCES; i++)
+			alSourceStop(m_sources[i]);
+		
+		alDeleteSources(NUM_SOURCES, m_sources);
+		m_sourcesinitialized = false;
+	}
+	
+	if (m_context) {
+		MakeCurrent();
+#ifdef AL_VERSION_1_1
+		alcDestroyContext((ALCcontext*)m_context);
+#else
+		alcDestroyContext(m_context);
+#endif
+		m_context = NULL;
+	}
+	
+#ifdef __linux__
+	// restore the signal state above.
+	signal(SIGHUP, SIG_DFL);
+#endif	
 	// let's see if we used the cd. if not, just leave it alone
 	SND_CDObject* pCD = SND_CDObject::Instance();
 	
@@ -323,6 +367,13 @@ SND_OpenALDevice::~SND_OpenALDevice()
 	if (m_cdrom)
 		delete m_cdrom;
 #endif
+#ifdef OUDE_OPENAL
+	if (m_audio)
+		alutExit();
+#else
+	if (m_device)
+		alcCloseDevice((ALCdevice*) m_device);
+#endif
 }
 
 
@@ -346,16 +397,21 @@ SND_WaveSlot* SND_OpenALDevice::LoadSample(const STR_String& name,
 			{
 				int buffer = waveslot->GetBuffer();
 				void* data = NULL;
+#ifndef __APPLE__
 				char loop = 'a';
+#endif
 				int sampleformat, bitrate, numberofchannels;
 				ALenum al_error = alGetError();
 				
 #ifdef OUDE_OPENAL
-				unsigned int samplerate, numberofsamples;		// openal_2.12
+				ALsizei samplerate, numberofsamples;		// openal_2.12
 #else
-				int samplerate, numberofsamples, frequency;  // openal_2.14+
+				int samplerate, numberofsamples;  // openal_2.14+
 #endif
 				
+				/* Give them some safe defaults just incase */
+				bitrate = numberofchannels = 0;
+
 				/* load the sample from memory? */
 				if (size && memlocation)
 				{
@@ -373,7 +429,12 @@ SND_WaveSlot* SND_OpenALDevice::LoadSample(const STR_String& name,
 #if defined(OUDE_OPENAL) || defined (__APPLE__)
 					alutLoadWAVMemory((char*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate);				//	openal_2.12
 #else
-					alutLoadWAVMemory((signed char*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate, &loop);//	openal_2.14+
+#ifdef AL_VERSION_1_1					
+					alutLoadWAVMemory((ALbyte*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate, &loop);//	openal_2.14+
+#else
+					 alutLoadWAVMemory((signed char*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate, &loop);//  openal_2.14+
+					 
+#endif 
 #endif
 					/* put it in the buffer */
 					alBufferData(m_buffers[buffer], sampleformat, data, numberofsamples, samplerate);
@@ -382,13 +443,9 @@ SND_WaveSlot* SND_OpenALDevice::LoadSample(const STR_String& name,
 				else
 				{
 #ifdef __APPLE__
-					alutLoadWAVFile((signed char*)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate);
-#elif defined(WIN32)
-					alutLoadWAVFile((signed char*)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate, &loop);
+					alutLoadWAVFile((ALbyte *)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate);
 #else
-					alutLoadWAV((char*)samplename.Ptr(), &data,
-						&sampleformat, &numberofsamples,
-						&samplerate, &frequency);
+					alutLoadWAVFile((ALbyte *)samplename.Ptr(), &sampleformat, &data, &numberofsamples, &samplerate, &loop);
 #endif
 					/* put it in the buffer */
 					alBufferData(m_buffers[buffer], sampleformat, data, numberofsamples, samplerate);
@@ -570,7 +627,7 @@ void SND_OpenALDevice::StopObject(int id) const
 
 	alSourcefv(m_sources[id], AL_POSITION, obpos);
 	alSourcefv(m_sources[id], AL_VELOCITY, obvel);
-	
+
 	alSourcef(m_sources[id], AL_GAIN, 1.0);
 	alSourcef(m_sources[id], AL_PITCH, 1.0);
 	alSourcei(m_sources[id], AL_LOOPING, AL_FALSE);
