@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowCarbon.cpp,v 1.15 2005/06/08 18:54:17 ton Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.17 2006/06/19 04:04:31 broken Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_WindowCarbon.cpp,v 1.15 2005/06/08 18:54:17 ton Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.17 2006/06/19 04:04:31 broken Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 10, 2001
@@ -182,7 +182,7 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
             (SInt32)this);					// Store a pointer to the class in the refCon
     */
         //GHOST_PRINT("GHOST_WindowCarbon::GHOST_WindowCarbon(): creating full-screen OpenGL context\n");
-        setDrawingContextType(GHOST_kDrawingContextTypeOpenGL);
+        setDrawingContextType(GHOST_kDrawingContextTypeOpenGL);;installDrawingContext(GHOST_kDrawingContextTypeOpenGL);
         updateDrawingContext();
         activateDrawingContext();        
     }
@@ -477,7 +477,7 @@ GHOST_TSuccess GHOST_WindowCarbon::installDrawingContext(GHOST_TDrawingContextTy
             }
             else {
                 //GHOST_PRINT("GHOST_WindowCarbon::installDrawingContext(): init full-screen OpenGL\n");
-                pixelFormat = ::aglChoosePixelFormat(0, 0, sPreferredFormatFullScreen);
+GDHandle device=::GetMainDevice();pixelFormat=::aglChoosePixelFormat(&device,1,sPreferredFormatFullScreen);
                 m_aglCtx = ::aglCreateContext(pixelFormat, 0);
                 if (!m_aglCtx) break;
 				if (!s_firstaglCtx) s_firstaglCtx = m_aglCtx;
@@ -690,8 +690,14 @@ GHOST_TSuccess GHOST_WindowCarbon::setWindowCustomCursorShape(GHOST_TUns8 *bitma
 	if (!m_customCursor) return GHOST_kFailure;
 	
 	for (y=0; y<16; y++) {
+#if !defined(__LITTLE_ENDIAN__)
 		m_customCursor->data[y] = uns16ReverseBits((bitmap[2*y]<<0) | (bitmap[2*y+1]<<8));
 		m_customCursor->mask[y] = uns16ReverseBits((mask[2*y]<<0) | (mask[2*y+1]<<8));
+#else
+		m_customCursor->data[y] = uns16ReverseBits((bitmap[2*y+1]<<0) | (bitmap[2*y]<<8));
+		m_customCursor->mask[y] = uns16ReverseBits((mask[2*y+1]<<0) | (mask[2*y]<<8));
+#endif
+			
 	}
 	
 	m_customCursor->hotSpot.h = hotX;
