@@ -2,7 +2,7 @@
 
 """ Registration info for Blender menus:
 Name: 'Texture Baker'
-Blender: 237
+Blender: 239
 Group: 'UV'
 Tooltip: 'Procedural to uvmapped texture baker'
 """
@@ -11,10 +11,11 @@ __author__ = "Jean-Michel Soler (jms)"
 __url__ = ("blender", "elysiun",
 "Official Page, http://jmsoler.free.fr/didacticiel/blender/tutor/cpl_mesh3d2uv2d_en.htm",
 "Communicate problems and errors, http://www.zoo-logique.org/3D.Blender/newsportal/thread.php?group=3D.Blender")
-__version__ = "0.2.6 2005/5/29"
+__version__ = "0.3.2 2005/12/28"
 
 __bpydoc__ = """\
-Texture Baker "bakes" Blender procedural materials (including textures): it saves them as 2d uv-mapped images.
+This script "bakes" Blender procedural materials (including textures): it saves
+them as 2d uv-mapped images.
 
 This script saves an uv texture layout of the chosen mesh, that can be used as
 an uv map for it.  It is a way to export procedurals from Blender as normal
@@ -23,21 +24,22 @@ with the mesh in games and other 3d applications.
 
 Usage:
 
-a) Enter face mode and define uv coordinates for your mesh (do not forget to choose a development shape);<br>
-b) Define its materials and textures;<br>
+a) Enter face mode and define uv coordinates for your mesh;<br>
+b) Define its materials and textures ;
 c) Run this script and check the console.
 
-Global variables:
+Global variables 
 
-a) FRAME (integer): the last frame of the animation, autodocumented.<br>
-b) LIMIT (integer): 0 or 1, uvcoords may exceed limits 0.0 to 1.0, this variable obliges the script to do a complete framing of the uvcoord.
+a)  FRAME  integer, the last frame of the animation, autodocumented .
+b)  LIMIT  integer, 0 or 1, uvcoords may exceed limits 0.0 to 1.0 , this variable
+obliges the script to do a complete framing of the uvcoord . 
 
 Notes:<br>
-   This script was based on a suggestion by Martin (Theeth) Poirier.
+   This script was based on a suggestion by Martin (Theeth) Poirier;<br>
 """
 
 #---------------------------------------------
-# Last release : 0.2.6 ,  2005/05/29 , 22h00
+# Last release : 0.3.1 ,  2005/10/21 , 20h23
 #---------------------------------------------
 #---------------------------------------------
 # (c) jm soler  07/2004 : 'Procedural Texture Baker'
@@ -47,6 +49,40 @@ Notes:<br>
 #
 #     Released under Blender Artistic Licence
 #
+#   0.3.2
+#     blender 2.40 update to deal with the new shape  
+#     key system . 
+#
+#   0.3.1 
+#     stupid bug correction    
+#     
+#   0.3.0
+#      TAILLEIMAGE variable 
+# 
+#   0.2.9
+#      -- little probleme with the KEEPRENDERWINDOW variable .
+#      removed . script seems to works correctly now .
+# 
+#   0.2.8
+#       -- added the forgotten image property in face
+#       data. a little longer but better.
+#       ( a remove double in the resulting mesh may be
+#       useful .) 
+#       -- the data.update() function problem is 
+#       corrected too
+#       -- no more layers problem . CAM and MESH are
+#       localised in layer 20 . This layer is
+#       the active one for the image rendering .
+#       -- mesh creation is cleaner, loop in double was
+#       removed and the abskey is set in frame 1 
+#       only  . This solves an other deform problem  .
+#       -- if user does not want an autosaved image,
+#       the "no replace" option leaves the render 
+#       window on the screen 
+#
+#   0.2.7 
+#      -- minor correction on line 147: "!=-1" added 
+#
 #   0.2.6
 #       -- Creation of LAMP object is removed and replaced
 #       by the use of the shadeless option  in material object
@@ -142,7 +178,7 @@ DIRNAME=Blender.Get('filename')
 # the file name from the path name
 #----------------------------------- 
 
-if DIRNAME.find(os.sep):
+if DIRNAME.find(os.sep)!=-1:
     k0=DIRNAME.split(os.sep)
 else:
     k0=DIRNAME.split('/')   
@@ -178,6 +214,10 @@ LIMIT=0
 XYLIMIT = [0.0, 0.0,1.0,1.0]    
 OBJPOS = 100.0
 DEBUG=1
+RENDERLAYER=20
+SCENELAYERS=[]
+
+
 
 helpmsg = """
 Texture Baker:
@@ -228,13 +268,14 @@ def RenameImage(RDIR, MYDIR, FILENAME, name):
     """
     newfname = RDIR + MYDIR + name
     if newfname.find('.png', -4) < 0 : newfname += '.png'
+	
     if not Blender.sys.exists(newfname):
        os.rename(FILENAME, newfname)
     else:
         name = Draw.PupStrInput ('ReName Image, please :', name, 32)
         RenameImage(RDIR, MYDIR, FILENAME, name)
 
-def SAVE_image (rc, name, FRAME):
+def SAVE_image (rc, name, FRAME, result):
    """  
 # ---------------------------
 # Function  SAVE_image
@@ -268,26 +309,25 @@ def SAVE_image (rc, name, FRAME):
    rc.startFrame(NEWFRAME)
    rc.endFrame(NEWFRAME)
    rc.renderAnim()
-   Blender.Scene.Render.CloseRenderWindow()
-
-   FILENAME = "%04d" % NEWFRAME
-   FILENAME = FILENAME.replace (' ', '0')
-   FILENAME = RDIR + MYDIR + FILENAME + '.png'
-
-   RenameImage(RDIR, MYDIR, FILENAME, name)
+   if result!=2 : 
+      Blender.Scene.Render.CloseRenderWindow()
+      FILENAME = "%04d" % NEWFRAME
+      FILENAME = FILENAME.replace (' ', '0')
+      FILENAME = RDIR + MYDIR + FILENAME + '.png'
+      RenameImage(RDIR, MYDIR, FILENAME, name)
 
    rc.endFrame(OLDEFRAME)
    rc.startFrame(OLDSFRAME)
    rc.setRenderPath(RENDERDIR)
 
-def SHOOT (XYlimit, frame, obj, name, FRAME):
+def SHOOT (XYlimit, frame, obj, name, FRAME, result):
    """
 # ---------------------------
 # Function  SHOOT
 #
 #  IN : XYlimit  list of 4 floats, smallest and biggest 
 #                uvcoords
-#       frame    cureente frame
+#       frame    current frame
 #       obj      for object location
 #       name     image name
 #       FRAME    the last animation's frame 
@@ -305,6 +345,7 @@ def SHOOT (XYlimit, frame, obj, name, FRAME):
       CAM, SC = GET_newobject('Camera','UVCAMERA')
       CAM.link(Cam)
       CAM.setName('UVCAMERA')
+      CAM.layers=[RENDERLAYER]
       Cam.lens = 30
       Cam.name = 'UVCamera'
 
@@ -324,22 +365,34 @@ def SHOOT (XYlimit, frame, obj, name, FRAME):
    OLDy = context.imageSizeY()
    OLDx = context.imageSizeX()
 
-   tres = Draw.PupMenu('TEXTURE OUT RESOLUTION : %t | 256 %x1 | 512 %x2 | 768 %x3 | 1024 %x4 | 2048 %x5 ')
+   TAILLEIMAGE='TEXTURE OUT RESOLUTION : %t |'
+   TAILLEIMAGE+='256 %x1 |'
+   TAILLEIMAGE+='512 %x2 |'
+   TAILLEIMAGE+='768 %x3 |'
+   TAILLEIMAGE+='1024 %x4 |'
+   TAILLEIMAGE+='2048 %x5 '
+   #TAILLEIMAGE+='| 4096 %x6 ' 
+   tres = Draw.PupMenu(TAILLEIMAGE)
 
    if (tres) == 1: res = 256
    elif (tres) == 2: res = 512
    elif (tres) == 3: res = 768
    elif (tres) == 4: res = 1024
    elif (tres) == 5: res = 2048
+   # elif (tres) == 6: res = 4096
    else: res = 512
+   #...
+ 
 
+   SCENELAYERS=SC.layers
+   SC.layers = [20]
    context.imageSizeY(res)
    context.imageSizeX(res)
-   SAVE_image (context, name, FRAME)
+   SAVE_image (context, name, FRAME, result)
    context.imageSizeY(OLDy)
    context.imageSizeX(OLDx)
-
-   if Camold :SC.setCurrentCamera(Camold)
+   SC.layers = SCENELAYERS
+   if Camold : SC.setCurrentCamera(Camold)
 
    Blender.Set ('curframe', frame)
 
@@ -384,6 +437,85 @@ def REST_Shadeless(SHADEDict):
 # release : 0.2.6 ,  2005/05/29 , end
 #----------------------------------- 
 
+
+#----------------------------------- 
+# release : 0.3.2 ,  2005/12/28 , 13h00
+#----------------------------------- 
+def Blender240update(MESH2,FRAME):
+     """ 
+# ---------------------------
+# Function  Blender240update
+#        
+#  IN : MESH2   a mesh data bloc
+#       FRAME , the animation frame limit
+#        
+#  ADD : an ipo curve to the shape key
+#        named "Key 1" 
+#        
+#  OUT : nothing
+# ---------------------------   
+     """
+     # ---------------------------   
+     #  recuperation des clef de morphing pour ce mesh
+     # ---------------------------   
+     key = MESH2.getKey()
+     # ---------------------------   
+     #  recuperation de l'Ipo
+     # ---------------------------   
+     ipo = key.ipo
+     # ---------------------------   
+     #  si l'ipo n'existe pas on la cree 
+     # ---------------------------   
+     if ipo == None:
+        noipo = Blender.Ipo.New("Key","keyipo")
+        key.ipo = noipo
+     # ---------------------------   
+     #   raccourci de l'expression
+     # ---------------------------   
+     ipo = key.ipo
+     # ---------------------------   
+     #   identification de la clef de morphing
+     # ---------------------------   
+     keyidentity = "Key 1"
+     # ---------------------------   
+     #   recuperation de la courbe correspondante
+     #  c'est toujours la courbe 0
+     # ---------------------------   
+     ipocurve = ipo.getCurve(0)
+     # ---------------------------   
+     #  si la courbe n'existe pas (normalement, elle n'existe pas mais
+     # on gre le risque pour faciliter une eventuelle rcupration de
+     # cette fonction dans un autre script ou pour les cas , certe peu
+     # probable, ou blender viendrait a etre modifie pour les ajouter 
+     # automatiquement ) on la cree ...
+     # ---------------------------   
+     if ipocurve == None:
+        ipocurve = ipo.addCurve(keyidentity)
+     # ---------------------------   
+     #  On applique l'attribut d'inetrpolation qui permet d'avoir
+     # une ligne droite
+     # ---------------------------   
+     ipocurve.setInterpolation("Linear")
+     # ---------------------------   
+     #  On retire tous les sommets qui pourraient se trouver sur la 
+     #  courbe (dans l'tat actuel, cette opration est une scurit 
+     #  superflue ) .
+     # ---------------------------   
+     while len(ipocurve.getPoints()) > 0:
+        ipocurve.delBezier(0)
+        ipocurve.recalc()
+     # ---------------------------   
+     #  On ajouter les sommets necessaires ...
+     # ---------------------------   
+     ipocurve.addBezier((-1,1))
+     # ---------------------------   
+     #   ... ce dernire n'est peut-tre pas absolument obligatoire .
+     # ---------------------------   
+     ipocurve.addBezier((FRAME+1,1))
+#----------------------------------- 
+# release : 0.3.2 ,  2005/12/28 , end
+#----------------------------------- 
+         
 def Mesh2UVCoord (LIMIT):
    """
 # ---------------------------
@@ -393,6 +525,8 @@ def Mesh2UVCoord (LIMIT):
 #  OUT:  nothing
 # ---------------------------   
    """
+   global PUTRAW, FRAME, SCENELAYERS
+
    try:
       MESH3D = Object.GetSelected()[0]
       if MESH3D.getType() == 'Mesh':
@@ -401,11 +535,11 @@ def Mesh2UVCoord (LIMIT):
          try:
             NewOBJECT=Blender.Object.Get('UVOBJECT')
             CurSCENE=Blender.Scene.getCurrent()            
-            MESH2 = NewOBJECT.getData()
-            
          except:
             NewOBJECT, CurSCENE = GET_newobject('Mesh','UVOBJECT')
-            MESH2 = Blender.NMesh.GetRaw()
+         MESH2 = NewOBJECT.getData()
+         MESH2.edges=[] 
+         NewOBJECT.layers=[RENDERLAYER]
 
          MESH2.faces=[]
          for f in MESH.faces:
@@ -423,18 +557,24 @@ def Mesh2UVCoord (LIMIT):
             f1.mode = f.mode
             f1.flag = f.flag
             f1.mat = f.mat
+            #----------------------------------- 
+            # release : 0.2.8 ,  2005/07/19 , end
+            #----------------------------------- 
+            try:
+              f1.image=f.image
+            except :
+              pass
 
          MESH2.materials = MESH.materials[:]
 
          NewOBJECT.setLocation (OBJPOS, OBJPOS, 0.0)
          NewOBJECT.setEuler (0.0, 0.0, 0.0)
-
          MESH2.removeAllKeys()
 
          MESH2.update()
          MESH2.insertKey (1, 'absolute')
          MESH2.update()
-
+  
          for f in MESH2.faces:
             for v in f.v:
                for n in [0,1]:
@@ -443,15 +583,23 @@ def Mesh2UVCoord (LIMIT):
                   exec "if v.co[%s] < XYLIMIT[%s]: XYLIMIT[%s] = v.co[%s]" % (n, n, n, n)
                v.co[2] = 0.0
 
-         if DEBUG: print XYLIMIT
+         print XYLIMIT
 
          MESH2.update()
          MESH2.insertKey (FRAME, 'absolute')
          MESH2.update()
 
+         #----------------------------------- 
+         # release : 0.3.2 ,  2005/12/28 , 13h00
+         #-----------------------------------            
+         Blender240update(MESH2,FRAME)       
+         #----------------------------------- 
+         # release : 0.3.2 ,  2005/12/28 , end
+         #-----------------------------------            
+         
          imagename = 'uvtext'
 
-         name = "CHANGE IMAGE NAME ? %t | Replace it | No replacing | Script help"
+         name = "CHANGE IMAGE NAME ? %t | Replace it | No replace | Script help"
          result = Draw.PupMenu(name)
 
          if result == 1:
@@ -467,9 +615,9 @@ def Mesh2UVCoord (LIMIT):
             #----------------------------------- 
 
             if LIMIT :
-                 SHOOT(XYLIMIT, FRAME, NewOBJECT, imagename, FRAME)
+                 SHOOT(XYLIMIT, FRAME, NewOBJECT, imagename, FRAME,result)
             else :
-                 SHOOT([0.0,0.0,1.0,1.0], FRAME, NewOBJECT, imagename, FRAME)
+                 SHOOT([0.0,0.0,1.0,1.0], FRAME, NewOBJECT, imagename, FRAME, result)
             #----------------------------------- 
             # release : 0.2.6,  2005/05/29 , 00h00
             #----------------------------------- 
@@ -481,18 +629,17 @@ def Mesh2UVCoord (LIMIT):
             Blender.Redraw()
 
          else:
-            Blender.ShowHelp('tex2uvbaker.py')
-            #Draw.PupMenu("Ready%t|Please check console for instructions")
-            if DEBUG: print helpmsg
+            Draw.PupMenu("Ready%t|Please check console for instructions")
+            print helpmsg
 
       else:
-         name = "ERROR: active object is not a mesh or has no UV coordinates"
+         name = "Error%t|Active object is not a mesh or has no UV coordinates"
          result = Draw.PupMenu(name)
          print 'problem : no object selected or not mesh'
 
    except:
-      name = "ERROR: active object is not a mesh or has no UV coordinates"
+      name = "Error%t|Active object is not a mesh or has no UV coordinates"
       result = Draw.PupMenu(name)
       print 'problem : no object selected or not mesh'
 
-Mesh2UVCoord(LIMIT)
+Mesh2UVCoord(LIMIT) 
\ No newline at end of file
