@@ -4,7 +4,7 @@
  * Stuff that definitely does not belong in the kernel! These will all
  * have to be removed in order to restore sanity.
  *
- * $Id: BKE_bad_level_calls.h,v 1.13 2005/05/08 21:20:32 ianwill Exp $ 
+ * $Id: BKE_bad_level_calls.h,v 1.24 2006/06/12 00:10:00 theeth Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,9 +37,12 @@
 #ifndef BKE_BAD_LEVEL_CALLS_H
 #define BKE_BAD_LEVEL_CALLS_H
 
+/* blender.c */
+void freeAllRad(void);
+void free_editText(void);
+void free_vertexpaint(void);
+
 /* readfile.c */
-struct PluginSeq;
-void open_plugin_seq(struct PluginSeq *pis, char *seqname);
 struct SpaceButs;
 void set_rects_butspace(struct SpaceButs *buts);
 struct SpaceImaSel;
@@ -47,8 +50,6 @@ void check_imasel_copy(struct SpaceImaSel *simasel);
 struct ScrArea;
 struct bScreen;
 void unlink_screen(struct bScreen *sc);
-void freeAllRad(void);
-void free_editText(void);
 void setscreen(struct bScreen *sc);
 void force_draw_all(int);
   /* otherwise the WHILE_SEQ doesn't work */
@@ -56,13 +57,22 @@ struct Sequence;
 struct ListBase;
 void build_seqar(struct ListBase *seqbase, struct Sequence  ***seqar, int *totseq);
 
+/* BPython API */
 struct ID;
 struct Script;
 struct Text;
+struct IpoDriver; /* DNA_curve_types.h */
+struct Object;
 void BPY_do_pyscript (struct ID *id, short int event);
 void BPY_clear_script (struct Script *script);
 void BPY_free_compiled_text (struct Text *text);
 void BPY_free_screen_spacehandlers (struct bScreen *sc);
+/* pydrivers */
+struct Object **BPY_pydriver_get_objects(struct IpoDriver *driver);
+float BPY_pydriver_eval(struct IpoDriver *driver);
+void BPY_pydriver_update(void);
+/* button python evaluation */
+int BPY_button_eval(char *expr, double *value);
 
 /* writefile.c */
 struct Oops;
@@ -72,20 +82,14 @@ void error(char *str, ...);
 /* anim.c */
 extern struct ListBase editNurb;
 
-/* displist.c */
-#include "DNA_world_types.h"	/* for render_types */
-#include "render_types.h"
-extern struct RE_Render R;
-float Phong_Spec(float *, float *, float *, int);
+void mainqenter (unsigned short event, short val);
 void waitcursor(int);
 void allqueue(unsigned short event, short val);
 #define REDRAWVIEW3D	0x4010
+#define REDRAWBUTSEDIT	0x4019
 struct Material;
 extern struct Material defmaterial;
 
-/* effect.c */
-void    RE_jitterate1(float *jit1, float *jit2, int num, float rad1);
-void    RE_jitterate2(float *jit1, float *jit2, int num, float rad2);
 
 /* exotic.c */
 void load_editMesh(void);
@@ -99,7 +103,6 @@ int saveover(char *str);
 /* image.c */
 #include "DNA_image_types.h"
 void free_realtime_image(Image *ima); // has to become a callback, opengl stuff
-void RE_make_existing_file(char *name); // from render, but these funcs should be moved anyway 
 
 /* ipo.c */
 void copy_view3d_lock(short val);	// was a hack, to make scene layer ipo's possible
@@ -109,11 +112,12 @@ void allspace(unsigned short event, short val) ;
 #define OOPS_TEST             2
 
 /* mball.c */
-extern ListBase editelems;
+extern struct ListBase editelems;
 
 /* object.c */
-/*  void BPY_free_scriptlink(ScriptLink *slink); */
-/*  void BPY_copy_scriptlink(ScriptLink *scriptlink); */
+struct ScriptLink;
+void BPY_free_scriptlink(struct ScriptLink *slink);
+void BPY_copy_scriptlink(struct ScriptLink *scriptlink);
 float *give_cursor(void);  // become a callback or argument
 void exit_posemode(int freedata);
 
@@ -126,16 +130,10 @@ short pupmenu(char *instr);  // will be general callback
 /* scene.c */
 #include "DNA_sequence_types.h"
 void free_editing(struct Editing *ed);	// scenes and sequences problem...
+void BPY_do_all_scripts (short int event);
+int BPY_call_importloader(char *name);
+
 
-/* texture.c */
-#define FLO 128
-#define INT	96
-struct EnvMap;
-struct Tex;
-void    RE_free_envmap(struct EnvMap *env);      
-struct EnvMap *RE_copy_envmap(struct EnvMap *env);
-void    RE_free_envmapdata(struct EnvMap *env);
-int     RE_envmaptex(struct Tex *tex, float *texvec, float *dxt, float *dyt);
 extern char texstr[20][12];	/* buttons.c */
 
 
@@ -145,7 +143,12 @@ void make_unique_prop_names(char *str);
 /* DerivedMesh.c */
 void bglBegin(int mode);
 void bglVertex3fv(float *vec);
+void bglVertex3f(float x, float y, float z);
 void bglEnd(void);
 
+struct DispListMesh;
+struct Object;
+struct DispListMesh *NewBooleanMeshDLM(struct Object *ob, struct Object *ob_select, int int_op_type);
+
 #endif
 
