@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_effect.h (mar-2001 nzc)
  *	
- * $Id: BKE_effect.h,v 1.14 2005/04/24 20:51:47 bjornmose Exp $ 
+ * $Id: BKE_effect.h,v 1.22 2006/01/28 18:33:06 hos Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,14 +34,26 @@
 #ifndef BKE_EFFECT_H
 #define BKE_EFFECT_H
 
+#include "DNA_object_types.h"
+
 struct Effect;
 struct ListBase;
-struct Object;
 struct PartEff;
-struct MTex;
-struct Mesh;
-struct WaveEff;
 struct Particle;
+struct Group;
+
+typedef struct pEffectorCache {
+	struct pEffectorCache *next, *prev;
+	Object *ob;
+	
+	/* precalculated variables */
+	float oldloc[3], oldspeed[3];
+	float scale, time_scale;
+	float guide_dist;
+	
+	Object obcopy;	/* for restoring transformation data */
+} pEffectorCache;
+
 
 struct Effect *add_effect(int type);
 void free_effect(struct Effect *eff);
@@ -50,29 +62,17 @@ struct Effect *copy_effect(struct Effect *eff);
 void copy_act_effect(struct Object *ob);
 void copy_effects(struct ListBase *lbn, struct ListBase *lb);
 void deselectall_eff(struct Object *ob);
-void set_buildvars(struct Object *ob, int *start, int *end);
-struct Particle *new_particle(struct PartEff *paf);
 struct PartEff *give_parteff(struct Object *ob);
 void where_is_particle(struct PartEff *paf, struct Particle *pa, float ctime, float *vec);
-void particle_tex(struct MTex *mtex, struct PartEff *paf, float *co, float *no);
-void make_particle_keys(int depth, int nr, struct PartEff *paf, struct Particle *part, float *force, int deform, struct MTex *mtex, unsigned int par_layer);
-void init_mv_jit(float *jit, int num,int seed2);
 void build_particle_system(struct Object *ob);
-void calc_wave_deform(struct WaveEff *wav, float ctime, float *co);
-int object_wave(struct Object *ob);                            
 
 /* particle deflector */
 #define PE_WIND_AS_SPEED 0x00000001
 
-void pdDoEffector(float *opco, float *force, float *speed, float cur_time, unsigned int par_layer,unsigned int flags);
-int pdDoDeflection(float opco[3], float npco[3], float opno[3],
-				   float npno[3], float life, float force[3], int def_depth,
-				   float cur_time, unsigned int par_layer, int *last_object,
-				   int *last_face, int *same_face);
+struct ListBase *pdInitEffectors(struct Object *obsrc, struct Group *group);
+void			pdEndEffectors(struct ListBase *lb);
+void			pdDoEffectors(struct ListBase *lb, float *opco, float *force, float *speed, float cur_time, float loc_time, unsigned int flags);
 
-int SoftBodyDetectCollision(float opco[3], float npco[3], float colco[3],
-        float facenormal[3], float *damp, float force[3], int mode,
-        float cur_time, unsigned int par_layer,struct Object *vertexowner);
 
 
 #endif
