@@ -5,7 +5,7 @@
  * any data in Blender. This block is not serialized, but built anew
  * for every fresh Blender run.
  *
- * $Id: BKE_global.h,v 1.25 2005/06/11 16:30:36 jesterking Exp $ 
+ * $Id: BKE_global.h,v 1.39 2006/05/30 13:38:22 jiri Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -66,62 +66,72 @@ struct EditMesh;
 
 typedef struct Global {
 
-    /* active pointers */
-    struct View3D *vd;
-    struct View2D *v2d;
-    struct SpaceIpo *sipo;
-    struct SpaceButs *buts;
-    struct SpaceImage *sima;
-    struct SpaceOops *soops;
-    struct SpaceSound *ssound;
-	struct SpaceAction *saction;	/* __NLA */
+	/* active pointers */
+	struct View3D *vd;
+	struct View2D *v2d;
+	struct SpaceIpo *sipo;
+	struct SpaceButs *buts;
+	struct SpaceImage *sima;
+	struct SpaceOops *soops;
+	struct SpaceSound *ssound;
+	struct SpaceAction *saction;		/* __NLA */
 	struct SpaceNla *snla;
-    struct Main *main;
-    struct Scene *scene;				/* denk aan file.c */
-    struct bScreen *curscreen;
-    struct Object *obedit;
+	struct Main *main;
+	struct Scene *scene;			/* denk aan file.c */
+	struct bScreen *curscreen;
+	struct Object *obedit;
+	char editModeTitleExtra[64];
 	
-    /* fonts, allocated global data */
-    struct BMF_Font *font, *fonts, *fontss;
+	/* fonts, allocated global data */
+	struct BMF_Font *font, *fonts, *fontss;
     
-    /* strings: lastsaved */
-    char ima[160], sce[160], lib[160];
+	/* strings: lastsaved */
+	char ima[160], sce[160], lib[160];
+
+	/* strings of recent opend files */
+	struct ListBase recent_files;
     
-    /* totals */
-    short totobj, totlamp, totobjsel, totcurve, totmesh, totmat;
-    int totvert, totedge, totface, totvertsel, totedgesel, totfacesel;
+	/* totals */
+	short totobj, totlamp, totobjsel, totcurve, totmesh;
+	short totbone, totbonesel;
+	int totvert, totedge, totface, totvertsel, totedgesel, totfacesel;
     
-    short machine, afbreek, moving, colact, zbuf;
-    short qual, background, imagewin, animspeed;
+	short afbreek, moving;
+	short qual, background;
 	short winpos, displaymode;	/* used to be in Render */
-	
+	short rendering;			/* to indicate render is busy, prevent renderwindow events etc */
 	/**
 	 * The current version of Blender.
 	 */
-    short version;
-	short simulf, fields, order, rt;
+	short version;
+	short simulf, order, rt;
 	int f;
 
-    /* Editmode lists */
+	/* Editmode lists */
 	struct EditMesh *editMesh;
     
-    float textcurs[4][2];
+	float textcurs[4][2];
     
-    /* Frank's variables */
-    int renderd;
-    int real_sfra, real_efra;
-    int	save_over;
+	/* Frank's variables */
+	int	save_over;
 
 	/* Reevan's __NLA variables */
-	struct	Object	*obpose;		/* Current posable object */
 	struct	ListBase edbo;			/* Armature Editmode bones */
  
 	/* Rob's variables */
 	int have_quicktime;
 	int ui_international;
+	int charstart;
+	int charmin;
+	int charmax;
+	struct VFont *selfont;
+	struct ListBase ttfdata;
 
-    /* this variable is written to / read from FileGlobal->fileflags */
-    int fileflags;
+	/* libtiff flag used to determine if shared library loaded for libtiff*/
+	int have_libtiff;
+
+	/* this variable is written to / read from FileGlobal->fileflags */
+	int fileflags;
     
 	/* save the allowed windowstate of blender when using -W or -w */
 	int windowstate;
@@ -129,84 +139,64 @@ typedef struct Global {
 	/* Janco's playing ground */
 	struct bSoundListener* listener;
 
-    /* Test thingy for Nzc */
-    int magic;       /* toggle use of experimental render pipe       */
+	/* Test thingy for Nzc */
 	int compat;      /* toggle compatibility mode for edge rendering */
 	int notonlysolid;/* T-> also edge-render transparent faces       */
-    int useRscale;   /* bitflag for using colour scaling             */
-    int useGscale;   /* bitflag for using colour scaling             */
-    int useBscale;   /* bitflag for using colour scaling             */
-    float cscale[4]; /* sliders for colour scaling                   */
-    int Rhisto;      /* flags for making histograms                  */
-    int Ghisto;
-    int Bhisto;
+	
+	/* confusing... G.f and G.flags */
+	int flags;
 
-    /* special versions */
-    short special1, special2;
-
-    int flags;
-        
 } Global;
 
 /* **************** GLOBAL ********************* */
 
 /* G.f */
-#define G_DISABLE_OK	1
-#define G_PLAYANIM		2
-#define G_TEST_DUPLI	4
-#define G_SIMULATION	8
-#define G_BACKBUFSEL	16
-#define G_PICKSEL		32
-#define G_DRAWNORMALS	64
-#define G_DRAWFACES		128
-#define G_FACESELECT	256
-#define G_DRAW_EXT		512
-#define G_VERTEXPAINT	1024
-#define G_ALLEDGES		2048
-#define G_DEBUG			4096
+#define G_DISABLE_OK	(1 <<  0)
+#define G_PLAYANIM		(1 <<  1)
+/* also uses G_FILE_AUTOPLAY */
+#define G_SIMULATION	(1 <<  3)
+#define G_BACKBUFSEL	(1 <<  4)
+#define G_PICKSEL		(1 <<  5)
+#define G_DRAWNORMALS	(1 <<  6)
+#define G_DRAWFACES		(1 <<  7)
+#define G_FACESELECT	(1 <<  8)
+#define G_DRAW_EXT		(1 <<  9)
+#define G_VERTEXPAINT	(1 << 10)
+#define G_ALLEDGES		(1 << 11)
+#define G_DEBUG			(1 << 12)
 #define G_DOSCRIPTLINKS (1 << 13)
-/* #define G_PROPORTIONAL	16384	removed! so can be used later for other stuff */
-#define G_WEIGHTPAINT	32768	
-#define G_TEXTUREPAINT	65536
+#define G_DRAW_VNORMALS	(1 << 14)
+#define G_WEIGHTPAINT	(1 << 15)	
+#define G_TEXTUREPAINT	(1 << 16)
 /* #define G_NOFROZEN	(1 << 17) also removed */
-#define G_DRAWEDGES	(1 << 18)
+#define G_DRAWEDGES		(1 << 18)
 #define G_DRAWCREASES	(1 << 19)
 #define G_DRAWSEAMS     (1 << 20)
 #define G_HIDDENEDGES   (1 << 21)
 
 /* Measurement info Drawing */
-#define G_DRAW_EDGELEN   (1 << 22) 
+#define G_DRAW_EDGELEN  (1 << 22) 
 #define G_DRAW_FACEAREA (1 << 23)
 #define G_DRAW_EDGEANG  (1 << 24)
 
-#define G_RECORDKEYS  (1 << 25)
+#define G_RECORDKEYS	(1 << 25)
 
 /* G.fileflags */
 
-#define G_AUTOPACK_BIT                  0
-#define G_FILE_COMPRESS_BIT             1
-#define G_FILE_AUTOPLAY_BIT             2
-#define G_FILE_ENABLE_ALL_FRAMES_BIT    3
-#define G_FILE_SHOW_DEBUG_PROPS_BIT     4
-#define G_FILE_SHOW_FRAMERATE_BIT       5
-#define G_FILE_SHOW_PROFILE_BIT         6
-#define G_FILE_LOCK_BIT                 7
-#define G_FILE_SIGN_BIT                 8
-#define G_FILE_PUBLISH_BIT				9
-#define G_FILE_NO_UI_BIT				10
-
-
-#define G_AUTOPACK               (1 << G_AUTOPACK_BIT)
-#define G_FILE_COMPRESS          (1 << G_FILE_COMPRESS_BIT)
-#define G_FILE_AUTOPLAY          (1 << G_FILE_AUTOPLAY_BIT)
-#define G_FILE_ENABLE_ALL_FRAMES (1 << G_FILE_ENABLE_ALL_FRAMES_BIT)
-#define G_FILE_SHOW_DEBUG_PROPS  (1 << G_FILE_SHOW_DEBUG_PROPS_BIT)
-#define G_FILE_SHOW_FRAMERATE    (1 << G_FILE_SHOW_FRAMERATE_BIT)
-#define G_FILE_SHOW_PROFILE      (1 << G_FILE_SHOW_PROFILE_BIT)
-#define G_FILE_LOCK              (1 << G_FILE_LOCK_BIT)
-#define G_FILE_SIGN              (1 << G_FILE_SIGN_BIT)
-#define G_FILE_PUBLISH			 (1 << G_FILE_PUBLISH_BIT)
-#define G_FILE_NO_UI			 (1 << G_FILE_NO_UI_BIT)
+#define G_AUTOPACK               (1 << 0)
+#define G_FILE_COMPRESS          (1 << 1)
+#define G_FILE_AUTOPLAY          (1 << 2)
+#define G_FILE_ENABLE_ALL_FRAMES (1 << 3)
+#define G_FILE_SHOW_DEBUG_PROPS  (1 << 4)
+#define G_FILE_SHOW_FRAMERATE    (1 << 5)
+#define G_FILE_SHOW_PROFILE      (1 << 6)
+#define G_FILE_LOCK              (1 << 7)
+#define G_FILE_SIGN              (1 << 8)
+#define G_FIle_PUBLISH			 (1 << 9)
+#define G_FILE_NO_UI			 (1 << 10)
+#define G_FILE_GAME_TO_IPO		 (1 << 11)
+#define G_FILE_GAME_MAT			 (1 << 12)
+#define G_FILE_DIAPLAY_LISTS	 (1 << 13)
 
 /* G.windowstate */
 #define G_WINDOWSTATE_USERDEF		0
@@ -219,10 +209,6 @@ typedef struct Global {
 #define G_QUIT		8
 #define G_SETSCENE	16
 
-/* G.flags: double? */
-#define G_FLAGS_AUTOPLAY_BIT	2 
-#define G_FLAGS_AUTOPLAY		(1 << G_FLAGS_AUTOPLAY_BIT)
-
 /* G.qual */
 #define R_SHIFTKEY		1
 #define L_SHIFTKEY		2
