@@ -1,5 +1,5 @@
 /**
- * $Id: BKE_object.h,v 1.5 2002/12/27 13:10:09 mein Exp $
+ * $Id: BKE_object.h,v 1.16 2006/06/14 08:50:41 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,12 +43,18 @@ struct Object;
 struct Camera;
 struct BoundBox;
 struct View3D;
+struct SoftBody;
 
 void clear_workob(void);
 void copy_baseflags(void);
 void copy_objectflags(void);
+struct SoftBody *copy_softbody(struct SoftBody *sb);
 void update_base_layer(struct Object *ob);
+
 void free_object(struct Object *ob);
+void object_free_display(struct Object *ob);
+void object_free_modifiers(struct Object *ob);
+
 void unlink_object(struct Object *ob);
 int exist_object(struct Object *obtest);
 void *add_camera(void);
@@ -67,31 +73,35 @@ void base_init_from_view3d(struct Base *base, struct View3D *v3d);
 struct Object *copy_object(struct Object *ob);
 void expand_local_object(struct Object *ob);
 void make_local_object(struct Object *ob);
-void set_mblur_offs(int blur);
+void set_mblur_offs(float blur);
+void set_field_offs(float field);
 void disable_speed_curve(int val);
+
 float bsystem_time(struct Object *ob, struct Object *par, float cfra, float ofs);
 void object_to_mat3(struct Object *ob, float mat[][3]);
 void object_to_mat4(struct Object *ob, float mat[][4]);
-void ob_parcurve(struct Object *ob, struct Object *par, float mat[][4]);
-void ob_parlimb(struct Object *ob, struct Object *par, float mat[][4]);
-void ob_parbone(struct Object *ob, struct Object *par, float mat[][4]);	/* __NLA */
-void give_parvert(struct Object *par, int nr, float *vec);
-void ob_parvert3(struct Object *ob, struct Object *par, float mat[][4]);
+
 void set_no_parent_ipo(int val);
-void set_dtime(int dtime);
+
 void disable_where_script(short on);
 int during_script(void);
+void disable_where_scriptlink(short on);
+int during_scriptlink(void);
+
 void where_is_object_time(struct Object *ob, float ctime);
 void where_is_object(struct Object *ob);
 void where_is_object_simul(struct Object *ob);
-void what_does_parent1(struct Object *par, int partype, int par1, int par2, int par3);
+
 void what_does_parent(struct Object *ob);
 struct BoundBox *unit_boundbox(void);
+void boundbox_set_from_min_max(struct BoundBox *bb, float min[3], float max[3]);
+struct BoundBox *object_get_boundbox(struct Object *ob);
 void minmax_object(struct Object *ob, float *min, float *max);
 void solve_tracking (struct Object *ob, float targetmat[][4]);
 void solve_constraints (struct Object *ob, short obtype, void *obdata, float ctime);
 
-	
+void object_handle_update(struct Object *ob);
+
 #ifdef __cplusplus
 }
 #endif
