@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_scene.h (mar-2001 nzc)
  *	
- * $Id: BKE_scene.h,v 1.7 2003/05/24 15:16:26 michel Exp $ 
+ * $Id: BKE_scene.h,v 1.12 2006/05/15 10:46:03 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,14 +40,32 @@ struct Base;
 struct AviCodecData;
 struct QuicktimeCodecData;
 
+/* sequence related defines */
+#define WHILE_SEQ(base)	{											\
+	int totseq_, seq_; Sequence **seqar;	\
+		build_seqar( base,  &seqar, &totseq_);	\
+			for(seq_ = 0; seq_ < totseq_; seq_++) {	\
+				seq= seqar[seq_];
+				
+				
+#define END_SEQ					}						\
+				if(seqar) MEM_freeN(seqar);		\
+}
+
+
+#define SETLOOPER(s, b) sce= s, b= sce->base.first; b; b= (b->next?b->next:sce->set?(sce=sce->set)->base.first:NULL)
+
+
 void free_avicodecdata(struct AviCodecData *acd);
 void free_qtcodecdata(struct QuicktimeCodecData *acd);
+
 void free_scene(struct Scene *me);
 struct Scene *add_scene(char *name);
-int object_in_scene(struct Object *ob, struct Scene *sce);
-void sort_baselist(struct Scene *sce);
+struct Base *object_in_scene(struct Object *ob, struct Scene *sce);
+
 void set_scene_bg(struct Scene *sce);
 void set_scene_name(char *name);
+
 int next_object(int val, struct Base **base, struct Object **ob);
 struct Object *scene_find_camera(struct Scene *sc);
 
@@ -55,5 +73,12 @@ struct Base *scene_add_base(struct Scene *sce, struct Object *ob);
 void scene_deselect_all(struct Scene *sce);
 void scene_select_base(struct Scene *sce, struct Base *selbase);
 
+/* checks for cycle, returns 1 if it's all OK */
+int scene_check_setscene(struct Scene *sce);
+
+void scene_update_for_newframe(struct Scene *sce, unsigned int lay);
+
+void scene_add_render_layer(struct Scene *sce);
+
 #endif
 
