@@ -2,7 +2,7 @@
 /*  curve.c 
  * 
  *  
- * $Id: curve.c,v 1.12 2005/04/17 18:00:32 ton Exp $
+ * $Id: curve.c,v 1.34 2006/07/06 14:20:44 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,8 +33,6 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#define STRUBI hack
-
 #include <math.h>  // floor
 #include <string.h>
 #include <stdlib.h>  
@@ -79,43 +77,6 @@ int cu_isectLL(float *v1, float *v2, float *v3, float *v4,
 			   short cox, short coy, 
 			   float *labda, float *mu, float *vec);
 
-
-#ifdef STRUBI
-/* hotfix; copies x*y array into extended (x+dx)*(y+dy) array
-old[] and new[] can be the same ! */
-int copyintoExtendedArray(float *old, int oldx, int oldy, float *new, int newx, int newy)
-{
-	int x, y, ttt, ooo;
-	float *oldp, *newp;
-        
-	if (newx < oldx || newy < oldy) return 0;
-	
-		
-	for (y = newy - 1; y >= oldy; y--) {	
-                ttt = y * newx;
-		for (x = newx - 1; x >= 0; x--) {
-			newp = new + 3 * (ttt + x);
-			newp[0] = 0.0; newp[1] = 0.0; newp[2] = 0.0;
-		}
-	}	
-
-	for (; y >= 0; y--) {
-		ttt = y * newx;
-                ooo = y * oldx;
-		for (x = newx - 1; x >= oldx; x--) {	
-			newp = new + 3 * (ttt + x);
-			newp[0] = 0.0; newp[1] = 0.0; newp[2] = 0.0;
-		}
-		for (; x  >= 0; x--) {
-			oldp = old + 3 * (ooo + x);
-			newp = new + 3 * (ttt + x);
-			VECCOPY(newp, oldp);
-		}
-	}
-	return 1;
-}
-#endif
-
 void unlink_curve(Curve *cu)
 {
 	int a;
@@ -145,8 +106,10 @@ void free_curve(Curve *cu)
 	
 	if(cu->mat) MEM_freeN(cu->mat);
 	if(cu->str) MEM_freeN(cu->str);
+	if(cu->strinfo) MEM_freeN(cu->strinfo);
 	if(cu->bb) MEM_freeN(cu->bb);
 	if(cu->path) free_path(cu->path);
+	if(cu->tb) MEM_freeN(cu->tb);
 }
 
 Curve *add_curve(int type)
@@ -165,8 +128,10 @@ Curve *add_curve(int type)
 	cu->pathlen= 100;
 	cu->resolu= cu->resolv= 6;
 	cu->width= 1.0;
+	cu->wordspace = 1.0;
 	cu->spacing= cu->linedist= 1.0;
 	cu->fsize= 1.0;
+	cu->ulheight = 0.05;	
 	cu->texflag= CU_AUTOSPACE;
 	
 	cu->bb= unit_boundbox();
@@ -189,6 +154,8 @@ Curve *copy_curve(Curve *cu)
 	}
 	
 	cun->str= MEM_dupallocN(cu->str);
+	cun->strinfo= MEM_dupallocN(cu->strinfo);	
+	cun->tb= MEM_dupallocN(cu->tb);
 	cun->bb= MEM_dupallocN(cu->bb);
 	
 	cun->key= copy_key(cu->key);
@@ -202,6 +169,9 @@ Curve *copy_curve(Curve *cu)
 	if(cun->ipo) cun->ipo= copy_ipo(cun->ipo);
 
 	id_us_plus((ID *)cun->vfont);
+	id_us_plus((ID *)cun->vfontb);	
+	id_us_plus((ID *)cun->vfonti);
+	id_us_plus((ID *)cun->vfontbi);
 	
 	return cun;
 }
@@ -312,28 +282,20 @@ void tex_space_curve(Curve *cu)
 		dl= dl->next;
 	}
 
-	if(doit) {
-		loc[0]= (min[0]+max[0])/2.0f;
-		loc[1]= (min[1]+max[1])/2.0f;
-		loc[2]= (min[2]+max[2])/2.0f;
-		
-		size[0]= (max[0]-min[0])/2.0f;
-		size[1]= (max[1]-min[1])/2.0f;
-		size[2]= (max[2]-min[2])/2.0f;
-	}
-	else {
-		loc[0]= loc[1]= loc[2]= 0.0f;
-		size[0]= size[1]= size[2]= 1.0f;
+	if(!doit) {
+		min[0] = min[1] = min[2] = -1.0f;
+		max[0] = max[1] = max[2] = 1.0f;
 	}
 	
-	bb->vec[0][0]=bb->vec[1][0]=bb->vec[2][0]=bb->vec[3][0]= loc[0]-size[0];
-	bb->vec[4][0]=bb->vec[5][0]=bb->vec[6][0]=bb->vec[7][0]= loc[0]+size[0];
+	loc[0]= (min[0]+max[0])/2.0f;
+	loc[1]= (min[1]+max[1])/2.0f;
+	loc[2]= (min[2]+max[2])/2.0f;
 	
-	bb->vec[0][1]=bb->vec[1][1]=bb->vec[4][1]=bb->vec[5][1]= loc[1]-size[1];
-	bb->vec[2][1]=bb->vec[3][1]=bb->vec[6][1]=bb->vec[7][1]= loc[1]+size[1];
+	size[0]= (max[0]-min[0])/2.0f;
+	size[1]= (max[1]-min[1])/2.0f;
+	size[2]= (max[2]-min[2])/2.0f;
 
-	bb->vec[0][2]=bb->vec[3][2]=bb->vec[4][2]=bb->vec[7][2]= loc[2]-size[2];
-	bb->vec[1][2]=bb->vec[2][2]=bb->vec[5][2]=bb->vec[6][2]= loc[2]+size[2];
+	boundbox_set_from_min_max(bb, min, max);
 
 	if(cu->texflag & CU_AUTOSPACE) {
 		VECCOPY(cu->loc, loc);
@@ -521,52 +483,7 @@ void minmaxNurb(Nurb *nu, float *min, float *max)
 /* ~~~~~~~~~~~~~~~~~~~~Non Uniform Rational B Spline calculations ~~~~~~~~~~~ */
 
 
-/* actually, doubles should be used here as much as possible */
-
-void extend_spline(float * pnts, int in, int out)
-{
-	float *_pnts;
-	double * add;
-	int i, j, k, in2;
-
-	_pnts = pnts;
-	add = (double*)MEM_mallocN((in)* sizeof(double), "extend_spline");
-
-        in2 = in -1;
-
-	for (k = 3; k > 0; k--){
-		pnts = _pnts;
-
-		/* copy points to 'add' */
-		for (i = 0; i < in; i++){
-			add[i] = *pnts;
-			pnts += 3;
-		}
-
-		/* inverse forward differencing */
-		for (i = 0; i < in2; i++){
-			for (j = in2; j > i; j--){
-				add[j] -= add[j - 1];
-			}
-		}
-
-		pnts = _pnts;
-		for (i = out; i > 0; i--){
-			*pnts = (float)(add[0]);
-			pnts += 3;
-			for (j = 0; j < in2; j++){
-				add[j] += add[j+1];
-			}
-		}
-
-		_pnts++;
-	}
-
-	MEM_freeN(add);
-}
-
-
-void calcknots(float *knots, short aantal, short order, short type)
+static void calcknots(float *knots, short aantal, short order, short type)
 /* knots: number of pnts NOT corrected for cyclic */
 /* type;	 0: uniform, 1: endpoints, 2: bezier */
 {
@@ -605,7 +522,7 @@ void calcknots(float *knots, short aantal, short order, short type)
 	}
 }
 
-void makecyclicknots(float *knots, short pnts, short order)
+static void makecyclicknots(float *knots, short pnts, short order)
 /* pnts, order: number of pnts NOT corrected for cyclic */
 {
 	int a, b, order2, c;
@@ -655,7 +572,7 @@ void makeknots(Nurb *nu, short uv, short type)	/* 0: uniform, 1: endpoints, 2: b
 	}
 }
 
-void basisNurb(float t, short order, short pnts, float *knots, float *basis, int *start, int *end)
+static void basisNurb(float t, short order, short pnts, float *knots, float *basis, int *start, int *end)
 {
 	float d, e;
 	int i, i1 = 0, i2 = 0 ,j, orderpluspnts, opp2, o2;
@@ -718,7 +635,7 @@ void basisNurb(float t, short order, short pnts, float *knots, float *basis, int
 }
 
 
-void makeNurbfaces(Nurb *nu, float *data) 
+void makeNurbfaces(Nurb *nu, float *data, int rowstride) 
 /* data  has to be 3*4*resolu*resolv in size, and zero-ed */
 {
 	BPoint *bp;
@@ -735,6 +652,9 @@ void makeNurbfaces(Nurb *nu, float *data)
 	/* allocate and initialize */
 	len= nu->pntsu*nu->pntsv;
 	if(len==0) return;
+	
+
+	
 	sum= (float *)MEM_callocN(sizeof(float)*len, "makeNurbfaces1");
 
 	resolu= nu->resolu;
@@ -863,6 +783,7 @@ void makeNurbfaces(Nurb *nu, float *data)
 			basis+= KNOTSV(nu);
 		}
 		u+= ustep;
+		if (rowstride!=0) in = (float*) (((unsigned char*) in) + (rowstride - 3*nu->resolv*sizeof(*in)));
 	}
 
 	/* free */
@@ -873,100 +794,13 @@ void makeNurbfaces(Nurb *nu, float *data)
 	MEM_freeN(jend);
 }
 
-
-void makeNurbcurve_forw(Nurb *nu, float *data)
-/* *data: has to be 3*4*pntsu*resolu in size and zero-ed */
-{
-	BPoint *bp;
-	float *basisu, *sum, *fp,  *in;
-	float u, ustart, uend, ustep, sumdiv;
-	int i, j, k, len, resolu, istart, iend;
-	int wanted, org;
-
-	if(nu->knotsu==0) return;
-	if(data==0) return;
-
-	/* allocate and init */
-	len= nu->pntsu;
-	if(len==0) return;
-	sum= (float *)MEM_callocN(sizeof(float)*len, "makeNurbcurve1");
-
-	resolu= nu->resolu*nu->pntsu;
-	if(resolu==0) {
-		MEM_freeN(sum);
-		return;
-	}
-
-	fp= nu->knotsu;
-	ustart= fp[nu->orderu-1];
-	uend= fp[nu->pntsu];
-	ustep= (uend-ustart)/(resolu-1);
-	basisu= (float *)MEM_mallocN(sizeof(float)*(nu->orderu+nu->pntsu), "makeNurbcurve3");
-
-	in= data;
-	u= ustart;
-	for (k = nu->orderu - 1; k < nu->pntsu; k++){
-
-		wanted = (int)((nu->knotsu[k+1] - nu->knotsu[k]) / ustep);
-		org = 4;	/* equal to order */
-		if (org > wanted) org = wanted;
-
-		for (j = org; j > 0; j--){
-
-			basisNurb(u, nu->orderu, nu->pntsu, nu->knotsu, basisu, &istart, &iend);
-			/* calc sum */
-			sumdiv= 0.0;
-			fp= sum;
-			for(i= istart; i<=iend; i++, fp++) {
-				/* do the rational component */
-				*fp= basisu[i];
-				sumdiv+= *fp;
-			}
-			if(sumdiv!=0.0) if(sumdiv<0.999 || sumdiv>1.001) {
-				/* is this normalizing needed? */
-				fp= sum;
-				for(i= istart; i<=iend; i++, fp++) {
-					*fp/= sumdiv;
-				}
-			}
-
-			/* one! (1.0) real point */
-			fp= sum;
-			bp= nu->bp+ istart;
-			for(i= istart; i<=iend; i++, bp++, fp++) {
-
-				if(*fp!=0.0) {
-					in[0]+= (*fp) * bp->vec[0];
-					in[1]+= (*fp) * bp->vec[1];
-					in[2]+= (*fp) * bp->vec[2];
-				}
-			}
-
-			in+=3;
-
-			u+= ustep;
-		}
-
-		if (wanted > org){
-			extend_spline(in - 3 * org, org, wanted);
-			in += 3 * (wanted - org);
-			u += ustep * (wanted - org);
-		}
-	}
-
-	/* free */
-	MEM_freeN(sum);
-	MEM_freeN(basisu);
-}
-
-
-void makeNurbcurve(Nurb *nu, float *data, int dim)
+void makeNurbcurve(Nurb *nu, float *data, int resolu, int dim)
 /* data has to be dim*4*pntsu*resolu in size and zero-ed */
 {
 	BPoint *bp;
 	float u, ustart, uend, ustep, sumdiv;
 	float *basisu, *sum, *fp,  *in;
-	int i, len, resolu, istart, iend, cycl;
+	int i, len, istart, iend, cycl;
 
 	if(nu->knotsu==0) return;
 	if(nu->orderu>nu->pntsu) return;
@@ -977,7 +811,7 @@ void makeNurbcurve(Nurb *nu, float *data, int dim)
 	if(len==0) return;
 	sum= (float *)MEM_callocN(sizeof(float)*len, "makeNurbcurve1");
 
-	resolu= nu->resolu*nu->pntsu;
+	resolu*= nu->pntsu;
 	if(resolu==0) {
 		MEM_freeN(sum);
 		return;
@@ -1047,7 +881,8 @@ void makeNurbcurve(Nurb *nu, float *data, int dim)
 	MEM_freeN(basisu);
 }
 
-void maakbez(float q0, float q1, float q2, float q3, float *p, int it)
+/* forward differencing method for bezier curve */
+void forward_diff_bezier(float q0, float q1, float q2, float q3, float *p, int it, int stride)
 {
 	float rt0,rt1,rt2,rt3,f;
 	int a;
@@ -1067,7 +902,7 @@ void maakbez(float q0, float q1, float q2, float q3, float *p, int it)
   
   	for(a=0; a<=it; a++) {
 		*p= q0;
-		p+= 3;
+		p+= stride;
 		q0+= q1;
  		q1+= q2;
  		q2+= q3;
@@ -1076,54 +911,49 @@ void maakbez(float q0, float q1, float q2, float q3, float *p, int it)
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
-void make_orco_surf(Curve *cu)
+float *make_orco_surf(Object *ob)
 {
+	Curve *cu= ob->data;
 	Nurb *nu;
 	int a, b, tot=0;
-	int sizeu, sizev;// ###
-	float *data;
-
-
+	int sizeu, sizev;
+	float *data, *orco;
+	
 	/* first calculate the size of the datablock */
 	nu= cu->nurb.first;
 	while(nu) {
-#ifdef STRUBI
-/* this is a bad hack: as we want to avoid the seam in a cyclic nurbs
-texture wrapping, reserve extra orco data space to save these extra needed
-vertex based UV coordinates for the meridian vertices.
-Vertices on the 0/2pi boundary are not duplicated inside the displist but later in
-the renderface/vert construction.
-
-See also blenderWorldManipulation.c: init_render_surf()
-
-*/
-
-		sizeu = nu->resolu; sizev = nu->resolv;
+		/* as we want to avoid the seam in a cyclic nurbs
+		texture wrapping, reserve extra orco data space to save these extra needed
+		vertex based UV coordinates for the meridian vertices.
+		Vertices on the 0/2pi boundary are not duplicated inside the displist but later in
+		the renderface/vert construction.
+		
+		See also convertblender.c: init_render_surf()
+		*/
+		
+		sizeu = nu->resolu; 
+		sizev = nu->resolv;
 		if (nu->flagu & CU_CYCLIC) sizeu++;
 		if (nu->flagv & CU_CYCLIC) sizev++;
-		if(nu->pntsv>1) tot+= sizeu * sizev;
-#else
-		if(nu->pntsv>1) tot+= nu->resolu*nu->resolv;
-#endif
+ 		if(nu->pntsv>1) tot+= sizeu * sizev;
+		
 		nu= nu->next;
 	}
-				/* makeNurbfaces wants zeros */
-	data= cu->orco= MEM_callocN(3*sizeof(float)*tot, "make_orco");
-
+	/* makeNurbfaces wants zeros */
+	data= orco= MEM_callocN(3*sizeof(float)*tot, "make_orco");
+	
 	nu= cu->nurb.first;
 	while(nu) {
 		if(nu->pntsv>1) {
 			sizeu = nu->resolu;
 			sizev = nu->resolv;
-#ifdef STRUBI
 			if (nu->flagu & CU_CYCLIC) sizeu++;
 			if (nu->flagv & CU_CYCLIC) sizev++;
-#endif
 			
 			if(cu->flag & CU_UV_ORCO) {
 				for(b=0; b< sizeu; b++) {
 					for(a=0; a< sizev; a++) {
-					
+						
 						if(sizev <2) data[0]= 0.0f;
 						else data[0]= -1.0f + 2.0f*((float)a)/(sizev - 1);
 						
@@ -1131,58 +961,135 @@ See also blenderWorldManipulation.c: init_render_surf()
 						else data[1]= -1.0f + 2.0f*((float)b)/(sizeu - 1);
 						
 						data[2]= 0.0;
-		
+						
 						data+= 3;
 					}
 				}
 			}
 			else {
-				makeNurbfaces(nu, data);
-#ifdef STRUBI 
-				for(b=0; b< nu->resolu; b++) {
-					for(a=0; a< nu->resolv; a++) {
-						data = cu->orco + 3 * (b * nu->resolv + a);
-						data[0]= (data[0]-cu->loc[0])/cu->size[0];
-						data[1]= (data[1]-cu->loc[1])/cu->size[1];
-						data[2]= (data[2]-cu->loc[2])/cu->size[2];
+				float *_tdata= MEM_callocN(nu->resolu*nu->resolv*3*sizeof(float), "temp data");
+				float *tdata= _tdata;
+				
+				makeNurbfaces(nu, tdata, 0);
+				
+				for(b=0; b<sizeu; b++) {
+					int use_b= b;
+					if (b==sizeu-1 && (nu->flagu & CU_CYCLIC))
+						use_b= 0;
+					
+					for(a=0; a<sizev; a++) {
+						int use_a= a;
+						if (a==sizev-1 && (nu->flagv & CU_CYCLIC))
+							use_a= 0;
+						
+						tdata = _tdata + 3 * (use_b * nu->resolv + use_a);
+						
+						data[0]= (tdata[0]-cu->loc[0])/cu->size[0];
+						data[1]= (tdata[1]-cu->loc[1])/cu->size[1];
+						data[2]= (tdata[2]-cu->loc[2])/cu->size[2];
+						data+= 3;
 					}
 				}
-				copyintoExtendedArray(cu->orco, nu->resolv, nu->resolu, cu->orco, sizev, sizeu);
-				/* copy U/V-cyclic orco's */
-				if (nu->flagv & CU_CYCLIC) {
-					b = sizeu - 1;	
-					for(a=0; a< sizev; a++) {
-						data = cu->orco + 3 * (b * sizev + a);
-						VECCOPY(data, cu->orco + 3*a);
-					}
-				}	
-				if (nu->flagu & CU_CYCLIC) {
-					a = sizev - 1;	
-					for(b=0; b< sizeu; b++) {
-						data = cu->orco + 3 * (b * sizev + a);
-						VECCOPY(data, cu->orco + 3 * b*sizev);
-					}
-				}	
-
-#else
-				tot= sizeu * sizev;
-				while(tot--) {
-					data[0]= (data[0]-cu->loc[0])/cu->size[0];
-					data[1]= (data[1]-cu->loc[1])/cu->size[1];
-					data[2]= (data[2]-cu->loc[2])/cu->size[2];
-	
-					data+= 3;
-				}
-#endif
+				
+				MEM_freeN(_tdata);
 			}
 		}
 		nu= nu->next;
 	}
-	/* loadkeypostype(22, base, base); */
-
+	
+	return orco;
 }
 
 
+	/* NOTE: This routine is tied to the order of vertex
+	 * built by displist and as passed to the renderer.
+	 */
+float *make_orco_curve(Object *ob)
+{
+	Curve *cu = ob->data;
+	DispList *dl;
+	int u, v, numVerts;
+	float *fp, *orco;
+	int remakeDisp = 0;
+
+	if (!(cu->flag&CU_UV_ORCO) && cu->key && cu->key->refkey) {
+		cp_cu_key(cu, cu->key->refkey, 0, count_curveverts(&cu->nurb));
+		makeDispListCurveTypes(ob, 1);
+		remakeDisp = 1;
+	}
+
+	/* Assumes displist has been built */
+
+	numVerts = 0;
+	for (dl=cu->disp.first; dl; dl=dl->next) {
+		if (dl->type==DL_INDEX3) {
+			numVerts += dl->nr;
+		} else if (dl->type==DL_SURF) {
+			/* convertblender.c uses the Surface code for creating renderfaces when cyclic U only (closed circle beveling) */
+			if (dl->flag & DL_CYCL_U) {
+				if (dl->flag & DL_CYCL_V)
+					numVerts += (dl->parts+1)*(dl->nr+1);
+				else
+					numVerts += dl->parts*(dl->nr+1);
+			}
+			else
+				numVerts += dl->parts*dl->nr;
+		}
+	}
+
+	fp= orco= MEM_mallocN(3*sizeof(float)*numVerts, "cu_orco");
+	for (dl=cu->disp.first; dl; dl=dl->next) {
+		if (dl->type==DL_INDEX3) {
+			for (u=0; u<dl->nr; u++, fp+=3) {
+				if (cu->flag & CU_UV_ORCO) {
+					fp[0]= 2.0f*u/(dl->nr-1) - 1.0f;
+					fp[1]= 0.0;
+					fp[2]= 0.0;
+				} else {
+					VECCOPY(fp, &dl->verts[u*3]);
+
+					fp[0]= (fp[0]-cu->loc[0])/cu->size[0];
+					fp[1]= (fp[1]-cu->loc[1])/cu->size[1];
+					fp[2]= (fp[2]-cu->loc[2])/cu->size[2];
+				}
+			}
+		} else if (dl->type==DL_SURF) {
+			int sizeu= dl->nr, sizev= dl->parts;
+			
+			/* exception as handled in convertblender.c too */
+			if (dl->flag & DL_CYCL_U) {
+				sizeu++;
+				if (dl->flag & DL_CYCL_V)
+					sizev++;
+			}
+			
+			for (u=0; u<sizev; u++) {
+				for (v=0; v<sizeu; v++,fp+=3) {
+					if (cu->flag & CU_UV_ORCO) {
+						fp[0]= 2.0f*u/(dl->parts-1) - 1.0f;
+						fp[1]= 2.0f*v/(dl->nr-1) - 1.0f;
+						fp[2]= 0.0;
+					} else {
+						int realv= v % dl->nr;
+
+						VECCOPY(fp, &dl->verts[(dl->nr*u + realv)*3]);
+
+						fp[0]= (fp[0]-cu->loc[0])/cu->size[0];
+						fp[1]= (fp[1]-cu->loc[1])/cu->size[1];
+						fp[2]= (fp[2]-cu->loc[2])/cu->size[2];
+					}
+				}
+			}
+		}
+	}
+
+	if (remakeDisp) {
+		makeDispListCurveTypes(ob, 0);
+	}
+
+	return orco;
+}
+
 
 /* ***************** BEVEL ****************** */
 
@@ -1194,6 +1101,10 @@ void makebevelcurve(Object *ob, ListBase *disp)
 	int nr, a;
 
 	cu= ob->data;
+	disp->first = disp->last = NULL;
+
+	/* if a font object is being edited, then do nothing */
+	if( ob == G.obedit && ob->type == OB_FONT ) return;
 
 	if(cu->bevobj && cu->bevobj!=ob) {
 		if(cu->bevobj->type==OB_CURVE) {
@@ -1204,7 +1115,7 @@ void makebevelcurve(Object *ob, ListBase *disp)
 
 				dl= bevcu->disp.first;
 				if(dl==0) {
-					makeDispList(cu->bevobj);
+					makeDispListCurveTypes(cu->bevobj, 0);
 					dl= bevcu->disp.first;
 				}
 				while(dl) {
@@ -1231,6 +1142,9 @@ void makebevelcurve(Object *ob, ListBase *disp)
 			}
 		}
 	}
+	else if(cu->ext1==0.0 && cu->ext2==0.0) {
+		;
+	}
 	else if(cu->ext2==0.0) {
 		dl= MEM_callocN(sizeof(DispList), "makebevelcurve2");
 		dl->verts= MEM_mallocN(2*3*sizeof(float), "makebevelcurve2");
@@ -1246,13 +1160,38 @@ void makebevelcurve(Object *ob, ListBase *disp)
 		fp[3]= fp[4]= 0.0;
 		fp[5]= cu->ext1;
 	}
+	else if( (cu->flag & (CU_FRONT|CU_BACK))==0 && cu->ext1==0.0f)	{ // we make a full round bevel in that case
+		
+		nr= 4+ 2*cu->bevresol;
+		   
+		dl= MEM_callocN(sizeof(DispList), "makebevelcurve p1");
+		dl->verts= MEM_mallocN(nr*3*sizeof(float), "makebevelcurve p1");
+		BLI_addtail(disp, dl);
+		dl->type= DL_POLY;
+		dl->parts= 1;
+		dl->flag= DL_BACK_CURVE;
+		dl->nr= nr;
+
+		/* a circle */
+		fp= dl->verts;
+		dhoek= (2.0f*M_PI/(nr));
+		hoek= -(nr-1)*dhoek;
+		
+		for(a=0; a<nr; a++) {
+			fp[0]= 0.0;
+			fp[1]= (float)(cos(hoek)*(cu->ext2));
+			fp[2]= (float)(sin(hoek)*(cu->ext2)) - cu->ext1;
+			hoek+= dhoek;
+			fp+= 3;
+		}
+	}
 	else {
 		short dnr;
 		
 		/* bevel now in three parts, for proper vertex normals */
 		/* part 1 */
 		dnr= nr= 2+ cu->bevresol;
-		if( (cu->flag & (CU_FRONT|CU_BACK))==0)	// we make a full round bevel in that case
+		if( (cu->flag & (CU_FRONT|CU_BACK))==0)
 			nr= 3+ 2*cu->bevresol;
 		   
 		dl= MEM_callocN(sizeof(DispList), "makebevelcurve p1");
@@ -1332,7 +1271,6 @@ void makebevelcurve(Object *ob, ListBase *disp)
 			fp+= 3;
 		}
 	}
-
 }
 
 int cu_isectLL(float *v1, float *v2, float *v3, float *v4, short cox, short coy, float *labda, float *mu, float *vec)
@@ -1369,7 +1307,7 @@ int cu_isectLL(float *v1, float *v2, float *v3, float *v4, short cox, short coy,
 }
 
 
-short bevelinside(BevList *bl1,BevList *bl2)
+static short bevelinside(BevList *bl1,BevList *bl2)
 {
 	/* is bl2 INSIDE bl1 ? with left-right method and "labda's" */
 	/* returns '1' if correct hole  */
@@ -1428,7 +1366,7 @@ struct bevelsort {
 	int dir;
 };
 
-int vergxcobev(const void *a1, const void *a2)
+static int vergxcobev(const void *a1, const void *a2)
 {
 	const struct bevelsort *x1=a1,*x2=a2;
 
@@ -1439,7 +1377,7 @@ int vergxcobev(const void *a1, const void *a2)
 
 /* this function cannot be replaced with atan2, but why? */
 
-void calc_bevel_sin_cos(float x1, float y1, float x2, float y2, float *sina, float *cosa)
+static void calc_bevel_sin_cos(float x1, float y1, float x2, float y2, float *sina, float *cosa)
 {
 	float t01, t02, x3, y3;
 
@@ -1476,7 +1414,7 @@ void calc_bevel_sin_cos(float x1, float y1, float x2, float y2, float *sina, flo
 
 }
 
-void alfa_bezpart(BezTriple *prevbezt, BezTriple *bezt, Nurb *nu, float *data_a)
+static void alfa_bezpart(BezTriple *prevbezt, BezTriple *bezt, Nurb *nu, float *data_a, int resolu)
 {
 	BezTriple *pprev, *next, *last;
 	float fac, dfac, t[4];
@@ -1499,9 +1437,9 @@ void alfa_bezpart(BezTriple *prevbezt, BezTriple *bezt, Nurb *nu, float *data_a)
 	else next= bezt+1;
 	
 	fac= 0.0;
-	dfac= 1.0f/(float)nu->resolu;
+	dfac= 1.0f/(float)resolu;
 	
-	for(a=0; a<nu->resolu; a++, fac+= dfac) {
+	for(a=0; a<resolu; a++, fac+= dfac) {
 		
 		set_four_ipo(fac, t, KEY_BSPLINE);
 		
@@ -1525,7 +1463,7 @@ void makeBevelList(Object *ob)
 	BevPoint *bevp, *bevp2, *bevp1 = NULL, *bevp0;
 	float  *data, *data_a, *v1, *v2, min, inp, x1, x2, y1, y2, vec[3];
 	struct bevelsort *sortdata, *sd, *sd1;
-	int a, b, len, nr, poly;
+	int a, b, len, nr, poly, resolu;
 
 	/* this function needs an object, because of tflag and upflag */
 	cu= ob->data;
@@ -1538,7 +1476,11 @@ void makeBevelList(Object *ob)
 	
 	while(nu) {
 		if(nu->pntsu>1) {
-		
+			if(G.rendering && cu->resolu_ren!=0) 
+				resolu= cu->resolu_ren;
+			else
+				resolu= nu->resolu;
+			
 			if((nu->type & 7)==CU_POLY) {
 	
 				len= nu->pntsu;
@@ -1564,7 +1506,7 @@ void makeBevelList(Object *ob)
 			}
 			else if((nu->type & 7)==CU_BEZIER) {
 	
-				len= nu->resolu*(nu->pntsu+ (nu->flagu & 1) -1)+1;	/* in case last point is not cyclic */
+				len= resolu*(nu->pntsu+ (nu->flagu & 1) -1)+1;	/* in case last point is not cyclic */
 				bl= MEM_callocN(sizeof(BevList)+len*sizeof(BevPoint), "makeBevelList");
 				BLI_addtail(&(cu->bev), bl);
 	
@@ -1583,8 +1525,8 @@ void makeBevelList(Object *ob)
 					bezt++;
 				}
 				
-				data= MEM_mallocN(3*sizeof(float)*(nu->resolu+1), "makeBevelList2");
-				data_a= MEM_callocN(sizeof(float)*(nu->resolu+1), "data_a");
+				data= MEM_mallocN(3*sizeof(float)*(resolu+1), "makeBevelList2");
+				data_a= MEM_callocN(sizeof(float)*(resolu+1), "data_a");
 				
 				while(a--) {
 					if(prevbezt->h2==HD_VECT && bezt->h1==HD_VECT) {
@@ -1604,13 +1546,13 @@ void makeBevelList(Object *ob)
 						v2= bezt->vec[0];
 						
 						/* always do all three, to prevent data hanging around */
-						maakbez(v1[0], v1[3], v2[0], v2[3], data, nu->resolu);
-						maakbez(v1[1], v1[4], v2[1], v2[4], data+1, nu->resolu);
-						maakbez(v1[2], v1[5], v2[2], v2[5], data+2, nu->resolu);
+						forward_diff_bezier(v1[0], v1[3], v2[0], v2[3], data, resolu, 3);
+						forward_diff_bezier(v1[1], v1[4], v2[1], v2[4], data+1, resolu, 3);
+						forward_diff_bezier(v1[2], v1[5], v2[2], v2[5], data+2, resolu, 3);
 						
 						if((nu->type & CU_2D)==0) {
 							if(cu->flag & CU_3D) {
-								alfa_bezpart(prevbezt, bezt, nu, data_a);
+								alfa_bezpart(prevbezt, bezt, nu, data_a, resolu);
 							}
 						}
 						
@@ -1626,7 +1568,7 @@ void makeBevelList(Object *ob)
 						
 						v1= data;
 						v2= data_a;
-						nr= nu->resolu;
+						nr= resolu;
 						
 						while(nr--) {
 							bevp->x= v1[0]; 
@@ -1637,7 +1579,7 @@ void makeBevelList(Object *ob)
 							v1+=3;
 							v2++;
 						}
-						bl->nr+= nu->resolu;
+						bl->nr+= resolu;
 	
 					}
 					prevbezt= bezt;
@@ -1657,7 +1599,7 @@ void makeBevelList(Object *ob)
 			}
 			else if((nu->type & 7)==CU_NURBS) {
 				if(nu->pntsv==1) {
-					len= nu->resolu*nu->pntsu;
+					len= resolu*nu->pntsu;
 					bl= MEM_mallocN(sizeof(BevList)+len*sizeof(BevPoint), "makeBevelList3");
 					BLI_addtail(&(cu->bev), bl);
 					bl->nr= len;
@@ -1667,7 +1609,7 @@ void makeBevelList(Object *ob)
 					bevp= (BevPoint *)(bl+1);
 	
 					data= MEM_callocN(4*sizeof(float)*len, "makeBevelList4");    /* has to be zero-ed */
-					makeNurbcurve(nu, data, 4);
+					makeNurbcurve(nu, data, resolu, 4);
 					
 					v1= data;
 					while(len--) {
@@ -1928,10 +1870,10 @@ void makeBevelList(Object *ob)
  *		1: nothing,  1:auto,  2:vector,  3:aligned
  */
 
-
-void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
+/* mode: is not zero when IpoCurve, is 2 when forced horizontal for autohandles */
+void calchandleNurb(BezTriple *bezt, BezTriple *prev, BezTriple *next, int mode)
 {
-	float *p1,*p2,*p3,pt[3];
+	float *p1,*p2,*p3, pt[3];
 	float dx1,dy1,dz1,dx,dy,dz,vx,vy,vz,len,len1,len2;
 
 	if(bezt->h1==0 && bezt->h2==0) return;
@@ -1955,29 +1897,19 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 	}
 	else p3= next->vec[1];
 
-	if(mode && bezt->h1==HD_AUTO && prev) {
-		dx= p2[0] - (p1[0]+p1[3])/2.0f;
-		dy= p2[1] - (p1[1]+p1[4])/2.0f;
-		dz= p2[2] - (p1[2]+p1[5])/2.0f;
-	}
-	else {
-		dx= p2[0]- p1[0];
-		dy= p2[1]- p1[1];
-		dz= p2[2]- p1[2];
-	}
-	len1= (float)sqrt(dx*dx+dy*dy+dz*dz);
+	dx= p2[0]- p1[0];
+	dy= p2[1]- p1[1];
+	dz= p2[2]- p1[2];
 	
-	if(mode && bezt->h2==HD_AUTO && next) {
-		dx1= (p3[0]+p3[-3])/2.0f - p2[0];
-		dy1= (p3[1]+p3[-2])/2.0f - p2[1];
-		dz1= (p3[2]+p3[-1])/2.0f - p2[2];
-	}
-	else {
-		dx1= p3[0]- p2[0];
-		dy1= p3[1]- p2[1];
-		dz1= p3[2]- p2[2];
-	}
-	len2= (float)sqrt(dx1*dx1+dy1*dy1+dz1*dz1);
+	if(mode) len1= dx;
+	else len1= (float)sqrt(dx*dx+dy*dy+dz*dz);
+	
+	dx1= p3[0]- p2[0];
+	dy1= p3[1]- p2[1];
+	dz1= p3[2]- p2[2];
+	
+	if(mode) len2= dx1;
+	else len2= (float)sqrt(dx1*dx1+dy1*dy1+dz1*dz1);
 
 	if(len1==0.0f) len1=1.0f;
 	if(len2==0.0f) len2=1.0f;
@@ -1989,7 +1921,8 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 		vz= dz1/len2 + dz/len1;
 		len= 2.5614f*(float)sqrt(vx*vx + vy*vy + vz*vz);
 		if(len!=0.0f) {
-		
+			int leftviolate=0, rightviolate=0;	/* for mode==2 */
+			
 			if(len1>5.0f*len2) len1= 5.0f*len2;	
 			if(len2>5.0f*len1) len2= 5.0f*len1;
 			
@@ -1998,13 +1931,79 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 				*(p2-3)= *p2-vx*len1;
 				*(p2-2)= *(p2+1)-vy*len1;
 				*(p2-1)= *(p2+2)-vz*len1;
+				
+				if(mode==2 && next && prev) {	// keep horizontal if extrema
+					float ydiff1= prev->vec[1][1] - bezt->vec[1][1];
+					float ydiff2= next->vec[1][1] - bezt->vec[1][1];
+					if( (ydiff1<=0.0 && ydiff2<=0.0) || (ydiff1>=0.0 && ydiff2>=0.0) ) {
+						bezt->vec[0][1]= bezt->vec[1][1];
+					}
+					else {						// handles should not be beyond y coord of two others
+						if(ydiff1<=0.0) { 
+							if(prev->vec[1][1] > bezt->vec[0][1]) {
+								bezt->vec[0][1]= prev->vec[1][1]; 
+								leftviolate= 1;
+							}
+						}
+						else {
+							if(prev->vec[1][1] < bezt->vec[0][1]) {
+								bezt->vec[0][1]= prev->vec[1][1]; 
+								leftviolate= 1;
+							}
+						}
+					}
+				}
 			}
 			if(bezt->h2==HD_AUTO) {
 				len2/=len;
 				*(p2+3)= *p2+vx*len2;
 				*(p2+4)= *(p2+1)+vy*len2;
 				*(p2+5)= *(p2+2)+vz*len2;
+				
+				if(mode==2 && next && prev) {	// keep horizontal if extrema
+					float ydiff1= prev->vec[1][1] - bezt->vec[1][1];
+					float ydiff2= next->vec[1][1] - bezt->vec[1][1];
+					if( (ydiff1<=0.0 && ydiff2<=0.0) || (ydiff1>=0.0 && ydiff2>=0.0) ) {
+						bezt->vec[2][1]= bezt->vec[1][1];
+					}
+					else {						// handles should not be beyond y coord of two others
+						if(ydiff1<=0.0) { 
+							if(next->vec[1][1] < bezt->vec[2][1]) {
+								bezt->vec[2][1]= next->vec[1][1]; 
+								rightviolate= 1;
+							}
+						}
+						else {
+							if(next->vec[1][1] > bezt->vec[2][1]) {
+								bezt->vec[2][1]= next->vec[1][1]; 
+								rightviolate= 1;
+							}
+						}
+					}
+				}
 			}
+			if(leftviolate || rightviolate) {	/* align left handle */
+				float h1[3], h2[3];
+				
+				VecSubf(h1, p2-3, p2);
+				VecSubf(h2, p2, p2+3);
+				len1= Normalise(h1);
+				len2= Normalise(h2);
+				
+				vz= INPR(h1, h2);
+				
+				if(leftviolate) {
+					*(p2+3)= *(p2)   - vz*len2*h1[0];
+					*(p2+4)= *(p2+1) - vz*len2*h1[1];
+					*(p2+5)= *(p2+2) - vz*len2*h1[2];
+				}
+				else {
+					*(p2-3)= *(p2)   + vz*len1*h2[0];
+					*(p2-2)= *(p2+1) + vz*len1*h2[1];
+					*(p2-1)= *(p2+2) + vz*len1*h2[2];
+				}
+			}
+			
 		}
 	}
 
@@ -2029,6 +2028,7 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 	len1= VecLenf(p2, p2-3);
 	if(len1==0.0) len1=1.0;
 	if(len2==0.0) len2=1.0;
+
 	if(bezt->f1 & 1) { /* order of calculation */
 		if(bezt->h2==HD_ALIGN) {	/* aligned */
 			len= len2/len1;
@@ -2216,7 +2216,8 @@ void sethandlesNurb(short code)
 {
 	/* code==1: set autohandle */
 	/* code==2: set vectorhandle */
-	/* if code==3 (HD_ALIGN) it toggle, vectorhandles become HD_FREE */
+	/* code==3 (HD_ALIGN) it toggle, vectorhandles become HD_FREE */
+	/* code==4: sets icu flag to become IPO_AUTO_HORIZ, horizontal extremes on auto-handles */
 	Nurb *nu;
 	BezTriple *bezt;
 	short a, ok=0;
@@ -2281,7 +2282,7 @@ void sethandlesNurb(short code)
 	}
 }
 
-void swapdata(void *adr1, void *adr2, int len)
+static void swapdata(void *adr1, void *adr2, int len)
 {
 
 	if(len<=0) return;
@@ -2405,3 +2406,57 @@ void switchdirectionNurb(Nurb *nu)
 		}
 	}
 }
+
+
+float (*curve_getVertexCos(Curve *cu, ListBase *lb, int *numVerts_r))[3]
+{
+	int i, numVerts = *numVerts_r = count_curveverts(lb);
+	float *co, (*cos)[3] = MEM_mallocN(sizeof(*cos)*numVerts, "cu_vcos");
+	Nurb *nu;
+
+	co = cos[0];
+	for (nu=lb->first; nu; nu=nu->next) {
+		if ((nu->type & 7)==CU_BEZIER) {
+			BezTriple *bezt = nu->bezt;
+
+			for (i=0; i<nu->pntsu; i++,bezt++) {
+				VECCOPY(co, bezt->vec[0]); co+=3;
+				VECCOPY(co, bezt->vec[1]); co+=3;
+				VECCOPY(co, bezt->vec[2]); co+=3;
+			}
+		} else {
+			BPoint *bp = nu->bp;
+
+			for (i=0; i<nu->pntsu*nu->pntsv; i++,bp++) {
+				VECCOPY(co, bp->vec); co+=3;
+			}
+		}
+	}
+
+	return cos;
+}
+
+void curve_applyVertexCos(Curve *cu, ListBase *lb, float (*vertexCos)[3])
+{
+	float *co = vertexCos[0];
+	Nurb *nu;
+	int i;
+
+	for (nu=lb->first; nu; nu=nu->next) {
+		if ((nu->type & 7)==CU_BEZIER) {
+			BezTriple *bezt = nu->bezt;
+
+			for (i=0; i<nu->pntsu; i++,bezt++) {
+				VECCOPY(bezt->vec[0], co); co+=3;
+				VECCOPY(bezt->vec[1], co); co+=3;
+				VECCOPY(bezt->vec[2], co); co+=3;
+			}
+		} else {
+			BPoint *bp = nu->bp;
+
+			for (i=0; i<nu->pntsu*nu->pntsv; i++,bp++) {
+				VECCOPY(bp->vec, co); co+=3;
+			}
+		}
+	}
+}
