@@ -1,6 +1,6 @@
 /* ipo.c
  * 
- * $Id: ipo.c,v 1.18 2005/05/08 20:10:58 ton Exp $
+ * $Id: ipo.c,v 1.40 2006/07/04 10:19:26 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,22 +41,23 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_ika_types.h"
-#include "DNA_sequence_types.h"
+#include "DNA_action_types.h"
+#include "DNA_armature_types.h"
+#include "DNA_curve_types.h"
 #include "DNA_camera_types.h"
-#include "DNA_sound_types.h"
 #include "DNA_lamp_types.h"
-#include "DNA_view3d_types.h"
+#include "DNA_ipo_types.h"
 #include "DNA_key_types.h"
-#include "DNA_scene_types.h"
-#include "DNA_texture_types.h"
 #include "DNA_material_types.h"
+#include "DNA_mesh_types.h"
 #include "DNA_object_types.h"
 #include "DNA_object_force.h"
-#include "DNA_mesh_types.h"
-#include "DNA_curve_types.h"
-#include "DNA_ipo_types.h"
-#include "DNA_action_types.h"
+#include "DNA_sequence_types.h"
+#include "DNA_scene_types.h"
+#include "DNA_sound_types.h"
+#include "DNA_texture_types.h"
+#include "DNA_view3d_types.h"
+#include "DNA_world_types.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
@@ -64,15 +65,17 @@
 #include "BKE_bad_level_calls.h"
 #include "BKE_utildefines.h"
 
-#include "BKE_main.h"
-#include "BKE_global.h"
-#include "BKE_library.h"
-#include "BKE_curve.h"
-#include "BKE_object.h"
+#include "BKE_action.h"
 #include "BKE_blender.h"
-#include "BKE_ipo.h"
+#include "BKE_curve.h"
 #include "BKE_constraint.h"
+#include "BKE_global.h"
+#include "BKE_ipo.h"
+#include "BKE_library.h"
+#include "BKE_main.h"
 #include "BKE_mesh.h"
+#include "BKE_object.h"
+#include "BPY_extern.h" /* for BPY_pydriver_eval() */
 
 #define SMALL -1.0e-10
 
@@ -89,7 +92,7 @@ int ob_ar[OB_TOTIPO]= {
 	OB_LOC_X, OB_LOC_Y, OB_LOC_Z, OB_DLOC_X, OB_DLOC_Y, OB_DLOC_Z, 
 	OB_ROT_X, OB_ROT_Y, OB_ROT_Z, OB_DROT_X, OB_DROT_Y, OB_DROT_Z, 
 	OB_SIZE_X, OB_SIZE_Y, OB_SIZE_Z, OB_DSIZE_X, OB_DSIZE_Y, OB_DSIZE_Z, 
-	OB_LAY, OB_TIME, OB_EFF_X, OB_EFF_Y, OB_EFF_Z, OB_COL_A,
+	OB_LAY, OB_TIME, OB_COL_R, OB_COL_G, OB_COL_B, OB_COL_A,
 	OB_PD_FSTR, OB_PD_FFALL, OB_PD_SDAMP, OB_PD_RDAMP, OB_PD_PERM
 };
 
@@ -125,7 +128,9 @@ int te_ar[TE_TOTIPO] ={
 	
 	TE_MG_TYP, TE_MGH, TE_MG_LAC, TE_MG_OCT, TE_MG_OFF, TE_MG_GAIN,
 	
-	TE_N_BAS1, TE_N_BAS2
+	TE_N_BAS1, TE_N_BAS2,
+	
+	TE_COL_R, TE_COL_G, TE_COL_B, TE_BRIGHT, TE_CONTRA
 };
 
 int seq_ar[SEQ_TOTIPO]= {
@@ -136,15 +141,6 @@ int cu_ar[CU_TOTIPO]= {
 	CU_SPEED
 };
 
-int key_ar[KEY_TOTIPO]= {
-	KEY_SPEED, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 
-	11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 
-	21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
-	32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42,
-	43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53,
-	54, 55, 56, 57, 58, 59, 60, 61, 62, 63
-};
-
 int wo_ar[WO_TOTIPO]= {
 	WO_HOR_R, WO_HOR_G, WO_HOR_B, WO_ZEN_R, WO_ZEN_G, WO_ZEN_B, 
 	WO_EXPOS, WO_MISI, WO_MISTDI, WO_MISTSTA, WO_MISTHI,
@@ -176,43 +172,98 @@ int snd_ar[SND_TOTIPO]= {
 	SND_VOLUME, SND_PITCH, SND_PANNING, SND_ATTEN
 };
 
+int fluidsim_ar[FLUIDSIM_TOTIPO]= {
+	FLUIDSIM_VISC, FLUIDSIM_TIME,
+	FLUIDSIM_GRAV_X , FLUIDSIM_GRAV_Y , FLUIDSIM_GRAV_Z ,
+	FLUIDSIM_VEL_X  , FLUIDSIM_VEL_Y  , FLUIDSIM_VEL_Z  ,
+	FLUIDSIM_ACTIVE 
+};
+
 
 
 float frame_to_float(int cfra)		/* see also bsystem_time in object.c */
 {
-	extern float bluroffs;	/* object.c */
+	extern float bluroffs;	/* bad stuff borrowed from object.c */
+	extern float fieldoffs;
 	float ctime;
 	
 	ctime= (float)cfra;
-	if(R.flag & R_SEC_FIELD) {
-		if((R.r.mode & R_FIELDSTILL)==0) ctime+= 0.5;
-	}
-	ctime+= bluroffs;
+	ctime+= bluroffs+fieldoffs;
 	ctime*= G.scene->r.framelen;
 	
 	return ctime;
 }
 
+/* includes ipo curve itself */
+void free_ipo_curve(IpoCurve *icu) 
+{
+	if(icu->bezt) MEM_freeN(icu->bezt);
+	if(icu->bp) MEM_freeN(icu->bp);
+	if(icu->driver) MEM_freeN(icu->driver);
+	MEM_freeN(icu);
+}
+
 /* do not free ipo itself */
 void free_ipo(Ipo *ipo)
 {
 	IpoCurve *icu;
 	
-	icu= ipo->curve.first;
-	while(icu) {
-		if(icu->bezt) MEM_freeN(icu->bezt);
-		icu= icu->next;
+	while( (icu= ipo->curve.first) ) {
+		BLI_remlink(&ipo->curve, icu);
+		free_ipo_curve(icu);
 	}
-	BLI_freelistN(&ipo->curve);
 }
 
+/* on adding new ipos, or for empty views */
+void ipo_default_v2d_cur(int blocktype, rctf *cur)
+{
+	if(blocktype==ID_CA) {
+		cur->xmin= G.scene->r.sfra;
+		cur->xmax= G.scene->r.efra;
+		cur->ymin= 0.0;
+		cur->ymax= 100.0;
+	}
+	else if ELEM5(blocktype, ID_MA, ID_CU, ID_WO, ID_LA, ID_CO) {
+		cur->xmin= (float)G.scene->r.sfra-0.1;
+		cur->xmax= G.scene->r.efra;
+		cur->ymin= (float)-0.1;
+		cur->ymax= (float)+1.1;
+	}
+	else if(blocktype==ID_TE) {
+		cur->xmin= (float)G.scene->r.sfra-0.1;
+		cur->xmax= G.scene->r.efra;
+		cur->ymin= (float)-0.1;
+		cur->ymax= (float)+1.1;
+	}
+	else if(blocktype==ID_SEQ) {
+		cur->xmin= -5.0;
+		cur->xmax= 105.0;
+		cur->ymin= (float)-0.1;
+		cur->ymax= (float)+1.1;
+	}
+	else if(blocktype==ID_KE) {
+		cur->xmin= (float)G.scene->r.sfra-0.1;
+		cur->xmax= G.scene->r.efra;
+		cur->ymin= (float)-0.1;
+		cur->ymax= (float)+2.1;
+	}
+	else {  /* ID_OB and everything else */
+		cur->xmin= G.scene->r.sfra;
+		cur->xmax= G.scene->r.efra;
+		cur->ymin= -5.0;
+		cur->ymax= +5.0;
+	}
+}
+
+
 Ipo *add_ipo(char *name, int idcode)
 {
 	Ipo *ipo;
 	
 	ipo= alloc_libblock(&G.main->ipo, ID_IP, name);
 	ipo->blocktype= idcode;
-	
+	ipo_default_v2d_cur(idcode, &ipo->cur);
+
 	return ipo;
 }
 
@@ -221,16 +272,15 @@ Ipo *copy_ipo(Ipo *ipo)
 	Ipo *ipon;
 	IpoCurve *icu;
 	
-	if(ipo==0) return 0;
+	if(ipo==NULL) return 0;
 	
 	ipon= copy_libblock(ipo);
 	
 	duplicatelist(&(ipon->curve), &(ipo->curve));
 
-	icu= ipon->curve.first;
-	while(icu) {
+	for(icu= ipo->curve.first; icu; icu= icu->next) {
 		icu->bezt= MEM_dupallocN(icu->bezt);
-		icu= icu->next;
+		if(icu->driver) icu->driver= MEM_dupallocN(icu->driver);
 	}
 	
 	return ipon;
@@ -269,7 +319,7 @@ void make_local_obipo(Ipo *ipo)
 		while(ob) {
 			if(ob->ipo==ipo) {
 				
-				if(ob->id.lib==0) {
+				if(ob->id.lib==NULL) {
 					ob->ipo= ipon;
 					ipon->id.us++;
 					ipo->id.us--;
@@ -313,7 +363,7 @@ void make_local_matipo(Ipo *ipo)
 		while(ma) {
 			if(ma->ipo==ipo) {
 				
-				if(ma->id.lib==0) {
+				if(ma->id.lib==NULL) {
 					ma->ipo= ipon;
 					ipon->id.us++;
 					ipo->id.us--;
@@ -357,7 +407,7 @@ void make_local_keyipo(Ipo *ipo)
 		while(key) {
 			if(key->ipo==ipo) {
 				
-				if(key->id.lib==0) {
+				if(key->id.lib==NULL) {
 					key->ipo= ipon;
 					ipon->id.us++;
 					ipo->id.us--;
@@ -372,7 +422,7 @@ void make_local_keyipo(Ipo *ipo)
 void make_local_ipo(Ipo *ipo)
 {
 	
-	if(ipo->id.lib==0) return;
+	if(ipo->id.lib==NULL) return;
 	if(ipo->id.us==1) {
 		ipo->id.lib= 0;
 		ipo->id.flag= LIB_LOCAL;
@@ -386,6 +436,16 @@ void make_local_ipo(Ipo *ipo)
 
 }
 
+IpoCurve *find_ipocurve(Ipo *ipo, int adrcode)
+{
+	if(ipo) {
+		IpoCurve *icu;
+		for(icu= ipo->curve.first; icu; icu= icu->next) {
+			if(icu->adrcode==adrcode) return icu;
+		}
+	}
+	return NULL;
+}
 
 void calchandles_ipocurve(IpoCurve *icu)
 {
@@ -404,8 +464,11 @@ void calchandles_ipocurve(IpoCurve *icu)
 		if(bezt->vec[0][0]>bezt->vec[1][0]) bezt->vec[0][0]= bezt->vec[1][0];
 		if(bezt->vec[2][0]<bezt->vec[1][0]) bezt->vec[2][0]= bezt->vec[1][0];
 
-		calchandleNurb(bezt, prev, next, 1);	/* 1==special autohandle */
-
+		if(icu->flag & IPO_AUTO_HORIZ) 
+			calchandleNurb(bezt, prev, next, 2);	/* 2==special autohandle && keep extrema horizontal */
+		else
+			calchandleNurb(bezt, prev, next, 1);	/* 1==special autohandle */
+	
 		prev= bezt;
 		if(a==1) {
 			next= 0;
@@ -438,7 +501,7 @@ void testhandles_ipocurve(IpoCurve *icu)
 	int flag, a;
 
 	bezt= icu->bezt;
-	if(bezt==0) return;
+	if(bezt==NULL) return;
 	
 	a= icu->totvert;
 	while(a--) {
@@ -655,6 +718,7 @@ void berekeny(float f1, float f2, float f3, float f4, float *o, int b)
 		o[a]= c0+t*c1+t*t*c2+t*t*t*c3;
 	}
 }
+
 void berekenx(float *f, float *o, int b)
 {
 	float t, c0, c1, c2, c3;
@@ -670,6 +734,104 @@ void berekenx(float *f, float *o, int b)
 	}
 }
 
+/* has to return a float value */
+static float eval_driver(IpoDriver *driver, float ipotime)
+{
+	
+	if(driver->type == IPO_DRIVER_TYPE_PYTHON) {
+		/* check for empty or invalid expression */
+		if ((driver->name[0] == '\0') ||
+			(driver->flag & IPO_DRIVER_FLAG_INVALID))
+			return 0.0f;
+		/* this evals the expression and returns its result:
+		 * (on errors it reports, then returns 0.0f) */
+		return BPY_pydriver_eval(driver);
+	}
+	else {
+		Object *ob= driver->ob;
+
+		if(ob==NULL) return 0.0f;
+		
+		if(driver->blocktype==ID_OB) {
+			/* depsgraph failure; ob ipos are calculated in where_is_object, this might get called too late */
+			if(ob->ipo && ob->ctime!=ipotime) {
+				calc_ipo_spec(ob->ipo, driver->adrcode, &ipotime);
+				return ipotime;
+			}
+			
+			switch(driver->adrcode) {
+			case OB_LOC_X:
+				return ob->loc[0];
+			case OB_LOC_Y:
+				return ob->loc[1];
+			case OB_LOC_Z:
+				return ob->loc[2];
+			case OB_ROT_X:
+				return ob->rot[0]/(M_PI_2/9.0);
+			case OB_ROT_Y:
+				return ob->rot[1]/(M_PI_2/9.0);
+			case OB_ROT_Z:
+				return ob->rot[2]/(M_PI_2/9.0);
+			case OB_SIZE_X:
+				return ob->size[0];
+			case OB_SIZE_Y:
+				return ob->size[1];
+			case OB_SIZE_Z:
+				return ob->size[2];
+			}
+		}
+		else {	/* ID_AR */
+			bPoseChannel *pchan= get_pose_channel(ob->pose, driver->name);
+			if(pchan && pchan->bone) {
+				float pose_mat[3][3];
+				float diff_mat[3][3], par_mat[3][3], ipar_mat[3][3];
+				float eul[3], size[3];
+				
+				/* we need the local transform = current transform - (parent transform + bone transform) */
+				
+				Mat3CpyMat4(pose_mat, pchan->pose_mat);
+				
+				if (pchan->parent) {
+					Mat3CpyMat4(par_mat, pchan->parent->pose_mat);
+					Mat3MulMat3(diff_mat, par_mat, pchan->bone->bone_mat);
+					
+					Mat3Inv(ipar_mat, diff_mat);
+				}
+				else {
+					Mat3Inv(ipar_mat, pchan->bone->bone_mat);
+				}
+				
+				Mat3MulMat3(diff_mat, ipar_mat, pose_mat);
+				
+				Mat3ToEul(diff_mat, eul);
+				Mat3ToSize(diff_mat, size);
+
+				switch(driver->adrcode) {
+				case OB_LOC_X:
+					return pchan->loc[0];
+				case OB_LOC_Y:
+					return pchan->loc[1];
+				case OB_LOC_Z:
+					return pchan->loc[2];
+				case OB_ROT_X:
+					return eul[0]/(M_PI_2/9.0);
+				case OB_ROT_Y:
+					return eul[1]/(M_PI_2/9.0);
+				case OB_ROT_Z:
+					return eul[2]/(M_PI_2/9.0);
+				case OB_SIZE_X:
+					return size[0];
+				case OB_SIZE_Y:
+					return size[1];
+				case OB_SIZE_Z:
+					return size[2];
+				}
+			}
+		}
+	}	
+	return 0.0f;
+}
+
 float eval_icu(IpoCurve *icu, float ipotime) 
 {
 	BezTriple *bezt, *prevbezt;
@@ -679,6 +841,10 @@ float eval_icu(IpoCurve *icu, float ipotime)
 	
 	cycyofs= 0.0;
 	
+	if(icu->driver) {
+		/* ipotime now serves as input for the curve */
+		ipotime= cvalue= eval_driver(icu->driver, ipotime);
+	}
 	if(icu->bezt) {
 		prevbezt= icu->bezt;
 		bezt= prevbezt+1;
@@ -809,12 +975,11 @@ void calc_ipo(Ipo *ipo, float ctime)
 {
 	IpoCurve *icu;
 	
-	icu= ipo->curve.first;
-	while(icu) {
-		
-		if( (icu->flag & IPO_LOCK)==0) calc_icu(icu, ctime);
-		
-		icu= icu->next;
+	if(ipo==NULL) return;
+	
+	for(icu= ipo->curve.first; icu; icu= icu->next) {
+		if(icu->driver || (icu->flag & IPO_LOCK)==0) 
+			calc_icu(icu, ctime);
 	}
 }
 
@@ -924,6 +1089,17 @@ static void *give_tex_poin(Tex *tex, int adrcode, int *type )
 		poin= &(tex->noisebasis); *type= IPO_SHORT; break;
 	case TE_N_BAS2:
 		poin= &(tex->noisebasis2); *type= IPO_SHORT; break;
+	case TE_COL_R:
+		poin= &(tex->rfac); break;
+	case TE_COL_G:
+		poin= &(tex->gfac); break;
+	case TE_COL_B:
+		poin= &(tex->bfac); break;
+	case TE_BRIGHT:
+		poin= &(tex->bright); break;
+	case TE_CONTRA:
+		poin= &(tex->contrast); break;
+
 	}
 	
 	return poin;
@@ -990,14 +1166,11 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 	Lamp *la;
 	Sequence *seq;
 	World *wo;
-	bAction			*act;
-	bActionChannel	*achan;
-	bPoseChannel	*pchan;
 
 	*type= IPO_FLOAT;
 
 	if( GS(id->name)==ID_OB) {
-	
+		
 		ob= (Object *)id;
 
 		switch(icu->adrcode) {
@@ -1072,36 +1245,6 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			break;
 		}
 	}
-	else if (GS(id->name)==ID_AC){
-		act= (bAction *)id;
-		achan = act->achan;
-		pchan = act->pchan;
-		if (!pchan || !achan)
-			return NULL;
-		switch (icu->adrcode){
-		case AC_QUAT_W:
-			poin= &(pchan->quat[0]); break;
-		case AC_QUAT_X:
-			poin= &(pchan->quat[1]); break;
-		case AC_QUAT_Y:
-			poin= &(pchan->quat[2]); break;
-		case AC_QUAT_Z:
-			poin= &(pchan->quat[3]); break;
-		case AC_LOC_X:
-			poin= &(pchan->loc[0]); break;
-		case AC_LOC_Y:
-			poin= &(pchan->loc[1]); break;
-		case AC_LOC_Z:
-			poin= &(pchan->loc[2]); break;			
-		case AC_SIZE_X:
-			poin= &(pchan->size[0]); break;
-		case AC_SIZE_Y:
-			poin= &(pchan->size[1]); break;
-		case AC_SIZE_Z:
-			poin= &(pchan->size[2]); break;
-		};
-	}
-
 	else if( GS(id->name)==ID_MA) {
 		
 		ma= (Material *)id;
@@ -1161,7 +1304,7 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			poin= &(ma->add); break;
 		}
 		
-		if(poin==0) {
+		if(poin==NULL) {
 			mtex= 0;
 			if(icu->adrcode & MA_MAP1) mtex= ma->mtex[0];
 			else if(icu->adrcode & MA_MAP2) mtex= ma->mtex[1];
@@ -1198,8 +1341,13 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 		
 	}
 	else if( GS(id->name)==ID_KE) {
+		KeyBlock *kb= ((Key *)id)->block.first;
 		
-		poin= &(icu->curval);
+		for(; kb; kb= kb->next)
+			if(kb->adrcode==icu->adrcode)
+				break;
+		if(kb)
+			poin= &(kb->curval);
 		
 	}
 	else if(GS(id->name)==ID_WO) {
@@ -1245,7 +1393,7 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			poin= &(wo->starsize); break;
 		}
 
-		if(poin==0) {
+		if(poin==NULL) {
 			mtex= 0;
 			if(icu->adrcode & MA_MAP1) mtex= wo->mtex[0];
 			else if(icu->adrcode & MA_MAP2) mtex= wo->mtex[1];
@@ -1290,7 +1438,7 @@ void *get_ipo_poin(ID *id, IpoCurve *icu, int *type)
 			poin= &(la->haint); break;
 		}
 		
-		if(poin==0) {
+		if(poin==NULL) {
 			mtex= 0;
 			if(icu->adrcode & MA_MAP1) mtex= la->mtex[0];
 			else if(icu->adrcode & MA_MAP2) mtex= la->mtex[1];
@@ -1364,7 +1512,7 @@ void set_icu_vars(IpoCurve *icu)
 			case MA_HASIZE:
 				icu->ymax= 10000.0; break;
 			case MA_HARD:
-				icu->ymax= 128.0; break;
+				icu->ymax= 511.0; break;
 			case MA_SPEC:
 				icu->ymax= 2.0; break;
 			case MA_MODE:
@@ -1460,7 +1608,7 @@ void set_icu_vars(IpoCurve *icu)
 			case TE_MG_TYP:
 				icu->vartype= IPO_SHORT;
 				icu->ipo= IPO_CONST;
-				icu->ymax= 4.0; break;
+				icu->ymax= 6.0; break;
 			case TE_MGH:
 				icu->ymin= 0.0001;
 				icu->ymax= 2.0; break;
@@ -1475,6 +1623,17 @@ void set_icu_vars(IpoCurve *icu)
 				icu->vartype= IPO_SHORT;
 				icu->ipo= IPO_CONST;
 				icu->ymax= 8.0; break;
+			case TE_COL_R:
+				icu->ymax= 0.0; break;
+			case TE_COL_G:
+				icu->ymax= 2.0; break;
+			case TE_COL_B:
+				icu->ymax= 2.0; break;
+			case TE_BRIGHT:
+				icu->ymax= 2.0; break;
+			case TE_CONTRA:
+				icu->ymax= 5.0; break;				
+
 		}
 	}
 	else if(icu->blocktype==ID_SEQ) {
@@ -1618,20 +1777,83 @@ void set_icu_vars(IpoCurve *icu)
 	}
 }
 
-
+/* not for actions or constraints! */
 void execute_ipo(ID *id, Ipo *ipo)
 {
 	IpoCurve *icu;
 	void *poin;
 	int type;
 	
-	if(ipo==0) return;
+	if(ipo==NULL) return;
 	
-	icu= ipo->curve.first;
-	while(icu) {
+	for(icu= ipo->curve.first; icu; icu= icu->next) {
 		poin= get_ipo_poin(id, icu, &type);
 		if(poin) write_ipo_poin(poin, type, icu->curval);
-		icu= icu->next;
+	}
+}
+
+void *get_pchan_ipo_poin(bPoseChannel *pchan, int adrcode)
+{
+	void *poin= NULL;
+	
+	switch (adrcode) {
+		case AC_QUAT_W:
+			poin= &(pchan->quat[0]); 
+			pchan->flag |= POSE_ROT;
+			break;
+		case AC_QUAT_X:
+			poin= &(pchan->quat[1]); 
+			pchan->flag |= POSE_ROT;
+			break;
+		case AC_QUAT_Y:
+			poin= &(pchan->quat[2]); 
+			pchan->flag |= POSE_ROT;
+			break;
+		case AC_QUAT_Z:
+			poin= &(pchan->quat[3]); 
+			pchan->flag |= POSE_ROT;
+			break;
+		case AC_LOC_X:
+			poin= &(pchan->loc[0]); 
+			pchan->flag |= POSE_LOC;
+			break;
+		case AC_LOC_Y:
+			poin= &(pchan->loc[1]); 
+			pchan->flag |= POSE_LOC;
+			break;
+		case AC_LOC_Z:
+			poin= &(pchan->loc[2]); 
+			pchan->flag |= POSE_LOC;
+			break;			
+		case AC_SIZE_X:
+			poin= &(pchan->size[0]); 
+			pchan->flag |= POSE_SIZE;
+			break;
+		case AC_SIZE_Y:
+			poin= &(pchan->size[1]); 
+			pchan->flag |= POSE_SIZE;
+			break;
+		case AC_SIZE_Z:
+			poin= &(pchan->size[2]); 
+			pchan->flag |= POSE_SIZE;
+			break;
+	}
+	return poin;
+}
+
+void execute_action_ipo(bActionChannel *achan, bPoseChannel *pchan)
+{
+
+	if(achan && achan->ipo) {
+		IpoCurve *icu;
+		for(icu= achan->ipo->curve.first; icu; icu= icu->next) {
+			void *poin= get_pchan_ipo_poin(pchan, icu->adrcode);
+			if(poin) {
+				write_ipo_poin(poin, IPO_FLOAT, icu->curval);
+				//printf("execute_action_ipo wrote_ipo_poin: %f\n", icu->curval);
+				//printf("%s has poin %p value %f\n", achan->name, poin, icu->curval);
+			}
+		}
 	}
 }
 
@@ -1648,7 +1870,7 @@ void do_ipo_nocalc(Ipo *ipo)
 	Camera *ca;
 	bSound *snd;
 	
-	if(ipo==0) return;
+	if(ipo==NULL) return;
 	
 	switch(ipo->blocktype) {
 	case ID_OB:
@@ -1722,7 +1944,7 @@ void do_mat_ipo(Material *ma)
 {
 	float ctime;
 	
-	if(ma==0 || ma->ipo==0) return;
+	if(ma==NULL || ma->ipo==NULL) return;
 	
 	ctime= frame_to_float(G.scene->r.cfra);
 	/* if(ob->ipoflag & OB_OFFS_OB) ctime-= ob->sf; */
@@ -1737,7 +1959,7 @@ void do_ob_ipo(Object *ob)
 	float ctime;
 	unsigned int lay;
 	
-	if(ob->ipo==0) return;
+	if(ob->ipo==NULL) return;
 
 	/* do not set ob->ctime here: for example when parent in invisible layer */
 	
@@ -1760,6 +1982,21 @@ void do_ob_ipo(Object *ob)
 	}
 }
 
+void do_ob_ipodrivers(Object *ob, Ipo *ipo, float ctime)
+{
+	IpoCurve *icu;
+	void *poin;
+	int type;
+	
+	for(icu= ipo->curve.first; icu; icu= icu->next) {
+		if(icu->driver) {
+			icu->curval= eval_icu(icu, ctime);
+			poin= get_ipo_poin((ID *)ob, icu, &type);
+			if(poin) write_ipo_poin(poin, type, icu->curval);
+		}
+	}
+}
+
 void do_seq_ipo(Sequence *seq)
 {
 	float ctime, div;
@@ -1767,9 +2004,15 @@ void do_seq_ipo(Sequence *seq)
 	/* seq_ipo has an exception: calc both fields immediately */
 	
 	if(seq->ipo) {
-		ctime= frame_to_float(G.scene->r.cfra - seq->startdisp);
-		div= (seq->enddisp - seq->startdisp)/100.0f;
-		if(div==0) return;
+		if((seq->flag & SEQ_IPO_FRAME_LOCKED) != 0) {
+			ctime = frame_to_float(G.scene->r.cfra);
+			div = 1.0;
+		} else {
+			ctime= frame_to_float(G.scene->r.cfra 
+					      - seq->startdisp);
+			div= (seq->enddisp - seq->startdisp)/100.0f;
+			if(div==0.0) return;
+		}
 		
 		/* 2nd field */
 		calc_ipo(seq->ipo, (ctime+0.5f)/div);
@@ -1788,35 +2031,51 @@ int has_ipo_code(Ipo *ipo, int code)
 {
 	IpoCurve *icu;
 	
-	if(ipo==0) return 0;
-	
-	icu= ipo->curve.first;
-	while(icu) {
+	if(ipo==NULL) return 0;
 	
+	for(icu= ipo->curve.first; icu; icu= icu->next) {
 		if(icu->adrcode==code) return 1;
-		
-		icu= icu->next;
 	}
 	return 0;
 }
 
-void do_all_ipos()
+void do_all_data_ipos()
 {
-	Base *base;
 	Material *ma;
 	Tex *tex;
 	World *wo;
 	Ipo *ipo;
 	Lamp *la;
+	Key *key;
 	Camera *ca;
 	bSound *snd;
 	Sequence *seq;
 	Editing *ed;
+	Base *base;
 	float ctime;
-	int set;
 
 	ctime= frame_to_float(G.scene->r.cfra);
 	
+	/* this exception cannot be depgraphed yet... what todo with objects in other layers?... */
+	for(base= G.scene->base.first; base; base= base->next) {
+		/* only update layer when an ipo */
+		if( has_ipo_code(base->object->ipo, OB_LAY) ) {
+			do_ob_ipo(base->object);
+			base->lay= base->object->lay;
+		}
+	}
+	
+	/* layers for the set...*/
+	if(G.scene->set) {
+		for(base= G.scene->set->base.first; base; base= base->next) {
+			if( has_ipo_code(base->object->ipo, OB_LAY) ) {
+				do_ob_ipo(base->object);
+				base->lay= base->object->lay;
+			}
+		}
+	}
+	
+	
 	ipo= G.main->ipo.first;
 	while(ipo) {
 		if(ipo->id.us && ipo->blocktype!=ID_OB) {
@@ -1825,45 +2084,22 @@ void do_all_ipos()
 		ipo= ipo->id.next;
 	}
 
-	/* NEW: current scene ob ipo's */
-	base= G.scene->base.first;
-	set= 0;
-	while(base) {
-
-		/* Do object ipos */
-		do_constraint_channels(&base->object->constraints, &base->object->constraintChannels, ctime);
-
-		if(base->object->ipo) {
-			/* do per object ipo the calc_ipo: because of possible timeoffs */
-			do_ob_ipo(base->object);
-			if(base->object->type==OB_MBALL) where_is_object(base->object);
-		}
-		base= base->next;
-		
-		if(base==0 && set==0 && G.scene->set) {
-			set= 1;
-			base= G.scene->set->base.first;
-		}
-	}
-
-	tex= G.main->tex.first;
-	while(tex) {
+	for(tex= G.main->tex.first; tex; tex= tex->id.next) {
 		if(tex->ipo) execute_ipo((ID *)tex, tex->ipo);
-		tex= tex->id.next;
 	}
 
-	ma= G.main->mat.first;
-	while(ma) {
+	for(ma= G.main->mat.first; ma; ma= ma->id.next) {
 		if(ma->ipo) execute_ipo((ID *)ma, ma->ipo);
-		ma= ma->id.next;
 	}
 
-	wo= G.main->world.first;
-	while(wo) {
+	for(wo= G.main->world.first; wo; wo= wo->id.next) {
 		if(wo->ipo) execute_ipo((ID *)wo, wo->ipo);
-		wo= wo->id.next;
 	}
-
+	
+	for(key= G.main->key.first; key; key= key->id.next) {
+		if(key->ipo) execute_ipo((ID *)key, key->ipo);
+	}
+	
 	la= G.main->lamp.first;
 	while(la) {
 		if(la->ipo) execute_ipo((ID *)la, la->ipo);
@@ -1882,39 +2118,16 @@ void do_all_ipos()
 		snd= snd->id.next;
 	}
 
-	/*just in case of...  WATCH IT: 2x */
-	base= G.scene->base.first;
-	while(base) {
-		
-		/* only update layer when an ipo */
-		if( has_ipo_code(base->object->ipo, OB_LAY) ) {
-			base->lay= base->object->lay;
-		}
-		
-		base= base->next;
-	}
-	
-	/* just in case...*/
-	if(G.scene->set) {
-		base= G.scene->set->base.first;
-		while(base) {
-			
-			/* only update layer when an ipo */
-			if( has_ipo_code(base->object->ipo, OB_LAY) ) {
-				base->lay= base->object->lay;
-			}
-			
-			base= base->next;
-		}
-	}
-
-	/* intrr: process FAC Ipos used as volume envelopes */
+	/* process FAC Ipos used as volume envelopes */
 	ed= G.scene->ed;
 	if (ed) {
 		seq= ed->seqbasep->first;
 		while(seq) {
-			if ((seq->type == SEQ_SOUND) && (seq->ipo)
-			  &&(seq->startdisp<=G.scene->r.cfra+2) && (seq->enddisp>G.scene->r.cfra)) do_seq_ipo(seq);
+			if ((seq->type == SEQ_RAM_SOUND
+			     || seq->type == SEQ_HD_SOUND) && (seq->ipo) && 
+				(seq->startdisp<=G.scene->r.cfra+2) && 
+			    (seq->enddisp>G.scene->r.cfra)) 
+					do_seq_ipo(seq);
 			seq= seq->next;
 		}
 	}
@@ -1926,10 +2139,9 @@ int calc_ipo_spec(Ipo *ipo, int adrcode, float *ctime)
 {
 	IpoCurve *icu;
 
-	if(ipo==0) return 0;
+	if(ipo==NULL) return 0;
 
-	icu= ipo->curve.first;
-	while(icu) {
+	for(icu= ipo->curve.first; icu; icu= icu->next) {
 		if(icu->adrcode == adrcode) {
 			if(icu->flag & IPO_LOCK);
 			else calc_icu(icu, *ctime);
@@ -1937,7 +2149,6 @@ int calc_ipo_spec(Ipo *ipo, int adrcode, float *ctime)
 			*ctime= icu->curval;
 			return 1;
 		}
-		icu= icu->next;
 	}
 	
 	return 0;
@@ -1950,11 +2161,11 @@ void clear_delta_obipo(Ipo *ipo)
 {
 	Object *ob;
 	
-	if(ipo==0) return;
+	if(ipo==NULL) return;
 	
 	ob= G.main->object.first;
 	while(ob) {
-		if(ob->id.lib==0) {
+		if(ob->id.lib==NULL) {
 			if(ob->ipo==ipo) {
 				memset(&ob->dloc, 0, 12);
 				memset(&ob->drot, 0, 12);
@@ -2000,8 +2211,7 @@ void make_cfra_list(Ipo *ipo, ListBase *elems)
 	int a;
 	
 	if(ipo->blocktype==ID_OB) {
-		icu= ipo->curve.first;
-		while(icu) {
+		for(icu= ipo->curve.first; icu; icu= icu->next) {
 			if(icu->flag & IPO_VISIBLE) {
 				switch(icu->adrcode) {
 				case OB_DLOC_X:
@@ -2039,12 +2249,10 @@ void make_cfra_list(Ipo *ipo, ListBase *elems)
 					break;
 				}
 			}
-			icu= icu->next;
 		}
 	}
 	else if(ipo->blocktype==ID_AC) {
-		icu= ipo->curve.first;
-		while(icu) {
+		for(icu= ipo->curve.first; icu; icu= icu->next) {
 			if(icu->flag & IPO_VISIBLE) {
 				switch(icu->adrcode) {
 				case AC_LOC_X:
@@ -2068,7 +2276,6 @@ void make_cfra_list(Ipo *ipo, ListBase *elems)
 					break;
 				}
 			}
-			icu= icu->next;
 		}
 	}
 	else {
@@ -2108,14 +2315,10 @@ int IPO_GetChannels(Ipo *ipo, IPO_Channel *channels)
 	
 	if(ipo==NULL) return 0;
 	
-	icu= ipo->curve.first;
-	while(icu) {
-		
+	for(icu= ipo->curve.first; icu; icu= icu->next) {
 		channels[total]= icu->adrcode;
 		total++;
 		if(total>31) break;
-		
-		icu= icu->next;
 	}
 	
 	return total;
@@ -2137,25 +2340,3 @@ float IPO_GetFloatValue(Ipo *ipo, IPO_Channel channel, float ctime)
 
 	return ctime;
 }
-
-
-void test_ipo_get()
-{
-	Object *ob;
-	int tot;
-	IPO_Channel chan[32];
-
-	ob = (G.scene->basact ? G.scene->basact->object : 0);
-	
-	if(ob==NULL) return;
-	if(ob->ipo==NULL) return;
-	
-	tot= IPO_GetChannels(ob->ipo, chan);
-	printf("tot %d \n", tot);
-	
-	while(tot--) {
-		printf("var1 %d \n", chan[tot]);
-	}
-	
-	printf("var1 %f \n", IPO_GetFloatValue(ob->ipo, chan[0], 10.0));
-}
