@@ -1,11 +1,6 @@
-
-/*  library.c 
+/** 
+ * $Id: library.c,v 1.19 2006/07/07 17:08:03 ton Exp $
  * 
- *  Contains management of ID's and libraries
- *  allocate and free of all library data
- * 
- * $Id: library.c,v 1.12 2005/04/02 15:36:56 zuster Exp $
- *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -34,6 +29,14 @@
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
+
+/*
+ *  Contains management of ID's and libraries
+ *  allocate and free of all library data
+ * 
+ */
+
+
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -56,7 +59,6 @@
 #include "DNA_meta_types.h"
 #include "DNA_material_types.h"
 #include "DNA_texture_types.h"
-#include "DNA_ika_types.h"
 #include "DNA_image_types.h"
 #include "DNA_wave_types.h"
 #include "DNA_lamp_types.h"
@@ -71,6 +73,10 @@
 #include "DNA_group_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_action_types.h"
+#include "DNA_userdef_types.h"
+#include "DNA_node_types.h"
+#include "DNA_nla_types.h"
+#include "DNA_effect_types.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_dynstr.h"
@@ -90,8 +96,8 @@
 #include "BKE_text.h"
 #include "BKE_texture.h"
 #include "BKE_scene.h"
+#include "BKE_icons.h"
 #include "BKE_image.h"
-#include "BKE_ika.h"
 #include "BKE_ipo.h"
 #include "BKE_key.h"
 #include "BKE_world.h"
@@ -100,9 +106,12 @@
 #include "BKE_lattice.h"
 #include "BKE_armature.h"
 #include "BKE_action.h"
+#include "BKE_node.h"
+#include "BKE_effect.h"
+
 #include "BPI_script.h"
 
-#define MAX_IDPUP		30	/* was 24 */
+#define MAX_IDPUP		60	/* was 24 */
 #define MAX_LIBARRAY	100 /* was 30, warning: used it readfile.c too */
 
 /* ************* general ************************ */
@@ -149,8 +158,6 @@ ListBase *wich_libbase(Main *mainlib, short type)
 			return &(mainlib->tex);
 		case ID_IM:
 			return &(mainlib->image);
-		case ID_IK:
-			return &(mainlib->ika);
 		case ID_WV:
 			return &(mainlib->wave);
 		case ID_LT:
@@ -184,6 +191,8 @@ ListBase *wich_libbase(Main *mainlib, short type)
 			return &(mainlib->armature);
 		case ID_AC:
 			return &(mainlib->action);
+		case ID_NT:
+			return &(mainlib->nodetree);
 	}
 	return 0;
 }
@@ -209,24 +218,25 @@ int set_listbasepointers(Main *main, ListBase **lb)
 	lb[8]= &(main->mesh);
 	lb[9]= &(main->curve);
 	lb[10]= &(main->mball);
-	lb[11]= &(main->ika);
-	lb[12]= &(main->wave);
-	lb[13]= &(main->latt);
-	lb[14]= &(main->lamp);
-	lb[15]= &(main->camera);
 
-	lb[16]= &(main->world);
-	lb[17]= &(main->screen);
-	lb[18]= &(main->object);
-	lb[19]= &(main->scene);
-	lb[20]= &(main->library);
-	lb[21]= &(main->text);
-	lb[22]= &(main->sound);
-	lb[23]= &(main->group);
+	lb[11]= &(main->wave);
+	lb[12]= &(main->latt);
+	lb[13]= &(main->lamp);
+	lb[14]= &(main->camera);
+
+	lb[15]= &(main->world);
+	lb[16]= &(main->screen);
+	lb[17]= &(main->object);
+	lb[18]= &(main->scene);
+	lb[19]= &(main->library);
+	lb[20]= &(main->text);
+	lb[21]= &(main->sound);
+	lb[22]= &(main->group);
+	lb[23]= &(main->nodetree);
 
 	lb[24]= samples;
 	lb[25]= &(main->script);
-	lb[26]=0;
+	lb[26]= NULL;
 
 	return 26;
 }
@@ -243,7 +253,7 @@ void *alloc_libblock(ListBase *lb, type, name)
 
 static ID *alloc_libblock_notest(short type)
 {
-	ID *id= 0;
+	ID *id= NULL;
 	
 	switch( type ) {
 		case ID_SCE:
@@ -273,9 +283,6 @@ static ID *alloc_libblock_notest(short type)
 		case ID_IM:
 			id= MEM_callocN(sizeof(Image), "image");
 			break;
-		case ID_IK:
-			id= MEM_callocN(sizeof(Ika), "ika");
-			break;
 		case ID_WV:
 			id= MEM_callocN(sizeof(Wave), "wave");
 			break;
@@ -324,19 +331,23 @@ static ID *alloc_libblock_notest(short type)
 		case ID_AC:
 			id = MEM_callocN(sizeof(bAction), "action");
 			break;
+		case ID_NT:
+			id = MEM_callocN(sizeof(bNodeTree), "action");
+			break;
 	}
 	return id;
 }
 
 // used everywhere in blenkernel and text.c
-void *alloc_libblock(ListBase *lb, short type, char *name)
+void *alloc_libblock(ListBase *lb, short type, const char *name)
 {
-	ID *id= 0;
+	ID *id= NULL;
 	
 	id= alloc_libblock_notest(type);
 	if(id) {
 		BLI_addtail(lb, id);
 		id->us= 1;
+		id->icon_id = 0;
 		*( (short *)id->name )= type;
 		new_id(lb, id, name);
 		/* alphabetic insterion: is in new_id */
@@ -344,19 +355,13 @@ void *alloc_libblock(ListBase *lb, short type, char *name)
 	return id;
 }
 
-/* GS reads the memory pointed at in a specific ordering. There are,
- * however two definitions for it. I have jotted them down here, both,
- * but I think the first one is actually used. The thing is that
- * big-endian systems might read this the wrong way round. OTOH, we
- * constructed the IDs that are read out with this macro explicitly as
- * well. I expect we'll sort it out soon... */
+/* GS reads the memory pointed at in a specific ordering. 
+   only use this definition, makes little and big endian systems
+   work fine, in conjunction with MAKE_ID */
 
 /* from blendef: */
 #define GS(a)	(*((short *)(a)))
 
-/* from misc_util: flip the bytes from x  */
-/*#define GS(x) (((unsigned char *)(x))[0] << 8 | ((unsigned char *)(x))[1]) */
-
 // used everywhere in blenkernel and text.c
 void *copy_libblock(void *rt)
 {
@@ -421,9 +426,6 @@ void free_libblock(ListBase *lb, void *idv)
 		case ID_IM:
 			free_image((Image *)id);
 			break;
-		case ID_IK:
-			free_ika((Ika *)id);
-			break;
 		case ID_WV:
 			/* free_wave(id); */
 			break;
@@ -472,13 +474,14 @@ void free_libblock(ListBase *lb, void *idv)
 		case ID_AC:
 			free_action((bAction *)id);
 			break;
+		case ID_NT:
+			ntreeFreeTree((bNodeTree *)id);
+			break;
 	}
 
 	BLI_remlink(lb, id);
 	MEM_freeN(id);
 
-	/* should not be here!! this is an interface-thing */
-	allspace(OOPS_TEST, 0);
 }
 
 void free_libblock_us(ListBase *lb, void *idv)		/* test users */
@@ -520,7 +523,7 @@ void free_main(Main *mainvar)
 
 /* ***************** ID ************************ */
 
-// only used in exotic.c
+
 ID *find_id(char *type, char *name)		/* type: "OB" or "MA" etc */
 {
 	ID *id;
@@ -536,9 +539,10 @@ ID *find_id(char *type, char *name)		/* type: "OB" or "MA" etc */
 	return 0;
 }
 
-static void get_flags_for_id(ID *id, char *buf) {
+static void get_flags_for_id(ID *id, char *buf) 
+{
 	int isfake= id->flag & LIB_FAKEUSER;
-
+	int isnode=0;
 		/* Writeout the flags for the entry, note there
 		 * is a small hack that writes 5 spaces instead
 		 * of 4 if no flags are displayed... this makes
@@ -546,10 +550,15 @@ static void get_flags_for_id(ID *id, char *buf) {
 		 * to have that explicit, oh well - zr
 		 */
 
+	if(GS(id->name)==ID_MA)
+		isnode= ((Material *)id)->use_nodes;
+	
 	if (id->us<0)
 		sprintf(buf, "-1W ");
-	else if (!id->lib && !isfake && id->us)
+	else if (!id->lib && !isfake && id->us && !isnode)
 		sprintf(buf, "     ");
+	else if(isnode)
+		sprintf(buf, "%c%cN%c ", id->lib?'L':' ', isfake?'F':' ', (id->us==0)?'O':' ');
 	else
 		sprintf(buf, "%c%c%c ", id->lib?'L':' ', isfake?'F':' ', (id->us==0)?'O':' ');
 }
@@ -562,6 +571,7 @@ static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, shor
 	
 	if (nr && nids>MAX_IDPUP) {
 		BLI_dynstr_append(pupds, "DataBrowse %x-2");
+		*nr= -2;
 	} else {
 		ID *id;
 		
@@ -569,6 +579,9 @@ static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, shor
 			char buf[32];
 			
 			if (nr && id==link) *nr= i+1;
+
+			if (U.uiflag & USER_HIDE_DOT && id->name[2]=='.')
+				continue;
 			
 			get_flags_for_id(id, buf);
 				
@@ -577,6 +590,21 @@ static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, shor
 			sprintf(buf, "%%x%d", i+1);
 			BLI_dynstr_append(pupds, buf);
 			
+			/* icon */
+			switch(GS(id->name))
+			{
+			case ID_MA: /* fall through */
+			case ID_TE: /* fall through */
+			case ID_IM: /* fall through */
+			case ID_WO: /* fall through */
+			case ID_LA: /* fall through */
+				sprintf(buf, "%%i%d", BKE_icon_getid(id) );
+				BLI_dynstr_append(pupds, buf);
+				break;
+			default:
+				break;
+			}
+			
 			if(id->next)
 				BLI_dynstr_append(pupds, "|");
 		}
@@ -611,6 +639,9 @@ static void IPOnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, sho
 				if (id==link)
 					*nr= i+1;
 					
+				if (U.uiflag & USER_HIDE_DOT && id->name[2]=='.')
+					continue;
+
 				get_flags_for_id(id, buf);
 				
 				BLI_dynstr_append(pupds, buf);
@@ -720,7 +751,7 @@ static void sort_alpha_id(ListBase *lb, ID *id)
 	
 }
 
-int new_id(ListBase *lb, ID *id, char *tname)
+int new_id(ListBase *lb, ID *id, const char *tname)
 /* only for local blocks: external en indirect blocks already have a unique ID */
 /* return 1: created a new name */
 {
@@ -832,6 +863,52 @@ void clear_id_newpoins()
 	}
 }
 
+/* only for library fixes */
+static void image_fix_relative_path(Image *ima)
+{
+	if(ima->id.lib==NULL) return;
+	if(strncmp(ima->name, "//", 2)==0) {
+		BLI_convertstringcode(ima->name, ima->id.lib->filename, 0);
+		BLI_makestringcode(G.sce, ima->name);
+	}
+}
+
+#define LIBTAG(a)	if(a && a->id.lib) {a->id.flag &=~LIB_INDIRECT; a->id.flag |= LIB_EXTERN;}
+
+static void lib_indirect_test_id(ID *id)
+{
+	
+	if(id->lib)
+		return;
+
+	if(GS(id->name)==ID_OB) {		
+		Object *ob= (Object *)id;
+		bActionStrip *strip;
+		Mesh *me;
+		PartEff *paf;
+		int a;
+
+		for (strip=ob->nlastrips.first; strip; strip=strip->next){
+			LIBTAG(strip->object); 
+			LIBTAG(strip->act);
+			LIBTAG(strip->ipo);
+		}
+	
+		for(a=0; a<ob->totcol; a++) {
+			LIBTAG(ob->mat[a]);
+		}
+		
+		paf = give_parteff(ob);
+		if (paf) 
+			LIBTAG(paf->group);
+	
+		LIBTAG(ob->dup_group);
+		me= ob->data;
+		LIBTAG(me);
+	}
+}
+
+
 /* if lib!=NULL, only all from lib local */
 void all_local(Library *lib)
 {
@@ -852,6 +929,10 @@ void all_local(Library *lib)
 					id->flag &= ~(LIB_EXTERN|LIB_INDIRECT|LIB_NEW);
 
 					if(id->lib) {
+						/* relative file patch */
+						if(GS(id->name)==ID_IM)
+							image_fix_relative_path((Image *)id);
+						
 						id->lib= NULL;
 						new_id(lbarray[a], id, 0);	/* new_id only does it with double names */
 						sort_alpha_id(lbarray[a], id);
@@ -868,6 +949,13 @@ void all_local(Library *lib)
 			new_id(lbarray[a], id, 0);
 		}
 	}
+
+	/* patch 3: make sure library data isn't indirect falsely... */
+	a= set_listbasepointers(G.main, lbarray);
+	while(a--) {
+		for(id= lbarray[a]->first; id; id=id->next)
+			lib_indirect_test_id(id);
+	}
 }
 
 
