@@ -1,7 +1,7 @@
 /* object.c
  *
  * 
- * $Id: object.c,v 1.43 2005/05/20 20:44:33 ton Exp $
+ * $Id: object.c,v 1.100 2006/07/20 15:27:55 n_t Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,15 +48,17 @@
 #include "DNA_constraint_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_group_types.h"
-#include "DNA_ika_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
+#include "DNA_modifier_types.h"
+#include "DNA_nla_types.h"
 #include "DNA_object_types.h"
 #include "DNA_object_force.h"
+#include "DNA_object_fluidsim.h"
 #include "DNA_oops_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
@@ -69,6 +71,7 @@
 #include "BKE_armature.h"
 #include "BKE_action.h"
 #include "BKE_deform.h"
+#include "BKE_DerivedMesh.h"
 #include "BKE_nla.h"
 
 #include "BLI_blenlib.h"
@@ -81,26 +84,30 @@
 #include "BKE_main.h"
 #include "BKE_global.h"
 
-#include "BKE_object.h"
+#include "BKE_anim.h"
 #include "BKE_blender.h"
-#include "BKE_screen.h"
+#include "BKE_constraint.h"
+#include "BKE_curve.h"
+#include "BKE_displist.h"
+#include "BKE_effect.h"
+#include "BKE_group.h"
+#include "BKE_icons.h"
 #include "BKE_ipo.h"
-#include "BKE_ika.h"
+#include "BKE_key.h"
+#include "BKE_lattice.h"
 #include "BKE_library.h"
 #include "BKE_mesh.h"
-#include "BKE_curve.h"
 #include "BKE_mball.h"
-#include "BKE_effect.h"
-#include "BKE_sca.h"
-#include "BKE_displist.h"
+#include "BKE_modifier.h"
+#include "BKE_object.h"
 #include "BKE_property.h"
-#include "BKE_anim.h"
-#include "BKE_group.h"
-#include "BKE_lattice.h"
-#include "BKE_constraint.h"
+#include "BKE_sca.h"
 #include "BKE_scene.h"
+#include "BKE_screen.h"
 #include "BKE_softbody.h"
 
+#include "LBM_fluidsim.h"
+
 #include "BPY_extern.h"
 
 /* Local function protos */
@@ -147,27 +154,31 @@ void update_base_layer(Object *ob)
 	}
 }
 
-static void free_hooks(ListBase *lb)
+void object_free_modifiers(Object *ob)
 {
-	while(lb->first) {
-		ObHook *hook= lb->first;
-		if(hook->indexar) MEM_freeN(hook->indexar);
-		BLI_remlink(lb, hook);
-		MEM_freeN(hook);
+	while (ob->modifiers.first) {
+		ModifierData *md = ob->modifiers.first;
+
+		BLI_remlink(&ob->modifiers, md);
+
+		modifier_free(md);
 	}
 }
 
-static void copy_hooks(ListBase *new, ListBase *old)
+/* here we will collect all local displist stuff */
+/* also (ab)used in depsgraph */
+void object_free_display(Object *ob)
 {
-	ObHook *hook, *hookn;
-	new->first= new->last= NULL;
-	
-	for(hook= old->first; hook; hook= hook->next) {
-		hookn= MEM_dupallocN(hook);
-		hookn->indexar= MEM_dupallocN(hookn->indexar);
-		BLI_addtail(new, hookn);
+	if(ob->derivedDeform) {
+		ob->derivedDeform->release(ob->derivedDeform);
+		ob->derivedDeform= NULL;
 	}
-
+	if(ob->derivedFinal) {
+		ob->derivedFinal->release(ob->derivedFinal);
+		ob->derivedFinal= NULL;
+	}
+	
+	freedisplist(&ob->disp);
 }
 
 /* do not free object itself */
@@ -175,6 +186,8 @@ void free_object(Object *ob)
 {
 	int a;
 	
+	object_free_display(ob);
+	
 	/* disconnect specific data */
 	if(ob->data) {
 		ID *id= ob->data;
@@ -198,15 +211,17 @@ void free_object(Object *ob)
 	ob->path= 0;
 	if(ob->ipo) ob->ipo->id.us--;
 	if(ob->action) ob->action->id.us--;
+	if(ob->dup_group) ob->dup_group->id.us--;
 	if(ob->defbase.first)
 		BLI_freelistN(&ob->defbase);
 	if(ob->pose) {
-		clear_pose(ob->pose);
+		free_pose_channels(ob->pose);
 		MEM_freeN(ob->pose);
 	}
 	free_effects(&ob->effect);
 	BLI_freelistN(&ob->network);
 	free_properties(&ob->prop);
+	object_free_modifiers(ob);
 	
 	free_sensors(&ob->sensors);
 	free_controllers(&ob->controllers);
@@ -216,16 +231,22 @@ void free_object(Object *ob)
 	free_constraint_channels(&ob->constraintChannels);
 	free_nlastrips(&ob->nlastrips);
 	
-	free_hooks(&ob->hooks);
-	
-	freedisplist(&ob->disp);
-	
 	BPY_free_scriptlink(&ob->scriptlink);
 	
 	if(ob->pd) MEM_freeN(ob->pd);
 	if(ob->soft) sbFree(ob->soft);
+	if(ob->fluidsimSettings) fluidsimSettingsFree(ob->fluidsimSettings);
 }
 
+static void unlink_object__unlinkModifierLinks(void *userData, Object *ob, Object **obpoin)
+{
+	Object *unlinkOb = userData;
+
+	if (*obpoin==unlinkOb) {
+		*obpoin = NULL;
+		ob->recalc |= OB_RECALC;
+	}
+}
 void unlink_object(Object *ob)
 {
 	Object *obt;
@@ -235,48 +256,84 @@ void unlink_object(Object *ob)
 	Scene *sce;
 	Curve *cu;
 	Tex *tex;
-	ObHook *hook;
+	Ipo *ipo;
 	Group *group;
+	bConstraint *con;
+	bActionStrip *strip;
 	int a;
-
+	char *str;
+	
 	unlink_controllers(&ob->controllers);
 	unlink_actuators(&ob->actuators);
 	
-	/* check all objects: parents en bevels */
+	/* check all objects: parents en bevels and fields */
 	obt= G.main->object.first;
 	while(obt) {
 		if(obt->id.lib==NULL) {
+			
 			if(obt->parent==ob) {
 				obt->parent= NULL;
-				if(ob->type==OB_LATTICE) freedisplist(&obt->disp);
+				obt->recalc |= OB_RECALC;
 			}
-			if(obt->track==ob) obt->track= NULL;
-			for(hook=obt->hooks.first; hook; hook= hook->next) {
-				if(hook->parent==ob) hook->parent= NULL;
+			
+			if(obt->track==ob) {
+				obt->track= NULL;
+				obt->recalc |= OB_RECALC_OB;
 			}
+			
+			modifiers_foreachObjectLink(obt, unlink_object__unlinkModifierLinks, ob);
+			
 			if ELEM(obt->type, OB_CURVE, OB_FONT) {
 				cu= obt->data;
-				if(cu->bevobj==ob) cu->bevobj= NULL;
-				if(cu->taperobj==ob) cu->taperobj= NULL;
-				if(cu->textoncurve==ob) cu->textoncurve= NULL;
-			}
-			if(obt->type==OB_IKA) {
-				Ika *ika= obt->data;
-				Deform *def= ika->def;
-				
-				if(ika->parent==ob) ika->parent= NULL;
-				a= ika->totdef;
-				while(a--) {
-					if(def->ob==ob) {
-						ika->totdef= 0;
-						MEM_freeN(ika->def);
-						ika->def= NULL;
-						break;
-					}
-					def++;
+				if(cu->bevobj==ob) {
+					cu->bevobj= NULL;
+					obt->recalc |= OB_RECALC;
+				}
+				if(cu->taperobj==ob) {
+					cu->taperobj= NULL;
+					obt->recalc |= OB_RECALC;
+				}
+				if(cu->textoncurve==ob) {
+					cu->textoncurve= NULL;
+					obt->recalc |= OB_RECALC;
 				}
 			}
+			else if(obt->type==OB_ARMATURE && obt->pose) {
+				bPoseChannel *pchan;
+				for(pchan= obt->pose->chanbase.first; pchan; pchan= pchan->next) {
+					for (con = pchan->constraints.first; con; con=con->next) {
+						if(ob==get_constraint_target(con, &str)) {
+							set_constraint_target(con, NULL, NULL);
+							obt->recalc |= OB_RECALC_DATA;
+						}
+					}
+					if(pchan->custom==ob)
+						pchan->custom= NULL;
+				}
+			}
+			
 			sca_remove_ob_poin(obt, ob);
+			
+			for (con = obt->constraints.first; con; con=con->next) {
+				if(ob==get_constraint_target(con, &str)) {
+					set_constraint_target(con, NULL, NULL);
+					obt->recalc |= OB_RECALC_OB;
+				}
+			}
+			
+			/* object is deflector or field */
+			if(ob->pd) {
+				if(give_parteff(obt))
+					obt->recalc |= OB_RECALC_DATA;
+				else if(obt->soft)
+					obt->recalc |= OB_RECALC_DATA;
+			}
+			
+			/* strips */
+			for(strip= ob->nlastrips.first; strip; strip= strip->next) {
+				if(strip->object==ob)
+					strip->object= NULL;
+			}
 		}
 		obt= obt->id.next;
 	}
@@ -288,7 +345,7 @@ void unlink_object(Object *ob)
 		for(a=0; a<MAX_MTEX; a++) {
 			if(mat->mtex[a] && ob==mat->mtex[a]->object) {
 				/* actually, test for lib here... to do */
-				mat->mtex[a]->object= 0;
+				mat->mtex[a]->object= NULL;
 			}
 		}
 
@@ -299,7 +356,7 @@ void unlink_object(Object *ob)
 	tex= G.main->tex.first;
 	while(tex) {
 		if(tex->env) {
-			if(tex->env->object == ob) tex->env->object= 0;
+			if(tex->env->object == ob) tex->env->object= NULL;
 		}
 		tex= tex->id.next;
 	}
@@ -313,10 +370,10 @@ void unlink_object(Object *ob)
 	/* worlds */
 	wrld= G.main->world.first;
 	while(wrld) {
-		if(wrld->id.lib==0) {
+		if(wrld->id.lib==NULL) {
 			for(a=0; a<MAX_MTEX; a++) {
 				if(wrld->mtex[a] && ob==wrld->mtex[a]->object)
-					wrld->mtex[a]->object =0;
+					wrld->mtex[a]->object= NULL;
 			}
 		}
 		
@@ -326,12 +383,23 @@ void unlink_object(Object *ob)
 	/* scenes */
 	sce= G.main->scene.first;
 	while(sce) {
-		if(sce->id.lib==0) {
-			if(sce->camera==ob) sce->camera= 0;
+		if(sce->id.lib==NULL) {
+			if(sce->camera==ob) sce->camera= NULL;
 		}
 		sce= sce->id.next;
 	}
-	/* keys */
+	/* ipos */
+	ipo= G.main->ipo.first;
+	while(ipo) {
+		if(ipo->id.lib==NULL) {
+			IpoCurve *icu;
+			for(icu= ipo->curve.first; icu; icu= icu->next) {
+				if(icu->driver && icu->driver->ob==ob)
+					icu->driver->ob= NULL;
+			}
+		}
+		ipo= ipo->id.next;
+	}
 	
 	/* screens */
 	sc= G.main->screen.first;
@@ -345,11 +413,11 @@ void unlink_object(Object *ob)
 					View3D *v3d= (View3D*) sl;
 
 					if(v3d->camera==ob) {
-						v3d->camera= 0;
+						v3d->camera= NULL;
 						if(v3d->persp>1) v3d->persp= 1;
 					}
 					if(v3d->localvd && v3d->localvd->camera==ob ) {
-						v3d->localvd->camera= 0;
+						v3d->localvd->camera= NULL;
 						if(v3d->localvd->persp>1) v3d->localvd->persp= 1;
 					}
 				}
@@ -394,6 +462,8 @@ int exist_object(Object *obtest)
 {
 	Object *ob;
 	
+	if(obtest==NULL) return 0;
+	
 	ob= G.main->object.first;
 	while(ob) {
 		if(ob==obtest) return 1;
@@ -413,6 +483,8 @@ void *add_camera()
 	cam->clipend= 100.0f;
 	cam->drawsize= 0.5f;
 	cam->ortho_scale= 6.0;
+	cam->flag |= CAM_SHOWTITLESAFE;
+	cam->passepartalpha = 0.2f;
 	
 	return cam;
 }
@@ -507,6 +579,7 @@ void *add_lamp(void)
 	la->soft= 3.0;
 	la->ray_samp= la->ray_sampy= la->ray_sampz= 1; 
 	la->area_size=la->area_sizey=la->area_sizez= 1.0; 
+	la->buffers= 1;
 	
 	return la;
 }
@@ -605,6 +678,9 @@ void free_lamp(Lamp *la)
 		if(mtex) MEM_freeN(mtex);
 	}
 	la->ipo= 0;
+
+	BKE_icon_delete(&la->id);
+	la->id.icon_id = 0;
 }
 
 void *add_wave()
@@ -625,7 +701,6 @@ static void *add_obdata_from_type(int type)
 	case OB_MBALL: return add_mball();
 	case OB_CAMERA: return add_camera();
 	case OB_LAMP: G.totlamp++; return add_lamp();
-	case OB_IKA: return add_ika();
 	case OB_LATTICE: return add_lattice();
 	case OB_WAVE: return add_wave();
 	case OB_ARMATURE: return add_armature();
@@ -646,7 +721,6 @@ static char *get_obdata_defname(int type)
 	case OB_MBALL: return "Mball";
 	case OB_CAMERA: return "Camera";
 	case OB_LAMP: return "Lamp";
-	case OB_IKA: return "Ika";
 	case OB_LATTICE: return "Lattice";
 	case OB_WAVE: return "Wave";
 	case OB_ARMATURE: return "Armature";
@@ -665,9 +739,6 @@ Object *add_object(int type)
 	Base *base;
 	char name[32];
 
-	if (G.obpose)
-		exit_posemode(1);
-	
 	strcpy(name, get_obdata_defname(type));
 	
 	ob= alloc_libblock(&G.main->object, ID_OB, name);
@@ -691,6 +762,8 @@ Object *add_object(int type)
 	Mat4One(ob->obmat);
 	ob->dt= OB_SHADED;
 	if(U.flag & USER_MAT_ON_OB) ob->colbits= -1;
+	ob->empty_drawtype= OB_ARROWS;
+	ob->empty_drawsize= 1.0;
 	
 	if(type==OB_CAMERA || type==OB_LAMP) {
 		ob->trackflag= OB_NEGZ;
@@ -714,6 +787,10 @@ Object *add_object(int type)
 	ob->anisotropicFriction[1] = 1.0f;
 	ob->anisotropicFriction[2] = 1.0f;
 	ob->gameflag= OB_PROP;
+
+	/* NT fluid sim defaults */
+	ob->fluidsimFlag = 0;
+	ob->fluidsimSettings = NULL;
 	
 	ob->data= add_obdata_from_type(type);
 	
@@ -721,7 +798,8 @@ Object *add_object(int type)
 
 	base= scene_add_base(G.scene, ob);
 	scene_select_base(G.scene, base);
-
+	ob->recalc |= OB_RECALC;
+	
 	return ob;
 }
 
@@ -767,8 +845,8 @@ SoftBody *copy_softbody(SoftBody *sb)
 Object *copy_object(Object *ob)
 {
 	Object *obn;
+	ModifierData *md;
 	int a;
-	bConstraintChannel *actcon;
 
 	obn= copy_libblock(ob);
 	
@@ -781,6 +859,13 @@ Object *copy_object(Object *ob)
 	obn->flag &= ~OB_FROMGROUP;
 	
 	copy_effects(&obn->effect, &ob->effect);
+	obn->modifiers.first = obn->modifiers.last= NULL;
+	
+	for (md=ob->modifiers.first; md; md=md->next) {
+		ModifierData *nmd = modifier_new(md->type);
+		modifier_copyData(md, nmd);
+		BLI_addtail(&obn->modifiers, nmd);
+	}
 	
 	obn->network.first= obn->network.last= 0;
 	
@@ -791,35 +876,46 @@ Object *copy_object(Object *ob)
 	copy_controllers(&obn->controllers, &ob->controllers);
 	copy_actuators(&obn->actuators, &ob->actuators);
 	
-	copy_pose(&obn->pose, ob->pose, 1);
+	if(ob->pose) {
+		copy_pose(&obn->pose, ob->pose, 1);
+		/* backwards compat... non-armatures can get poses in older files? */
+		if(ob->type==OB_ARMATURE)
+			armature_rebuild_pose(obn, obn->data);
+	}
 	copy_defgroups(&obn->defbase, &ob->defbase);
 	copy_nlastrips(&obn->nlastrips, &ob->nlastrips);
 	copy_constraints (&obn->constraints, &ob->constraints);
 
-	copy_hooks( &obn->hooks, &ob->hooks);
-	
-	actcon = clone_constraint_channels (&obn->constraintChannels, &ob->constraintChannels, ob->activecon);
-	/* If the active constraint channel was in this list, update it */
-	if (actcon)
-		obn->activecon = actcon;
-
+	clone_constraint_channels (&obn->constraintChannels, &ob->constraintChannels);
 
 	/* increase user numbers */
 	id_us_plus((ID *)obn->data);
 	id_us_plus((ID *)obn->ipo);
 	id_us_plus((ID *)obn->action);
+	id_us_plus((ID *)obn->dup_group);
+
 	for(a=0; a<obn->totcol; a++) id_us_plus((ID *)obn->mat[a]);
 	
 	obn->disp.first= obn->disp.last= NULL;
 	
 	if(ob->pd) obn->pd= MEM_dupallocN(ob->pd);
 	obn->soft= copy_softbody(ob->soft);
+
+	/* NT copy fluid sim setting memory */
+	if(obn->fluidsimSettings) {
+		obn->fluidsimSettings = fluidsimSettingsCopy(ob->fluidsimSettings);
+		obn->fluidsimSettings->orgMesh = (Mesh *)obn->data;
+	}
 	
+	obn->derivedDeform = NULL;
+	obn->derivedFinal = NULL;
+
 	return obn;
 }
 
 void expand_local_object(Object *ob)
 {
+	bActionStrip *strip;
 	int a;
 	
 	id_lib_extern((ID *)ob->action);
@@ -829,6 +925,10 @@ void expand_local_object(Object *ob)
 	for(a=0; a<ob->totcol; a++) {
 		id_lib_extern((ID *)ob->mat[a]);
 	}
+	for (strip=ob->nlastrips.first; strip; strip=strip->next) {
+		id_lib_extern((ID *)strip->act);
+	}
+
 }
 
 void make_local_object(Object *ob)
@@ -899,15 +999,20 @@ void make_local_object(Object *ob)
 
 /* there is also a timing calculation in drawobject() */
 
-float bluroffs= 0.0;
+float bluroffs= 0.0f, fieldoffs= 0.0f;
 int no_speed_curve= 0;
 
-void set_mblur_offs(int blur)
+/* ugly calls from render */
+void set_mblur_offs(float blur)
 {
-	bluroffs= R.r.blurfac*((float)blur);
-	bluroffs/= (float)R.r.osa;
+	bluroffs= blur;
 }
-	
+
+void set_field_offs(float field)
+{
+	fieldoffs= field;
+}
+
 void disable_speed_curve(int val)
 {
 	no_speed_curve= val;
@@ -918,20 +1023,10 @@ float bsystem_time(Object *ob, Object *par, float cfra, float ofs)
 {
 	/* returns float ( see frame_to_float in ipo.c) */
 
-	/* 2nd field */
-	if(R.flag & R_SEC_FIELD) {
-		if(R.r.mode & R_FIELDSTILL); else cfra+= .5;
-	}
+	cfra+= bluroffs+fieldoffs;
 
-
-	if(ob && (ob->flag & OB_FROMDUPLI));
-	else {
-			/* motion blur */
-		cfra+= bluroffs;
-	
-		/* global time */
-		cfra*= G.scene->r.framelen;	
-	}
+	/* global time */
+	cfra*= G.scene->r.framelen;	
 	
 	if(no_speed_curve==0) if(ob && ob->ipo) cfra= calc_ipo_time(ob->ipo, cfra);
 	
@@ -1004,7 +1099,7 @@ void object_to_mat4(Object *ob, float mat[][4])
 
 int enable_cu_speed= 1;
 
-void ob_parcurve(Object *ob, Object *par, float mat[][4])
+static void ob_parcurve(Object *ob, Object *par, float mat[][4])
 {
 	Curve *cu;
 	float q[4], vec[4], dir[3], *quat, x1, ctime;
@@ -1013,7 +1108,8 @@ void ob_parcurve(Object *ob, Object *par, float mat[][4])
 	Mat4One(mat);
 	
 	cu= par->data;
-	if(cu->path==NULL || cu->path->data==NULL) calc_curvepath(par);
+	if(cu->path==NULL || cu->path->data==NULL) /* only happens on reload file, but violates depsgraph still... fix! */
+		makeDispListCurveTypes(par, 0);
 	if(cu->path==NULL) return;
 	
 	/* exception, timeoffset is regarded as distance offset */
@@ -1021,8 +1117,12 @@ void ob_parcurve(Object *ob, Object *par, float mat[][4])
 		SWAP(float, timeoffs, ob->sf);
 	}
 	
+	/* catch exceptions: feature for nla stride editing */
+	if(ob->ipoflag & OB_DISABLE_PATH) {
+		ctime= 0.0f;
+	}
 	/* catch exceptions: curve paths used as a duplicator */
-	if(enable_cu_speed) {
+	else if(enable_cu_speed) {
 		ctime= bsystem_time(ob, par, (float)G.scene->r.cfra, 0.0);
 		
 		if(calc_ipo_spec(cu->ipo, CU_SPEED, &ctime)==0) {
@@ -1069,79 +1169,50 @@ void ob_parcurve(Object *ob, Object *par, float mat[][4])
 	}
 }
 
-void ob_parbone(Object *ob, Object *par, float mat[][4])
+static void ob_parbone(Object *ob, Object *par, float mat[][4])
 {	
-	Bone *bone;
+	bPoseChannel *pchan;
 	bArmature *arm;
-
-	Mat4One(mat);
+	float vec[3];
+	
 	arm=get_armature(par);
-	if (!arm)
+	if (!arm) {
+		Mat4One(mat);
 		return;
-
+	}
+	
 	/* Make sure the bone is still valid */
-	bone = get_named_bone(arm, ob->parsubstr);
-	if (!bone){
-		printf ("Lost bone %s\n", ob->parsubstr);
+	pchan= get_pose_channel(par->pose, ob->parsubstr);
+	if (!pchan){
+		printf ("Bone parent: no bone %s\n", ob->parsubstr);
+		Mat4One(mat);
 		return;
 	}
 
-	apply_pose_armature(arm, par->pose, 1);	/* Hopefully can set doit parameter in the future */
-	where_is_bone (par, bone); 
-
-	/* Translate by negative bone */
-	get_objectspace_bone_matrix(bone, mat, 0, 1);
+	/* get bone transform */
+	Mat4CpyMat4(mat, pchan->pose_mat);
 
+	/* but for backwards compatibility, the child has to move to the tail */
+	VECCOPY(vec, mat[1]);
+	VecMulf(vec, pchan->bone->length);
+	VecAddf(mat[3], mat[3], vec);
 }
 
-void ob_parlimb(Object *ob, Object *par, float mat[][4])
-{	
-	Ika *ika;
-	Limb *li;
-	float ang=0.0;
-	int cur=0;
-	
-	/* in local ob space */
-	Mat4One(mat);
-	
-	ika= par->data;
-	li= ika->limbbase.first;
-	while(li) {
-		ang+= li->alpha;
-		if(cur==ob->par1 || li->next==0) break;
-		
-		cur++;
-		li= li->next;
-	}
-	
-	mat[0][0]= (float)cos(ang);
-	mat[1][0]= (float)-sin(ang);
-	mat[0][1]= (float)sin(ang);
-	mat[1][1]= (float)cos(ang);
-	
-	mat[3][0]= li->eff[0];
-	mat[3][1]= li->eff[1];
-	
-}
-
-void give_parvert(Object *par, int nr, float *vec)
+static void give_parvert(Object *par, int nr, float *vec)
 {
 	EditMesh *em = G.editMesh;
-	Mesh *me;
 	EditVert *eve;
 /*  	extern ListBase editNurb; already in bad lev calls */
 	Nurb *nu;
 	Curve *cu;
 	BPoint *bp;
-	DispList *dl;
 	BezTriple *bezt;
-	float *fp;
 	int a, count;
 	
 	vec[0]=vec[1]=vec[2]= 0.0;
 	
 	if(par->type==OB_MESH) {
-		if(par==G.obedit) {
+		if(G.obedit && (par->data==G.obedit->data)) {
 			if(nr >= G.totvert) nr= 0;
 
 			count= 0;
@@ -1156,19 +1227,15 @@ void give_parvert(Object *par, int nr, float *vec)
 			}
 		}
 		else {
-			me= par->data;
-			if(me->totvert) {
-				if(nr >= me->totvert) nr= 0;
-				
-				/* is there a deform */
-				dl= find_displist(&par->disp, DL_VERTS);
-				if(dl) {
-					fp= dl->verts+3*nr;
-					VECCOPY(vec, fp);
-				}
-				else {
-					MVert *mvert= me->mvert + nr;
-					VECCOPY(vec, mvert->co);
+			/* maybe this is against the derivedmesh philosphy, but where_is_object is called
+			   by code that is called by build_mesh... (when ob->sf!=0.0) so it can cycle eternally */
+			DerivedMesh *dm = par->derivedDeform; //mesh_get_derived_deform(par, &needsFree);
+			
+			if(dm) {
+				int tot= dm->getNumVerts(dm);
+				if(tot) {
+					if(nr >= tot) nr= 0;
+					dm->getVertCo(dm, nr, vec);
 				}
 			}
 		}
@@ -1209,24 +1276,10 @@ void give_parvert(Object *par, int nr, float *vec)
 		}
 
 	}
-	else if(par->type==OB_IKA) {
-		Ika *ika= par->data;
-		Limb *li= ika->limbbase.first;
-		int cur= 1;
-		if(nr) {
-			while(li) {
-				if(cur==nr || li->next==0) break;
-				cur++;
-				li= li->next;
-			}
-			vec[0]= li->eff[0];
-			vec[1]= li->eff[1];
-		}
-	}
 	else return;
 }
 
-void ob_parvert3(Object *ob, Object *par, float mat[][4])
+static void ob_parvert3(Object *ob, Object *par, float mat[][4])
 {
 	float cmat[3][3], v1[3], v2[3], v3[3], q[4];
 
@@ -1260,12 +1313,6 @@ void set_no_parent_ipo(int val)
 	no_parent_ipo= val;
 }
 
-static float timefac= 1.0;		/* 50 Hz, dtime:2 */
-void set_dtime(int dtime)
-{
-	timefac= ((float)(dtime-1))/2.0f;
-}
-
 static int during_script_flag=0;
 void disable_where_script(short on)
 {
@@ -1298,31 +1345,34 @@ void where_is_object_time(Object *ob, float ctime)
 	/* this one only calculates direct attached parent and track */
 	/* is faster, but should keep track of timeoffs */
 	
-	if(ob==0) return;
-
-	if( ctime != ob->ctime) {
-		ob->ctime= ctime;
+	if(ob==NULL) return;
+	
+	/* this is needed to be able to grab objects with ipos, otherwise it always freezes them */
+	stime= bsystem_time(ob, 0, ctime, 0.0);
+	if(stime != ob->ctime) {
+		ob->ctime= stime;
 		
 		if(ob->ipo) {
-			
-			stime= bsystem_time(ob, 0, ctime, 0.0);
-
 			calc_ipo(ob->ipo, stime);
 			execute_ipo((ID *)ob, ob->ipo);
-		}			
+		}
+		else 
+			do_all_object_actions(ob);
+		
+		/* do constraint ipos ... */
+		do_constraint_channels(&ob->constraints, &ob->constraintChannels, ctime);
 	}
-
-
-	if(ob->type==OB_IKA) {
-		Ika *ika= ob->data;
-		if(ika->parent) where_is_object_time(ika->parent, ctime);
+	else {
+		/* but, the drivers have to be done */
+		if(ob->ipo) do_ob_ipodrivers(ob, ob->ipo, stime);
 	}
-
+	
 	if(ob->parent) {
 		par= ob->parent;
 
 		if(ob->ipoflag & OB_OFFS_PARENT) ctime-= ob->sf;
 		
+		/* hurms, code below conflicts with depgraph... (ton) */
 		pop= 0;
 		if(no_parent_ipo==0 && ctime != par->ctime) {
 		
@@ -1342,7 +1392,7 @@ void where_is_object_time(Object *ob, float ctime)
 		if(ob->partype & PARSLOW) {
 			// include framerate
 
-			fac1= (float)(timefac/(1.0+ fabs(ob->sf)));
+			fac1= (1.0f/(1.0f+ fabs(ob->sf)));
 			if(fac1>=1.0) return;
 			fac2= 1.0f-fac1;
 			
@@ -1410,11 +1460,6 @@ static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int si
 		Mat4MulSerie(totmat, par->obmat, tmat,         
 			NULL, NULL, NULL, NULL, NULL, NULL);
 		break;
-	case PARLIMB:	
-		ob_parlimb(ob, par, tmat);
-		Mat4MulSerie(totmat, par->obmat, tmat,         
-			NULL, NULL, NULL, NULL, NULL, NULL);
-		break;
 		
 	case PARVERT1:
 		Mat4One(totmat);
@@ -1434,15 +1479,8 @@ static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int si
 		break;
 		
 	case PARSKEL:
-#if 0
-		if (ob!=G.obedit)
-			Mat4One(totmat);
-		else
-			Mat4CpyMat4(totmat, par->obmat);
+		Mat4CpyMat4(totmat, par->obmat);
 		break;
-#else
-			Mat4CpyMat4(totmat, par->obmat);
-#endif
 	}
 	
 	// total 
@@ -1499,9 +1537,6 @@ void solve_tracking (Object *ob, float targetmat[][4])
 void where_is_object(Object *ob)
 {
 	
-	/* these have been mem copied */
-	if(ob->flag & OB_FROMDUPLI) return;
-	
 	where_is_object_time(ob, (float)G.scene->r.cfra);
 }
 
@@ -1569,59 +1604,61 @@ void solve_constraints (Object *ob, short obtype, void *obdata, float ctime)
 	float	smat[3][3], rmat[3][3], mat[3][3];
 	float enf;
 
-	for (con = ob->constraints.first; con; con=con->next){
+	for (con = ob->constraints.first; con; con=con->next) {
+		// inverse kinematics is solved seperate 
+		if (con->type==CONSTRAINT_TYPE_KINEMATIC) continue;
+		// and this we can skip completely
+		if (con->flag & CONSTRAINT_DISABLE) continue;
+		// local constraints are handled in armature.c only 
+		if (con->flag & CONSTRAINT_LOCAL) continue;
+
 		/* Clear accumulators if necessary*/
-		if (clear){
-			clear=0;
-			a=0;
-			tot=0;
+		if (clear) {
+			clear= 0;
+			a= 0;
+			tot= 0;
 			memset(aquat, 0, sizeof(float)*4);
 			memset(aloc, 0, sizeof(float)*3);
 			memset(asize, 0, sizeof(float)*3);
 		}
 		
-		/* Check this constraint only if it has some enforcement */
-		if (!(con->flag & CONSTRAINT_DISABLE))
-		{
-			if (con->enforce==0)
-				enf = 0.001f;
-				enf = con->enforce;
+		enf = con->enforce;	// value from ipos (from action channels)
 
-			/* Get the targetmat */
-			get_constraint_target_matrix(con, obtype, obdata, tmat, size, ctime);
-			
-			Mat4CpyMat4(focusmat, tmat);
-			
-			/* Extract the components & accumulate */
-			Mat4ToQuat(focusmat, quat);
-			VECCOPY(loc, focusmat[3]);
-			Mat3CpyMat4(mat, focusmat);
-			Mat3ToSize(mat, size);
-			
-			a+=enf;
-			tot++;
-			
-			for(i=0; i<3; i++){
-				aquat[i+1]+=(quat[i+1]) * enf;
-				aloc[i]+=(loc[i]) * enf;
-				asize[i]+=(size[i]-1.0f) * enf;
-			}
-			aquat[0]+=(quat[0])*enf;
-			Mat4CpyMat4(lastmat, focusmat);
+		/* Get the targetmat */
+		get_constraint_target_matrix(con, obtype, obdata, tmat, size, ctime);
+		
+		Mat4CpyMat4(focusmat, tmat);
+		
+		/* Extract the components & accumulate */
+		Mat4ToQuat(focusmat, quat);
+		VECCOPY(loc, focusmat[3]);
+		Mat3CpyMat4(mat, focusmat);
+		Mat3ToSize(mat, size);
+		
+		a+= enf;
+		tot++;
+		
+		for(i=0; i<3; i++) {
+			aquat[i+1]+=(quat[i+1]) * enf;
+			aloc[i]+=(loc[i]) * enf;
+			asize[i]+=(size[i]-1.0f) * enf;
 		}
+		aquat[0]+=(quat[0])*enf;
+		Mat4CpyMat4(lastmat, focusmat);
+		
+		/* removed for now, probably becomes option? (ton) */
 		
 		/* If the next constraint is not the same type (or there isn't one),
 		 *	then evaluate the accumulator & request a clear */
-		if ((!con->next)||(con->next && con->next->type!=con->type))
-		{
-			clear=1;
+		if (TRUE) { //(!con->next)||(con->next && con->next->type!=con->type)) {
+			clear= 1;
 			Mat4CpyMat4(oldmat, ob->obmat);
 
 			/*	If we have several inputs, do a blend of them */
-			if (tot){
-				if (tot>1){
-					if (a){
-						for (i=0; i<3; i++){
+			if (tot) {
+				if (tot>1) {
+					if (a) {
+						for (i=0; i<3; i++) {
 							asize[i]=1.0f + (asize[i]/(a));
 							aloc[i]=(aloc[i]/a);
 						}
@@ -1639,14 +1676,13 @@ void solve_constraints (Object *ob, short obtype, void *obdata, float ctime)
 					
 				}	
 				/* If we only have one, blend with the current obmat */
-				else{
+				else {
 					float solution[4][4];
 					float delta[4][4];
 					float imat[4][4];
 					float identity[4][4];
-					float worldmat[4][4];
 					
-					if (con->type!=CONSTRAINT_TYPE_KINEMATIC){
+					if (con->type!=CONSTRAINT_TYPE_KINEMATIC) {
 						/* If we're not an IK constraint, solve the constraint then blend it to the previous one */
 						evaluate_constraint(con, ob, obtype, obdata, lastmat);
 						
@@ -1663,46 +1699,13 @@ void solve_constraints (Object *ob, short obtype, void *obdata, float ctime)
 						Mat4MulMat4 (ob->obmat, delta, oldmat);
 
 					}
-					else{
-						/* Interpolate the target between the chain's unconstrained endpoint and the effector loc */
-						if (obtype==TARGET_BONE){
-							get_objectspace_bone_matrix(obdata, oldmat, 1, 1);
-							
-							Mat4MulMat4(worldmat, oldmat, ob->parent->obmat);
-
-							Mat4BlendMat4(focusmat, worldmat, lastmat, a);
-							
-							evaluate_constraint(con, ob, obtype, obdata, focusmat);
-						}
-					}
 				}
 			}
 		}
 	}	
 }
 
-void what_does_parent1(Object *par, int partype, int par1, int par2, int par3)
-{
-
-	clear_workob();
-	Mat4One(workob.parentinv);
-	workob.parent= par;
-	if(par) 
-		workob.track= par->track;	/* WATCH IT: THATS NOT NICE CODE */
-	workob.partype= partype;
-	workob.par1= par1;
-	workob.par2= par2;
-	workob.par3= par3;
-
-	if (par){
-		workob.constraints.first = par->constraints.first;
-		workob.constraints.last = par->constraints.last;
-	}
-
-	where_is_object(&workob);
-}
-
-
+/* for calculation of the inverse parent transform, only used for editor */
 void what_does_parent(Object *ob)
 {
 
@@ -1731,18 +1734,39 @@ void what_does_parent(Object *ob)
 BoundBox *unit_boundbox()
 {
 	BoundBox *bb;
-	
+	float min[3] = {-1,-1,-1}, max[3] = {-1,-1,-1};
+
 	bb= MEM_mallocN(sizeof(BoundBox), "bb");
-
-	bb->vec[0][0]=bb->vec[1][0]=bb->vec[2][0]=bb->vec[3][0]= -1.0;
-	bb->vec[4][0]=bb->vec[5][0]=bb->vec[6][0]=bb->vec[7][0]= 1.0;
+	boundbox_set_from_min_max(bb, min, max);
 	
-	bb->vec[0][1]=bb->vec[1][1]=bb->vec[4][1]=bb->vec[5][1]= -1.0;
-	bb->vec[2][1]=bb->vec[3][1]=bb->vec[6][1]=bb->vec[7][1]= 1.0;
+	return bb;
+}
 
-	bb->vec[0][2]=bb->vec[3][2]=bb->vec[4][2]=bb->vec[7][2]= -1.0;
-	bb->vec[1][2]=bb->vec[2][2]=bb->vec[5][2]=bb->vec[6][2]= 1.0;
+void boundbox_set_from_min_max(BoundBox *bb, float min[3], float max[3])
+{
+	bb->vec[0][0]=bb->vec[1][0]=bb->vec[2][0]=bb->vec[3][0]= min[0];
+	bb->vec[4][0]=bb->vec[5][0]=bb->vec[6][0]=bb->vec[7][0]= max[0];
 	
+	bb->vec[0][1]=bb->vec[1][1]=bb->vec[4][1]=bb->vec[5][1]= min[1];
+	bb->vec[2][1]=bb->vec[3][1]=bb->vec[6][1]=bb->vec[7][1]= max[1];
+
+	bb->vec[0][2]=bb->vec[3][2]=bb->vec[4][2]=bb->vec[7][2]= min[2];
+	bb->vec[1][2]=bb->vec[2][2]=bb->vec[5][2]=bb->vec[6][2]= max[2];
+}
+
+BoundBox *object_get_boundbox(Object *ob)
+{
+	BoundBox *bb= NULL;
+	
+	if(ob->type==OB_MESH) {
+		bb = mesh_get_bb(ob->data);
+	}
+	else if ELEM3(ob->type, OB_CURVE, OB_SURF, OB_FONT) {
+		bb= ( (Curve *)ob->data )->bb;
+	}
+	else if(ob->type==OB_MBALL) {
+		bb= ob->bb;
+	}
 	return bb;
 }
 
@@ -1761,7 +1785,7 @@ void minmax_object(Object *ob, float *min, float *max)
 	case OB_SURF:
 		cu= ob->data;
 		
-		if(cu->bb==0) tex_space_curve(cu);
+		if(cu->bb==NULL) tex_space_curve(cu);
 		bb= *(cu->bb);
 		
 		for(a=0; a<8; a++) {
@@ -1769,13 +1793,25 @@ void minmax_object(Object *ob, float *min, float *max)
 			DO_MINMAX(bb.vec[a], min, max);
 		}
 		break;
-
+	case OB_ARMATURE:
+		if(ob->pose) {
+			bPoseChannel *pchan;
+			for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+				VECCOPY(vec, pchan->pose_head);
+				Mat4MulVecfl(ob->obmat, vec);
+				DO_MINMAX(vec, min, max);
+				VECCOPY(vec, pchan->pose_tail);
+				Mat4MulVecfl(ob->obmat, vec);
+				DO_MINMAX(vec, min, max);
+			}
+			break;
+		}
+		/* no break, get_mesh will give NULL and it passes on to default */
 	case OB_MESH:
 		me= get_mesh(ob);
 		
 		if(me) {
-			if(me->bb==0) tex_space_mesh(me);
-			bb= *(me->bb);
+			bb = *mesh_get_bb(me);
 			
 			for(a=0; a<8; a++) {
 				Mat4MulVecfl(ob->obmat, bb.vec[a]);
@@ -1800,4 +1836,42 @@ void minmax_object(Object *ob, float *min, float *max)
 	}
 }
 
+/* the main object update call, for object matrix, constraints, keys and displist (modifiers) */
+/* requires flags to be set! */
+void object_handle_update(Object *ob)
+{
+	if(ob->recalc & OB_RECALC) {
+		
+		if(ob->recalc & OB_RECALC_OB) where_is_object(ob);
+		
+		if(ob->recalc & OB_RECALC_DATA) {
+			
+//			printf("recalcdata %s\n", ob->id.name+2);
+			
+			/* includes all keys and modifiers */
+			if(ob->type==OB_MESH) {
+				makeDispListMesh(ob);
+			}
+			else if(ob->type==OB_MBALL) {
+				makeDispListMBall(ob);
+			} 
+			else if(ELEM3(ob->type, OB_CURVE, OB_SURF, OB_FONT)) {
+				makeDispListCurveTypes(ob, 0);
+			}
+			else if(ob->type==OB_LATTICE) {
+				lattice_calc_modifiers(ob);
+			}
+			else if(ob->type==OB_ARMATURE) {
+				/* this actually only happens for reading old files... */
+				if(ob->pose==NULL || (ob->pose->flag & POSE_RECALC))
+					armature_rebuild_pose(ob, ob->data);
+				do_all_pose_actions(ob);
+				where_is_pose(ob);
+			}
+		}
+	
+		ob->recalc &= ~OB_RECALC;
+	}
+}
+
 
