@@ -1,7 +1,7 @@
 /*  scene.c
  *  
  * 
- * $Id: scene.c,v 1.6 2003/05/21 01:21:01 phase Exp $
+ * $Id: scene.c,v 1.42 2006/06/22 15:25:30 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,6 +33,7 @@
  */
 
 #include <stdio.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -41,48 +42,50 @@
 #ifndef WIN32 
 #include <unistd.h>
 #else
-#include "BLI_winstuff.h"
 #include <io.h>
 #endif
 #include "MEM_guardedalloc.h"
 
-#include "nla.h"				/* for __NLA : IMPORTANT Do not delete me yet! */
-#ifdef __NLA					/* for __NLA : IMPORTANT Do not delete me yet! */
-#include "DNA_armature_types.h"	/* for __NLA : IMPORTANT Do not delete me yet! */
-#include "BKE_armature.h"		/* for __NLA : IMPORTANT Do not delete me yet! */
-#include "BKE_action.h"			/* for __NLA : IMPORTANT Do not delete me yet! */
-#endif							/* for __NLA : IMPORTANT Do not delete me yet! */
-
+#include "DNA_armature_types.h"	
 #include "DNA_constraint_types.h"
-#include "DNA_scene_types.h"
-#include "DNA_object_types.h"
-#include "DNA_scriptlink_types.h"
-#include "DNA_meta_types.h"
-#include "DNA_ika_types.h"
+#include "DNA_curve_types.h"
+#include "DNA_group_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_material_types.h"
-#include "DNA_mesh_types.h"
-#include "DNA_group_types.h"
-#include "DNA_curve_types.h"
+#include "DNA_meta_types.h"
+#include "DNA_object_types.h"
+#include "DNA_scene_types.h"
+#include "DNA_scriptlink_types.h"
 #include "DNA_userdef_types.h"
 
-#include "BLI_blenlib.h"
-
-#include "BKE_bad_level_calls.h"
-#include "BKE_utildefines.h"
-
-#include "BKE_global.h"
-#include "BKE_main.h"
+#include "BKE_action.h"			
 #include "BKE_anim.h"
-
+#include "BKE_armature.h"		
+#include "BKE_bad_level_calls.h"
+#include "BKE_constraint.h"
+#include "BKE_depsgraph.h"
+#include "BKE_global.h"
+#include "BKE_group.h"
+#include "BKE_ipo.h"
+#include "BKE_key.h"
 #include "BKE_library.h"
-
-#include "BPY_extern.h"
+#include "BKE_main.h"
+#include "BKE_node.h"
+#include "BKE_object.h"
 #include "BKE_scene.h"
 #include "BKE_world.h"
-#include "BKE_ipo.h"
-#include "BKE_ika.h"
-#include "BKE_key.h"
+#include "BKE_utildefines.h"
+
+#include "BPY_extern.h"
+#include "BLI_arithb.h"
+#include "BLI_blenlib.h"
+
+#include "nla.h"
+
+#ifdef WIN32
+#else
+#include <sys/time.h>
+#endif
 
 void free_avicodecdata(AviCodecData *acd)
 {
@@ -111,6 +114,8 @@ void free_qtcodecdata(QuicktimeCodecData *qcd)
 	}
 }
 
+/* copy_scene moved to src/header_info.c... should be back */
+
 /* do not free scene itself */
 void free_scene(Scene *sce)
 {
@@ -126,7 +131,6 @@ void free_scene(Scene *sce)
 	BLI_freelistN(&sce->base);
 	free_editing(sce->ed);
 	if(sce->radio) MEM_freeN(sce->radio);
-	if(sce->fcam) MEM_freeN(sce->fcam);
 	sce->radio= 0;
 	
 	BPY_free_scriptlink(&sce->scriptlink);
@@ -140,15 +144,35 @@ void free_scene(Scene *sce)
 		MEM_freeN(sce->r.qtcodecdata);
 		sce->r.qtcodecdata = NULL;
 	}
+	
+	BLI_freelistN(&sce->markers);
+	BLI_freelistN(&sce->r.layers);
+	
+	if(sce->toolsettings){
+		MEM_freeN(sce->toolsettings);
+		sce->toolsettings = NULL;	
+	}
+	
+	if (sce->theDag) {
+		free_forest(sce->theDag);
+		MEM_freeN(sce->theDag);
+	}
+	
+	if(sce->nodetree) {
+		ntreeFreeTree(sce->nodetree);
+		MEM_freeN(sce->nodetree);
+	}
 }
 
 Scene *add_scene(char *name)
 {
 	Scene *sce;
-	
+
 	sce= alloc_libblock(&G.main->scene, ID_SCE, name);
 	sce->lay= 1;
-
+	sce->selectmode= SCE_SELECT_VERTEX;
+	sce->editbutsize= 0.1;
+	
 	sce->r.mode= R_GAMMA;
 	sce->r.cfra= 1;
 	sce->r.sfra= 1;
@@ -157,8 +181,8 @@ Scene *add_scene(char *name)
 	sce->r.ysch= 256;
 	sce->r.xasp= 1;
 	sce->r.yasp= 1;
-	sce->r.xparts= 1;
-	sce->r.yparts= 1;
+	sce->r.xparts= 4;
+	sce->r.yparts= 4;
 	sce->r.size= 100;
 	sce->r.planes= 24;
 	sce->r.quality= 90;
@@ -167,18 +191,43 @@ Scene *add_scene(char *name)
 	sce->r.framelen= 1.0;
 	sce->r.frs_sec= 25;
 
+	sce->r.postgamma= 1.0;
+	sce->r.postsat= 1.0;
+	sce->r.postmul= 1.0;
+	
+	sce->r.focus= 0.9;
+	sce->r.zgamma= 1.0;
+	sce->r.zsigma= 4.0;
+	sce->r.zblur= 10.0;
+	sce->r.zmin= 0.8;
+	
 	sce->r.xplay= 640;
 	sce->r.yplay= 480;
 	sce->r.freqplay= 60;
 	sce->r.depth= 32;
 
-	if (sce->r.avicodecdata) {
-printf("this is not good\n");
-	}
-//	sce->r.imtype= R_TARGA;
-
 	sce->r.stereomode = 1;  // no stereo
 
+	sce->toolsettings = MEM_callocN(sizeof(struct ToolSettings),"Tool Settings Struct");
+	sce->toolsettings->cornertype=1;
+	sce->toolsettings->degr = 90; 
+	sce->toolsettings->step = 9;
+	sce->toolsettings->turn = 1; 				
+	sce->toolsettings->extr_offs = 1; 
+	sce->toolsettings->doublimit = 0.001;
+	sce->toolsettings->segments = 32;
+	sce->toolsettings->rings = 32;
+	sce->toolsettings->vertices = 32;
+	sce->toolsettings->editbutflag = 1;
+	sce->toolsettings->uvcalc_radius = 1.0f;
+	sce->toolsettings->uvcalc_cubesize = 1.0f;
+	sce->toolsettings->uvcalc_mapdir = 1;
+	sce->toolsettings->uvcalc_mapalign = 1;
+	sce->toolsettings->unwrapper = 1;
+	
+	sce->jumpframe = 10;
+	sce->audio.mixrate = 44100;
+
 	strcpy(sce->r.backbuf, "//backbuf");
 	strcpy(sce->r.pic, U.renderdir);
 	strcpy(sce->r.ftype, "//ftype");
@@ -186,174 +235,24 @@ printf("this is not good\n");
 	BLI_init_rctf(&sce->r.safety, 0.1f, 0.9f, 0.1f, 0.9f);
 	sce->r.osa= 8;
 	
+	/* note; in header_info.c the scene copy happens..., if you add more to renderdata it has to be checked there */
+	scene_add_render_layer(sce);
+	
 	return sce;
 }
 
-int object_in_scene(Object *ob, Scene *sce)
+Base *object_in_scene(Object *ob, Scene *sce)
 {
 	Base *base;
 	
 	base= sce->base.first;
 	while(base) {
-		if(base->object == ob) return 1;
+		if(base->object == ob) return base;
 		base= base->next;
 	}
-	return 0;
+	return NULL;
 }
 
-void sort_baselist(Scene *sce)
-{
-	/* in order of parent and track */
-	ListBase tempbase, noparentbase, notyetbase;
-	Base *base, *test=NULL;
-	Object *par;
-	int doit, domore= 0, lastdomore=1;
-	
-	
-	/* keep same order when nothing has changed! */
-	
-	while(domore!=lastdomore) {
-
-		lastdomore= domore;
-		domore= 0;
-		tempbase.first= tempbase.last= 0;
-		noparentbase.first= noparentbase.last= 0;
-		notyetbase.first= notyetbase.last= 0;
-		
-		while( (base= sce->base.first) ) {
-			BLI_remlink(&sce->base, base);
-			
-			par= 0;
-			if(base->object->type==OB_IKA) {
-				Ika *ika= base->object->data;
-				par= ika->parent;
-			}
-
-			if(par || base->object->parent || base->object->track) {
-				
-				doit= 0;
-				if(base->object->parent) doit++;
-				if(base->object->track) doit++;
-				
-			/* Count constraints */
-				{
-					bConstraint *con;
-					for (con = base->object->constraints.first; con; con=con->next){
-						switch (con->type){
-						case CONSTRAINT_TYPE_KINEMATIC:
-							{
-								bKinematicConstraint *data=con->data;
-								if (data->tar) doit++;
-							}
-							break;
-						case CONSTRAINT_TYPE_TRACKTO:
-							{
-								bTrackToConstraint *data=con->data;
-								if (data->tar) doit++;
-							}
-							break;
-						case CONSTRAINT_TYPE_NULL:
-							break;
-						case CONSTRAINT_TYPE_ROTLIKE:
-							{
-								bRotateLikeConstraint *data=con->data;
-								if (data->tar) doit++;
-								
-							}
-							break;
-						case CONSTRAINT_TYPE_LOCLIKE:
-							{
-								bLocateLikeConstraint *data=con->data;
-								if (data->tar) doit++;
-							}
-							break;
-						case CONSTRAINT_TYPE_ACTION:
-							{
-								bActionConstraint *data=con->data;
-								if (data->tar) doit++;
-							}
-							break;
-						default:
-							break;
-						}
-					}
-				}
-				
-				if(par) doit++;
-				
-				test= tempbase.first;
-				while(test) {
-					
-					if(test->object==base->object->parent) doit--;
-					if(test->object==base->object->track) doit--;
-					if(test->object==par) doit--;
-					
-					/* Decrement constraints */
-					{
-						bConstraint *con;
-						for (con = base->object->constraints.first; con; con=con->next){
-							switch (con->type){
-							case CONSTRAINT_TYPE_KINEMATIC:
-								{
-									bKinematicConstraint *data=con->data;
-									if (test->object == data->tar && test->object!=base->object) doit--;
-								}
-								break;
-							case CONSTRAINT_TYPE_TRACKTO:
-								{
-									bTrackToConstraint *data=con->data;
-									if (test->object == data->tar && test->object!=base->object) doit--;
-								}
-								break;
-							case CONSTRAINT_TYPE_NULL:
-								break;
-							case CONSTRAINT_TYPE_ROTLIKE:
-								{
-									bRotateLikeConstraint *data=con->data;
-									if (test->object == data->tar && test->object!=base->object) doit--;
-									
-								}
-								break;
-							case CONSTRAINT_TYPE_LOCLIKE:
-								{
-									bLocateLikeConstraint *data=con->data;
-									if (test->object == data->tar && test->object!=base->object) doit--;
-								}
-								break;
-							case CONSTRAINT_TYPE_ACTION:
-								{
-									bActionConstraint *data=con->data;
-									if (test->object == data->tar && test->object!=base->object) doit--;
-								}
-								break;
-							default:
-								break;
-							}
-						}
-					}
-					
-					if(doit==0) break;
-					test= test->next;
-				}
-				
-				if(test) BLI_insertlink(&tempbase, test, base);
-				else {
-					BLI_addhead(&tempbase, base);
-					domore++;
-				}
-				
-			}
-			else BLI_addtail(&noparentbase, base);
-			
-		}
-		sce->base= noparentbase;
-		addlisttolist(&sce->base, &tempbase);
-		addlisttolist(&sce->base, &notyetbase);
-
-	}
-}
-
-
 void set_scene_bg(Scene *sce)
 {
 	Base *base;
@@ -364,6 +263,9 @@ void set_scene_bg(Scene *sce)
 	
 	G.scene= sce;
 	
+	/* check for cyclic sets, for reading old files but also for definite security (py?) */
+	scene_check_setscene(G.scene);
+	
 	/* deselect objects (for dataselect) */
 	ob= G.main->object.first;
 	while(ob) {
@@ -383,34 +285,31 @@ void set_scene_bg(Scene *sce)
 	}
 
 	/* sort baselist */
-	sort_baselist(sce);
+	DAG_scene_sort(sce);
 
 	/* copy layers and flags from bases to objects */
 	base= G.scene->base.first;
 	while(base) {
+		ob= base->object;
+		ob->lay= base->lay;
 		
-		base->object->lay= base->lay;
-		
-		base->flag &= ~OB_FROMGROUP;
-		flag= base->object->flag & OB_FROMGROUP;
+		/* group patch... */
+		base->flag &= ~(OB_FROMGROUP);
+		flag= ob->flag & (OB_FROMGROUP);
 		base->flag |= flag;
 		
-		base->object->ctime= -1234567.0;	/* force ipo to be calculated later */
+		ob->flag= base->flag;
+		
+		ob->ctime= -1234567.0;	/* force ipo to be calculated later */
 		base= base->next;
 	}
-
-	do_all_ipos();	/* layers/materials */
-
-	BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
-	do_all_keys();
-#ifdef __NLA
-	do_all_actions();
-#endif
-	do_all_ikas();
-
-
+	/* no full animation update, this to enable render code to work (render code calls own animation updates) */
+	
+	/* do we need FRAMECHANGED in set_scene? */
+//	if (G.f & G_DOSCRIPTLINKS) BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
 }
 
+/* called from creator.c */
 void set_scene_name(char *name)
 {
 	Scene *sce;
@@ -430,24 +329,22 @@ void set_scene_name(char *name)
  */
 int next_object(int val, Base **base, Object **ob)
 {
-	extern ListBase duplilist;
-	static Object *dupob;
+	static ListBase *duplilist= NULL;
+	static DupliObject *dupob;
 	static int fase;
 	int run_again=1;
 	
 	/* init */
 	if(val==0) {
 		fase= F_START;
-		dupob= 0;
+		dupob= NULL;
 	}
 	else {
-		
+
 		/* run_again is set when a duplilist has been ended */
 		while(run_again) {
 			run_again= 0;
-			
-				
-				
+
 			/* the first base */
 			if(fase==F_START) {
 				*base= G.scene->base.first;
@@ -481,30 +378,40 @@ int next_object(int val, Base **base, Object **ob)
 				}
 			}
 			
-			if(*base == 0) fase= F_START;
+			if(*base == NULL) fase= F_START;
 			else {
 				if(fase!=F_DUPLI) {
 					if( (*base)->object->transflag & OB_DUPLI) {
 						
-						make_duplilist(G.scene, (*base)->object);
-						dupob= duplilist.first;
+						duplilist= object_duplilist(G.scene, (*base)->object);
+						
+						dupob= duplilist->first;
 						
 					}
 				}
 				/* handle dupli's */
 				if(dupob) {
 					
-					*ob= dupob;
+					Mat4CpyMat4(dupob->ob->obmat, dupob->mat);
+					
+					(*base)->flag |= OB_FROMDUPLI;
+					*ob= dupob->ob;
 					fase= F_DUPLI;
 					
-					dupob= dupob->id.next;
+					dupob= dupob->next;
 				}
 				else if(fase==F_DUPLI) {
 					fase= F_SCENE;
-					free_duplilist();
+					(*base)->flag &= ~OB_FROMDUPLI;
+					
+					for(dupob= duplilist->first; dupob; dupob= dupob->next) {
+						Mat4CpyMat4(dupob->ob->obmat, dupob->omat);
+					}
+					
+					free_object_duplilist(duplilist);
+					duplilist= NULL;
 					run_again= 1;
 				}
-				
 			}
 		}
 	}
@@ -555,3 +462,79 @@ void scene_select_base(Scene *sce, Base *selbase)
 
 	sce->basact= selbase;
 }
+
+/* checks for cycle, returns 1 if it's all OK */
+int scene_check_setscene(Scene *sce)
+{
+	Scene *scene;
+	
+	if(sce->set==NULL) return 1;
+	
+	/* LIB_DOIT is the free flag to tag library data */
+	for(scene= G.main->scene.first; scene; scene= scene->id.next)
+		scene->id.flag &= ~LIB_DOIT;
+	
+	scene= sce;
+	while(scene->set) {
+		scene->id.flag |= LIB_DOIT;
+		/* when set has flag set, we got a cycle */
+		if(scene->set->id.flag & LIB_DOIT)
+			break;
+		scene= scene->set;
+	}
+	
+	if(scene->set) {
+		/* the tested scene gets zero'ed, that's typically current scene */
+		sce->set= NULL;
+		return 0;
+	}
+	return 1;
+}
+
+/* applies changes right away */
+void scene_update_for_newframe(Scene *sce, unsigned int lay)
+{
+	Base *base;
+	Object *ob;
+	
+	/* object ipos are calculated in where_is_object */
+	do_all_data_ipos();
+	
+	if (G.f & G_DOSCRIPTLINKS) BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
+	
+	while(sce) {
+		if(sce->theDag==NULL)
+			DAG_scene_sort(sce);
+		
+		DAG_scene_update_flags(sce, lay);   // only stuff that moves or needs display still
+		
+		for(base= sce->base.first; base; base= base->next) {
+			ob= base->object;
+			
+			object_handle_update(ob);   // bke_object.h
+			
+			/* only update layer when an ipo */
+			if(ob->ipo && has_ipo_code(ob->ipo, OB_LAY) ) {
+				base->lay= ob->lay;
+			}
+		}
+		sce= sce->set;
+	}
+}
+
+/* return default layer, also used to patch old files */
+void scene_add_render_layer(Scene *sce)
+{
+	SceneRenderLayer *srl;
+	int tot= 1 + BLI_countlist(&sce->r.layers);
+	
+	srl= MEM_callocN(sizeof(SceneRenderLayer), "new render layer");
+	sprintf(srl->name, "%d RenderLayer", tot);
+	BLI_addtail(&sce->r.layers, srl);
+
+	/* note, this is also in render, pipeline.c, to make layer when scenedata doesnt have it */
+	srl->lay= (1<<20) -1;
+	srl->layflag= 0x7FFF;	/* solid ztra halo edge strand */
+	srl->passflag= SCE_PASS_COMBINED|SCE_PASS_Z;
+}
+
