@@ -1,7 +1,7 @@
 /* text.c
  *
  *
- * $Id: text.c,v 1.5 2003/07/10 18:32:42 phlo Exp $
+ * $Id: text.c,v 1.21 2006/04/03 20:31:10 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,6 +39,7 @@
 
 #include "BLI_blenlib.h"
 
+#include "DNA_scene_types.h"
 #include "DNA_text_types.h"
 
 #include "BKE_bad_level_calls.h"
@@ -48,7 +49,7 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 
-#include "BPY_extern.h" // Blender Python library
+#include "BPY_extern.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -73,7 +74,8 @@ A text should relate to a file as follows -
 	TXT_ISEXT - should always be set if the Text is not to be written into
 					the .blend
 	TXT_ISSCRIPT - should be set if the user has designated the text
-					as a script.
+					as a script. (NEW: this was unused, but now it is needed by
+					space handler script links (see header_view3d.c, for example)
 
 ->>> see also: /makesdna/DNA_text_types.h
 
@@ -139,14 +141,18 @@ int txt_get_undostate(void)
 void free_text(Text *text)
 {
 	TextLine *tmp;
-	
-	for (tmp= text->lines.first; tmp; tmp= tmp->next)
+
+	for (tmp= text->lines.first; tmp; tmp= tmp->next) {
 		MEM_freeN(tmp->line);
+		if (tmp->format)
+		  MEM_freeN(tmp->format);
+	}
 	
 	BLI_freelistN(&text->lines);
 
 	if(text->name) MEM_freeN(text->name);
 	MEM_freeN(text->undo_buf);
+	if (text->compiled) BPY_free_compiled_text(text);
 }
 
 Text *add_empty_text(void) 
@@ -170,7 +176,8 @@ Text *add_empty_text(void)
 
 	tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 	tmp->line= (char*) MEM_mallocN(1, "textline_string");
-
+	tmp->format= (char*) MEM_mallocN(2, "Syntax_format");
+	
 	tmp->line[0]=0;
 	tmp->len= 0;
 				
@@ -211,19 +218,24 @@ int reopen_text(Text *text)
 	TextLine *tmp;
 	char sdir[FILE_MAXDIR];
 	char sfile[FILE_MAXFILE];
+	char str[FILE_MAXDIR+FILE_MAXFILE];
 
 	if (!text || !text->name) return 0;
 	
-	BLI_split_dirfile(text->name, sdir, sfile);
+	BLI_strncpy(str, text->name, FILE_MAXDIR+FILE_MAXFILE);
+	BLI_convertstringcode(str, G.sce, G.scene->r.cfra);
+	BLI_split_dirfile(str, sdir, sfile);
 	
-	fp= fopen(text->name, "r");
+	fp= fopen(str, "r");
 	if(fp==NULL) return 0;
 
 	/* free memory: */
 
-	for (tmp= text->lines.first; tmp; tmp= tmp->next)
+	for (tmp= text->lines.first; tmp; tmp= tmp->next) {
 		MEM_freeN(tmp->line);
-
+		if (tmp->format) MEM_freeN(tmp->format);
+	}
+	
 	BLI_freelistN(&text->lines);
 
 	text->lines.first= text->lines.last= NULL;
@@ -257,7 +269,8 @@ int reopen_text(Text *text)
 		if (buffer[i]=='\n') {
 			tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 			tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
-				
+			tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
+			
 			if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 			tmp->line[llen]=0;
 			tmp->len= llen;
@@ -276,6 +289,7 @@ int reopen_text(Text *text)
 	if (llen!=0 || text->nlines==0) {
 		tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 		tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
+		tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
 		
 		if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 
@@ -304,10 +318,14 @@ Text *add_text(char *file)
 	Text *ta;
 	char sdir[FILE_MAXDIR];
 	char sfile[FILE_MAXFILE];
+	char str[FILE_MAXDIR+FILE_MAXFILE];
 
-	BLI_split_dirfile(file, sdir, sfile);
+	BLI_strncpy(str, file, FILE_MAXDIR+FILE_MAXFILE);
+	if (G.scene) /* can be NULL (bg mode) */
+		BLI_convertstringcode(str, G.sce, G.scene->r.cfra);
+	BLI_split_dirfile(str, sdir, sfile);
 	
-	fp= fopen(file, "r");
+	fp= fopen(str, "r");
 	if(fp==NULL) return NULL;
 	
 	ta= alloc_libblock(&G.main->text, ID_TXT, sfile);
@@ -344,7 +362,8 @@ Text *add_text(char *file)
 		if (buffer[i]=='\n') {
 			tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 			tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
-				
+			tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
+			
 			if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 			tmp->line[llen]=0;
 			tmp->len= llen;
@@ -363,6 +382,7 @@ Text *add_text(char *file)
 	if (llen!=0 || ta->nlines==0) {
 		tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 		tmp->line= (char*) MEM_mallocN(llen+1, "textline_string");
+		tmp->format= (char*) MEM_mallocN(llen+2, "Syntax_format");
 		
 		if(llen) memcpy(tmp->line, &buffer[i-llen], llen);
 
@@ -405,8 +425,10 @@ Text *copy_text(Text *ta)
 	while (line) {
 		tmp= (TextLine*) MEM_mallocN(sizeof(TextLine), "textline");
 		tmp->line= MEM_mallocN(line->len+1, "textline_string");
-		strcpy(tmp->line, line->line);
+		tmp->format= MEM_mallocN(line->len+2, "Syntax_format");
 		
+		strcpy(tmp->line, line->line);
+
 		tmp->len= line->len;
 		
 		BLI_addtail(&tan->lines, tmp);
@@ -424,12 +446,14 @@ Text *copy_text(Text *ta)
 /* Editing utility functions */
 /*****************************/
 
-static void make_new_line (TextLine *line, char *newline) 
+static void make_new_line (TextLine *line, char *newline, char *newformat) 
 {
-	MEM_freeN(line->line);
+	if (line->line) MEM_freeN(line->line);
+	if (line->format) MEM_freeN(line->format);
 	
 	line->line= newline;
-	line->len= strlen(newline);	
+	line->len= strlen(newline);
+	line->format= newformat;
 }
 
 static TextLine *txt_new_line(char *str)
@@ -440,6 +464,8 @@ static TextLine *txt_new_line(char *str)
 	
 	tmp= (TextLine *) MEM_mallocN(sizeof(TextLine), "textline");
 	tmp->line= MEM_mallocN(strlen(str)+1, "textline_string");
+	tmp->format= MEM_mallocN(strlen(str)+2, "Syntax_format");
+	
 	strcpy(tmp->line, str);
 	
 	tmp->len= strlen(str);
@@ -456,6 +482,8 @@ static TextLine *txt_new_linen(char *str, int n)
 	
 	tmp= (TextLine *) MEM_mallocN(sizeof(TextLine), "textline");
 	tmp->line= MEM_mallocN(n+1, "textline_string");
+	tmp->format= MEM_mallocN(n+2, "Syntax_format");
+	
 	BLI_strncpy(tmp->line, str, n+1);
 	
 	tmp->len= strlen(tmp->line);
@@ -850,7 +878,7 @@ int txt_has_sel(Text *text)
 static void txt_delete_sel (Text *text)
 {
 	TextLine *tmpl;
-	char *buf;
+	char *buf, *format;
 	
 	if (!text) return;
 	if (!text->curl) return;
@@ -867,12 +895,13 @@ static void txt_delete_sel (Text *text)
 	}
 
 	buf= MEM_mallocN(text->curc+(text->sell->len - text->selc)+1, "textline_string");
+	format= MEM_mallocN(text->curc+(text->sell->len - text->selc)+2, "Syntax_format");
 	
 	strncpy(buf, text->curl->line, text->curc);
 	strcpy(buf+text->curc, text->sell->line + text->selc);
 	buf[text->curc+(text->sell->len - text->selc)]=0;
-	
-	make_new_line(text->curl, buf);
+
+	make_new_line(text->curl, buf, format);
 	
 	tmpl= text->sell;
 	while (tmpl != text->curl) {
@@ -1310,6 +1339,14 @@ void txt_print_undo(Text *text)
 			ops= "Delete text block";
 		} else if (op==UNDO_IBLOCK) {
 			ops= "Insert text block";
+		} else if (op==UNDO_INDENT) {
+			ops= "Indent ";
+		} else if (op==UNDO_UNINDENT) {
+			ops= "Unindent ";
+		} else if (op==UNDO_COMMENT) {
+			ops= "Comment ";
+		} else if (op==UNDO_UNCOMMENT) {
+			ops= "Uncomment ";
 		} else {
 			ops= "Unknown";
 		}
@@ -1361,6 +1398,28 @@ void txt_print_undo(Text *text)
 			linep= linep+(text->undo_buf[i]<<16); i++;
 			linep= linep+(text->undo_buf[i]<<24); i++;
 			printf ("> (%d)", linep);
+		} else if (op==UNDO_INDENT || op==UNDO_UNINDENT) {
+			i++;
+
+			charp= text->undo_buf[i]; i++;
+			charp= charp+(text->undo_buf[i]<<8); i++;
+
+			linep= text->undo_buf[i]; i++;
+			linep= linep+(text->undo_buf[i]<<8); i++;
+			linep= linep+(text->undo_buf[i]<<16); i++;
+			linep= linep+(text->undo_buf[i]<<24); i++;
+			
+			printf ("to <%d, %d> ", linep, charp);
+
+			charp= text->undo_buf[i]; i++;
+			charp= charp+(text->undo_buf[i]<<8); i++;
+
+			linep= text->undo_buf[i]; i++;
+			linep= linep+(text->undo_buf[i]<<8); i++;
+			linep= linep+(text->undo_buf[i]<<16); i++;
+			linep= linep+(text->undo_buf[i]<<24); i++;
+			
+			printf ("from <%d, %d>", linep, charp);
 		}
 		
 		printf (" %d\n",  i);
@@ -1379,7 +1438,7 @@ static void txt_undo_add_op(Text *text, int op)
 
 static void txt_undo_add_block(Text *text, int op, char *buf)
 {
-	unsigned int length;
+	int length;
 	
 	length= strlen(buf);
 	
@@ -1473,8 +1532,8 @@ static void txt_undo_add_charop(Text *text, int op, char c)
 
 void txt_do_undo(Text *text)
 {
-	int op= text->undo_buf[text->undo_pos], i;
-	unsigned int linep;
+	int op= text->undo_buf[text->undo_pos];
+	unsigned int linep, i;
 	unsigned short charp;
 	TextLine *holdl;
 	int holdc, holdln;
@@ -1582,7 +1641,7 @@ void txt_do_undo(Text *text)
 			linep= (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
 
 			buf= MEM_mallocN(linep+1, "dblock buffer");
-			for (i=0; i < (int)linep; i++){
+			for (i=0; i < linep; i++){
 				buf[(linep-1)-i]= text->undo_buf[text->undo_pos]; 
 				text->undo_pos--;
 			}
@@ -1633,7 +1692,54 @@ void txt_do_undo(Text *text)
 			text->undo_pos--;
 
 			break;
+		case UNDO_INDENT:
+		case UNDO_UNINDENT:
+		case UNDO_COMMENT:
+		case UNDO_UNCOMMENT:
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//linep is now the end line of the selection
 			
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos--;
+			charp = (charp<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//charp is the last char selected or text->line->len
+			//set the selcetion for this now
+			text->selc = charp;
+			text->sell = text->lines.first;
+			for (i= 0; i < linep; i++) {
+				text->sell = text->sell->next;
+			}
+
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//first line to be selected
+			
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos--;
+			charp = (charp<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//first postion to be selected
+			text->curc = charp;
+			text->curl = text->lines.first;
+			for (i = 0; i < linep; i++) {
+				text->curl = text->curl->next;
+			}
+
+			
+			if (op==UNDO_INDENT) {
+				unindent(text);
+			} else if (op== UNDO_UNINDENT) {
+				indent(text);
+			} else if (op == UNDO_COMMENT) {
+				uncomment(text);
+			} else if (op == UNDO_UNCOMMENT) {
+				comment(text);
+			}
+
+			text->undo_pos--;
+			break;
 		default:
 			error("Undo buffer error - resetting");
 			text->undo_pos= -1;
@@ -1647,7 +1753,7 @@ void txt_do_undo(Text *text)
 void txt_do_redo(Text *text)
 {
 	char op;
-	unsigned int linep;
+	unsigned int linep, i;
 	unsigned short charp;
 	char *buf;
 	
@@ -1660,7 +1766,7 @@ void txt_do_redo(Text *text)
 	}
 	
 	undoing= 1;
-	
+
 	switch(op) {
 		case UNDO_CLEFT:
 			txt_move_left(text, 0);
@@ -1788,7 +1894,52 @@ void txt_do_redo(Text *text)
 			linep= linep+(text->undo_buf[text->undo_pos]<<24); text->undo_pos++;
 
 			break;
+		case UNDO_INDENT:
+		case UNDO_UNINDENT:
+		case UNDO_COMMENT:
+		case UNDO_UNCOMMENT:
+			text->undo_pos++;
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos++;
+			charp = charp+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			//charp is the first char selected or 0
+			
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<16); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<24); text->undo_pos++;
+			//linep is now the first line of the selection			
+			//set the selcetion for this now
+			text->curc = charp;
+			text->curl = text->lines.first;
+			for (i= 0; i < linep; i++) {
+				text->curl = text->curl->next;
+			}
+			
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos++;
+			charp = charp+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			//last postion to be selected
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<16); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<24); text->undo_pos++;
+			//Last line to be selected
+			
+			text->selc = charp;
+			text->sell = text->lines.first;
+			for (i = 0; i < linep; i++) {
+				text->sell = text->sell->next;
+			}
 
+			if (op==UNDO_INDENT) {
+				indent(text);
+			} else if (op== UNDO_UNINDENT) {
+				unindent(text);
+			} else if (op == UNDO_COMMENT) {
+				comment(text);
+			} else if (op == UNDO_UNCOMMENT) {
+				uncomment(text);
+			}
+			break;
 		default:
 			error("Undo buffer error - resetting");
 			text->undo_pos= -1;
@@ -1805,7 +1956,7 @@ void txt_do_redo(Text *text)
 
 void txt_split_curline (Text *text) {
 	TextLine *ins;
-	char *left, *right;
+	char *left, *right, *fleft, *fright;
 	
 	if (!text) return;
 	if (!text->curl) return;
@@ -1815,22 +1966,27 @@ void txt_split_curline (Text *text) {
 	/* Make the two half strings */
 
 	left= MEM_mallocN(text->curc+1, "textline_string");
+	fleft= MEM_mallocN(text->curc+2, "Syntax_format");
 	if (text->curc) memcpy(left, text->curl->line, text->curc);
 	left[text->curc]=0;
 	
 	right= MEM_mallocN(text->curl->len - text->curc+1, "textline_string");
+	fright= MEM_mallocN(text->curl->len - text->curc+2, "Syntax_format");
 	if (text->curl->len - text->curc) memcpy(right, text->curl->line+text->curc, text->curl->len-text->curc);
 	right[text->curl->len - text->curc]=0;
 
 	MEM_freeN(text->curl->line);
+	if (text->curl->format) MEM_freeN(text->curl->format);
 
 	/* Make the new TextLine */
 	
 	ins= MEM_mallocN(sizeof(TextLine), "textline");
 	ins->line= left;
+	ins->format= fleft;
 	ins->len= text->curc;
 	
 	text->curl->line= right;
+	text->curl->format= fright;
 	text->curl->len= text->curl->len - text->curc;
 	
 	BLI_insertlinkbefore(&text->lines, text->curl, ins);	
@@ -1852,6 +2008,8 @@ static void txt_delete_line (Text *text, TextLine *line)
 	BLI_remlink (&text->lines, line);
 	
 	if (line->line) MEM_freeN(line->line);
+	if (line->format) MEM_freeN(line->format);
+
 	MEM_freeN(line);
 
 	txt_make_dirty(text);
@@ -1860,18 +2018,19 @@ static void txt_delete_line (Text *text, TextLine *line)
 
 static void txt_combine_lines (Text *text, TextLine *linea, TextLine *lineb)
 {
-	char *tmp;
+	char *tmp, *format;
 	
 	if (!text) return;
 	
 	if(!linea || !lineb) return;
 	
 	tmp= MEM_mallocN(linea->len+lineb->len+1, "textline_string");
+	format= MEM_mallocN(linea->len+lineb->len+1, "Syntax_format");
 	
 	strcpy(tmp, linea->line);
 	strcat(tmp, lineb->line);
-	
-	make_new_line(linea, tmp);
+
+	make_new_line(linea, tmp, format);
 	
 	txt_delete_line(text, lineb); 
 	
@@ -1886,12 +2045,10 @@ void txt_delete_char (Text *text)
 	if (!text) return;
 	if (!text->curl) return;
 
-	if (txt_has_sel(text)) {
-		txt_delete_sel(text);
-		return;
+	if (txt_has_sel(text)) { /* deleting a selection */
+	  txt_delete_sel(text);
 	}
-	
-	if (text->curc== text->curl->len) { /* Appending two lines */
+	else if (text->curc== text->curl->len) { /* Appending two lines */
 		if (text->curl->next) {
 			txt_combine_lines(text, text->curl, text->curl->next);
 			txt_pop_sel(text);
@@ -1921,31 +2078,30 @@ void txt_backspace_char (Text *text) {
 	if (!text) return;
 	if (!text->curl) return;
 	
-	if (txt_has_sel(text)) {
-		txt_delete_sel(text);
-		return;
+	if (txt_has_sel(text)) { /* deleting a selection */
+	  txt_delete_sel(text);
 	}
-	
-	if (text->curc==0) { /* Appending two lines */
-		if (text->curl->prev) {
-			text->curl= text->curl->prev;
-			text->curc= text->curl->len;
+	else if (text->curc==0) { /* Appending two lines */
+	    if (text->curl->prev) {
+	      text->curl= text->curl->prev;
+	      text->curc= text->curl->len;
 			
-			txt_combine_lines(text, text->curl, text->curl->next);
-			txt_pop_sel(text);
-		}
-	} else { /* Just backspacing a char */
-		int i= text->curc-1;
+	      txt_combine_lines(text, text->curl, text->curl->next);
+	      txt_pop_sel(text);
+	    }
+	} 
+	else { /* Just backspacing a char */
+	  int i= text->curc-1;
 		
-		c= text->curl->line[i];
-		while(i< text->curl->len) {
-			text->curl->line[i]= text->curl->line[i+1];
-			i++;
-		}
-		text->curl->len--;
-		text->curc--;
+	  c= text->curl->line[i];
+	  while(i< text->curl->len) {
+	    text->curl->line[i]= text->curl->line[i+1];
+	    i++;
+	  }
+	  text->curl->len--;
+	  text->curc--;
 		
-		txt_pop_sel(text);
+	  txt_pop_sel(text);
 	}
 
 	txt_make_dirty(text);
@@ -1956,7 +2112,7 @@ void txt_backspace_char (Text *text) {
 
 int txt_add_char (Text *text, char add) {
 	int len;
-	char *tmp;
+	char *tmp, *format;
 	
 	if (!text) return 0;
 	if (!text->curl) return 0;
@@ -1971,6 +2127,7 @@ int txt_add_char (Text *text, char add) {
 	txt_delete_sel(text);
 	
 	tmp= MEM_mallocN(text->curl->len+2, "textline_string");
+	format= MEM_mallocN(text->curl->len+4, "Syntax_format");
 	
 	if(text->curc) memcpy(tmp, text->curl->line, text->curc);
 	tmp[text->curc]= add;
@@ -1978,8 +2135,7 @@ int txt_add_char (Text *text, char add) {
 	len= text->curl->len - text->curc;
 	if(len>0) memcpy(tmp+text->curc+1, text->curl->line+text->curc, len);
 	tmp[text->curl->len+1]=0;
-	
-	make_new_line(text->curl, tmp);
+	make_new_line(text->curl, tmp, format);
 		
 	text->curc++;
 
@@ -1991,3 +2147,253 @@ int txt_add_char (Text *text, char add) {
 	if(!undoing) txt_undo_add_charop(text, UNDO_INSERT, add);
 	return 1;
 }
+
+void indent(Text *text)
+{
+	int len, num;
+	char *tmp, *format;
+	char add = '\t';
+	
+	if (!text) return;
+	if (!text->curl) return;
+	if (!text->sell) return;
+
+	num = 0;
+	while (TRUE)
+	{
+		tmp= MEM_mallocN(text->curl->len+2, "textline_string");
+		format= MEM_mallocN(text->curl->len+3, "Syntax_format");
+		
+		text->curc = 0; 
+		if(text->curc) memcpy(tmp, text->curl->line, text->curc);
+		tmp[text->curc]= add;
+		
+		len= text->curl->len - text->curc;
+		if(len>0) memcpy(tmp+text->curc+1, text->curl->line+text->curc, len);
+		tmp[text->curl->len+1]=0;
+
+		make_new_line(text->curl, tmp, format);
+			
+		text->curc++;
+		
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
+		}
+	}
+	text->curc = 0;
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
+	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_INDENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
+}
+
+void unindent(Text *text)
+{
+	int num = 0;
+	char remove = '\t';
+	
+	if (!text) return;
+	if (!text->curl) return;
+	if (!text->sell) return;
+
+	while(TRUE)
+	{
+		int i = 0;
+		
+		if (text->curl->line[i] == remove)
+		{
+			while(i< text->curl->len) {
+				text->curl->line[i]= text->curl->line[i+1];
+				i++;
+			}
+			text->curl->len--;
+		}
+			 
+	
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
+		}
+		
+	}
+	text->curc = 0;
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
+	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_UNINDENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
+}
+
+void comment(Text *text)
+{
+	int len, num;
+	char *tmp, *format;
+	char add = '#';
+	
+	if (!text) return;
+	if (!text->curl) return;
+	if (!text->sell) return;// Need to change this need to check if only one line is selected ot more then one
+
+	num = 0;
+	while (TRUE)
+	{
+		tmp= MEM_mallocN(text->curl->len+2, "textline_string");
+		format = MEM_mallocN(text->curl->len+3, "Syntax_format");
+		
+		text->curc = 0; 
+		if(text->curc) memcpy(tmp, text->curl->line, text->curc);
+		tmp[text->curc]= add;
+		
+		len= text->curl->len - text->curc;
+		if(len>0) memcpy(tmp+text->curc+1, text->curl->line+text->curc, len);
+		tmp[text->curl->len+1]=0;
+
+		make_new_line(text->curl, tmp, format);
+			
+		text->curc++;
+		
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
+		}
+	}
+	text->curc = 0;
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
+	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_COMMENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
+}
+
+void uncomment(Text *text)
+{
+	int num = 0;
+	char remove = '#';
+	
+	if (!text) return;
+	if (!text->curl) return;
+	if (!text->sell) return;
+
+	while(TRUE)
+	{
+		int i = 0;
+		
+		if (text->curl->line[i] == remove)
+		{
+			while(i< text->curl->len) {
+				text->curl->line[i]= text->curl->line[i+1];
+				i++;
+			}
+			text->curl->len--;
+		}
+			 
+	
+		txt_make_dirty(text);
+		txt_clean_text(text);
+		
+		if(text->curl == text->sell) 
+		{
+			text->selc = text->sell->len;
+			break;
+		} else {
+			text->curl = text->curl->next;
+			num++;
+		}
+		
+	}
+	text->curc = 0;
+	while( num > 0 )
+	{
+		text->curl = text->curl->prev;
+		num--;
+	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_UNCOMMENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
+}
+
+int setcurr_tab (Text *text)
+{
+	int i = 0;
+	int test = 0;
+	char *word = ":";
+	char *comm = "#";
+	char back_words[3][7] = {"return", "break", "pass"};
+	if (!text) return 0;
+	if (!text->curl) return 0;
+	
+	while (text->curl->line[i] == '\t')
+	{
+		//we only count thos tabs that are before any text or before the curs;
+		if (i == text->curc)
+		{
+			return i;
+		} else {
+			i++;
+		}
+	}
+	if(strstr(text->curl->line, word))
+	{
+		//if we find a : then add a tab but not if it is in a comment
+		if(strcspn(text->curl->line, word) < strcspn(text->curl->line, comm))
+		{
+			i++;
+
+		}
+	}
+
+	while(test < 3)
+	{
+		//if there are these 3 key words then remove a tab because we are done with the block
+		if(strstr(text->curl->line, back_words[test]) && i > 0)
+		{
+			if(strcspn(text->curl->line, back_words[test]) < strcspn(text->curl->line, comm))
+			{
+				i--;
+			}
+		}
+		test++;
+	}
+	return i;
+}
+
