@@ -3,7 +3,7 @@
 /**
  * blenlib/BLI_arithb.h    mar 2001 Nzc
  *
- * $Id: BLI_arithb.h,v 1.18 2005/04/22 20:16:01 blendix Exp $ 
+ * $Id: BLI_arithb.h,v 1.25 2006/06/03 17:21:45 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -155,6 +155,8 @@ Mat3ToEul(
 	float *eul
 );
 
+void compatible_eul(float *eul, float *oldrot);
+
 /**
  * @section Quaternion arithmetic routines
  */
@@ -175,6 +177,11 @@ QuatMul(
 	float *
 );
 	void 
+QuatMulVecf(
+	float *q, 
+	float *v
+);
+	void 
 NormalQuat(
 	float *
 );
@@ -190,6 +197,29 @@ QuatSub(
 	float *q1, 
 	float *q2
 );
+	void 
+QuatConj(
+	float *q 
+);
+	void 
+QuatInv(
+	float *q 
+);
+	void 
+QuatMulf(
+	float *q,
+	float f
+);
+	float 
+QuatDot(
+	float *q1, 
+	float *q2
+);
+	void
+printquat(
+	char *str,
+	float q[4]
+);
 
 
 void QuatInterpol(float *result, float *quat1, float *quat2, float t);
@@ -461,6 +491,12 @@ VecLength(
 VecMulf(
 	float *v1, 
 	float f
+);
+	int
+VecLenCompare(
+	float *v1,
+	float *v2,
+	float limit
 );
 	int 
 VecCompare(
@@ -517,6 +553,13 @@ VecAddf(
 	float *v, 
 	float *v1, 
 	float *v2
+);
+	void
+VecLerpf(
+	float *target, 
+	float *a, 
+	float *b,
+	float t
 );
 	void 
 VecUpMat3(
@@ -593,6 +636,12 @@ VecAngle3(
 	float *v3
 );
 
+	float
+VecAngle2(
+	float *v1, 
+	float *v2
+);
+
 	void 
 i_lookat(
 	float vx, float vy, 
@@ -615,6 +664,8 @@ hsv_to_rgb(
 	float *g, float *b
 );
 
+void hex_to_rgb(char *hexcol, float *r, float *g, float *b);
+
 	void 
 rgb_to_hsv(
 	float r, float g, float b, 
@@ -835,6 +886,7 @@ Normalise2(
 void tubemap(float x, float y, float z, float *u, float *v);
 void spheremap(float x, float y, float z, float *u, float *v);
 			  
+int LineIntersectsTriangle(float p1[3], float p2[3], float v0[3], float v1[3], float v2[3], float *lambda);
 			  
 #ifdef __cplusplus
 }
