@@ -1,5 +1,5 @@
 /**
- * $Id: BLI_blenlib.h,v 1.13 2005/05/20 12:18:11 jesterking Exp $
+ * $Id: BLI_blenlib.h,v 1.19 2006/06/19 13:52:59 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -59,7 +59,7 @@
  * - The blenlib uses type defines from makesdna/, and functions from
  * standard libraries.
  * 
- * $Id: BLI_blenlib.h,v 1.13 2005/05/20 12:18:11 jesterking Exp $ 
+ * $Id: BLI_blenlib.h,v 1.19 2006/06/19 13:52:59 ton Exp $ 
 */
 
 #ifndef BLI_BLENLIB_H
@@ -91,8 +91,9 @@ extern "C" {
 
 /* BLI_util.h */
 char *BLI_gethome(void);
-void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file);
+void BLI_make_file_string(const char *relabase, char *string,  const char *dir, const char *file);
 void BLI_make_exist(char *dir);
+void BLI_make_existing_file(char *name);
 void BLI_split_dirfile(char *string, char *dir, char *file);
 int BLI_testextensie(char *str, char *ext);
 void addlisttolist(ListBase *list1, ListBase *list2);
@@ -111,6 +112,13 @@ int BLI_countlist(struct ListBase *listbase);
 void BLI_freelinkN(ListBase *listbase, void *vlink);
 void BLI_splitdirstring(char *di,char *fi);
 
+	/**
+	 * dir can be any input, like from buttons, and this function
+	 * converts it to a regular full path.
+	 * Also removes garbage from directory paths, like /../ or double slashes etc 
+	 */
+void BLI_cleanup_dir(const char *relabase, char *dir);
+
 	/**
 	 * Blender's path code replacement function.
 	 * Bases @a path strings leading with "//" by the
@@ -123,9 +131,9 @@ void BLI_splitdirstring(char *di,char *fi);
 	 * @a framenum The framenumber to replace the frame code with.
 	 * @retval Returns true if the path was relative (started with "//").
 	 */
-int BLI_convertstringcode(char *path, char *basepath, int framenum);
+int BLI_convertstringcode(char *path, const char *basepath, int framenum);
 
-void BLI_makestringcode(char *fromfile, char *str);
+void BLI_makestringcode(const char *relfile, char *file);
 
 	/**
 	 * Change every @a from in @a string into @a to. The
@@ -248,7 +256,7 @@ int    BLI_exist(char *name);
 void  BLI_recurdir_fileops(char *dirname);
 int BLI_link(char *file, char *to);
 int BLI_backup(char *file, char *from, char *to);
-
+int BLI_is_writable(char *filename);
 
 /**
  * @attention Do not confuse with BLI_exist
@@ -256,6 +264,7 @@ int BLI_backup(char *file, char *from, char *to);
 int   BLI_exists(char *file);
 int   BLI_copy_fileops(char *file, char *to);
 int   BLI_rename(char *from, char *to);
+int   BLI_gzip(char *from, char *to);
 int   BLI_delete(char *file, int dir, int recursive);
 int   BLI_move(char *file, char *to);
 int   BLI_touch(char *file);
@@ -269,19 +278,21 @@ char *BLI_last_slash(char *string);
  *
  * @return True if @a rect is empty.
  */
-int BLI_rcti_is_empty(struct rcti * rect);
+int  BLI_rcti_is_empty(struct rcti * rect);
 void BLI_init_rctf(struct rctf *rect, float xmin, float xmax, float ymin, float ymax);
+void BLI_init_rcti(struct rcti *rect, int xmin, int xmax, int ymin, int ymax);
+void BLI_translate_rctf(struct rctf *rect, float x, float y);
+void BLI_translate_rcti(struct rcti *rect, int x, int y);
 int  BLI_in_rcti(struct rcti * rect, int x, int y);
 int  BLI_in_rctf(struct rctf *rect, float x, float y);
 int  BLI_isect_rctf(struct rctf *src1, struct rctf *src2, struct rctf *dest);
-/* why oh why doesn't this work? */
-//void BLI_union_rctf(struct rctf *rct1, struct rctf *rct2);
+int  BLI_isect_rcti(struct rcti *src1, struct rcti *src2, struct rcti *dest);
 void BLI_union_rctf(struct rctf *rcta, struct rctf *rctb);
 
 /* scanfill.c: used in displist only... */
 struct EditVert *BLI_addfillvert(float *vec);
 struct EditEdge *BLI_addfilledge(struct EditVert *v1, struct EditVert *v2);
-int BLI_edgefill(int mode);  /* DE HOOFD FILL ROUTINE */
+int BLI_edgefill(int mode, int mat_nr);
 void BLI_end_edgefill(void);
 
 /* noise.h: */
@@ -326,26 +337,9 @@ void BLI_setErrorCallBack(void (*f)(char*));
  */
 void BLI_setInterruptCallBack(int (*f)(void));
 
-/**
- * Before scanfilling is done, these two references need to be set. If
- * the object reference is NULL, the function will fail. If the object
- * is set, but no colour is available, colour can be omitted. 
- *
- * @attention Also see BLI_setScanFillColourRef
- */
-void BLI_setScanFillObjectRef(void* ob); 
-
-/**
- * Before scanfilling is done, these two references need to be set. If
- * the object reference is NULL, the function will fail. If the object
- * is set, but no colour is available, colour can be omitted. 
- *
- * @attention Also see BLI_setScanFillObjectRef
- */
-void BLI_setScanFillColourRef(char* c);
-
 int BLI_strcasecmp(const char *s1, const char *s2);
 int BLI_strncasecmp(const char *s1, const char *s2, int n);
+void BLI_timestr(double time, char *str);
 
 #define PRNTSUB(type,arg)			printf(#arg ": %" #type " ", arg)
 
