@@ -1,5 +1,5 @@
 /**
- * $Id: scanfill.c,v 1.9 2005/06/04 16:22:49 lukep Exp $
+ * $Id: scanfill.c,v 1.14 2006/06/30 16:07:50 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -54,19 +54,6 @@
 /* callbacks for errors and interrupts and some goo */
 static void (*BLI_localErrorCallBack)(char*) = NULL;
 static int (*BLI_localInterruptCallBack)(void) = NULL;
-static void *objectref = NULL;
-static char *colourref = NULL;
-
-
-void BLI_setScanFillObjectRef(void* ob)
-{
-	objectref = ob;
-}
-
-void BLI_setScanFillColourRef(char* c)
-{
-	colourref = c;
-}
 
 void BLI_setErrorCallBack(void (*f)(char*))
 {
@@ -86,8 +73,9 @@ void callLocalErrorCallBack(char* msg)
 	}
 }
 
+#if 0
 /* ignore if the interrupt wasn't set */
-int callLocalInterruptCallBack(void)
+static int callLocalInterruptCallBack(void)
 {
 	if (BLI_localInterruptCallBack) {
 		return BLI_localInterruptCallBack();
@@ -95,7 +83,7 @@ int callLocalInterruptCallBack(void)
 		return 0;
 	}
 }
-
+#endif
 
 /* local types */
 typedef struct PolyFill {
@@ -112,39 +100,21 @@ typedef struct ScFillVert {
 
 
 /* local funcs */
-int vergscdata(const void *a1, const void *a2);
-int vergpoly(const void *a1, const void *a2);
-void *new_mem_element(int size);
-void addfillface(EditVert *v1, EditVert *v2, EditVert *v3);
-int boundinside(PolyFill *pf1, PolyFill *pf2);
-int boundisect(PolyFill *pf2, PolyFill *pf1);
-void mergepolysSimp(PolyFill *pf1, PolyFill *pf2)	/* pf2 added to pf1 */;
-EditEdge *existfilledge(EditVert *v1, EditVert *v2);
-short addedgetoscanvert(ScFillVert *sc, EditEdge *eed);
-short testedgeside(float *v1, float *v2, float *v3);
-short testedgeside2(float *v1, float *v2, float *v3);
-short boundinsideEV(EditEdge *eed, EditVert *eve)	/* is eve within boundbox eed */;
-void testvertexnearedge(void);
-void scanfill(PolyFill *pf);
-void fill_mesh(void);
-ScFillVert *addedgetoscanlist(EditEdge *eed, int len);
-void splitlist(ListBase *tempve, ListBase *temped, short nr);
 
-/* This one is also used in isect.c Keep it here until we know what to do with isect.c */
-#define COMPLIMIT	0.0003
+#define COMPLIMIT	0.00003
 
-ScFillVert *scdata;
+static ScFillVert *scdata;
 
 ListBase fillvertbase = {0,0};
 ListBase filledgebase = {0,0};
 ListBase fillfacebase = {0,0};
 
-short cox, coy;
+static short cox, coy;
 
 /* ****  FUBCTIONS FOR QSORT *************************** */
 
 
-int vergscdata(const void *a1, const void *a2)
+static int vergscdata(const void *a1, const void *a2)
 {
 	const ScFillVert *x1=a1,*x2=a2;
 	
@@ -156,7 +126,7 @@ int vergscdata(const void *a1, const void *a2)
 	return 0;
 }
 
-int vergpoly(const void *a1, const void *a2)
+static int vergpoly(const void *a1, const void *a2)
 {
 	const PolyFill *x1=a1, *x2=a2;
 
@@ -181,7 +151,7 @@ struct mem_elements {
    free in the end, with argument '-1'
 */
 
-void *new_mem_element(int size)
+static void *new_mem_element(int size)
 {
 	int blocksize= 16384;
 	static int offs= 0;		/* the current free adress */
@@ -239,12 +209,9 @@ EditVert *BLI_addfillvert(float *vec)
 	eve= new_mem_element(sizeof(EditVert));
 	BLI_addtail(&fillvertbase, eve);
 	
-	if(vec) {
-		*(eve->co)     = *(vec);
-		*(eve->co + 1) = *(vec + 1);
-		*(eve->co + 2) = *(vec + 2);
-	}
-/*  	VECCOPY(eve->co, vec); */
+	eve->co[0] = vec[0];
+	eve->co[1] = vec[1];
+	eve->co[2] = vec[2];
 
 	return eve;	
 }
@@ -262,7 +229,7 @@ EditEdge *BLI_addfilledge(EditVert *v1, EditVert *v2)
 	return newed;
 }
 
-void addfillface(EditVert *v1, EditVert *v2, EditVert *v3)
+static void addfillface(EditVert *v1, EditVert *v2, EditVert *v3, int mat_nr)
 {
 	/* does not make edges */
 	EditFace *evl;
@@ -274,31 +241,10 @@ void addfillface(EditVert *v1, EditVert *v2, EditVert *v3)
 	evl->v2= v2;
 	evl->v3= v3;
 	evl->f= 2;
-	/* G.obedit is Object*, actcol is char */
-/*  	if(G.obedit && G.obedit->actcol) evl->mat_nr= G.obedit->actcol-1; */
-	if (objectref && colourref && *colourref) {
-		evl->mat_nr = *colourref - 1;
-	} else {
-		evl->mat_nr = 0;
-	}
+	evl->mat_nr= mat_nr;
 }
 
-
-int boundinside(PolyFill *pf1, PolyFill *pf2)
-{
-	/* is pf2 INSIDE pf1 ? using bounding box */
-	/* test first if polys exist */
-
-	if(pf1->edges==0 || pf2->edges==0) return 0;
-
-	if(pf2->max[cox]<pf1->max[cox])
-		if(pf2->max[coy]<pf1->max[coy])
-			if(pf2->min[cox]>pf1->min[cox])
-				if(pf2->min[coy]>pf1->min[coy]) return 1;
-	return 0;
-}
-
-int boundisect(PolyFill *pf2, PolyFill *pf1)
+static int boundisect(PolyFill *pf2, PolyFill *pf1)
 {
 	/* has pf2 been touched (intersected) by pf1 ? with bounding box */
 	/* test first if polys exist */
@@ -322,10 +268,7 @@ int boundisect(PolyFill *pf2, PolyFill *pf1)
 }
 
 
-
-
-
-void mergepolysSimp(PolyFill *pf1, PolyFill *pf2)	/* add pf2 to pf1 */
+static void mergepolysSimp(PolyFill *pf1, PolyFill *pf2)	/* add pf2 to pf1 */
 {
 	EditVert *eve;
 	EditEdge *eed;
@@ -348,23 +291,7 @@ void mergepolysSimp(PolyFill *pf1, PolyFill *pf2)	/* add pf2 to pf1 */
 	pf1->f= (pf1->f | pf2->f);
 }
 
-
-
-EditEdge *existfilledge(EditVert *v1, EditVert *v2)
-{
-	EditEdge *eed;
-
-	eed= filledgebase.first;
-	while(eed) {
-		if(eed->v1==v1 && eed->v2==v2) return eed;
-		if(eed->v2==v1 && eed->v1==v2) return eed;
-		eed= eed->next;
-	}
-	return 0;
-}
-
-
-short testedgeside(float *v1, float *v2, float *v3)
+static short testedgeside(float *v1, float *v2, float *v3)
 /* is v3 to the right of v1-v2 ? With exception: v3==v1 || v3==v2 */
 {
 	float inp;
@@ -380,19 +307,7 @@ short testedgeside(float *v1, float *v2, float *v3)
 	return 1;
 }
 
-short testedgeside2(float *v1, float *v2, float *v3)
-/* is v3 to the right of v1-v2 ? no intersection allowed! */
-{
-	float inp;
-
-	inp= (v2[cox]-v1[cox])*(v1[coy]-v3[coy])
-	    +(v1[coy]-v2[coy])*(v1[cox]-v3[cox]);
-
-	if(inp<=0.0) return 0;
-	return 1;
-}
-
-short addedgetoscanvert(ScFillVert *sc, EditEdge *eed)
+static short addedgetoscanvert(ScFillVert *sc, EditEdge *eed)
 {
 	/* find first edge to the right of eed, and insert eed before that */
 	EditEdge *ed;
@@ -436,7 +351,7 @@ short addedgetoscanvert(ScFillVert *sc, EditEdge *eed)
 }
 
 
-ScFillVert *addedgetoscanlist(EditEdge *eed, int len)
+static ScFillVert *addedgetoscanlist(EditEdge *eed, int len)
 {
 	/* inserts edge at correct location in ScFillVert list */
 	/* returns sc when edge already exists */
@@ -461,13 +376,13 @@ ScFillVert *addedgetoscanlist(EditEdge *eed, int len)
 	sc= (ScFillVert *)bsearch(&scsearch,scdata,len,
 	    sizeof(ScFillVert), vergscdata);
 
-	if(sc==0) printf("Error in search edge: %lx\n",eed);
+	if(sc==0) printf("Error in search edge: %p\n",eed);
 	else if(addedgetoscanvert(sc,eed)==0) return sc;
 
 	return 0;
 }
 
-short boundinsideEV(EditEdge *eed, EditVert *eve)
+static short boundinsideEV(EditEdge *eed, EditVert *eve)
 /* is eve inside boundbox eed */
 {
 	float minx,maxx,miny,maxy;
@@ -493,7 +408,7 @@ short boundinsideEV(EditEdge *eed, EditVert *eve)
 }
 
 
-void testvertexnearedge(void)
+static void testvertexnearedge(void)
 {
 	/* only vertices with ->h==1 are being tested for
 		being close to an edge, if true insert */
@@ -560,7 +475,7 @@ void testvertexnearedge(void)
 	}
 }
 
-void splitlist(ListBase *tempve, ListBase *temped, short nr)
+static void splitlist(ListBase *tempve, ListBase *temped, short nr)
 {
 	/* everything is in templist, write only poly nr to fillist */
 	EditVert *eve,*nextve;
@@ -590,7 +505,7 @@ void splitlist(ListBase *tempve, ListBase *temped, short nr)
 }
 
 
-void scanfill(PolyFill *pf)
+static void scanfill(PolyFill *pf, int mat_nr)
 {
 	ScFillVert *sc = NULL, *sc1;
 	EditVert *eve,*v1,*v2,*v3;
@@ -621,18 +536,18 @@ void scanfill(PolyFill *pf)
 			if(eed->v1->co[coy]==eed->v2->co[coy]) {
 				if(eed->v1->f==255 && eed->v2->f!=255) {
 					eed->v2->f= 255;
-					eed->v2->vn= eed->v1->vn;
+					eed->v2->tmp.v= eed->v1->tmp.v;
 				}
 				else if(eed->v2->f==255 && eed->v1->f!=255) {
 					eed->v1->f= 255;
-					eed->v1->vn= eed->v2->vn;
+					eed->v1->tmp.v= eed->v2->tmp.v;
 				}
 				else if(eed->v2->f==255 && eed->v1->f==255) {
-					eed->v1->vn= eed->v2->vn;
+					eed->v1->tmp.v= eed->v2->tmp.v;
 				}
 				else {
 					eed->v2->f= 255;
-					eed->v2->vn= eed->v1;
+					eed->v2->tmp.v = eed->v1->tmp.v;
 				}
 			}
 		}
@@ -665,14 +580,18 @@ void scanfill(PolyFill *pf)
 		nexted= eed->next;
 		eed->f= 0;
 		BLI_remlink(&filledgebase,eed);
-		if(eed->v1->f==255) {
-			v1= eed->v1;
-			while(eed->v1->f==255 && eed->v1->vn!=v1) eed->v1= eed->v1->vn;
-		}
-		if(eed->v2->f==255) {
-			v2= eed->v2;
-			while(eed->v2->f==255 && eed->v2->vn!=v2) eed->v2= eed->v2->vn;
-		}
+/* commented all of this out, this I have no idea for what it is for, probably from ancient past */
+/* it does crash blender, since it uses mixed original and new vertices (ton) */
+//		if(eed->v1->f==255) {
+//			v1= eed->v1;
+//			while((eed->v1->f == 255) && (eed->v1->tmp.v != v1)) 
+//				eed->v1 = eed->v1->tmp.v;
+//		}
+//		if(eed->v2->f==255) {
+//			v2= eed->v2;
+//			while((eed->v2->f == 255) && (eed->v2->tmp.v != v2))
+//				eed->v2 = eed->v2->tmp.v;
+//		}
 		if(eed->v1!=eed->v2) addedgetoscanlist(eed,verts);
 
 		eed= nexted;
@@ -775,7 +694,7 @@ void scanfill(PolyFill *pf)
 				else {
 					/* new triangle */
 					/* printf("add face %x %x %x\n",v1,v2,v3); */
-					addfillface(v1, v2, v3);
+					addfillface(v1, v2, v3, mat_nr);
 					totface++;
 					BLI_remlink((ListBase *)&(sc->first),ed1);
 					BLI_addtail(&filledgebase,ed1);
@@ -844,11 +763,11 @@ void scanfill(PolyFill *pf)
 
 
 
-int BLI_edgefill(int mode)  /* THE MAIN FILL ROUTINE */
+int BLI_edgefill(int mode, int mat_nr)
 {
 	/*
 	  - fill works with its own lists, so create that first (no faces!)
-	  - for vertices, put in ->vn the old pointer
+	  - for vertices, put in ->tmp.v the old pointer
 	  - struct elements xs en ys are not used here: don't hide stuff in it
 	  - edge flag ->f becomes 2 when it's a new edge
 	  - mode: & 1 is check for crossings, then create edges (TO DO )
@@ -903,12 +822,12 @@ int BLI_edgefill(int mode)  /* THE MAIN FILL ROUTINE */
 	eve= fillvertbase.first;
 	while(eve) {
 		if(v2) {
-			if( FloatCompare(v2, eve->co,  0.0003)==0) {
+			if( FloatCompare(v2, eve->co, COMPLIMIT)==0) {
 				len= CalcNormFloat(v1, v2, eve->co, norm);
 				if(len != 0.0) break;
 			}
 		}
-		else if(FloatCompare(v1, eve->co,  0.0003)==0) {
+		else if(FloatCompare(v1, eve->co, COMPLIMIT)==0) {
 			v2= eve->co;
 		}
 		eve= eve->next;
@@ -1020,10 +939,10 @@ int BLI_edgefill(int mode)  /* THE MAIN FILL ROUTINE */
 
 
 	/* CURRENT STATUS:
-	- eve->f  :1= availalble in edges
-	- eve->xs :polynumber
-	- eve->h  :amount of edges connected to vertex
-	- eve->vn :store! original vertex number
+	- eve->f      :1= availalble in edges
+	- eve->xs     :polynumber
+	- eve->h      :amount of edges connected to vertex
+	- eve->tmp.v  :store! original vertex number
 
 	- eed->f  :
 	- eed->f1 :poly number
@@ -1122,7 +1041,7 @@ int BLI_edgefill(int mode)  /* THE MAIN FILL ROUTINE */
 	for(a=0;a<poly;a++) {
 		if(pf->edges>1) {
 			splitlist(&tempve,&temped,pf->nr);
-			scanfill(pf);
+			scanfill(pf, mat_nr);
 		}
 		pf++;
 	}
