@@ -1,5 +1,5 @@
 /**
- * $Id: storage.c,v 1.14 2004/05/11 12:50:04 ton Exp $
+ * $Id: storage.c,v 1.32 2006/03/19 13:28:01 elubie Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,13 +36,7 @@
 #include <stdio.h>
 #include <stdlib.h>	
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
-
-#include <windows.h>
 #include "BLI_winstuff.h"
 #include <sys/types.h>
 #include <io.h>
@@ -56,7 +50,7 @@
 #include <time.h>
 #include <sys/stat.h>
 
-#if defined(__sgi) || defined(__sun__) || defined(__sun) || defined(__sparc) || defined(__sparc__)
+#if !defined(linux) && (defined(__sgi) || defined(__sun__) || defined(__sun) || defined(__sparc) || defined(__sparc__))
 #include <sys/statfs.h>
 #endif
 
@@ -65,7 +59,7 @@
 #include <sys/mount.h>
 #endif
 
-#if defined(linux) || defined(__CYGWIN32__)
+#if defined(linux) || defined(__CYGWIN32__) || defined(__hpux)
 #include <sys/vfs.h>
 #endif
 
@@ -110,6 +104,8 @@ struct statfs {
 #include "BLI_util.h"
 #include "BLI_linklist.h"
 
+#include "BKE_utildefines.h"
+
 /* vars: */
 static int totnum,actnum;
 static struct direntry *files;
@@ -152,7 +148,13 @@ int BLI_compare(struct direntry *entry1, struct direntry *entry2)
 	}
 	if ((entry1->type & S_IFMT) < (entry2->type & S_IFMT)) return (-1);
 	if ((entry1->type & S_IFMT) > (entry2->type & S_IFMT)) return (1);
-	return (strcasecmp(entry1->relname,entry2->relname));
+	
+	/* make sure "." and ".." are always first */
+	if( strcmp(entry1->relname, ".")==0 ) return (-1);
+	if( strcmp(entry2->relname, ".")==0 ) return (1);
+	if( strcmp(entry1->relname, "..")==0 ) return (-1);
+	
+	return (BLI_strcasecmp(entry1->relname,entry2->relname));
 }
 
 
@@ -188,13 +190,13 @@ double BLI_diskfree(char *dir)
 		if (slash) slash[1] = 0;
 	} else strcpy(name,"/");
 
-#if defined (__FreeBSD__) || defined (linux) || defined (__OpenBSD__) 
+#if defined (__FreeBSD__) || defined (linux) || defined (__OpenBSD__) || defined (__APPLE__) 
 	if (statfs(name, &disk)) return(-1);
 #endif
 #ifdef __BeOS
 	return -1;
 #endif
-#if defined (__sgi) || defined (__sun__) || defined (__sun) || defined(__sparc) || defined(__sparc__)
+#if !defined(linux) && (defined (__sgi) || defined (__sun__) || defined (__sun) || defined(__sparc) || defined(__sparc__))
 
 	if (statfs(name, &disk, sizeof(struct statfs), 0)){
 		/* printf("diskfree: Couldn't get information about %s.\n",dir); */
@@ -235,7 +237,7 @@ void BLI_builddir(char *dirname, char *relname)
 		return;
 	}
 
-	if (dir = (DIR *)opendir(".")){
+	if ( (dir = (DIR *)opendir(".")) ){
 		while ((fname = (struct dirent*) readdir(dir)) != NULL) {
 			
 			if(hide_dot && fname->d_name[0]=='.' && fname->d_name[1]!='.' && fname->d_name[1]!=0);
@@ -245,7 +247,7 @@ void BLI_builddir(char *dirname, char *relname)
 				if (dlink){
 					strcpy(buf+rellen,fname->d_name);
 	
-					dlink->name = strdup(buf);
+					dlink->name = BLI_strdup(buf);
 	
 					if (dlink->name[0] == '.') {
 						if (dlink->name[1] == 0) seen_ = 1;
@@ -264,14 +266,14 @@ void BLI_builddir(char *dirname, char *relname)
 			if (seen_ == 0) {	/* Cachefs PATCH */
 				dlink = (struct dirlink *)malloc(sizeof(struct dirlink));
 				strcpy(buf+rellen,"./.");
-				dlink->name = strdup(buf);
+				dlink->name = BLI_strdup(buf);
 				BLI_addhead(dirbase,dlink);
 				newnum++;
 			}
 			if (seen__ == 0) {	/* MAC PATCH */
 				dlink = (struct dirlink *)malloc(sizeof(struct dirlink));
 				strcpy(buf+rellen,"./..");
-				dlink->name = strdup(buf);
+				dlink->name = BLI_strdup(buf);
 				BLI_addhead(dirbase,dlink);
 				newnum++;
 			}
@@ -316,12 +318,16 @@ void BLI_adddirstrings()
 	char size[250];
 	static char * types[8] = {"---", "--x", "-w-", "-wx", "r--", "r-x", "rw-", "rwx"};
 	int num, mode;
-	int num1, num2, num3, num4, st_size;
-
+	int num1, num2, num3, num4;
+#ifdef WIN32
+	__int64 st_size;
+#else
+	long long st_size;
+#endif
+	
 	struct direntry * file;
 	struct tm *tm;
 	time_t zero= 0;
-	struct passwd *pwuser;
 
 	file = &files[0];
 	
@@ -356,12 +362,15 @@ void BLI_adddirstrings()
 #ifdef WIN32
 		strcpy(files[num].owner,"user");
 #else
-		pwuser = getpwuid(files[num].s.st_uid);
-		if ( pwuser ) {
-		   strcpy(files[num].owner, pwuser->pw_name);
-		} else {
-		   sprintf(files[num].owner, "%d", files[num].s.st_uid);
-                }
+		{
+			struct passwd *pwuser;
+			pwuser = getpwuid(files[num].s.st_uid);
+			if ( pwuser ) {
+			strcpy(files[num].owner, pwuser->pw_name);
+			} else {
+				sprintf(files[num].owner, "%d", files[num].s.st_uid);
+            }
+		}
 #endif
 
 		tm= localtime(&files[num].s.st_mtime);
@@ -370,7 +379,7 @@ void BLI_adddirstrings()
 		strftime(files[num].time, 8, "%H:%M", tm);
 		strftime(files[num].date, 16, "%d-%b-%y", tm);
 
-		st_size= (int)files[num].s.st_size;
+		st_size= files[num].s.st_size;
 		
 		num1= st_size % 1000;
 		num2= st_size/1000;
@@ -386,17 +395,17 @@ void BLI_adddirstrings()
 		else if(num1) sprintf(files[num].size, "%15d", num1);
 		else sprintf(files[num].size, "0");
 
-		strftime(datum, 32, "%d-%b-%y %R", tm);
+		strftime(datum, 32, "%d-%b-%y %H:%M", tm);
 
 		if (st_size < 1000) {
-			sprintf(size, "%10d", st_size);
+			sprintf(size, "%10d", (int) st_size);
 		} else if (st_size < 1000 * 1000) {
-			sprintf(size, "%6d %03d", st_size / 1000, st_size % 1000);
+			sprintf(size, "%6d %03d", (int) (st_size / 1000), (int) (st_size % 1000));
 		} else if (st_size < 100 * 1000 * 1000) {
-			sprintf(size, "%2d %03d %03d", st_size / (1000 * 1000), (st_size / 1000) % 1000, st_size % 1000);
+			sprintf(size, "%2d %03d %03d", (int) (st_size / (1000 * 1000)), (int) ((st_size / 1000) % 1000), (int) ( st_size % 1000));
 		} else {
-			sprintf(size, "> %4.1f M", st_size / (1024.0 * 1024.0));
-			sprintf(size, "%10d", st_size);
+			sprintf(size, "> %4.1f M", (double) (st_size / (1024.0 * 1024.0)));
+			sprintf(size, "%10d", (int) st_size);
 		}
 
 		sprintf(buf,"%s %s %10s %s", files[num].date, files[num].time, size,
@@ -405,7 +414,7 @@ void BLI_adddirstrings()
 		sprintf(buf,"%s %s %s %7s %s %s %10s %s", file->mode1, file->mode2, file->mode3, files[num].owner, files[num].date, files[num].time, size,
 			files[num].relname);
 
-		files[num].string=malloc(strlen(buf)+1);
+		files[num].string=MEM_mallocN(strlen(buf)+1, "filestring");
 		if (files[num].string){
 			strcpy(files[num].string,buf);
 		}
@@ -452,8 +461,19 @@ int BLI_filesize(int file)
 int BLI_exist(char *name)
 {
 	struct stat st;
-
-	if (stat(name,&st)) return(0);
+#ifdef WIN32
+	/*  in Windows stat doesn't recognize dir ending on a slash 
+		To not break code where the ending slash is expected we
+		don't mess with the argument name directly here - elubie */
+	char tmp[FILE_MAXDIR+FILE_MAXFILE];
+	int len;
+	BLI_strncpy(tmp, name, FILE_MAXDIR+FILE_MAXFILE);
+	len = strlen(tmp);
+	if (len > 3 && ( tmp[len-1]=='\\' || tmp[len-1]=='/') ) tmp[len-1] = '\0';
+	if (stat(tmp,&st)) return(0);
+#else
+	if (stat(name,&st)) return(0);	
+#endif
 	return(st.st_mode);
 }
 
