@@ -3,7 +3,7 @@
  * various string, file, list operations.
  *
  *
- * $Id: util.c,v 1.15 2004/05/10 20:35:46 zuster Exp $
+ * $Id: util.c,v 1.35 2006/02/11 15:02:23 elubie Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -74,6 +74,16 @@
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
+
+// copies from BKE_utildefines
+#ifndef FILE_MAXDIR
+#define FILE_MAXDIR  160
+#endif
+
+#ifndef FILE_MAXFILE
+#define FILE_MAXFILE 80
+#endif
+
 /* local */
 
 static int add_win32_extension(char *name);
@@ -107,23 +117,29 @@ int BLI_stringdec(char *string, char *kop, char *staart, unsigned short *numlen)
 	len2 = len =  strlen( string);
 	
 	if (len > 6) {
-		if (strncasecmp(string + len - 6, ".blend", 6) == 0) len -= 6;
-		else if (strncasecmp(string + len - 6, ".trace", 6) == 0) len -= 6;
+		if (BLI_strncasecmp(string + len - 6, ".blend", 6) == 0) len -= 6;
+		else if (BLI_strncasecmp(string + len - 6, ".trace", 6) == 0) len -= 6;
 	}
 	
+	if (len > 9) {
+		if (BLI_strncasecmp(string + len - 9, ".blend.gz", 9) == 0) len -= 9;
+	}
+		
+	
 	if (len == len2) {
 		if (len > 4) {
 			/* handle .jf0 en .jf1 for jstreams */
-			if (strncasecmp(string + len - 4, ".jf", 3) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".tga", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".jpg", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".png", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".txt", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".cyc", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".enh", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".rgb", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".psx", 4) == 0) len -= 4;
-			else if (strncasecmp(string + len - 4, ".ble", 4) == 0) len -= 4;
+			if (BLI_strncasecmp(string + len - 4, ".jf", 3) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".tga", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".jpg", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".png", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".txt", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".cyc", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".enh", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".rgb", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".psx", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".ble", 4) == 0) len -= 4;
+			else if (BLI_strncasecmp(string + len - 4, ".exr", 4) == 0) len -= 4;
 		}
 	}
 	
@@ -413,56 +429,218 @@ int BLI_streq(char *a, char *b) {
 	return (strcmp(a, b)==0);
 }
 int BLI_strcaseeq(char *a, char *b) {
-	return (strcasecmp(a, b)==0);
+	return (BLI_strcasecmp(a, b)==0);
 }
 
-void BLI_makestringcode(char *fromfile, char *str)
+/* ******************** string encoding ***************** */
+
+/* This is quite an ugly function... its purpose is to
+ * take the dir name, make it absolute, and clean it up, replacing
+ * excess file entry stuff (like /tmp/../tmp/../)
+ * note that dir isn't protected for max string names... 
+ */
+
+void BLI_cleanup_dir(const char *relabase, char *dir)
 {
-	char *slash, len, temp[512];
+	short a;
+	char *start, *eind;
+	
+	BLI_convertstringcode(dir, relabase, 0);
+	
+#ifdef WIN32
+	if(dir[0]=='.') {	/* happens for example in FILE_MAIN */
+	   dir[0]= '\\';
+	   dir[1]= 0;
+	   return;
+	}	
 
-	strcpy(temp, fromfile);
-
-	/* Find the last slash */
-	slash = (strrchr(temp, '/')>strrchr(temp, '\\'))
-	    ? strrchr(temp, '/') : strrchr(temp, '\\');
-	if(slash) {
-		*(slash+1)= 0;
-		len= strlen(temp);
-		if(len) {
-			if(strncmp(str, temp, len)==0) {
-				temp[0]= '/';
-				temp[1]= '/';
-				strcpy(temp+2, str+len);
-				strcpy(str, temp);
-			}
+	while ( (start = strstr(dir, "\\..\\")) ) {
+		eind = start + strlen("\\..\\") - 1;
+		a = start-dir-1;
+		while (a>0) {
+			if (dir[a] == '\\') break;
+			a--;
 		}
+		strcpy(dir+a,eind);
+	}
+
+	while ( (start = strstr(dir,"\\.\\")) ){
+		eind = start + strlen("\\.\\") - 1;
+		strcpy(start,eind);
+	}
+
+	while ( (start = strstr(dir,"\\\\" )) ){
+		eind = start + strlen("\\\\") - 1;
+		strcpy(start,eind);
+	}
+
+	if((a = strlen(dir))){				/* remove the '\\' at the end */
+		while(a>0 && dir[a-1] == '\\'){
+			a--;
+			dir[a] = 0;
+		}
+	}
+
+	strcat(dir, "\\");
+#else	
+	if(dir[0]=='.') {	/* happens, for example in FILE_MAIN */
+	   dir[0]= '/';
+	   dir[1]= 0;
+	   return;
+	}	
+
+	while ( (start = strstr(dir, "/../")) ) {
+		eind = start + strlen("/../") - 1;
+		a = start-dir-1;
+		while (a>0) {
+			if (dir[a] == '/') break;
+			a--;
+		}
+		strcpy(dir+a,eind);
+	}
+
+	while ( (start = strstr(dir,"/./")) ){
+		eind = start + strlen("/./") - 1;
+		strcpy(start,eind);
+	}
+
+	while ( (start = strstr(dir,"//" )) ){
+		eind = start + strlen("//") - 1;
+		strcpy(start,eind);
+	}
+
+	if( (a = strlen(dir)) ){				/* remove all '/' at the end */
+		while(dir[a-1] == '/'){
+			a--;
+			dir[a] = 0;
+			if (a<=0) break;
+		}
+	}
+
+	strcat(dir, "/");
+#endif
+}
+
+
+void BLI_makestringcode(const char *relfile, char *file)
+{
+	char * p;
+	char * q;
+	char * lslash;
+	int len=0;
+	char temp[FILE_MAXDIR+FILE_MAXFILE];
+	char res[FILE_MAXDIR+FILE_MAXFILE];
+	
+	/* if file is already relative, bail out */
+	if(file[0]=='/' && file[1]=='/') return;
+	
+	strcpy(temp, relfile);
+	
+#ifdef WIN32
+	if (strlen(file) > 2) {
+		if ( temp[1] == ':' && file[1] == ':' && temp[0] != file[0] )
+			return;
+	}
+#endif
+
+	BLI_char_switch(temp, '\\', '/');
+	BLI_char_switch(file, '\\', '/');
+
+	/* the last slash in the file indicates where the path part ends */
+	lslash = BLI_last_slash(temp);
+
+	if (lslash) 
+	{	
+		/* find the prefix of the filename that is equal for both filenames.
+		   This is replaced by the two slashes at the beginning */
+		p = temp;
+		q = file;
+		while (*p == *q) {
+			++p; ++q;
+		}
+		/* we might have passed the slash when the beginning of a dir matches 
+		   so we rewind. Only check on the actual filename
+		*/
+		if (*q != '/') {
+			while ( (q >= file) && (*q != '/') ) { --q; --p; }
+		} 
+		else if (*p != '/') {
+			while ( (p >= temp) && (*p != '/') ) { --p; --q; }
+		}
+		
+		strcpy(res,	"//");
+
+		/* p now points to the slash that is at the beginning of the part
+		   where the path is different from the relative path. 
+		   We count the number of directories we need to go up in the
+		   hierarchy to arrive at the common 'prefix' of the path
+		*/			
+		while (p && p < lslash)	{
+			if (*p == '/') 
+				strcat(res,	"../");
+			++p;
+		}
+
+		strcat(res, q+1); /* don't copy the slash at the beginning */
+		
+#ifdef	WIN32
+		BLI_char_switch(res+2, '/', '\\');
+#endif
+		strcpy(file, res);
 	}
 }
 
-int BLI_convertstringcode(char *path, char *basepath, int framenum)
+int BLI_convertstringcode(char *path, const char *basepath, int framenum)
 {
-	int len, wasrelative= (strncmp(path, "//", 2)==0);
+	int len, wasrelative;
+	char tmp[FILE_MAXDIR+FILE_MAXFILE];
+	char base[FILE_MAXDIR];
+	
+	strcpy(tmp, path);
+	strcpy(base, basepath);
+	
+	/* push slashes into unix mode - strings entering this part are
+	   potentially messed up: having both back- and forward slashes.
+	   Here we push into one conform direction, and at the end we
+	   push them into the system specific dir. This ensures uniformity
+	   of paths and solving some problems (and prevent potential future
+	   ones) -jesterKing. */
+	BLI_char_switch(tmp, '\\', '/');
+	BLI_char_switch(base, '\\', '/');
+	
+	wasrelative= (strncmp(tmp, "//", 2)==0);
 
-	if (path[0] == '/' && path[1] == '/') {
-		char *filepart= BLI_strdup(path+2); /* skip code */
-		char *lslash= BLI_last_slash(basepath);
+	if (tmp[0] == '/' && tmp[1] == '/') {
+		char *filepart= BLI_strdup(tmp+2); /* skip code */
+		char *lslash= BLI_last_slash(base);
 
 		if (lslash) {
-			int baselen= (int) (lslash-basepath) + 1;
+			int baselen= (int) (lslash-base) + 1;
 
-			memcpy(path, basepath, baselen);
-			strcpy(path+baselen, filepart);
+			memcpy(tmp, base, baselen);
+			strcpy(tmp+baselen, filepart);
 		} else {
-			strcpy(path, filepart);
+			strcpy(tmp, filepart);
 		}
 		
 		MEM_freeN(filepart);
 	}
 
-	len= strlen(path);
-	if(len && path[len-1]=='#') {
-		sprintf(path+len-1, "%04d", framenum);
+	len= strlen(tmp);
+	if(len && tmp[len-1]=='#') {
+		sprintf(tmp+len-1, "%04d", framenum);
 	}
+	
+	strcpy(path, tmp);
+#ifdef WIN32
+	/* skip first two chars, which in case of
+	   absolute path will be drive:/blabla and
+	   in case of relpath //blabla/. So relpath
+	   // will be retained, rest will be nice and
+	   shiny win32 backward slashes :) -jesterKing
+	*/
+	BLI_char_switch(path+2, '/', '\\');
+#endif
 
 	return wasrelative;
 }
@@ -490,35 +668,39 @@ char *BLI_gethome(void) {
 	#else /* Windows */
 		char * ret;
 		static char dir[512];
-		char tmpdir[512];
+
+		/* Check for %HOME% env var */
 
 		ret = getenv("HOME");
 		if(ret) {
-			sprintf(dir, "%s\.blender", ret);
+			sprintf(dir, "%s\\.blender", ret);
 			if (BLI_exists(dir)) return dir;
 		}
-				
-		/* add user profile support for WIN 2K / NT */
-		ret = getenv("USERPROFILE");
-		if (ret) {
-			if (BLI_exists(ret)) { /* from fop, also below... */
-				sprintf(dir, "%s/Application Data/Blender Foundation/Blender", ret);
-				BLI_recurdir_fileops(dir);
-				if (BLI_exists(dir)) {
-					strcat(dir,"/.blender");
-					if(BLI_exists(dir)) return(dir);
-				}
-			}
-		}
+
+		/* else, check install dir (path containing blender.exe) */
 
 		BLI_getInstallationDir(dir);
 
 		if (BLI_exists(dir))
 		{
-			strcat(dir,"/.blender");
+			strcat(dir,"\\.blender");
 			if (BLI_exists(dir)) return(dir);
 		}
 
+				
+		/* add user profile support for WIN 2K / NT */
+		ret = getenv("USERPROFILE");
+		if (ret) {
+			if (BLI_exists(ret)) { /* from fop, also below... */
+				sprintf(dir, "%s\\Application Data\\Blender Foundation\\Blender", ret);
+				BLI_recurdir_fileops(dir);
+				if (BLI_exists(dir)) {
+					strcat(dir,"\\.blender");
+					if(BLI_exists(dir)) return(dir);
+				}
+			}
+		}
+
 		/* 
 		   Saving in the Windows dir is less than desirable. 
 		   Use as a last resort ONLY! (aphex)
@@ -538,22 +720,34 @@ char *BLI_gethome(void) {
 	#endif
 }
 
-static void char_switch(char *string, char from, char to) 
+void BLI_clean(char *path)
 {
+	if(path==0) return;
+#ifdef WIN32
+	if(path && strlen(path)>2) {
+		BLI_char_switch(path+2, '/', '\\');
+	}
+#else
+	BLI_char_switch(path, '\\', '/');
+#endif
+}
+
+void BLI_char_switch(char *string, char from, char to) 
+{
+	if(string==0) return;
 	while (*string != 0) {
 		if (*string == from) *string = to;
 		string++;
 	}
 }
 
-void BLI_make_exist(char *dir)
-{
+void BLI_make_exist(char *dir) {
 	int a;
 
 	#ifdef WIN32
-		char_switch(dir, '/', '\\');
+		BLI_char_switch(dir, '/', '\\');
 	#else
-		char_switch(dir, '\\', '/');
+		BLI_char_switch(dir, '\\', '/');
 	#endif	
 	
 	a = strlen(dir);
@@ -587,13 +781,37 @@ void BLI_make_exist(char *dir)
 #endif
 }
 
-void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file)
+void BLI_make_existing_file(char *name)
 {
+	char di[FILE_MAXDIR], fi[FILE_MAXFILE];
+	
+	strcpy(di, name);
+	BLI_splitdirstring(di, fi);
+	
+	/* test exist */
+	if (BLI_exists(di) == 0) {
+		BLI_recurdir_fileops(di);
+	}
+}
+
+
+void BLI_make_file_string(const char *relabase, char *string,  const char *dir, const char *file)
+{
+	int sl;
 
 	if (!string || !dir || !file) return; /* We don't want any NULLs */
 	
 	string[0]= 0; /* ton */
 
+	/* we first push all slashes into unix mode, just to make sure we don't get
+	   any mess with slashes later on. -jesterKing */
+	/* constant strings can be passed for those parameters - don't change them - elubie */
+	/*
+	BLI_char_switch(relabase, '\\', '/');
+	BLI_char_switch(dir, '\\', '/');
+	BLI_char_switch(file, '\\', '/');
+	*/
+
 	/* Resolve relative references */	
 	if (relabase && dir[0] == '/' && dir[1] == '/') {
 		char *lslash;
@@ -610,9 +828,15 @@ void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file)
 	
 	strcat(string, dir);
 
-	/* Make sure string ends in one (and only one) slash */
-	if (string[strlen(string)-1] != '/' && string[strlen(string)-1] != '\\')
-		strcat(string, "/");
+	/* Make sure string ends in one (and only one) slash */	
+	/* first trim all slashes from the end of the string */
+	sl = strlen(string);
+	while (sl>0 && ( string[sl-1] == '/' || string[sl-1] == '\\') ) {
+		string[sl-1] = '\0';
+		sl--;
+	}
+	/* since we've now removed all slashes, put back one slash at the end. */
+	strcat(string, "/");
 	
 	while (*file && (*file == '/' || *file == '\\')) /* Trim slashes from the front of file */
 		file++;
@@ -620,11 +844,7 @@ void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file)
 	strcat (string, file);
 	
 	/* Push all slashes to the system preferred direction */
-	#ifdef WIN32
-		char_switch(string, '/', '\\');
-	#else
-		char_switch(string, '\\', '/');
-	#endif	
+	BLI_clean(string);
 }
 
 int BLI_testextensie(char *str, char *ext)
@@ -637,7 +857,7 @@ int BLI_testextensie(char *str, char *ext)
 
 	if(a==0 || b==0 || b>=a) {
 		retval = 0;
-	} else if (strcasecmp(ext, str + a - b)) {
+	} else if (BLI_strcasecmp(ext, str + a - b)) {
 		retval = 0;	
 	} else {
 		retval = 1;
@@ -646,6 +866,8 @@ int BLI_testextensie(char *str, char *ext)
 	return (retval);
 }
 
+
+
 void BLI_split_dirfile(char *string, char *dir, char *file)
 {
 	int a;
@@ -654,16 +876,19 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
 	file[0]= 0;
 
 #ifdef WIN32
+	BLI_char_switch(string, '/', '\\'); /* make sure we have a valid path format */
+
 	if (strlen(string)) {
+		int len;
 		if (string[0] == '/' || string[0] == '\\') { 
-			strcpy(dir, string);
+			BLI_strncpy(dir, string, FILE_MAXDIR);
 		} else if (string[1] == ':' && string[2] == '\\') {
-            strcpy(dir, string);
+            BLI_strncpy(dir, string, FILE_MAXDIR);
         } else {
             BLI_getwdN(dir);
             strcat(dir,"/");
             strcat(dir,string);
-            strcpy(string,dir);
+            BLI_strncpy(string,dir,FILE_MAXDIR+FILE_MAXFILE);
         }
 
         BLI_make_exist(dir);
@@ -675,9 +900,20 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
         if(a>=4 && dir[a-1]=='\\') dir[a-1] = 0;
 
         if (S_ISDIR(BLI_exist(dir))) {
-            strcpy(file,string + strlen(dir));
 
-            if (strrchr(file,'\\')) strcpy(file,strrchr(file,'\\')+1);
+	    /* copy from end of string into file, to ensure filename itself isn't truncated 
+	       if string is too long. (aphex) */
+
+	    len = FILE_MAXFILE - strlen(string);
+
+	    if (len < 0)
+		BLI_strncpy(file,string + abs(len),FILE_MAXFILE);
+	    else
+		BLI_strncpy(file,string,FILE_MAXFILE);
+	    
+            if (strrchr(string,'\\')){
+		BLI_strncpy(file,strrchr(string,'\\')+1,FILE_MAXFILE);
+	    }
 
             if (a = strlen(dir)) {
                 if (dir[a-1] != '\\') strcat(dir,"\\");
@@ -687,7 +923,7 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
 			a = strlen(dir) - 1;
 			while(a>0 && dir[a] != '\\') a--;
 			dir[a + 1] = 0;
-			strcpy(file, string + strlen(dir));
+			BLI_strncpy(file, string + strlen(dir),FILE_MAXFILE);
 		}
 		
 	}
@@ -734,15 +970,6 @@ void BLI_split_dirfile(char *string, char *dir, char *file)
 	}
 #endif
 }
-
-// copies from BKE_utildefines
-#ifndef FILE_MAXDIR
-#define FILE_MAXDIR  160
-#endif
-
-#ifndef FILE_MAXFILE
-#define FILE_MAXFILE 80
-#endif
  
 static int add_win32_extension(char *name)
 {
@@ -866,7 +1093,7 @@ void BLI_where_am_i(char *fullname, char *name)
 char* BLI_getbundle(void) {
 	CFURLRef bundleURL;
 	CFStringRef pathStr;
-	char path[MAXPATHLEN];
+	static char path[MAXPATHLEN];
 	CFBundleRef mainBundle = CFBundleGetMainBundle();
 
 	bundleURL = CFBundleCopyBundleURL(mainBundle);
@@ -876,3 +1103,59 @@ char* BLI_getbundle(void) {
 }
 #endif
 
+int BLI_strcasecmp(const char *s1, const char *s2) {
+	int i;
+
+	for (i=0; ; i++) {
+		char c1 = tolower(s1[i]);
+		char c2 = tolower(s2[i]);
+
+		if (c1<c2) {
+			return -1;
+		} else if (c1>c2) {
+			return 1;
+		} else if (c1==0) {
+			break;
+		}
+	}
+
+	return 0;
+}
+
+int BLI_strncasecmp(const char *s1, const char *s2, int n) {
+	int i;
+
+	for (i=0; i<n; i++) {
+		char c1 = tolower(s1[i]);
+		char c2 = tolower(s2[i]);
+
+		if (c1<c2) {
+			return -1;
+		} else if (c1>c2) {
+			return 1;
+		} else if (c1==0) {
+			break;
+		}
+	}
+
+	return 0;
+}
+
+void BLI_timestr(double time, char *str)
+{
+	/* format 00:00:00.00 (hr:min:sec) string has to be 12 long */
+	int  hr= ((int) time) / (60*60);
+	int min= ( ((int) time) / 60 ) % 60;
+	int sec= ((int) (time)) % 60;
+	int hun= ((int) (time * 100.0)) % 100;
+	
+	if (hr) {
+		sprintf(str, "%.2d:%.2d:%.2d.%.2d",hr,min,sec,hun);
+	} else {
+		sprintf(str, "%.2d:%.2d.%.2d",min,sec,hun);
+	}
+	
+	str[11]=0;
+}
+
+
