@@ -2,7 +2,7 @@
  *
  * Functions for struct-dna, the genetic file dot c!
  *
- * $Id: genfile.c,v 1.6 2003/05/24 20:04:27 hos Exp $
+ * $Id: genfile.c,v 1.9 2006/03/30 22:44:33 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -61,9 +61,14 @@
 
 #include "genfile.h"
 
+/* gcc 4.1 on mingw was complaining that __int64 was alredy defined
+actually is saw the line below as typedef long long long long... 
+Anyhow, since its alredy defined, its safe to do an ifndef here- Cambpell*/
 #ifdef FREE_WINDOWS
+#ifndef __int64
 typedef long long __int64;
 #endif
+#endif
 
 /*
  * - please note: no builtin security to detect input of double structs
@@ -483,47 +488,6 @@ struct SDNA *dna_sdna_from_data(void *data, int datalen, int do_endian_swap)
 	
 	return sdna;
 }
-		/* XXX, this routine was added because at one
-		 * point I thought using the dna more would be
-		 * nice, but really thats a flawed idea, you
-		 * already know about your memory structures when
-		 * you are compiling no need to redirect that
-		 * through the DNA, the python stuff should be
-		 * changed to not use this routine (a bit 
-		 * o' work). - zr
-		 */
-int BLO_findstruct_offset(char *structname, char *member)
-{
-	extern char DNAstr[];	/* DNA.c */
-	extern int DNAlen;
-	
-	struct SDNA *sdna;
-	int a, offset;
-	short *sp;
-
-	sdna= dna_sdna_from_data(DNAstr, DNAlen, 0);
-	
-	sp= findstruct_name(sdna, structname);
-	
-	if(sp) {
-		a= sp[1];	/* nr of elems */
-		sp+= 2;
-		offset= 0;
-		
-		while(a--) {
-			if(strcmp(sdna->names[sp[1]], member)==0) {
-				dna_freestructDNA(sdna);
-				return offset;
-			}
-
-			offset+= elementsize(sdna, sp[0], sp[1]);			
-			sp+= 2;
-		}
-	}
-	
-	dna_freestructDNA(sdna);
-	return -1;
-}
 
 /* ******************** END READ DNA ********************** */
 
@@ -885,7 +849,7 @@ static void reconstruct_elem(struct SDNA *newsdna, struct SDNA *oldsdna, char *t
 		}
 		else if(array) {		/* name is an array */
 
-			if( strncmp(name, oname, array)==0 ) {			/* basis equal */
+			if(oname[array]=='[' && strncmp(name, oname, array)==0 ) {			/* basis equal */
 				
 				cursize= arraysize(name, strlen(name));
 				oldsize= arraysize(oname, strlen(oname));
