@@ -73,7 +73,7 @@
  *     Additionally, it needs the types from the jpeg lib.
  */
 /*
- * $Id: IMB_imbuf.h,v 1.15 2005/03/11 20:16:13 sirdude Exp $ 
+ * $Id: IMB_imbuf.h,v 1.24 2006/06/28 17:46:46 elubie Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -174,6 +174,28 @@ struct ImBuf *IMB_allocImBuf(short x, short y,
 						 unsigned char d, unsigned int flags,
 						 unsigned char bitmap);
 
+/**
+ *
+ * Increase reference count to imbuf
+ * (to delete an imbuf you have to call freeImBuf as many times as it
+ * is referenced)
+ *
+ * @attention Defined in allocimbuf.c
+ */
+
+void IMB_refImBuf(struct ImBuf * ibuf);
+
+/**
+ *
+ * @attention Defined in allocimbuf.c
+ */
+void IMB_cache_limiter_insert(struct ImBuf * i);
+void IMB_cache_limiter_unmanage(struct ImBuf * i);
+void IMB_cache_limiter_touch(struct ImBuf * i);
+void IMB_cache_limiter_ref(struct ImBuf * i);
+void IMB_cache_limiter_unref(struct ImBuf * i);
+int IMB_cache_limiter_get_refcount(struct ImBuf * i);
+
 /**
  *
  * @attention Defined in allocimbuf.c
@@ -185,6 +207,7 @@ struct ImBuf *IMB_dupImBuf(struct ImBuf *ibuf1);
  * @attention Defined in allocimbuf.c
  */
 short addzbufImBuf(struct ImBuf * ibuf);
+short addzbuffloatImBuf(struct ImBuf * ibuf);
 
 /**
  *
@@ -196,31 +219,8 @@ void IMB_freecmapImBuf(struct ImBuf * ibuf);
  *
  * @attention Defined in rectop.c
  */
-void IMB_rectop(struct ImBuf *dbuf,
-			struct ImBuf *sbuf,
-			int destx,
-			int desty,
-			int srcx,
-			int srcy,
-			int width,
-			int height,
-			void (*operation)(unsigned int *, unsigned int*, int, int),
-			int value);
-
-/**
- *
- * @attention Defined in rectop.c
- */
-void IMB_rectoptot(struct ImBuf *dbuf,
-			   struct ImBuf *sbuf,
-			   void (*operation)(unsigned int *, unsigned int*, int, int),
-			   int value);
-
-/**
- *
- * @attention Defined in rectop.c
- */
-void IMB_rectcpy(unsigned int *drect, unsigned int *srect, int x, int dummy);
+void IMB_rectcpy(struct ImBuf *drect, struct ImBuf *srect, int destx,
+	int desty, int srcx, int srcy, int width, int height);
 
 /**
  * Return the length (in frames) of the given @a anim.
@@ -240,6 +240,8 @@ void IMB_close_anim(struct anim * anim);
  */
 
 int ismovie(char *name);
+void IMB_anim_set_preseek(struct anim * anim, int preseek);
+int IMB_anim_get_preseek(struct anim * anim);
 
 /**
  *
@@ -284,6 +286,7 @@ void IMB_antialias(struct ImBuf * ibuf);
  * @attention Defined in filter.c
  */
 void IMB_filter(struct ImBuf *ibuf);
+void IMB_filterN(struct ImBuf *out, struct ImBuf *in);
 
 /**
  *
@@ -353,6 +356,8 @@ int imb_get_anim_type(char * name);
 void IMB_de_interlace(struct ImBuf *ibuf);
 void IMB_interlace(struct ImBuf *ibuf);
 void IMB_gamwarp(struct ImBuf *ibuf, double gamma);
+void IMB_rect_from_float(struct ImBuf *ibuf);
+void IMB_float_from_rect(struct ImBuf *ibuf);
 
 /**
  * Change the ordering of the colour bytes pointed to by rect from
@@ -360,7 +365,7 @@ void IMB_gamwarp(struct ImBuf *ibuf, double gamma);
  *
  * @attention Defined in imageprocess.c
  */
-void IMB_convert_rgba_to_abgr(int size, unsigned int *rect);
+void IMB_convert_rgba_to_abgr(struct ImBuf *ibuf);
 
 /**
  * Change the ordering of the colour bytes pointed to by rect from
@@ -490,13 +495,20 @@ void IMB_cspace(struct ImBuf *ibuf, float mat[][4]);
  * @attention Defined in allocimbuf.c
  */
 void IMB_freezbufImBuf(struct ImBuf * ibuf);
+void IMB_freezbuffloatImBuf(struct ImBuf * ibuf);
 
 /**
  *
  * @attention Defined in rectop.c
  */
-void IMB_rectfill(unsigned int *drect, unsigned int *srect, int x, int value);
+void IMB_rectfill(struct ImBuf *drect, float col[4]);
 
+/* exported for image tools in blender, to quickly allocate 32 bits rect */
+short imb_addrectImBuf(struct ImBuf * ibuf);
+void imb_freerectImBuf(struct ImBuf * ibuf);
+
+short imb_addrectfloatImBuf(struct ImBuf * ibuf);
+void imb_freerectfloatImBuf(struct ImBuf * ibuf);
 
 #ifdef WITH_QUICKTIME
 /**
@@ -513,4 +525,8 @@ void quicktime_exit(void);
 
 #endif //WITH_QUICKTIME
 
+/* intern/dynlibtiff.c */
+void libtiff_init(void);
+void libtiff_exit(void);
+
 #endif
