@@ -29,7 +29,7 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  * iris.c
  *
- * $Id: iris.c,v 1.12 2005/04/27 11:52:50 sirdude Exp $
+ * $Id: iris.c,v 1.13 2006/01/28 18:33:09 hos Exp $
  */
 
 #include <string.h>
@@ -398,7 +398,7 @@ struct ImBuf *imb_loadiris(unsigned char *mem, int flags)
 	
 	if (ibuf) {
 		if (ibuf->rect) 
-			IMB_convert_rgba_to_abgr(ibuf->x*ibuf->y, ibuf->rect);
+			IMB_convert_rgba_to_abgr(ibuf);
 	}
 
 	return(ibuf);
@@ -639,13 +639,13 @@ short imb_saveiris(struct ImBuf * ibuf, char *name, int flags)
 	zsize = (ibuf->depth + 7) >> 3;
 	if (flags & IB_zbuf &&  ibuf->zbuf != 0) zsize = 8;
 	
-	IMB_convert_rgba_to_abgr(ibuf->x*ibuf->y, ibuf->rect);
+	IMB_convert_rgba_to_abgr(ibuf);
 	test_endian_zbuf(ibuf);
 
 	ret = output_iris(ibuf->rect, ibuf->x, ibuf->y, zsize, name, ibuf->zbuf);
 
 	/* restore! Quite clumsy, 2 times a switch... maybe better a malloc ? */
-	IMB_convert_rgba_to_abgr(ibuf->x*ibuf->y, ibuf->rect);
+	IMB_convert_rgba_to_abgr(ibuf);
 	test_endian_zbuf(ibuf);
 
 	return(ret);
