@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_editcurve.h,v 1.9 2005/06/13 19:15:01 stiv Exp $
+ * $Id: BDR_editcurve.h,v 1.13 2006/01/03 02:35:44 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -62,10 +62,6 @@ void hideNurb(int swap);
 void revealNurb(void);
 void selectswapNurb(void);
 void subdivideNurb(void);
-short findnearestNurbvert(short sel, struct Nurb **nurb, 
-						  struct BezTriple **bezt, struct BPoint **bp);
-void findselectedNurbvert(struct Nurb **nu, struct BezTriple **bezt, 
-						  struct BPoint **bp);
 
 int convertspline(short type, struct Nurb *nu);
 void setsplinetype(short type);
@@ -84,7 +80,8 @@ void selectconnected_nurb(void);
 void selectrow_nurb(void);
 void adduplicate_nurb(void);
 void delNurb(void);
-void join_curve(int type);
+void nurb_set_smooth(short event);
+int join_curve(int type);
 struct Nurb *addNurbprim(int type, int stype, int newname);
 void default_curve_ipo(struct Curve *cu);
 void add_primitiveCurve(int stype);
@@ -92,7 +89,6 @@ void add_primitiveNurb(int type);
 void clear_tilt(void);
 void clever_numbuts_curve(void);         
 int bezt_compare (const void *e1, const void *e2);
-void curve_changes_other_objects(struct Object *ob);
 
 extern void undo_push_curve(char *name);
 
