@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_drawimage.h,v 1.11 2005/04/24 11:57:23 blendix Exp $
+ * $Id: BIF_drawimage.h,v 1.14 2006/06/12 14:39:07 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,6 +35,7 @@
 
 struct ScrArea;
 struct SpaceImage;
+struct Render;
 
 void do_imagebuts(unsigned short event);
 void calc_image_view(struct SpaceImage *sima, char mode);
@@ -46,8 +47,12 @@ void image_viewmove(int mode);
 void image_viewzoom(unsigned short event, int invert);
 void image_viewcentre(void);
 void uvco_to_areaco(float *vec, short *mval);
-void uvco_to_areaco_noclip(float *vec, short *mval);
+void uvco_to_areaco_noclip(float *vec, int *mval);
 void what_image(struct SpaceImage *sima);
+void image_preview_event(int event);
+
+void imagewindow_render_callbacks(struct Render *re);
+void imagewindow_toggle_render(void);
 
 #endif
 
