@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editmesh.h,v 1.37 2005/02/16 15:02:58 guitargeek Exp $
+ * $Id: BIF_editmesh.h,v 1.67 2006/06/08 05:41:57 briggs Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,12 +42,14 @@ struct EditVert;
 struct Mesh;
 struct bDeformGroup;
 struct View3D;
+struct EditSelection;
 
 // edge and face flag both
 #define EM_FGON		2
 // face flag
 #define EM_FGON_DRAW	1
 
+extern unsigned int em_vertoffs, em_solidoffs, em_wireoffs;
 
 /* ******************* editmesh.c */
 extern void make_editMesh(void);
@@ -63,9 +65,12 @@ extern void separate_mesh(void);
 extern void separate_mesh_loose(void);
 
 /* ******************* editmesh_add.c */
+extern void make_prim(int type, float imat[3][3], short tot, short seg,
+		short subdiv, float dia, float d, short ext, short fill,
+        float cent[3]);
 extern void add_primitiveMesh(int type);
 extern void adduplicate_mesh(void);
-extern void addvert_mesh(void);
+extern void add_click_mesh(void);
 extern void addedgeface_mesh(void);
 
 /* ******************* editmesh_lib.c */
@@ -75,11 +80,19 @@ extern void EM_clear_flag_all(int flag);
 
 extern void EM_select_face(struct EditFace *efa, int sel);
 extern void EM_select_edge(struct EditEdge *eed, int sel);
+extern float EM_face_area(struct EditFace *efa);
+extern float EM_face_perimeter(struct EditFace *efa);
+extern void EM_editselection_center(float *center, struct EditSelection *ese);
+extern void EM_editselection_normal(float *normal, struct EditSelection *ese);
+extern void EM_editselection_plane(float *plane, struct EditSelection *ese);
 
 extern void EM_deselect_flush(void);	// vertices to edges/faces (exception!)
 extern void EM_select_flush(void);	// vertices to edges/faces (exception!)
 extern void EM_selectmode_set(void); // when mode changes
 extern void EM_selectmode_flush(void); // when selection changes
+extern void EM_convertsel(short oldmode, short selectmode);
+extern void EM_remove_selection(void *data, int type);
+extern void EM_store_selection(void *data, int type);
 
 extern int EM_nfaces_selected(void);
 extern int EM_nvertices_selected(void);
@@ -87,20 +100,27 @@ extern int EM_nvertices_selected(void);
 extern int faceselectedAND(struct EditFace *efa, int flag);
 extern void recalc_editnormals(void);
 extern void flip_editnormals(void);
-extern void vertexnormals(int testflip);
 
 /* ******************* editmesh_mods.c */
 
+extern void EM_init_index_arrays(int forVert, int forEdge, int forFace);
+extern void EM_free_index_arrays(void);
+
+extern struct EditVert *EM_get_vert_for_index(int index);
+extern struct EditEdge *EM_get_edge_for_index(int index);
+extern struct EditFace *EM_get_face_for_index(int index);
+
 extern void EM_select_face_fgon(struct EditFace *efa, int sel);
 
 extern int EM_init_backbuf_border(short xmin, short ymin, short xmax, short ymax);
 extern int EM_mask_init_backbuf_border(short mcords[][2], short tot, short xmin, short ymin, short xmax, short ymax);
 extern int EM_init_backbuf_circle(short xs, short ys, short rads);
-extern int EM_check_backbuf_border(int index);
-extern void EM_free_backbuf_border(void);
+extern int EM_check_backbuf(unsigned int index);
+extern void EM_free_backbuf(void);
 
 extern void EM_selectmode_menu(void);
 
+
 extern void vertexnoise(void);
 extern void vertexsmooth(void);
 extern void righthandfaces(int select);
@@ -126,6 +146,9 @@ extern void editmesh_align_view_to_selected(struct View3D *v3d, int axis);
 
 	/* Selection */
 extern void select_non_manifold(void);
+extern void select_sharp_edges(void);
+extern void select_linked_flat_faces(void);
+extern void select_faces_by_numverts(int numverts);
 extern void select_more(void);
 extern void select_less(void);
 extern void selectrandom_mesh(void);
@@ -133,29 +156,41 @@ extern void editmesh_select_by_material(int index);
 extern void editmesh_deselect_by_material(int index);
 
 extern void Edge_Menu(void);
+extern void select_mesh_group_menu(void);
 extern void editmesh_mark_seam(int clear);
+extern void loop_multiselect(int looptype);
+
 
 /* ******************* editmesh_loop.c */
 
 #define KNIFE_PROMPT 0
 #define KNIFE_EXACT 1
 #define KNIFE_MIDPOINT 2
+#define KNIFE_MULTICUT 3
 
+extern void CutEdgeloop(int numcuts);
 extern void KnifeSubdivide(char mode);
 extern void LoopMenu(void);
 
 #define LOOP_SELECT	1
 #define LOOP_CUT	2
 
-extern void loopoperations(char mode);
-extern void vertex_loop_select(void); 
+extern short sharesFace(struct EditEdge* e1, struct EditEdge* e2);
 
 /* ******************* editmesh_tools.c */
-extern void convert_to_triface(int all);
+
+#define SUBDIV_SELECT_ORIG      0
+#define SUBDIV_SELECT_INNER     1
+#define SUBDIV_SELECT_INNER_SEL 2
+#define SUBDIV_SELECT_LOOPCUT 3
+
+extern void convert_to_triface(int direction);
 extern int removedoublesflag(short flag, float limit);
 extern void xsortvert_flag(int flag);
 extern void hashvert_flag(int flag);
-extern void subdivideflag(int flag, float rad, int beauty);
+
+extern void esubdivideflag(int flag, float rad, int beauty, int numcuts, int selecttype);
+
 extern void extrude_mesh(void);
 extern void split_mesh(void);
 extern void extrude_repeat_mesh(int steps, float offs);
@@ -167,7 +202,27 @@ extern void join_triangles(void);
 extern void edge_flip(void);
 extern void fill_mesh(void);
 extern void bevel_menu();
+extern void mesh_set_smooth_faces(short event);
 void edge_rotate_selected(int dir);
+int EdgeSlide(short immediate, float imperc);
+int EdgeLoopDelete(void);
+void mesh_rip(void);
  
+struct EditVert *editedge_getOtherVert(struct EditEdge *eed, struct EditVert *ev);
+struct EditVert *editedge_getSharedVert(struct EditEdge *eed, struct EditEdge *eed2);
+int editedge_containsVert(struct EditEdge *eed, struct EditVert *eve);
+int editface_containsVert(struct EditFace *efa, struct EditVert *eve);
+int editface_containsEdge(struct EditFace *efa, struct EditEdge *eed);
+
+void shape_copy_select_from(void);
+void shape_propagate(void);
+
+int collapseEdges(void);
+int merge_firstlast(int first, int uvmerge);
+int merge_target( int target, int uvmerge);
+
+void pathselect(void);
+void loop_to_region(void);
+void region_to_loop(void);
 #endif
 
