@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editseq.h,v 1.9 2005/04/30 21:19:18 ton Exp $
+ * $Id: BIF_editseq.h,v 1.13 2006/07/04 19:46:08 schlaile Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,7 +40,10 @@ void				add_sequence(int type);
 void				borderselect_seq(void);
 void				boundbox_seq(void);
 void				change_sequence(void);
-void				clear_last_seq(void);
+void                    update_seq_ipo_rect(struct Sequence * seq);
+struct Sequence*                get_last_seq();
+void                            set_last_seq_to_null();
+void				clear_seq_belonging_to_ipo(struct Ipo * ipo);
 void				clever_numbuts_seq(void);
 void				del_seq(void);
 void				enter_meta(void);
@@ -57,6 +60,7 @@ void				swap_select_seq(void);
 void				touch_seq_files(void);
 void				transform_seq(int mode, int context);
 void				un_meta(void);
+void				seq_cut(int cutframe);
 
 /* drawseq.c */
 void do_seqbuttons(short);
