@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editview.h,v 1.8 2005/04/23 20:49:23 ton Exp $
+ * $Id: BIF_editview.h,v 1.10 2005/10/05 13:08:07 broken Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,13 +36,15 @@
 struct Base;
 struct Object;
 struct Camera;
+struct View3D;
 
 void	arrows_move_cursor(unsigned short event);
 void	borderselect(void);
 void	circle_select(void);
 void	deselectall(void);
-void selectall_type(short obtype);
-void selectall_layer(unsigned int layernum);
+void	selectswap(void);
+void	selectall_type(short obtype);
+void	selectall_layer(unsigned int layernum);
 void	draw_sel_circle(short *mval, short *mvalo, float rad, float rado, int selecting);
 void	fly(void);
 int		gesture(void);
@@ -52,5 +54,7 @@ void	set_active_base(struct Base *base);
 void	set_active_object(struct Object *ob);
 void	set_render_border(void);
 
+void	view3d_edit_clipping(struct View3D *v3d);
+
 #endif
 
