@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_renderwin.h,v 1.6 2004/12/27 19:28:49 ton Exp $
+ * $Id: BIF_renderwin.h,v 1.10 2006/06/26 14:57:56 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,9 +30,11 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
+struct Render;
 struct ScrArea;
+struct RenderStats;
 
-void calc_renderwin_rectangle(int posmask, int renderpos_r[2], int rendersize_r[2]);
+void calc_renderwin_rectangle(int rectx, int recty, int posmask, int renderpos_r[2], int rendersize_r[2]);
 
 void BIF_close_render_display(void);
 
@@ -50,3 +52,15 @@ void BIF_redraw_render_rect(void);
 void BIF_swap_render_rects(void);
 void BIF_toggle_render_display(void);
 
+void BIF_init_render_callbacks(struct Render *re, int do_display);
+void BIF_end_render_callbacks(void);
+
+/* should not be here, ~WIP~ */
+void make_renderinfo_string(struct RenderStats *rs, char *str);
+
+/* space for info text */
+#define RW_HEADERY		18
+
+/* header print for window */
+#define RW_MAXTEXT		512
+
