@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_resources.h,v 1.41 2005/06/04 16:22:49 lukep Exp $
+ * $Id: BIF_resources.h,v 1.57 2006/07/03 09:49:12 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,6 +33,8 @@
 #ifndef BIF_RESOURCES_H
 #define BIF_RESOURCES_H
 
+/* elubie: TODO: move the typedef for icons to BIF_interface_icons.h */
+/* and add/replace include of BIF_resources.h by BIF_interface_icons.h */
 typedef enum {
 #define BIFICONID_FIRST		(ICON_VIEW3D)
 	ICON_VIEW3D,
@@ -50,12 +52,12 @@ typedef enum {
 	ICON_NLA,
 	ICON_SCRIPTWIN,
 	ICON_TIME,
-	ICON_SPACE1,
+	ICON_NODE,
 	ICON_SPACE2,
 	ICON_SPACE3,
 	ICON_SPACE4,
-	ICON_SPACE5,
-	ICON_SPACE6,
+	ICON_TRIA_LEFT,
+	ICON_TRIA_UP,
 
 	ICON_ORTHO,
 	ICON_PERSP,
@@ -76,8 +78,8 @@ typedef enum {
 	ICON_SORTSIZE,
 	ICON_LONGDISPLAY,
 	ICON_SHORTDISPLAY,
-	ICON_TRIA_OPEN,
-	ICON_TRIA_CLOSED,
+	ICON_TRIA_DOWN,
+	ICON_TRIA_RIGHT,
 
 	ICON_VIEW_AXIS_ALL,
 	ICON_VIEW_AXIS_NONE,
@@ -91,9 +93,9 @@ typedef enum {
 	ICON_MAYBE_ITS_A_LASSO,
 	ICON_BLANK1,	/* ATTENTION, someone decided to use this throughout blender
 	                   and didn't care to neither rename it nor update the PNG */
-	ICON_BLANK2,
-	ICON_BLANK3,
-	ICON_BLANK4,
+	ICON_VERSE,
+	ICON_MOD_BOOLEAN,
+	ICON_ARMATURE,
 	ICON_PAUSE,
 	ICON_ALIGN,
 	ICON_REC,
@@ -117,8 +119,8 @@ typedef enum {
 	ICON_ENVMAP,
 	ICON_TRANSP_HLT,
 	ICON_TRANSP_DEHLT,
-	ICON_RADIO_DEHLT,
-	ICON_RADIO_HLT,
+	ICON_CIRCLE_DEHLT,
+	ICON_CIRCLE_HLT,
 	ICON_TPAINT_DEHLT,
 	ICON_TPAINT_HLT,
 	ICON_WPAINT_DEHLT,
@@ -158,7 +160,7 @@ typedef enum {
 	ICON_BPIBFOLDER_ERR,
 	ICON_UGLY_GREEN_RING,
 	ICON_GHOST,
-	ICON_BLANK32,
+	ICON_SORTBYEXT,
 	ICON_BLANK33,
 	ICON_VERTEXSEL,
 	ICON_EDGESEL,
@@ -180,8 +182,8 @@ typedef enum {
 	ICON_CHECKBOX_HLT,
 	ICON_LINK,
 	ICON_INLINK,
-	ICON_BEVELBUT_HLT,
-	ICON_BEVELBUT_DEHLT,
+	ICON_ZOOMIN,
+	ICON_ZOOMOUT,
 	ICON_PASTEDOWN,
 	ICON_COPYDOWN,
 	ICON_CONSTANT,
@@ -231,7 +233,7 @@ typedef enum {
 	ICON_IPO_DEHLT,
 	ICON_LIBRARY_DEHLT,
 	ICON_IMAGE_DEHLT,
-	ICON_WINDOW_FULLSCREEN,
+	ICON_EYEDROPPER,
 	ICON_WINDOW_WINDOW,
 	ICON_PANEL_CLOSE,
 	
@@ -268,9 +270,9 @@ typedef enum {
 	ICON_PROP_CON,
 	ICON_SYNTAX,
 	ICON_SYNTAX_OFF,
-	ICON_BLANK52,
-	ICON_BLANK53,
-	ICON_BLANK54,
+	ICON_MONKEY,
+	ICON_HAIR,
+	ICON_PLUS,
 	ICON_VIEWMOVE,
 	ICON_HOME,
 	ICON_CLIPUV_DEHLT,
@@ -284,24 +286,34 @@ typedef enum {
 	ICON_MAN_SCALE,
 	ICON_MANIPUL,
 	ICON_BLANK65,
-	ICON_BLANK66,
-	ICON_BLANK67,
-	ICON_BLANK68,
-	ICON_BLANK69,
-	ICON_BLANK70,
-	ICON_BLANK71,
-	ICON_BLANK72,
-	ICON_BLANK73,
-	ICON_BLANK74,
-	ICON_BLANK75,
-	ICON_BLANK76,
+	ICON_MODIFIER,
+	ICON_MOD_WAVE,
+	ICON_MOD_BUILD,
+	ICON_MOD_DECIM,
+	ICON_MOD_MIRROR,
+	ICON_MOD_SOFT,
+	ICON_MOD_SUBSURF,
+	ICON_SEQ_SEQUENCER,
+	ICON_SEQ_PREVIEW,
+	ICON_SEQ_LUMA_WAVEFORM,
+	ICON_SEQ_CHROMA_SCOPE,
 	ICON_ROTATE,
 	ICON_CURSOR,
 	ICON_ROTATECOLLECTION,
 	ICON_ROTATECENTER,
-	ICON_ROTACTIVE
-	
-#define BIFICONID_LAST		(ICON_ROTACTIVE)
+	ICON_ROTACTIVE,
+
+	VICON_VIEW3D,
+	VICON_EDIT,
+	VICON_EDITMODE_DEHLT,
+	VICON_EDITMODE_HLT,
+	VICON_DISCLOSURE_TRI_RIGHT,
+	VICON_DISCLOSURE_TRI_DOWN,
+	VICON_MOVE_UP,
+	VICON_MOVE_DOWN,
+	VICON_X
+
+#define BIFICONID_LAST		(VICON_X)
 #define BIFNICONIDS			(BIFICONID_LAST-BIFICONID_FIRST + 1)
 } BIFIconID;
 
@@ -399,6 +411,8 @@ enum {
 	TH_REDALERT,
 	TH_CUSTOM,
 	
+	TH_BUT_TEXTFIELD_HI,
+	
 	TH_THEMEUI,
 // common colors among spaces
 	
@@ -416,6 +430,8 @@ enum {
 	TH_WIRE,
 	TH_SELECT,
 	TH_ACTIVE,
+	TH_GROUP,
+	TH_GROUP_ACTIVE,
 	TH_TRANSFORM,
 	TH_VERTEX,
 	TH_VERTEX_SELECT,
@@ -434,7 +450,30 @@ enum {
 	TH_SYNTAX_V,
 	TH_SYNTAX_C,
 	TH_SYNTAX_L,
-	TH_SYNTAX_N
+	TH_SYNTAX_N,
+	
+	TH_BONE_SOLID,
+	TH_BONE_POSE,
+	
+	TH_STRIP,
+	TH_STRIP_SELECT,
+	
+	TH_LAMP,
+	
+	TH_NODE,
+	TH_NODE_IN_OUT,
+	TH_NODE_OPERATOR,
+	TH_NODE_CONVERTOR,
+	TH_NODE_GROUP,
+	
+	TH_SEQ_MOVIE,
+	TH_SEQ_IMAGE,
+	TH_SEQ_SCENE,
+	TH_SEQ_AUDIO,
+	TH_SEQ_EFFECT,
+	TH_SEQ_PLUGIN,
+	TH_SEQ_TRANSITION,
+	TH_SEQ_META,
 	
 };
 /* XXX WARNING: previous is saved in file, so do not change order! */
@@ -477,6 +516,12 @@ void 	BIF_GetThemeColor4ubv(int colorid, char *col);
 // get a theme color from specified space type
 void	BIF_GetThemeColorType4ubv(int colorid, int spacetype, char *col);
 
+// blends and shades between two color pointers
+void	BIF_ColorPtrBlendShade3ubv(char *cp1, char *cp2, float fac, int offset);
+
+// get a 3 byte color, blended and shaded between two other char color pointers
+void	BIF_GetColorPtrBlendShade3ubv(char *cp1, char *cp2, char *col, float fac, int offset);
+
 
 struct ScrArea;
 
@@ -488,13 +533,6 @@ void	BIF_resources_free		(void);
 void	BIF_colors_init			(void);
 void	BIF_load_ui_colors		(void);
 
-
-// icon API
-int		BIF_get_icon_width		(BIFIconID icon);
-int		BIF_get_icon_height		(BIFIconID icon);
-void	BIF_draw_icon			(BIFIconID icon);
-void	BIF_draw_icon_blended	(BIFIconID icon, int colorid, int shade);
-
 /* only for buttons in theme editor! */
 char 	*BIF_ThemeGetColorPtr(struct bTheme *btheme, int spacetype, int colorid);
 char 	*BIF_ThemeColorsPup(int spacetype);
