@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_transform.h,v 1.31 2005/05/10 04:21:11 theeth Exp $
+ * $Id: BIF_transform.h,v 1.38 2005/08/23 18:13:30 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,7 +37,6 @@
 
 /* MODE AND NUMINPUT FLAGS */
 #define TFM_INIT			-1
-#define TFM_REPEAT			0
 #define TFM_TRANSLATION		1
 #define TFM_ROTATION		2
 #define TFM_RESIZE			3
@@ -51,13 +50,15 @@
 #define TFM_PUSHPULL		12
 #define TFM_CREASE			13
 #define TFM_MIRROR			14
+#define TFM_BONESIZE		15
+#define TFM_BONE_ENVELOPE	16
 
 /* TRANSFORM CONTEXTS */
 #define CTX_NONE			0
 #define CTX_TEXTURE			1
 #define CTX_EDGE			2
 #define CTX_NO_PET			4
-#define CTX_NO_NOR_RECALC	8
+#define CTX_TWEAK			8
 
 void initTransform(int mode, int context);
 void Transform(void);
@@ -73,11 +74,15 @@ void BIF_setLocalAxisConstraint(char axis, char *text);
 void BIF_setLocalLockConstraint(char axis, char *text);
 void BIF_drawConstraint(void);
 void BIF_drawPropCircle(void);
+void BIF_getPropCenter(float *center);
 
 void BIF_TransformSetUndo(char *str);
 
+void BIF_selectOrientation(void);
+
 /* view3d manipulators */
-void ManipulatorTransform(int mode);
+void initManipulator(int mode);
+void ManipulatorTransform();
 
 int BIF_do_manipulator(struct ScrArea *sa);
 void BIF_draw_manipulator(struct ScrArea *sa);
