@@ -1,5 +1,5 @@
 /**
- * $Id: BSE_headerbuttons.h,v 1.21 2005/05/19 19:58:52 ton Exp $
+ * $Id: BSE_headerbuttons.h,v 1.28 2006/06/27 11:55:33 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,15 +36,17 @@
 struct uiBlock;
 struct ScrArea;
 struct ID;
+struct SpaceIpo;
+struct Ipo;
 
 /* these used to be in blender/src/headerbuttons.c: */
-#define SPACEICONMAX  15 /* See release/datafiles/blenderbuttons */
+#define SPACEICONMAX  16 /* See release/datafiles/blenderbuttons */
 #define XIC 20
 #define YIC 20
 
 int std_libbuttons(struct uiBlock *block, 
        short xco, short yco, int pin, short *pinpoin, 
-       int browse, struct ID *id, struct ID *parid, 
+       int browse, short id_code, short special, struct ID *id, struct ID *parid, 
        short *menupoin, int users, 
        int lib, int del, int autobut, int keepbut);
 
@@ -87,8 +89,9 @@ void seq_buttons(void);
 void sound_buttons(void);
 void text_buttons(void);
 void script_buttons(void);
-void time_buttons(struct ScrArea *sa);
 void view3d_buttons(void);
+void time_buttons(struct ScrArea *sa);
+void node_buttons(struct ScrArea *sa);
 
 void do_global_buttons(unsigned short event);
 void do_global_buttons2(short event);
@@ -106,14 +109,19 @@ void do_oops_buttons(short event);
 void do_seq_buttons(short event);
 void do_sound_buttons(unsigned short event);
 void do_text_buttons(unsigned short event);
-void do_time_buttons(struct ScrArea *sa, unsigned short event);
 void do_script_buttons(unsigned short event);
 void do_view3d_buttons(short event); 
+void do_time_buttons(struct ScrArea *sa, unsigned short event);
+void do_node_buttons(struct ScrArea *sa, unsigned short event); 
 
 void do_headerbuttons(short event);
 
+/* header_ipo.c */
+void spaceipo_assign_ipo(struct SpaceIpo *si, struct Ipo *ipo);
+
 /* header_text.c */
 void do_text_editmenu_to3dmenu(void *arg, int event);
+void do_text_formatmenu_convert(void *arg, int event);
 
 /* header_info.c */
 void do_info_add_meshmenu(void *arg, int event);
@@ -123,11 +131,16 @@ void do_info_add_metamenu(void *arg, int event);
 void do_info_add_lampmenu(void *arg, int event);
 void do_info_addmenu(void *arg, int event);
 
+/* header_node.c */
+void do_node_addmenu(void *arg, int event);
+
 /* header_view3d.c */
+void do_view3d_select_objectmenu(void *arg, int event);
 void do_view3d_select_object_groupedmenu(void *arg, int event);
 void do_view3d_select_object_linkedmenu(void *arg, int event);
 void do_view3d_select_object_layermenu(void *arg, int event);
 void do_view3d_select_object_typemenu(void *arg, int event);
+void do_view3d_select_faceselmenu(void *arg, int event);
 void do_view3d_select_meshmenu(void *arg, int event);
 void do_view3d_select_curvemenu(void *arg, int event);
 void do_view3d_edit_snapmenu(void *arg, int event);
@@ -139,6 +152,7 @@ void do_view3d_object_mirrormenu(void *arg, int event);
 void do_view3d_edit_mesh_normalsmenu(void *arg, int event);
 void do_view3d_edit_mesh_verticesmenu(void *arg, int event);
 void do_view3d_edit_mesh_edgesmenu(void *arg, int event);
+void do_view3d_edit_mesh_facesmenu(void *arg, int event);
 void do_view3d_edit_curve_segmentsmenu(void *arg, int event);
 void do_view3d_edit_curve_showhidemenu(void *arg, int event); 
 
