@@ -1,4 +1,4 @@
-/* $Id: blendef.h,v 1.36 2005/05/11 20:01:42 ton Exp $
+/* $Id: blendef.h,v 1.53 2006/06/29 13:46:39 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,8 +34,8 @@
 
 /* **************** MAX ********************* */
 
-#define MAXFRAME	30000
-#define MAXFRAMEF	30000.0f
+#define MAXFRAME	300000
+#define MAXFRAMEF	300000.0f
 
 #define MINFRAME	1
 #define MINFRAMEF	1.0
@@ -89,6 +89,7 @@
 
 #define SELECT			1
 #define ACTIVE			2
+#define DESELECT		0
 #define NOT_YET			0
 
 
@@ -115,9 +116,6 @@
 #define FNT_PDRAW 1
 #define FNT_HAEBERLI 2
 
-/* isect and scanfill */
-#define COMPLIMIT	0.0003
-
 /* getbutton */
 
 /* do_global_buttons(event) */
@@ -164,7 +162,7 @@
 #define B_REDRCURW3D		63
 #define B_FLIPINFOMENU		64
 #define B_FLIPFULLSCREEN	65
-#define B_EASYTOOLBOX		66
+#define B_PLAINMENUS		66
 
 
 #define B_SHOWSPLASH		70
@@ -208,9 +206,7 @@
 #define B_SCENELOCK		140
 #define B_LOCALVIEW		141
 #define B_U_CAPSLOCK	142
-#define B_EDITMODE		143
-#define B_VPAINT		144
-#define B_FACESEL		145
+
 #define B_VIEWBUT		146
 #define B_PERSP			147
 #define B_PROPTOOL		148
@@ -218,9 +214,7 @@
 #define B_VIEWTRANS		150
 #define B_VIEWZOOM		151
 #define B_STARTGAME		152
-#define	B_POSEMODE		153
-#define	B_TEXTUREPAINT	154
-#define B_WPAINT		155
+
 #define B_MODESELECT	156
 #define B_AROUND		157
 #define B_SEL_VERT		158
@@ -230,7 +224,7 @@
 #define B_MAN_ROT		162
 #define B_MAN_SCALE		163
 #define B_HEMESHBROWSE	164
-#define B_HEMESHLOCAL 165
+#define B_HEMESHLOCAL	165
 
 /* IPO: 200 */
 #define B_IPOHOME		201
@@ -241,16 +235,20 @@
 #define B_IPOEXTRAP		206
 #define B_IPOCYCLIC		207
 #define B_IPOMAIN		208
-#define B_IPOSHOWKEY		209
-#define B_IPOCYCLICX		210
+#define B_IPOSHOWKEY	209
+#define B_IPOCYCLICX	210
 	/* warn: also used for oops and seq */
-#define B_VIEW2DZOOM		211
+#define B_VIEW2DZOOM	211
 #define B_IPOPIN		212
+#define B_IPO_ACTION_OB		213
+#define B_IPO_ACTION_KEY	214
+
 
 /* OOPS: 250 */
 #define B_OOPSHOME		251
 #define B_OOPSBORDER		252
 #define B_NEWOOPS		253
+#define B_OOPSVIEWSEL		254
 
 /* INFO: 300 */
 /* watch: also in filesel.c and editobject.c */
@@ -279,6 +277,8 @@
 #define B_UITHEMEIMPORT		323
 #define B_UITHEMEEXPORT		324
 
+#define B_MEMCACHELIMIT         325
+
 /* Definitions for the fileselect buttons in user prefs */
 #define B_FONTDIRFILESEL  	330
 #define B_TEXTUDIRFILESEL  	331
@@ -308,6 +308,15 @@
 #define B_SIMAGEPAINTTOOL	362
 #define B_SIMAPACKIMA		363
 #define B_SIMAGESAVE		364
+#define B_SIMACLONEBROWSE	365
+#define B_SIMACLONEDELETE	366
+#define B_SIMABRUSHCHANGE	367
+#define B_SIMANOTHING		368
+#define B_SIMACURVES		369
+#define B_SIMARANGE			370
+#define B_SIMA_USE_ALPHA	371
+#define B_SIMA_SHOW_ALPHA	372
+#define B_SIMA_SHOW_ZBUF	373
 
 /* BUTS: 400 */
 #define B_BUTSHOME		401
@@ -368,10 +377,15 @@
 #define B_TL_NEXTKEY	755
 #define B_TL_STOP		756
 
-/* NLA: 801-900 */
+/* NLA: 801-850 */
 #define B_NLAHOME		801
 
-/* FREE 900 - 999 */
+/* NODE: 851-900 */
+#define B_NODEHOME			851
+#define B_NODE_USEMAT		852
+#define B_NODE_USESCENE		853
+
+/* FREE 901 - 999 */
 
 
 #define B_NOTHING		-1
@@ -383,13 +397,24 @@
 #define B_KEEPORIG		2
 #define B_BEAUTY		4
 #define B_SMOOTH		8
+#define B_BEAUTY_SHORT  16
+#define B_AUTOFGON		32
 #define B_KNIFE			0x80
 #define B_PERCENTSUBD	0x40
-
+#define B_MESH_X_MIRROR	0x100
 
 /* DISPLAYMODE */
-#define R_DISPLAYVIEW	0
+#define R_DISPLAYIMAGE	0
 #define R_DISPLAYWIN	1
-#define R_DISPLAYAUTO	2
+#define R_DISPLAYSCREEN	2
+
+	/* Gvp.flag and Gwp.flag */
+#define VP_COLINDEX	1
+#define VP_AREA		2
+#define VP_SOFT		4
+#define VP_NORMALS	8
+#define VP_SPRAY	16
+#define VP_MIRROR_X	32
+#define VP_HARD		64
 
 #endif
