@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_constraint_types.h,v 1.10 2004/09/05 20:21:16 theeth Exp $
+ * $Id: DNA_constraint_types.h,v 1.24 2006/05/06 15:26:52 schlaile Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,6 +40,7 @@
 
 struct Action;
 
+/* channels reside in Object or Action (ListBase) constraintChannels */
 typedef struct bConstraintChannel{
 	struct bConstraintChannel *next, *prev;
 	Ipo			*ipo;
@@ -50,31 +51,27 @@ typedef struct bConstraintChannel{
 typedef struct bConstraint{
 	struct bConstraint *next, *prev;
 	void		*data;		/*	Constraint data	(a valid constraint type) */
-	Ipo			*ipo;		/*	Constraint ipo	*/
-	char		type;		/*	Constraint type	*/
-	char		otype;		/*	Old type - for menu callbacks */
+	short		type;		/*	Constraint type	*/
 	short		flag;		/*	Flag */
 	short		reserved1;	
 	char		name[30];	/*	Constraint name	*/
 
 	float		enforce;
-	float		time;		
-	float		offset[3];	/* Target location offset */
-	float		orient[3];	/* Target orientation offset */
-	float		roll[3];	/* Target roll offset (needed?) */
 } bConstraint;
 
 /* Single-target subobject constraints */
 typedef struct bKinematicConstraint{
 	Object		*tar;
 	float		tolerance;		/* Acceptable distance from target */
-	int			iterations;		/* Maximum number of iterations to try */
+	short		iterations;		/* Maximum number of iterations to try */
+	short		flag;			/* Like CONSTRAINT_IK_TIP */
+	int			rootbone, pad;	/* index to rootbone, if zero go all the way to mother bone */
 	char		subtarget[32];	/* String to specify sub-object target */
 
-	float		cacheeff[3];	/* Target location cache */
-	int			reserved1;
-
-	float		cachemat[4][4];	/* Result cache */
+	float		weight;			/* Weight of goal in IK tree */
+	float		orientweight;	/* Amount of rotation a target applies on chain */
+	float		grabtarget[3];	/* for target-less IK */
+	float		pad2;
 } bKinematicConstraint;
 
 typedef struct bTrackToConstraint{
@@ -98,13 +95,32 @@ typedef struct bLocateLikeConstraint{
 	char		subtarget[32];
 } bLocateLikeConstraint;
 
+typedef struct bMinMaxConstraint{
+	Object		*tar;
+	int			minmaxflag;
+	float			offset;
+	short			sticky;
+	short			stuck;
+	float			cache[3];
+	char		subtarget[32];
+} bMinMaxConstraint;
+
+typedef struct bSizeLikeConstraint{
+	Object		*tar;
+	int			flag;
+	int			reserved1;
+	char		subtarget[32];
+} bSizeLikeConstraint;
+
 typedef struct bActionConstraint{
 	Object		*tar;
-	int			type;
-	short		start;
-	short		end;
+	short		type;
+	short		local;
+	int		start;
+	int		end;
 	float		min;
 	float		max;
+	int             pad;
 	struct bAction	*act;
 	char		subtarget[32];
 } bActionConstraint;
@@ -166,58 +182,77 @@ typedef struct bStretchToConstraint{
 #define CONSTRAINT_TYPE_SIZELIMIT	7	/* Unimplemented */
 #define CONSTRAINT_TYPE_ROTLIKE		8	
 #define CONSTRAINT_TYPE_LOCLIKE		9	
-#define CONSTRAINT_TYPE_SIZELIKE	10	/* Unimplemented */
+#define CONSTRAINT_TYPE_SIZELIKE	10
 #define CONSTRAINT_TYPE_PYTHON		11	/* Unimplemented */
 #define CONSTRAINT_TYPE_ACTION		12
 #define CONSTRAINT_TYPE_LOCKTRACK	13	/* New Tracking constraint that locks an axis in place - theeth */
 #define CONSTRAINT_TYPE_DISTANCELIMIT	14 
 #define CONSTRAINT_TYPE_STRETCHTO	15  /* claiming this to be mine :) is in tuhopuu bjornmose */ 
+#define CONSTRAINT_TYPE_MINMAX      16  /* floor constraint */
 
 
 /* bConstraint.flag */
-#define CONSTRAINT_EXPAND		0x00000001
-#define CONSTRAINT_DONE			0x00000002
-#define CONSTRAINT_DISABLE		0x00000004
-#define CONSTRAINT_LOOPTESTED	0x00000008
-#define CONSTRAINT_NOREFRESH	0x00000010
+		/* expand for UI */
+#define CONSTRAINT_EXPAND		0x01
+		/* pre-check for illegal object name or bone name */
+#define CONSTRAINT_DISABLE		0x04
+		/* flags 0x2 and 0x8 were used in past, skip this */
+		/* to indicate which Ipo should be shown, maybe for 3d access later too */
+#define CONSTRAINT_ACTIVE		0x10
+		/* only for Pose, evaluates constraints in posechannel local space */
+#define CONSTRAINT_LOCAL		0x20
 
-#define CONSTRAINT_EXPAND_BIT	0
-#define CONSTRAINT_DONE_BIT		1
-#define CONSTRAINT_DISABLE_BIT	2
 
 /* bConstraintChannel.flag */
-#define CONSTRAINT_CHANNEL_SELECT	0x00000001
+#define CONSTRAINT_CHANNEL_SELECT	0x01
+
+/* bRotateLikeConstraint.flag */
+#define ROTLIKE_X		0x01
+#define ROTLIKE_Y		0x02
+#define ROTLIKE_Z		0x04
 
 /* bLocateLikeConstraint.flag */
-#define LOCLIKE_X		0x00000001
-#define LOCLIKE_Y		0x00000002
-#define LOCLIKE_Z		0x00000004
-#define LOCSPACE		0x00000008
+#define LOCLIKE_X		0x01
+#define LOCLIKE_Y		0x02
+#define LOCLIKE_Z		0x04
+#define LOCSPACE		0x08
+ 
+/* bSizeLikeConstraint.flag */
+#define SIZELIKE_X		0x01
+#define SIZELIKE_Y		0x02
+#define SIZELIKE_Z		0x04
+#define SIZESPACE		0x08
 
 /* Axis flags */
-#define LOCK_X		0x00000000
-#define LOCK_Y		0x00000001
-#define LOCK_Z		0x00000002
+#define LOCK_X		0x00
+#define LOCK_Y		0x01
+#define LOCK_Z		0x02
 
-#define UP_X		0x00000000
-#define UP_Y		0x00000001
-#define UP_Z		0x00000002
+#define UP_X		0x00
+#define UP_Y		0x01
+#define UP_Z		0x02
 
-#define TRACK_X		0x00000000
-#define TRACK_Y		0x00000001
-#define TRACK_Z		0x00000002
-#define TRACK_nX	0x00000003
-#define TRACK_nY	0x00000004
-#define TRACK_nZ	0x00000005
+#define TRACK_X		0x00
+#define TRACK_Y		0x01
+#define TRACK_Z		0x02
+#define TRACK_nX	0x03
+#define TRACK_nY	0x04
+#define TRACK_nZ	0x05
 
-#define VOLUME_XZ	0x00000000
-#define VOLUME_X	0x00000001
-#define VOLUME_Z	0x00000002
-#define NO_VOLUME	0x00000003
+#define VOLUME_XZ	0x00
+#define VOLUME_X	0x01
+#define VOLUME_Z	0x02
+#define NO_VOLUME	0x03
 
-#define PLANE_X		0x00000000
-#define PLANE_Y		0x00000001
-#define PLANE_Z		0x00000002
+#define PLANE_X		0x00
+#define PLANE_Y		0x01
+#define PLANE_Z		0x02
+
+/* bKinematicConstraint->flag */
+#define CONSTRAINT_IK_TIP		1
+#define CONSTRAINT_IK_ROT		2
+#define CONSTRAINT_IK_AUTO		4
+#define CONSTRAINT_IK_TEMP		8
 
 #endif
 
