@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_ipo_types.h (mar-2001 nzc)
  *
- * $Id: DNA_ipo_types.h,v 1.14 2005/03/01 13:14:37 ton Exp $ 
+ * $Id: DNA_ipo_types.h,v 1.19 2006/02/27 11:45:42 n_t Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -59,7 +59,6 @@ typedef short IPO_Channel;
 
 #define TOB_IPO	1
 #define TOB_IPODROT	2
-#define TOB_IKA	4
 
 /* disptype */
 #define IPO_DISPDEGR	1
@@ -96,10 +95,6 @@ typedef short IPO_Channel;
 
 #define OB_TIME		20
 
-#define OB_EFF_X	21
-#define OB_EFF_Y	22
-#define OB_EFF_Z	23
-
 #define OB_COL_R	21
 #define OB_COL_G	22
 #define OB_COL_B	23
@@ -176,8 +171,8 @@ typedef short IPO_Channel;
 
 /* ******************** */
 
-#define TE_TOTIPO	21
-#define TE_TOTNAM	21
+#define TE_TOTIPO	26
+#define TE_TOTNAM	26
 
 #define TE_NSIZE	1
 #define TE_NDEPTH	2
@@ -205,6 +200,12 @@ typedef short IPO_Channel;
 #define TE_N_BAS1	20
 #define TE_N_BAS2	21
 
+#define TE_COL_R    22
+#define TE_COL_G    23
+#define TE_COL_B    24
+#define TE_BRIGHT	25
+#define TE_CONTRA	26
+
 /* ******************** */
 
 #define SEQ_TOTIPO	1
@@ -325,10 +326,30 @@ typedef short IPO_Channel;
 #define CO_ROLL		9
 */
 
-/* these are ipo-specific */
+/* ******************** */
+/* fluidsim ipos NT */
+
+#define FLUIDSIM_TOTIPO	9
+#define FLUIDSIM_TOTNAM	9
+
+#define FLUIDSIM_VISC   1
+#define FLUIDSIM_TIME   2
+
+#define FLUIDSIM_GRAV_X 3
+#define FLUIDSIM_GRAV_Y 4
+#define FLUIDSIM_GRAV_Z 5
+
+#define FLUIDSIM_VEL_X  6
+#define FLUIDSIM_VEL_Y  7
+#define FLUIDSIM_VEL_Z  8
+
+#define FLUIDSIM_ACTIVE 9
+
+
+/* these are IpoCurve specific */
 /* **************** IPO ********************* */
 
-/* vartype */
+/* icu->vartype */
 #define IPO_CHAR		0
 #define IPO_SHORT		1
 #define IPO_INT			2
@@ -336,33 +357,35 @@ typedef short IPO_Channel;
 #define IPO_FLOAT		4
 #define IPO_DOUBLE		5
 #define IPO_FLOAT_DEGR	6
-/* very special case, in keys */
+	/* very special case, in keys */
 #define IPO_BEZTRIPLE	100
 #define IPO_BPOINT		101
 
-
+/* icu->vartype */
 #define IPO_BITS		16
 #define IPO_CHAR_BIT	16
 #define IPO_SHORT_BIT	17
 #define IPO_INT_BIT		18
 
-/* ipo */
+/* icu->ipo */
 #define IPO_CONST		0
 #define IPO_LIN			1
 #define IPO_BEZ			2
 #define IPO_MIXED		3
 
-/* extrap */
+/* icu->extrap */
 #define IPO_HORIZ		0
 #define IPO_DIR			1
 #define IPO_CYCL		2
 #define IPO_CYCLX		3
 
-/* flag */
+/* icu->flag */
 #define IPO_VISIBLE		1
 #define IPO_SELECT		2
 #define IPO_EDIT		4
 #define IPO_LOCK		8
+#define IPO_AUTO_HORIZ	16
+#define IPO_ACTIVE		32
 
 #endif
 
