@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_material_types.h (mar-2001 nzc)
  *
- * $Id: DNA_material_types.h,v 1.28 2005/05/27 17:52:52 eeshlo Exp $ 
+ * $Id: DNA_material_types.h,v 1.42 2006/06/26 17:50:48 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,10 +34,9 @@
 #ifndef DNA_MATERIAL_TYPES_H
 #define DNA_MATERIAL_TYPES_H
 
-/*  #include "BLI_listBase.h" */
-
 #include "DNA_ID.h"
 #include "DNA_scriptlink_types.h"
+#include "DNA_listBase.h"
 
 #ifndef MAX_MTEX
 #define MAX_MTEX	10
@@ -47,37 +46,42 @@ struct MTex;
 struct Ipo;
 struct Material;
 struct ColorBand;
+struct Group;
+struct bNodeTree;
 
 /* WATCH IT: change type? also make changes in ipo.h  */
 
 typedef struct Material {
 	ID id;
 	
-	short colormodel, lay;		/* lay: for dynamics (old engine, until 2.04) */
+	short colormodel, flag;	
+	/* note, keep this below synced with render_types.h */
 	float r, g, b;
 	float specr, specg, specb;
 	float mirr, mirg, mirb;
 	float ambr, ambb, ambg;
-	
 	float amb, emit, ang, spectra, ray_mirror;
 	float alpha, ref, spec, zoffs, add;
 	float translucency;
+	/* end synced with render_types.h */
+	
 	float fresnel_mir, fresnel_mir_i;
 	float fresnel_tra, fresnel_tra_i;
-	float filter, pad0;		/* filter added, for raytrace transparency */
+	float filter;		/* filter added, for raytrace transparency and transmissivity */
+	float tx_limit, tx_falloff;
 	short ray_depth, ray_depth_tra;
 	short har;
 	char seed1, seed2;
 	
-	int mode; 
-	int mode2; /* even more material settings :) */
+	int mode, mode_l;		/* mode_l is the or-ed result of all layer modes */
 	short flarec, starc, linec, ringc;
 	float hasize, flaresize, subsize, flareboost;
+	float strand_sta, strand_end, strand_ease;
+	float sbias;			/* shadow bias */
 	
 	/* for buttons and render*/
-	char rgbsel, texact, pr_type, pad;
-	short pr_back, pr_lamp, septex, pad4;
-	int pad5;
+	char rgbsel, texact, pr_type, use_nodes;
+	short pr_back, pr_lamp, septex, ml_flag;	/* ml_flag is for disable base material */
 	
 	/* shaders */
 	short diff_shader, spec_shader;
@@ -96,7 +100,9 @@ typedef struct Material {
 	float rampfac_col, rampfac_spec;
 
 	struct MTex *mtex[10];
+	struct bNodeTree *nodetree;	
 	struct Ipo *ipo;
+	struct Group *group;	/* light group */
 	
 	/* dynamic properties */
 	float friction, fh, reflect;
@@ -106,7 +112,7 @@ typedef struct Material {
 	/* yafray: absorption color, dispersion parameters and material preset menu */
 	float YF_ar, YF_ag, YF_ab, YF_dscale, YF_dpwr;
 	int YF_dsmp, YF_preset, YF_djit;
-
+	
 	ScriptLink scriptlink;
 } Material;
 
@@ -123,6 +129,10 @@ typedef struct Material {
 #define MA_YUV			2
 #define MA_HSV			3
 
+/* flag */
+		/* for render */
+#define MA_IS_USED		1
+
 /* mode (is int) */
 #define MA_TRACEBLE		1
 #define MA_SHADOW		2
@@ -142,6 +152,7 @@ typedef struct Material {
 #define MA_FACETEXTURE	0x800
 #define MA_HALOTEX		0x1000
 #define MA_HALOPUNO		0x2000
+#define MA_ONLYCAST		0x2000
 #define MA_NOMIST		0x4000
 #define MA_HALO_SHADE	0x4000
 #define MA_HALO_FLARE	0x8000
@@ -153,12 +164,19 @@ typedef struct Material {
 #define MA_RAMP_SPEC	0x200000
 #define MA_RAYBIAS		0x400000
 #define MA_FULL_OSA		0x800000
+#define MA_TANGENT_STR	0x1000000
+#define MA_SHADBUF		0x2000000
+		/* note; we drop MA_TANGENT_STR later to become tangent_u */
+#define MA_TANGENT_V	0x4000000
+
+#define	MA_MODE_MASK	0x3ffffff	/* all valid mode bits */
 
 /* diff_shader */
 #define MA_DIFF_LAMBERT		0
 #define MA_DIFF_ORENNAYAR	1
 #define MA_DIFF_TOON		2
-#define MA_DIFF_MINNAERT        3
+#define MA_DIFF_MINNAERT    3
+#define MA_DIFF_FRESNEL     4
 
 /* spec_shader */
 #define MA_SPEC_COOKTORR	0
@@ -186,6 +204,13 @@ typedef struct Material {
 #define MA_RAMP_DIFF		6
 #define MA_RAMP_DARK		7
 #define MA_RAMP_LIGHT		8
+#define MA_RAMP_OVERLAY		9
+#define MA_RAMP_DODGE		10
+#define MA_RAMP_BURN		11
+#define MA_RAMP_HUE			12
+#define MA_RAMP_SAT			13
+#define MA_RAMP_VAL			14
+#define MA_RAMP_COLOR		15
 
 /* texco */
 #define TEXCO_ORCO		1
@@ -200,8 +225,11 @@ typedef struct Material {
 #define TEXCO_OSA		512
 #define TEXCO_WINDOW	1024
 #define NEED_UV			2048
-/* optim = use simpler AA */
-#define TEXCO_OPTIM		4096
+#define TEXCO_TANGENT	4096
+	/* still stored in vertex->accum, 1 D */
+#define TEXCO_STRAND	8192
+#define TEXCO_STRESS	16384
+#define TEXCO_SPEED		32768
 
 /* mapto */
 #define MAP_COL			1
@@ -219,11 +247,18 @@ typedef struct Material {
 #define MAP_AMB			2048
 #define MAP_DISPLACE	4096
 #define MAP_WARP		8192
+#define MAP_LAYER		16384
 
 /* pr_type */
 #define MA_FLAT			0
 #define MA_SPHERE		1
 #define MA_CUBE			2
+#define MA_MONKEY		3
+#define MA_SPHERE_A		4
+#define MA_TEXTURE		5
+#define MA_LAMP			6
+#define MA_SKY			7
+#define MA_HAIR			10
 
 /* pr_back */
 #define MA_DARK			1
