@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_mesh_types.h,v 1.24 2005/04/04 04:20:31 zuster Exp $ 
+ * $Id: DNA_mesh_types.h,v 1.31 2006/04/03 19:02:21 briggs Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,6 +42,7 @@ struct Key;
 struct Material;
 struct MVert;
 struct MEdge;
+struct MFace;
 struct MCol;
 struct MSticky;
 struct Mesh;
@@ -64,30 +65,26 @@ typedef struct Mesh {
 	struct BoundBox *bb;
 
 	ListBase effect;
-	ListBase disp;
 	
 	struct Ipo *ipo;
 	struct Key *key;
 	struct Material **mat;
 
-	void *mface, *dface, *tface;
+	struct MFace *mface;
+	struct TFace *tface;
+	void *dface;
 	struct MVert *mvert;
 	struct MEdge *medge;
 	struct MDeformVert *dvert;	/* __NLA */
 	struct MCol *mcol;
 	struct MSticky *msticky;
 	struct Mesh *texcomesh;
-	float *orco;
-
-		/* not written in file, caches derived mesh */
-	struct DerivedMesh *derived;
-		/* hacky place to store temporary decimated mesh */
-	struct DispListMesh *decimated;
-
+	struct MSelect *mselect;
+	
 	struct OcInfo *oc;		/* not written in file */
 	void *sumohandle;
 
-	int totvert, totedge, totface;
+	int totvert, totedge, totface, totselect, pad2;
 	int texflag;
 	
 	float loc[3];
@@ -159,14 +156,14 @@ typedef struct Mesh {
 #define TF_SUB		3
 
 /* tface->unwrap */
-#define TF_SEAM1	1
-#define TF_SEAM2	2
-#define TF_SEAM3	4
-#define TF_SEAM4	8
-#define TF_PIN1	    16
-#define TF_PIN2	    32
-#define TF_PIN3	    64
-#define TF_PIN4	    128
+#define TF_DEPRECATED1	1
+#define TF_DEPRECATED2	2
+#define TF_DEPRECATED3	4
+#define TF_DEPRECATED4	8
+#define TF_PIN1		    16
+#define TF_PIN2		    32
+#define TF_PIN3	   		64
+#define TF_PIN4	    	128
 
 #define MESH_MAX_VERTS 2000000000L
 
