@@ -1,5 +1,5 @@
 /**
- * $Id: DNA_meshdata_types.h,v 1.7 2004/09/24 12:58:21 ton Exp $ 
+ * $Id: DNA_meshdata_types.h,v 1.20 2006/04/03 19:02:21 briggs Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,8 +36,8 @@ struct Bone;
 
 typedef struct MFace {
 	unsigned int v1, v2, v3, v4;
-	char puno, mat_nr;
-	char edcode, flag;
+	char pad, mat_nr;
+	char edcode, flag;	/* we keep edcode, for conversion to edges draw flags in old files */
 } MFace;
 
 typedef struct MEdge {
@@ -49,13 +49,12 @@ typedef struct MEdge {
 typedef struct MDeformWeight {
 	int				def_nr;
 	float			weight;
-	struct Bone		*data;		/* Runtime: Does not need to be valid in file */
 } MDeformWeight;
 
 typedef struct MDeformVert {
 	struct MDeformWeight *dw;
 	int totweight;
-	int reserved1;
+	int flag;	// flag only in use for weightpaint now
 } MDeformVert;
 
 typedef struct MVert {
@@ -72,16 +71,26 @@ typedef struct MSticky {
 	float co[2];
 } MSticky;
 
+typedef struct MSelect {
+	int index;
+	int type;
+} MSelect;
 /* mvert->flag (1=SELECT) */
 #define ME_SPHERETEST	2
 #define ME_SPHERETEMP	4
 #define ME_HIDE			16
+#define ME_VERT_MERGED		(1<<6)
+#define ME_VERT_STEPINDEX	(1<<7)
 
 /* medge->flag (1=SELECT)*/
-#define ME_EDGEDRAW		2
-#define ME_SEAM         4
-#define ME_FGON			8
+#define ME_EDGEDRAW			(1<<1)
+#define ME_SEAM				(1<<2)
+#define ME_FGON				(1<<3)
 						// reserve 16 for ME_HIDE
+#define ME_EDGERENDER		(1<<5)
+#define ME_LOOSEEDGE		(1<<7)
+#define ME_SEAM_LAST		(1<<8)
+#define ME_EDGE_STEPINDEX	(1<<15)
 
 /* puno = vertexnormal (mface) */
 #define ME_FLIPV1		1
@@ -100,9 +109,13 @@ typedef struct MSticky {
 #define ME_V4V1			8
 
 /* flag (mface) */
-#define ME_SMOOTH		1
-#define ME_FACE_SEL		2
-						/* flag ME_HIDE is used here too */ 
-
+#define ME_SMOOTH			1
+#define ME_FACE_SEL			2
+						/* flag ME_HIDE==16 is used here too */ 
+#define ME_FACE_STEPINDEX	(1<<7)
 
 #endif
+/* mselect->type */
+#define ME_VSEl	0
+#define ME_ESEl 1
+#define ME_FSEL 2
