@@ -1,10 +1,5 @@
 /**
- * blenlib/DNA_scene_types.h (mar-2001 nzc)
- *
- * Renderrecipe and scene decription. The fact that there is a
- * hierarchy here is a bit strange, and not desirable.
- *
- * $Id: DNA_scene_types.h,v 1.43 2005/05/27 17:52:52 eeshlo Exp $ 
+ * $Id: DNA_scene_types.h,v 1.74 2006/07/02 18:26:20 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -53,6 +48,7 @@ struct World;
 struct Scene;
 struct Image;
 struct Group;
+struct bNodeTree;
 
 typedef struct Base {
 	struct Base *next, *prev;
@@ -63,10 +59,10 @@ typedef struct Base {
 } Base;
 
 typedef struct AviCodecData {
-	void			*lpFormat;			/* save format */
-	void			*lpParms;			/* compressor options */
-	unsigned int	cbFormat;			/* size of lpFormat buffer */
-	unsigned int	cbParms;			/* size of lpParms buffer */
+	void			*lpFormat;  /* save format */
+	void			*lpParms;   /* compressor options */
+	unsigned int	cbFormat;	    /* size of lpFormat buffer */
+	unsigned int	cbParms;	    /* size of lpParms buffer */
 
 	unsigned int	fccType;            /* stream type, for consistency */
 	unsigned int	fccHandler;         /* compressor */
@@ -82,15 +78,32 @@ typedef struct AviCodecData {
 
 typedef struct QuicktimeCodecData {
 
-	void			*cdParms;			/* codec/compressor options */
-	void			*pad;				/* padding */
+	void			*cdParms;   /* codec/compressor options */
+	void			*pad;	    /* padding */
 
-	unsigned int	cdSize;				/* size of cdParms buffer */
-	unsigned int	pad2;				/* padding */
+	unsigned int	cdSize;		    /* size of cdParms buffer */
+	unsigned int	pad2;		    /* padding */
 
 	char			qtcodecname[128];
 } QuicktimeCodecData;
 
+typedef struct FFMpegCodecData {
+        int type;
+        int codec;
+        int audio_codec;
+        int video_bitrate;
+        int audio_bitrate;
+        int gop_size;
+        int flags;
+
+	int rc_min_rate;
+	int rc_max_rate;
+	int rc_buffer_size;
+	int mux_packet_size;
+	int mux_rate;
+} FFMpegCodecData;
+
+
 typedef struct AudioData {
 	int mixrate;
 	float main;		/* Main mix in dB */
@@ -98,12 +111,46 @@ typedef struct AudioData {
 	short pad[3];
 } AudioData;
 
+typedef struct SceneRenderLayer {
+	struct SceneRenderLayer *next, *prev;
+	
+	char name[32];
+	struct Scene *scene;	/* unused still */
+	unsigned int lay;		/* scene->lay itself has priority over this */
+	short layflag;
+	short passflag;
+} SceneRenderLayer;
+
+/* srl->layflag */
+#define SCE_LAY_SOLID	1
+#define SCE_LAY_ZTRA	2
+#define SCE_LAY_HALO	4
+#define SCE_LAY_EDGE	8
+#define SCE_LAY_SKY		16
+#define SCE_LAY_STRAND	32
+
+/* srl->passflag */
+#define SCE_PASS_COMBINED	1
+#define SCE_PASS_Z			2
+#define SCE_PASS_RGBA		4
+#define SCE_PASS_DIFFUSE	8
+#define SCE_PASS_SPEC		16
+#define SCE_PASS_SHADOW		32
+#define SCE_PASS_AO			64
+#define SCE_PASS_RAY		128
+#define SCE_PASS_NORMAL		256
+#define SCE_PASS_VECTOR		512
+
 typedef struct RenderData {
 	struct AviCodecData *avicodecdata;
 	struct QuicktimeCodecData *qtcodecdata;
+        struct FFMpegCodecData ffcodecdata;
+
+	int cfra, sfra, efra;	/* fames as in 'images' */
+
+	int images, framapto, pad3;
+	short flag, pad1;
 
-	short cfra, sfra, efra;	/* fames as in 'images' */
-	short images, framapto, flag;
 	float ctime;			/* use for calcutions */
 	float framelen, blurfac;
 
@@ -111,14 +158,13 @@ typedef struct RenderData {
 	float edgeR, edgeG, edgeB;
 	
 	short fullscreen, xplay, yplay, freqplay;	/* standalone player */
-	short depth, attrib, rt1, rt2;				/* standalone player */
+	short depth, attrib, rt1, rt2;			/* standalone player */
 
-	short stereomode;					/* standalone player stereo settings */
+	short stereomode;	/* standalone player stereo settings */
 	
 	short dimensionspreset;		/* for the dimensions presets menu */
  	
- 	short pad[2];
-
+ 	short filtertype;	/* filter is box, tent, gauss, mitch, etc */
 
 	short size, maximsize;	/* size in %, max in Kb */
 	/* from buttons: */
@@ -146,14 +192,14 @@ typedef struct RenderData {
 	 * The number of part to use in the y direction
 	 */
 	short yparts;
-	/* should rewrite this I think... */
-	rctf safety, border;
         
-	short winpos, planes, imtype;
+	short winpos, planes, imtype, subimtype;
+	
 	/** Mode bits:                                                           */
 	/* 0: Enable backbuffering for images                                    */
 	short bufflag;
  	short quality;
+	
 	/**
 	 * Flags for render settings. Use bit-masking to access the settings.
 	 * 0: enable sequence output rendering                                   
@@ -210,6 +256,15 @@ typedef struct RenderData {
 	 * identical materials with this number.*/
 	short same_mat_redux;
 	
+	/* safety, border and display rect */
+	rctf safety, border;
+	rcti disprect;
+	
+	/* information on different layers to be rendered */
+	ListBase layers;
+	short actlay, pad;
+	int pad2;
+	
 	/**
 	 * The gamma for the normal rendering. Used when doing
 	 * oversampling, to correctly blend subpixels to pixels.  */
@@ -234,7 +289,7 @@ typedef struct RenderData {
 	float YF_gamma, YF_exposure, YF_raybias, YF_AApixelsize, YF_AAthreshold;
 
 	char backbuf[160], pic[160], ftype[160];
-	
+
 } RenderData;
 
 
@@ -254,6 +309,43 @@ typedef struct TimeMarker {
 	unsigned int flag;
 } TimeMarker;
 
+typedef struct ToolSettings {
+	/* Subdivide Settings */
+	short cornertype;
+	short editbutflag;
+	/* Extrude Tools */
+	short degr; 
+	short step;
+	short turn; 
+
+	short pad1;
+	
+	float extr_offs; 
+	float doublimit;
+	
+	/* Primitive Settings */
+	/* UV Sphere */
+	short segments;
+	short rings;
+	
+	/* Cylinder - Tube - Circle */
+	short vertices;
+
+	/* UV Calculation */
+	short unwrapper;
+	float uvcalc_radius;
+	float uvcalc_cubesize;
+	short uvcalc_mapdir;
+	short uvcalc_mapalign;
+	short uvcalc_flag;
+
+	short pad2;
+	
+	/* Select Group Threshold */
+	float select_thresh;
+
+} ToolSettings;
+
 typedef struct Scene {
 	ID id;
 	struct Object *camera;
@@ -264,7 +356,6 @@ typedef struct Scene {
 	
 	ListBase base;
 	struct Base *basact;
-	struct Group *group;
 	
 	float cursor[3];
 	float twcent[3];			/* center for transform widget */
@@ -272,9 +363,12 @@ typedef struct Scene {
 	unsigned int lay;
 	
 	/* editmode stuff */
-	short selectmode, pad;
-	short proportional, prop_mode;
 	float editbutsize;                      /* size of normals */
+	short selectmode;
+	short proportional, prop_mode;
+	
+	short use_nodes;
+	struct bNodeTree *nodetree;	
 	
 	void *ed;
 	struct Radio *radio;
@@ -282,6 +376,8 @@ typedef struct Scene {
 	
 	struct GameFraming framing;
 
+	struct ToolSettings *toolsettings;
+
 	/* migrate or replace? depends on some internal things... */
 	/* no, is on the right place (ton) */
 	struct RenderData r;
@@ -291,10 +387,13 @@ typedef struct Scene {
 	
 	ListBase markers;
 	
+	short jumpframe;
+	short pad1, pad2, pad3;
+	
 	/* none of the dependancy graph  vars is mean to be saved */
 	struct  DagForest *theDag;
 	short dagisvalid, dagflags;
-	int pad1;
+	short pad4, recalc;				/* recalc = counterpart of ob->recalc */
 } Scene;
 
 
@@ -318,29 +417,47 @@ typedef struct Scene {
 #define R_RADIO			0x0100
 #define R_BORDER		0x0200
 #define R_PANORAMA		0x0400
-#define R_MOVIECROP		0x0800
+#define R_CROP			0x0800
 #define R_COSMO			0x1000
 #define R_ODDFIELD		0x2000
 #define R_MBLUR			0x4000
 #define R_UNIFIED       0x8000
 #define R_RAYTRACE      0x10000
+		/* R_GAUSS is obsolete, but used to retrieve setting from old files */
 #define R_GAUSS      	0x20000
+		/* fbuf obsolete... */
 #define R_FBUF			0x40000
 #define R_THREADS		0x80000
-#define R_ZBLUR			0x100000
+#define R_SPEED			0x100000
 
+/* filtertype */
+#define R_FILTER_BOX	0
+#define R_FILTER_TENT	1
+#define R_FILTER_QUAD	2
+#define R_FILTER_CUBIC	3
+#define R_FILTER_CATROM	4
+#define R_FILTER_GAUSS	5
+#define R_FILTER_MITCH	6
 
 /* yafray: renderer flag (not only exclusive to yafray) */
 #define R_INTERN	0
 #define R_YAFRAY	1
 
 /* scemode */
-#define R_DOSEQ			0x0001
-#define R_BG_RENDER		0x0002
-#define R_PASSEPARTOUT	0x0004
-
-#define R_EXTENSION		0x0010
-#define R_OGL			0x0020
+#define R_DOSEQ				0x0001
+#define R_BG_RENDER			0x0002
+		/* passepartout is camera option now, keep this for backward compatibility */
+#define R_PASSEPARTOUT		0x0004
+#define R_PREVIEWBUTS		0x0008
+#define R_EXTENSION			0x0010
+#define R_NODE_PREVIEW		0x0020
+#define R_DOCOMP			0x0040
+#define R_COMP_CROP			0x0080
+#define R_FREE_IMAGE		0x0100
+#define R_SINGLE_LAYER		0x0200
+#define R_EXR_TILE_FILE		0x0400
+#define R_COMP_FREE			0x0800
+#define R_NO_IMAGE_LOAD		0x1000
 
 /* alphamode */
 #define R_ADDSKY		0
@@ -367,28 +484,19 @@ typedef struct Scene {
 #define R_AVICODEC	18
 #define R_QUICKTIME 	19
 #define R_BMP		20
+#define R_RADHDR	21
+#define R_TIFF		22
+#define R_OPENEXR	23
+#define R_FFMPEG        24
+#define R_FRAMESERVER   25
+#define R_CINEON	26
+#define R_DPX		27
 
-/* **************** RENDER ********************* */
-/* mode flag is same as for renderdata */
-/* flag */
-#define R_ZTRA			1
-#define R_HALO			2
-#define R_SEC_FIELD		4
-#define R_LAMPHALO		8
-#define R_RENDERING		16
-#define R_ANIMRENDER	32
-#define R_REDRAW_PRV	64
+/* subimtype, flag options for imtype */
+#define R_OPENEXR_HALF	1
+#define R_OPENEXR_ZBUF	2
+#define R_PREVIEW_JPG	4
 
-/* vlakren->flag (vlak = face in dutch) char!!! */
-#define R_SMOOTH		1
-#define R_VISIBLE		2
-#define R_NOPUNOFLIP	8
-#define R_FULL_OSA		16
-#define R_FACE_SPLIT	32
-#define R_DIVIDE_24		64 /* Tells render to divide face other way. */
-
-/* vertren->texofs (texcoordinate offset relative to vertren->orco */
-#define R_UVOFS3	1
 
 /* **************** SCENE ********************* */
 #define RAD_PHASE_PATCHES	1
@@ -404,6 +512,9 @@ typedef struct Scene {
 #define SCE_SELECT_EDGE		2
 #define SCE_SELECT_FACE		4
 
+/* sce->recalc (now in use by previewrender) */
+#define SCE_PRV_CHANGED		1
+
 /* sce->prop_mode (proportional falloff) */
 #define PROP_SMOOTH            0
 #define PROP_SPHERE            1
@@ -423,6 +534,8 @@ typedef struct Scene {
 #define AUDIO_SYNC		2
 #define AUDIO_SCRUB		4
 
+#define FFMPEG_MULTIPLEX_AUDIO  1
+#define FFMPEG_AUTOSPLIT_OUTPUT 2
 
 #ifdef __cplusplus
 }
