@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_sequence_types.h (mar-2001 nzc)
- *	
- * $Id: DNA_sequence_types.h,v 1.7 2003/07/13 20:16:49 larstiq Exp $ 
+ *
+ * $Id: DNA_sequence_types.h,v 1.22 2006/03/26 21:36:42 schlaile Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,45 +42,49 @@
 struct Ipo;
 struct Scene;
 
+/* strlens; 80= FILE_MAXFILE, 160= FILE_MAXDIR */
+
 typedef struct StripElem {
-	char name[40];
+	char name[80];
 	struct ImBuf *ibuf;
 	struct StripElem *se1, *se2, *se3;
-	short ok, nr;
-	int pad;
-		
+	short ok;
+	unsigned char isneeded;
+	unsigned char pad;
+	int nr;
 } StripElem;
 
 typedef struct Strip {
 	struct Strip *next, *prev;
-	short rt, len, us, done;
+	int rt, len, us, done;
 	StripElem *stripdata;
-	char dir[80];
-	short orx, ory;
-	int pad;
-		
+	char dir[160];
+	int orx, ory;
 } Strip;
 
 
 typedef struct PluginSeq {
-	char name[80];
+	char name[256];
 	void *handle;
-	
+
 	char *pname;
-	
+
 	int vars, version;
 
 	void *varstr;
 	float *cfra;
-	
+
 	float data[32];
 
+	void *instance_private_data;
+	void **current_private_data;
+
 	void (*doit)(void);
 
 	void (*callback)(void);
 } PluginSeq;
 
-
+/* The sequence structure is the basic struct used by any strip. each of the strips uses a different sequence structure.*/
 /* WATCH IT: first part identical to ID (for use in ipo's) */
 
 typedef struct Sequence {
@@ -88,40 +92,44 @@ typedef struct Sequence {
 	struct Sequence *next, *prev, *newseq;
 	void *lib;
 	char name[24];
-	
-	short flag, type;
+
+	short flag, type;	/*flags bitmap (see below) and the type of sequence*/
 	int len;
 	int start, startofs, endofs;
 	int startstill, endstill;
 	int machine, depth;
-	int startdisp, enddisp;
+	int startdisp, enddisp;	/*starting and ending points in the sequence*/
 	float mul, handsize;
-	int sfra;
-	
+	int sfra;		/* starting frame according to the timeline of the scene */
+
 	Strip *strip;
 	StripElem *curelem;
-	
+
 	struct Ipo *ipo;
 	struct Scene *scene;
 	struct anim *anim;
 	float facf0, facf1;
-	
+
 	PluginSeq *plugin;
 
 	/* pointers for effects: */
 	struct Sequence *seq1, *seq2, *seq3;
-	
+
 	/* meta */
 	ListBase seqbase;
-	
+
 	struct bSound *sound;	/* the linked "bSound" object */
-	float level, pan;		/* level in dB (0=full), pan -1..1 */
-	int curpos;				/* last sample position in audio_fill() */
+        struct hdaudio *hdaudio; /* external hdaudio object */
+	float level, pan;	/* level in dB (0=full), pan -1..1 */
+	int curpos;		/* last sample position in audio_fill() */
+	float strobe;
+
+	void *effectdata;	/* Struct pointer for effect settings */
+
+	int anim_preseek;
 	int pad;
-	
 } Sequence;
-#
-#
+
 typedef struct MetaStack {
 	struct MetaStack *next, *prev;
 	ListBase *oldbasep;
@@ -132,10 +140,27 @@ typedef struct Editing {
 	ListBase *seqbasep;
 	ListBase seqbase;
 	ListBase metastack;
-	short flag, rt;
-	int pad;
+	short flag;
+	short pad;
+	int rt;
 } Editing;
 
+/* ************* Effect Variable Structs ********* */
+typedef struct WipeVars {
+	float edgeWidth,angle;
+	short forward, wipetype;
+} WipeVars;
+
+typedef struct GlowVars {	
+	float fMini;	/*	Minimum intensity to trigger a glow */
+	float fClamp;
+	float fBoost;	/*	Amount to multiply glow intensity */
+    float dDist;	/*	Radius of glow blurring */
+	int	dQuality;
+	int	bNoComp;	/*	SHOW/HIDE glow buffer */
+} GlowVars;
+
+
 /* ***************** SEQUENCE ****************** */
 
 /* seq->flag */
@@ -144,13 +169,19 @@ typedef struct Editing {
 #define SEQ_OVERLAP		8
 #define SEQ_FILTERY		16
 #define SEQ_MUTE		32
+#define SEQ_MAKE_PREMUL	64
+#define SEQ_REVERSE_FRAMES	128
+#define SEQ_IPO_FRAME_LOCKED    256
+#define SEQ_EFFECT_NOT_LOADED   512
 
-/* seq->type WATCH IT: BIT 3!!! */
+/* seq->type WATCH IT: SEQ_EFFECT BIT is used to determine if this is an effect strip!!! */
 #define SEQ_IMAGE		0
 #define SEQ_META		1
 #define SEQ_SCENE		2
 #define SEQ_MOVIE		3
-#define SEQ_SOUND		4
+#define SEQ_RAM_SOUND		4
+#define SEQ_HD_SOUND            5
+#define SEQ_MOVIE_AND_HD_SOUND  6 /* helper for add_sequence */
 
 #define SEQ_EFFECT		8
 #define SEQ_CROSS		8
@@ -162,6 +193,8 @@ typedef struct Editing {
 #define SEQ_MUL			14
 #define SEQ_OVERDROP	15
 #define SEQ_PLUGIN		24
+#define SEQ_WIPE		25
+#define SEQ_GLOW		26
 
 #endif
 
