@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_space_types.h (mar-2001 nzc)
  *	
- * $Id: DNA_space_types.h,v 1.30 2005/05/19 19:58:53 ton Exp $ 
+ * $Id: DNA_space_types.h,v 1.46 2006/07/13 13:19:27 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,7 +48,9 @@ struct Image;
 struct SpaceIpo;
 struct BlendHandle;
 struct TreeStore;
-
+struct RenderInfo;
+struct bNodeTree;
+struct uiBlock;
 
 	/**
 	 * The base structure all the other spaces
@@ -86,8 +88,11 @@ typedef struct SpaceIpo {
 	
 	void *editipo;
 	ListBase ipokey;
+	
+	/* the ipo context we need to store */
 	struct Ipo *ipo;
 	struct ID *from;
+	char actname[32], constname[32];
 
 	short totipo, pin;
 	short butofs, channel;
@@ -103,6 +108,7 @@ typedef struct SpaceButs {
 	int spacetype;
 	float blockscale;
 	struct ScrArea *area;
+	struct RenderInfo *ri;
 
 	short blockhandler[8];
 
@@ -116,15 +122,14 @@ typedef struct SpaceButs {
 	
 	short texnr;
 	char texfrom, showgroup;
-
-	short rectx, recty;		/* preview render */
-	unsigned int *rect;
-	short cury, modeltype;
-
+	
+	short modeltype;
 	short scriptblock;
 	short scaflag;
-	short re_align, pad1;
-	int pad2;
+	short re_align;
+	
+	short oldkeypress;		/* for keeping track of the sub tab key cycling */
+	char pad, flag;
 	
 	char texact, tab[7];	/* storing tabs for each context */
 		
@@ -140,9 +145,12 @@ typedef struct SpaceSeq {
 
 	View2D v2d;
 	
+	float xof, yof;	/* offset for drawing the image preview */
 	short mainb, zoom;
-	int pad2;
-	
+	short chanshown;
+	short pad2;
+	int flag;
+	int pad;
 } SpaceSeq;
 
 typedef struct SpaceFile {
@@ -214,12 +222,18 @@ typedef struct SpaceImage {
 	View2D v2d;
 	
 	struct Image *image;
-	float zoom;
-	short mode, pin;
+	struct CurveMapping *cumap;
+	short mode, menunr;
 	short imanr, curtile;
-	float xof, yof;
-	short flag, lock;
+	int flag;
+	short pad1, lock;
 	
+	float zoom, pad2;
+	
+	float xof, yof;			/* user defined offset, image is centered */
+	
+	float centx, centy;		/* storage for offset while render drawing */
+	char *info_str;			/* info string for render */
 } SpaceImage;
 
 typedef struct SpaceNla{
@@ -289,7 +303,26 @@ typedef struct SpaceTime {
 	
 } SpaceTime;
 
+typedef struct SpaceNode {
+	SpaceLink *next, *prev;
+	int spacetype;
+	float blockscale;
+	struct ScrArea *area;
+	
+	View2D v2d;
+	
+	struct ID *id, *from;		/* context, no need to save in file? well... pinning... */
+	short flag, menunr;			/* menunr: browse id block in header */
+	float aspect;
+	void *curfont;
+	
+	struct bNodeTree *nodetree, *edittree;
+	int treetype, pad;			/* treetype: as same nodetree->type */
+	
+} SpaceNode;
 
+/* snode->flag */
+#define SNODE_DO_PREVIEW	1
 
 #
 #
@@ -380,28 +413,12 @@ typedef struct SpaceImaSel {
 /* **************** SPACE ********************* */
 
 
-/* view3d->flag */ /* Now in DNA_view3d_types.h */
-/*
-#define V3D_DISPIMAGE		1
-#define V3D_DISPBGPIC		2
-#define V3D_SETUPBUTS		4
-#define V3D_NEEDBACKBUFDRAW	8
-#define V3D_MODE			(16+32+64+128)
-#define V3D_EDITMODE		16
-#define V3D_VERTEXPAINT		32
-#define V3D_FACESELECT		64
-#define V3D_POSEMODE		128
-*/
+/* view3d  Now in DNA_view3d_types.h */
 
-/* view3d->around */ /* Now in DNA_view3d_types.h */
-/*
-#define V3D_CENTRE		0
-#define V3D_CENTROID	3
-#define V3D_CURSOR		1
-#define V3D_LOCAL		2
-*/
+/* button defines in BIF_butspace.h */
 
-/* buts defines in BIF_butspace.h */
+/* sbuts->flag */
+#define SB_PRV_OSA			1
 
 /* these values need to be hardcoded in blender.h SpaceFile: struct dna does not recognize defines */
 #define FILE_MAXDIR			160
@@ -423,6 +440,7 @@ typedef struct SpaceImaSel {
 #define FILE_AUTOSELECT		16
 #define FILE_ACTIVELAY		32
 #define FILE_ATCURSOR		64
+#define FILE_SYNCPOSE		128
 
 /* sfile->sort */
 #define FILE_SORTALPHA		0
@@ -455,8 +473,17 @@ typedef struct SpaceImaSel {
 #define SI_STICKYUVS    16
 #define SI_DRAWSHADOW   32
 #define SI_SELACTFACE   64
-#define SI_NOPIXELSNAP  128
+#define SI_DEPRECATED	128
 #define SI_LOCALSTICKY  256
+#define SI_COORDFLOATS  512
+#define SI_PIXELSNAP	1024
+#define SI_LIVE_UNWRAP	2048
+#define SI_USE_ALPHA	0x1000
+#define SI_SHOW_ALPHA	0x2000
+#define SI_SHOW_ZBUF	0x4000
+		/* next two for render window dislay */
+#define SI_PREVSPACE	0x8000
+#define SI_FULLWINDOW	0x10000
 
 /* SpaceText flags (moved from DNA_text_types.h) */
 
@@ -486,6 +513,7 @@ typedef struct SpaceImaSel {
 #define OOPS_LAY	1024
 #define OOPS_LI		2048
 #define OOPS_IM		4096
+#define OOPS_AR		8192
 
 /* SpaceOops->outlinevis */
 #define SO_ALL_SCENES	0
@@ -494,6 +522,7 @@ typedef struct SpaceImaSel {
 #define SO_SELECTED		3
 #define SO_ACTIVE		4
 #define SO_SAME_TYPE	5
+#define SO_GROUPS		6
 
 /* SpaceOops->storeflag */
 #define SO_TREESTORE_CLEANUP	1
@@ -544,6 +573,15 @@ typedef struct SpaceImaSel {
 #define TIME_ALL_ANIM_WIN		4
 #define TIME_ALL_BUTS_WIN		8
 #define TIME_WITH_SEQ_AUDIO		16
+#define TIME_SEQ                        32
 
+/* sseq->mainb */
+#define SEQ_DRAW_SEQUENCE         0
+#define SEQ_DRAW_IMG_IMBUF        1
+#define SEQ_DRAW_IMG_WAVEFORM     2
+#define SEQ_DRAW_IMG_VECTORSCOPE  3
+
+/* sseq->flag */
+#define SEQ_DRAWFRAMES  1
 
 #endif
