@@ -1,7 +1,7 @@
 /**
  * blenkernel/DNA_userdef_types.h (mar-2001 nzc)
  *
- *	$Id: DNA_userdef_types.h,v 1.22 2003/11/14 15:49:25 ton Exp $
+ *	$Id: DNA_userdef_types.h,v 1.56 2006/06/08 19:05:58 schlaile Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,6 +40,7 @@
 // global, button colors
 
 typedef struct ThemeUI {
+	char outline[4];
 	char neutral[4];
 	char action[4];
 	char setting[4];
@@ -47,6 +48,7 @@ typedef struct ThemeUI {
 	char setting2[4];
 	char num[4];
 	char textfield[4];
+	char textfield_hi[4];
 	char popup[4];
 	char text[4];
 	char text_hi[4];
@@ -55,13 +57,13 @@ typedef struct ThemeUI {
 	char menu_hilite[4];
 	char menu_text[4];
 	char menu_text_hi[4];
-
-	char but_drawtype, pad;
-	short pad1;
+	
+	char but_drawtype;
+	char pad1[3];
 
 } ThemeUI;
 
-// try to put them all in one, if needed a spacial struct can be created as well
+// try to put them all in one, if needed a special struct can be created as well
 // for example later on, when we introduce wire colors for ob types or so...
 typedef struct ThemeSpace {
 	char back[4];
@@ -77,13 +79,25 @@ typedef struct ThemeSpace {
 	char grid[4]; 
 	
 	char wire[4], select[4];
-	char active[4], transform[4];
+	char lamp[4];
+	char active[4], group[4], group_active[4], transform[4];
 	char vertex[4], vertex_select[4];
 	char edge[4], edge_select[4];
-	char face[4], face_select[4];
+	char edge_seam[4], edge_facesel[4];
+	char face[4], face_select[4];	// solid faces
+	char face_dot[4];				// selected color
+	char normal[4];
+	char bone_solid[4], bone_pose[4];
+	char strip[4], strip_select[4];
 	
-	char vertex_size, pad;
-	short pad1;
+	char vertex_size, facedot_size;
+	char bpad[2];
+
+	char syntaxl[4], syntaxn[4], syntaxb[4]; // syntax for textwindow and nodes
+	char syntaxv[4], syntaxc[4];
+	
+	char movie[4], image[4], scene[4], audio[4];		// for sequence editor
+	char effect[4], plugin[4], transition[4], meta[4];
 	
 } ThemeSpace;
 
@@ -107,6 +121,10 @@ typedef struct bTheme {
 	ThemeSpace timasel;
 	ThemeSpace text;
 	ThemeSpace toops;
+	ThemeSpace ttime;
+	ThemeSpace tnode;
+
+	unsigned char bpad[4], bpad1[4];
 	
 } bTheme;
 
@@ -116,7 +134,7 @@ typedef struct SolidLight {
 } SolidLight;
 
 typedef struct UserDef {
-	short flag, dupflag;
+	int flag, dupflag;
 	int savetime;
 	char tempdir[160];	// FILE_MAXDIR length
 	char fontdir[160];
@@ -126,11 +144,13 @@ typedef struct UserDef {
 	char plugseqdir[160];
 	char pythondir[160];
 	char sounddir[160];
+	/* yafray: temporary xml export directory */
+	char yfexportdir[160];
 	short versions, vrmlflag;	// tmp for export, will be replaced by strubi
 	int gameflags;
 	int wheellinescroll;
-	short uiflag, language;
-	int userpref;
+	int uiflag, language;
+	short userpref, viewzoom;
 	short console_buffer;	//console vars here for tuhopuu compat, --phase
 	short console_out;
 	int mixbufsize;
@@ -140,9 +160,19 @@ typedef struct UserDef {
 	short menuthreshold1, menuthreshold2;
 	char fontname[256];		// FILE_MAXDIR+FILE length
 	struct ListBase themes;
-	short undosteps, pad0;
+	short undosteps;
+	short curssize;
 	short tb_leftmouse, tb_rightmouse;
 	struct SolidLight light[3];
+	short tw_hotspot, tw_flag, tw_handlesize, tw_size;
+	int textimeout, texcollectrate;
+        int memcachelimit;
+        short frameserverport;
+	short pad;
+	short obcenter_dia;
+	short rvisize;		/* rotating view icon size */
+	short rvibright;	/* rotating view icon brightness */
+	short pad1;
 } UserDef;
 
 extern UserDef U; /* from usiblender.c !!!! */
@@ -150,72 +180,89 @@ extern UserDef U; /* from usiblender.c !!!! */
 /* ***************** USERDEF ****************** */
 
 /* flag */
-#define AUTOSAVE		1
-#define AUTOGRABGRID	2
-#define AUTOROTGRID		4
-#define AUTOSIZEGRID	8
-#define SCENEGLOBAL		16
-#define TRACKBALL		32
-#define DUPLILINK		64
-#define FSCOLLUM		128
-#define MAT_ON_OB		256
-#define NO_CAPSLOCK		512
-#define VIEWMOVE		1024
-#define TOOLTIPS		2048
-#define TWOBUTTONMOUSE	4096
-#define NONUMPAD		8192
+#define USER_AUTOSAVE			1
+#define USER_AUTOGRABGRID		2
+#define USER_AUTOROTGRID		4
+#define USER_AUTOSIZEGRID		8
+#define USER_SCENEGLOBAL		16
+#define USER_TRACKBALL			32
+#define USER_DUPLILINK			64
+#define USER_FSCOLLUM			128
+#define USER_MAT_ON_OB			256
+#define USER_NO_CAPSLOCK		512
+#define USER_VIEWMOVE			1024
+#define USER_TOOLTIPS			2048
+#define USER_TWOBUTTONMOUSE		4096
+#define USER_NONUMPAD			8192
+#define USER_LMOUSESELECT		16384
+#define USER_FILECOMPRESS		32768
+
+/* viewzom */
+#define USER_ZOOM_CONT			0
+#define USER_ZOOM_SCALE			1
+#define USER_ZOOM_DOLLY			2
 
 /* uiflag */
 
-#define	KEYINSERTACT	1
-#define	KEYINSERTOBJ	2
-#define WHEELZOOMDIR	4
-#define FILTERFILEEXTS	8
-#define DRAWVIEWINFO	16
-#define EVTTOCONSOLE	32		// print ghost events, here for tuhopuu compat. --phase
+#define	USER_KEYINSERTACT		1
+#define	USER_KEYINSERTOBJ		2
+#define USER_WHEELZOOMDIR		4
+#define USER_FILTERFILEEXTS		8
+#define USER_DRAWVIEWINFO		16
+#define USER_PLAINMENUS			32		// old EVTTOCONSOLE print ghost events, here for tuhopuu compat. --phase
 								// old flag for hide pulldown was here 
-#define FLIPFULLSCREEN	128
-#define ALLWINCODECS	256
-#define MENUOPENAUTO	512
+#define USER_FLIPFULLSCREEN		128
+#define USER_ALLWINCODECS		256
+#define USER_MENUOPENAUTO		512
+#define USER_PANELPINNED		1024
+#define USER_AUTOPERSP     		2048
+#define USER_LOCKAROUND     	4096
+#define USER_GLOBALUNDO     	8192
+#define USER_ORBIT_SELECTION	16384
+#define USER_KEYINSERTAVAI		32768
+#define USER_HIDE_DOT			65536
+#define USER_SHOW_ROTVIEWICON	131072
+#define USER_SHOW_VIEWPORTNAME	262144
 
 /* transopts */
 
-#define	TR_TOOLTIPS		1
-#define	TR_BUTTONS		2
-#define TR_MENUS		4
-#define TR_FILESELECT	8
-#define TR_TEXTEDIT		16
-#define TR_ALL			32
+#define	USER_TR_TOOLTIPS		1
+#define	USER_TR_BUTTONS			2
+#define USER_TR_MENUS			4
+#define USER_TR_FILESELECT		8
+#define USER_TR_TEXTEDIT		16
+#define USER_DOTRANSLATE		32
+#define USER_USETEXTUREFONT		64
 
 /* dupflag */
 
-#define DUPMESH			1
-#define DUPCURVE		2
-#define DUPSURF			4
-#define DUPFONT			8
-#define DUPMBALL		16
-#define DUPLAMP			32
-#define DUPIPO			64
-#define DUPMAT			128
-#define DUPTEX			256
-#define	DUPARM			512
-#define	DUPACT			1024
+#define USER_DUP_MESH			1
+#define USER_DUP_CURVE			2
+#define USER_DUP_SURF			4
+#define USER_DUP_FONT			8
+#define USER_DUP_MBALL			16
+#define USER_DUP_LAMP			32
+#define USER_DUP_IPO			64
+#define USER_DUP_MAT			128
+#define USER_DUP_TEX			256
+#define	USER_DUP_ARM			512
+#define	USER_DUP_ACT			1024
 
 /* gameflags */
 
-#define USERDEF_VERTEX_ARRAYS_BIT        0
-#define USERDEF_DISABLE_SOUND_BIT        1
-#define USERDEF_DISABLE_MIPMAP_BIT       2
+#define USER_VERTEX_ARRAYS		1
+#define USER_DISABLE_SOUND		2
+#define USER_DISABLE_MIPMAP		4
 
-#define USERDEF_VERTEX_ARRAYS        (1 << USERDEF_VERTEX_ARRAYS_BIT)
-#define USERDEF_DISABLE_SOUND        (1 << USERDEF_DISABLE_SOUND_BIT)
-#define USERDEF_DISABLE_MIPMAP       (1 << USERDEF_DISABLE_MIPMAP_BIT)
 
 /* vrml flag */
 
-#define USERDEF_VRML_LAYERS		1
-#define USERDEF_VRML_AUTOSCALE	2
-#define USERDEF_VRML_TWOSIDED	4
+#define USER_VRML_LAYERS		1
+#define USER_VRML_AUTOSCALE		2
+#define USER_VRML_TWOSIDED		4
+
+/* tw_flag (transform widget) */
+
 
 #endif
 
