@@ -1,5 +1,5 @@
 /**
- * $Id: makesdna.c,v 1.14 2005/05/02 13:28:12 ton Exp $
+ * $Id: makesdna.c,v 1.21 2006/01/28 18:33:14 hos Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -49,7 +49,7 @@
  * numbers give more output.
  * */
 
-#define DNA_VERSION_DATE "$Id: makesdna.c,v 1.14 2005/05/02 13:28:12 ton Exp $"
+#define DNA_VERSION_DATE "$Id: makesdna.c,v 1.21 2006/01/28 18:33:14 hos Exp $"
 
 #include <string.h>
 #include <stdlib.h>
@@ -93,9 +93,11 @@ char *includefiles[] = {
 	"DNA_curve_types.h",
 	"DNA_mesh_types.h",
 	"DNA_meshdata_types.h",
+	"DNA_modifier_types.h",
 	"DNA_lattice_types.h",	
 	"DNA_object_types.h",
 	"DNA_object_force.h",
+	"DNA_object_fluidsim.h",
 	"DNA_world_types.h",
 	"DNA_radio_types.h",
 	"DNA_scene_types.h",
@@ -110,7 +112,6 @@ char *includefiles[] = {
 	"DNA_fileglobal_types.h",
 	"DNA_sequence_types.h",
 	"DNA_effect_types.h",
-	"DNA_ika_types.h",
 	"DNA_oops_types.h",
 	"DNA_property_types.h",
 	"DNA_sensor_types.h",
@@ -122,6 +123,8 @@ char *includefiles[] = {
 	"DNA_action_types.h",
 	"DNA_constraint_types.h",
 	"DNA_nla_types.h",
+	"DNA_node_types.h",
+	"DNA_color_types.h",
 	// if you add files here, please add them at the end
 	// of makesdna.c (this file) as well
 
@@ -202,16 +205,6 @@ void dna_write(FILE *file, void *pntr, int size);
  */
 void printStructLenghts(void);
 
-/**
- *
- */ 
-int make_structDNA(FILE *file);
-
-/**
- *
- */ 
-int main(int argc, char ** argv); 
-
 
 
 /* ************************************************************************** */
@@ -550,7 +543,7 @@ int convert_include(char *filename)
 					/* first lets make it all nice strings */
 					md1= md+1;
 					while(*md1 != '}') {
-						if( ((long)md1) > ((long)mainend) ) break;
+						if(md1>mainend) break;
 						
 						if(*md1==',' || *md1==' ') *md1= 0;
 						md1++;
@@ -559,7 +552,7 @@ int convert_include(char *filename)
 					/* read types and names until first character that is not '}' */
 					md1= md+1;
 					while( *md1 != '}' ) {
-						if( ((long)md1) > ((long)mainend) ) break;
+						if(md1>mainend) break;
 						
 						/* skip when it says 'struct' or 'unsigned' */
 						if(*md1) {
@@ -576,13 +569,13 @@ int convert_include(char *filename)
 							
 							/* read until ';' */
 							while( *md1 != ';' ) {
-								if( ((long)md1) > ((long)mainend) ) break;
+								if(md1>mainend) break;
 								
 								if(*md1) {
 									/* We've got a name. slen needs
 									 * correction for function
 									 * pointers! */
-									slen= strlen(md1);
+									slen= (int) strlen(md1);
 									if( md1[slen-1]==';' ) {
 										md1[slen-1]= 0;
 
@@ -683,7 +676,7 @@ int calculate_structlens(void)
 					type= sp[0];
 					cp= names[sp[1]];
 					
-					namelen= strlen(cp);
+					namelen= (int) strlen(cp);
 					/* is it a pointer or function pointer? */
 					if(cp[0]=='*' || cp[1]=='*') {
 						has_pointer = 1;
@@ -841,7 +834,7 @@ void printStructLenghts(void)
 }
 
 
-int make_structDNA(FILE *file)
+int make_structDNA(char *baseDirectory, FILE *file)
 {
 	int len, i;
 	short *sp;
@@ -884,26 +877,16 @@ int make_structDNA(FILE *file)
 	// the defines above shouldn't be output in the padding file...
 	firststruct = nr_types;
 	
-#ifndef BASE_HEADER
-#define BASE_HEADER "../"
-#endif
-
 	/* add all include files defined in the global array                     */
 	/* Since the internal file+path name buffer has limited length, I do a   */
 	/* little test first...                                                  */
 	/* Mind the breaking condition here!                                     */
 	if (debugSDNA) printf("\tStart of header scan:\n"); 
 	for (i = 0; strlen(includefiles[i]); i++) {
-  		if (debugSDNA) printf("\t|-- Converting %s%s\n", BASE_HEADER, includefiles[i]); 
-		if (strlen(includefiles[i]) > SDNA_MAX_FILENAME_LENGTH) {
-			/* this would cause coredumps*/
-			printf("*** \tError in makesdna: the specified filenames is too long "
-				   "for parsing.\n\tFile: %s%s\n", BASE_HEADER, includefiles[i]);			
-		} else {
-			sprintf(str, "%s%s", BASE_HEADER, includefiles[i]);
-			if (convert_include(str)) {
-				return (1);
-			}
+		sprintf(str, "%s%s", baseDirectory, includefiles[i]);
+  		if (debugSDNA) printf("\t|-- Converting %s\n", str); 
+		if (convert_include(str)) {
+			return (1);
 		}
 	}
 	if (debugSDNA) printf("\tFinished scanning %d headers.\n", i); 
@@ -962,7 +945,7 @@ int make_structDNA(FILE *file)
 		/* calculate size of datablock with strings */
 		cp= names[nr_names-1];
 		cp+= strlen(names[nr_names-1]) + 1;			/* +1: null-terminator */
-		len= (long)cp - (long)(names[0]);
+		len= (long) (cp - (char*) names[0]);
 		len= (len+3) & ~3;
 		dna_write(file, names[0], len);
 		
@@ -975,7 +958,7 @@ int make_structDNA(FILE *file)
 		/* calculate datablock size */
 		cp= types[nr_types-1];
 		cp+= strlen(types[nr_types-1]) + 1;		/* +1: null-terminator */
-		len= (long)cp - (long)(types[0]);
+		len= (long) (cp - (char*) types[0]);
 		len= (len+3) & ~3;
 		
 		dna_write(file, types[0], len);
@@ -997,7 +980,7 @@ int make_structDNA(FILE *file)
 		/* calc datablock size */
 		sp= structs[nr_structs-1];
 		sp+= 2+ 2*( sp[1] );
-		len= (long)sp - (long)(structs[0]);
+		len= (long) ((char*) sp - (char*) structs[0]);
 		len= (len+3) & ~3;
 		
 		dna_write(file, structs[0], len);
@@ -1055,13 +1038,17 @@ static void make_bad_file(char *file)
 	fclose(fp);
 }
 
+#ifndef BASE_HEADER
+#define BASE_HEADER "../"
+#endif
+
 int main(int argc, char ** argv)
 {
 	FILE *file;
 	int return_status = 0;
 
-	if (argc != 2) {
-		printf("Usage: %s outfile.c\n", argv[0]);
+	if (argc!=2 && argc!=3) {
+		printf("Usage: %s outfile.c [base directory]\n", argv[0]);
 		return_status = 1;
 	} else {
 		file = fopen(argv[1], "w");
@@ -1069,8 +1056,16 @@ int main(int argc, char ** argv)
 			printf ("Unable to open file: %s\n", argv[1]);
 			return_status = 1;
 		} else {
+			char baseDirectory[256];
+
+			if (argc==3) {
+				strcpy(baseDirectory, argv[2]);
+			} else {
+				strcpy(baseDirectory, BASE_HEADER);
+			}
+
 			fprintf (file, "unsigned char DNAstr[]= {\n");
-			if (make_structDNA(file)) {
+			if (make_structDNA(baseDirectory, file)) {
 				// error
 				fclose(file);
 				make_bad_file(argv[1]);
@@ -1108,9 +1103,11 @@ int main(int argc, char ** argv)
 #include "DNA_curve_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_lattice_types.h"	
 #include "DNA_object_types.h"
 #include "DNA_object_force.h"
+#include "DNA_object_fluidsim.h"
 #include "DNA_world_types.h"
 #include "DNA_radio_types.h"
 #include "DNA_scene_types.h"
@@ -1123,7 +1120,6 @@ int main(int argc, char ** argv)
 #include "DNA_fileglobal_types.h"
 #include "DNA_sequence_types.h"
 #include "DNA_effect_types.h"
-#include "DNA_ika_types.h"
 #include "DNA_oops_types.h"
 #include "DNA_property_types.h"
 #include "DNA_sensor_types.h"
@@ -1135,4 +1131,6 @@ int main(int argc, char ** argv)
 #include "DNA_action_types.h"
 #include "DNA_constraint_types.h"
 #include "DNA_nla_types.h"
+#include "DNA_node_types.h"
+#include "DNA_color_types.h"
 /* end of list */
