@@ -1,5 +1,5 @@
 /* 
- * $Id: BGL.c,v 1.13 2005/02/27 20:07:26 stiv Exp $
+ * $Id: BGL.c,v 1.20 2006/04/16 15:28:50 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,13 +35,13 @@
  * implementation.  The BGL submodule "wraps" OpenGL functions and constants,
  * allowing script writers to make OpenGL calls in their Python scripts. */
 
-#include "BGL.h"
+#include "BGL.h" /*This must come first */
 
+#include "MEM_guardedalloc.h"
+#include "gen_utils.h"
 
 static int type_size( int type );
 static Buffer *make_buffer( int type, int ndimensions, int *dimensions );
-static int Buffer_ass_slice( PyObject * self, int begin, int end,
-			     PyObject * seq );
 
 static char Method_Buffer_doc[] =
 	"(type, dimensions, [template]) - Create a new Buffer object\n\n\
@@ -50,7 +50,7 @@ static char Method_Buffer_doc[] =
 [template] - A sequence of matching dimensions to the buffer to be created\n\
   which will be used to initialize the Buffer.\n\n\
 If a template is not passed in all fields will be initialized to 0.\n\n\
-The type should be one of GL_BYTE, GL_SHORT, GL_INT, or GL_FLOAT.\n\
+The type should be one of GL_BYTE, GL_SHORT, GL_INT, GL_FLOAT, or GL_DOUBLE.\n\
 If the dimensions are specified as an int a linear buffer will be\n\
 created. If a sequence is passed for the dimensions the buffer\n\
 will have len(sequence) dimensions, where the size for each dimension\n\
@@ -138,6 +138,8 @@ static int type_size(int type)
 		  return sizeof(int);
 		case GL_FLOAT: 
 		  return sizeof(float);
+		case GL_DOUBLE:
+		  return sizeof(double);
 	}
 	return -1;
 }
@@ -171,7 +173,9 @@ static Buffer *make_buffer(int type, int ndimensions, int *dimensions)
 		else if (type==GL_INT) 
 			buffer->buf.asint[i]= 0;
 		else if (type==GL_FLOAT) 
-		    buffer->buf.asfloat[i]= 0.0;
+		    buffer->buf.asfloat[i]= 0.0f;
+		else if (type==GL_DOUBLE)
+			buffer->buf.asdouble[i]= 0.0;
 	}
 	return buffer;
 }
@@ -188,7 +192,7 @@ static PyObject *Method_Buffer (PyObject *self, PyObject *args)
 	        return EXPP_ReturnPyObjError(PyExc_AttributeError,
 	                        "expected an int and one or two PyObjects");
 
-	if (type!=GL_BYTE && type!=GL_SHORT && type!=GL_INT && type!=GL_FLOAT) {
+	if (type!=GL_BYTE && type!=GL_SHORT && type!=GL_INT && type!=GL_FLOAT && type!=GL_DOUBLE) {
 		PyErr_SetString(PyExc_AttributeError, "type");
 		return NULL;
 	}
@@ -243,6 +247,7 @@ static PyObject *Buffer_item(PyObject *self, int i)
 			case GL_SHORT: return Py_BuildValue("h", buf->buf.asshort[i]);
 			case GL_INT: return Py_BuildValue("i", buf->buf.asint[i]);
 			case GL_FLOAT: return Py_BuildValue("f", buf->buf.asfloat[i]);
+			case GL_DOUBLE: return Py_BuildValue("d", buf->buf.asdouble[i]);
 		}
 	} else {
 		Buffer *newbuf;
@@ -324,6 +329,9 @@ static int Buffer_ass_item(PyObject *self, int i, PyObject *v)
 	} else if (buf->type==GL_FLOAT) {
 		if (!PyArg_Parse(v, "f;Coordinates must be floats", &buf->buf.asfloat[i]))
 			return -1;
+	} else if (buf->type==GL_DOUBLE) {
+		if (!PyArg_Parse(v, "d;Coordinates must be floats", &buf->buf.asdouble[i]))
+			return -1;
 	}
 	return 0;
 }
@@ -413,9 +421,6 @@ static PyObject *Buffer_repr(PyObject *self)
 	return repr;
 }
 
-/* BGL_Wrap defined in BGL.h */
-
-/* #ifndef __APPLE__ */
 
 BGL_Wrap(2, Accum,          void,     (GLenum, GLfloat))
 BGL_Wrap(2, AlphaFunc,      void,     (GLenum, GLclampf))
@@ -749,8 +754,6 @@ BGLU_Wrap(5, PickMatrix,       	void,     	(GLdouble, GLdouble, GLdouble, GLdoub
 BGLU_Wrap(9, Project,			GLint,		(GLdouble, GLdouble, GLdouble, GLdoubleP, GLdoubleP, GLintP, GLdoubleP, GLdoubleP, GLdoubleP))
 BGLU_Wrap(9, UnProject,			GLint,		(GLdouble, GLdouble, GLdouble, GLdoubleP, GLdoubleP, GLintP, GLdoubleP, GLdoubleP, GLdoubleP))
 
-/* #endif */
-
 #undef MethodDef
 #define MethodDef(func) {"gl"#func, Method_##func, METH_VARARGS, "no string"}
 #define MethodDefu(func) {"glu"#func, Method_##func, METH_VARARGS, "no string"}
@@ -1091,11 +1094,11 @@ PyObject *BGL_Init(void)
 
 	buffer_Type.ob_type = &PyType_Type;
 
-#define EXPP_ADDCONST(x) PyDict_SetItemString(dict, #x, PyInt_FromLong(x))
+#define EXPP_ADDCONST(x) EXPP_dict_set_item_str(dict, #x, PyInt_FromLong(x))
 
 /* So, for example:
  * EXPP_ADDCONST(GL_CURRENT_BIT) becomes
- * PyDict_SetItemString(dict, "GL_CURRENT_BIT", PyInt_FromLong(GL_CURRENT_BIT)) */
+ * EXPP_dict_set_item_str(dict, "GL_CURRENT_BIT", PyInt_FromLong(GL_CURRENT_BIT)) */
 
 	EXPP_ADDCONST(GL_CURRENT_BIT);
 	EXPP_ADDCONST(GL_POINT_BIT);
@@ -1443,6 +1446,7 @@ PyObject *BGL_Init(void)
 	EXPP_ADDCONST(GL_INT);
 	EXPP_ADDCONST(GL_UNSIGNED_INT);
 	EXPP_ADDCONST(GL_FLOAT);
+	EXPP_ADDCONST(GL_DOUBLE);
 	EXPP_ADDCONST(GL_2_BYTES);
 	EXPP_ADDCONST(GL_3_BYTES);
 	EXPP_ADDCONST(GL_4_BYTES);
@@ -1578,3 +1582,4 @@ PyObject *BGL_Init(void)
       
 	return mod;
 }
+
