@@ -1,5 +1,5 @@
 /* 
- * $Id: BGL.h,v 1.11 2005/03/19 21:08:11 zuster Exp $
+ * $Id: BGL.h,v 1.13 2005/10/02 17:09:11 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,28 +43,9 @@
 #include <config.h>
 #endif
 
-#include "MEM_guardedalloc.h"
-
-#include "BMF_Api.h"
-
-#include "DNA_screen_types.h"
-#include "DNA_space_types.h"
-#include "DNA_text_types.h"
-
-#include "BKE_global.h"
-
+#include <Python.h>
 #include "BIF_gl.h"
-#include "BIF_screen.h"
-#include "BIF_space.h"
-#include "BIF_interface.h"
-#include "BIF_mywindow.h"
 
-#include "interface.h"
-#include "mydevice.h"		/*@ for all the event constants */
-
-#include "Python.h"
-
-#include "gen_utils.h"
 
 /*@ Buffer Object */
 /*@ For Python access to OpenGL functions requiring a pointer. */
@@ -82,6 +63,7 @@ typedef struct _Buffer {
 		short *asshort;
 		int *asint;
 		float *asfloat;
+		double *asdouble;
 
 		void *asvoid;
 	} buf;
