@@ -1,5 +1,5 @@
 /* 
- * $Id: Curve.c,v 1.28 2005/06/13 19:15:01 stiv Exp $
+ * $Id: Curve.c,v 1.39 2006/04/16 15:28:50 stiv Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -29,25 +29,20 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include <Python.h>
-#include "Curve.h"
-#include <stdio.h>
-
-#include <BLI_arithb.h>
-#include <BLI_blenlib.h>
-#include <BKE_main.h>
-#include <BKE_displist.h>
-#include <BKE_global.h>
-#include <BKE_object.h>
-#include <BKE_library.h>
-#include <BKE_curve.h>
-#include <BKE_utildefines.h>
-#include <MEM_guardedalloc.h>	/* because we wil be mallocing memory */
+#include "Curve.h" /*This must come first*/
 
+#include "BKE_main.h"
+#include "BKE_displist.h"
+#include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_curve.h"
+#include "MEM_guardedalloc.h"	/* because we wil be mallocing memory */
 #include "CurNurb.h"
 #include "Material.h"
 #include "Object.h"
+#include "Key.h"
 #include "gen_utils.h"
+#include "mydevice.h"
 
 
 /*****************************************************************************/
@@ -78,16 +73,11 @@ static PyObject *M_Curve_Get( PyObject * self, PyObject * args );
 /*  Python BPy_Curve instance methods declarations:                          */
 /*****************************************************************************/
 
-PyObject *Curve_getName( BPy_Curve * self );
-PyObject *Curve_setName( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getPathLen( BPy_Curve * self );
 static PyObject *Curve_setPathLen( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getTotcol( BPy_Curve * self );
 static PyObject *Curve_setTotcol( BPy_Curve * self, PyObject * args );
-PyObject *Curve_getMode( BPy_Curve * self );
-PyObject *Curve_setMode( BPy_Curve * self, PyObject * args );
-PyObject *Curve_getBevresol( BPy_Curve * self );
-PyObject *Curve_setBevresol( BPy_Curve * self, PyObject * args );
+#if 0
 PyObject *Curve_getResolu( BPy_Curve * self );
 PyObject *Curve_setResolu( BPy_Curve * self, PyObject * args );
 PyObject *Curve_getResolv( BPy_Curve * self );
@@ -98,6 +88,7 @@ PyObject *Curve_getExt1( BPy_Curve * self );
 PyObject *Curve_setExt1( BPy_Curve * self, PyObject * args );
 PyObject *Curve_getExt2( BPy_Curve * self );
 PyObject *Curve_setExt2( BPy_Curve * self, PyObject * args );
+#endif
 static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getLoc( BPy_Curve * self );
@@ -107,10 +98,10 @@ static PyObject *Curve_setRot( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getSize( BPy_Curve * self );
 static PyObject *Curve_setSize( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getNumCurves( BPy_Curve * self );
+static PyObject *Curve_getKey( BPy_Curve * self );
 static PyObject *Curve_isNurb( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_isCyclic( BPy_Curve * self, PyObject * args);
 static PyObject *Curve_getNumPoints( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getNumPoints( BPy_Curve * self, PyObject * args );
 
 static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args );
@@ -120,14 +111,16 @@ static PyObject *Curve_getMaterials( BPy_Curve * self );
 static PyObject *Curve_getBevOb( BPy_Curve * self );
 static PyObject *Curve_setBevOb( BPy_Curve * self, PyObject * args );
 
+static PyObject *Curve_getTaperOb( BPy_Curve * self );
+static PyObject *Curve_setTaperOb( BPy_Curve * self, PyObject * args );
+
 static PyObject *Curve_getIter( BPy_Curve * self );
 static PyObject *Curve_iterNext( BPy_Curve * self );
 
 PyObject *Curve_getNurb( BPy_Curve * self, int n );
 static int Curve_length( PyInstanceObject * inst );
-void update_displists( void *data );
 
-void makeDispList( Object * ob );
+
 struct chartrans *text_to_curve( Object * ob, int mode );
 
 
@@ -207,6 +200,8 @@ Sets a control point "},
 	 METH_VARARGS, "(3-tuple) - Sets curve size"},
 	{"getNumCurves", ( PyCFunction ) Curve_getNumCurves,
 	 METH_NOARGS, "() - Gets number of curves in Curve"},
+	{"getKey", ( PyCFunction ) Curve_getKey,
+	 METH_NOARGS, "() - Gets curve key"},
 	{"isNurb", ( PyCFunction ) Curve_isNurb,
 	 METH_VARARGS,
 	 "(nothing or integer) - returns 1 if curve is type Nurb, O otherwise."},
@@ -227,6 +222,10 @@ Sets a control point "},
 	 "() - returns Bevel Object assigned to this Curve"},
 	{"setBevOb", ( PyCFunction ) Curve_setBevOb, METH_VARARGS,
 	 "() - assign a Bevel Object to this Curve"},
+	{"getTaperOb", ( PyCFunction ) Curve_getTaperOb, METH_NOARGS,
+	 "() - returns Taper Object assigned to this Curve"},
+	{"setTaperOb", ( PyCFunction ) Curve_setTaperOb, METH_VARARGS,
+	 "() - assign a Taper Object to this Curve"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -240,10 +239,6 @@ static int CurveSetAttr( BPy_Curve * msh, char *name, PyObject * v );
 static PyObject *CurveGetAttr( BPy_Curve * msh, char *name );
 static PyObject *CurveRepr( BPy_Curve * msh );
 
-PyObject *Curve_CreatePyObject( struct Curve *curve );
-int Curve_CheckPyObject( PyObject * py_obj );
-struct Curve *Curve_FromPyObject( PyObject * py_obj );
-
 static PySequenceMethods Curve_as_sequence = {
 	( inquiry ) Curve_length,	/* sq_length   */
 	( binaryfunc ) 0,	/* sq_concat */
@@ -437,7 +432,8 @@ PyObject *Curve_Init( void )
 {
 	PyObject *submodule;
 
-	Curve_Type.ob_type = &PyType_Type;
+	if( PyType_Ready( &Curve_Type) < 0) /* set exception.  -1 is failure */
+		return NULL;
 
 	submodule =
 		Py_InitModule3( "Blender.Curve", M_Curve_methods,
@@ -599,6 +595,7 @@ PyObject *Curve_getResolu( BPy_Curve * self )
 PyObject *Curve_setResolu( BPy_Curve * self, PyObject * args )
 {
 	short value;
+	Nurb *nu;
 
 	if( !PyArg_ParseTuple( args, "h", &value ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
@@ -609,6 +606,10 @@ PyObject *Curve_setResolu( BPy_Curve * self, PyObject * args )
 			"acceptable values are between 128 and 1" ) );
 	self->curve->resolu = value;
 
+	/* propagate the change through all the curves */
+	for ( nu = self->curve->nurb.first; nu; nu = nu->next )
+		nu->resolu = value;
+
 	return EXPP_incr_ret( Py_None );
 }
 
@@ -784,7 +785,7 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 {
 	PyObject *listargs = 0;
 	Nurb *ptrnurb = self->curve->nurb.first;
-	int numcourbe, numpoint, i, j;
+	int numcourbe = 0, numpoint = 0, i, j;
 
 	if( !ptrnurb ) {
 		Py_INCREF( Py_None );
@@ -810,14 +811,14 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 	if( ptrnurb->bp )
 		for( i = 0; i < 4; i++ )
 			ptrnurb->bp[numpoint].vec[i] =
-				PyFloat_AsDouble( PyList_GetItem
+				(float)PyFloat_AsDouble( PyList_GetItem
 						  ( listargs, i ) );
 
 	if( ptrnurb->bezt )
 		for( i = 0; i < 3; i++ )
 			for( j = 0; j < 3; j++ )
 				ptrnurb->bezt[numpoint].vec[i][j] =
-					PyFloat_AsDouble( PyList_GetItem
+					(float)PyFloat_AsDouble( PyList_GetItem
 							  ( listargs,
 							    i * 3 + j ) );
 
@@ -828,6 +829,7 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 {
 	PyObject *liste = PyList_New( 0 );	/* return values */
+	PyObject *item;
 
 	Nurb *ptrnurb;
 	int i, j;
@@ -860,22 +862,18 @@ static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 						"point index out of range" ) );
 
 	if( ptrnurb->bp ) {	/* if we are a nurb curve, you get 4 values */
-		for( i = 0; i < 4; i++ )
-			PyList_Append( liste,
-				       PyFloat_FromDouble( ptrnurb->
-							   bp[numpoint].
-							   vec[i] ) );
-	}
-
-	if( ptrnurb->bezt ) {	/* if we are a bezier, you get 9 values */
+		for( i = 0; i < 4; i++ ) {
+			item = PyFloat_FromDouble( ptrnurb->bp[numpoint].vec[i] );
+			PyList_Append( liste, item );
+			Py_DECREF(item);
+		}
+	} else if( ptrnurb->bezt ) {	/* if we are a bezier, you get 9 values */
 		for( i = 0; i < 3; i++ )
-			for( j = 0; j < 3; j++ )
-				PyList_Append( liste,
-					       PyFloat_FromDouble( ptrnurb->
-								   bezt
-								   [numpoint].
-								   vec[i]
-								   [j] ) );
+			for( j = 0; j < 3; j++ ) {
+				item = PyFloat_FromDouble( ptrnurb->bezt[numpoint].vec[i][j] );
+				PyList_Append( liste, item );
+				Py_DECREF(item);
+			}
 	}
 
 	return liste;
@@ -905,7 +903,7 @@ static PyObject *Curve_setLoc( BPy_Curve * self, PyObject * args )
 			 ( PyExc_TypeError, "expected a list" ) );
 	for( i = 0; i < 3; i++ ) {
 		PyObject *xx = PyList_GetItem( listargs, i );
-		self->curve->loc[i] = PyFloat_AsDouble( xx );
+		self->curve->loc[i] = (float)PyFloat_AsDouble( xx );
 	}
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -935,7 +933,7 @@ static PyObject *Curve_setRot( BPy_Curve * self, PyObject * args )
 			 ( PyExc_TypeError, "expected a list" ) );
 	for( i = 0; i < 3; i++ ) {
 		PyObject *xx = PyList_GetItem( listargs, i );
-		self->curve->rot[i] = PyFloat_AsDouble( xx );
+		self->curve->rot[i] = (float)PyFloat_AsDouble( xx );
 	}
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -964,7 +962,7 @@ static PyObject *Curve_setSize( BPy_Curve * self, PyObject * args )
 			 ( PyExc_TypeError, "expected a list" ) );
 	for( i = 0; i < 3; i++ ) {
 		PyObject *xx = PyList_GetItem( listargs, i );
-		self->curve->size[i] = PyFloat_AsDouble( xx );
+		self->curve->size[i] = (float)PyFloat_AsDouble( xx );
 	}
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -985,7 +983,7 @@ static PyObject *Curve_getNumCurves( BPy_Curve * self )
 	/* get curve */
 	ptrnurb = self->curve->nurb.first;
 	if( ptrnurb ) {		/* we have some nurbs in this curve */
-		while( 1 ) {
+		for(;;) {
 			++num_curves;
 			ptrnurb = ptrnurb->next;
 			if( !ptrnurb )	/* no more curves */
@@ -1003,6 +1001,20 @@ static PyObject *Curve_getNumCurves( BPy_Curve * self )
 					"couldn't get number of curves" ) );
 }
 
+/*
+ * get the key object linked to this curve
+ */
+
+static PyObject *Curve_getKey( BPy_Curve * self )
+{
+	PyObject *keyObj;
+
+	if (self->curve->key)
+		keyObj = Key_CreatePyObject(self->curve->key);
+	else keyObj = EXPP_incr_ret(Py_None);
+
+	return keyObj;
+}
 
 /*
  * count the number of points in a given spline
@@ -1263,11 +1275,9 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 
 PyObject *Curve_update( BPy_Curve * self )
 {
-/*	update_displists( ( void * ) self->curve );  */
-	freedisplist( &self->curve->disp ); 
+	Object_updateDag( (void*) self->curve );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 /*
@@ -1327,6 +1337,54 @@ PyObject *Curve_setBevOb( BPy_Curve * self, PyObject * args )
 	return EXPP_incr_ret( Py_None );
 }
 
+/*****************************************************************************/
+/* Function:    Curve_getTaperOb                                               */
+/* Description: Get the taper object assign to the curve.                    */
+/*****************************************************************************/
+
+
+static PyObject *Curve_getTaperOb( BPy_Curve * self)
+{
+	if( self->curve->taperobj ) {
+		return Object_CreatePyObject( self->curve->taperobj );
+	}
+
+	return EXPP_incr_ret( Py_None );
+}
+
+/*****************************************************************************/
+/* Function:    Curve_setTaperOb                                               */
+/* Description: Assign a taper object to the curve.                          */
+/*****************************************************************************/
+
+PyObject *Curve_setTaperOb( BPy_Curve * self, PyObject * args )
+{
+	BPy_Object *pytaperobj;
+
+	/* Parse and check input args */
+	if( !PyArg_ParseTuple( args, "O", &pytaperobj) ) {
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+					"expected object or None argument" ) );
+	}
+
+	/* Accept None */
+	if( (PyObject *)pytaperobj == Py_None ) {
+		self->curve->taperobj = (Object *)NULL;
+	} else {
+	/* Accept Object with type 'Curve' */
+		if( Object_CheckPyObject( ( PyObject * ) pytaperobj ) && 
+			pytaperobj->object->type == OB_CURVE) {
+			self->curve->taperobj = 
+				Object_FromPyObject( ( PyObject * ) pytaperobj );
+		} else {
+			return ( EXPP_ReturnPyObjError( PyExc_TypeError,
+						"expected Curve object type or None argument" ) );
+		}
+	}
+
+	return EXPP_incr_ret( Py_None );
+}
+
 /*
  * Curve_getIter
  *
@@ -1452,34 +1510,38 @@ static PyObject *CurveGetAttr( BPy_Curve * self, char *name )
 
 	if( strcmp( name, "name" ) == 0 )
 		attr = PyString_FromString( self->curve->id.name + 2 );
-	if( strcmp( name, "pathlen" ) == 0 )
+	else if( strcmp( name, "pathlen" ) == 0 )
 		attr = PyInt_FromLong( self->curve->pathlen );
-	if( strcmp( name, "totcol" ) == 0 )
+	else if( strcmp( name, "totcol" ) == 0 )
 		attr = PyInt_FromLong( self->curve->totcol );
-	if( strcmp( name, "flag" ) == 0 )
+	else if( strcmp( name, "flag" ) == 0 )
 		attr = PyInt_FromLong( self->curve->flag );
-	if( strcmp( name, "bevresol" ) == 0 )
+	else if( strcmp( name, "bevresol" ) == 0 )
 		attr = PyInt_FromLong( self->curve->bevresol );
-	if( strcmp( name, "resolu" ) == 0 )
+	else if( strcmp( name, "resolu" ) == 0 )
 		attr = PyInt_FromLong( self->curve->resolu );
-	if( strcmp( name, "resolv" ) == 0 )
+	else if( strcmp( name, "resolv" ) == 0 )
 		attr = PyInt_FromLong( self->curve->resolv );
-	if( strcmp( name, "width" ) == 0 )
+	else if( strcmp( name, "width" ) == 0 )
 		attr = PyFloat_FromDouble( self->curve->width );
-	if( strcmp( name, "ext1" ) == 0 )
+	else if( strcmp( name, "ext1" ) == 0 )
 		attr = PyFloat_FromDouble( self->curve->ext1 );
-	if( strcmp( name, "ext2" ) == 0 )
+	else if( strcmp( name, "ext2" ) == 0 )
 		attr = PyFloat_FromDouble( self->curve->ext2 );
-	if( strcmp( name, "loc" ) == 0 )
+	else if( strcmp( name, "loc" ) == 0 )
 		return Curve_getLoc( self );
-	if( strcmp( name, "rot" ) == 0 )
+	else if( strcmp( name, "rot" ) == 0 )
 		return Curve_getRot( self );
-	if( strcmp( name, "size" ) == 0 )
+	else if( strcmp( name, "size" ) == 0 )
 		return Curve_getSize( self );
-	if( strcmp( name, "bevob" ) == 0 )
+	else if( strcmp( name, "bevob" ) == 0 )
 		return Curve_getBevOb( self );
+	else if( strcmp( name, "taperob" ) == 0 )
+		return Curve_getTaperOb( self );
+	else if( strcmp( name, "key" ) == 0 )
+		return Curve_getKey( self );
 #if 0
-	if( strcmp( name, "numpts" ) == 0 )
+	else if( strcmp( name, "numpts" ) == 0 )
 		return Curve_getNumPoints( self );
 #endif
 
@@ -1514,6 +1576,8 @@ static int CurveSetAttr( BPy_Curve * self, char *name, PyObject * value )
 		error = Curve_setName( self, valtuple );
 	else if( strcmp( name, "pathlen" ) == 0 )
 		error = Curve_setPathLen( self, valtuple );
+	else if( strcmp( name, "bevresol" ) == 0 )
+		error = Curve_setBevresol( self, valtuple );
 	else if( strcmp( name, "resolu" ) == 0 )
 		error = Curve_setResolu( self, valtuple );
 	else if( strcmp( name, "resolv" ) == 0 )
@@ -1532,6 +1596,8 @@ static int CurveSetAttr( BPy_Curve * self, char *name, PyObject * value )
 		error = Curve_setSize( self, valtuple );
 	else if( strcmp( name, "bevob" ) == 0 )
 		error = Curve_setBevOb( self, valtuple );
+	else if( strcmp( name, "taperob" ) == 0 )
+		error = Curve_setTaperOb( self, valtuple );
 
 	else {			/* Error */
 		Py_DECREF( valtuple );
@@ -1603,54 +1669,3 @@ struct Curve *Curve_FromPyObject( PyObject * py_obj )
 
 }
 
-
-
-/*
- * NOTE:  this func has been replaced by freedisplist() in the recent
- *        display list refactoring.
- *
- * walk across all objects looking for curves
- *  so we can update their ob's disp list
- */
-
-void update_displists( void *data )
-{
-#if 0
-	Base *base;
-	Object *ob;
-	unsigned int layer;
-
-	/* background */
-	layer = G.scene->lay;
-
-	base = G.scene->base.first;
-	while( base ) {
-		if( base->lay & layer ) {
-			ob = base->object;
-
-			if( ELEM( ob->type, OB_CURVE, OB_SURF ) ) {
-				if( ob != G.obedit ) {
-					if( ob->data == data ) {
-						makeDispList( ob );
-					}
-				}
-			} else if( ob->type == OB_FONT ) {
-				Curve *cu = ob->data;
-				if( cu->textoncurve ) {
-					if( ( ( Curve * ) cu->textoncurve->
-					      data )->key ) {
-						text_to_curve( ob, 0 );
-						makeDispList( ob );
-					}
-				}
-			}
-		}
-		if( base->next == 0 && G.scene->set
-		    && base == G.scene->base.last )
-			base = G.scene->set->base.first;
-		else
-			base = base->next;
-	}
-#endif
-}
-
