@@ -1,5 +1,5 @@
 /* 
- * $Id: Draw.c,v 1.39 2005/06/12 05:54:15 ianwill Exp $
+ * $Id: Draw.c,v 1.52 2006/06/27 17:51:11 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,40 +35,33 @@
  * implementation.	Non-trivial original comments are marked with an
  * @ symbol at their beginning. */
 
-#include "Draw.h"
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "Draw.h" /*This must come first*/
 
 #include "BLI_blenlib.h"
 #include "MEM_guardedalloc.h"
-
 #include "BMF_Api.h"
-
 #include "DNA_screen_types.h"
-
-#include "DNA_text_types.h"
-
 #include "BKE_global.h"
 #include "BKE_image.h"
-#include "BKE_library.h"
 #include "BKE_object.h"
-
+#include "BKE_main.h"
 #include "BIF_gl.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_interface.h"
-#include "BIF_mywindow.h"
 #include "BIF_toolbox.h"
-
 #include "BPI_script.h"		/* script struct */
-
 #include "Image.h"              /* for accessing Blender.Image objects */
 #include "IMB_imbuf_types.h"    /* for the IB_rect define */
-
 #include "interface.h"
 #include "mydevice.h"		/*@ for all the event constants */
+#include "gen_utils.h"
+#include "Window.h"
+#include "../BPY_extern.h"
+
+/* used so we can get G.scene->r.cfra for getting the
+current image frame, some images change frame if they are a sequence */
+#include "DNA_scene_types.h"
 
 /* these delimit the free range for button events */
 #define EXPP_BUTTON_EVENTS_OFFSET 1001
@@ -108,6 +101,7 @@ static PyObject *Method_Menu( PyObject * self, PyObject * args );
 static PyObject *Method_Toggle( PyObject * self, PyObject * args );
 static PyObject *Method_Slider( PyObject * self, PyObject * args );
 static PyObject *Method_Scrollbar( PyObject * self, PyObject * args );
+static PyObject *Method_ColorPicker( PyObject * self, PyObject * args );
 static PyObject *Method_Number( PyObject * self, PyObject * args );
 static PyObject *Method_String( PyObject * self, PyObject * args );
 static PyObject *Method_GetStringWidth( PyObject * self, PyObject * args );
@@ -119,6 +113,8 @@ static PyObject *Method_PupFloatInput( PyObject * self, PyObject * args );
 static PyObject *Method_PupStrInput( PyObject * self, PyObject * args );
 /* next by Jonathan Merritt (lancelet): */
 static PyObject *Method_Image( PyObject * self, PyObject * args);
+/* CLEVER NUMBUT */
+static PyObject *Method_PupBlock( PyObject * self, PyObject * args );
 
 static uiBlock *Get_uiBlock( void );
 static void py_slider_update( void *butv, void *data2_unused );
@@ -215,6 +211,15 @@ new Scrollbar\n\n\
 	A non-zero value (default) enables the events. A zero value supresses them.\n\
 [tooltip=] The button's tooltip";
 
+static char Method_ColorPicker_doc[] = 
+	"(event, x, y, width, height, initial, [tooltip]) - Create a new Button \
+Color picker button\n\n\
+(event) The event number to pass to the button event function when the color changes\n\
+(x, y) The lower left coordinate of the button\n\
+(width, height) The button width and height\n\
+(initial) 3-Float tuple of the color (values between 0 and 1)\
+[tooltip=] The button's tooltip";
+
 static char Method_Number_doc[] =
 	"(name, event, x, y, width, height, initial, min, max, [tooltip]) - Create a \
 new Number button\n\n\
@@ -289,6 +294,22 @@ static char Method_PupStrInput_doc[] =
 (max = 20) - The maximum number of chars the user can input;\n\
 Return the user input value or None on user exit";
 
+static char Method_PupBlock_doc[] =
+	"(title, sequence) - Display a pop-up block.\n\
+(title) - The title of the block.\n\
+(sequence) - A sequence defining what the block contains. \
+The order of the list is the order of appearance, from top down.\n\
+Possible format for sequence items:\n\
+[value is an object created with Create]\n\
+\ttext: Defines a label in the block\n\
+\t(text, value, tooltip = ''): Defines a toggle button \n\
+\t(text, value, min, max, tooltip = ''): Defines a num or string button \n\
+\t\t\tdepending on the value.\n\
+\t\tFor string, max is the maximum length of the text and min is unused.\n\
+Return 1 if the pop-up is confirmed, 0 otherwise. \n\
+Warning: On cancel, the value objects are brought back to there previous values, \
+\texcept for string values which will still contain the modified values.\n";
+
 static char Method_Exit_doc[] = "() - Exit the windowing interface";
 
 /*
@@ -311,6 +332,7 @@ static struct PyMethodDef Draw_methods[] = {
 	MethodDef( Menu ),
 	MethodDef( Slider ),
 	MethodDef( Scrollbar ),
+	MethodDef( ColorPicker ),
 	MethodDef( Number ),
 	MethodDef( String ),
 	MethodDef( GetStringWidth ),
@@ -319,6 +341,7 @@ static struct PyMethodDef Draw_methods[] = {
 	MethodDef( PupIntInput ),
 	MethodDef( PupFloatInput ),
 	MethodDef( PupStrInput ),
+	MethodDef( PupBlock ),
 	MethodDef( Image ),
 	MethodDef( Exit ),
 	MethodDef( Redraw ),
@@ -347,7 +370,7 @@ static void Button_dealloc( PyObject * self )
 {
 	Button *but = ( Button * ) self;
 
-	if( but->type == 3 ) {
+	if( but->type == BSTRING_TYPE ) {
 		if( but->val.asstr )
 			MEM_freeN( but->val.asstr );
 	}
@@ -358,14 +381,16 @@ static void Button_dealloc( PyObject * self )
 static PyObject *Button_getattr( PyObject * self, char *name )
 {
 	Button *but = ( Button * ) self;
-
+	
 	if( strcmp( name, "val" ) == 0 ) {
-		if( but->type == 1 )
+		if( but->type == BINT_TYPE )
 			return Py_BuildValue( "i", but->val.asint );
-		else if( but->type == 2 )
+		else if( but->type == BFLOAT_TYPE )
 			return Py_BuildValue( "f", but->val.asfloat );
-		else if( but->type == 3 )
+		else if( but->type == BSTRING_TYPE )
 			return Py_BuildValue( "s", but->val.asstr );
+		else if( but->type == BVECTOR_TYPE )
+			return Py_BuildValue( "fff", but->val.asvec[0], but->val.asvec[1], but->val.asvec[2] );
 	}
 
 	PyErr_SetString( PyExc_AttributeError, name );
@@ -377,35 +402,66 @@ static int Button_setattr( PyObject * self, char *name, PyObject * v )
 	Button *but = ( Button * ) self;
 
 	if( strcmp( name, "val" ) == 0 ) {
-		if( but->type == 1 )
-			PyArg_Parse( v, "i", &but->val.asint );
-		else if( but->type == 2 )
-			PyArg_Parse( v, "f", &but->val.asfloat );
-		else if( but->type == 3 ) {
+		if( but->type == BINT_TYPE && PyNumber_Check(v) ) {
+			PyObject *pyVal = PyNumber_Int( v );
+			if (pyVal) {
+				but->val.asint = (int)PyInt_AS_LONG( pyVal );
+				Py_DECREF(pyVal);
+				return 0;
+			}
+		}
+		else if( but->type == BFLOAT_TYPE && PyNumber_Check(v) ) {
+			PyObject *pyVal = PyNumber_Float( v );
+			if (pyVal) {
+				but->val.asfloat = (float)PyFloat_AS_DOUBLE( pyVal );
+				Py_DECREF(pyVal);
+				return 0;
+			}
+		}
+		else if( but->type == BVECTOR_TYPE ) {
+			if ( PyArg_ParseTuple( v, "fff", but->val.asvec, but->val.asvec+1, but->val.asvec+2 ) )
+				return 0;
+		}
+		else if( but->type == BSTRING_TYPE && PyString_Check(v) ) {
 			char *newstr;
-			PyArg_Parse( v, "s", &newstr );
+			unsigned int newlen;
+
+			PyString_AsStringAndSize( v, &newstr, &newlen );
+			
+			if (newlen+1> UI_MAX_DRAW_STR)
+				return EXPP_ReturnIntError( PyExc_ValueError, "Error: button string length exceeded max limit (399 chars).");
 
 			/* if the length of the new string is the same as */
 			/* the old one, just copy, else delete and realloc. */
-			if( but->slen == strlen( newstr ) ) {
+			if( but->slen == newlen ) {
 				BLI_strncpy( but->val.asstr, newstr,
 					     but->slen + 1 );
+
+				return 0;
+
 			} else {
 				MEM_freeN( but->val.asstr );
-				but->slen = strlen( newstr );
+				but->slen = newlen;
 				but->val.asstr =
 					MEM_mallocN( but->slen + 1,
 						     "button setattr" );
 				BLI_strncpy( but->val.asstr, newstr,
 					     but->slen + 1 );
+
+				return 0;
 			}
 		}
 	} else {
-		PyErr_SetString( PyExc_AttributeError, name );
-		return -1;
+		/*
+		 * Accessing the wrong attribute.
+		 */
+		return EXPP_ReturnIntError( PyExc_AttributeError, name );
 	}
 
-	return 0;
+	/*
+	 * Correct attribute but value is incompatible with current button value.
+	 */
+	return EXPP_ReturnIntError( PyExc_ValueError, "value incompatible with current button type" );
 }
 
 static PyObject *Button_repr( PyObject * self )
@@ -415,7 +471,9 @@ static PyObject *Button_repr( PyObject * self )
 
 static Button *newbutton( void )
 {
-	Button *but = ( Button * ) PyObject_NEW( Button, &Button_Type );
+	Button *but = NULL;
+	
+	but = ( Button * ) PyObject_NEW( Button, &Button_Type );
 
 	return but;
 }
@@ -546,12 +604,14 @@ void BPY_spacescript_do_pywin_event( SpaceScript * sc, unsigned short event,
 		int pass_ascii = 0;
 		if (ascii > 31 && ascii != 127) {
 			pass_ascii = 1;
-			PyDict_SetItemString(g_blenderdict, "event", PyInt_FromLong((long)ascii));
+			EXPP_dict_set_item_str(g_blenderdict, "event",
+					PyInt_FromLong((long)ascii));
 		}
 		exec_callback( sc, sc->script->py_event,
 			Py_BuildValue( "(ii)", event, val ) );
 		if (pass_ascii)
-			PyDict_SetItemString(g_blenderdict, "event", PyString_FromString(""));
+			EXPP_dict_set_item_str(g_blenderdict, "event",
+					PyString_FromString(""));
 	}
 }
 
@@ -706,31 +766,45 @@ static PyObject *Method_Draw( PyObject * self, PyObject * args )
 
 static PyObject *Method_Create( PyObject * self, PyObject * args )
 {
-	Button *but;
-	PyObject *in;
+	Button *but = NULL;
+	PyObject *val;
+	char *newstr;
 
-	if( !PyArg_ParseTuple( args, "O", &in ) )
-		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected PyObject argument" );
+	but = newbutton();
 
-	but = newbutton(  );
-	if( PyFloat_Check( in ) ) {
-		but->type = 2;
-		but->val.asfloat = PyFloat_AsDouble( in );
-	} else if( PyInt_Check( in ) ) {
-		but->type = 1;
-		but->val.asint = PyInt_AsLong( in );
-	} else if( PyString_Check( in ) ) {
-		char *newstr = PyString_AsString( in );
-
-		but->type = 3;
-		but->slen = strlen( newstr );
-		but->val.asstr = MEM_mallocN( but->slen + 1, "button string" );
-
-		strcpy( but->val.asstr, newstr );
+	if ( PyArg_ParseTuple( args, "fff", but->val.asvec, but->val.asvec+1, but->val.asvec+2 ) ) {
+		but->type = BVECTOR_TYPE;
+	}
+	else if ( PyArg_ParseTuple( args, "O!", &PyFloat_Type, &val ) ) {
+		but->val.asfloat = (float)PyFloat_AS_DOUBLE(val);
+		but->type = BFLOAT_TYPE;
+	}
+	else if ( PyArg_ParseTuple( args, "O!", &PyInt_Type, &val ) ) {
+		but->val.asint = (int)PyInt_AS_LONG(val);
+		but->type = BINT_TYPE;
+	}
+	else if ( PyArg_ParseTuple( args, "s#", &newstr, &but->slen ) ) {
+		if (but->slen + 1 > UI_MAX_DRAW_STR) {
+			PyObject_DEL( (PyObject *) but );
+			but = NULL;
+			PyErr_SetString( PyExc_TypeError, "string is longer then 399 chars");
+		} else {
+			but->type = BSTRING_TYPE;
+			but->val.asstr = MEM_mallocN( but->slen + 1, "button string" );
+			BLI_strncpy( but->val.asstr, newstr, but->slen+1 );
+		}
+	}
+	else {
+		PyObject_DEL( (PyObject *) but );
+		but = NULL;
+		PyErr_SetString( PyExc_TypeError, "expected string, float, int or 3-float tuple argument" );
+	}
+	
+	if (but != NULL) {
+		PyErr_Clear();
 	}
 
-	return ( PyObject * ) but;
+	return (PyObject*) but;
 }
 
 static uiBlock *Get_uiBlock( void )
@@ -776,7 +850,7 @@ static PyObject *Method_Button( PyObject * self, PyObject * args )
 	block = Get_uiBlock(  );
 
 	if( block )
-		uiDefBut( block, BUT, event, name, x, y, w, h, 0, 0, 0, 0, 0,
+		uiDefBut( block, BUT, event, name, (short)x, (short)y, (short)w, (short)h, 0, 0, 0, 0, 0,
 			  tip );
 
 	return EXPP_incr_ret( Py_None );
@@ -800,12 +874,12 @@ static PyObject *Method_Menu( PyObject * self, PyObject * args )
 			"button event argument must be in the range [0, 16382]");
 
 	but = newbutton(  );
-	but->type = 1;
+	but->type = BINT_TYPE;
 	but->val.asint = def;
 
 	block = Get_uiBlock(  );
 	if( block )
-		uiDefButI( block, MENU, event, name, x, y, w, h,
+		uiDefButI( block, MENU, event, name, (short)x, (short)y, (short)w, (short)h,
 			   &but->val.asint, 0, 0, 0, 0, tip );
 
 	return ( PyObject * ) but;
@@ -829,12 +903,12 @@ static PyObject *Method_Toggle( PyObject * self, PyObject * args )
 			"button event argument must be in the range [0, 16382]");
 
 	but = newbutton(  );
-	but->type = 1;
+	but->type = BINT_TYPE;
 	but->val.asint = def;
 
 	block = Get_uiBlock(  );
 	if( block )
-		uiDefButI( block, TOG, event, name, x, y, w, h,
+		uiDefButI( block, TOG, event, name, (short)x, (short)y, (short)w, (short)h,
 			   &but->val.asint, 0, 0, 0, 0, tip );
 
 	return ( PyObject * ) but;
@@ -863,7 +937,7 @@ static void py_slider_update( void *butv, void *data2_unused )
 	disable_where_script( 1 );
 
 	spacescript_do_pywin_buttons( curarea->spacedata.first,
-		uiButGetRetVal( but ) );
+		(unsigned short)uiButGetRetVal( but ) );
 
 	/* XXX useless right now, investigate better before a bcon 5 */
 	ret = M_Window_Redraw( 0, ref );
@@ -901,18 +975,18 @@ static PyObject *Method_Slider( PyObject * self, PyObject * args )
 	if( PyFloat_Check( inio ) ) {
 		float ini, min, max;
 
-		ini = PyFloat_AsDouble( inio );
-		min = PyFloat_AsDouble( mino );
-		max = PyFloat_AsDouble( maxo );
+		ini = (float)PyFloat_AsDouble( inio );
+		min = (float)PyFloat_AsDouble( mino );
+		max = (float)PyFloat_AsDouble( maxo );
 
-		but->type = 2;
+		but->type = BFLOAT_TYPE;
 		but->val.asfloat = ini;
 
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButF( block, NUMSLI, event, name, x, y, w,
-					  h, &but->val.asfloat, min, max, 0, 0,
+			ubut = uiDefButF( block, NUMSLI, event, name, (short)x, (short)y, (short)w,
+					  (short)h, &but->val.asfloat, min, max, 0, 0,
 					  tip );
 			if( realtime )
 				uiButSetFunc( ubut, py_slider_update, ubut,
@@ -925,14 +999,14 @@ static PyObject *Method_Slider( PyObject * self, PyObject * args )
 		min = PyInt_AsLong( mino );
 		max = PyInt_AsLong( maxo );
 
-		but->type = 1;
+		but->type = BINT_TYPE;
 		but->val.asint = ini;
 
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButI( block, NUMSLI, event, name, x, y, w,
-					  h, &but->val.asint, min, max, 0, 0,
+			ubut = uiDefButI( block, NUMSLI, event, name, (short)x, (short)y, (short)w,
+					  (short)h, &but->val.asint, (float)min, (float)max, 0, 0,
 					  tip );
 			if( realtime )
 				uiButSetFunc( ubut, py_slider_update, ubut,
@@ -970,20 +1044,20 @@ another int and string as arguments" );
 	but = newbutton(  );
 
 	if( PyFloat_Check( inio ) )
-		but->type = 2;
+		but->type = BFLOAT_TYPE;
 	else
-		but->type = 1;
+		but->type = BINT_TYPE;
 
-	ini = PyFloat_AsDouble( inio );
-	min = PyFloat_AsDouble( mino );
-	max = PyFloat_AsDouble( maxo );
+	ini = (float)PyFloat_AsDouble( inio );
+	min = (float)PyFloat_AsDouble( mino );
+	max = (float)PyFloat_AsDouble( maxo );
 
-	if( but->type == 2 ) {
+	if( but->type == BFLOAT_TYPE ) {
 		but->val.asfloat = ini;
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButF( block, SCROLL, event, "", x, y, w, h,
+			ubut = uiDefButF( block, SCROLL, event, "", (short)x, (short)y, (short)w, (short)h,
 					  &but->val.asfloat, min, max, 0, 0,
 					  tip );
 			if( realtime )
@@ -991,11 +1065,11 @@ another int and string as arguments" );
 					      NULL );
 		}
 	} else {
-		but->val.asint = ini;
+		but->val.asint = (int)ini;
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButI( block, SCROLL, event, "", x, y, w, h,
+			ubut = uiDefButI( block, SCROLL, event, "", (short)x, (short)y, (short)w, (short)h,
 					  &but->val.asint, min, max, 0, 0,
 					  tip );
 			if( realtime )
@@ -1007,6 +1081,54 @@ another int and string as arguments" );
 	return ( PyObject * ) but;
 }
 
+static PyObject *Method_ColorPicker( PyObject * self, PyObject * args )
+{
+	char USAGE_ERROR[] = "expected a 3-float tuple of values between 0 and 1";
+	Button *but;
+	PyObject *inio;
+	uiBlock *block;
+	char *tip = NULL;
+	float col[3];
+	int event;
+	short x, y, w, h;
+	
+	if( !PyArg_ParseTuple( args, "ihhhhO!|s", &event,
+			       &x, &y, &w, &h, &PyTuple_Type, &inio, &tip ) )
+ 		return EXPP_ReturnPyObjError( PyExc_TypeError,
+					      "expected five ints, one tuple and optionally a string as arguments." );
+ 
+ 	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_ValueError,
+ 			"button event argument must be in the range [0, 16382]");
+ 
+	if ( !PyArg_ParseTuple( inio, "fff", col, col+1, col+2 ) )
+		return EXPP_ReturnPyObjError( PyExc_ValueError, USAGE_ERROR);
+
+	if	(	col[0] < 0 || col[0] > 1
+		||	col[1] < 0 || col[1] > 1
+		||	col[2] < 0 || col[2] > 1 )
+		return EXPP_ReturnPyObjError( PyExc_ValueError, USAGE_ERROR);
+
+	if ( EXPP_check_sequence_consistency( inio, &PyFloat_Type ) != 1 )
+		return EXPP_ReturnPyObjError( PyExc_ValueError, USAGE_ERROR);
+ 
+	but = newbutton();
+ 
+	but->type = BVECTOR_TYPE;
+	but->val.asvec[0] = col[0];
+	but->val.asvec[1] = col[1];
+	but->val.asvec[2] = col[2];
+	
+	block = Get_uiBlock(  );
+	if( block ) {
+		uiBut *ubut;
+		ubut = uiDefButF( block, COL, event, "", x, y, w, h, but->val.asvec, 0, 0, 0, 0, tip);
+	}
+
+ 	return ( PyObject * ) but;
+}
+
+
 static PyObject *Method_Number( PyObject * self, PyObject * args )
 {
 	uiBlock *block;
@@ -1029,19 +1151,28 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 	but = newbutton(  );
 
 	if( PyFloat_Check( inio ) ) {
-		float ini, min, max;
+		float ini, min, max, range, precission=0;
 
-		ini = PyFloat_AsDouble( inio );
-		min = PyFloat_AsDouble( mino );
-		max = PyFloat_AsDouble( maxo );
-
-		but->type = 2;
+		ini = (float)PyFloat_AsDouble( inio );
+		min = (float)PyFloat_AsDouble( mino );
+		max = (float)PyFloat_AsDouble( maxo );
+		
+		range= fabs(max-min); /* Click step will be a 10th of the range. */
+		if (!range) range= 1.0f; /* avoid any odd errors */
+		
+		/* set the precission to display*/
+		if      (range>=100.0f) precission=1.0f;
+		else if (range>=10.0f) precission=2.0f;
+		else if (range>=1.0f) precission=3.0f;
+		else precission=4.0f;
+		
+		but->type = BFLOAT_TYPE;
 		but->val.asfloat = ini;
 
 		block = Get_uiBlock(  );
 		if( block )
-			uiDefButF( block, NUM, event, name, x, y, w, h,
-				   &but->val.asfloat, min, max, 0, 0, tip );
+			uiDefButF( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
+				   &but->val.asfloat, min, max, 10*range, precission, tip );
 	} else {
 		int ini, min, max;
 
@@ -1049,13 +1180,13 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 		min = PyInt_AsLong( mino );
 		max = PyInt_AsLong( maxo );
 
-		but->type = 1;
+		but->type = BINT_TYPE;
 		but->val.asint = ini;
 
 		block = Get_uiBlock(  );
 		if( block )
-			uiDefButI( block, NUM, event, name, x, y, w, h,
-				   &but->val.asint, min, max, 0, 0, tip );
+			uiDefButI( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
+				   &but->val.asint, (float)min, (float)max, 0, 0, tip );
 	}
 
 	return ( PyObject * ) but;
@@ -1080,16 +1211,15 @@ static PyObject *Method_String( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
 			"button event argument must be in the range [0, 16382]");
 
-	if (len > (UI_MAX_DRAW_STR - 1)) {
-		len = UI_MAX_DRAW_STR - 1;
-		newstr[len] = '\0';
-	}
+	if (len > (UI_MAX_DRAW_STR - 1))
+		return EXPP_ReturnPyObjError( PyExc_ValueError,
+			"The maximum length of a string is 399, your value is too high.");
 
 	real_len = strlen(newstr);
 	if (real_len > len) real_len = len;
 
 	but = newbutton(  );
-	but->type = 3;
+	but->type = BSTRING_TYPE;
 	but->slen = len;
 	but->val.asstr = MEM_mallocN( len + 1, "pybutton str" );
 
@@ -1101,8 +1231,8 @@ static PyObject *Method_String( PyObject * self, PyObject * args )
 
 	block = Get_uiBlock(  );
 	if( block )
-		uiDefBut( block, TEX, event, info_str, x, y, w, h,
-			  but->val.asstr, 0, len, 0, 0, tip );
+		uiDefBut( block, TEX, event, info_str, (short)x, (short)y, (short)w, (short)h,
+			  but->val.asstr, 0, (float)len, 0, 0, tip );
 
 	return ( PyObject * ) but;
 }
@@ -1201,7 +1331,7 @@ static PyObject *Method_PupIntInput( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected 1 string and 3 int arguments" );
 
-	if( button( &var, min, max, text ) == 0 ) {
+	if( button( &var, (short)min, (short)max, text ) == 0 ) {
 		Py_INCREF( Py_None );
 		return Py_None;
 	}
@@ -1270,6 +1400,129 @@ static PyObject *Method_PupStrInput( PyObject * self, PyObject * args )
 				      "couldn't create a PyString" );
 }
 
+static PyObject *Method_PupBlock( PyObject * self, PyObject * args )
+{
+	PyObject *pyList, *pyItem;
+	float min, max;
+	int len, i;
+	char *title;
+
+	if (!PyArg_ParseTuple( args, "sO", &title, &pyList ) || !PySequence_Check( pyList ))
+		return EXPP_ReturnPyObjError( PyExc_TypeError, "expected a string and a sequence" );
+
+
+	len = PySequence_Length(pyList);
+
+	if (len == 0)
+		return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string and a non-empty sequence." );
+
+	if (len > 24) /* LIMIT DEFINED IN toolbox.c	*/
+		return EXPP_ReturnPyObjError( PyExc_ValueError, "sequence cannot have more than 24 elements" );
+
+	for ( i=0 ; i<len ; i++ ) {
+		PyObject *pyMin = NULL, *pyMax = NULL;
+		PyObject *f1, *f2;
+		Button *but = NULL;
+		int tlen;
+		char *text, *tip = NULL;
+
+		pyItem = PySequence_GetItem( pyList, i );
+		if (!pyItem)
+			return NULL;
+
+		if (PyString_Check( pyItem )) {
+			tlen = -2;	/* single string for label, giving it a special len for later */
+		}
+		else if (PyTuple_Check( pyItem )) {
+			/* tuple for other button, get the length for later */
+			tlen = PyTuple_Size( pyItem );
+		}
+		else {
+			/* Neither a string or a tuple, error */
+			Py_DECREF( pyItem );
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string or a tuple containing 2 to 5 values." );
+		}
+
+		switch (tlen) {
+		case -2:		/*	LABEL	*/
+			text = PyString_AsString( pyItem );
+			add_numbut(i, LABEL, text, 0, 0, NULL, NULL);
+			break;
+		case 2:		/*	TOGGLE	(no tooltip)	*/
+		case 3:		/*	TOGGLE	*/
+			if (!PyArg_ParseTuple( pyItem, "sO!|s", &text, &Button_Type, &but, &tip )) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a tuple containing a string, a Button object and optionally a string for toggles" );
+			}
+
+			if (but->type != BINT_TYPE) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "Button object for toggles should hold an integer" );
+			}
+
+			add_numbut(i, TOG|INT, text, 0, 0, &but->val.asint, tip);
+			break;
+		case 4:		/*	TEX and NUM (no tooltip)	*/
+		case 5:		/*	TEX and NUM	*/
+			if (!PyArg_ParseTuple( pyItem, "sO!OO|s", &text, &Button_Type, &but, &pyMin, &pyMax, &tip )) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a tuple containing a string, a Button object, two numerical values and optionally a string for Text and Num buttons" );
+			}
+
+			f1 = PyNumber_Float(pyMin);
+			f2 = PyNumber_Float(pyMax);
+
+			if (!f1 || !f2) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a tuple containing a string, a Button object, two numerical values and optionally a string for Text and Num buttons" );
+			}
+
+			min = (float)PyFloat_AS_DOUBLE(f1);
+			max = (float)PyFloat_AS_DOUBLE(f2);
+			Py_DECREF( f1 );
+			Py_DECREF( f2 );
+
+			switch ( but->type ) {
+			case BINT_TYPE:
+				add_numbut(i, NUM|INT, text, min, max, &but->val.asint, tip);
+				break;
+			case BFLOAT_TYPE:
+				add_numbut(i, NUM|FLO, text, min, max, &but->val.asfloat, tip);
+				break;
+			case BSTRING_TYPE:
+				if (max+1>UI_MAX_DRAW_STR) {
+					Py_DECREF( pyItem );
+					return EXPP_ReturnPyObjError( PyExc_ValueError, "length of a string buttons must be less then 400" );
+				}
+				max = (float)floor(max);
+
+				if (max > but->slen) {
+					int old_len = but->slen;
+					char *old_str = but->val.asstr;
+					but->slen = (int)max;
+					but->val.asstr = MEM_callocN( but->slen + 1, "button pupblock");
+					BLI_strncpy( but->val.asstr, old_str, old_len + 1 );
+					MEM_freeN(old_str);
+				}
+
+				add_numbut(i, TEX, text, 0.0f, max, but->val.asstr, tip);
+			}
+
+			break;
+		default:
+			Py_DECREF( pyItem );
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string or a tuple containing 2 to 5 values." );
+		}
+		Py_DECREF( pyItem );
+	}
+
+	if (do_clever_numbuts(title, len, REDRAW))
+		return EXPP_incr_ret_True();
+	else
+		return EXPP_incr_ret_False();
+}
+
+
 /*****************************************************************************
  * Function:            Method_Image                                         *
  * Python equivalent:   Blender.Draw.Image                                   *
@@ -1309,11 +1562,14 @@ static PyObject *Method_Image( PyObject * self, PyObject * args )
 
 	/* load the image data if necessary */
 	if( !image->ibuf )      /* if no image data is available ... */
-		load_image( image, IB_rect, "", 0 );    /* ... load it */
+		load_image( image, IB_rect, G.sce,  G.scene->r.cfra );	/* loading it */
 	if( !image->ibuf )      /* if failed to load the image */
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 			"couldn't load image data in Blender" );
 
+	/* Update the time tag of the image */
+	tag_image_time(image);
+
 	/* set up a valid clipping rectangle.  if no clip rectangle was
 	 * given, this results in inclusion of the entire image.  otherwise,
 	 * the clipping is just checked against the bounds of the image.
@@ -1401,11 +1657,11 @@ PyObject *Draw_Init( void )
 	dict = PyModule_GetDict( submodule );
 
 #define EXPP_ADDCONST(x) \
-	PyDict_SetItemString(dict, #x, PyInt_FromLong(x))
+	EXPP_dict_set_item_str(dict, #x, PyInt_FromLong(x))
 
 	/* So, for example:
 	 * EXPP_ADDCONST(LEFTMOUSE) becomes
-	 * PyDict_SetItemString(dict, "LEFTMOUSE", PyInt_FromLong(LEFTMOUSE)) 
+	 * EXPP_dict_set_item_str(dict, "LEFTMOUSE", PyInt_FromLong(LEFTMOUSE)) 
 	 */
 
 	EXPP_ADDCONST( LEFTMOUSE );
