@@ -1,5 +1,5 @@
 /* 
- * $Id: Registry.c,v 1.6 2005/04/16 05:25:42 ianwill Exp $
+ * $Id: Registry.c,v 1.7 2005/07/18 03:50:37 ascotan Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,11 +30,9 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
 
-#include "Registry.h"
-
-#include <stdio.h>
-#include <BKE_global.h> /* G.f & G_DEBUG */
+#include "Registry.h" /*This must come first */
 
+#include "BKE_global.h"
 #include "gen_utils.h"
 
 
