@@ -1,5 +1,5 @@
 /*
- * $Id: Text3d.c,v 1.9 2005/06/12 05:54:15 ianwill Exp $
+ * $Id: Text3d.c,v 1.21 2006/07/01 16:59:10 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,32 +30,24 @@
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
+ 
+#include "Text3d.h" /*This must come first*/
+ 
 #include "DNA_object_types.h"
-#include "DNA_scene_types.h"
-#include "DNA_view3d_types.h"
-#include "DNA_vfont_types.h"
 #include "MEM_guardedalloc.h"
-#include "BKE_object.h"
-#include "BDR_editobject.h"
-#include "BKE_displist.h"
-#include "MEM_guardedalloc.h"
-#include "mydevice.h"
-#include "blendef.h"
-#include "Text3d.h"
+#include "BKE_curve.h"
+#include "BKE_library.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
+#include "BIF_editfont.h"	/* do_textedit() */
 #include "Curve.h"
 #include "constant.h"
-#include "Types.h"
 #include "Font.h"
-
-#include "mydevice.h"
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <errno.h>
+#include "gen_utils.h"
 
 //no prototypes declared in header files - external linkage outside of python
 extern VFont *get_builtin_font(void);  
 extern void freedisplist(struct ListBase *lb);
-extern VFont *get_builtin_font(void);
 extern VFont *give_vfontpointer(int);
 extern VFont *exist_vfont(char *str);
 extern VFont *load_vfont(char *name);
@@ -272,6 +264,11 @@ PyObject *M_Text3d_New( PyObject * self, PyObject * args )
 	bltext3d->str= MEM_mallocN(12, "str");
 	strcpy(bltext3d->str, "Text");
 	bltext3d->pos= 4;
+
+	bltext3d->strinfo= MEM_callocN(12*sizeof(CharInfo), "strinfo");
+	bltext3d->totbox= bltext3d->actbox= 1;
+	bltext3d->tb= MEM_callocN(MAXTEXTBOX*sizeof(TextBox), "textbox");
+	bltext3d->tb[0].w = bltext3d->tb[0].h = 0.0;
 	
 	if( bltext3d == NULL )	/* bail out if add_curve() failed */
 		return ( EXPP_ReturnPyObjError
@@ -359,11 +356,11 @@ PyObject *M_Text3d_Get( PyObject * self, PyObject * args )
 
 static PyObject *generate_ModuleIntConstant(char *name, int value)
 {
-	PyObject *constant = M_constant_New();
+	PyObject *constant = PyConstant_New();
 
-	constant_insert((BPy_constant*)constant, 
+	PyConstant_Insert((BPy_constant*)constant, 
 		"value", PyInt_FromLong(value));
-	constant_insert((BPy_constant*)constant, 
+	PyConstant_Insert((BPy_constant*)constant, 
 		"name", PyString_FromString(name));
 
 	Py_INCREF(constant);
@@ -418,26 +415,31 @@ static PyObject *Text3dGetAttr( BPy_Text3d * self, char *name )
 	return Py_FindMethod( BPy_Text3d_methods, ( PyObject * ) self, name );
 }
 
-/*****************************************************************************/
-/* Function:    EffectSetAttr                                                */
-/* Description: This is a callback function for the BPy_Effect type. It   */
-/*              sets Effect Data attributes (member variables). */
-/*****************************************************************************/
+/*****************************************************************************
+ * Function:    Text3dSetAttr                                                
+ * Description: Callback function for the BPy_Effect type to  
+ *              sets Text3d Data attributes (member variables). 
+ * 
+ ****************************************************************************/
+
 static int Text3dSetAttr( BPy_Text3d * self, char *name, PyObject * value )
 {
 	return 0;		/* normal exit */
 }
 
-/*****************************************************************************/
-/* Function:    Text3dRepr                                                   */
-/* Description: This is a callback function for the BPy_Effect type. It      */
-/*              builds a meaninful string to represent effcte objects.       */
-/*****************************************************************************/
+
+/****************************************************************************
+ * Function:    Text3dRepr                                                   
+ * Description: Callback function for the BPy_Text3d type to It      
+ *               build a meaninful string to represent Text3d objects.      
+ *
+ ***************************************************************************/
 
 static PyObject *Text3dRepr( BPy_Text3d * self )
 {
-	char *str = "";
-	return PyString_FromString( str );
+	/* skip over CU in idname.  CUTEXT */
+	return PyString_FromFormat( "[Text3d \"%s\"]",
+								self->curve->id.name + 2 ); 
 }
 
 
@@ -488,26 +490,46 @@ static PyObject *Text3d_setName( BPy_Text3d * self, PyObject * args )
 static PyObject *Text3d_setText( BPy_Text3d * self, PyObject * args )
 {
 	char *text;
-	if( !PyArg_ParseTuple( args, "s", &text  ) )
-		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected string argument" ) );
-	if( self ) {
+
+	if( !PyArg_ParseTuple( args, "s", &text ) )
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+				"expected string argument" );
+
+	/*
+	 * If the text is currently being edited, then we have to put the
+	 * text into the edit buffer.
+	 */
+
+	if( G.obedit && G.obedit->data == self->curve ) {
+		short qual = G.qual;
+		G.qual = 0;		/* save key qualifier, then clear it */
+		self->curve->pos = self->curve->len = 0;
+		while ( *text )
+			do_textedit( 0, 0, *text++ );
+		G.qual = qual;
+	} else {
+		short len = (short)strlen(text);
 		MEM_freeN( self->curve->str );
-		self->curve->str = MEM_mallocN( strlen (text)+1, "str" );
+		self->curve->str = MEM_callocN( len+sizeof(wchar_t), "str" );
 		strcpy( self->curve->str, text );
-		self->curve->pos = strlen ( text );
-		self->curve->len = strlen ( text );
+		self->curve->pos = len;
+		self->curve->len = len;
+
+		if( self->curve->strinfo )
+			MEM_freeN( self->curve->strinfo );
+		/* don't know why this is +4, just duplicating load_editText() */
+		self->curve->strinfo = MEM_callocN( (len+4) *sizeof(CharInfo),
+				"strinfo");
 	}
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static PyObject *Text3d_getText( BPy_Text3d * self )
 {
-	if ( strlen(self->curve->str) )
-		return PyString_FromString (self->curve->str);
-	else 
-		return Py_None;
+	if( self->curve->str )
+		return PyString_FromString( self->curve->str );
+
+	Py_RETURN_NONE;
 }
 
 static PyObject* Text3d_getDrawMode(BPy_Text3d* self)
@@ -945,7 +967,6 @@ PyObject *M_Text3d_LoadFont( PyObject * self, PyObject * args )
 	vf= exist_vfont(fontfile);
 	if( vf )
 		return Font_CreatePyObject( vf );
-	/*	return EXPP_incr_ret( Py_None ); */
 	/* No use for that -- lukep
 	 else
 		vf= NULL;
@@ -954,8 +975,9 @@ PyObject *M_Text3d_LoadFont( PyObject * self, PyObject * args )
 
 	if( file || !strcmp (fontfile, "<builtin>") ) {
 		load_vfont( fontfile );
-		if(fclose) fclose( file );
-		if( (vf=exist_vfont( fontfile )) )
+		if(file) fclose( file );
+		vf = exist_vfont( fontfile );
+		if(vf)
 			return Font_CreatePyObject( vf );
 		return EXPP_incr_ret( Py_None );
 	}
