@@ -1,5 +1,5 @@
 /* 
- * $Id: Types.c,v 1.21 2005/05/22 17:40:00 stiv Exp $
+ * $Id: Types.c,v 1.34 2006/06/14 04:41:30 khughes Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -29,7 +29,37 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
 
-#include "Types.h"
+#include "Types.h" 
+
+/* 
+   stuff pasted from the old Types.h
+   is only need here now
+*/
+
+extern PyTypeObject Action_Type, Armature_Type;
+extern PyTypeObject Pose_Type;
+extern PyTypeObject BezTriple_Type, Bone_Type, Button_Type;
+extern PyTypeObject Camera_Type;
+extern PyTypeObject CurNurb_Type;
+extern PyTypeObject Curve_Type;
+extern PyTypeObject Effect_Type, Font_Type;
+extern PyTypeObject Image_Type, Ipo_Type, IpoCurve_Type;
+extern PyTypeObject Lamp_Type, Lattice_Type;
+extern PyTypeObject Material_Type, Metaball_Type, MTex_Type;
+extern PyTypeObject NMFace_Type, NMVert_Type, NMCol_Type, NMesh_Type;
+extern PyTypeObject MFace_Type, MVert_Type, PVert_Type, MEdge_Type, MCol_Type,
+	   Mesh_Type;
+extern PyTypeObject Object_Type;
+extern PyTypeObject Group_Type;
+extern PyTypeObject Particle_Type;
+extern PyTypeObject Scene_Type, RenderData_Type;
+extern PyTypeObject Text_Type, Text3d_Type, Texture_Type;
+extern PyTypeObject World_Type;
+extern PyTypeObject property_Type;
+extern PyTypeObject buffer_Type, constant_Type, euler_Type;
+extern PyTypeObject matrix_Type, quaternion_Type, rgbTuple_Type, vector_Type;
+extern PyTypeObject point_Type;
+extern PyTypeObject Modifier_Type, Modifiers_Type;
 
 char M_Types_doc[] = "The Blender Types module\n\n\
 This module is a dictionary of all Blender Python types";
@@ -48,10 +78,10 @@ struct PyMethodDef Null_methods[] = { {NULL, NULL, 0, NULL} };
 void types_InitAll( void )
 {
 	Action_Type.ob_type = &PyType_Type;
+	Pose_Type.ob_type = &PyType_Type;
 	Armature_Type.ob_type = &PyType_Type;
 	BezTriple_Type.ob_type = &PyType_Type;
 	Bone_Type.ob_type = &PyType_Type;
-	Build_Type.ob_type = &PyType_Type;
 	Button_Type.ob_type = &PyType_Type;
 	Camera_Type.ob_type = &PyType_Type;
 	CurNurb_Type.ob_type = &PyType_Type;
@@ -69,14 +99,19 @@ void types_InitAll( void )
 	NMFace_Type.ob_type = &PyType_Type;
 	NMVert_Type.ob_type = &PyType_Type;
 	NMesh_Type.ob_type = &PyType_Type;
+	MFace_Type.ob_type = &PyType_Type;
+   	MVert_Type.ob_type = &PyType_Type;
+   	PVert_Type.ob_type = &PyType_Type;
+   	MEdge_Type.ob_type = &PyType_Type;
+   	MCol_Type.ob_type = &PyType_Type;
+   	Mesh_Type.ob_type = &PyType_Type;
 	Object_Type.ob_type = &PyType_Type;
-	Particle_Type.ob_type = &PyType_Type;
+	Group_Type.ob_type = &PyType_Type;
 	RenderData_Type.ob_type = &PyType_Type;
 	Scene_Type.ob_type = &PyType_Type;
 	Text_Type.ob_type = &PyType_Type;
 	Text3d_Type.ob_type = &PyType_Type;
 	Texture_Type.ob_type = &PyType_Type;
-	Wave_Type.ob_type = &PyType_Type;
 	World_Type.ob_type = &PyType_Type;
 	buffer_Type.ob_type = &PyType_Type;
 	constant_Type.ob_type = &PyType_Type;
@@ -86,6 +121,10 @@ void types_InitAll( void )
 	rgbTuple_Type.ob_type = &PyType_Type;
 	vector_Type.ob_type = &PyType_Type;
 	property_Type.ob_type = &PyType_Type;
+	point_Type.ob_type = &PyType_Type;
+	PyType_Ready( &Modifier_Type );
+	PyType_Ready( &Modifiers_Type );
+
 }
 
 /*****************************************************************************/
@@ -103,73 +142,95 @@ PyObject *Types_Init( void )
 	/* The Blender Object Type */
 
 	PyDict_SetItemString( dict, "ObjectType",
-			      ( PyObject * ) & Object_Type );
+			      ( PyObject * ) &Object_Type );
 
 	/* Blender Object Data Types */
 
-	PyDict_SetItemString( dict, "SceneType", ( PyObject * ) & Scene_Type );
-	PyDict_SetItemString( dict, "RenderDataType",
-			      ( PyObject * ) & RenderData_Type );
+	PyDict_SetItemString( dict, "GroupType",
+			      ( PyObject * ) &Group_Type );
 
-	PyDict_SetItemString( dict, "NMeshType", ( PyObject * ) & NMesh_Type );
+	PyDict_SetItemString( dict, "SceneType", ( PyObject * ) &Scene_Type );
+	PyDict_SetItemString( dict, "RenderDataType",
+			      ( PyObject * ) &RenderData_Type );
+
+	PyDict_SetItemString( dict, "NMeshType", ( PyObject * ) &NMesh_Type );
 	PyDict_SetItemString( dict, "NMFaceType",
-			      ( PyObject * ) & NMFace_Type );
+			      ( PyObject * ) &NMFace_Type );
 	PyDict_SetItemString( dict, "NMVertType",
-			      ( PyObject * ) & NMVert_Type );
-	PyDict_SetItemString( dict, "NMColType", ( PyObject * ) & NMCol_Type );
+			      ( PyObject * ) &NMVert_Type );
+	PyDict_SetItemString( dict, "NMColType", ( PyObject * ) &NMCol_Type );
+
+	PyDict_SetItemString( dict, "MeshType", ( PyObject * ) &Mesh_Type );
+	PyDict_SetItemString( dict, "MFaceType",
+			      ( PyObject * ) &MFace_Type );
+	PyDict_SetItemString( dict, "MEdgeType",
+			      ( PyObject * ) &MEdge_Type );
+	PyDict_SetItemString( dict, "MVertType",
+			      ( PyObject * ) &MVert_Type );
+	PyDict_SetItemString( dict, "PVertType",
+			      ( PyObject * ) &PVert_Type );
+	PyDict_SetItemString( dict, "MColType", ( PyObject * ) &MCol_Type );
 
 	PyDict_SetItemString( dict, "ArmatureType",
-			      ( PyObject * ) & Armature_Type );
-	PyDict_SetItemString( dict, "BoneType", ( PyObject * ) & Bone_Type );
+			      ( PyObject * ) &Armature_Type );
+	PyDict_SetItemString( dict, "BoneType", ( PyObject * ) &Bone_Type );
 
 	PyDict_SetItemString( dict, "CurNurb_Type",
-			      ( PyObject * ) & CurNurb_Type );
-	PyDict_SetItemString( dict, "CurveType", ( PyObject * ) & Curve_Type );
+			      ( PyObject * ) &CurNurb_Type );
+	PyDict_SetItemString( dict, "CurveType", ( PyObject * ) &Curve_Type );
 
-	PyDict_SetItemString( dict, "IpoType", ( PyObject * ) & Ipo_Type );
+	PyDict_SetItemString( dict, "IpoType", ( PyObject * ) &Ipo_Type );
 	PyDict_SetItemString( dict, "MetaballType",
-			      ( PyObject * ) & Metaball_Type );
+			      ( PyObject * ) &Metaball_Type );
 
 	PyDict_SetItemString( dict, "CameraType",
-			      ( PyObject * ) & Camera_Type );
-	PyDict_SetItemString( dict, "ImageType", ( PyObject * ) & Image_Type );
-	PyDict_SetItemString( dict, "LampType", ( PyObject * ) & Lamp_Type );
-	PyDict_SetItemString( dict, "TextType", ( PyObject * ) & Text_Type );
-	PyDict_SetItemString( dict, "Text3dType", ( PyObject * ) & Text3d_Type );
+			      ( PyObject * ) &Camera_Type );
+	PyDict_SetItemString( dict, "ImageType", ( PyObject * ) &Image_Type );
+	PyDict_SetItemString( dict, "LampType", ( PyObject * ) &Lamp_Type );
+	PyDict_SetItemString( dict, "TextType", ( PyObject * ) &Text_Type );
+	PyDict_SetItemString( dict, "Text3dType", ( PyObject * ) &Text3d_Type );
 	PyDict_SetItemString( dict, "MaterialType",
-			      ( PyObject * ) & Material_Type );
+			      ( PyObject * ) &Material_Type );
 
 	PyDict_SetItemString( dict, "ButtonType",
-			      ( PyObject * ) & Button_Type );
+			      ( PyObject * ) &Button_Type );
 
 	PyDict_SetItemString( dict, "LatticeType",
-			      ( PyObject * ) & Lattice_Type );
+			      ( PyObject * ) &Lattice_Type );
 
 	PyDict_SetItemString( dict, "TextureType",
-			      ( PyObject * ) & Texture_Type );
-	PyDict_SetItemString( dict, "MTexType", ( PyObject * ) & MTex_Type );
+			      ( PyObject * ) &Texture_Type );
+	PyDict_SetItemString( dict, "MTexType", ( PyObject * ) &MTex_Type );
 
 	/* External helper Types available to the main ones above */
 
 	PyDict_SetItemString( dict, "vectorType",
-			      ( PyObject * ) & vector_Type );
+			      ( PyObject * ) &vector_Type );
 	PyDict_SetItemString( dict, "bufferType",
-			      ( PyObject * ) & buffer_Type );
+			      ( PyObject * ) &buffer_Type );
 	PyDict_SetItemString( dict, "constantType",
-			      ( PyObject * ) & constant_Type );
+			      ( PyObject * ) &constant_Type );
 	PyDict_SetItemString( dict, "rgbTupleType",
-			      ( PyObject * ) & rgbTuple_Type );
+			      ( PyObject * ) &rgbTuple_Type );
 	PyDict_SetItemString( dict, "matrix_Type",
-			      ( PyObject * ) & matrix_Type );
-	PyDict_SetItemString( dict, "eulerType", ( PyObject * ) & euler_Type );
+			      ( PyObject * ) &matrix_Type );
+	PyDict_SetItemString( dict, "eulerType", ( PyObject * ) &euler_Type );
 	PyDict_SetItemString( dict, "quaternionType",
-			      ( PyObject * ) & quaternion_Type );
+			      ( PyObject * ) &quaternion_Type );
 	PyDict_SetItemString( dict, "BezTripleType",
-			      ( PyObject * ) & BezTriple_Type );
+			      ( PyObject * ) &BezTriple_Type );
 	PyDict_SetItemString( dict, "ActionType",
-			      ( PyObject * ) & Action_Type );
+			      ( PyObject * ) &Action_Type );
+	PyDict_SetItemString( dict, "PoseType",
+			      ( PyObject * ) &Pose_Type );
 	PyDict_SetItemString( dict, "propertyType",
-			      ( PyObject * ) & property_Type );
+			      ( PyObject * ) &property_Type );
+	PyDict_SetItemString( dict, "pointType",
+			      ( PyObject * ) &point_Type );
+	PyDict_SetItemString( dict, "ModifierType",
+			      ( PyObject * ) &Modifier_Type );
+	PyDict_SetItemString( dict, "ModifiersType",
+			      ( PyObject * ) &Modifiers_Type );
 
 	return submodule;
 }
