@@ -1,5 +1,5 @@
 /* 
- * $Id: Window.c,v 1.34 2005/06/11 05:30:14 ianwill Exp $
+ * $Id: Window.c,v 1.40 2006/04/16 15:28:50 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,41 +32,36 @@
 */
 
 #include <Python.h>
-#include <stdio.h>
-
-#include <blendef.h>		/* OBACT */
-#include <BDR_editobject.h>	/* enter / leave editmode */
-#include <BKE_global.h>
-#include <BKE_library.h>
-#include <BKE_object.h>		/* for during_script() and during_scriptlink() */
-#include <BKE_scene.h>		/* scene_find_camera() */
-#include <BIF_usiblender.h>
-#include <BIF_mywindow.h>
-#include <BSE_headerbuttons.h>
-#include <BSE_filesel.h>
-#include <BIF_editmesh.h>	/* for undo_push_mesh() */
-#include <BIF_screen.h>
-#include <BIF_space.h>
-#include <BIF_drawtext.h>
-#include <BIF_mywindow.h> /* L/M/R_MOUSE bitflags */
-#include <BIF_spacetypes.h>
-#include <mydevice.h>
-#include <DNA_view3d_types.h>
-#include <DNA_screen_types.h>
-#include <DNA_space_types.h>
-#include <DNA_scene_types.h>
-#include <DNA_text_types.h>
-#include <DNA_vec_types.h>	/* for rcti struct */
 
+#include "BDR_editobject.h"	/* enter / leave editmode */
+#include "BKE_global.h"
+#include "BKE_main.h"
+#include "BKE_object.h"		/* for during_script() and during_scriptlink() */
+#include "BKE_scene.h"		/* scene_find_camera() */
+#include "BPI_script.h"
+#include "BIF_mywindow.h"
+#include "BSE_headerbuttons.h"
+#include "BSE_filesel.h"
+#include "BIF_editmesh.h"	/* for undo_push_mesh() */
+#include "BIF_screen.h"
+#include "BIF_space.h"
+#include "BIF_drawtext.h"
+#include "DNA_view3d_types.h"
+#include "DNA_space_types.h"
+#include "DNA_scene_types.h"
+#include "DNA_text_types.h"
+#include "DNA_object_types.h"
+#include "mydevice.h"
+#include "blendef.h"		/* OBACT */
 #include "windowTheme.h"
-#include "gen_utils.h"
-#include "matrix.h"
-#include "vector.h"
+#include "Mathutils.h"
 #include "constant.h"
+#include "gen_utils.h"
 
 
 /* See Draw.c */
 extern int EXPP_disable_force_draw;
+extern void setcameratoview3d(void);
 
 /*****************************************************************************/
 /* Python API function prototypes for the Window module.		*/
@@ -856,7 +851,7 @@ static PyObject *M_Window_GetViewMatrix( PyObject * self )
 
 	viewmat =
 		( PyObject * ) newMatrixObject( ( float * ) G.vd->viewmat, 4,
-						4 );
+						4, Py_WRAP );
 
 	if( !viewmat )
 		return EXPP_ReturnPyObjError( PyExc_MemoryError,
@@ -880,7 +875,7 @@ static PyObject *M_Window_GetPerspMatrix( PyObject * self )
 
 	perspmat =
 		( PyObject * ) newMatrixObject( ( float * ) G.vd->persmat, 4,
-						4 );
+						4, Py_WRAP );
 
 	if( !perspmat )
 		return EXPP_ReturnPyObjError( PyExc_MemoryError,
@@ -894,20 +889,23 @@ static PyObject *M_Window_EditMode( PyObject * self, PyObject * args )
 	short status = -1;
 	char *undo_str = "From script";
 	int undo_str_len = 11;
+	int do_undo = 1;
 
-	if( !PyArg_ParseTuple
-	    ( args, "|hs#", &status, &undo_str, &undo_str_len ) )
+	if( !PyArg_ParseTuple( args,
+				"|hs#i", &status, &undo_str, &undo_str_len, &do_undo ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected nothing or an int (bool) and a string as arguments" );
+				"expected optional int (bool), string and int (bool) as arguments" );
 
 	if( status >= 0 ) {
 		if( status ) {
 			if( !G.obedit )
 				enter_editmode(  );
 		} else if( G.obedit ) {
-			if( undo_str_len > 63 )
-				undo_str[63] = '\0';	/* 64 is max */
-			undo_push_mesh( undo_str );	/* use better solution after 2.34 */
+			if( do_undo && U.undosteps != 0 ) {
+				if( undo_str_len > 63 )
+					undo_str[63] = '\0';	/* 64 is max */
+				undo_push_mesh( undo_str );	/* use better solution after 2.34 */
+			}
 			exit_editmode( 1 );
 		}
 	}
@@ -991,7 +989,6 @@ static PyObject *M_Window_ViewLayers( PyObject * self, PyObject * args )
 static PyObject *M_Window_CameraView( PyObject * self, PyObject * args )
 {
 	short camtov3d = 0;
-	void setcameratoview3d( void );	/* view.c, used in toets.c */
 
 	if( !PyArg_ParseTuple( args, "|i", &camtov3d ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
@@ -1355,28 +1352,28 @@ PyObject *Window_Init( void )
 	if( dict )
 		PyDict_SetItemString( dict, "Theme", Theme_Init(  ) );
 
-	Types = M_constant_New(  );
-	Qual = M_constant_New(  );
-	MButs = M_constant_New(  );
+	Types = PyConstant_New(  );
+	Qual = PyConstant_New(  );
+	MButs = PyConstant_New(  );
 
 	if( Types ) {
 		BPy_constant *d = ( BPy_constant * ) Types;
 
-		constant_insert( d, "VIEW3D", PyInt_FromLong( SPACE_VIEW3D ) );
-		constant_insert( d, "IPO", PyInt_FromLong( SPACE_IPO ) );
-		constant_insert( d, "OOPS", PyInt_FromLong( SPACE_OOPS ) );
-		constant_insert( d, "BUTS", PyInt_FromLong( SPACE_BUTS ) );
-		constant_insert( d, "FILE", PyInt_FromLong( SPACE_FILE ) );
-		constant_insert( d, "IMAGE", PyInt_FromLong( SPACE_IMAGE ) );
-		constant_insert( d, "INFO", PyInt_FromLong( SPACE_INFO ) );
-		constant_insert( d, "SEQ", PyInt_FromLong( SPACE_SEQ ) );
-		constant_insert( d, "IMASEL", PyInt_FromLong( SPACE_IMASEL ) );
-		constant_insert( d, "SOUND", PyInt_FromLong( SPACE_SOUND ) );
-		constant_insert( d, "ACTION", PyInt_FromLong( SPACE_ACTION ) );
-		constant_insert( d, "TEXT", PyInt_FromLong( SPACE_TEXT ) );
-		constant_insert( d, "NLA", PyInt_FromLong( SPACE_NLA ) );
-		constant_insert( d, "SCRIPT", PyInt_FromLong( SPACE_SCRIPT ) );
-		constant_insert( d, "TIME", PyInt_FromLong( SPACE_TIME ) );
+		PyConstant_Insert( d, "VIEW3D", PyInt_FromLong( SPACE_VIEW3D ) );
+		PyConstant_Insert( d, "IPO", PyInt_FromLong( SPACE_IPO ) );
+		PyConstant_Insert( d, "OOPS", PyInt_FromLong( SPACE_OOPS ) );
+		PyConstant_Insert( d, "BUTS", PyInt_FromLong( SPACE_BUTS ) );
+		PyConstant_Insert( d, "FILE", PyInt_FromLong( SPACE_FILE ) );
+		PyConstant_Insert( d, "IMAGE", PyInt_FromLong( SPACE_IMAGE ) );
+		PyConstant_Insert( d, "INFO", PyInt_FromLong( SPACE_INFO ) );
+		PyConstant_Insert( d, "SEQ", PyInt_FromLong( SPACE_SEQ ) );
+		PyConstant_Insert( d, "IMASEL", PyInt_FromLong( SPACE_IMASEL ) );
+		PyConstant_Insert( d, "SOUND", PyInt_FromLong( SPACE_SOUND ) );
+		PyConstant_Insert( d, "ACTION", PyInt_FromLong( SPACE_ACTION ) );
+		PyConstant_Insert( d, "TEXT", PyInt_FromLong( SPACE_TEXT ) );
+		PyConstant_Insert( d, "NLA", PyInt_FromLong( SPACE_NLA ) );
+		PyConstant_Insert( d, "SCRIPT", PyInt_FromLong( SPACE_SCRIPT ) );
+		PyConstant_Insert( d, "TIME", PyInt_FromLong( SPACE_TIME ) );
 
 		PyModule_AddObject( submodule, "Types", Types );
 	}
@@ -1384,15 +1381,15 @@ PyObject *Window_Init( void )
 	if( Qual ) {
 		BPy_constant *d = ( BPy_constant * ) Qual;
 
-		constant_insert( d, "LALT", PyInt_FromLong( L_ALTKEY ) );
-		constant_insert( d, "RALT", PyInt_FromLong( R_ALTKEY ) );
-		constant_insert( d, "ALT", PyInt_FromLong( LR_ALTKEY ) );
-		constant_insert( d, "LCTRL", PyInt_FromLong( L_CTRLKEY ) );
-		constant_insert( d, "RCTRL", PyInt_FromLong( R_CTRLKEY ) );
-		constant_insert( d, "CTRL", PyInt_FromLong( LR_CTRLKEY ) );
-		constant_insert( d, "LSHIFT", PyInt_FromLong( L_SHIFTKEY ) );
-		constant_insert( d, "RSHIFT", PyInt_FromLong( R_SHIFTKEY ) );
-		constant_insert( d, "SHIFT", PyInt_FromLong( LR_SHIFTKEY ) );
+		PyConstant_Insert( d, "LALT", PyInt_FromLong( L_ALTKEY ) );
+		PyConstant_Insert( d, "RALT", PyInt_FromLong( R_ALTKEY ) );
+		PyConstant_Insert( d, "ALT", PyInt_FromLong( LR_ALTKEY ) );
+		PyConstant_Insert( d, "LCTRL", PyInt_FromLong( L_CTRLKEY ) );
+		PyConstant_Insert( d, "RCTRL", PyInt_FromLong( R_CTRLKEY ) );
+		PyConstant_Insert( d, "CTRL", PyInt_FromLong( LR_CTRLKEY ) );
+		PyConstant_Insert( d, "LSHIFT", PyInt_FromLong( L_SHIFTKEY ) );
+		PyConstant_Insert( d, "RSHIFT", PyInt_FromLong( R_SHIFTKEY ) );
+		PyConstant_Insert( d, "SHIFT", PyInt_FromLong( LR_SHIFTKEY ) );
 
 		PyModule_AddObject( submodule, "Qual", Qual );
 	}
@@ -1400,9 +1397,9 @@ PyObject *Window_Init( void )
 	if( MButs ) {
 		BPy_constant *d = ( BPy_constant * ) MButs;
 
-		constant_insert( d, "L", PyInt_FromLong( L_MOUSE ) );
-		constant_insert( d, "M", PyInt_FromLong( M_MOUSE ) );
-		constant_insert( d, "R", PyInt_FromLong( R_MOUSE ) );
+		PyConstant_Insert( d, "L", PyInt_FromLong( L_MOUSE ) );
+		PyConstant_Insert( d, "M", PyInt_FromLong( M_MOUSE ) );
+		PyConstant_Insert( d, "R", PyInt_FromLong( R_MOUSE ) );
 
 		PyModule_AddObject( submodule, "MButs", MButs );
 	}
