@@ -1,5 +1,5 @@
 /* 
- * $Id: World.c,v 1.29 2005/05/27 05:39:40 ianwill Exp $
+ * $Id: World.c,v 1.33 2006/04/16 15:28:50 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,25 +43,21 @@
  * (self, name = "MyName")
  */
 
-#include <BKE_main.h>
-#include <BKE_global.h>
-#include <BKE_object.h>
-#include <BKE_library.h>
-#include <BLI_blenlib.h>
-#include <BSE_editipo.h>
-#include <BIF_space.h>
-#include <mydevice.h>
+#include "World.h"  /*This must come first*/
 
-#include <DNA_scene_types.h>  /* for G.scene */
-
-#include "World.h"
+#include "DNA_scene_types.h"  /* for G.scene */
+#include "BKE_global.h"
+#include "BKE_world.h"
+#include "BKE_main.h"
+#include "BKE_library.h"
+#include "BLI_blenlib.h"
+#include "BSE_editipo.h"
+#include "BIF_space.h"
+#include "mydevice.h"
 #include "Ipo.h"
-
-
-#include "constant.h"
-#include "rgbTuple.h"
 #include "gen_utils.h"
 
+
 #define IPOKEY_ZENITH   0
 #define IPOKEY_HORIZON  1
 #define IPOKEY_MIST     2
@@ -278,8 +274,6 @@ PyTypeObject World_Type = {
 static PyObject *M_World_New( PyObject * self, PyObject * args,
 			      PyObject * kwords )
 {
-
-	World *add_world( char *name );
 	char *name = NULL;
 	BPy_World *pyworld;
 	World *blworld;
@@ -579,7 +573,7 @@ static PyObject *World_setSkytype( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "i", &skytype ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
-	self->world->skytype = skytype;
+	self->world->skytype = (short)skytype;
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -616,7 +610,7 @@ static PyObject *World_setMode( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "i", &mode ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
-	self->world->mode = mode;
+	self->world->mode = (short)mode;
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -665,7 +659,7 @@ static PyObject *World_setMistype( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "i", &mistype ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
-	self->world->mistype = mistype;
+	self->world->mistype = (short)mistype;
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -693,9 +687,9 @@ static PyObject *World_setHor( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "O", &listargs ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected list argument" ) );
-	self->world->horr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->horg = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->horb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->horr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->horg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->horb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -720,9 +714,9 @@ static PyObject *World_setZen( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "O", &listargs ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected list argument" ) );
-	self->world->zenr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->zeng = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->zenb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->zenr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->zeng = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->zenb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -755,9 +749,9 @@ static PyObject *World_setAmb( BPy_World * self, PyObject * args )
 	if( PyList_Size( listargs ) != 3 )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "wrong list size" ) );
-	self->world->ambr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->ambg = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->ambb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->ambr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->ambg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->ambb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -792,17 +786,17 @@ static PyObject *World_setStar( BPy_World * self, PyObject * args )
 	if( PyList_Size( listargs ) != 7 )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "wrong list size" ) );
-	self->world->starr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->starg = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->starb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->starr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->starg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->starb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	self->world->starsize =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
 	self->world->starmindist =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 4 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 4 ) );
 	self->world->stardist =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 5 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 5 ) );
 	self->world->starcolnoise =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 6 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 6 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -838,13 +832,13 @@ static PyObject *World_setMist( BPy_World * self, PyObject * args )
 	if( PyList_Size( listargs ) != 4 )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "wrong list size" ) );
-	self->world->misi = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->misi = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
 	self->world->miststa =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
 	self->world->mistdist =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	self->world->misthi =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -957,30 +951,43 @@ static PyObject *World_GetAttr( BPy_World * self, char *name )
 
 static int World_SetAttr( BPy_World * self, char *name, PyObject * value )
 {
+	PyObject *error=NULL;
 	PyObject *valtuple = Py_BuildValue( "(O)", value );
 
 	if( !valtuple )
 		return EXPP_ReturnIntError( PyExc_MemoryError,
 					    "WorldSetAttr: couldn't parse args" );
-	if( strcmp( name, "name" ) == 0 )
-		World_setName( self, valtuple );
-	if( strcmp( name, "skytype" ) == 0 )
-		World_setSkytype( self, valtuple );
-	if( strcmp( name, "mode" ) == 0 )
-		World_setMode( self, valtuple );
-	if( strcmp( name, "mistype" ) == 0 )
-		World_setMistype( self, valtuple );
-	if( strcmp( name, "hor" ) == 0 )
-		World_setHor( self, valtuple );
-	if( strcmp( name, "zen" ) == 0 )
-		World_setZen( self, valtuple );
-	if( strcmp( name, "amb" ) == 0 )
-		World_setAmb( self, valtuple );
-	if( strcmp( name, "star" ) == 0 )
-		World_setStar( self, valtuple );
-	if( strcmp( name, "mist" ) == 0 )
-		World_setMist( self, valtuple );
-	return 0;		/* normal exit */
+	else if( strcmp( name, "name" ) == 0 )
+		error = World_setName( self, valtuple );
+	else if( strcmp( name, "skytype" ) == 0 )
+		error = World_setSkytype( self, valtuple );
+	else if( strcmp( name, "mode" ) == 0 )
+		error = World_setMode( self, valtuple );
+	else if( strcmp( name, "mistype" ) == 0 )
+		error = World_setMistype( self, valtuple );
+	else if( strcmp( name, "hor" ) == 0 )
+		error = World_setHor( self, valtuple );
+	else if( strcmp( name, "zen" ) == 0 )
+		error = World_setZen( self, valtuple );
+	else if( strcmp( name, "amb" ) == 0 )
+		error = World_setAmb( self, valtuple );
+	else if( strcmp( name, "star" ) == 0 )
+		error = World_setStar( self, valtuple );
+	else if( strcmp( name, "mist" ) == 0 )
+		error = World_setMist( self, valtuple );
+	else {
+		Py_DECREF( valtuple );
+		return ( EXPP_ReturnIntError( PyExc_KeyError,
+						  "attribute not found" ) );
+	}
+
+	Py_DECREF(valtuple);
+
+	if( error != Py_None )
+		return -1;
+
+	Py_DECREF( Py_None );
+	return 0;
 }
 
 /**
@@ -1104,37 +1111,37 @@ static PyObject *World_insertIpoKey( BPy_World * self, PyObject * args )
 	map = texchannel_to_adrcode(self->world->texact);
 
 	if(key == IPOKEY_ZENITH) {
-		insertkey((ID *)self->world, WO_ZEN_R);
-		insertkey((ID *)self->world, WO_ZEN_G);
-		insertkey((ID *)self->world, WO_ZEN_B);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_R);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_G);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_B);
 	}
 	if(key == IPOKEY_HORIZON) {
-		insertkey((ID *)self->world, WO_HOR_R);
-		insertkey((ID *)self->world, WO_HOR_G);
-		insertkey((ID *)self->world, WO_HOR_B);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_R);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_G);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_B);
 	}
 	if(key == IPOKEY_MIST) {
-		insertkey((ID *)self->world, WO_MISI);
-		insertkey((ID *)self->world, WO_MISTDI);
-		insertkey((ID *)self->world, WO_MISTSTA);
-		insertkey((ID *)self->world, WO_MISTHI);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISI);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTDI);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTSTA);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTHI);
 	}
 	if(key == IPOKEY_STARS) {
-		insertkey((ID *)self->world, WO_STAR_R);
-		insertkey((ID *)self->world, WO_STAR_G);
-		insertkey((ID *)self->world, WO_STAR_B);
-		insertkey((ID *)self->world, WO_STARDIST);
-		insertkey((ID *)self->world, WO_STARSIZE);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_R);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_G);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_B);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARDIST);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARSIZE);
 	}
 	if(key == IPOKEY_OFFSET) {
-		insertkey((ID *)self->world, map+MAP_OFS_X);
-		insertkey((ID *)self->world, map+MAP_OFS_Y);
-		insertkey((ID *)self->world, map+MAP_OFS_Z);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_X);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Y);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Z);
 	}
 	if(key == IPOKEY_SIZE) {
-		insertkey((ID *)self->world, map+MAP_SIZE_X);
-		insertkey((ID *)self->world, map+MAP_SIZE_Y);
-		insertkey((ID *)self->world, map+MAP_SIZE_Z);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_X);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Y);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Z);
 	}
 
 	allspace(REMAKEIPO, 0);
