@@ -12,7 +12,9 @@ A Blender Curve can consist of multiple curves. Try converting a Text object to
 type Bezier or Nurb.  The underlying curves can be accessed with
 the [] operator.  Operator [] returns an object of type CurNurb.
 
-The Curve module also supports the Python iterator interface.  This means you can access the curves in a Curve or the control points in a CurNurb using a python for statement.
+The Curve module also supports the Python iterator interface.  This means you
+can access the curves in a Curve and the control points in a CurNurb using a
+Python B{for} statement.
 
 
 Add a Curve to a Scene Example::
@@ -61,19 +63,34 @@ class Curve:
   This object gives access to Curve-specific data in Blender.
   
   @ivar name: The Curve Data name.
+  @type name: string
   @ivar pathlen: The Curve Data path length.
+  @type pathlen: int
   @ivar totcol: The Curve Data maximal number of linked materials.
-  @ivar flag: The Curve Data flag value; see function getFlag for the semantics.
+  @type totcol: int
+  @ivar flag: The Curve Data flag value; see L{getFlag()} for the semantics.
   @ivar bevresol: The Curve Data bevel resolution.
+  @type bevresol: float
   @ivar resolu: The Curve Data U-resolution.
+  @type resolu: float
   @ivar resolv: The Curve Data V-resolution.
+  @type resolv: float
   @ivar width: The Curve Data width.
+  @type width: float
   @ivar ext1: The Curve Data extent 1(for bevels).
+  @type ext1: float
   @ivar ext2: The Curve Data extent2 (for bevels).
+  @type ext2: float
   @ivar loc: The Curve Data location(from the center).
+  @type loc: list of 3 floats
   @ivar rot: The Curve Data rotation(from the center).
+  @type rot: list of 3 floats
   @ivar size: The Curve Data size(from the center).
+  @type size: list of 3 floats
   @ivar bevob: The Curve Bevel Object
+  @type bevob: Blender L{Object<Object.Object>} or None
+  @ivar key: The Key object associated with this Curve, if any.
+  @type key: Blender L{Key<Key.Key>}
   """
 
   def getName():
@@ -85,7 +102,7 @@ class Curve:
   def setName(name):
     """
     Set the name of this Curve Data object.
-    @rtype: PyNone
+    @rtype: None
     @type name: string
     @param name: The new name.
     """
@@ -100,7 +117,7 @@ class Curve:
   def setPathLen(len):
     """
     Set this Curve's path length.
-    @rtype: PyNone
+    @rtype: None
     @type len: int
     @param len: the new curve's length.
     """
@@ -115,7 +132,7 @@ class Curve:
   def setTotcol(totcol):
     """
     Set the number of materials linked to the Curve.
-    @rtype: PyNone
+    @rtype: None
     @type totcol: int
     @param totcol: number of materials linked.
     """
@@ -136,7 +153,7 @@ class Curve:
   def setFlag(val):
     """
     Set the Curve flag value.  The flag corresponds to the Blender settings for 3D, Front, Back, CurvePath and CurveFollow.  This parameter is a bitfield.
-    @rtype: PyNone
+    @rtype: None
     @type val: integer bitfield
     @param val : The Curve's flag bits.  See L{getFlag} for the meaning of the individual bits.
     """
@@ -150,7 +167,7 @@ class Curve:
   def setBevresol(bevelresol):
     """
     Set the Curve's bevel resolution value.
-    @rtype: PyNone
+    @rtype: None
     @type bevelresol: float
     @param bevelresol: The new Curve's bevel resolution value.
     """
@@ -164,7 +181,7 @@ class Curve:
   def setResolu(resolu):
     """
     Set the Curve's U-resolution value.
-    @rtype: PyNone
+    @rtype: None
     @type resolu: float
     @param resolu: The new Curve's U-resolution value.
     """
@@ -178,7 +195,7 @@ class Curve:
   def setResolv(resolv):
     """
     Set the Curve's V-resolution value.
-    @rtype: PyNone
+    @rtype: None
     @type resolv: float
     @param resolv: The new Curve's V-resolution value.
     """
@@ -192,7 +209,7 @@ class Curve:
   def setWidth(width):
     """
     Set the Curve's width value. 
-    @rtype: PyNone
+    @rtype: None
     @type width: float
     @param width: The new Curve's width value. 
     """
@@ -206,7 +223,7 @@ class Curve:
   def setExt1(ext1):
     """
     Set the Curve's ext1 value. 
-    @rtype: PyNone
+    @rtype: None
     @type ext1: float
     @param ext1: The new Curve's ext1 value. 
     """
@@ -220,26 +237,35 @@ class Curve:
   def setExt2(ext2):
     """
     Set the Curve's ext2 value.
-    @rtype: PyNone 
+    @rtype: None 
     @type ext2: float
     @param ext2: The new Curve's ext2 value. 
     """
 
   def getControlPoint(numcurve,numpoint):
     """
-    Get the curve's control point value. The numpoint arg is an index into the list of points and starts with 0.
+    Get the curve's control point value (B{deprecated}).  The numpoint arg
+    is an index into the list of points and starts with 0.  B{Note}: new
+    scripts should use the [] operator on Curves and CurNurbs.  Example::
+      curve = Blender.Curve.Get('Curve')
+      p0 = curve[0][0]    # get first point from first nurb
+             # -- OR --
+      nurb = curve[0]     # get first nurb
+      p0 = nurb[0]        # get nurb's first point
+
     @type numcurve: int
     @type numpoint: int
     @rtype: list of floats
     @return: depends upon the curve's type.
       - type Bezier : a list of nine floats.  Values are x, y, z for handle-1, vertex and handle-2 
       - type Nurb : a list of 4 floats.  Values are x, y, z, w.
+
     """
 
   def setControlPoint( numcurve, numpoint, controlpoint):
     """
     Set the Curve's controlpoint value.   The numpoint arg is an index into the list of points and starts with 0.
-    @rtype: PyNone
+    @rtype: None
     @type numcurve: int
     @type numpoint: int
     @type controlpoint: list
@@ -251,14 +277,15 @@ class Curve:
 
   def appendPoint( numcurve, new_control_point ):
     """
-      add a new control point to the indicated curve.
-      @rtype: PyNone
+      Add a new control point to the indicated curve (B{deprecated}).
+      New scripts should use L{CurNurb.append()}.
+      @rtype: None
       @type numcurve: int
-      @type new_control_point: list xyzw or BezTriple
+      @type new_control_point: list of floats or BezTriple
       @param numcurve:  index for spline in Curve, starting from 0
       @param new_control_point: depends on curve's type.
         - type Bezier: a BezTriple 
-	- type Nurb: a list of four floats for the xyzw values
+        - type Nurb: a list of four or five floats for the xyzw values
       @raise AttributeError:  throws exception if numcurve is out of range.
     """
 
@@ -280,7 +307,7 @@ class Curve:
   def setLoc(location):
     """
     Set the curve's location value.
-    @rtype: PyNone 
+    @rtype: None 
     @type location: list[3]
     @param location: The new Curve's location values. 
     """
@@ -294,7 +321,7 @@ class Curve:
   def setRot(rotation):
     """
     Set the Curve's rotation value. 
-    @rtype: PyNone
+    @rtype: None
     @type rotation: list[3]
     @param rotation: The new Curve's rotation values. 
     """
@@ -308,7 +335,7 @@ class Curve:
   def setSize(size):
     """
     Set the curve size value.
-    @rtype: PyNone 
+    @rtype: None 
     @type size: list[3]
     @param size: The new Curve's size values. 
     """
@@ -323,35 +350,53 @@ class Curve:
   def getBevOb():
     """
     Returns the Bevel Object (BevOb) assigned to the Curve.
-    @rtype: Blender Object or PyNone
+    @rtype: Blender Object or None
     @return: Bevel Object (BevOb) assigned to the Curve.
     """
 
   def setBevOb( object ):
     """
     Assign a Bevel Object (BevOb) to the Curve.  Passing None as the object parameter removes the bevel.
-    @rtype: PyNone
-    @return: PyNone
+    @rtype: None
+    @return: None
     @type object: Curve type Blender Object
     @param object: Blender Object to assign as Bevel Object (BevOb)
-    @raise TypeError: throws exception if the parameter is not a Curve type Blender Object or PyNone
+    @raise TypeError: throws exception if the parameter is not a Curve type Blender Object or None
+    """
+
+  def getTaperOb():
+    """
+    Returns the Taper Object (TaperOb) assigned to the Curve.
+    @rtype: Blender Object or None
+    @return: Taper Object (TaperOb) assigned to the Curve.
+    """
+
+  def setTaperOb( object ):
+    """
+    Assign a Taper Object (TaperOb) to the Curve.  Passing None as the object parameter removes the taper.
+    @rtype: None
+    @return: None
+    @type object: Curve type Blender Object
+    @param object: Blender Object to assign as Taper Object (TaperOb)
+    @raise TypeError: throws exception if the parameter is not a Curve type Blender Object or None
     """
 
   def update():
     """
     Updates display list for a Curve.
     Used after making changes to control points.
-    
     You B{must} use this if you want to see your changes!
-    @rtype: PyNone
-    @return: PyNone
+    @rtype: None
+    @return: None
     """
 
   def isNurb( curve_num ):
       """
-      method used to determine whether a CurNurb is of type Bezier or of type Nurb.
+      Tells type of a CurNurb (B{deprecated}).
+      New scripts should use L{CurNurb.isNurb()}.
+
       @rtype: integer
-      @return:  Zero if curve is type Bezier, One if curve is of type Nurb.
+      @return:  Zero if curve is type Bezier, one if curve is of type Nurb.
       @type curve_num: integer
       @param curve_num: zero-based index into list of curves in this Curve.
       @raise AttributeError:  throws exception if curve_num is out of range.
@@ -359,7 +404,8 @@ class Curve:
 
   def isCyclic( curve_num ):
       """
-      Boolean method checks whether the curve is cyclic (closed) or not.
+      Tells whether or not a CurNurb is cyclic (closed) (B{deprecated}).
+      New scripts should use L{CurNurb.isCyclic()}.
 
       @rtype: boolean
       @return: True if is cyclic, False if not
@@ -376,12 +422,21 @@ class Curve:
 
   def getNumPoints( curve_num ):
       """
-      Get the number of control points in the curve.
+      Get the number of control points in the curve (B{deprecated}).
+      New scripts should use the len operator (I{len(curve)}).
       @type curve_num: integer
       @param curve_num: zero-based index into list of curves in this Curve
       @rtype: integer
       """
 
+  def getKey():
+      """
+      Return the L{Key<Key.Key>} object containing the keyframes for this
+      curve, if any.
+      @rtype: L{Key<Key.Key>} object or None
+      """
+
+
 class CurNurb:
     """
     The CurNurb Object
@@ -392,16 +447,19 @@ class CurNurb:
 
     The CurNurb also supports the sequence protocol which means you can access the control points of a CurNurb using the [] operator.
 
-    @ivar flagU: The CurNurb knot flag U (0: uniform, 1: endpoints, 2: bezier)
-    @ivar flagV: The CurNurb knot flag V (0: uniform, 1: endpoints, 2: bezier)
+    @ivar flagU: The CurNurb knot flag U.  See L{setFlagU} for description.
+    @type flagU: int
+    @ivar flagV: The CurNurb knot flag V.  See L{setFlagU} for description.
+    @type flagV: int
     @ivar type: The type of the curve (Poly: 0, Bezier: 1, NURBS: 4)
+    @type type: int
     """
 	
     def __setitem__( n, point ):
       """
 	  Replace the Nth point in the curve. The type of the argument must match the type of the curve. List of 4 floats (optional 5th float is the tilt value in radians) for Nurbs or BezTriple for Bezier.
-      @rtype: PyNone
-      @return: PyNone
+      @rtype: None
+      @return: None
       @type n: integer
       @param n: the index of the element to replace
       @type point: BezTriple or list of 4 floats (optional 5th float is the tilt value in radians)
@@ -420,8 +478,8 @@ class CurNurb:
     def append( new_point ):
       """
       Appends a new point to a curve.  This method appends points to both Bezier and Nurb curves. The type of the argument must match the type of the curve. List of 4 floats (optional 5th float is the tilt value in radians) for Nurbs or BezTriple for Bezier.
-      @rtype: PyNone
-      @return: PyNone
+      @rtype: None
+      @return: None
       @type new_point: BezTriple or list of 4 floats (optional 5th float is the tilt value in radians)
       @param new_point: the new point to be appended to the curve.  The new point can be either a BezTriple type or a list of 4 floats in x,y,z,w (optionally tilt in radians as 5th value) format for a Nurb curve.
       """
@@ -429,8 +487,8 @@ class CurNurb:
     def setMatIndex( index ):
       """
       Sets the Material index for this CurNurb.
-      @rtype: PyNone
-      @return: PyNone
+      @rtype: None
+      @return: None
       @type index:  integer
       @param index: the new value for the Material number of this CurNurb.  No range checking is done.
       """
@@ -458,48 +516,53 @@ class CurNurb:
 
     def getFlagU():
       """
-      Get the CurNurb knot flag U 
+      Get the CurNurb knot flag U.  
       @rtype: integer
-      @return: 0 - uniform, 1 - endpoints, 2 - bezier
+      @return: See L{setFlagU} for description of return value.
       """
 
-    def setFlagU( value ):
+    def setFlagU( flag ):
       """
-      Set the CurNurb knot flag U (knots are recalculated automatically)
-      @type value: integer
-      @param value: CurNurb knot flag (0 - uniform, 1 - endpoints, 2 - bezier)
-      @rtype: PyNone
-      @return: PyNone
+      Set the entire CurNurb knot flag U (knots are recalculated automatically).
+      The flag can be one of six values:
+           - 0 or 1: uniform knots
+           - 2 or 3: endpoints knots
+           - 4 or 5: bezier knots
+      Bit 0 controls whether or not the curve is cyclic (1 = cyclic).
+      @type flag: integer
+      @param flag: CurNurb knot flag
+      @rtype: None
+      @return: None
       """
 
     def getFlagV():
       """
-      Get the CurNurb knot flag V 
+      Get the CurNurb knot flag V.
       @rtype: integer
-      @return: 0 - uniform, 1 - endpoints, 2 - bezier
+      @return: See L{setFlagU} for description of return value.
       """
 
     def setFlagV( value ):
       """
-      Set the CurNurb knot flag V (knots are recalculated automatically)
+      Set the CurNurb knot flag V (knots are recalculated automatically).
       @type value: integer
-      @param value: CurNurb knot flag (0 - uniform, 1 - endpoints, 2 - bezier)
-      @rtype: PyNone
-      @return: PyNone
+      @param value: See L{setFlagU} for description of return.
+      @rtype: None
+      @return: None
       """
 
     def getType():
       """
-      Get the type of the curve 
+      Get the type of the curve.
       @rtype: integer
       @return:  0 - Poly, 1 - Bezier, 4 - NURBS
       """
 
-    def getType( value ):
+    def setType( value ):
       """
       Set the type of the curve and converts the curve to its new type if needed
       @type value: integer
       @param value: CurNurb type flag (0 - Poly, 1 - Bezier, 4 - NURBS)
-      @rtype: PyNone
-      @return: PyNone
+      @rtype: None
+      @return: None
       """
