@@ -7,8 +7,10 @@ Draw
 ====
 
 B{New}:
- - access to ascii values in L{events<Register>} callbacks;
+ - access to ASCII values in L{events<Register>} callbacks;
  - 'large' fonts for L{Text} and L{GetStringWidth}.
+ - Pop-up blocks with L{PupBlock}
+ - Color Picker button with L{ColorPicker}
 
 This module provides access to a B{windowing interface} in Blender.  Its widgets
 include many kinds of buttons: push, toggle, menu, number, string, slider,
@@ -243,7 +245,7 @@ def Register(draw = None, event = None, button = None):
       they will stay until Draw.Exit is called.  It's enough to redraw the
       screen, when a relevant event is caught.
   @note: only during the B{event} callback: the L{Blender}.ascii variable holds
-      the ascii integer value (if it exists and is valid) of the current event.
+      the ASCII integer value (if it exists and is valid) of the current event.
   """
 
 def Redraw(after = 0):
@@ -263,10 +265,11 @@ def Draw():
 def Create(value):
   """
   Create a default Button object.
-  @type value: int, float or string
+  @type value: int, float, string or 3 floats
   @param value: The value to store in the button.
   @rtype: Blender Button
   @return: The Button created.
+  @note: String values must have less then 400 characters.
   """
 
 def PushButton(name, event, x, y, width, height, tooltip = None):
@@ -323,7 +326,7 @@ def PupIntInput(text, default, min, max):
   """
   Create an integer number input pop-up.
 
-  This allows python to use Blender's integer number popup input.
+  This allows python to use Blender's integer number pop-up input.
 
   Example::
     default = 50
@@ -338,13 +341,13 @@ def PupIntInput(text, default, min, max):
       print 'no user input'
     
   @type text: string
-  @param text: The text that is displayed in the popup.
+  @param text: The text that is displayed in the pop-up.
   @type default: int
-  @param default: The value that the popup is set to initially.
+  @param default: The value that the pop-up is set to initially.
   @type min: int
-  @param min: The lowest value the popup will allow.
+  @param min: The lowest value the pop-up will allow.
   @type max: int
-  @param max: The highest value the popup will allow.
+  @param max: The highest value the pop-up will allow.
   @rtype: int
   @return: the number chosen or None if none was chosen.
   """
@@ -353,7 +356,7 @@ def PupFloatInput(text, default, min, max, clickStep, floatLen):
   """
   Create a floating point number input pop-up.
   
-  This allows python to use Blender's floating point popup input.
+  This allows python to use Blender's floating point pop-up input.
 
   Example::
     default = 50
@@ -370,13 +373,13 @@ def PupFloatInput(text, default, min, max, clickStep, floatLen):
       print 'no user input'
     
   @type text: string
-  @param text: The text that is displayed in the popup.
+  @param text: The text that is displayed in the pop-up.
   @type default: float
-  @param default: The value that the popup is set to initially.
+  @param default: The value that the pop-up is set to initially.
   @type min: float
-  @param min: The lowest value the popup will allow.
+  @param min: The lowest value the pop-up will allow.
   @type max: float
-  @param max: The highest value the popup will allow.
+  @param max: The highest value the pop-up will allow.
   @type clickStep: int
   @param clickStep: How much is incremented per user click, 100 will increment 1.0, 10 will increment 0.1 etc.
   @type floatLen: int
@@ -389,23 +392,67 @@ def PupStrInput(text, default, max = 20):
   """
   Create a string input pop-up.
   
-  This allows python to use Blender's string popup input.
+  This allows python to use Blender's string pop-up input.
 
   Example::
     Blender.Draw.PupStrInput("Name:", "untitled", 25)
     
   @type text: string
-  @param text: The text that is displayed in the popup.
+  @param text: The text that is displayed in the pop-up.
   @type default: string
-  @param default: The value that the popup is set to initially.  If it's longer
+  @param default: The value that the pop-up is set to initially.  If it's longer
       then 'max', it's truncated.
   @type max: int
-  @param max: The most characters the popup input will allow.  If not given
+  @param max: The most characters the pop-up input will allow.  If not given
       it defaults to 20 chars.  It should be in the range [1, 100].
   @rtype: string
   @return: The text entered by the user or None if none was chosen.
   """
 
+def PupBlock(title, sequence):
+	"""
+	Display a pop-up block.
+	
+	Possible formats for the items in the sequence parameter.
+	(Value are objects created with L{Create})
+		- string:	Defines a label
+		- (string, Value, string): Defines a toggle button. The first string is the text on the button, the optional second string is the tooltip.
+		- (string, Value, min, max, string): Defines a numeric or string button, depending on the content of Value.  The first string is the text on the button, the optional second string is the tooltip. I{For string, max is the maximum length of the string and min is unused.}
+		
+	Example::
+		import Blender
+		
+		text = Blender.Draw.Create("short text")
+		f = Blender.Draw.Create(1.0)
+		i = Blender.Draw.Create(2)
+		tog = Blender.Draw.Create(0)
+		
+		block = []
+		
+		block.append(("Name: ", text, 0, 30, "this is some tool tip"))
+		block.append("Some Label")
+		block.append(("Value: ", f, 0.0, 100.0))
+		block.append(("Value: ", i, 0, 100))
+		block.append(("Option", tog, "another tooltip"))
+		
+		retval = Blender.Draw.PupBlock("PupBlock test", block)
+		
+		print "PupBlock returned", retval
+		
+		print "text\\t", text
+		print "float\\t", f
+		print "int\\t", i
+		print "toggle\\t", tog
+
+	@warning: On cancel, the Value objects are brought back to there initial values except for string values which will still contain the modified values.
+	@type title: string
+	@param title: The title of the block.
+	@param sequence: A sequence defining what the block contains.
+		The order of the list is the order of appearance, from top down.
+	@rtype: int
+	@return: 1 if the pop-up is confirmed, 0 otherwise
+	"""
+
 def Menu(name, event, x, y, width, height, default, tooltip = None):
   """
   Create a new Menu Button object.
@@ -535,6 +582,31 @@ def Slider(name, event, x, y, width, height, initial, min, max, realtime = 1,
 #  @return: The Button created.
 #  """
 
+def ColorPicker(event, x, y, width, height, initial, tooltip = None):
+  """
+  Create a new Color Picker Button object.
+  @type event: int
+  @param event: The event number to pass to the button event function when
+      activated.
+  @type x: int
+  @type y: int
+  @param x: The lower left x (horizontal) coordinate of the button.
+  @param y: The lower left y (vertical) coordinate of the button.
+  @type width: int
+  @type height: int
+  @param width: The button width.
+  @param height: The button height.
+  @type initial: 3-float tuple
+  @param initial:  The initial color value. All values must be between 0 and 1
+  @type tooltip: string
+  @param tooltip: The button's tooltip (the string that appears when the mouse
+      is kept over the button).
+  @rtype: Blender Button
+  @return: The Button created.
+  @note: The color picker will not work if the Register's event function is None.
+  @note: Using the same button variable with more then 1 color picker at a time will corrupt memory.
+  """
+
 def Number(name, event, x, y, width, height, initial, min, max, tooltip = None):
   """
   Create a new Number Button object.
@@ -562,6 +634,19 @@ def Number(name, event, x, y, width, height, initial, min, max, tooltip = None):
       is kept over the button).
   @rtype: Blender Button
   @return: The Button created.
+
+  I{B{Example:}}
+
+  This example draws a single floating point value::
+    from Blender import Draw
+    b= Draw.Create(0.0) # Data for floating point button
+    def bevent(evt):
+      print 'My Button event:', evt
+    def gui():
+      global b
+      b= Draw.Number('value: ', 1000, 0,0, 200, 20, b.val, 0,10, 'some text tip')
+
+    Draw.Register(gui, None, bevent) # we are not going to worry about keyboard and mouse events
   """
 
 
@@ -606,12 +691,19 @@ def GetStringWidth(string, fontsize = 'normal'):
 def Text(string, fontsize = 'normal'):
   """
   Draw a string on the screen.
+
+  Text location is set using the OpenGL raster location functions L{BGL.glRasterPos} before the text is drawn.
+  This sets the text location from the lower left corner of the current window.
+
+  Text color is set using the OpenGL color functions L{BGL.glColor} before the text is drawn.
+  
   @type string: string
   @param string: The text string to draw.
   @type fontsize: string
   @param fontsize: The size of the font: 'large', 'normal', 'small' or 'tiny'.
   @rtype: int
   @return: The width of I{string} drawn with the chosen I{fontsize}.
+  @note: For drawing text in the 3d view see the workaround in L{BGL.glRasterPos}
   """
 
 def Image(image, x, y, zoomx=1.0, zoomy=1.0, clipx=0, clipy=0, clipw=-1, cliph=-1):
@@ -632,6 +724,10 @@ def Image(image, x, y, zoomx=1.0, zoomy=1.0, clipx=0, clipy=0, clipw=-1, cliph=-
   either the width or the height of the clipping rectangle are negative then
   the corresponding dimension (width or height) is set to include as much of 
   the image as possible.
+  
+  For drawing images with alpha blending with the background you will need to enable blending as shown in the example.
+  
+  
 
   Example::
    import Blender
@@ -640,7 +736,12 @@ def Image(image, x, y, zoomx=1.0, zoomy=1.0, clipx=0, clipy=0, clipw=-1, cliph=-
    myimage = Image.Load('myimage.png')
 
    def gui():
+        BGL.glEnable( BGL.GL_BLEND ) # Only needed for alpha blending images with background.
+        BGL.glBlendFunc(BGL.GL_SRC_ALPHA, BGL.GL_ONE_MINUS_SRC_ALPHA) 
+        
         Draw.Image(myimage, 50, 50)
+        
+        BGL.glDisable( BGL.GL_BLEND )
    def event(evt, val):
         if evt == Draw.ESCKEY:
               Draw.Exit()
@@ -680,6 +781,6 @@ class Button:
   The Button object
   =================
     This object represents a button in Blender's GUI.
-  @type val: int or float or string (depends on button type).
+  @type val: int or float, string or 3-float tuple (depends on button type).
   @ivar val: The button's value.
   """
