@@ -31,14 +31,19 @@ def Load (filename):
   @return: A Blender Image object with the data from I{filename}.
   """
 
-def New (name):
+def New (name, width, height, depth):
   """
-  Create a new Image object (not implemented yet!).
+  Create a new Image object.
   @type name: string
   @param name: The name of the new Image object.
+  @type width: int
+  @param width: The width of the new Image object, between 1 and 5000.
+  @type height: int
+  @param height: The height of the new Image object, between 1 and 5000.
+  @type depth: int
+  @param depth: The colour depth of the new Image object. (8:Grey, 24:RGB, 32:RGBA). (Not implimented yet, all new images will be 24bit)
   @rtype: Blender Image
   @return: A new Blender Image object.
-  @warn: This function wasn't implemented yet.  It simply returns None.
   """
 
 def Get (name = None):
@@ -52,6 +57,13 @@ def Get (name = None):
       - (): A list with all Image objects in the current scene.
   """
 
+def GetCurrent ():
+  """
+  Get the currently displayed Image from Blenders UV/Image window.
+  When multiple images are displayed, the last active UV/Image windows image is used.
+  @rtype: Blender Image
+  @return: The Current Blender Image, If there is no current image it returns None.
+  """
 
 class Image:
   """
@@ -67,6 +79,10 @@ class Image:
      axis.
   @ivar yrep: Texture tiling: the number of repetitions in the y (vertical)
      axis.
+  @ivar start: Texture's animation start frame [0, 128].
+  @ivar end: Texture's animation end frame [0, 128].
+  @ivar speed: Texture's animation speed [1, 100].
+  @ivar packed: Boolean, True when the Texture is packed (readonly).
   @ivar bindcode: Texture's bind code (readonly).
   """
 
@@ -96,15 +112,27 @@ class Image:
 
   def getPixelF(x, y):
     """
-	Get the the colors of the current pixel in the form [r,g,b,a].
-	Returned values are floats normalized to 0.0 - 1.0.
-	Pixel coordinates are in the range from 0 to N-1.  See L{getMaxXY}
-	@returns: [ r, g, b, a]
-	@rtype: list of 4 floats
-	@type x: int
-	@type y: int
-	@param x:  the x coordinate of pixel.
-	@param y:  the y coordinate of pixel.  
+    Get the the colors of the current pixel in the form [r,g,b,a].
+    Returned values are floats normalized to 0.0 - 1.0.
+    Pixel coordinates are in the range from 0 to N-1.  See L{getMaxXY}
+    @returns: [ r, g, b, a]
+    @rtype: list of 4 floats
+    @type x: int
+    @type y: int
+    @param x:  the x coordinate of pixel.
+    @param y:  the y coordinate of pixel.  
+    """
+  def getPixelI(x, y):
+    """
+    Get the the colors of the current pixel in the form [r,g,b,a].
+    Returned values are ints normalized to 0 - 255.
+    Pixel coordinates are in the range from 0 to N-1.  See L{getMaxXY}
+    @returns: [ r, g, b, a]
+    @rtype: list of 4 ints
+    @type x: int
+    @type y: int
+    @param x:  the x coordinate of pixel.
+    @param y:  the y coordinate of pixel.  
     """
 
   def getMaxXY():
@@ -114,6 +142,13 @@ class Image:
     @rtype: list of 2 ints
     """
 
+  def getMinXY():
+    """
+    Get the x & y origin for the image. Image coordinates range from 0 to size-1.
+    @returns: [x, y]
+    @rtype: list of 2 ints
+    """
+
   def getXRep():
     """
     Get the number of repetitions in the x (horizontal) axis for this Image.
@@ -135,6 +170,24 @@ class Image:
     @rtype: int
     """
 
+  def getStart():
+    """
+    Get the Image's start frame. Used for animated textures.
+    @rtype: int
+    """
+
+  def getEnd():
+    """
+    Get the Image's end frame. Used for animated textures.
+    @rtype: int
+    """
+
+  def getSpeed():
+    """
+    Get the Image's speed (fps). Used for animated textures.
+    @rtype: int
+    """
+
   def reload():
     """
     Reloads this image from the filesystem.  If used within a loop you need to
@@ -197,3 +250,79 @@ class Image:
     @type yrep: int
     @param yrep: The new value in [1, 16].
     """
+
+  def setStart(start):
+    """
+    Get the Image's start frame. Used for animated textures.
+    @type start: int
+    @param start: The new value in [0, 128].
+    """
+
+  def setEnd(end):
+    """
+    Set the Image's end frame. Used for animated textures.
+    @type end: int
+    @param end: The new value in [0, 128].
+    """
+
+  def setSpeed(speed):
+    """
+    Set the Image's speed (fps). Used for animated textures.
+    @type speed: int
+    @param speed: The new value in [1, 100].
+    """
+
+  def setPixelF(x, y, (r, g, b,a )):
+    """
+    Set the the colors of the current pixel in the form [r,g,b,a].
+    Color values must be floats in the range 0.0 - 1.0.
+    Pixel coordinates are in the range from 0 to N-1.  See L{getMaxXY}
+    @type x: int
+    @type y: int
+    @type r: float
+    @type g: float
+    @type b: float
+    @type a: float
+    @returns: nothing
+    @rtype: none
+    """
+    
+  def setPixelI(x, y, (r, g, b, a)):
+    """
+    Set the the colors of the current pixel in the form [r,g,b,a].
+    Color values must be ints in the range 0 - 255.
+    Pixel coordinates are in the range from 0 to N-1.  See L{getMaxXY}
+    @type x: int
+    @type y: int
+    @type r: int
+    @type g: int
+    @type b: int
+    @type a: int
+    @returns: nothing
+    @rtype: none
+    """
+    
+  def save():
+    """
+    Saves the current image.
+    @returns: nothing
+    @rtype: none
+    """
+  
+  def pack():
+    """
+    Packs the image into the current blend file.
+    @note: An error will be raised if the image is alredy packed or the filename path does not exist.
+    @returns: nothing
+    @rtype: none
+    """
+
+  def unpack(mode):
+    """
+    Unpacks the image to the images filename.
+    @param mode: One of the values in Blender.Unpackmodes dict.
+    @note: An error will be raised if the image is not packed or the filename path does not exist.
+    @returns: nothing
+    @rtype: none
+    @type mode: int
+    """
