@@ -17,23 +17,60 @@ Example::
   context = scn.getRenderingContext()
   
   Render.EnableDispWin()
-  context.enableExtensions(1)
-  context.setRenderPath("C:/myRenderdir/")
-  context.sizePreset(Scene.PC)
-  context.setImageType(Render.AVIRAW)
-  context.startFrame(2)
-  context.endFrame(10)
+  context.extensions = True
+  context.renderPath = "//myRenderdir/"
+  context.sizePreset(Render.PC)
+  context.imageType = Render.AVIRAW
+  context.sFrame = 2
+  context.eFrame = 10
   context.renderAnim()
-
-  context.setImageType(Render.TARGA)
-  context.framesPerSec(15)
-  context.startFrame(15)
-  context.endFrame(22)
+  
+  context.imageType = Render.TARGA
+  context.fps = 15
+  context.sFrame = 15
+  context.eFrame = 22
   context.renderAnim()
-
+  
   Render.CloseRenderWindow()
-  print context.framesPerSec()
-  print context.currentFrame()
+  print context.fps
+  print context.cFrame
+
+@type Modes: readonly dictionary
+@var Modes: Constant dict used for with L{RenderData.mode} bitfield attribute.  
+Values can be ORed together.  Individual bits can also be set/cleared with
+boolean attributes.
+  - OSA: Oversampling (anti-aliasing) enabled
+  - SHADOW: Shadow calculation enabled
+  - GAMMA: Gamma correction enabled
+  - ENVMAP: Environment map rendering enabled
+  - TOONSHADING: Toon edge shading enabled
+  - FIELDRENDER: Field rendering enabled
+  - FIELDTIME: Time difference in field calculations I{disabled}
+  - RADIOSITY: Radiosity rendering enabled
+  - BORDER_RENDER: Small cut-out rendering enabled
+  - PANORAMA: Panorama rendering enabled
+  - CROP: Crop image during border renders
+  - ODDFIELD: Odd field first rendering enabled
+  - MBLUR: Motion blur enabled
+  - UNIFIED: Unified Renderer enabled
+  - RAYTRACING: Ray tracing enabled
+  - THREADS: Render in two threads enabled
+
+@type SceModes: readonly dictionary
+@var SceModes: Constant dict used for with L{RenderData.sceneMode} bitfield attribute.  
+Values can be ORed together.  Individual bits can also be set/cleared with
+boolean attributes.
+  - SEQUENCER: Enables sequencer output rendering.
+  - EXTENSION: Adds extensions to the output when rendering animations.
+
+@type FramingModes: readonly dictionary
+@var FramingModes: Constant dict used for with L{RenderData.gameFrame}
+attribute.  One of the following modes can be active:
+  - BARS: Show the entire viewport in the display window, using bar
+    horizontally or vertically.
+  - EXTEND: Show the entire viewport in the display window, viewing more
+    horizontally or vertically
+  - SCALE: Stretch or squeeze the viewport to fill the display window.
 
 """
 
@@ -44,12 +81,14 @@ def CloseRenderWindow():
 
 def EnableDispView():
   """
-  Render in the 3d View area.
+  Render in the 3d View area.  B{Note} this method is deprecated; 
+  use the l{displayMode} attribute instead.
   """
 
 def EnableDispWin():
   """
   Render in Render window.
+  B{Note} this method is deprecated; use the l{displayMode} attribute instead.
   """
 
 def SetRenderWinPos(locationList):
@@ -85,8 +124,164 @@ class RenderData:
   """
   The RenderData object
   =====================
-    This object gives access to Scene rendering contexts in Blender.
+  This object gives access to Scene rendering contexts in Blender.
+  @ivar unified: Unified Renderer enabled.  
+  Also see B{UNIFIED} in L{Modes} constant dict.
+  @type unified: boolean
+  @ivar renderwinSize: Size of the rendering window. Valid values are 25, 50,
+  75, or 100.
+  @type renderwinSize: int
+  @ivar xParts: Number of horizontal parts for image render.
+  Values are clamped to the range [2,512].
+  @type xParts: int
+  @ivar fieldRendering: Field rendering enabled. 
+  Also see B{FIELDRENDER} in L{Modes} constant dict.
+  @type fieldRendering: boolean
+  @ivar gammaCorrection: Gamma correction enabled. 
+  Also see B{GAMMA} in L{Modes} constant dict.
+  @type gammaCorrection: boolean
+  @ivar eFrame: Ending frame for rendering.
+  Values are clamped to the range [1,MAXFRAME].
+  @type eFrame: int
+  @ivar radiosityRender: Radiosity rendering enabled.
+  @type radiosityRender: boolean
+  @ivar sizeX: Image width (in pixels).
+  Values are clamped to the range [4,10000].
+  @type sizeX: int
+  @ivar shadow: Shadow calculation enabled. 
+  Also see B{SHADOW} in L{Modes} constant dict.
+  @type shadow: boolean
+  @ivar aspectX: Horizontal aspect ratio.
+  Values are clamped to the range [1,200].
+  @type aspectX: int
+  @ivar mode: Mode bitfield.  See L{Modes} constant dict for values.
+  @type mode: bitfield
+  @ivar fieldTimeDisable: Time difference in field calculations I{disabled}.
+  @type fieldTimeDisable: int
+  @ivar cFrame: The current frame for rendering.
+  Values are clamped to the range [1,MAXFRAME].
+  @type cFrame: int
+  @ivar crop: Crop image during border renders. 
+  Also see B{CROP} in L{Modes} constant dict.
+  @type crop: boolean
+  @ivar sFrame: Starting frame for rendering.
+  Values are clamped to the range [1,MAXFRAME].
+  @type sFrame: int
+  @ivar backbuf: Backbuffer image enabled.
+  @type backbuf: boolean
+  @ivar OSALevel: Oversampling (anti-aliasing) level.  Valid values are
+  5, 8, 11, or 16.
+  @type OSALevel: int
+  @ivar displayMode: Render output in separate window or 3D view.
+  Valid values are 0 (display in image editor view), 1 (display in render
+  window), or 2 (display full screen).
+  @type displayMode: int
+  @ivar threads: Render in two threads enabled. 
+  Also see B{THREADS} in L{Modes} constant dict.
+  @type threads: boolean
+  @ivar backbufPath: Path to a background image (setting loads image).
+  @type backbufPath: string
+  @ivar toonShading: Toon edge shading enabled. 
+  Also see B{TOONSHADING} in L{Modes} constant dict.
+  @type toonShading: boolean
+  @ivar sceneMode: Scene mode bitfield.  See L{SceModes} constant dict for
+  values.
+  @type sceneMode: bitfield
+  @ivar gameFrameColor: RGB color triplet for bars.  
+  Values are clamped in the range [0.0,1.0].
+  @type gameFrameColor: list of RGB 3 floats
+  @ivar extensions: Add extensions to output (when rendering animations).
+  Also see B{EXTENSION} in L{SceModes} constant dict.
+  @type extensions: boolean
+  @ivar sizeY: Image height (in pixels).
+  Values are clamped to the range [4,10000].
+  @type sizeY: int
+  @ivar renderer: Rendering engine choice.  
+  Valid values are 0 (internal) or 1 (Yafray).
+  @type renderer: int
+  @ivar sequencer: Enables sequencer output rendering.
+  Also see B{SEQUENCER} in L{SceModes} constant dict.
+  @type sequencer: boolean
+  @ivar panorama: Panorama rendering enabled. 
+  Also see B{PANORAMA} in L{Modes} constant dict.
+  @type panorama: boolean
+  @ivar rayTracing: Ray tracing enabled. 
+  Also see B{RAYTRACING} in L{Modes} constant dict.
+  @type rayTracing: boolean
+  @ivar renderPath: The path to output the rendered images.
+  @type renderPath: string
+  @ivar gameFrame: Game framing type.  See L{FramingModes} constant dict.
+  @type gameFrame: int
+  @ivar aspectY: Vertical aspect ratio.
+  Values are clamped to the range [1,200].
+  @type aspectY: int
+  @ivar imageType: File format for saving images.  See the module's constants
+  for values.
+  @type imageType: int
+  @ivar ftypePath: The path to Ftype file.
+  @type ftypePath: string
+  @ivar border: The border for border rendering.  The format is
+  [xmin,ymin,xmax,ymax].  Values are clamped to [0.0,1.0].
+  @type border: list of 4 floats.
+  @ivar edgeColor: RGB color triplet for edges in Toon shading (unified
+  renderer).
+  Values are clamped in the range [0.0,1.0].
+  @type edgeColor: list of 3 RGB floats
+  @ivar yParts: Number of vertical parts for image render.  
+  Values are clamped to the range [2,512].
+  @type yParts: int
+  @ivar imagePlanes: Image depth in bits.  Valid values are 8, 24, or 32.
+  @type imagePlanes: int
+  @ivar borderRender: Small cut-out rendering enabled. 
+  Also see B{BORDER_RENDER} in L{Modes} constant dict.
+  @type borderRender: boolean
+  @ivar oversampling: Oversampling (anti-aliasing) enabled. 
+  Also see B{OSA} in L{Modes} constant dict.
+  @type oversampling: boolean
+  @ivar fps: Frames per second.
+  Values are clamped to the range [1,120].
+  @type fps: int
+  @ivar timeCode: Get the current frame in HH:MM:SS:FF format.  Read-only.
+  @type timeCode: string
+  @ivar environmentMap: Environment map rendering enabled. 
+  Also see B{ENVMAP} in L{Modes} constant dict.
+  @type environmentMap: boolean
+  @ivar motionBlur: Motion blur enabled. 
+  Also see B{MBLUR} in L{Modes} constant dict.
+  @type motionBlur: boolean
+  @ivar oddFieldFirst: Odd field first rendering enabled. 
+  Also see B{ODDFIELD} in L{Modes} constant dict.
+  @type oddFieldFirst: boolean
+  @ivar alphaMode: Setting for sky/background.  Valid values are 0 (fill
+  background with sky), 1 (multiply alpha in advance), or 2 (alpha and color
+  values remain unchanged).
+  @type alphaMode: int
+  @ivar gaussFilter: Gauss filter size.
+  Values are clamped to the range [0.5,1.5].
+  @type gaussFilter: float
+  @ivar mblurFactor: Motion blur factor.
+  Values are clamped to the range [0.01,5.0].
+  @type mblurFactor: float
+  @ivar mapOld: Number of frames the Map Old will last
+  Values are clamped to the range [1,900].
+  @type mapOld: int
+  @ivar mapNew: New mapping value (in frames).
+  Values are clamped to the range [1,900].
+  @type mapNew: int
+  @ivar set: The scene linked as a set to this scene.  Values are an existing
+  scene or None (setting to None clears the set).  The scene argument cannot
+  cause a circular link.
+  @type set: BPy_Scene or None
   """
+
+  def currentFrame(frame = None):
+    """
+    Get/set the current frame.
+    @type frame: int (optional)
+    @param frame: must be between 1 - 30000
+    @rtype: int (if prototype is empty)
+    @return: Current frame for the scene.
+    """
  
   def render():
     """
@@ -98,6 +293,23 @@ class RenderData:
     Render a series of frames to an output directory.
     """
 
+  def saveRenderedImage(filename, zbuffer=0):
+    """
+    Saves the image rendered using RenderData.render() to the filename and path
+    given in the variable 'filename'.
+    
+    Make sure the filename you provide makes a valid path when added to the "render path"
+    (setRenderPath/getRenderPath) to make up the absolute path.
+    If you want to render to a new absolute path just set the renderpath to an
+    empty string and use the absolute path as the filename.
+    @param filename: The path+filename for the rendered image.
+    @type zbuffer: int 
+    @param zbuffer: Whether or not to render the zbuffer along with the image.
+    @type filename: string 
+    @since: 2.40
+    @requires: You must have an image currently rendered before calling this method
+    """
+
   def play():
     """
     play animation of rendered images/avi (searches Pics: field).
@@ -227,7 +439,7 @@ class RenderData:
 
   def enableOversampling(toggle):
     """
-    Enable/disable oversampling (anit-aliasing).
+    Enable/disable oversampling (anti-aliasing).
     @type toggle: int
     @param toggle: pass 1 for on / 0 for off
     """
@@ -285,7 +497,7 @@ class RenderData:
 
   def enableKey():
     """
-    Enable alpha and colour values remain unchanged.
+    Enable alpha and color values remain unchanged.
     """
 
   def enableShadow(toggle):
@@ -330,6 +542,13 @@ class RenderData:
     @param size: can be 25, 50, 75 or 100 (percent).
     """
 
+  def getRenderWinSize():
+    """
+    Get the size of the render window.
+    @rtype: int
+    @return: window size; can be 25, 50, 75 or 100 (percent).
+    """
+
   def enableFieldRendering(toggle):
     """
     Enable/disable field rendering
@@ -353,7 +572,7 @@ class RenderData:
 
   def enableGaussFilter(toggle):
     """
-    Enable/disable Gauss sampling filter for antialiasing.
+    Enable/disable Gauss sampling filter for anti-aliasing.
     @type toggle: int
     @param toggle: pass 1 for on / 0 for off
     """
@@ -472,15 +691,15 @@ class RenderData:
     Set the type of image to output from the render.
     @type type: enum constant
     @param type: must be one of 13 constants:
-        - AVIRAW: Uncompressed AVI files. AVI is a commonly used format on Windows plattforms
-        - AVIJPEG: AVI movie w/ Jpeg images
+        - AVIRAW: Uncompressed AVI files. AVI is a commonly used format on Windows platforms
+        - AVIJPEG: AVI movie w/ JPEG images
         - AVICODEC: AVI using win32 codec
         - QUICKTIME: Quicktime movie (if enabled)
         - TARGA: Targa files
         - RAWTGA: Raw Targa files
         - PNG: Png files
         - BMP: Bitmap files
-        - JPEG90: Jpeg files
+        - JPEG90: JPEG files
         - HAMX: Hamx files
         - IRIS: Iris files
         - IRIZ: Iris + z-buffer files
@@ -489,7 +708,7 @@ class RenderData:
 
   def quality(qual = None):
     """
-    Get/set quality get/setting for JPEG images, AVI Jpeg and SGI movies.
+    Get/set quality get/setting for JPEG images, AVI JPEG and SGI movies.
     @type qual: int (optional)
     @param qual: must be between 10 - 100
     @rtype: int (if prototype is empty)
@@ -507,7 +726,7 @@ class RenderData:
 
   def enableGrayscale():
     """
-    Images are saved with BW (grayscale) data.
+    Images are saved with black and white (grayscale) data.
     """
 
   def enableRGBColor():
@@ -560,7 +779,7 @@ class RenderData:
     Set yafray global Illumination method.
     @type type: enum constant
     @param type: must be one of 3 constants:
-        - NONE: Dont use GI illumination
+        - NONE: Do not use GI illumination
         - SKYDOME: Use Skydome method
         - GIFULL: Use Full method
     """
@@ -575,6 +794,15 @@ class RenderData:
     @return: Current yafray global illumination intensity for the scene.
     """
 
+  def yafrayGIIndirPower(power = None):
+    """
+    Get/set GI indirect lighting intensity scale.
+    @type power: float (optional)
+    @param power: must be between 0.01 - 100.0
+    @rtype: float (if prototype is empty)
+    @return: Current yafray indirect illumination intensity for the scene.
+    """
+
   def yafrayGIDepth(depth = None):
     """
     Get/set number of bounces of the indirect light.
@@ -645,7 +873,7 @@ class RenderData:
 
   def enableYafrayGITunePhotons(toggle):
     """
-    Enable/disable show the photonmap directly in the render for tuning.
+    Enable/disable show the photon map directly in the render for tuning.
     YafrayMethod must be GIFULL and Photons enabled.
     @type toggle: int
     @param toggle: pass 1 for on / 0 for off
@@ -736,7 +964,7 @@ class RenderData:
 
   def enableGameFrameStretch():
     """
-    Enble stretch or squeeze the viewport to fill the display window.
+    Enable stretch or squeeze the viewport to fill the display window.
     """
 
   def enableGameFrameExpose():
