@@ -1,5 +1,5 @@
 /* 
- * $Id: windowTheme.c,v 1.7 2005/06/11 05:30:14 ianwill Exp $
+ * $Id: windowTheme.c,v 1.12 2006/06/08 19:05:58 schlaile Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,10 +30,10 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
 
-#include <MEM_guardedalloc.h>
-#include <BLI_blenlib.h>
-#include <DNA_screen_types.h>
-#include "windowTheme.h"
+#include "windowTheme.h" /*This must come first*/
+
+#include "BLI_blenlib.h"
+#include "MEM_guardedalloc.h"
 #include "charRGBA.h"
 #include "gen_utils.h"
 
@@ -156,6 +156,7 @@ static PyObject *ThemeSpace_getAttr( BPy_ThemeSpace * self, char *name )
 		ELSEIF_TSP_RGBA( hilite )
 		ELSEIF_TSP_RGBA( grid )
 		ELSEIF_TSP_RGBA( wire )
+		ELSEIF_TSP_RGBA( lamp )
 		ELSEIF_TSP_RGBA( select )
 		ELSEIF_TSP_RGBA( active )
 		ELSEIF_TSP_RGBA( transform )
@@ -169,25 +170,39 @@ static PyObject *ThemeSpace_getAttr( BPy_ThemeSpace * self, char *name )
 		ELSEIF_TSP_RGBA( face_select )
 		ELSEIF_TSP_RGBA( face_dot )
 		ELSEIF_TSP_RGBA( normal )
+		ELSEIF_TSP_RGBA( bone_solid )
+		ELSEIF_TSP_RGBA( bone_pose )
+		ELSEIF_TSP_RGBA( strip )
+		ELSEIF_TSP_RGBA( strip_select )
 		ELSEIF_TSP_RGBA( syntaxl )
 		ELSEIF_TSP_RGBA( syntaxn )
 		ELSEIF_TSP_RGBA( syntaxb )
 		ELSEIF_TSP_RGBA( syntaxv )
 		ELSEIF_TSP_RGBA( syntaxc )
+		ELSEIF_TSP_RGBA( movie )
+		ELSEIF_TSP_RGBA( image )
+		ELSEIF_TSP_RGBA( scene )
+		ELSEIF_TSP_RGBA( audio )
+		ELSEIF_TSP_RGBA( effect )
+		ELSEIF_TSP_RGBA( plugin )
+		ELSEIF_TSP_RGBA( transition )
+		ELSEIF_TSP_RGBA( meta )
 		else if( !strcmp( name, "vertex_size" ) )
 		attrib = Py_BuildValue( "i", tsp->vertex_size );
 		else if( !strcmp( name, "facedot_size" ) )
 		attrib = Py_BuildValue( "i", tsp->facedot_size );
 	else if( !strcmp( name, "__members__" ) )
-		attrib = Py_BuildValue( "[sssssssssssssssssssssssssssssss]", "theme",
+		attrib = Py_BuildValue("[sssssssssssssssssssssssssssssssssssssssssss]", "theme",
 					"back", "text", "text_hi", "header",
 					"panel", "shade1", "shade2", "hilite",
-					"grid", "wire", "select", "active",
+					"grid", "wire", "lamp", "select", "active",
 					"transform", "vertex", "vertex_select",
 					"edge", "edge_select", "edge_seam",
 					"edge_facesel", "face", "face_select",
-					"face_dot", "normal",
+					"face_dot", "normal", "bone_solid", "bone_pose",
+					"strip", "strip_select",
 					"syntaxl", "syntaxn", "syntaxb", "syntaxv", "syntaxc",
+					"movie", "image", "scene", "audio", "effect", "plugin", "transition", "meta", 
 					"vertex_size", "facedot_size" );
 
 	if( attrib != Py_None )
@@ -215,6 +230,7 @@ static int ThemeSpace_setAttr( BPy_ThemeSpace * self, char *name,
 		ELSEIF_TSP_RGBA( hilite )
 		ELSEIF_TSP_RGBA( grid )
 		ELSEIF_TSP_RGBA( wire )
+		ELSEIF_TSP_RGBA( lamp )
 		ELSEIF_TSP_RGBA( select )
 		ELSEIF_TSP_RGBA( active )
 		ELSEIF_TSP_RGBA( transform )
@@ -228,11 +244,23 @@ static int ThemeSpace_setAttr( BPy_ThemeSpace * self, char *name,
 		ELSEIF_TSP_RGBA( face_select )
 		ELSEIF_TSP_RGBA( face_dot )
 		ELSEIF_TSP_RGBA( normal )
+		ELSEIF_TSP_RGBA( bone_solid )
+		ELSEIF_TSP_RGBA( bone_pose )
+		ELSEIF_TSP_RGBA( strip )
+		ELSEIF_TSP_RGBA( strip_select )
 		ELSEIF_TSP_RGBA( syntaxl )
 		ELSEIF_TSP_RGBA( syntaxn )
 		ELSEIF_TSP_RGBA( syntaxb )
 		ELSEIF_TSP_RGBA( syntaxv )
 		ELSEIF_TSP_RGBA( syntaxc )
+		ELSEIF_TSP_RGBA( movie )
+		ELSEIF_TSP_RGBA( image )
+		ELSEIF_TSP_RGBA( scene )
+		ELSEIF_TSP_RGBA( audio )
+		ELSEIF_TSP_RGBA( effect )
+		ELSEIF_TSP_RGBA( plugin )
+		ELSEIF_TSP_RGBA( transition )
+		ELSEIF_TSP_RGBA( meta )
 		else if( !strcmp( name, "vertex_size" ) ) {
 		int val;
 
@@ -241,7 +269,7 @@ static int ThemeSpace_setAttr( BPy_ThemeSpace * self, char *name,
 						    "expected integer value" );
 
 		val = ( int ) PyInt_AsLong( value );
-		tsp->vertex_size = EXPP_ClampInt( val,
+		tsp->vertex_size = (char)EXPP_ClampInt( val,
 						  EXPP_THEME_VTX_SIZE_MIN,
 						  EXPP_THEME_VTX_SIZE_MAX );
 		ret = 0;
@@ -254,7 +282,7 @@ static int ThemeSpace_setAttr( BPy_ThemeSpace * self, char *name,
 						    "expected integer value" );
 
 		val = ( int ) PyInt_AsLong( value );
-		tsp->vertex_size = EXPP_ClampInt( val,
+		tsp->vertex_size = (char)EXPP_ClampInt( val,
 						  EXPP_THEME_FDOT_SIZE_MIN,
 						  EXPP_THEME_FDOT_SIZE_MAX );
 		ret = 0;
@@ -359,6 +387,7 @@ static PyObject *ThemeUI_getAttr( BPy_ThemeUI * self, char *name )
 		ELSEIF_TUI_RGBA( setting2 )
 		ELSEIF_TUI_RGBA( num )
 		ELSEIF_TUI_RGBA( textfield )
+		ELSEIF_TUI_RGBA( textfield_hi )
 		ELSEIF_TUI_RGBA( popup )
 		ELSEIF_TUI_RGBA( text )
 		ELSEIF_TUI_RGBA( text_hi )
@@ -373,7 +402,7 @@ static PyObject *ThemeUI_getAttr( BPy_ThemeUI * self, char *name )
 		attrib = Py_BuildValue( "[ssssssssssssssssss]", "theme",
 					"outline", "neutral", "action",
 					"setting", "setting1", "setting2",
-					"num", "textfield", "popup", "text",
+					"num", "textfield", "textfield_hi", "popup", "text",
 					"text_hi", "menu_back", "menu_item",
 					"menu_hilite", "menu_text",
 					"menu_text_hi", "drawType" );
@@ -399,6 +428,7 @@ static int ThemeUI_setAttr( BPy_ThemeUI * self, char *name, PyObject * value )
 		ELSEIF_TUI_RGBA( setting2 )
 		ELSEIF_TUI_RGBA( num )
 		ELSEIF_TUI_RGBA( textfield )
+		ELSEIF_TUI_RGBA( textfield_hi )
 		ELSEIF_TUI_RGBA( popup )
 		ELSEIF_TUI_RGBA( text )
 		ELSEIF_TUI_RGBA( text_hi )
@@ -415,7 +445,7 @@ static int ThemeUI_setAttr( BPy_ThemeUI * self, char *name, PyObject * value )
 						    "expected integer value" );
 
 		val = ( int ) PyInt_AsLong( value );
-		tui->but_drawtype = EXPP_ClampInt( val,
+		tui->but_drawtype = (char)EXPP_ClampInt( val,
 						   EXPP_THEME_DRAWTYPE_MIN,
 						   EXPP_THEME_DRAWTYPE_MAX );
 		ret = 0;
