@@ -32,7 +32,7 @@
 
     formfactors.c	nov/dec 1992
 
-    $Id: radfactors.c,v 1.7 2004/04/07 18:19:30 ton Exp $
+    $Id: radfactors.c,v 1.9 2006/01/28 18:33:15 hos Exp $
 
  *************************************** */
 
@@ -53,11 +53,7 @@
 #include "BIF_screen.h"
 
 #include "radio.h"
-#include "render.h"       /* for `RE_zbufferall_radio and RE_zbufferall_radio */
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "RE_render_ext.h"       /* for `RE_zbufferall_radio and RE_zbufferall_radio */
 
 /* locals */
 void rad_setmatrices(RadView *vw);
@@ -290,7 +286,7 @@ void hemizbuf(RadView *vw)
 	int a, b, inda, hres;
 
 	rad_setmatrices(vw);
-	RE_zbufferall_radio(vw, RG.elem, RG.totelem);
+	RE_zbufferall_radio(vw, RG.elem, RG.totelem, RG.re);	/* Render for when we got renderfaces */
 
 	/* count factors */
 	if(vw->recty==vw->rectx) factors= RG.topfactors;
@@ -524,6 +520,7 @@ void rad_init_energy()
 void progressiverad()
 {
 	RPatch *shoot;
+	float unshot[3];
 	int it= 0;
 
 	rad_printstatus();
@@ -539,8 +536,16 @@ void progressiverad()
 		
 		drawpatch_ext(shoot, 0x88FF00);
 
+		if(shoot->first->f & RAD_TWOSIDED) {
+			VECCOPY(unshot, shoot->unshot);
+			VecMulf(shoot->norm, -1.0);
+			if(makeformfactors(shoot))
+				applyformfactors(shoot);
+			VecMulf(shoot->norm, -1.0);
+			VECCOPY(shoot->unshot, unshot);
+		}
+	
 		if( makeformfactors(shoot) ) {
-		
 			applyformfactors(shoot);
 	
 			it++;
