@@ -1,7 +1,7 @@
 /*
  * envmap_ext.h
  *
- * $Id: envmap.h,v 1.4 2002/12/27 13:10:33 mein Exp $
+ * $Id: envmap.h,v 1.7 2006/01/28 18:33:16 hos Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,7 +40,12 @@
  * environment map as texture. 
  * (initrender.c)
  */
-void make_envmaps(void);
+
+struct Render;
+struct TexResult;
+
+void make_envmaps(struct Render *re);
+int envmaptex(struct Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, struct TexResult *texres);
 
 #endif /* ENVMAP_EXT_H */
 
