@@ -1,15 +1,12 @@
-
-/*  envmap.c        RENDER
+/* 
+ * $Id: envmap.c,v 1.31 2006/05/11 10:27:12 ton Exp $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,34 +20,25 @@
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
  *
- * The Original Code is: all of this file.
+ * Contributors: 2004/2005/2006 Blender Foundation, full recode
  *
- * Contributor(s): none yet.
- *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
- * 
- *  may 1999
- * 
- * $Id: envmap.c,v 1.20 2005/06/04 16:22:50 lukep Exp $
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <math.h>
 #include <string.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 /* external modules: */
 #include "MEM_guardedalloc.h"
+
 #include "BLI_arithb.h"
 #include "BLI_blenlib.h"
-#include "BKE_utildefines.h"
+#include "BLI_threads.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"        /* for rectcpy */
 
-#include "DNA_texture_types.h"
+#include "DNA_group_types.h"
 #include "DNA_image_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
@@ -59,89 +47,23 @@
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_global.h"
-#include "BKE_world.h"   // init_render_world
 #include "BKE_image.h"   // BKE_write_ibuf 
+#include "BKE_texture.h"
+#include "BKE_utildefines.h"
 
 #include "MTC_matrixops.h"
 
-#include "SDL_thread.h"
-#undef main 
-#define main main /* stupid SDL_main redefines main as SDL_main */
-
 /* this module */
-#include "RE_callbacks.h"
-#include "render.h"
+#include "render_types.h"
+#include "renderpipeline.h"
 #include "envmap.h"
-#include "mydevice.h"
 #include "rendercore.h" 
-#include "renderHelp.h"
+#include "renderdatabase.h" 
 #include "texture.h"
 #include "zbuf.h"
+#include "initrender.h"
 
 
-/* ------------------------------------------------------------------------- */
-
-EnvMap *RE_add_envmap(void)
-{
-	EnvMap *env;
-	
-	env= MEM_callocN(sizeof(EnvMap), "envmap");
-	env->type= ENV_CUBE;
-	env->stype= ENV_STATIC;
-	env->clipsta= 0.1;
-	env->clipend= 100.0;
-	env->cuberes= 100;
-	
-	return env;
-} /* end of EnvMap *RE_add_envmap() */
-
-/* ------------------------------------------------------------------------- */
-
-EnvMap *RE_copy_envmap(EnvMap *env)
-{
-	EnvMap *envn;
-	int a;
-	
-	envn= MEM_dupallocN(env);
-	envn->ok= 0;
-	for(a=0; a<6; a++) envn->cube[a]= 0;
-	if(envn->ima) id_us_plus((ID *)envn->ima);
-	
-	return envn;
-}
-
-/* ------------------------------------------------------------------------- */
-
-void RE_free_envmapdata(EnvMap *env)
-{
-	Image *ima;
-	unsigned int a, part;
-	
-	for(part=0; part<6; part++) {
-		ima= env->cube[part];
-		if(ima) {
-			if(ima->ibuf) IMB_freeImBuf(ima->ibuf);
-
-			for(a=0; a<BLI_ARRAY_NELEMS(ima->mipmap); a++) {
-				if(ima->mipmap[a]) IMB_freeImBuf(ima->mipmap[a]);
-			}
-			MEM_freeN(ima);
-			env->cube[part]= 0;
-		}
-	}
-	env->ok= 0;
-}
-
-/* ------------------------------------------------------------------------- */
-
-void RE_free_envmap(EnvMap *env)
-{
-	
-	RE_free_envmapdata(env);
-	MEM_freeN(env);
-	
-}
-
 /* ------------------------------------------------------------------------- */
 
 static void envmap_split_ima(EnvMap *env)
@@ -151,7 +73,7 @@ static void envmap_split_ima(EnvMap *env)
 /*  	extern rectcpy(); */
 	int dx, part;
 	
-	RE_free_envmapdata(env);	
+	BKE_free_envmapdata(env);	
 	
 	dx= env->ima->ibuf->y;
 	dx/= 2;
@@ -168,18 +90,18 @@ static void envmap_split_ima(EnvMap *env)
 			ima->ok= 1;
 			env->cube[part]= ima;
 		}
-		IMB_rectop(env->cube[0]->ibuf, env->ima->ibuf, 
-			0, 0, 0, 0, dx, dx, IMB_rectcpy, 0);
-		IMB_rectop(env->cube[1]->ibuf, env->ima->ibuf, 
-			0, 0, dx, 0, dx, dx, IMB_rectcpy, 0);
-		IMB_rectop(env->cube[2]->ibuf, env->ima->ibuf, 
-			0, 0, 2*dx, 0, dx, dx, IMB_rectcpy, 0);
-		IMB_rectop(env->cube[3]->ibuf, env->ima->ibuf, 
-			0, 0, 0, dx, dx, dx, IMB_rectcpy, 0);
-		IMB_rectop(env->cube[4]->ibuf, env->ima->ibuf, 
-			0, 0, dx, dx, dx, dx, IMB_rectcpy, 0);
-		IMB_rectop(env->cube[5]->ibuf, env->ima->ibuf, 
-			0, 0, 2*dx, dx, dx, dx, IMB_rectcpy, 0);
+		IMB_rectcpy(env->cube[0]->ibuf, env->ima->ibuf, 
+			0, 0, 0, 0, dx, dx);
+		IMB_rectcpy(env->cube[1]->ibuf, env->ima->ibuf, 
+			0, 0, dx, 0, dx, dx);
+		IMB_rectcpy(env->cube[2]->ibuf, env->ima->ibuf, 
+			0, 0, 2*dx, 0, dx, dx);
+		IMB_rectcpy(env->cube[3]->ibuf, env->ima->ibuf, 
+			0, 0, 0, dx, dx, dx);
+		IMB_rectcpy(env->cube[4]->ibuf, env->ima->ibuf, 
+			0, 0, dx, dx, dx, dx);
+		IMB_rectcpy(env->cube[5]->ibuf, env->ima->ibuf, 
+			0, 0, 2*dx, dx, dx, dx);
 		env->ok= 2;
 	}
 }
@@ -187,51 +109,83 @@ static void envmap_split_ima(EnvMap *env)
 /* ------------------------------------------------------------------------- */
 /* ****************** RENDER ********************** */
 
-static void envmap_renderdata(EnvMap *env)
+/* copy current render */
+static Render *envmap_render_copy(Render *re, EnvMap *env)
 {
-	extern void init_filt_mask(void);
-	static RE_Render envR;
-	static Object *camera;
+	Render *envre;
 	int cuberes;
 	
-	if(env) {
-		envR= R;
-		camera= G.scene->camera;
-		
-		cuberes = (env->cuberes * R.r.size) / 100;
-		cuberes &= 0xFFFC;
-		env->lastsize= R.r.size;
-		R.rectx= R.r.xsch= R.recty= R.r.ysch= cuberes;
-		R.afmx= R.afmy= R.r.xsch/2;
-		R.xstart= R.ystart= -R.afmx;
-		R.xend= R.yend= R.xstart+R.rectx-1;
-
-		R.r.mode &= ~(R_BORDER | R_PANORAMA | R_ORTHO | R_MBLUR | R_GAUSS);
-		R.r.xparts= R.r.yparts= 1;
-		R.r.bufflag= 0;
-		R.r.size= 100;
-		R.ycor= 1.0; 
-		R.r.yasp= R.r.xasp= 1;
-		
-		R.near= env->clipsta;
-		R.far= env->clipend;
-		
-		G.scene->camera= env->object;
-		
-	}
-	else {
-		/* this to make sure init_renderdisplay works */
-		envR.winx= R.winx;
-		envR.winy= R.winy;
-		envR.winxof= R.winxof;
-		envR.winyof= R.winyof;
-		
-		R= envR;
-		G.scene->camera= camera;
-	}
+	envre= RE_NewRender("Envmap");
 	
-	/* gauss, gamma, etc */
-	init_filt_mask();
+	env->lastsize= re->r.size;
+	cuberes = (env->cuberes * re->r.size) / 100;
+	cuberes &= 0xFFFC;
+	
+	/* this flag has R_ZTRA in it for example */
+	envre->flag= re->flag;
+	
+	/* set up renderdata */
+	envre->r= re->r;
+	envre->r.mode &= ~(R_BORDER | R_PANORAMA | R_ORTHO | R_MBLUR);
+	envre->r.layers.first= envre->r.layers.last= NULL;
+	envre->r.filtertype= 0;
+	envre->r.xparts= envre->r.yparts= 2;
+	envre->r.bufflag= 0;
+	envre->r.size= 100;
+	envre->r.yasp= envre->r.xasp= 1;
+	
+	RE_InitState(envre, &re->r, cuberes, cuberes, NULL);
+	envre->scene= re->scene;	/* unsure about this... */
+
+	/* view stuff in env render */
+	envre->lens= 16.0f;
+	if(env->type==ENV_PLANE)
+		envre->lens*= env->viewscale;
+	envre->ycor= 1.0f; 
+	envre->clipsta= env->clipsta;	/* render_scene_set_window() respects this for now */
+	envre->clipend= env->clipend;
+	
+	RE_SetCamera(envre, env->object);
+	
+	/* callbacks */
+	envre->display_draw= re->display_draw;
+	envre->test_break= re->test_break;
+	
+	/* and for the evil stuff; copy the database... */
+	envre->totvlak= re->totvlak;
+	envre->totvert= re->totvert;
+	envre->tothalo= re->tothalo;
+	envre->totlamp= re->totlamp;
+	envre->lights= re->lights;
+	envre->vertnodeslen= re->vertnodeslen;
+	envre->vertnodes= re->vertnodes;
+	envre->blohalen= re->blohalen;
+	envre->bloha= re->bloha;
+	envre->blovllen= re->blovllen;
+	envre->blovl= re->blovl;
+	envre->oc= re->oc;
+	
+	return envre;
+}
+
+static void envmap_free_render_copy(Render *envre)
+{
+
+	envre->totvlak= 0;
+	envre->totvert= 0;
+	envre->tothalo= 0;
+	envre->totlamp= 0;
+	envre->lights.first= envre->lights.last= NULL;
+	envre->vertnodeslen= 0;
+	envre->vertnodes= NULL;
+	envre->blohalen= 0;
+	envre->bloha= NULL;
+	envre->blovllen= 0;
+	envre->blovl= NULL;
+	envre->oc.adrbranch= NULL;
+	envre->oc.adrnode= NULL;
+	
+	RE_FreeRender(envre);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -268,8 +222,9 @@ static void envmap_transmatrix(float mat[][4], int part)
 
 /* ------------------------------------------------------------------------- */
 
-static void env_rotate_scene(float mat[][4], int mode)
+static void env_rotate_scene(Render *re, float mat[][4], int mode)
 {
+	GroupObject *go;
 	VlakRen *vlr = NULL;
 	VertRen *ver = NULL;
 	LampRen *lar = NULL;
@@ -286,8 +241,8 @@ static void env_rotate_scene(float mat[][4], int mode)
 		MTC_Mat3CpyMat4(imat, mat);
 	}
 	
-	for(a=0; a<R.totvert; a++) {
-		if((a & 255)==0) ver= R.blove[a>>8];
+	for(a=0; a<re->totvert; a++) {
+		if((a & 255)==0) ver= RE_findOrAddVert(re, a);
 		else ver++;
 		
 		MTC_Mat4MulVecfl(tmat, ver->co);
@@ -302,15 +257,15 @@ static void env_rotate_scene(float mat[][4], int mode)
 		Normalise(ver->n);
 	}
 	
-	for(a=0; a<R.tothalo; a++) {
-		if((a & 255)==0) har= R.bloha[a>>8];
+	for(a=0; a<re->tothalo; a++) {
+		if((a & 255)==0) har= re->bloha[a>>8];
 		else har++;
 	
 		MTC_Mat4MulVecfl(tmat, har->co);
 	}
 		
-	for(a=0; a<R.totvlak; a++) {
-		if((a & 255)==0) vlr= R.blovl[a>>8];
+	for(a=0; a<re->totvlak; a++) {
+		if((a & 255)==0) vlr= re->blovl[a>>8];
 		else vlr++;
 		
 		xn= vlr->n[0];
@@ -323,10 +278,10 @@ static void env_rotate_scene(float mat[][4], int mode)
 		Normalise(vlr->n);
 	}
 	
-	set_normalflags();
+	set_normalflags(re);
 	
-	for(a=0; a<R.totlamp; a++) {
-		lar= R.la[a];
+	for(go=re->lights.first; go; go= go->next) {
+		lar= go->lampren;
 		
 		/* removed here some horrible code of someone in NaN who tried to fix
 		   prototypes... just solved by introducing a correct cmat[3][3] instead
@@ -357,25 +312,36 @@ static void env_rotate_scene(float mat[][4], int mode)
 
 /* ------------------------------------------------------------------------- */
 
-static void env_layerflags(unsigned int notlay)
+static void env_layerflags(Render *re, unsigned int notlay)
 {
 	VlakRen *vlr = NULL;
 	int a;
 	
-	for(a=0; a<R.totvlak; a++) {
-		if((a & 255)==0) vlr= R.blovl[a>>8];
+	/* invert notlay, so if face is in multiple layers it will still be visible,
+	   unless all 'notlay' bits match the face bits.
+	   face: 0110
+	   not:  0100
+	   ~not: 1011
+	   now (face & ~not) is true
+	*/
+	
+	notlay= ~notlay;
+	
+	for(a=0; a<re->totvlak; a++) {
+		if((a & 255)==0) vlr= re->blovl[a>>8];
 		else vlr++;
-		if(vlr->lay & notlay) vlr->flag &= ~R_VISIBLE;
+		if((vlr->lay & notlay)==0)
+			vlr->flag &= ~R_VISIBLE;
 	}
 }
 
-static void env_hideobject(Object *ob)
+static void env_hideobject(Render *re, Object *ob)
 {
 	VlakRen *vlr = NULL;
 	int a;
 	
-	for(a=0; a<R.totvlak; a++) {
-		if((a & 255)==0) vlr= R.blovl[a>>8];
+	for(a=0; a<re->totvlak; a++) {
+		if((a & 255)==0) vlr= re->blovl[a>>8];
 		else vlr++;
 		if(vlr->ob == ob) vlr->flag &= ~R_VISIBLE;
 	}
@@ -383,14 +349,14 @@ static void env_hideobject(Object *ob)
 
 /* ------------------------------------------------------------------------- */
 
-static void env_set_imats()
+static void env_set_imats(Render *re)
 {
 	Base *base;
 	float mat[4][4];
 	
 	base= G.scene->base.first;
 	while(base) {
-		MTC_Mat4MulMat4(mat, base->object->obmat, R.viewmat);
+		MTC_Mat4MulMat4(mat, base->object->obmat, re->viewmat);
 		MTC_Mat4Invert(base->object->imat, mat);
 		
 		base= base->next;
@@ -400,113 +366,110 @@ static void env_set_imats()
 
 /* ------------------------------------------------------------------------- */
 
-static void render_envmap(EnvMap *env)
+static void render_envmap(Render *re, EnvMap *env)
 {
-	/* only the cubemap is implemented */
+	/* only the cubemap and planar map is implemented */
+	Render *envre;
 	ImBuf *ibuf;
 	Image *ima;
+	float orthmat[4][4];
 	float oldviewinv[4][4], mat[4][4], tmat[4][4];
 	short part;
 	
 	/* need a recalc: ortho-render has no correct viewinv */
-	MTC_Mat4Invert(oldviewinv, R.viewmat);
+	MTC_Mat4Invert(oldviewinv, re->viewmat);
 
-	/* do first, envmap_renderdata copies entire R struct */
-	if(R.rectz) MEM_freeN(R.rectz); R.rectz= NULL;
-	if(R.rectot) MEM_freeN(R.rectot); R.rectot= NULL;
-	if(R.rectftot) MEM_freeN(R.rectftot); R.rectftot= NULL;
+	envre= envmap_render_copy(re, env);
 	
-	/* setup necessary globals */
-	envmap_renderdata(env);
+	/* precalc orthmat for object */
+	MTC_Mat4CpyMat4(orthmat, env->object->obmat);
+	MTC_Mat4Ortho(orthmat);
 	
-	RE_local_init_render_display();
-
-	R.rectot= MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectot");
-	R.rectz=  MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectz");
+	/* need imat later for texture imat */
+	MTC_Mat4MulMat4(mat, orthmat, re->viewmat);
+	MTC_Mat4Invert(tmat, mat);
+	MTC_Mat3CpyMat4(env->obimat, tmat);
 
 	for(part=0; part<6; part++) {
-
-		RE_local_clear_render_display(R.win);
-		fillrect(R.rectot, R.rectx, R.recty, 0);
+		if(env->type==ENV_PLANE && part!=1)
+			continue;
 		
-		RE_setwindowclip(1,-1); /*  no jit:(-1) */
+		re->display_clear(envre->result);
 		
-		MTC_Mat4CpyMat4(tmat, G.scene->camera->obmat);
-		MTC_Mat4Ortho(tmat);
+		MTC_Mat4CpyMat4(tmat, orthmat);
 		envmap_transmatrix(tmat, part);
 		MTC_Mat4Invert(mat, tmat);
 		/* mat now is the camera 'viewmat' */
 
-		MTC_Mat4CpyMat4(R.viewmat, mat);
-		MTC_Mat4CpyMat4(R.viewinv, tmat);
+		MTC_Mat4CpyMat4(envre->viewmat, mat);
+		MTC_Mat4CpyMat4(envre->viewinv, tmat);
 		
 		/* we have to correct for the already rotated vertexcoords */
-		MTC_Mat4MulMat4(tmat, oldviewinv, R.viewmat);
+		MTC_Mat4MulMat4(tmat, oldviewinv, envre->viewmat);
 		MTC_Mat4Invert(env->imat, tmat);
 		
-		env_rotate_scene(tmat, 1);
-		init_render_world();
-		setzbufvlaggen(RE_projectverto);
-		env_layerflags(env->notlay);
-		env_hideobject(env->object);
-		env_set_imats();
+		env_rotate_scene(envre, tmat, 1);
+		init_render_world(envre);
+		project_renderdata(envre, projectverto, 0, 0);
+		env_layerflags(envre, env->notlay);
+		env_hideobject(envre, env->object);
+		env_set_imats(envre);
 				
-		if(RE_local_test_break()==0) {
-
-			RE_local_printrenderinfo(0.0, part);
-
-			if(R.r.mode & R_OSA) zbufshadeDA();
-			else zbufshade();
-
+		if(re->test_break()==0) {
+			RE_TileProcessor(envre, 0);
 		}
 		
 		/* rotate back */
-		env_rotate_scene(tmat, 0);
+		env_rotate_scene(envre, tmat, 0);
 
-		if(RE_local_test_break()==0) {
-			ibuf= IMB_allocImBuf(R.rectx, R.recty, 24, IB_rect, 0);
+		if(re->test_break()==0) {
+			RenderLayer *rl= envre->result->layers.first;
+			
+			ibuf= IMB_allocImBuf(envre->rectx, envre->recty, 24, IB_rect, 0);
+			ibuf->rect_float= rl->rectf;
+			IMB_rect_from_float(ibuf);
+			ibuf->rect_float= NULL;
+				
 			ima= MEM_callocN(sizeof(Image), "image");
-			memcpy(ibuf->rect, R.rectot, 4*ibuf->x*ibuf->y);
+			
 			ima->ibuf= ibuf;
 			ima->ok= 1;
 			env->cube[part]= ima;
 		}
 		
-		if(RE_local_test_break()) break;
+		if(re->test_break()) break;
 
 	}
 	
-	if(R.rectz) MEM_freeN(R.rectz); R.rectz= NULL;
-	if(R.rectot) MEM_freeN(R.rectot); R.rectot= NULL;
-	if(R.rectftot) MEM_freeN(R.rectftot); R.rectftot= NULL;
-	
-	if(RE_local_test_break()) RE_free_envmapdata(env);
+	if(re->test_break()) BKE_free_envmapdata(env);
 	else {
-		if(R.r.mode & R_OSA) env->ok= ENV_OSA;
+		if(envre->r.mode & R_OSA) env->ok= ENV_OSA;
 		else env->ok= ENV_NORMAL;
-		env->lastframe= G.scene->r.cfra;
+		env->lastframe= G.scene->r.cfra;	/* hurmf */
 	}
 	
 	/* restore */
-	envmap_renderdata(0);
-	env_set_imats();
-	init_render_world();
+	envmap_free_render_copy(envre);
+	env_set_imats(re);
 
 }
 
 /* ------------------------------------------------------------------------- */
 
-void make_envmaps()
+void make_envmaps(Render *re)
 {
 	Tex *tex;
 	int do_init= 0, depth= 0, trace;
 	
-	if (!(R.r.mode & R_ENVMAP)) return;
+	if (!(re->r.mode & R_ENVMAP)) return;
 	
 	/* we dont raytrace, disabling the flag will cause ray_transp render solid */
-	trace= (R.r.mode & R_RAYTRACE);
-	R.r.mode &= ~R_RAYTRACE;
+	trace= (re->r.mode & R_RAYTRACE);
+	re->r.mode &= ~R_RAYTRACE;
 
+	re->i.infostr= "Creating Environment maps";
+	re->stats_draw(&re->i);
+	
 	/* 5 = hardcoded max recursion level */
 	while(depth<5) {
 		tex= G.main->tex.first;
@@ -523,21 +486,21 @@ void make_envmaps()
 								
 								if(tex->env->ok) {
 										/* free when OSA, and old one isn't OSA */
-									if((R.r.mode & R_OSA) && tex->env->ok==ENV_NORMAL) 
-										RE_free_envmapdata(tex->env);
+									if((re->r.mode & R_OSA) && tex->env->ok==ENV_NORMAL) 
+										BKE_free_envmapdata(tex->env);
 										/* free when size larger */
-									else if(tex->env->lastsize < R.r.size) 
-										RE_free_envmapdata(tex->env);
+									else if(tex->env->lastsize < re->r.size) 
+										BKE_free_envmapdata(tex->env);
 										/* free when env is in recalcmode */
 									else if(tex->env->recalc)
-										RE_free_envmapdata(tex->env);
+										BKE_free_envmapdata(tex->env);
 								}
 								
 								if(tex->env->ok==0 && depth==0) tex->env->recalc= 1;
 								
 								if(tex->env->ok==0) {
 									do_init= 1;
-									render_envmap(tex->env);
+									render_envmap(re, tex->env);
 									
 									if(depth==tex->env->depth) tex->env->recalc= 0;
 								}
@@ -552,59 +515,69 @@ void make_envmaps()
 	}
 
 	if(do_init) {
-		RE_local_init_render_display();
-		RE_local_clear_render_display(R.win);
-		R.flag |= R_REDRAW_PRV;
+		re->display_init(re->result);
+		re->display_clear(re->result);
+		// re->flag |= R_REDRAW_PRV;
 	}	
 	// restore
-	R.r.mode |= trace;
+	re->r.mode |= trace;
 
 }
 
 /* ------------------------------------------------------------------------- */
 
-static int envcube_isect(float *vec, float *answ)
+static int envcube_isect(EnvMap *env, float *vec, float *answ)
 {
 	float labda;
 	int face;
 	
-	/* which face */
-	if( vec[2]<=-fabs(vec[0]) && vec[2]<=-fabs(vec[1]) ) {
-		face= 0;
-		labda= -1.0/vec[2];
-		answ[0]= labda*vec[0];
-		answ[1]= labda*vec[1];
-	}
-	else if( vec[2]>=fabs(vec[0]) && vec[2]>=fabs(vec[1]) ) {
+	if(env->type==ENV_PLANE) {
 		face= 1;
+		
 		labda= 1.0/vec[2];
-		answ[0]= labda*vec[0];
-		answ[1]= -labda*vec[1];
-	}
-	else if( vec[1]>=fabs(vec[0]) ) {
-		face= 2;
-		labda= 1.0/vec[1];
-		answ[0]= labda*vec[0];
-		answ[1]= labda*vec[2];
-	}
-	else if( vec[0]<=-fabs(vec[1]) ) {
-		face= 3;
-		labda= -1.0/vec[0];
-		answ[0]= labda*vec[1];
-		answ[1]= labda*vec[2];
-	}
-	else if( vec[1]<=-fabs(vec[0]) ) {
-		face= 4;
-		labda= -1.0/vec[1];
-		answ[0]= -labda*vec[0];
-		answ[1]= labda*vec[2];
+		answ[0]= env->viewscale*labda*vec[0];
+		answ[1]= -env->viewscale*labda*vec[1];
 	}
 	else {
-		face= 5;
-		labda= 1.0/vec[0];
-		answ[0]= -labda*vec[1];
-		answ[1]= labda*vec[2];
+		/* which face */
+		if( vec[2]<=-fabs(vec[0]) && vec[2]<=-fabs(vec[1]) ) {
+			face= 0;
+			labda= -1.0/vec[2];
+			answ[0]= labda*vec[0];
+			answ[1]= labda*vec[1];
+		}
+		else if( vec[2]>=fabs(vec[0]) && vec[2]>=fabs(vec[1]) ) {
+			face= 1;
+			labda= 1.0/vec[2];
+			answ[0]= labda*vec[0];
+			answ[1]= -labda*vec[1];
+		}
+		else if( vec[1]>=fabs(vec[0]) ) {
+			face= 2;
+			labda= 1.0/vec[1];
+			answ[0]= labda*vec[0];
+			answ[1]= labda*vec[2];
+		}
+		else if( vec[0]<=-fabs(vec[1]) ) {
+			face= 3;
+			labda= -1.0/vec[0];
+			answ[0]= labda*vec[1];
+			answ[1]= labda*vec[2];
+		}
+		else if( vec[1]<=-fabs(vec[0]) ) {
+			face= 4;
+			labda= -1.0/vec[1];
+			answ[0]= -labda*vec[0];
+			answ[1]= labda*vec[2];
+		}
+		else {
+			face= 5;
+			labda= 1.0/vec[0];
+			answ[0]= -labda*vec[1];
+			answ[1]= labda*vec[2];
+		}
 	}
+	
 	answ[0]= 0.5+0.5*answ[0];
 	answ[1]= 0.5+0.5*answ[1];
 	return face;
@@ -638,7 +611,7 @@ static void set_dxtdyt(float *dxts, float *dyts, float *dxt, float *dyt, int fac
 
 int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexResult *texres)
 {
-	extern SDL_mutex *load_ibuf_lock; // initrender.c
+	extern Render R;				/* only in this call */
 	/* texvec should be the already reflected normal */
 	EnvMap *env;
 	Image *ima;
@@ -654,10 +627,10 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 		env->ima= tex->ima;
 		if(env->ima && env->ima->ok) {
 			// Now thread safe
-			if(load_ibuf_lock) SDL_mutexP(load_ibuf_lock);
+			BLI_lock_thread(LOCK_MALLOC);
 			if(env->ima->ibuf==NULL) ima_ibuf_is_nul(tex, tex->ima);
-			if(load_ibuf_lock) SDL_mutexV(load_ibuf_lock);
 			if(env->ima->ok && env->ok==0) envmap_split_ima(env);
+			BLI_unlock_thread(LOCK_MALLOC);
 		}
 	}
 
@@ -669,16 +642,16 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 	
 	/* rotate to envmap space, if object is set */
 	VECCOPY(vec, texvec);
-	if(env->object) MTC_Mat4Mul3Vecfl(env->object->imat, vec);
+	if(env->object) MTC_Mat3MulVecfl(env->obimat, vec);
 	else MTC_Mat4Mul3Vecfl(R.viewinv, vec);
 	
-	face= envcube_isect(vec, sco);
+	face= envcube_isect(env, vec, sco);
 	ima= env->cube[face];
 	
 	if(osatex) {
 		if(env->object) {
-			MTC_Mat4Mul3Vecfl(env->object->imat, dxt);
-			MTC_Mat4Mul3Vecfl(env->object->imat, dyt);
+			MTC_Mat3MulVecfl(env->obimat, dxt);
+			MTC_Mat3MulVecfl(env->obimat, dyt);
 		}
 		else {
 			MTC_Mat4Mul3Vecfl(R.viewinv, dxt);
@@ -695,7 +668,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			texr1.nor= texr2.nor= NULL;
 
 			VecAddf(vec, vec, dxt);
-			face1= envcube_isect(vec, sco);
+			face1= envcube_isect(env, vec, sco);
 			VecSubf(vec, vec, dxt);
 			
 			if(face!=face1) {
@@ -708,7 +681,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			/* here was the nasty bug! results were not zero-ed. FPE! */
 			
 			VecAddf(vec, vec, dyt);
-			face1= envcube_isect(vec, sco);
+			face1= envcube_isect(env, vec, sco);
 			VecSubf(vec, vec, dyt);
 			
 			if(face!=face1) {
