@@ -1,14 +1,10 @@
-/*  shadbuf.c        RENDER
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,20 +18,10 @@
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
  *
- * The Original Code is: all of this file.
+ * Contributor(s): 2004-2006, Blender Foundation
  *
- * Contributor(s): none yet.
- *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
- * 
- *  april 95
- * 
- * $Id: shadbuf.c,v 1.15 2005/04/23 20:49:23 ton Exp $
- *
- * 27-Jun-2001 switched the shadow buffer for UR to the object-shadow
- * buffers, and removed all references and fixes for UR rendering from
- * this one.
- * */
+ * ***** END GPL LICENSE BLOCK *****
+ */
 
 #include <math.h>
 #include <string.h>
@@ -44,17 +30,22 @@
 #include "MEM_guardedalloc.h"
 
 #include "DNA_lamp_types.h"
-#include "BKE_utildefines.h"
-#include "BLI_arithb.h"
 
-#include "render.h"
+#include "BKE_global.h"
+#include "BKE_utildefines.h"
+
+#include "BLI_arithb.h"
+#include "BLI_blenlib.h"
+#include "BLI_jitter.h"
+
+#include "renderpipeline.h"
+#include "render_types.h"
+#include "renderdatabase.h"
 
 #include "shadbuf.h"
-#include "renderHelp.h"
-#include "jitter.h"
 #include "zbuf.h"
 
-/* XXX, could be better implemented...
+/* XXX, could be better implemented... this is for endian issues
 */
 #if defined(__sgi) || defined(__sparc) || defined(__sparc__) || defined (__PPC__) || defined (__ppc__) || defined (__BIG_ENDIAN__)
 #define RCOMP	3
@@ -70,86 +61,33 @@
 
 /* ------------------------------------------------------------------------- */
 
+/* initshadowbuf() in convertBlenderScene.c */
 
-void RE_initshadowbuf(LampRen *lar, float mat[][4])
-{
-	struct ShadBuf *shb;
-	float hoek, temp, viewinv[4][4];
-
-	/* if(la->spsi<16) return; */
-
-	/* memory reservation */
-	shb= (struct ShadBuf *)MEM_callocN( sizeof(struct ShadBuf),"initshadbuf");
-	lar->shb= shb;
-
-	if(shb==NULL) return;
-
-	VECCOPY(shb->co, lar->co);
-	
-	/* percentage render: keep track of min and max */
-	shb->size= (lar->bufsize*R.r.size)/100;
-	if(shb->size<512) shb->size= 512;
-	else if(shb->size > lar->bufsize) shb->size= lar->bufsize;
-
-	shb->size &= ~15;	/* make sure its multiples of 16 */
-
-	shb->samp= lar->samp;
-	shb->soft= lar->soft;
-	shb->shadhalostep= lar->shadhalostep;
-	
-	shb->zbuf= (unsigned long *)MEM_mallocN( sizeof(unsigned long)*(shb->size*shb->size)/256, "initshadbuf2");
-	shb->cbuf= (char *)MEM_callocN( (shb->size*shb->size)/256, "initshadbuf3");
-
-	if(shb->zbuf==0 || shb->cbuf==0) {
-		if(shb->zbuf) MEM_freeN(shb->zbuf);
-		MEM_freeN(lar->shb);
-		lar->shb= 0;		
-		return;
-	}
-
-	MTC_Mat4Ortho(mat);
-	MTC_Mat4Invert(shb->winmat, mat);	/* winmat is temp */
-
-	/* matrix: combination of inverse view and lampmat */
-	/* calculate again: the ortho-render has no correct viewinv */
-	MTC_Mat4Invert(viewinv, R.viewmat);
-	MTC_Mat4MulMat4(shb->viewmat, viewinv, shb->winmat);
-
-	/* projection */
-	hoek= saacos(lar->spotsi);
-	temp= 0.5*shb->size*cos(hoek)/sin(hoek);
-	shb->d= lar->clipsta;
-
-	shb->pixsize= (shb->d)/temp;
-
-	shb->far= lar->clipend;
-	/* bias is percentage, made 2x karger because of correction for angle of incidence */
-	/* when a ray is closer to parallel of a face, bias value is increased during render */
-	shb->bias= (0.02*lar->bias)*0x7FFFFFFF;
-	shb->bias= shb->bias*(100/R.r.size);
-	
-}
 /* ------------------------------------------------------------------------- */
 
-
-static void lrectreadRectz(int x1, int y1, int x2, int y2, char *r1) /* reads part from rectz in r1 */
+static void copy_to_ztile(int *rectz, int size, int x1, int y1, int tile, char *r1)
 {
-	unsigned int len4, *rz;	
+	int len4, *rz;	
+	int x2, y2;
+	
+	x2= x1+tile;
+	y2= y1+tile;
+	if(x2>=size) x2= size-1;
+	if(y2>=size) y2= size-1;
 
-	if(x1>=R.rectx || x2>=R.rectx || y1>=R.recty || y2>=R.recty) return;
-	if(x1>x2 || y1>y2) return;
+	if(x1>=x2 || y1>=y2) return;
 
-	len4= 4*(x2- x1+1);
-	rz= R.rectz+R.rectx*y1+x1;
-	for(;y1<=y2;y1++) {
-		memcpy(r1,rz,len4);
-		rz+= R.rectx;
+	len4= 4*(x2- x1);
+	rz= rectz + size*y1 + x1;
+	for(; y1<y2; y1++) {
+		memcpy(r1, rz, len4);
+		rz+= size;
 		r1+= len4;
 	}
 }
 
 #if 0
-static int sizeoflampbuf(struct ShadBuf *shb)
+static int sizeoflampbuf(ShadBuf *shb)
 {
 	int num,count=0;
 	char *cp;
@@ -163,6 +101,7 @@ static int sizeoflampbuf(struct ShadBuf *shb)
 }
 #endif
 
+/* not threadsafe... */
 static float *give_jitter_tab(int samp)
 {
 	/* these are all possible jitter tables, takes up some
@@ -180,7 +119,7 @@ static float *give_jitter_tab(int samp)
 	for(a=0; a<samp-1; a++) offset+= tab[a];
 
 	if(ctab[samp]==0) {
-		initjit(jit[offset], samp*samp);
+		BLI_initjit(jit[offset], samp*samp);
 		ctab[samp]= 1;
 	}
 		
@@ -188,86 +127,75 @@ static float *give_jitter_tab(int samp)
 	
 }
 
-void makeshadowbuf(LampRen *lar)
+static void make_jitter_weight_tab(ShadBuf *shb, short filtertype) 
 {
-	struct ShadBuf *shb= lar->shb;
-	float panophi;
-	float temp, wsize, dist;
-	int *rz, *rz1, verg, verg1;
+	float *jit, totw= 0.0f;
+	int a, tot=shb->samp*shb->samp;
+	
+	shb->weight= MEM_mallocN(sizeof(float)*tot, "weight tab lamp");
+	
+	for(jit= shb->jit, a=0; a<tot; a++, jit+=2) {
+		if(filtertype==LA_SHADBUF_TENT) 
+			shb->weight[a]= 0.71f - sqrt(jit[0]*jit[0] + jit[1]*jit[1]);
+		else if(filtertype==LA_SHADBUF_GAUSS) 
+			shb->weight[a]= RE_filter_value(R_FILTER_GAUSS, 1.8f*sqrt(jit[0]*jit[0] + jit[1]*jit[1]));
+		else
+			shb->weight[a]= 1.0f;
+		
+		totw+= shb->weight[a];
+	}
+	
+	totw= 1.0f/totw;
+	for(a=0; a<tot; a++) {
+		shb->weight[a]*= totw;
+	}
+}
+
+/* create Z tiles (for compression): this system is 24 bits!!! */
+static void compress_shadowbuf(ShadBuf *shb, int *rectz, int square)
+{
+	ShadSampleBuf *shsample;
+	float dist;
 	unsigned long *ztile;
-	int a, x, y, minx, miny, byt1, byt2, tempmode;
-	short temprx,tempry, square;
+	int *rz, *rz1, verg, verg1, size= shb->size;
+	int a, x, y, minx, miny, byt1, byt2;
 	char *rc, *rcline, *ctile, *zt;
-
-	panophi = getPanoPhi();
 	
-	/* store viewvars */
-	temprx= R.rectx; tempry= R.recty; 
-	tempmode= R.r.mode;
-	R.r.mode &= ~R_ORTHO;
-	R.rectx= R.recty= shb->size;
-
-	shb->jit= give_jitter_tab(shb->samp);
-
-	/* matrices and window: in R.winmat the transformation is being put,
-		transforming from observer view to lamp view, including lamp window matrix */
+	shsample= MEM_mallocN( sizeof(ShadSampleBuf), "shad sample buf");
+	BLI_addtail(&shb->buffers, shsample);
 	
-	wsize= shb->pixsize*(shb->size/2.0);
-
-	i_window(-wsize, wsize, -wsize, wsize, shb->d, shb->far, shb->winmat);
-
-	MTC_Mat4MulMat4(shb->persmat, shb->viewmat, shb->winmat);
+	shsample->zbuf= MEM_mallocN( sizeof(unsigned long)*(size*size)/256, "initshadbuf2");
+	shsample->cbuf= MEM_callocN( (size*size)/256, "initshadbuf3");
 	
-	/* temp, will be restored */
-	MTC_Mat4SwapMat4(shb->persmat, R.winmat);
-
-	/* zbuffering */
-	if(R.rectz) MEM_freeN(R.rectz);
- 	R.rectz= (unsigned int *)MEM_mallocN(sizeof(int)*shb->size*shb->size,"makeshadbuf");
-	rcline= MEM_mallocN(256*4+sizeof(int),"makeshadbuf2");
-
-	/* store: panorama rot */
-	temp= panophi;
-	panophi= 0.0;
-	pushTempPanoPhi(0.0);
-
-	/* pano interference here? */
-	setzbufvlaggen(projectvert);
-
-	popTempPanoPhi();
-	panophi= temp;
+	ztile= shsample->zbuf;
+	ctile= shsample->cbuf;
 	
-	zbuffershad(lar);
+	/* help buffer */
+	rcline= MEM_mallocN(256*4+sizeof(int), "makeshadbuf2");
 	
-	square= lar->mode & LA_SQUARE;
-
-	/* create Z tiles (for compression): this system is 24 bits!!! */
-	
-	ztile= shb->zbuf;
-	ctile= shb->cbuf;
-	for(y=0; y<shb->size; y+=16) {
-		if(y< shb->size/2) miny= y+15-shb->size/2;
-		else miny= y-shb->size/2;	
-				
-		for(x=0; x<shb->size; x+=16) {
-
+	for(y=0; y<size; y+=16) {
+		if(y< size/2) miny= y+15-size/2;
+		else miny= y-size/2;	
+		
+		for(x=0; x<size; x+=16) {
+			
 			/* is tile within spotbundle? */
-			a= shb->size/2;
+			a= size/2;
 			if(x< a) minx= x+15-a;
 			else minx= x-a;	
 			
 			dist= sqrt( (float)(minx*minx+miny*miny) );
-
+			
 			if(square==0 && dist>(float)(a+12)) {	/* 12, tested with a onlyshadow lamp */
 				a= 256; verg= 0; /* 0x80000000; */ /* 0x7FFFFFFF; */
 				rz1= (&verg)+1;
 			} 
 			else {
-				lrectreadRectz(x, y, MIN2(shb->size-1,x+15), MIN2(shb->size-1,y+15), rcline);
+				copy_to_ztile(rectz, size, x, y, 16, rcline);
 				rz1= (int *)rcline;
 				
 				verg= (*rz1 & 0xFFFFFF00);
-
+				
 				for(a=0;a<256;a++,rz1++) {
 					if( (*rz1 & 0xFFFFFF00) !=verg) break;
 				}
@@ -277,7 +205,7 @@ void makeshadowbuf(LampRen *lar)
 				*ztile= *(rz1-1);
 			}
 			else {
-
+				
 				/* ACOMP etc. are defined to work L/B endian */
 				
 				rc= rcline;
@@ -289,7 +217,7 @@ void makeshadowbuf(LampRen *lar)
 				for(a=1;a<256;a++,rc+=4) {
 					byt1 &= (verg==rc[ACOMP]);
 					byt2 &= (verg1==rc[BCOMP]);
-
+					
 					if(byt1==0) break;
 				}
 				if(byt1 && byt2) {	/* only store byte */
@@ -297,7 +225,7 @@ void makeshadowbuf(LampRen *lar)
 					*ztile= (unsigned long)MEM_mallocN(256+4, "tile1");
 					rz= (int *)*ztile;
 					*rz= *rz1;
-
+					
 					zt= (char *)(rz+1);
 					rc= rcline;
 					for(a=0; a<256; a++, zt++, rc+=4) *zt= rc[GCOMP];	
@@ -307,7 +235,7 @@ void makeshadowbuf(LampRen *lar)
 					*ztile= (unsigned long)MEM_mallocN(2*256+4,"Tile2");
 					rz= (int *)*ztile;
 					*rz= *rz1;
-
+					
 					zt= (char *)(rz+1);
 					rc= rcline;
 					for(a=0; a<256; a++, zt+=2, rc+=4) {
@@ -334,20 +262,84 @@ void makeshadowbuf(LampRen *lar)
 	}
 
 	MEM_freeN(rcline);
-	MEM_freeN(R.rectz); R.rectz= NULL;
+
+}
+
+void makeshadowbuf(Render *re, LampRen *lar)
+{
+	ShadBuf *shb= lar->shb;
+	float wsize, *jitbuf, twozero[2]= {0.0f, 0.0f};
+	int *rectz, samples;
 	
-	/* old globals back */
-	R.rectx= temprx; R.recty= tempry;
-	R.r.mode= tempmode;
-	MTC_Mat4SwapMat4(shb->persmat, R.winmat);
+	/* jitter, weights */
+	shb->jit= give_jitter_tab(shb->samp);
+	make_jitter_weight_tab(shb, lar->filtertype);
+	
+	shb->totbuf= lar->buffers;
+	if(shb->totbuf==4) jitbuf= give_jitter_tab(2);
+	else if(shb->totbuf==9) jitbuf= give_jitter_tab(3);
+	else jitbuf= twozero;
+	
+	/* matrices and window: in winmat the transformation is being put,
+		transforming from observer view to lamp view, including lamp window matrix */
+	wsize= shb->pixsize*(shb->size/2.0);
+
+	i_window(-wsize, wsize, -wsize, wsize, shb->d, shb->clipend, shb->winmat);
+	MTC_Mat4MulMat4(shb->persmat, shb->viewmat, shb->winmat);
+	/* temp, will be restored */
+	MTC_Mat4SwapMat4(shb->persmat, re->winmat);
+
+	/* zbuffering */
+ 	rectz= MEM_mapallocN(sizeof(int)*shb->size*shb->size, "makeshadbuf");
+
+	project_renderdata(re, projectvert, 0, 0);
+	
+	for(samples=0; samples<shb->totbuf; samples++) {
+		zbuffer_shadow(re, lar, rectz, shb->size, jitbuf[2*samples], jitbuf[2*samples+1]);
+		/* create Z tiles (for compression): this system is 24 bits!!! */
+		compress_shadowbuf(shb, rectz, lar->mode & LA_SQUARE);
+	}
+	
+	MEM_freeN(rectz);
+	
+	/* old matrix back */
+	MTC_Mat4SwapMat4(shb->persmat, re->winmat);
 
 	/* printf("lampbuf %d\n", sizeoflampbuf(shb)); */
 }
 
-static int firstreadshadbuf(struct ShadBuf *shb, int xs, int ys, int nr)
+void freeshadowbuf(LampRen *lar)
+{
+	if(lar->shb) {
+		ShadBuf *shb= lar->shb;
+		ShadSampleBuf *shsample;
+		int b, v;
+		
+		v= (shb->size*shb->size)/256;
+		
+		for(shsample= shb->buffers.first; shsample; shsample= shsample->next) {
+			unsigned long *ztile= shsample->zbuf;
+			char *ctile= shsample->cbuf;
+			
+			for(b=0; b<v; b++, ztile++, ctile++)
+				if(*ctile) MEM_freeN((void *) *ztile);
+			
+			MEM_freeN(shsample->zbuf);
+			MEM_freeN(shsample->cbuf);
+		}
+		BLI_freelistN(&shb->buffers);
+		
+		if(shb->weight) MEM_freeN(shb->weight);
+		MEM_freeN(lar->shb);
+		
+		lar->shb= NULL;
+	}
+}
+
+
+static int firstreadshadbuf(ShadBuf *shb, ShadSampleBuf *shsample, int **rz, int xs, int ys, int nr)
 {
 	/* return a 1 if fully compressed shadbuf-tile && z==const */
-	static int *rz;
 	int ofs;
 	char *ct;
 
@@ -357,13 +349,13 @@ static int firstreadshadbuf(struct ShadBuf *shb, int xs, int ys, int nr)
    
 	/* calc z */
 	ofs= (ys>>4)*(shb->size>>4) + (xs>>4);
-	ct= shb->cbuf+ofs;
+	ct= shsample->cbuf+ofs;
 	if(*ct==0) {
 	    if(nr==0) {
-			rz= *( (int **)(shb->zbuf+ofs) );
+			*rz= *( (int **)(shsample->zbuf+ofs) );
 			return 1;
 	    }
-		else if(rz!= *( (int **)(shb->zbuf+ofs) )) return 0;
+		else if(*rz!= *( (int **)(shsample->zbuf+ofs) )) return 0;
 		
 	    return 1;
 	}
@@ -371,7 +363,8 @@ static int firstreadshadbuf(struct ShadBuf *shb, int xs, int ys, int nr)
 	return 0;
 }
 
-static float readshadowbuf(struct ShadBuf *shb, int bias, int xs, int ys, int zs)	/* return 1.0 : fully in light */
+/* return 1.0 : fully in light */
+static float readshadowbuf(ShadBuf *shb, ShadSampleBuf *shsample, int bias, int xs, int ys, int zs)	
 {
 	float temp;
 	int *rz, ofs;
@@ -388,8 +381,8 @@ static float readshadowbuf(struct ShadBuf *shb, int bias, int xs, int ys, int zs
 
 	/* calc z */
 	ofs= (ys>>4)*(shb->size>>4) + (xs>>4);
-	ct= shb->cbuf+ofs;
-	rz= *( (int **)(shb->zbuf+ofs) );
+	ct= shsample->cbuf+ofs;
+	rz= *( (int **)(shsample->zbuf+ofs) );
 
 	if(*ct==3) {
 		ct= ((char *)rz)+3*16*(ys & 15)+3*(xs & 15);
@@ -436,34 +429,35 @@ static float readshadowbuf(struct ShadBuf *shb, int bias, int xs, int ys, int zs
 
 /* the externally called shadow testing (reading) function */
 /* return 1.0: no shadow at all */
-float testshadowbuf(struct ShadBuf *shb, float *rco, float *dxco, float *dyco, float inp)
+float testshadowbuf(ShadBuf *shb, float *rco, float *dxco, float *dyco, float inp)
 {
-	float fac, co[4], dx[3], dy[3], aantal=0;
-	float xs1,ys1, siz, *j, xres, yres;
-	int xs,ys, zs, bias;
-	short a,num;
+	ShadSampleBuf *shsample;
+	float fac, co[4], dx[3], dy[3], shadfac=0.0f;
+	float xs1,ys1, siz, *jit, *weight, xres, yres;
+	int xs, ys, zs, bias, *rz;
+	short a, num;
 	
-	/* if(inp <= 0.0) return 1.0; */
+	if(inp <= 0.0f) return 0.0f;
 
 	/* rotate renderco en osaco */
-	siz= 0.5*(float)shb->size;
+	siz= 0.5f*(float)shb->size;
 	VECCOPY(co, rco);
-	co[3]= 1.0;
+	co[3]= 1.0f;
 
 	MTC_Mat4MulVec4fl(shb->persmat, co);	/* rational hom co */
 
-	xs1= siz*(1.0+co[0]/co[3]);
-	ys1= siz*(1.0+co[1]/co[3]);
+	xs1= siz*(1.0f+co[0]/co[3]);
+	ys1= siz*(1.0f+co[1]/co[3]);
 
-	/* Clip for z: near and far clip values of the shadow buffer. We
+	/* Clip for z: clipsta and clipend clip values of the shadow buffer. We
 		* can test for -1.0/1.0 because of the properties of the
 		* coordinate transformations. */
 	fac= (co[2]/co[3]);
 
-	if(fac>=1.0) {
-		return 0.0;
-	} else if(fac<= -1.0) {
-		return 1.0;
+	if(fac>=1.0f) {
+		return 0.0f;
+	} else if(fac<= -1.0f) {
+		return 1.0f;
 	}
 
 	zs= ((float)0x7FFFFFFF)*fac;
@@ -474,69 +468,75 @@ float testshadowbuf(struct ShadBuf *shb, float *rco, float *dxco, float *dyco, f
 	
 	/* with inp==1.0, bias is half the size. correction value was 1.1, giving errors 
 	   on cube edges, with one side being almost frontal lighted (ton)  */
-	bias= (1.5-inp*inp)*shb->bias;
+	bias= (1.5f-inp*inp)*shb->bias;
 
 	if(num==1) {
-		return readshadowbuf(shb, bias, (int)xs1, (int)ys1, zs);
+		for(shsample= shb->buffers.first; shsample; shsample= shsample->next)
+			shadfac += readshadowbuf(shb, shsample, bias, (int)xs1, (int)ys1, zs);
+		
+		return shadfac/(float)shb->totbuf;
 	}
 
+	/* calculate filter size */
 	co[0]= rco[0]+dxco[0];
 	co[1]= rco[1]+dxco[1];
 	co[2]= rco[2]+dxco[2];
 	co[3]= 1.0;
-	MTC_Mat4MulVec4fl(shb->persmat,co);	/* rational hom co */
+	MTC_Mat4MulVec4fl(shb->persmat,co);     /* rational hom co */
 	dx[0]= xs1- siz*(1.0+co[0]/co[3]);
 	dx[1]= ys1- siz*(1.0+co[1]/co[3]);
-
+	
 	co[0]= rco[0]+dyco[0];
 	co[1]= rco[1]+dyco[1];
 	co[2]= rco[2]+dyco[2];
 	co[3]= 1.0;
-	MTC_Mat4MulVec4fl(shb->persmat,co);	/* rational hom co */
+	MTC_Mat4MulVec4fl(shb->persmat,co);     /* rational hom co */
 	dy[0]= xs1- siz*(1.0+co[0]/co[3]);
 	dy[1]= ys1- siz*(1.0+co[1]/co[3]);
-
+	
 	xres= fac*( fabs(dx[0])+fabs(dy[0]) );
 	yres= fac*( fabs(dx[1])+fabs(dy[1]) );
-
 	if(xres<fac) xres= fac;
 	if(yres<fac) yres= fac;
 	
 	xs1-= (xres)/2;
 	ys1-= (yres)/2;
 
-	j= shb->jit;
-
-	if(xres<16.0 && yres<16.0) {
-	    if(firstreadshadbuf(shb, (int)xs1, (int)ys1, 0)) {
-			if(firstreadshadbuf(shb, (int)(xs1+xres), (int)ys1, 1)) {
-				if(firstreadshadbuf(shb, (int)xs1, (int)(ys1+yres), 1)) {
-					if(firstreadshadbuf(shb, (int)(xs1+xres), (int)(ys1+yres), 1)) {
-						return readshadowbuf(shb, bias,(int)xs1, (int)ys1, zs);
+	if(xres<16.0f && yres<16.0f) {
+		shsample= shb->buffers.first;
+	    if(firstreadshadbuf(shb, shsample, &rz, (int)xs1, (int)ys1, 0)) {
+			if(firstreadshadbuf(shb, shsample, &rz, (int)(xs1+xres), (int)ys1, 1)) {
+				if(firstreadshadbuf(shb, shsample, &rz, (int)xs1, (int)(ys1+yres), 1)) {
+					if(firstreadshadbuf(shb, shsample, &rz, (int)(xs1+xres), (int)(ys1+yres), 1)) {
+						return readshadowbuf(shb, shsample, bias,(int)xs1, (int)ys1, zs);
 					}
 				}
 			}
 	    }
 	}
-
-	for(a=num;a>0;a--) {
-		/* instead of jit i tried random: ugly! */
-		/* note: the plus 0.5 gives best sampling results, jit used to go from 0-1 */
-		/* xs1 and ys1 are already corrected to be corner of sample area */
-		xs= xs1 + xres*(j[0] + 0.5);
-		ys= ys1 + yres*(j[1] + 0.5);
-		j+=2;
+	
+	for(shsample= shb->buffers.first; shsample; shsample= shsample->next) {
+		jit= shb->jit;
+		weight= shb->weight;
 		
-		aantal+= readshadowbuf(shb, bias, xs, ys, zs);
+		for(a=num; a>0; a--, jit+=2, weight++) {
+			/* instead of jit i tried random: ugly! */
+			/* note: the plus 0.5 gives best sampling results, jit goes from -0.5 to 0.5 */
+			/* xs1 and ys1 are already corrected to be corner of sample area */
+			xs= xs1 + xres*(jit[0] + 0.5f);
+			ys= ys1 + yres*(jit[1] + 0.5f);
+			
+			shadfac+= *weight * readshadowbuf(shb, shsample, bias, xs, ys, zs);
+		}
 	}
 
 	/* Renormalizes for the sample number: */
-	return aantal/( (float)(num) );
+	return shadfac/(float)shb->totbuf;
 }
 
 /* different function... sampling behind clipend can be LIGHT, bias is negative! */
 /* return: light */
-static float readshadowbuf_halo(struct ShadBuf *shb, int xs, int ys, int zs)
+static float readshadowbuf_halo(ShadBuf *shb, ShadSampleBuf *shsample, int xs, int ys, int zs)
 {
 	float temp;
 	int *rz, ofs;
@@ -552,8 +552,8 @@ static float readshadowbuf_halo(struct ShadBuf *shb, int xs, int ys, int zs)
 
 	/* calc z */
 	ofs= (ys>>4)*(shb->size>>4) + (xs>>4);
-	ct= shb->cbuf+ofs;
-	rz= *( (int **)(shb->zbuf+ofs) );
+	ct= shsample->cbuf+ofs;
+	rz= *( (int **)(shsample->zbuf+ofs) );
 
 	if(*ct==3) {
 		ct= ((char *)rz)+3*16*(ys & 15)+3*(xs & 15);
@@ -612,6 +612,7 @@ float shadow_halo(LampRen *lar, float *p1, float *p2)
 {
 	/* p1 p2 already are rotated in spot-space */
 	ShadBuf *shb= lar->shb;
+	ShadSampleBuf *shsample;
 	float co[4], siz;
 	float labda, labdao, labdax, labday, ldx, ldy;
 	float zf, xf1, yf1, zf1, xf2, yf2, zf2;
@@ -708,7 +709,7 @@ float shadow_halo(LampRen *lar, float *p1, float *p2)
 		if(labda==labdao || labda>=1.0) break;
 		
 		zf= zf1 + labda*(zf2-zf1);
-		count+= 1.0;
+		count+= (float)shb->totbuf;
 
 		if(zf<= -1.0) lightcount += 1.0;	/* close to the spot */
 		else {
@@ -717,7 +718,8 @@ float shadow_halo(LampRen *lar, float *p1, float *p2)
 			if(zf>=1.0) z= 0x7FFFF000;
 			else z= (int)(0x7FFFF000*zf);
 			
-			lightcount+= readshadowbuf_halo(shb, x, y, z);
+			for(shsample= shb->buffers.first; shsample; shsample= shsample->next)
+				lightcount+= readshadowbuf_halo(shb, shsample, x, y, z);
 			
 		}
 	}
