@@ -1,17 +1,12 @@
-/* texture.c
+/*
+ * $Id: texture.c,v 1.68 2006/07/13 15:41:36 ton Exp $
  *
- *
- * $Id: texture.c,v 1.46 2005/05/29 10:44:52 ton Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,11 +20,9 @@
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
  *
- * The Original Code is: all of this file.
+ * Contributor(s): 2004-2006, Blender Foundation, full recode
  *
- * Contributor(s): none yet.
- *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <stdio.h>
@@ -37,10 +30,6 @@
 #include <string.h>
 #include <math.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "MTC_matrixops.h"
 
 #include "BLI_blenlib.h"
@@ -70,11 +59,19 @@
 #include "BKE_key.h"
 #include "BKE_ipo.h"
 
-#include "render.h"
+#include "renderpipeline.h"
+#include "render_types.h"
 #include "rendercore.h"
 #include "envmap.h"
 #include "texture.h"
 
+/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
+/* defined in pipeline.c, is hardcopy of active dynamic allocated Render */
+/* only to be used here in this file, it's for speed */
+extern struct Render R;
+/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
+
+
 /* prototypes */
 static int calcimanr(int cfra, Tex *tex);
 
@@ -135,7 +132,8 @@ int calcimanr(int cfra, Tex *tex)
 	return imanr;
 }
 
-void init_render_texture(Tex *tex)
+/* note; this function is called in src/drawview.c for animated background image, option should move to kernel */
+void init_render_texture(Render *re, Tex *tex)
 {
 	Image *ima;
 	int imanr;
@@ -174,12 +172,6 @@ void init_render_texture(Tex *tex)
 			}
 		}
 	}
-	if(tex->imaflag & (TEX_ANTIALI+TEX_ANTISCALE)) {
-		if(tex->ima && tex->ima->lastquality<R.osa) {
-			if(tex->ima->ibuf) IMB_freeImBuf(tex->ima->ibuf);
-			tex->ima->ibuf= 0;
-		}
-	}
 	
 	if(tex->type==TEX_PLUGIN) {
 		if(tex->plugin && tex->plugin->doit) {
@@ -194,8 +186,14 @@ void init_render_texture(Tex *tex)
 		tex->extend= TEX_CLIP;
 		
 		if(tex->env) {
-			if(R.flag & R_RENDERING) {
-				if(tex->env->stype==ENV_ANIM) RE_free_envmapdata(tex->env);
+			if(tex->env->type==ENV_PLANE)
+				tex->extend= TEX_EXTEND;
+			
+			/* only free envmap when rendermode was set to render envmaps, for previewrender */
+			if(G.rendering) {
+				if (re->r.mode & R_ENVMAP)
+					if(tex->env->stype==ENV_ANIM) 
+						BKE_free_envmapdata(tex->env);
 			}
 		}
 	}
@@ -203,13 +201,13 @@ void init_render_texture(Tex *tex)
 
 /* ------------------------------------------------------------------------- */
 
-void init_render_textures()
+void init_render_textures(Render *re)
 {
 	Tex *tex;
 	
 	tex= G.main->tex.first;
 	while(tex) {
-		if(tex->id.us) init_render_texture(tex);
+		if(tex->id.us) init_render_texture(re, tex);
 		tex= tex->id.next;
 	}
 	
@@ -218,27 +216,6 @@ void init_render_textures()
 
 /* ------------------------------------------------------------------------- */
 
-void end_render_texture(Tex *tex)
-{
-
-
-}
-
-/* ------------------------------------------------------------------------- */
-
-void end_render_textures()
-{
-	Tex *tex;
-
-	tex= G.main->tex.first;
-	while(tex) {
-		if(tex->id.us) end_render_texture(tex);
-		tex= tex->id.next;
-	}
-
-}
-
-/* ------------------------------------------------------------------------- */
 
 /* this allows colorbanded textures to control normals as well */
 static void tex_normal_derivate(Tex *tex, TexResult *texres)
@@ -283,15 +260,15 @@ static int blend(Tex *tex, float *texvec, TexResult *texres)
 		y= texvec[1];
 	}
 
-	if(tex->stype==0) {	/* lin */
+	if(tex->stype==TEX_LIN) {	/* lin */
 		texres->tin= (1.0+x)/2.0;
 	}
-	else if(tex->stype==1) {	/* quad */
+	else if(tex->stype==TEX_QUAD) {	/* quad */
 		texres->tin= (1.0+x)/2.0;
 		if(texres->tin<0.0) texres->tin= 0.0;
 		else texres->tin*= texres->tin;
 	}
-	else if(tex->stype==2) {	/* ease */
+	else if(tex->stype==TEX_EASE) {	/* ease */
 		texres->tin= (1.0+x)/2.0;
 		if(texres->tin<=.0) texres->tin= 0.0;
 		else if(texres->tin>=1.0) texres->tin= 1.0;
@@ -300,10 +277,13 @@ static int blend(Tex *tex, float *texvec, TexResult *texres)
 			texres->tin= (3.0*t-2.0*t*texres->tin);
 		}
 	}
-	else if(tex->stype==3) { /* diag */
+	else if(tex->stype==TEX_DIAG) { /* diag */
 		texres->tin= (2.0+x+y)/4.0;
 	}
-	else {  /* sphere */
+	else if(tex->stype==TEX_RAD) { /* radial */
+		texres->tin= (atan2(y,x) / (2*M_PI) + 0.5);
+	}
+	else {  /* sphere TEX_SPHERE */
 		texres->tin= 1.0-sqrt(x*x+	y*y+texvec[2]*texvec[2]);
 		if(texres->tin<0.0) texres->tin= 0.0;
 		if(tex->stype==5) texres->tin*= texres->tin;  /* halo */
@@ -311,7 +291,7 @@ static int blend(Tex *tex, float *texvec, TexResult *texres)
 
 	BRICONT;
 
-	return 0;
+	return TEX_INT;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -321,7 +301,7 @@ static int blend(Tex *tex, float *texvec, TexResult *texres)
 
 static int clouds(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=0;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv = TEX_INT;
 	
 	texres->tin = BLI_gTurbulence(tex->noisesize, texvec[0], texvec[1], texvec[2], tex->noisedepth, (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
 
@@ -332,7 +312,7 @@ static int clouds(Tex *tex, float *texvec, TexResult *texres)
 		texres->nor[2] = BLI_gTurbulence(tex->noisesize, texvec[0], texvec[1], texvec[2] + tex->nabla, tex->noisedepth,  (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
 		
 		tex_normal_derivate(tex, texres);
-		rv += 2;
+		rv |= TEX_NOR;
 	}
 
 	if (tex->stype==1) {
@@ -343,7 +323,7 @@ static int clouds(Tex *tex, float *texvec, TexResult *texres)
 		texres->tb = BLI_gTurbulence(tex->noisesize, texvec[1], texvec[2], texvec[0], tex->noisedepth, (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
 		BRICONTRGB;
 		texres->ta = 1.0;
-		return (rv+1);
+		return (rv | TEX_RGB);
 	}
 
 	BRICONT;
@@ -416,7 +396,7 @@ static float wood_int(Tex *tex, float x, float y, float z)
 
 static int wood(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=TEX_INT;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv=TEX_INT;
 
 	texres->tin = wood_int(tex, texvec[0], texvec[1], texvec[2]);
 	if (texres->nor!=NULL) {
@@ -426,7 +406,7 @@ static int wood(Tex *tex, float *texvec, TexResult *texres)
 		texres->nor[2] = wood_int(tex, texvec[0], texvec[1], texvec[2] + tex->nabla);
 		
 		tex_normal_derivate(tex, texres);
-		rv = TEX_NOR;
+		rv |= TEX_NOR;
 	}
 
 	BRICONT;
@@ -467,7 +447,7 @@ static float marble_int(Tex *tex, float x, float y, float z)
 
 static int marble(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=TEX_INT;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv=TEX_INT;
 
 	texres->tin = marble_int(tex, texvec[0], texvec[1], texvec[2]);
 
@@ -479,7 +459,7 @@ static int marble(Tex *tex, float *texvec, TexResult *texres)
 		
 		tex_normal_derivate(tex, texres);
 		
-		rv = TEX_NOR;
+		rv |= TEX_NOR;
 	}
 
 	BRICONT;
@@ -559,7 +539,7 @@ static int magic(Tex *tex, float *texvec, TexResult *texres)
 	BRICONTRGB;
 	texres->ta= 1.0;
 	
-	return 1;
+	return TEX_RGB;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -567,27 +547,41 @@ static int magic(Tex *tex, float *texvec, TexResult *texres)
 /* newnoise: stucci also modified to use different noisebasis */
 static int stucci(Tex *tex, float *texvec, TexResult *texres)
 {
-	float b2, ofs;
-
-	if(texres->nor == NULL) return 0;
-
+	float nor[3], b2, ofs;
+	int retval= TEX_INT;
+	
+	b2= BLI_gNoise(tex->noisesize, texvec[0], texvec[1], texvec[2], (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
+	
 	ofs= tex->turbul/200.0;
 
-	texres->tin=b2= BLI_gNoise(tex->noisesize, texvec[0], texvec[1], texvec[2], (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
 	if(tex->stype) ofs*=(b2*b2);
-	texres->nor[0] = BLI_gNoise(tex->noisesize, texvec[0]+ofs, texvec[1], texvec[2], (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
-	texres->nor[1] = BLI_gNoise(tex->noisesize, texvec[0], texvec[1]+ofs, texvec[2], (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);	
-	texres->nor[2] = BLI_gNoise(tex->noisesize, texvec[0], texvec[1], texvec[2]+ofs, (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
+	nor[0] = BLI_gNoise(tex->noisesize, texvec[0]+ofs, texvec[1], texvec[2], (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
+	nor[1] = BLI_gNoise(tex->noisesize, texvec[0], texvec[1]+ofs, texvec[2], (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);	
+	nor[2] = BLI_gNoise(tex->noisesize, texvec[0], texvec[1], texvec[2]+ofs, (tex->noisetype!=TEX_NOISESOFT), tex->noisebasis);
 
-	tex_normal_derivate(tex, texres);
+	texres->tin= nor[2];
 	
-	if(tex->stype==2) {
-		texres->nor[0]= -texres->nor[0];
-		texres->nor[1]= -texres->nor[1];
-		texres->nor[2]= -texres->nor[2];
+	if(texres->nor) { 
+		
+		VECCOPY(texres->nor, nor);
+		tex_normal_derivate(tex, texres);
+		
+		if(tex->stype==2) {
+			texres->nor[0]= -texres->nor[0];
+			texres->nor[1]= -texres->nor[1];
+			texres->nor[2]= -texres->nor[2];
+		}
+		
+		retval |= TEX_NOR;
 	}
-
-	return 2;
+	
+	if(tex->stype==2) 
+		texres->tin= 1.0f-texres->tin;
+	
+	if(texres->tin<0.0f)
+		texres->tin= 0.0f;
+	
+	return retval;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -595,7 +589,7 @@ static int stucci(Tex *tex, float *texvec, TexResult *texres)
 
 static float mg_mFractalOrfBmTex(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=0;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv = TEX_INT;
 	float (*mgravefunc)(float, float, float, float, float, float, int);
 
 	if (tex->stype==TEX_MFRACTAL)
@@ -614,7 +608,7 @@ static float mg_mFractalOrfBmTex(Tex *tex, float *texvec, TexResult *texres)
 		texres->nor[2] = tex->ns_outscale*mgravefunc(texvec[0], texvec[1], texvec[2] + offs, tex->mg_H, tex->mg_lacunarity, tex->mg_octaves, tex->noisebasis);
 		
 		tex_normal_derivate(tex, texres);
-		rv += 2;
+		rv |= TEX_NOR;
 	}
 
 	BRICONT;
@@ -625,7 +619,7 @@ static float mg_mFractalOrfBmTex(Tex *tex, float *texvec, TexResult *texres)
 
 static float mg_ridgedOrHybridMFTex(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=0;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv = TEX_INT;
 	float (*mgravefunc)(float, float, float, float, float, float, float, float, int);
 
 	if (tex->stype==TEX_RIDGEDMF)
@@ -644,7 +638,7 @@ static float mg_ridgedOrHybridMFTex(Tex *tex, float *texvec, TexResult *texres)
 		texres->nor[2] = tex->ns_outscale*mgravefunc(texvec[0], texvec[1], texvec[2] + offs, tex->mg_H, tex->mg_lacunarity, tex->mg_octaves, tex->mg_offset, tex->mg_gain, tex->noisebasis);
 		
 		tex_normal_derivate(tex, texres);
-		rv += 2;
+		rv |= TEX_NOR;
 	}
 
 	BRICONT;
@@ -656,7 +650,7 @@ static float mg_ridgedOrHybridMFTex(Tex *tex, float *texvec, TexResult *texres)
 
 static float mg_HTerrainTex(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=0;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv = TEX_INT;
 
 	texres->tin = tex->ns_outscale*mg_HeteroTerrain(texvec[0], texvec[1], texvec[2], tex->mg_H, tex->mg_lacunarity, tex->mg_octaves, tex->mg_offset, tex->noisebasis);
 
@@ -669,7 +663,7 @@ static float mg_HTerrainTex(Tex *tex, float *texvec, TexResult *texres)
 		texres->nor[2] = tex->ns_outscale*mg_HeteroTerrain(texvec[0], texvec[1], texvec[2] + offs, tex->mg_H, tex->mg_lacunarity, tex->mg_octaves, tex->mg_offset, tex->noisebasis);
 		
 		tex_normal_derivate(tex, texres);
-		rv += 2;
+		rv |= TEX_NOR;
 	}
 
 	BRICONT;
@@ -681,7 +675,7 @@ static float mg_HTerrainTex(Tex *tex, float *texvec, TexResult *texres)
 
 static float mg_distNoiseTex(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=0;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv = TEX_INT;
 
 	texres->tin = mg_VLNoise(texvec[0], texvec[1], texvec[2], tex->dist_amount, tex->noisebasis, tex->noisebasis2);
 
@@ -694,7 +688,7 @@ static float mg_distNoiseTex(Tex *tex, float *texvec, TexResult *texres)
 		texres->nor[2] = mg_VLNoise(texvec[0], texvec[1], texvec[2] + offs, tex->dist_amount, tex->noisebasis, tex->noisebasis2);
 
 		tex_normal_derivate(tex, texres);
-		rv += 2;
+		rv |= TEX_NOR;
 	}
 
 	BRICONT;
@@ -710,7 +704,7 @@ static float mg_distNoiseTex(Tex *tex, float *texvec, TexResult *texres)
 
 static float voronoiTex(Tex *tex, float *texvec, TexResult *texres)
 {
-	int rv=0;	/* return value, int:0, col:1, nor:2, everything:3 */
+	int rv = TEX_INT;
 	float da[4], pa[12];	/* distance and point coordinate arrays of 4 nearest neighbours */
 	float aw1 = fabs(tex->vn_w1);
 	float aw2 = fabs(tex->vn_w2);
@@ -767,13 +761,13 @@ static float voronoiTex(Tex *tex, float *texvec, TexResult *texres)
 		texres->nor[2] = sc * fabs(tex->vn_w1*da[0] + tex->vn_w2*da[1] + tex->vn_w3*da[2] + tex->vn_w4*da[3]);
 		
 		tex_normal_derivate(tex, texres);
-		rv += 2;
+		rv |= TEX_NOR;
 	}
 
 	if (tex->vn_coltype) {
 		BRICONTRGB;
 		texres->ta = 1.0;
-		return (rv+1);
+		return (rv | TEX_RGB);
 	}
 	
 	BRICONT;
@@ -803,7 +797,7 @@ static int texnoise(Tex *tex, TexResult *texres)
 	texres->tin= ((float)val)/div;;
 
 	BRICONT;
-	return 0;
+	return TEX_INT;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -847,7 +841,7 @@ static int plugintex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex
 }
 
 
-static int cubemap_glob(MTex *mtex, VlakRen *vlr, float x, float y, float z, float *adr1, float *adr2)
+static int cubemap_glob(VlakRen *vlr, float x, float y, float z, float *adr1, float *adr2)
 {
 	float x1, y1, z1, nor[3];
 	int ret;
@@ -884,16 +878,18 @@ static int cubemap_glob(MTex *mtex, VlakRen *vlr, float x, float y, float z, flo
 
 /* ------------------------------------------------------------------------- */
 
+/* mtex argument only for projection switches */
 static int cubemap(MTex *mtex, VlakRen *vlr, float x, float y, float z, float *adr1, float *adr2)
 {
-	int proj[4], ret= 0;
+	int proj[4]={0, ME_PROJXY, ME_PROJXZ, ME_PROJYZ}, ret= 0;
 	
 	if(vlr) {
 		int index;
 		
 		/* Mesh vertices have such flags, for others we calculate it once based on orco */
 		if((vlr->puno & (ME_PROJXY|ME_PROJXZ|ME_PROJYZ))==0) {
-			if(vlr->v1->orco) {
+			/* test for v1, vlr can be faked for baking */
+			if(vlr->v1 && vlr->v1->orco) {
 				float nor[3];
 				CalcNormFloat(vlr->v1->orco, vlr->v2->orco, vlr->v3->orco, nor);
 				
@@ -901,19 +897,21 @@ static int cubemap(MTex *mtex, VlakRen *vlr, float x, float y, float z, float *a
 				else if( fabs(nor[0])<fabs(nor[1]) && fabs(nor[2])<fabs(nor[1]) ) vlr->puno |= ME_PROJXZ;
 				else vlr->puno |= ME_PROJYZ;
 			}
-			else return cubemap_glob(mtex, vlr, x, y, z, adr1, adr2);
+			else return cubemap_glob(vlr, x, y, z, adr1, adr2);
 		}
 		
-		/* the mtex->proj{xyz} have type char. maybe this should be wider? */
-		/* casting to int ensures that the index type is right.            */
-		index = (int) mtex->projx;
-		proj[index]= ME_PROJXY;
+		if(mtex) {
+			/* the mtex->proj{xyz} have type char. maybe this should be wider? */
+			/* casting to int ensures that the index type is right.            */
+			index = (int) mtex->projx;
+			proj[index]= ME_PROJXY;
 
-		index = (int) mtex->projy;
-		proj[index]= ME_PROJXZ;
+			index = (int) mtex->projy;
+			proj[index]= ME_PROJXZ;
 
-		index = (int) mtex->projz;
-		proj[index]= ME_PROJYZ;
+			index = (int) mtex->projz;
+			proj[index]= ME_PROJYZ;
+		}
 		
 		if(vlr->puno & proj[1]) {
 			*adr1 = (x + 1.0) / 2.0;
@@ -931,7 +929,7 @@ static int cubemap(MTex *mtex, VlakRen *vlr, float x, float y, float z, float *a
 		}		
 	} 
 	else {
-		return cubemap_glob(mtex, vlr, x, y, z, adr1, adr2);
+		return cubemap_glob(vlr, x, y, z, adr1, adr2);
 	}
 	
 	return ret;
@@ -939,7 +937,7 @@ static int cubemap(MTex *mtex, VlakRen *vlr, float x, float y, float z, float *a
 
 /* ------------------------------------------------------------------------- */
 
-static int cubemap_ob(MTex *mtex, VlakRen *vlr, float x, float y, float z, float *adr1, float *adr2)
+static int cubemap_ob(Object *ob, VlakRen *vlr, float x, float y, float z, float *adr1, float *adr2)
 {
 	float x1, y1, z1, nor[3];
 	int ret;
@@ -947,7 +945,7 @@ static int cubemap_ob(MTex *mtex, VlakRen *vlr, float x, float y, float z, float
 	if(vlr==NULL) return 0;
 	
 	VECCOPY(nor, vlr->n);
-	if(mtex->object) MTC_Mat4Mul3Vecfl(mtex->object->imat, nor);
+	if(ob) MTC_Mat4Mul3Vecfl(ob->imat, nor);
 	
 	x1= fabs(nor[0]);
 	y1= fabs(nor[1]);
@@ -976,11 +974,15 @@ static int cubemap_ob(MTex *mtex, VlakRen *vlr, float x, float y, float z, float
 static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float *dyt)
 {
 	Tex *tex;
+	Object *ob= NULL;
 	float fx, fy, fac1, area[8];
-	int ok, proj, areaflag= 0, wrap;
+	int ok, proj, areaflag= 0, wrap, texco;
 	
+	/* mtex variables localized, only cubemap doesn't cooperate yet... */
 	wrap= mtex->mapping;
 	tex= mtex->tex;
+	ob= mtex->object;
+	texco= mtex->texco;
 
 	if(R.osa==0) {
 		
@@ -991,8 +993,8 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 		else if(wrap==MTEX_TUBE) tubemap(t[0], t[1], t[2], &fx, &fy);
 		else if(wrap==MTEX_SPHERE) spheremap(t[0], t[1], t[2], &fx, &fy);
 		else {
-			if(mtex->texco==TEXCO_OBJECT) cubemap_ob(mtex, vlr, t[0], t[1], t[2], &fx, &fy);
-			else if(mtex->texco==TEXCO_GLOB) cubemap_glob(mtex, vlr, t[0], t[1], t[2], &fx, &fy);
+			if(texco==TEXCO_OBJECT) cubemap_ob(ob, vlr, t[0], t[1], t[2], &fx, &fy);
+			else if(texco==TEXCO_GLOB) cubemap_glob(vlr, t[0], t[1], t[2], &fx, &fy);
 			else cubemap(mtex, vlr, t[0], t[1], t[2], &fx, &fy);
 		}
 		
@@ -1029,8 +1031,10 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 			fy= (t[1] + 1.0) / 2.0;
 			dxt[0]/= 2.0; 
 			dxt[1]/= 2.0;
+			dxt[2]/= 2.0;
 			dyt[0]/= 2.0; 
 			dyt[1]/= 2.0;
+			dyt[2]/= 2.0;
 		}
 		else if ELEM(wrap, MTEX_TUBE, MTEX_SPHERE) {
 			/* exception: the seam behind (y<0.0) */
@@ -1066,24 +1070,30 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 		}
 		else {
 
-			if(mtex->texco==TEXCO_OBJECT) proj = cubemap_ob(mtex, vlr, t[0], t[1], t[2], &fx, &fy);
-			else if (mtex->texco==TEXCO_GLOB) proj = cubemap_glob(mtex, vlr, t[0], t[1], t[2], &fx, &fy);
+			if(texco==TEXCO_OBJECT) proj = cubemap_ob(ob, vlr, t[0], t[1], t[2], &fx, &fy);
+			else if (texco==TEXCO_GLOB) proj = cubemap_glob(vlr, t[0], t[1], t[2], &fx, &fy);
 			else proj = cubemap(mtex, vlr, t[0], t[1], t[2], &fx, &fy);
 
 			if(proj==1) {
-				dxt[1]= dxt[2];
-				dyt[1]= dyt[2];
+				SWAP(float, dxt[1], dxt[2]);
+				SWAP(float, dyt[1], dyt[2]);
 			}
 			else if(proj==2) {
+				float f1= dxt[0], f2= dyt[0];
 				dxt[0]= dxt[1];
 				dyt[0]= dyt[1];
 				dxt[1]= dxt[2];
 				dyt[1]= dyt[2];
+				dxt[2]= f1;
+				dyt[2]= f2;
 			}
 			dxt[0]/= 2.0; 
 			dxt[1]/= 2.0;
+			dxt[2]/= 2.0;
+			
 			dyt[0]/= 2.0; 
 			dyt[1]/= 2.0;
+			dyt[2]/= 2.0;
 		}
 		
 		/* if area, then reacalculate dxt[] and dyt[] */
@@ -1098,7 +1108,9 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 		
 		/* repeat */
 		if(tex->extend==TEX_REPEAT) {
+			float max= 1.0f;
 			if(tex->xrepeat>1) {
+				max= tex->xrepeat;
 				fx *= tex->xrepeat;
 				dxt[0]*= tex->xrepeat;
 				dyt[0]*= tex->xrepeat;
@@ -1106,12 +1118,19 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 				else if(fx<0.0) fx+= 1-(int)(fx);
 			}
 			if(tex->yrepeat>1) {
+				if(max<tex->yrepeat)
+					max= tex->yrepeat;
 				fy *= tex->yrepeat;
 				dxt[1]*= tex->yrepeat;
 				dyt[1]*= tex->yrepeat;
 				if(fy>1.0) fy -= (int)(fy);
 				else if(fy<0.0) fy+= 1-(int)(fy);
 			}
+			if(max!=1.0f) {
+				dxt[2]*= max;
+				dyt[2]*= max;
+			}
+			
 		}
 		/* crop */
 		if(tex->cropxmin!=0.0 || tex->cropxmax!=1.0) {
@@ -1133,7 +1152,6 @@ static void do_2d_mapping(MTex *mtex, float *t, VlakRen *vlr, float *dxt, float
 	}
 }
 
-
 /* ************************************** */
 
 static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexResult *texres)
@@ -1164,8 +1182,6 @@ static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex,
 		break;
 	case TEX_STUCCI:
 		retval= stucci(tex, texvec, texres); 
-		if (tex->flag & TEX_COLORBAND);
-		else texres->tin= 0.0;	// stucci doesnt return Tin, for backwards compat...
 		break;
 	case TEX_NOISE:
 		retval= texnoise(tex, texres); 
@@ -1173,6 +1189,7 @@ static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex,
 	case TEX_IMAGE:
 		if(osatex) retval= imagewraposa(tex, tex->ima, texvec, dxt, dyt, texres); 
 		else retval= imagewrap(tex, tex->ima, texvec, texres); 
+		tag_image_time(tex->ima); /* tag image as having being used */
 		break;
 	case TEX_PLUGIN:
 		retval= plugintex(tex, texvec, dxt, dyt, osatex, texres);
@@ -1229,31 +1246,37 @@ static int multitex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex,
 	return retval;
 }
 
-int multitex_ext(Tex *tex, float *texvec, float *tin, float *tr, float *tg, float *tb, float *ta)
+int multitex_ext(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexResult *texres)
 {
-	TexResult texr;
-	float dummy[3];
-	int retval;
 	
-	/* does not return Tin, hackish... */
-	if(tex->type==TEX_STUCCI) {
-		texr.nor= dummy;
-		dummy[0]= 1.0;
-		dummy[1]= dummy[2]= 0.0;
-	}
-	else texr.nor= NULL;
+	if(tex==NULL) return 0;
 	
-	retval= multitex(tex, texvec, NULL, NULL, 0, &texr);
-	if(tex->type==TEX_STUCCI) {
-		*tin= 0.5 + 0.7*texr.nor[0];
-		CLAMP(*tin, 0.0, 1.0);
+	/* Image requires 2d mapping conversion */
+	if(tex->type==TEX_IMAGE) {
+		MTex mtex;
+		float texvec_l[3], dxt_l[3], dyt_l[3];
+		
+		mtex.mapping= MTEX_FLAT;
+		mtex.tex= tex;
+		mtex.object= NULL;
+		mtex.texco= TEXCO_ORCO;
+		
+		VECCOPY(texvec_l, texvec);
+		if(dxt && dyt) {
+			VECCOPY(dxt_l, dxt);
+			VECCOPY(dyt_l, dyt);
+		}
+		else {
+			dxt_l[0]= dxt_l[1]= dxt_l[2]= 0.0f;
+			dyt_l[0]= dyt_l[1]= dyt_l[2]= 0.0f;
+		}
+		
+		do_2d_mapping(&mtex, texvec_l, NULL, dxt_l, dyt_l);
+
+		return multitex(tex, texvec_l, dxt_l, dyt_l, osatex, texres);
 	}
-	else *tin= texr.tin;
-	*tr= texr.tr;
-	*tg= texr.tg;
-	*tb= texr.tb;
-	*ta= texr.ta;
-	return retval;
+	else
+		return multitex(tex, texvec, dxt, dyt, osatex, texres);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1285,9 +1308,9 @@ static void texture_rgb_blend(float *in, float *tex, float *out, float fact, flo
 	case MTEX_SCREEN:
 		fact*= facg;
 		facm= 1.0-facg;
-		in[0]= 1.0-(facm+fact*(1.0-tex[0]))*(1.0-out[0]);
-		in[1]= 1.0-(facm+fact*(1.0-tex[1]))*(1.0-out[1]);
-		in[2]= 1.0-(facm+fact*(1.0-tex[2]))*(1.0-out[2]);
+		in[0]= 1.0 - (facm+fact*(1.0-tex[0])) * (1.0-out[0]);
+		in[1]= 1.0 - (facm+fact*(1.0-tex[1])) * (1.0-out[1]);
+		in[2]= 1.0 - (facm+fact*(1.0-tex[2])) * (1.0-out[2]);
 		break;
 
 	case MTEX_SUB:
@@ -1458,6 +1481,9 @@ void do_material_tex(ShadeInput *shi)
 			else if(mtex->texco==TEXCO_NORM) {
 				co= shi->orn; dx= shi->dxno; dy= shi->dyno;
 			}
+			else if(mtex->texco==TEXCO_TANGENT) {
+				co= shi->tang; dx= shi->dxno; dy= shi->dyno;
+			}
 			else if(mtex->texco==TEXCO_GLOB) {
 				co= shi->gl; dx= shi->dxco; dy= shi->dyco;
 			}
@@ -1467,6 +1493,24 @@ void do_material_tex(ShadeInput *shi)
 			else if(mtex->texco==TEXCO_WINDOW) {
 				co= shi->winco; dx= shi->dxwin; dy= shi->dywin;
 			}
+			else if(mtex->texco==TEXCO_STRAND) {
+				co= tempvec; dx= dxt; dy= dyt;
+				co[0]= shi->strand;
+				co[1]= co[2]= 0.0f;
+				dx[0]= shi->dxstrand;
+				dx[1]= dx[2]= 0.0f;
+				dy[0]= shi->dystrand;
+				dy[1]= dy[2]= 0.0f;
+			}
+			else if(mtex->texco==TEXCO_STRESS) {
+				co= tempvec; dx= dxt; dy= dyt;
+				co[0]= shi->stress;
+				co[1]= co[2]= 0.0f;
+				dx[0]= 0.0f;
+				dx[1]= dx[2]= 0.0f;
+				dy[0]= 0.0f;
+				dy[1]= dy[2]= 0.0f;
+			}
 			else continue;	// can happen when texco defines disappear and it renders old files
 			
 			/* de pointer defines if bumping happens */
@@ -1494,24 +1538,25 @@ void do_material_tex(ShadeInput *shi)
 				else texvec[2]= 0.0;
 
 				if(shi->osatex) {
-
+					
 					if(mtex->projx) {
 						dxt[0]= dx[mtex->projx-1];
 						dyt[0]= dy[mtex->projx-1];
 					}
-					else dxt[0]= 0.0;
+					else dxt[0]= dyt[0]= 0.0f;
+					
 					if(mtex->projy) {
 						dxt[1]= dx[mtex->projy-1];
 						dyt[1]= dy[mtex->projy-1];
 					}
-					else dxt[1]= 0.0;
-					if(mtex->projx) {
+					else dxt[1]= dyt[1]= 0.0f;
+					if(mtex->projz) {
 						dxt[2]= dx[mtex->projz-1];
 						dyt[2]= dy[mtex->projz-1];
 					}
-					else dxt[2]= 0.0;
+					else dxt[2]= dyt[2]= 0.0;
 				}
-
+				
 				do_2d_mapping(mtex, texvec, shi->vlr, dxt, dyt);
 
 				/* translate and scale */
@@ -1547,7 +1592,7 @@ void do_material_tex(ShadeInput *shi)
 						dyt[1]= mtex->size[1]*dy[mtex->projy-1];
 					}
 					else dxt[1]= 0.0;
-					if(mtex->projx) {
+					if(mtex->projz) {
 						dxt[2]= mtex->size[2]*dx[mtex->projz-1];
 						dyt[2]= mtex->size[2]*dy[mtex->projz-1];
 					}
@@ -1584,12 +1629,9 @@ void do_material_tex(ShadeInput *shi)
 				}
 			}
 			else {
-				texres.ta*= stencilTin;
 				Tnor*= stencilTin;
-				texres.tin*= stencilTin;
 			}
 			
-			
 			if(texres.nor) {
 				if((rgbnor & TEX_NOR)==0) {
 					/* make our own normal */
@@ -1620,7 +1662,7 @@ void do_material_tex(ShadeInput *shi)
 					warpvec[2]= mtex->warpfac*texres.nor[2];
 					warpdone= 1;
 				}
-				
+#if 0				
 				if(mtex->texflag & MTEX_VIEWSPACE) {
 					// rotate to global coords
 					if(mtex->texco==TEXCO_ORCO || mtex->texco==TEXCO_UV) {
@@ -1634,11 +1676,15 @@ void do_material_tex(ShadeInput *shi)
 						}
 					}
 				}
+#endif				
 			}
 
 			/* mapping */
 			if(mtex->mapto & (MAP_COL+MAP_COLSPEC+MAP_COLMIR)) {
-				float tcol[3];
+				float tcol[3], colfac;
+				
+				/* stencil maps on the texture control slider, not texture intensity value */
+				colfac= mtex->colfac*stencilTin;
 				
 				tcol[0]=texres.tr; tcol[1]=texres.tg; tcol[2]=texres.tb;
 				
@@ -1653,15 +1699,15 @@ void do_material_tex(ShadeInput *shi)
 				else texres.tin= texres.ta;
 				
 				if(mtex->mapto & MAP_COL) {
-					texture_rgb_blend(&shi->r, tcol, &shi->r, texres.tin, mtex->colfac, mtex->blendtype);
+					texture_rgb_blend(&shi->r, tcol, &shi->r, texres.tin, colfac, mtex->blendtype);
 				}
 				if(mtex->mapto & MAP_COLSPEC) {
-					texture_rgb_blend(&shi->specr, tcol, &shi->specr, texres.tin, mtex->colfac, mtex->blendtype);
+					texture_rgb_blend(&shi->specr, tcol, &shi->specr, texres.tin, colfac, mtex->blendtype);
 				}
 				if(mtex->mapto & MAP_COLMIR) {
 					// exception for envmap only
 					if(tex->type==TEX_ENVMAP && mtex->blendtype==MTEX_BLEND) {
-						fact= texres.tin*mtex->colfac;
+						fact= texres.tin*colfac;
 						facm= 1.0- fact;
 						shi->refcol[0]= fact + facm*shi->refcol[0];
 						shi->refcol[1]= fact*tcol[0] + facm*shi->refcol[1];
@@ -1669,7 +1715,7 @@ void do_material_tex(ShadeInput *shi)
 						shi->refcol[3]= fact*tcol[2] + facm*shi->refcol[3];
 					}
 					else {
-						texture_rgb_blend(&shi->mirr, tcol, &shi->mirr, texres.tin, mtex->colfac, mtex->blendtype);
+						texture_rgb_blend(&shi->mirr, tcol, &shi->mirr, texres.tin, colfac, mtex->blendtype);
 					}
 				}
 			}
@@ -1685,14 +1731,37 @@ void do_material_tex(ShadeInput *shi)
 						fact= Tnor*tex->norfac;
 						if(fact>1.0) fact= 1.0; else if(fact<-1.0) fact= -1.0;
 						facm= 1.0- fact;
-						shi->vn[0]= facm*shi->vn[0] + fact*texres.nor[0];
-						shi->vn[1]= facm*shi->vn[1] + fact*texres.nor[1];
-						shi->vn[2]= facm*shi->vn[2] + fact*texres.nor[2];
+						if(shi->mat->mode & MA_TANGENT_V) {
+							shi->tang[0]= facm*shi->tang[0] + fact*texres.nor[0];
+							shi->tang[1]= facm*shi->tang[1] + fact*texres.nor[1];
+							shi->tang[2]= facm*shi->tang[2] + fact*texres.nor[2];
+						}
+						else {
+							shi->vn[0]= facm*shi->vn[0] + fact*texres.nor[0];
+							shi->vn[1]= facm*shi->vn[1] + fact*texres.nor[1];
+							shi->vn[2]= facm*shi->vn[2] + fact*texres.nor[2];
+						}
 					}
 					else {
-						shi->vn[0]+= Tnor*tex->norfac*texres.nor[0];
-						shi->vn[1]+= Tnor*tex->norfac*texres.nor[1];
-						shi->vn[2]+= Tnor*tex->norfac*texres.nor[2];
+						if(shi->mat->mode & MA_TANGENT_V) {
+							shi->tang[0]+= Tnor*tex->norfac*texres.nor[0];
+							shi->tang[1]+= Tnor*tex->norfac*texres.nor[1];
+							shi->tang[2]+= Tnor*tex->norfac*texres.nor[2];
+						}
+						else {
+							float nor[3], dot;
+						
+							/* prevent bump to become negative normal */
+							nor[0]= Tnor*tex->norfac*texres.nor[0];
+							nor[1]= Tnor*tex->norfac*texres.nor[1];
+							nor[2]= Tnor*tex->norfac*texres.nor[2];
+							
+							dot= 0.5f + 0.5f*INPR(nor, shi->vn);
+							
+							shi->vn[0]+= dot*nor[0];
+							shi->vn[1]+= dot*nor[1];
+							shi->vn[2]+= dot*nor[2];
+						}
 					}					
 					Normalise(shi->vn);
 					
@@ -1720,14 +1789,14 @@ void do_material_tex(ShadeInput *shi)
 				
 				if(rgbnor & TEX_RGB) {
 					if(texres.talpha) texres.tin= texres.ta;
-					else texres.tin= (0.35*texres.tr+0.45*texres.tg+0.2*texres.tb);
+					else texres.tin= (0.35f*texres.tr+0.45f*texres.tg+0.2f*texres.tb);
 				}
 
 				if(mtex->maptoneg & MAP_DISPLACE) {
-					factt= (0.5-texres.tin)*mtex->dispfac; facmm= 1.0-factt;
+					factt= (texres.tin-0.5f)*mtex->dispfac*stencilTin; facmm= 1.0f-factt;
 				}
 				else {
-					factt= (texres.tin-0.5)*mtex->dispfac; facmm= 1.0-factt;
+					factt= (0.5f-texres.tin)*mtex->dispfac*stencilTin; facmm= 1.0f-factt;
 				}
 
 				if(mtex->blendtype==MTEX_BLEND) {
@@ -1750,6 +1819,9 @@ void do_material_tex(ShadeInput *shi)
 			}
 
 			if(mtex->mapto & MAP_VARS) {
+				/* stencil maps on the texture control slider, not texture intensity value */
+				float varfac= mtex->varfac*stencilTin;
+				
 				if(rgbnor & TEX_RGB) {
 					if(texres.talpha) texres.tin= texres.ta;
 					else texres.tin= (0.35*texres.tr+0.45*texres.tg+0.2*texres.tb);
@@ -1758,25 +1830,25 @@ void do_material_tex(ShadeInput *shi)
 				if(mtex->mapto & MAP_REF) {
 					int flip= mtex->maptoneg & MAP_REF;
 
-					shi->refl= texture_value_blend(mtex->def_var, shi->refl, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					shi->refl= texture_value_blend(mtex->def_var, shi->refl, texres.tin, varfac, mtex->blendtype, flip);
 					if(shi->refl<0.0) shi->refl= 0.0;
 				}
 				if(mtex->mapto & MAP_SPEC) {
 					int flip= mtex->maptoneg & MAP_SPEC;
 					
-					shi->spec= texture_value_blend(mtex->def_var, shi->spec, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					shi->spec= texture_value_blend(mtex->def_var, shi->spec, texres.tin, varfac, mtex->blendtype, flip);
 					if(shi->spec<0.0) shi->spec= 0.0;
 				}
 				if(mtex->mapto & MAP_EMIT) {
 					int flip= mtex->maptoneg & MAP_EMIT;
 
-					shi->emit= texture_value_blend(mtex->def_var, shi->emit, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					shi->emit= texture_value_blend(mtex->def_var, shi->emit, texres.tin, varfac, mtex->blendtype, flip);
 					if(shi->emit<0.0) shi->emit= 0.0;
 				}
 				if(mtex->mapto & MAP_ALPHA) {
 					int flip= mtex->maptoneg & MAP_ALPHA;
 
-					shi->alpha= texture_value_blend(mtex->def_var, shi->alpha, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					shi->alpha= texture_value_blend(mtex->def_var, shi->alpha, texres.tin, varfac, mtex->blendtype, flip);
 					if(shi->alpha<0.0) shi->alpha= 0.0;
 					else if(shi->alpha>1.0) shi->alpha= 1.0;
 				}
@@ -1785,7 +1857,7 @@ void do_material_tex(ShadeInput *shi)
 					float har;  // have to map to 0-1
 					
 					har= ((float)shi->har)/128.0;
-					har= 128.0*texture_value_blend(mtex->def_var, har, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					har= 128.0*texture_value_blend(mtex->def_var, har, texres.tin, varfac, mtex->blendtype, flip);
 					
 					if(har<1.0) shi->har= 1; 
 					else if(har>511.0) shi->har= 511;
@@ -1794,21 +1866,28 @@ void do_material_tex(ShadeInput *shi)
 				if(mtex->mapto & MAP_RAYMIRR) {
 					int flip= mtex->maptoneg & MAP_RAYMIRR;
 
-					shi->ray_mirror= texture_value_blend(mtex->def_var, shi->ray_mirror, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					shi->ray_mirror= texture_value_blend(mtex->def_var, shi->ray_mirror, texres.tin, varfac, mtex->blendtype, flip);
 					if(shi->ray_mirror<0.0) shi->ray_mirror= 0.0;
 					else if(shi->ray_mirror>1.0) shi->ray_mirror= 1.0;
 				}
 				if(mtex->mapto & MAP_TRANSLU) {
 					int flip= mtex->maptoneg & MAP_TRANSLU;
 
-					shi->translucency= texture_value_blend(mtex->def_var, shi->translucency, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					shi->translucency= texture_value_blend(mtex->def_var, shi->translucency, texres.tin, varfac, mtex->blendtype, flip);
 					if(shi->translucency<0.0) shi->translucency= 0.0;
 					else if(shi->translucency>1.0) shi->translucency= 1.0;
 				}
+				if(mtex->mapto & MAP_LAYER) {
+					int flip= mtex->maptoneg & MAP_LAYER;
+					
+					shi->layerfac= texture_value_blend(mtex->def_var, shi->layerfac, texres.tin, varfac, mtex->blendtype, flip);
+					if(shi->layerfac<0.0) shi->layerfac= 0.0;
+					else if(shi->layerfac>1.0) shi->layerfac= 1.0;
+				}
 				if(mtex->mapto & MAP_AMB) {
 					int flip= mtex->maptoneg & MAP_AMB;
 
-					shi->amb= texture_value_blend(mtex->def_var, shi->amb, texres.tin, mtex->varfac, mtex->blendtype, flip);
+					shi->amb= texture_value_blend(mtex->def_var, shi->amb, texres.tin, varfac, mtex->blendtype, flip);
 					if(shi->amb<0.0) shi->amb= 0.0;
 					else if(shi->amb>1.0) shi->amb= 1.0;
 				}
@@ -1945,7 +2024,7 @@ void do_halo_tex(HaloRen *har, float xn, float yn, float *colf)
 /* ------------------------------------------------------------------------- */
 
 /* hor and zen are RGB vectors, blend is 1 float, should all be initialized */
-void do_sky_tex(float *lo, float *dxyview, float *hor, float *zen, float *blend)
+void do_sky_tex(float *rco, float *lo, float *dxyview, float *hor, float *zen, float *blend, int skyflag)
 {
 	MTex *mtex;
 	TexResult texres;
@@ -1989,7 +2068,7 @@ void do_sky_tex(float *lo, float *dxyview, float *hor, float *zen, float *blend)
 			
 			case TEXCO_H_SPHEREMAP:
 			case TEXCO_H_TUBEMAP:
-				if(R.wrld.skytype & WO_ZENUP) {
+				if(skyflag & WO_ZENUP) {
 					if(mtex->texco==TEXCO_H_TUBEMAP) tubemap(lo[0], lo[2], lo[1], tempvec, tempvec+1);
 					else spheremap(lo[0], lo[2], lo[1], tempvec, tempvec+1);
 					/* tube/spheremap maps for outside view, not inside */
@@ -2013,6 +2092,22 @@ void do_sky_tex(float *lo, float *dxyview, float *hor, float *zen, float *blend)
 					MTC_Mat4MulVecfl(mtex->object->imat, tempvec);
 					co= tempvec;
 				}
+				break;
+				
+			case TEXCO_GLOB:
+				if(rco) {
+					VECCOPY(tempvec, rco);
+					MTC_Mat4MulVecfl(R.viewinv, tempvec);
+					co= tempvec;
+				}
+				else
+					co= lo;
+				
+//				VECCOPY(shi->dxgl, shi->dxco);
+//				MTC_Mat3MulVecfl(R.imat, shi->dxco);
+//				VECCOPY(shi->dygl, shi->dyco);
+//				MTC_Mat3MulVecfl(R.imat, shi->dyco);
+				break;
 			}
 			
 			/* placement */			
@@ -2079,7 +2174,7 @@ void do_sky_tex(float *lo, float *dxyview, float *hor, float *zen, float *blend)
 				if(mtex->mapto & (WOMAP_ZENUP+WOMAP_ZENDOWN)) {
 					ok= 0;
 					if(R.wrld.skytype & WO_SKYREAL) {
-						if((R.wrld.skytype & WO_ZENUP)) {
+						if((skyflag & WO_ZENUP)) {
 							if(mtex->mapto & WOMAP_ZENUP) ok= 1;
 						}
 						else if(mtex->mapto & WOMAP_ZENDOWN) ok= 1;
@@ -2331,7 +2426,7 @@ void render_realtime_texture(ShadeInput *shi)
 		tex2.type= TEX_IMAGE;
 	}
 	
-	if(((int)(shi->ys+0.5)) & 1) tex= &tex1; else tex= &tex2;	// threadsafe
+	if(shi->ys & 1) tex= &tex1; else tex= &tex2;	// threadsafe
 	
 	ima = shi->vlr->tface->tpage;
 	if(ima) {
