@@ -1,5 +1,5 @@
 /**
- * $Id: drawipo.c,v 1.47 2005/05/14 11:28:53 lukep Exp $
+ * $Id: drawipo.c,v 1.84 2006/06/25 12:30:33 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -45,10 +45,12 @@
 #endif   
 
 #include "BMF_Api.h"
+#include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
+#include "DNA_action_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_key_types.h"
@@ -59,45 +61,52 @@
 #include "DNA_sequence_types.h"
 #include "DNA_userdef_types.h"
 
-#include "BKE_utildefines.h"
 #include "BKE_curve.h"
-#include "BKE_ipo.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
+#include "BKE_ipo.h"
 #include "BKE_key.h"
+#include "BKE_utildefines.h"
 
+#include "BIF_cursors.h"
 #include "BIF_gl.h"
+#include "BIF_graphics.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
 #include "BIF_interface.h"
 #include "BIF_mywindow.h"
 #include "BIF_space.h"
+#include "BIF_toolbox.h"
 #include "BIF_glutil.h"
+#include "BIF_editseq.h"
 
 #include "BSE_drawipo.h"
 #include "BSE_view.h"
 #include "BSE_editipo.h"
 #include "BSE_editipo_types.h"
-#include "BSE_editaction_types.h"
 #include "BSE_editnla_types.h"
 
+#include "BPY_extern.h"
+
 #include "mydevice.h"
 #include "blendef.h"
 #include "butspace.h"	// shouldnt be...
+#include "winlay.h"
 
 /* local define... also used in editipo ... */
 #define ISPOIN(a, b, c)                       ( (a->b) && (a->c) )  
 #define ISPOIN3(a, b, c, d)           ( (a->b) && (a->c) && (a->d) )
 #define ISPOIN4(a, b, c, d, e)        ( (a->b) && (a->c) && (a->d) && (a->e) )   
 
-#define IPOBUTX	65
-#define IPOSTEP 35			/* minimum pixels per gridstep */
+		/* minimum pixels per gridstep */
+#define IPOSTEP 35
 
 static float ipogrid_dx, ipogrid_dy, ipogrid_startx, ipogrid_starty;
 static int ipomachtx, ipomachty;
 
-static int vertymin, vertymax, horxmin, horxmax;	/* globals om LEFTMOUSE op scrollbar te testen */
+static int vertymin, vertymax, horxmin, horxmax;	/* globals to test LEFTMOUSE for scrollbar */
 
-extern short ACTWIDTH;
+extern short ACTWIDTH;		/* this is ugly! */
 
 static void scroll_prstr(float x, float y, float val, char dir, int disptype)
 {
@@ -148,13 +157,27 @@ static void step_to_grid(float *step, int *macht)
 		*step= rem*pow(10.0, (float)*macht);
 		
 		// partial of a frame have no meaning
-		if(curarea->spacetype==SPACE_TIME) {
+		switch(curarea->spacetype) {
+		case SPACE_TIME: {
 			SpaceTime *stime= curarea->spacedata.first;
 			if(stime->flag & TIME_DRAWFRAMES) {
 				rem = 1.0;
 				*step = 1.0;
 			}
+			break;
 		}
+		case SPACE_SEQ: {
+			SpaceTime * sseq= curarea->spacedata.first;
+			if (sseq->flag & SEQ_DRAWFRAMES) {
+				rem = 1.0;
+				*step = 1.0;
+			}
+		}
+		default:
+			break;
+		}
+
+		
 		
 		if(rem==1.0) (*macht)++;	// prevents printing 1.0 2.0 3.0 etc
 	}
@@ -180,12 +203,25 @@ void calc_ipogrid()
 	if(G.v2d==0) return;
 	
 	/* detect of we have seconds or frames, should become argument */
-	if(curarea->spacetype==SPACE_TIME) {
+
+	switch(curarea->spacetype) {
+	case SPACE_TIME: {
 		SpaceTime *stime= curarea->spacedata.first;
 		if(!(stime->flag & TIME_DRAWFRAMES)) {
 			secondgrid= 1;
 			secondiv= 0.01 * (float)G.scene->r.frs_sec;
 		}
+		break;
+	}
+	case SPACE_SEQ: {
+		SpaceSeq * sseq = curarea->spacedata.first;
+		if (!(sseq->flag & SEQ_DRAWFRAMES)) {
+			secondgrid = 1;
+			secondiv = 0.01 * (float)G.scene->r.frs_sec;
+		}
+	}
+	default:
+		break;
 	}
 	
 	space= G.v2d->cur.xmax - G.v2d->cur.xmin;
@@ -307,11 +343,21 @@ void draw_ipogrid(void)
 	/* Limits box */
 	if(curarea->spacetype==SPACE_IPO) {
 		if(G.sipo->blocktype==ID_SEQ) {
+			Sequence * last_seq = get_last_seq();
+			float start = 0.0;
+			float end = 100.0;
+
+			if (last_seq && 
+			    ((last_seq->flag & SEQ_IPO_FRAME_LOCKED) != 0)) {
+				start = last_seq->startdisp;
+				end = last_seq->enddisp;
+			}
+
 			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
-			glRectf(0.0,  0.0,  100.0,  1.0); 
+			glRectf(start,  0.0,  end,  1.0); 
 			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 		}
-		else if(ELEM(G.sipo->blocktype, ID_CU, IPO_CO)) {
+		else if(ELEM(G.sipo->blocktype, ID_CU, ID_CO)) {
 			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
 			glRectf(0.0,  1.0,  G.v2d->cur.xmax,  1.0); 
 			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
@@ -338,7 +384,7 @@ void ipoco_to_areaco(View2D *v2d, float *vec, short *mval)
 {
 	float x, y;
 	
-	mval[0]= 3200;
+	mval[0]= IS_CLIPPED;
 	
 	x= (vec[0] - v2d->cur.xmin)/(v2d->cur.xmax-v2d->cur.xmin);
 	y= (vec[1] - v2d->cur.ymin)/(v2d->cur.ymax-v2d->cur.ymin);
@@ -380,7 +426,66 @@ int in_ipo_buttons(void)
 	else return 1;
 }
 
+static View2D *spacelink_get_view2d(SpaceLink *sl)
+{
+	if(sl->spacetype==SPACE_IPO) 
+		return &((SpaceIpo *)sl)->v2d;
+	else if(sl->spacetype==SPACE_SOUND) 
+		return &((SpaceSound *)sl)->v2d;
+	if(sl->spacetype==SPACE_ACTION) 
+		return &((SpaceAction *)sl)->v2d;
+	if(sl->spacetype==SPACE_NLA) 
+		return &((SpaceNla *)sl)->v2d;
+	if(sl->spacetype==SPACE_TIME) 
+		return &((SpaceTime *)sl)->v2d;
+	if(sl->spacetype==SPACE_SEQ)
+		return &((SpaceSeq *)sl)->v2d;
+	return NULL;
+}
 
+/* copies changes in this view from or to all 2d views with lock option open */
+/* do not call this inside of drawing routines, to prevent eternal loops */
+void view2d_do_locks(ScrArea *cursa, int flag)
+{
+	ScrArea *sa;
+	View2D *v2d, *curv2d;
+	SpaceLink *sl;
+	
+	curv2d= spacelink_get_view2d(cursa->spacedata.first);
+	if(curv2d==NULL) return;
+	if((curv2d->flag & V2D_VIEWLOCK)==0) return;
+
+	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+		if(sa!=cursa) {
+			for(sl= sa->spacedata.first; sl; sl= sl->next) {
+				
+				v2d= spacelink_get_view2d(sl);
+				if(v2d) {
+					if(v2d->flag & V2D_VIEWLOCK) {
+						if(flag & V2D_LOCK_COPY) {
+							v2d->cur.xmin= curv2d->cur.xmin;
+							v2d->cur.xmax= curv2d->cur.xmax;
+						}
+						else {
+							curv2d->cur.xmin= v2d->cur.xmin;
+							curv2d->cur.xmax= v2d->cur.xmax;
+							scrarea_queue_winredraw(sa);
+						}
+						
+						if(flag & V2D_LOCK_REDRAW) {
+							if(sl == sa->spacedata.first)
+								scrarea_do_windraw(sa);
+						}
+						else
+							scrarea_queue_winredraw(sa);
+					}
+				}
+			}
+		}
+	}
+}
+
+/* event based, note: curarea is in here... */
 void view2d_zoom(View2D *v2d, float factor, int winx, int winy) 
 {
 	float dx= factor*(v2d->cur.xmax-v2d->cur.xmin);
@@ -394,6 +499,7 @@ void view2d_zoom(View2D *v2d, float factor, int winx, int winy)
 		v2d->cur.ymax-= dy;
 	}
 	test_view2d(v2d, winx, winy);
+	view2d_do_locks(curarea, V2D_LOCK_COPY);
 }
 
 
@@ -407,13 +513,15 @@ void test_view2d(View2D *v2d, int winx, int winy)
 	if(v2d->scroll & L_SCROLL) winx-= SCROLLB;
 	if(v2d->scroll & B_SCROLL) winy-= SCROLLH;
 	
+	/* header completely closed window */
+	if(winy<=0) return;
+	
 	cur= &v2d->cur;
 	tot= &v2d->tot;
 	
 	dx= cur->xmax-cur->xmin;
 	dy= cur->ymax-cur->ymin;
 
-
 	/* Reevan's test */
 	if (v2d->keepzoom & V2D_LOCKZOOM_Y)
 		v2d->cur.ymax=v2d->cur.ymin+((float)winy);
@@ -584,6 +692,27 @@ void test_view2d(View2D *v2d, int winx, int winy)
 	}
 }
 
+#define IPOBUTX 70
+static int calc_ipobuttonswidth(ScrArea *sa)
+{
+	SpaceIpo *sipo= sa->spacedata.first;
+	EditIpo *ei;
+	int ipowidth = IPOBUTX;
+	int a;
+	
+	/* default width when no space ipo or no channels */
+	if (sipo == NULL) return IPOBUTX;
+	if ((sipo->totipo==0) || (sipo->editipo==NULL)) return IPOBUTX;
+
+	ei= sipo->editipo;
+	
+	for(a=0; a<sipo->totipo; a++, ei++) {
+		if (BMF_GetStringWidth(G.font, ei->name) + 18 > ipowidth) 
+			ipowidth = BMF_GetStringWidth(G.font, ei->name) + 18;
+	}
+	return ipowidth;
+
+}
 
 void calc_scrollrcts(ScrArea *sa, View2D *v2d, int winx, int winy)
 {
@@ -592,17 +721,23 @@ void calc_scrollrcts(ScrArea *sa, View2D *v2d, int winx, int winy)
 	v2d->mask.ymax= winy;
 	
 	if(sa->spacetype==SPACE_ACTION) {
-		v2d->mask.xmin+= ACTWIDTH;
-		v2d->hor.xmin+=ACTWIDTH;
+		if(sa->winx > ACTWIDTH+50) { 
+			v2d->mask.xmin+= ACTWIDTH;
+			v2d->hor.xmin+=ACTWIDTH;
+		}
 	}
 	else if(sa->spacetype==SPACE_NLA){
-		v2d->mask.xmin+= NLAWIDTH;
-		v2d->hor.xmin+=NLAWIDTH;
+		if(sa->winx > NLAWIDTH+50) { 
+			v2d->mask.xmin+= NLAWIDTH;
+			v2d->hor.xmin+=NLAWIDTH;
+		}
 	}
 	else if(sa->spacetype==SPACE_IPO) {
-		v2d->mask.xmax-= IPOBUTX;
+		int ipobutx = calc_ipobuttonswidth(sa);
 		
-		if(v2d->mask.xmax<IPOBUTX)
+		v2d->mask.xmax-= ipobutx;
+		
+		if(v2d->mask.xmax<ipobutx)
 			v2d->mask.xmax= winx;
 	}
 	
@@ -640,7 +775,7 @@ static void draw_solution_line(View2D *v2d, float h)
 	vec[0]= v2d->cur.xmin;
 	vec[1]= h;
 	ipoco_to_areaco(v2d, vec, mval);
-	if(mval[0]!=3200) {
+	if(mval[0]!=IS_CLIPPED) {
 		glBegin(GL_LINES);
 		glVertex2f(v2d->vert.xmin, mval[1]);
 		glVertex2f(v2d->vert.xmax, mval[1]);
@@ -757,10 +892,16 @@ void drawscroll(int disptype)
 			
 			if(curarea->spacetype==SPACE_OOPS);
 			else if(curarea->spacetype==SPACE_SEQ) {
-				fac2= val/(float)G.scene->r.frs_sec;
-				tim= floor(fac2);
-				fac2= fac2-tim;
-				scroll_prstr(fac, 3.0+(float)(hor.ymin), tim+G.scene->r.frs_sec*fac2/100.0, 'h', disptype);
+				SpaceSeq * sseq = curarea->spacedata.first;
+				if (sseq->flag & SEQ_DRAWFRAMES) {
+					ipomachtx = 1;
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
+				} else {
+					fac2= val/(float)G.scene->r.frs_sec;
+					tim= floor(fac2);
+					fac2= fac2-tim;
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), tim+G.scene->r.frs_sec*fac2/100.0, 'h', disptype);
+				}
 			}
 			else if (curarea->spacetype==SPACE_SOUND) {
 				SpaceSound *ssound= curarea->spacedata.first;
@@ -786,6 +927,21 @@ void drawscroll(int disptype)
 					scroll_prstr(fac, 3.0+(float)(hor.ymin), fac2, 'h', disptype);
 				}
 			}
+			else if (curarea->spacetype==SPACE_IPO) {
+				EditIpo *ei= get_active_editipo();
+				
+				if(ei && ei->icu && ei->icu->driver) {
+					int adrcode= ei->icu->driver->adrcode;
+					
+					if(adrcode==OB_ROT_X || adrcode==OB_ROT_Y || adrcode==OB_ROT_Z) {
+						scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'v', IPO_DISPDEGR);
+					}
+					else 
+						scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
+				}
+				else 
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
+			}
 			else {
 				scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
 			}
@@ -859,19 +1015,22 @@ static void draw_ipobuts(SpaceIpo *sipo)
 {
 	ScrArea *area= sipo->area;
 	View2D *v2d= &sipo->v2d;
+	Object *ob= OBACT;
 	uiBlock *block;
 	uiBut *but;
 	EditIpo *ei;
-	int a, y, sel, tot;
+	int a, y, sel, tot, ipobutx;
 	char naam[20];
 	
-	if(area->winx<IPOBUTX) return;
+	if(area->winx< calc_ipobuttonswidth(area)) return;
 	
 	if(sipo->butofs) {
 		tot= 30+IPOBUTY*sipo->totipo;
 		if(tot<area->winy) sipo->butofs= 0;
 	}
 	
+	ipobutx = calc_ipobuttonswidth(area);
+	
 	BIF_ThemeColor(TH_SHADE2);
 	glRects(v2d->mask.xmax,  0,  area->winx,  area->winy);
 	
@@ -886,9 +1045,18 @@ static void draw_ipobuts(SpaceIpo *sipo)
 
 	ei= sipo->editipo;
 	y= area->winy-30+sipo->butofs;
+	
+	if(ob && sipo->blocktype==ID_KE) {
+		int icon;
+		if(ob->shapeflag & OB_SHAPE_LOCK) icon= ICON_PIN_HLT; else icon= ICON_PIN_DEHLT;
+		uiDefIconButBitS(block, TOG, OB_SHAPE_LOCK, B_SETKEY, icon, 
+						 v2d->mask.xmax+18,y,25,20, &ob->shapeflag, 0, 0, 0, 0, "Always show the current Shape for this Object");
+		y-= IPOBUTY;
+	}
+	
 	for(a=0; a<sipo->totipo; a++, ei++, y-=IPOBUTY) {
 		// this button defines visiblity, bit zero of flag (IPO_VISIBLE)
-		but= uiDefButS(block, TOG|BIT|0, a+1, ei->name,  v2d->mask.xmax+18, y, IPOBUTX-15, IPOBUTY-1, &(ei->flag), 0, 0, 0, 0, "");
+		but= uiDefButBitS(block, TOG, IPO_VISIBLE, a+1, ei->name,  v2d->mask.xmax+18, y, ipobutx-15, IPOBUTY-1, &(ei->flag), 0, 0, 0, 0, "");
 		// no hilite, its not visible, but most of all the winmatrix is not correct later on...
 		uiButSetFlag(but, UI_TEXT_LEFT|UI_NO_HILITE);
 		
@@ -900,50 +1068,21 @@ static void draw_ipobuts(SpaceIpo *sipo)
 			sel= ei->flag & (IPO_SELECT + IPO_EDIT);
 			
 			uiEmboss((float)(v2d->mask.xmax+8), (float)(y+2), (float)(v2d->mask.xmax+15), (float)(y+IPOBUTY-2), sel);
+			
+			if(ei->icu->driver) {
+				cpack(0x0);
+				fdrawbox((float)v2d->mask.xmax+11,  (float)y+8,  (float)v2d->mask.xmax+12.5, (float)y+9.5);
+			}
 		}
 		
-		if(sipo->blocktype==ID_KE) {
-			Key *key= (Key *)sipo->from;
-			if(key==0 || key->type==KEY_NORMAL) break;
+		if(ei->flag & IPO_ACTIVE) {
+			cpack(0x0);
+			fdrawbox(v2d->mask.xmax+7,  y+1,  v2d->mask.xmax+16, y+IPOBUTY-1);
 		}
 	}
 	uiDrawBlock(block);
 }
 
-static void calc_ipoverts(SpaceIpo *sipo)
-{
-	View2D *v2d= &sipo->v2d;
-	EditIpo *ei;
-	BezTriple *bezt;
-	BPoint *bp;
-	int a, b;
-	
-	ei= G.sipo->editipo;
-	for(a=0; a<sipo->totipo; a++, ei++) {
-		if ISPOIN(ei, flag & IPO_VISIBLE, icu) {
-			if(ei->icu->bezt) {
-				bezt= ei->icu->bezt;
-				b= ei->icu->totvert;
-				while(b--) {
-					ipoco_to_areaco(v2d, bezt->vec[0], bezt->s[0]);
-					ipoco_to_areaco(v2d, bezt->vec[1], bezt->s[1]);
-					ipoco_to_areaco(v2d, bezt->vec[2], bezt->s[2]);
-					bezt++;
-				}
-			}
-			else if(ei->icu->bp) {
-				bp= ei->icu->bp;
-				b= ei->icu->totvert;
-				while(b--) {
-					ipoco_to_areaco(v2d, bp->vec, bp->s);
-					bp++;
-				}
-			}
-		}
-	}
-}
-
-
 static void draw_ipovertices(int sel)
 {
 	EditIpo *ei;
@@ -1233,7 +1372,9 @@ static void draw_ipocurves(int sel)
 								glVertex2fv(v1);
 							}
 							else {
-								resol= 3.0*sqrt(bezt->vec[1][0] - prevbezt->vec[1][0]);
+								/* resol not depending on horizontal resolution anymore, drivers for example... */
+								if(icu->driver) resol= 32;
+								else resol= 3.0*sqrt(bezt->vec[1][0] - prevbezt->vec[1][0]);
 								
 								if(resol<2) {
 									v1[0]= prevbezt->vec[1][0]+cycxofs;
@@ -1255,8 +1396,8 @@ static void draw_ipocurves(int sel)
 									
 									correct_bezpart(v1, v2, v3, v4);
 									
-									maakbez(v1[0], v2[0], v3[0], v4[0], data, resol);
-									maakbez(v1[1], v2[1], v3[1], v4[1], data+1, resol);
+									forward_diff_bezier(v1[0], v2[0], v3[0], v4[0], data, resol, 3);
+									forward_diff_bezier(v1[1], v2[1], v3[1], v4[1], data+1, resol, 3);
 									
 									fp= data;
 									while(resol--) {
@@ -1371,16 +1512,17 @@ static void draw_key(SpaceIpo *sipo, int visible)
 	View2D *v2d= &sipo->v2d;
 	Key *key;
 	KeyBlock *kb, *act=NULL;
+	Object *ob= OBACT;
 	unsigned int col;
+	int index;
 	
-	key= (Key *)sipo->from;
-	if(key==0)
+	key= ob_get_key((Object *)sipo->from);
+	if(key==NULL)
 		return;
 	
 	if(key->type== KEY_RELATIVE) if(visible==0) return;
 	
-	kb= key->block.first;
-	while(kb) {
+	for(index=1, kb= key->block.first; kb; kb= kb->next, index++) {
 		if(kb->type==KEY_LINEAR) setlinestyle(2);
 		else if(kb->type==KEY_BSPLINE) setlinestyle(4);
 		else setlinestyle(0);
@@ -1388,7 +1530,7 @@ static void draw_key(SpaceIpo *sipo, int visible)
 		if(kb==key->refkey) col= 0x22FFFF;
 		else col= 0xFFFF00;
 		
-		if( (kb->flag & SELECT)==0)	col-= 0x225500;
+		if(ob->shapenr!=index) col-= 0x225500;
 		else act= kb;
 		
 		cpack(col);
@@ -1398,7 +1540,6 @@ static void draw_key(SpaceIpo *sipo, int visible)
 		glVertex2f(v2d->cur.xmax, kb->pos);
 		glEnd();
 		
-		kb= kb->next;
 	}
 	
 	if(act) {
@@ -1425,11 +1566,14 @@ static void draw_key(SpaceIpo *sipo, int visible)
 #define B_MUL_IPO		3402
 #define B_TRANS_IPO		3403
 #define B_IPO_NONE		3404
+#define B_IPO_DRIVER	3405
+#define B_IPO_REDR		3406
+#define B_IPO_DEPCHANGE	3407
+#define B_IPO_DRIVERTYPE 3408
 
 static float hspeed= 0;
 
-
-static void boundbox_ipo_visible(SpaceIpo *si)
+static void boundbox_ipo_curves(SpaceIpo *si)
 {
 	EditIpo *ei;
 	Key *key;
@@ -1456,7 +1600,7 @@ static void boundbox_ipo_visible(SpaceIpo *si)
 	}
 	/* keylines? */
 	if(si->blocktype==ID_KE) {
-		key= (Key *)si->from;
+		key= ob_get_key((Object *)si->from);
 		if(key && key->block.first) {
 			kb= key->block.first;
 			if(kb->pos < si->v2d.tot.ymin) si->v2d.tot.ymin= kb->pos;
@@ -1471,6 +1615,7 @@ static void boundbox_ipo_visible(SpaceIpo *si)
 /* is used for both read and write... */
 static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float max)
 {
+	Object *ob= OBACT;
 	EditIpo *ei;
 	BezTriple *bezt;
 	float median[3];
@@ -1514,20 +1659,15 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 	/* check for keys */
 	if(tot==0) {
 		if(G.sipo->blocktype==ID_KE) {
-			Key *key= (Key *)G.sipo->from;
+			Key *key= ob_get_key((Object *)G.sipo->from);
 			KeyBlock *kb;
 			
-			if(key==0) return;
+			if(key==NULL || ob->shapenr==0) return;
 			iskey= 1;
 			
-			kb= key->block.first;
-			while(kb) {
-				if(kb->flag & SELECT) {
-					median[1]+= kb->pos;
-					tot++;
-				}
-				kb= kb->next;
-			}
+			kb= BLI_findlink(&key->block, ob->shapenr-1);
+			median[1]+= kb->pos;
+			tot++;
 		}
 	}
 	if(tot==0) return;
@@ -1540,20 +1680,21 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 	
 		VECCOPY(si->median, median);
 		
+		uiBlockBeginAlign(block);
 		if(tot==1) {
 			if(iskey) 
 				uiDefButF(block, NUM, B_TRANS_IPO, "Key Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 10, 0, "");
 			else {
-				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex X:",	10, 100, 300, 19, &(si->median[0]), min, max, 100, 0, "");
-				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex X:",	10, 100, 150, 19, &(si->median[0]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex Y:",	160, 100, 150, 19, &(si->median[1]), min, max, 100, 0, "");
 			}
 		}
 		else {
 			if(iskey) 
 				uiDefButF(block, NUM, B_TRANS_IPO, "Median Key Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 10, 0, "");
 			else {
-				uiDefButF(block, NUM, B_TRANS_IPO, "Median X:",	10, 100, 300, 19, &(si->median[0]), min, max, 100, 0, "");
-				uiDefButF(block, NUM, B_TRANS_IPO, "Median Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Median X:",	10, 100, 150, 19, &(si->median[0]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Median Y:",	160, 100, 150, 19, &(si->median[1]), min, max, 100, 0, "");
 			}
 		}
 	}
@@ -1561,19 +1702,15 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 		VecSubf(median, si->median, median);
 
 		if(G.sipo->blocktype==ID_KE) {
-			Key *key= (Key *)G.sipo->from;
+			Key *key= ob_get_key((Object *)G.sipo->from);
 			KeyBlock *kb;
 			
-			if(key==0) return;
+			if(key==NULL || ob->shapenr==0) return;
 			
-			kb= key->block.first;
-			while(kb) {
-				if(kb->flag & SELECT) {
-					kb->pos+= median[1];
-					tot++;
-				}
-				kb= kb->next;
-			}			
+			kb= BLI_findlink(&key->block, ob->shapenr-1);
+			kb->pos+= median[1];
+			tot++;
+
 			sort_keys(key);
 		}
 	}
@@ -1617,8 +1754,33 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 
 void do_ipobuts(unsigned short event)
 {
-
+	Object *ob= OBACT;
+	EditIpo *ei;
+	
+	if(G.sipo->from==NULL) return;
+	
 	switch(event) {
+	case B_IPO_REDR:
+		ei= get_active_editipo();
+		if(ei) {
+			if(ei->icu->driver) {
+				if (ei->icu->driver->type == IPO_DRIVER_TYPE_PYTHON) {
+ 					/* first del pydriver's global dict, just in case
+					 * an available pydrivers.py module needs to be reloaded */
+					BPY_pydriver_update();
+					/* eval user's expression once for validity; update DAG */
+					BPY_pydriver_eval(ei->icu->driver);
+					DAG_scene_sort(G.scene);
+				}
+				else if(G.sipo->blocktype==ID_KE || G.sipo->blocktype==ID_AC) 
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+				else
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_OB);
+			}
+		}
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWVIEW3D, 0);
+		break;
 	case B_SETSPEED:
 		set_speed_editipo(hspeed);
 		break;
@@ -1631,43 +1793,239 @@ void do_ipobuts(unsigned short event)
 		editipo_changed(G.sipo, 1);
 		allqueue(REDRAWIPO, 0);
 		break;
+	case B_SETKEY:
+		ob->shapeflag &= ~OB_SHAPE_TEMPLOCK;
+		DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		break;
+	case B_IPO_DRIVER:
+		ei= get_active_editipo();
+		if(ei) {
+			if(ei->icu==NULL) {
+				ei->icu= verify_ipocurve(G.sipo->from, G.sipo->blocktype, G.sipo->actname, G.sipo->constname, ei->adrcode);
+				ei->flag |= IPO_SELECT;
+				ei->icu->flag= ei->flag;
+			}
+			if(ei->icu->driver) {
+				MEM_freeN(ei->icu->driver);
+				ei->icu->driver= NULL;
+				if(ei->icu->bezt==NULL) {
+					BLI_remlink( &(G.sipo->ipo->curve), ei->icu);
+					free_ipo_curve(ei->icu);
+					ei->icu= NULL;
+				}
+			}
+			else {
+				ei->icu->driver= MEM_callocN(sizeof(IpoDriver), "ipo driver");
+				ei->icu->driver->blocktype= ID_OB;
+				ei->icu->driver->adrcode= OB_LOC_X;
+			}
+
+			allqueue(REDRAWVIEW3D, 0);
+			allqueue(REDRAWIPO, 0);
+			allqueue(REDRAWBUTSEDIT, 0);
+			DAG_scene_sort(G.scene);
+			
+			BIF_undo_push("Add/Remove Ipo driver");
+		}
+		break;
+	case B_IPO_DRIVERTYPE:
+		ei= get_active_editipo();
+		if(ei) {
+			if(ei->icu->driver) {
+				IpoDriver *driver= ei->icu->driver;
+
+				if(driver->type == IPO_DRIVER_TYPE_PYTHON) {
+					/* pydriver expression shouldn't reference own ob,
+					 * so we need to store ob ptr to check against it */
+					driver->ob= ob;
+				}
+				else {
+					driver->ob= NULL;
+					driver->blocktype= ID_OB;
+					driver->adrcode= OB_LOC_X;
+					driver->flag &= ~IPO_DRIVER_FLAG_INVALID;
+				}
+			}
+			allqueue(REDRAWVIEW3D, 0);
+			allqueue(REDRAWIPO, 0);
+			allqueue(REDRAWBUTSEDIT, 0);
+			DAG_scene_sort(G.scene);
+	
+			BIF_undo_push("Change Ipo driver type");
+		}
+		break;
+	case B_IPO_DEPCHANGE:
+		ei= get_active_editipo();
+		if(ei) {
+			if(ei->icu->driver) {
+				IpoDriver *driver= ei->icu->driver;
+				
+				if(driver->type == IPO_DRIVER_TYPE_PYTHON) {
+				}
+				else {
+					if(driver->ob) {
+						if(ob==driver->ob) {
+							error("Cannot assign a Driver to own Object");
+							driver->ob= NULL;
+						}
+						else {
+							/* check if type is still OK */
+							if(driver->ob->type==OB_ARMATURE && driver->blocktype==ID_AR);
+							else driver->blocktype= ID_OB;
+						}
+					}
+				}
+				DAG_scene_sort(G.scene);
+				
+				if(G.sipo->blocktype==ID_KE || G.sipo->blocktype==ID_AC) 
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+				else
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_OB);
+			}
+		}
+		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		break;
 	}
 }
 
+static char *ipodriver_modeselect_pup(Object *ob)
+{
+	static char string[265];
+	char tmpstr[64];
+	char formatstring[64];
+	
+	strcpy(string, "Driver type: %t");
+	
+	strcpy(formatstring, "|%s %%x%d %%i%d");
+	
+	if(ob) {
+		sprintf(tmpstr,formatstring,"Object",ID_OB, ICON_OBJECT);
+		strcat(string,tmpstr);
+	}
+	if(ob && ob->type==OB_ARMATURE) {
+		sprintf(tmpstr,formatstring,"Pose",ID_AR, ICON_POSE_DEHLT);
+		strcat(string,tmpstr);
+	}
+	
+	return (string);
+}
+
+static char *ipodriver_channelselect_pup(void)
+{
+	static char string[1024];
+	char *tmp;
+	
+	strcpy(string, "Driver channel: %t");
+	tmp= string+strlen(string);
+	
+	tmp+= sprintf(tmp, "|Loc X %%x%d", OB_LOC_X);
+	tmp+= sprintf(tmp, "|Loc Y %%x%d", OB_LOC_Y);
+	tmp+= sprintf(tmp, "|Loc Z %%x%d", OB_LOC_Z);
+	tmp+= sprintf(tmp, "|Rot X %%x%d", OB_ROT_X);
+	tmp+= sprintf(tmp, "|Rot Y %%x%d", OB_ROT_Y);
+	tmp+= sprintf(tmp, "|Rot Z %%x%d", OB_ROT_Z);
+	tmp+= sprintf(tmp, "|Scale X %%x%d", OB_SIZE_X);
+	tmp+= sprintf(tmp, "|Scale Y %%x%d", OB_SIZE_Y);
+	tmp+= sprintf(tmp, "|Scale Z %%x%d", OB_SIZE_Z);
+	
+	return (string);
+}
 
 static void ipo_panel_properties(short cntrl)	// IPO_HANDLER_PROPERTIES
 {
-	extern int totipo_vis;	// editipo.c
+	extern int totipo_curve;	// editipo.c
 	uiBlock *block;
+	EditIpo *ei;
+	char name[48];
 	
 	block= uiNewBlock(&curarea->uiblocks, "ipo_panel_properties", UI_EMBOSS, UI_HELV, curarea->win);
 	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE | cntrl);
 	uiSetPanelHandler(IPO_HANDLER_PROPERTIES);  // for close and esc
 	if(uiNewPanel(curarea, block, "Transform Properties", "Ipo", 10, 230, 318, 204)==0) return;
 
+	/* this is new panel height, newpanel doesnt force new size on existing panels */
+	uiNewPanelHeight(block, 204);
+	
+	/* driver buttons first */
+	ei= get_active_editipo();
+	if(ei) {
+		
+		sprintf(name, "Driven Channel: %s", ei->name);
+		uiDefBut(block, LABEL, 0, name,		10, 265, 200, 19, NULL, 1.0, 0.0, 0, 0, "");
+		
+		if(ei->icu && ei->icu->driver) {
+			IpoDriver *driver= ei->icu->driver;
 
-	boundbox_ipo_visible(G.sipo);	// should not be needed... transform/draw calls should update
+			uiDefBut(block, BUT, B_IPO_DRIVER, "Remove",				210,265,100,20, NULL, 0.0f, 0.0f, 0, 0, "Remove Driver for this Ipo Channel");
+			
+			uiBlockBeginAlign(block);
+			uiDefIconButS(block, TOG, B_IPO_DRIVERTYPE, ICON_PYTHON, 10,240,25,20, &driver->type, (float)IPO_DRIVER_TYPE_NORMAL, (float)IPO_DRIVER_TYPE_PYTHON, 0, 0, "Use a one-line Python Expression as Driver");
+
+			if(driver->type == IPO_DRIVER_TYPE_PYTHON) {
+				uiDefBut(block, TEX, B_IPO_REDR, "",				35,240,275,20, driver->name, 0, 127, 0, 0, "Python Expression");
+	 			if(driver->flag & IPO_DRIVER_FLAG_INVALID) {
+					uiDefBut(block, LABEL, 0, "Error: invalid Python expression",
+							5,215,230,19, NULL, 0, 0, 0, 0, "");
+				}
+				uiBlockEndAlign(block);
+			}
+			else {
+				uiDefIDPoinBut(block, test_obpoin_but, ID_OB, B_IPO_DEPCHANGE, "OB:",	35, 240, 125, 20, &(driver->ob), "Driver Object");
+				if(driver->ob) {
+					int icon=ICON_OBJECT;
+					
+					if(driver->ob->type==OB_ARMATURE && driver->blocktype==ID_AR) {
+						icon = ICON_POSE_DEHLT;
+						uiDefBut(block, TEX, B_IPO_REDR, "BO:",				10,220,150,20, driver->name, 0, 31, 0, 0, "Bone name");
+					}
+					else driver->blocktype= ID_OB;	/* safety when switching object button */
+					
+					uiBlockBeginAlign(block);
+					uiDefIconTextButS(block, MENU, B_IPO_DEPCHANGE, icon, 
+									  ipodriver_modeselect_pup(driver->ob), 165,240,145,20, &(driver->blocktype), 0, 0, 0, 0, "Driver type");
+
+					uiDefButS(block, MENU, B_IPO_REDR, 
+								ipodriver_channelselect_pup(),			165,220,145,20, &(driver->adrcode), 0, 0, 0, 0, "Driver channel");
+				}
+				uiBlockEndAlign(block);
+			}
+		}
+		else {
+			uiDefBut(block, BUT, B_IPO_DRIVER, "Add Driver",	210,265,100,19, NULL, 0.0f, 0.0f, 0, 0, "Create a Driver for this Ipo Channel");
+		}
+	}
+	else 
+		uiDefBut(block, LABEL, 0, " ",		10, 265, 150, 19, NULL, 1.0, 0.0, 0, 0, "");
+
+	boundbox_ipo_curves(G.sipo);	// should not be needed... transform/draw calls should update
 	
 	/* note ranges for buttons below are idiot... we need 2 ranges, one for sliding scale, one for real clip */
-	if(G.sipo->ipo && G.sipo->ipo->curve.first && totipo_vis) {
+	if(G.sipo->ipo && G.sipo->ipo->curve.first && totipo_curve) {
 		extern int totipo_vertsel;	// editipo.c
 		uiDefBut(block, LABEL, 0, "Visible curves",		10, 200, 150, 19, NULL, 1.0, 0.0, 0, 0, "");
 		
-		uiDefButF(block, NUM, B_MUL_IPO, "Xmin:",		10, 180, 150, 19, &G.sipo->tot.xmin, G.sipo->tot.xmin-1000.0, 18000.0, 100, 0, "");
-		uiDefButF(block, NUM, B_MUL_IPO, "Xmax:",		160, 180, 150, 19, &G.sipo->tot.xmax, G.sipo->tot.ymin-1000.0, 18000.0, 100, 0, "");
+		uiBlockBeginAlign(block);
+		uiDefButF(block, NUM, B_MUL_IPO, "Xmin:",		10, 180, 150, 19, &G.sipo->tot.xmin, G.sipo->tot.xmin-1000.0, MAXFRAMEF, 100, 0, "");
+		uiDefButF(block, NUM, B_MUL_IPO, "Xmax:",		160, 180, 150, 19, &G.sipo->tot.xmax, G.sipo->tot.ymin-1000.0, MAXFRAMEF, 100, 0, "");
 		
 		uiDefButF(block, NUM, B_MUL_IPO, "Ymin:",		10, 160, 150, 19, &G.sipo->tot.ymin, G.sipo->tot.ymin-1000.0, 5000.0, 100, 0, "");
 		uiDefButF(block, NUM, B_MUL_IPO, "Ymax:",		160, 160, 150, 19, &G.sipo->tot.ymax, G.sipo->tot.ymin-1000.0, 5000.0, 100, 0, "");
 
 		/* SPEED BUTTON */
 		if(totipo_vertsel) {
-			uiDefButF(block, NUM, B_IPO_NONE, "Speed:",			10,130,150,19, &hspeed, 0.0, 180.0, 1, 0, "");
+			uiBlockBeginAlign(block);
+			uiDefButF(block, NUM, B_IPO_NONE, "Speed:",		10,130,150,19, &hspeed, 0.0, 180.0, 1, 0, "");
 			uiDefBut(block, BUT, B_SETSPEED,"SET",			160,130,50,19, 0, 0, 0, 0, 0, "");
 		}
 	}
 
 	/* this one also does keypositions */
-	if(G.sipo->ipo) ipo_editvertex_buts(block, G.sipo, -10000, 18000);
+	if(G.sipo->ipo) ipo_editvertex_buts(block, G.sipo, -10000, MAXFRAMEF);
 }
 
 static void ipo_blockhandlers(ScrArea *sa)
@@ -1706,7 +2064,9 @@ void drawipospace(ScrArea *sa, void *spacedata)
 	
 	uiFreeBlocksWin(&sa->uiblocks, sa->win);	/* for panel handler to work */
 	
-	v2d->hor.xmax+=IPOBUTX;
+	test_editipo(0);	/* test if current editipo is correct, make_editipo sets v2d->cur, call here because of calc_ipobuttonswidth() */
+	
+	v2d->hor.xmax+=calc_ipobuttonswidth(sa);
 	calc_scrollrcts(sa, G.v2d, sa->winx, sa->winy);
 
 	BIF_GetThemeColor3fv(TH_BACK, col);
@@ -1728,10 +2088,7 @@ void drawipospace(ScrArea *sa, void *spacedata)
 		} 
 	}
 
-	test_editipo();	/* test if current editipo is correct, make_editipo sets v2d->cur */
-
 	myortho2(v2d->cur.xmin, v2d->cur.xmax, v2d->cur.ymin, v2d->cur.ymax);
-	
  
 	if(sipo->editipo) {
 		
@@ -1748,8 +2105,6 @@ void drawipospace(ScrArea *sa, void *spacedata)
 		calc_ipogrid();	
 		draw_ipogrid();
 		
-		calc_ipoverts(sipo);
-		
 		draw_cfra(sipo);
 		
 		/* ipokeys */
@@ -1835,15 +2190,17 @@ void scroll_ipobuts()
 /* total mess function, especially with mousewheel, needs cleanup badly (ton) */
 int view2dzoom(unsigned short event)
 {
+	ScrArea *sa;
 	float fac, dx, dy, wtemp;
 	short mval[2], mvalo[2];
+	short is_wheel= (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE);
 	
-	areawinset(curarea->win);	/* from buttons */
-	curarea->head_swap= 0;
 	getmouseco_areawin(mvalo);
+	mval[0]= mvalo[0];
+	mval[1]= mvalo[1];
 	
-	while( (get_mbut()&(L_MOUSE|M_MOUSE)) || (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) {
-	
+	while( (get_mbut()&(L_MOUSE|M_MOUSE)) || is_wheel ) {
+		
 		/* regular mousewheel:   zoom regular
 		* alt-shift mousewheel: zoom y only
 		* alt-ctrl mousewheel:  zoom x only
@@ -1924,30 +2281,57 @@ int view2dzoom(unsigned short event)
 				}
 			}
 		}
-		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
+
+		if (ELEM(event, WHEELUPMOUSE, WHEELDOWNMOUSE) || mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
 			
 			if(U.viewzoom!=USER_ZOOM_CONT) {
 				mvalo[0]= mval[0];
 				mvalo[1]= mval[1];
 			}
 			
-			G.v2d->cur.xmin+= dx;
-			G.v2d->cur.xmax-= dx;
-			
-			if ELEM5(curarea->spacetype, SPACE_SEQ, SPACE_SOUND, SPACE_ACTION, SPACE_NLA, SPACE_TIME);
+			if( ELEM(curarea->spacetype, SPACE_NLA, SPACE_ACTION) ) {
+				if(mvalo[0] < G.v2d->mask.xmin) {
+					G.v2d->cur.ymin+= dy;
+					G.v2d->cur.ymax-= dy;
+				}
+				else {
+					G.v2d->cur.xmin+= dx;
+					G.v2d->cur.xmax-= dx;
+				}
+			}
+			else if (ELEM(curarea->spacetype, SPACE_SOUND, SPACE_TIME)) {
+				G.v2d->cur.xmin+= dx;
+				G.v2d->cur.xmax-= dx;
+			}
+			else if (curarea->spacetype == SPACE_SEQ) {
+				/* less sensitivity on y scale */
+				G.v2d->cur.xmin+= dx;
+				G.v2d->cur.xmax-= dx;
+				if (!(ELEM(event, WHEELUPMOUSE, WHEELDOWNMOUSE))) {
+					G.v2d->cur.ymin+= dy/2;
+					G.v2d->cur.ymax-= dy/2;
+				}
+			}
 			else {
+				G.v2d->cur.xmin+= dx;
+				G.v2d->cur.xmax-= dx;
 				G.v2d->cur.ymin+= dy;
 				G.v2d->cur.ymax-= dy;
 			}
-		
+			
 			test_view2d(G.v2d, curarea->winx, curarea->winy);	/* cur min max rects */
+			
+			sa= curarea;	/* now when are you going to kill this one! */
+			view2d_do_locks(curarea, V2D_LOCK_COPY|V2D_LOCK_REDRAW);
+			areawinset(sa->win);
+
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 		}
 		else BIF_wait_for_statechange();
 		/* return if we were using the mousewheel
 		*/
-		if ( (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) return 1;
+		if ( is_wheel ) return 1;
 	}
 	return 1;
 }
@@ -1959,17 +2343,14 @@ void center_currframe(void)
 	 */
 	float width;
   
-	areawinset(curarea->win);
-	curarea->head_swap= 0;
-
 	width = G.v2d->cur.xmax - G.v2d->cur.xmin;
 	G.v2d->cur.xmin = CFRA - 0.5*(width);
 	G.v2d->cur.xmax = CFRA + 0.5*(width);
 
 	test_view2d(G.v2d, curarea->winx, curarea->winy);
-	scrarea_do_windraw(curarea);
-	screen_swapbuffers();
-	curarea->head_swap= 0;
+	view2d_do_locks(curarea, V2D_LOCK_COPY);
+
+	scrarea_queue_winredraw(curarea);
 }
 
 /* total mess function, especially with mousewheel, needs cleanup badly (ton) */
@@ -1978,11 +2359,12 @@ int view2dmove(unsigned short event)
 	/* return 1 when something was done */
 	float facx=0.0, facy=0.0, dx, dy, left=1.0, right=1.0;
 	short mval[2], mvalo[2], leftret=1, mousebut;
+	short is_wheel= (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE);
+	int oldcursor, cursor;
+	Window *win;
+	
+	/* when wheel is used, we only draw it once */
 	
-	/* init */
-	scrarea_do_windraw(curarea);
-	curarea->head_swap= 0;
-
 	/* try to do some zooming if the
 	 * middlemouse and ctrl are pressed
 	 * or if the mousewheel is being used.
@@ -1993,75 +2375,101 @@ int view2dmove(unsigned short event)
 	if (U.flag & USER_LMOUSESELECT) mousebut = R_MOUSE;
 		else mousebut = L_MOUSE;
 	
-	if ( (G.qual & LR_CTRLKEY) || (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) {
+	if ( (G.qual & LR_CTRLKEY) || is_wheel ) {
 		/* patch for oops & buttonswin, standard scroll no zoom */
 		if(curarea->spacetype==SPACE_OOPS) {
 			SpaceOops *soops= curarea->spacedata.first;
 			if(soops->type==SO_OUTLINER);
 			else if (view2dzoom(event)) {
-				curarea->head_swap= 0;
 				return 0;
 			}
 		}
 		else if(curarea->spacetype==SPACE_BUTS && (G.qual & LR_CTRLKEY)==0);
 		else if (view2dzoom(event)) {
-			curarea->head_swap= 0;
 			return 0;
 		}
 	}
 	
 	/* test where mouse is */
 	getmouseco_areawin(mvalo);
+	/* initialize this too */
+	mval[0]= mvalo[0];
+	mval[1]= mvalo[1];
 	
-	if ELEM7(curarea->spacetype, SPACE_IPO, SPACE_SEQ, SPACE_OOPS, SPACE_SOUND, SPACE_ACTION, SPACE_NLA, SPACE_TIME) 
-		{
-			if( BLI_in_rcti(&G.v2d->mask, (int)mvalo[0], (int)mvalo[1]) ) {
-				facx= (G.v2d->cur.xmax-G.v2d->cur.xmin)/(float)(G.v2d->mask.xmax-G.v2d->mask.xmin);
-				facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
-			}
-			else if(IN_2D_VERT_SCROLL((int)mvalo)) {
-				facy= -(G.v2d->tot.ymax-G.v2d->tot.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
-				if(get_mbut() & mousebut) {
-					/* which part of scrollbar should move? */
-					if(mvalo[1]< (vertymin+vertymax)/2 ) right= 0.0;
-					else left= 0.0;
-					leftret= 0;
-				}
-			}
-			else if(IN_2D_HORIZ_SCROLL((int)mvalo)) {
-				facx= -(G.v2d->tot.xmax-G.v2d->tot.xmin)/(float)(G.v2d->mask.xmax-G.v2d->mask.xmin);
-				if(get_mbut() & mousebut) {
-					/* which part of scrollbar should move? */
-					if(mvalo[0]< (horxmin+horxmax)/2 ) right= 0.0;
-					else left= 0.0;
-					leftret= 0;
-				}
-			}
+	if ELEM7(curarea->spacetype, SPACE_IPO, SPACE_SEQ, SPACE_OOPS, SPACE_SOUND, SPACE_ACTION, SPACE_NLA, SPACE_TIME) {
+
+		if( BLI_in_rcti(&G.v2d->mask, (int)mvalo[0], (int)mvalo[1]) ) {
+			facx= (G.v2d->cur.xmax-G.v2d->cur.xmin)/(float)(G.v2d->mask.xmax-G.v2d->mask.xmin);
+			facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
 		}
-		else {
-			facx= (G.v2d->cur.xmax-G.v2d->cur.xmin)/(float)(curarea->winx);
-			facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(curarea->winy);		
+		/* stoopid exception to allow scroll in lefthand side */
+		else if(curarea->spacetype==SPACE_ACTION && BLI_in_rcti(&G.v2d->mask, ACTWIDTH+(int)mvalo[0], (int)mvalo[1]) ) {
+			facx= 0.0f;
+			facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
 		}
-		
-		/* no x move in outliner */
-		if(curarea->spacetype==SPACE_OOPS && G.v2d->scroll) facx= 0.0;
-		
-		/* no y move in audio & time */
-		if ELEM(curarea->spacetype, SPACE_SOUND, SPACE_TIME) facy= 0.0;
-		
-		if(get_mbut() & mousebut && leftret) return 0;
-		if(facx==0.0 && facy==0.0) return 1;
-		
-    while( (get_mbut()&(L_MOUSE|M_MOUSE)) || 
-           (event==WHEELUPMOUSE) ||
-           (event==WHEELDOWNMOUSE) ) {
+		else if(curarea->spacetype==SPACE_NLA && BLI_in_rcti(&G.v2d->mask, NLAWIDTH+(int)mvalo[0], (int)mvalo[1]) ) {
+			facx= 0.0f;
+			facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
+		}
+		else if(IN_2D_VERT_SCROLL((int)mvalo)) {
+			facy= -(G.v2d->tot.ymax-G.v2d->tot.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
+			if(get_mbut() & mousebut) {
+				/* which part of scrollbar should move? */
+				if(mvalo[1]< (vertymin+vertymax)/2 ) right= 0.0;
+				else left= 0.0;
+				leftret= 0;
+			}
+			if(is_wheel)
+				facy= -facy;
+		}
+		else if(IN_2D_HORIZ_SCROLL((int)mvalo)) {
+			facx= -(G.v2d->tot.xmax-G.v2d->tot.xmin)/(float)(G.v2d->mask.xmax-G.v2d->mask.xmin);
+			if(get_mbut() & mousebut) {
+				/* which part of scrollbar should move? */
+				if(mvalo[0]< (horxmin+horxmax)/2 ) right= 0.0;
+				else left= 0.0;
+				leftret= 0;
+			}
+		} 
+	}
+	else {
+		facx= (G.v2d->cur.xmax-G.v2d->cur.xmin)/(float)(curarea->winx);
+		facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(curarea->winy);		
+	}
+	
+	cursor = BC_NSEW_SCROLLCURSOR;
+	
+	/* no x move in outliner */
+	if(curarea->spacetype==SPACE_OOPS && G.v2d->scroll) {
+		facx= 0.0;
+		cursor = BC_NS_SCROLLCURSOR;
+	}
+	
+	/* no y move in audio & time */
+	if ELEM(curarea->spacetype, SPACE_SOUND, SPACE_TIME) {
+		facy= 0.0;
+		cursor = BC_EW_SCROLLCURSOR;
+	}
+	
+	/* store the old cursor to temporarily change it */
+	oldcursor=get_cursor();
+	win=winlay_get_active_window();
+
+	
+	if(get_mbut() & mousebut && leftret) return 0;
+	if(facx==0.0 && facy==0.0) return 1;
+	
+	if (!is_wheel) SetBlenderCursor(cursor);
+	
+	while( (get_mbut()&(L_MOUSE|M_MOUSE)) || is_wheel) {
 
       /* If the mousewheel is used with shift key
        * the scroll up and down. If the mousewheel
        * is used with the ctrl key then scroll left
        * and right.
        */
-		if (event==WHEELUPMOUSE || event==WHEELDOWNMOUSE) {
+		if (is_wheel) {
+			
 			if(event==WHEELDOWNMOUSE) {	
 				facx= -facx; facy= -facy;
 			}
@@ -2089,38 +2497,42 @@ int view2dmove(unsigned short event)
 				else return 0;
 				break;
 			}
-			mval[0]= mvalo[0]+1; // force redraw below
-			mval[1]= mvalo[1]+1;
 		}
 		else {
+
+			
 			getmouseco_areawin(mval);
 			dx= facx*(mvalo[0]-mval[0]);
 			dy= facy*(mvalo[1]-mval[1]);
 		}
 
-		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
-
+		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || is_wheel) {
+			ScrArea *sa;
+			
 			G.v2d->cur.xmin+= left*dx;
 			G.v2d->cur.xmax+= right*dx;
 			G.v2d->cur.ymin+= left*dy;
 			G.v2d->cur.ymax+= right*dy;
-				
+			
 			test_view2d(G.v2d, curarea->winx, curarea->winy);
-				
+			
+			sa= curarea;	/* bad global */
+			view2d_do_locks(curarea, V2D_LOCK_COPY|V2D_LOCK_REDRAW);
+			areawinset(sa->win);
+			
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 				
 			mvalo[0]= mval[0];
 			mvalo[1]= mval[1];
-				
 		}
 		else BIF_wait_for_statechange();
 			/* return if we were using the mousewheel
 			*/
-		if ( (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) return 1;
+		if ( is_wheel ) return 1;
 	}
 
-    curarea->head_swap= 0;
+	window_set_cursor(win, oldcursor);
     return 1;
 }
 
