@@ -1,5 +1,5 @@
 /**
- * $Id: drawtext.c,v 1.51 2005/06/04 16:22:50 lukep Exp $
+ * $Id: drawtext.c,v 1.61 2006/07/13 17:13:31 elubie Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -168,12 +168,12 @@ void free_txt_data(void) {
 
 static int render_string (char *in) {
 	SpaceText *st= curarea->spacedata.first;
-	int r= 0, i;
+	int r = 0, i = 0;
 	
 	while(*in) {
 		if (*in=='\t') {
 			if (temp_char_pos && *(in-1)=='\t') i= st->tabnumber;
-			else i= st->tabnumber - (temp_char_pos%st->tabnumber);
+			else if (st->tabnumber > 0) i= st->tabnumber - (temp_char_pos%st->tabnumber);
 
 			while(i--) temp_char_write(' ', r);
 		} else temp_char_write(*in, r);
@@ -202,151 +202,111 @@ void get_format_string(void)
 	
 	while(tmp) {
 		in_line = tmp->line;
-		len = strlen(in_line);
-		spot = 0;
-		tabs = 0;
-		//see how many tabs we have
-		for(a = 0; a <len; a++) {
-			c = (unsigned char) in_line[a];
-			if(c == '\t') {
-				tabs++;
-			}
-		}
-		//calculate the amount of MEM_mallocN we neen
-		mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len; // +2 for good measure
-		if (tmp->format) MEM_freeN(tmp->format);
-		tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
 		
-		for (a = 0; a < len; a++) {
-			c = (unsigned char) in_line[a];
+		len = strlen(in_line);
+		/* weak code... but we dont want crashes (ton) */
+		if(len>2000-1) {
+			if (tmp->format) MEM_freeN(tmp->format);
+			tmp->format= NULL;
+		}
+		else {
+			
+			spot = 0;
+			tabs = 0;
+			//see how many tabs we have
+			for(a = 0; a <len; a++) {
+				c = (unsigned char) in_line[a];
+				if(c == '\t') {
+					tabs++;
+				}
+			}
+			//calculate the amount of MEM_mallocN we neen
+			mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len; // +2 for good measure
+			if (tmp->format) MEM_freeN(tmp->format);
+			tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+			
+			for (a = 0; a < len; a++) {
+				c = (unsigned char) in_line[a];
 
-			check[0] = c;
-			check[1] = '\0';
+				check[0] = c;
+				check[1] = '\0';
 
-			if (check_delim(check))
-			{
-				switch (c) {
-					case '\"':
-						if(in_line[a] == '\"' && in_line[a+1] == '\"' && in_line[a+2] == '\"') { 
-							format[spot] = format[spot+1] = format[spot+2] = 'l';
-							spot +=3;
-							a += 3;
-							while(in_line[a] != '\"' || in_line[a-1] != '\"' || in_line[a-2] != '\"') {
-								c = (unsigned char) in_line[a];
-								if(a >= len) {
-									format[spot] = '\0';
-									memcpy(tmp->format, format, strlen(format));
-									if(!(tmp= tmp->next)) {
-										return;
-									} else {
-										in_line = tmp->line;
-										len = strlen(in_line);
-										tabs = 0;
-										for(b = 0; b <len; b++) {
-											c = (unsigned char) in_line[b];
-											if(c == '\t') {
-												tabs++;
-											}
-										}
-										mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
-										if (tmp->format) MEM_freeN(tmp->format);
-										tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
-										a = 0; spot = 0;
-									}
-								} else {
-									if(c == '\t' || c == ' ') {
-										if(c == '\t') {
-											for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
-												format[spot] = ' ';
-												spot++;
-											}
-											a++;
+				if (check_delim(check))
+				{
+					switch (c) {
+						case '\"':
+							if(in_line[a] == '\"' && in_line[a+1] == '\"' && in_line[a+2] == '\"') { 
+								format[spot] = format[spot+1] = format[spot+2] = 'l';
+								spot +=3;
+								a += 3;
+								while(in_line[a] != '\"' || in_line[a-1] != '\"' || in_line[a-2] != '\"') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
 										} else {
-											format[spot] = ' ';
+											in_line = tmp->line;
+											len = strlen(in_line);
+											tabs = 0;
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
+											}
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
+										}
+									} else {
+										if(c == '\t' || c == ' ') {
+											if(c == '\t') {
+												for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+													format[spot] = ' ';
+													spot++;
+												}
+												a++;
+											} else {
+												format[spot] = ' ';
+												a++; spot++;
+										}
+										} else {
+											format[spot] = 'l';
 											a++; spot++;
-									}
-									} else {
-										format[spot] = 'l';
-										a++; spot++;
+										}
 									}
 								}
-							}
-							format[spot] = 'l';
-							spot++;
-						} else {
-							format[spot] = 'l';
-							a++; spot++;
-							while(in_line[a] != '\"') {
-								c = (unsigned char) in_line[a];
-								if(a >= len) {
-									format[spot] = '\0';
-									memcpy(tmp->format, format, strlen(format));
-									if(!(tmp= tmp->next)) {
-										return;
-									} else {
-										in_line = tmp->line;
-										len = strlen(in_line);
-										for(b = 0; b <len; b++) {
-											c = (unsigned char) in_line[b];
-											if(c == '\t') {
-												tabs++;
+								format[spot] = 'l';
+								spot++;
+							} else {
+								format[spot] = 'l';
+								a++; spot++;
+								while(in_line[a] != '\"') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
+										} else {
+											in_line = tmp->line;
+											len = strlen(in_line);
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
 											}
+											//calculate the amount of MEM_mallocN we neen
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
 										}
-										//calculate the amount of MEM_mallocN we neen
-										mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
-										if (tmp->format) MEM_freeN(tmp->format);
-										tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
-										a = 0; spot = 0;
 									}
-								}
-								if(c == '\t' || c == ' ') {
-									if(c == '\t') {
-										for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
-											format[spot] = ' ';
-											spot++;
-										}
-										a++;
-									} else {
-										format[spot] = ' ';
-										a++; spot++;
-									}
-								} else {
-									format[spot] = 'l';
-									a++; spot++;
-								}
-							}
-							format[spot] = 'l';
-							spot++;
-						}
-						break;
-					case '\'':
-						if(in_line[a] == '\'' && in_line[a+1] == '\'' && in_line[a+2] == '\'') { 
-							format[spot] = format[spot+1] = format[spot+2] = 'l';
-							spot +=3;
-							a += 3;
-							while(in_line[a] != '\'' || in_line[a-1] != '\'' || in_line[a-2] != '\'') {
-								c = (unsigned char) in_line[a];
-								if(a >= len) {
-									format[spot] = '\0';
-									memcpy(tmp->format, format, strlen(format));
-									if(!(tmp= tmp->next)) {
-										return;
-									} else {
-										in_line = tmp->line;
-										len = strlen(in_line);
-										tabs = 0;
-										for(b = 0; b <len; b++) {
-											c = (unsigned char) in_line[b];
-											if(c == '\t') {
-												tabs++;
-											}
-										}
-										mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
-										if (tmp->format) MEM_freeN(tmp->format);
-										tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
-										a = 0; spot = 0;
-									}
-								} else {
 									if(c == '\t' || c == ' ') {
 										if(c == '\t') {
 											for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
@@ -363,132 +323,152 @@ void get_format_string(void)
 										a++; spot++;
 									}
 								}
+								format[spot] = 'l';
+								spot++;
 							}
-							format[spot] = 'l';
-							spot++;
-						} else {
-							format[spot] = 'l';
-							a++; spot++;
-							while(in_line[a] != '\'') {
-								c = (unsigned char) in_line[a];
-								if(a >= len) {
-									format[spot] = '\0';
-									memcpy(tmp->format, format, strlen(format));
-									if(!(tmp= tmp->next)) {
-										return;
-									} else {
-										in_line = tmp->line;
-										len = strlen(in_line);
-										for(b = 0; b <len; b++) {
-											c = (unsigned char) in_line[b];
-											if(c == '\t') {
-												tabs++;
+							break;
+						case '\'':
+							if(in_line[a] == '\'' && in_line[a+1] == '\'' && in_line[a+2] == '\'') { 
+								format[spot] = format[spot+1] = format[spot+2] = 'l';
+								spot +=3;
+								a += 3;
+								while(in_line[a] != '\'' || in_line[a-1] != '\'' || in_line[a-2] != '\'') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
+										} else {
+											in_line = tmp->line;
+											len = strlen(in_line);
+											tabs = 0;
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
 											}
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
+										}
+									} else {
+										if(c == '\t' || c == ' ') {
+											if(c == '\t') {
+												for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+													format[spot] = ' ';
+													spot++;
+												}
+												a++;
+											} else {
+												format[spot] = ' ';
+												a++; spot++;
+											}
+										} else {
+											format[spot] = 'l';
+											a++; spot++;
 										}
-										//calculate the amount of MEM_mallocN we neen
-										mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
-										if (tmp->format) MEM_freeN(tmp->format);
-										tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
-										a = 0; spot = 0;
 									}
 								}
+								format[spot] = 'l';
+								spot++;
+							} else {
+								format[spot] = 'l';
+								a++; spot++;
+								while(in_line[a] != '\'') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
+										} else {
+											in_line = tmp->line;
+											len = strlen(in_line);
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
+											}
+											//calculate the amount of MEM_mallocN we neen
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
+										}
+									}
+									if(c == '\t' || c == ' ') {
+										if(c == '\t') {
+											for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+												format[spot] = ' ';
+												spot++;
+											}
+											a++;
+										} else {
+											format[spot] = ' ';
+											a++; spot++;
+										}
+									} else {
+										format[spot] = 'l';
+										a++; spot++;
+									}
+								}
+								format[spot] = 'l';
+								spot++;
+							}
+							break;
+						case '#':
+							while(a<len) {
+								c = (unsigned char) in_line[a];
 								if(c == '\t' || c == ' ') {
 									if(c == '\t') {
 										for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
-											format[spot] = ' ';
+											format[spot] = '#';
 											spot++;
 										}
 										a++;
 									} else {
-										format[spot] = ' ';
-										a++; spot++;
-									}
-								} else {
-									format[spot] = 'l';
-									a++; spot++;
-								}
-							}
-							format[spot] = 'l';
-							spot++;
-						}
-						break;
-					case '#':
-						while(a<len) {
-							c = (unsigned char) in_line[a];
-							if(c == '\t' || c == ' ') {
-								if(c == '\t') {
-									for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
 										format[spot] = '#';
-										spot++;
+										a++; spot++;
 									}
-									a++;
 								} else {
 									format[spot] = '#';
 									a++; spot++;
 								}
-							} else {
-								format[spot] = '#';
-								a++; spot++;
 							}
-						}
-						break;
-					case ' ':
-						format[spot] = ' ';
-						spot++;
-						break;
-					case '\t':
-						for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+							break;
+						case ' ':
 							format[spot] = ' ';
 							spot++;
+							break;
+						case '\t':
+							for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+								format[spot] = ' ';
+								spot++;
+							}
+							break;
+						default:
+							format[spot] = 'q';
+							spot++;
+							 
+							break;
+					}
+				} else if (check_numbers(check)) {
+					while (a < len) {
+						c = (unsigned char) in_line[a];
+						other[0] = c;
+						other[1] = '\0';
+						if (check_delim(other) && c != '.') {
+							a--; break;
+						} else {
+							format[spot] = 'n';
+							a++; spot++;
 						}
-						break;
-					default:
-						format[spot] = 'q';
-						spot++;
-						 
-						break;
-				}
-			} else if (check_numbers(check)) {
-				while (a < len) {
-					c = (unsigned char) in_line[a];
-					other[0] = c;
-					other[1] = '\0';
-					if (check_delim(other) && c != '.') {
-						a--; break;
-					} else {
-						format[spot] = 'n';
-						a++; spot++;
 					}
-				}
-			} else {
-				letter = 0;
-				while (a < len) {
-					c = (unsigned char) in_line[a];
-					other[0] = c;
-					other[1] = '\0';
-					if (check_delim(other)) {
-						a--; 
-						break;
-					} else {
-						check[letter] = (unsigned char) in_line[a];
-						letter++; 
-						a++;
-					}
-				}
-				check[letter] = '\0';
-				if (check_builtinfuncs(check)) {
-					for (b = 0; b < strlen(check); b++) {
-						format[spot] = 'b'; 
-						spot++;
-					}
-				} else if (check_specialvars(check)) { /*If TRUE then color and color next word*/
-					for (b = 0; b < strlen(check); b++) {
-						format[spot] = 'b';
-						spot++;
-					}
-					a++;
-					format[spot] = 'q';
-					spot++; a++;
+				} else {
 					letter = 0;
 					while (a < len) {
 						c = (unsigned char) in_line[a];
@@ -504,20 +484,50 @@ void get_format_string(void)
 						}
 					}
 					check[letter] = '\0';
-					for (b = 0; b < strlen(check); b++) {
-						format[spot] = 'v';
-						spot++;
-					}
-				}else {
-					for (b = 0; b < strlen(check); b++) {
+					if (check_builtinfuncs(check)) {
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'b'; 
+							spot++;
+						}
+					} else if (check_specialvars(check)) { /*If TRUE then color and color next word*/
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'b';
+							spot++;
+						}
+						a++;
 						format[spot] = 'q';
-						spot++;
+						spot++; a++;
+						letter = 0;
+						while (a < len) {
+							c = (unsigned char) in_line[a];
+							other[0] = c;
+							other[1] = '\0';
+							if (check_delim(other)) {
+								a--; 
+								break;
+							} else {
+								check[letter] = (unsigned char) in_line[a];
+								letter++; 
+								a++;
+							}
+						}
+						check[letter] = '\0';
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'v';
+							spot++;
+						}
+					}else {
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'q';
+							spot++;
+						}
 					}
 				}
 			}
+			format[spot] = '\0';
+			memcpy(tmp->format, format, strlen(format));
 		}
-		format[spot] = '\0';
-		memcpy(tmp->format, format, strlen(format));
+		
 		tmp = tmp->next;
 	}
 }
@@ -535,7 +545,7 @@ static int text_draw(SpaceText *st, char *str, int cshift, int maxwidth, int dra
 	w= w-cshift;
 
 	if (draw) {
-		if(st->showsyntax) {
+		if(st->showsyntax && format) {
 			int amount, a;
 			char out[2];
 			format = format+cshift;
@@ -737,7 +747,7 @@ static void calc_text_rcts(SpaceText *st)
 	CLAMP(st->txtbar.ymax, 2, curarea->winy-2);
 
 	st->pix_per_line= (float) ltexth/curarea->winy;
-	if (st->pix_per_line<.1) st->pix_per_line=.1;
+	if (st->pix_per_line<.1) st->pix_per_line=.1f;
 
 	lbarstart= MIN2(txt_get_span(st->text->lines.first, st->text->curl), 
 				txt_get_span(st->text->lines.first, st->text->sell));
@@ -989,6 +999,8 @@ void drawtextspace(ScrArea *sa, void *spacedata)
 	curarea->win_swap= WIN_BACK_OK;
 }
 
+/* Moves the view to the cursor location,
+  also used to make sure the view isnt outside the file */
 void pop_space_text (SpaceText *st)
 {
 	int i, x;
@@ -1234,7 +1246,7 @@ void txt_copy_selectbuffer (Text *text)
 
 
 #ifdef _WIN32
-char *unixNewLine(char *buffer)
+static char *unixNewLine(char *buffer)
 {
 	char *p, *p2, *output;
 	
@@ -1247,7 +1259,7 @@ char *unixNewLine(char *buffer)
 	return(output);
 }
 
-char *winNewLine(char *buffer)
+static char *winNewLine(char *buffer)
 {
 	char *p, *p2, *output;
 	int add= 0;
@@ -1276,10 +1288,13 @@ void txt_paste_clipboard(Text *text) {
 	if ( OpenClipboard(NULL) ) {
 		HANDLE hData = GetClipboardData( CF_TEXT );
 		buffer = (char*)GlobalLock( hData );
-		buffer = unixNewLine(buffer);
-		txt_insert_buf(text, buffer);
+		if (buffer) {
+			buffer = unixNewLine(buffer);
+			if (buffer) txt_insert_buf(text, buffer);
+		}
 		GlobalUnlock( hData );
 		CloseClipboard();
+		MEM_freeN(buffer);
 	}
 #endif
 }
@@ -1291,17 +1306,19 @@ void txt_copy_clipboard(Text *text) {
 	if (OpenClipboard(NULL)) {
 		HLOCAL clipbuffer;
 		char* buffer;
+		
+		if (copybuffer) {
+			copybuffer = winNewLine(copybuffer);
 
-		copybuffer = winNewLine(copybuffer);
+			EmptyClipboard();
+			clipbuffer = LocalAlloc(LMEM_FIXED,((bufferlength+1)));
+			buffer = (char *) LocalLock(clipbuffer);
 
-		EmptyClipboard();
-		clipbuffer = LocalAlloc(LMEM_FIXED,((bufferlength+1)));
-		buffer = (char *) LocalLock(clipbuffer);
-
-		strncpy(buffer, copybuffer, bufferlength);
-		buffer[bufferlength] =  '\0';
-		LocalUnlock(clipbuffer);
-		SetClipboardData(CF_TEXT,clipbuffer);
+			strncpy(buffer, copybuffer, bufferlength);
+			buffer[bufferlength] =  '\0';
+			LocalUnlock(clipbuffer);
+			SetClipboardData(CF_TEXT,clipbuffer);
+		}
 		CloseClipboard();
 	}
 
@@ -1528,7 +1545,14 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			}
 			break; /* BREAK C */
 		case DKEY:
-			if (G.qual == LR_CTRLKEY) {
+			if (G.qual == (LR_CTRLKEY|LR_SHIFTKEY)) {
+				//uncommenting
+				txt_order_cursors(text);
+				uncomment(text);
+				do_draw = 1;
+				if (st->showsyntax) get_format_string();
+				break;
+			} else if (G.qual == LR_CTRLKEY) {
 				txt_delete_char(text);
 				if (st->showsyntax) get_format_string();
 				do_draw= 1;
@@ -1797,13 +1821,19 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			pop_space_text(st);
 			break;
 		case LEFTARROWKEY:
-			txt_move_left(text, G.qual & LR_SHIFTKEY);
+			if (G.qual & LR_COMMANDKEY)
+				txt_move_bol(text, G.qual & LR_SHIFTKEY);
+			else
+				txt_move_left(text, G.qual & LR_SHIFTKEY);
 			set_tabs(text);
 			do_draw= 1;
 			pop_space_text(st);
 			break;
 		case RIGHTARROWKEY:
-			txt_move_right(text, G.qual & LR_SHIFTKEY);
+			if (G.qual & LR_COMMANDKEY)
+				txt_move_eol(text, G.qual & LR_SHIFTKEY);
+			else
+				txt_move_right(text, G.qual & LR_SHIFTKEY);
 			set_tabs(text);
 			do_draw= 1;
 			pop_space_text(st);
@@ -2031,3 +2061,108 @@ static int check_numbers(char *string)
 	}
 	return 0;
 }
+
+void convert_tabs (struct SpaceText *st, int tab)
+{
+	Text *text = st->text;
+	TextLine *tmp;
+	char *check_line, *new_line, *format;
+	int a, j, extra, number; //unknown for now
+	
+	if (!text) return;
+	
+	tmp = text->lines.first;
+	
+	//first convert to all space, this make it alot easier to convert to tabs because there is no mixtures of ' ' && '\t'
+	while(tmp) {
+		check_line = tmp->line;
+		new_line = MEM_mallocN(render_string(check_line)+1, "Converted_Line");
+		format = MEM_mallocN(render_string(check_line)+1, "Converted_Syntax_format");
+		j = 0;
+		for (a=0; a < strlen(check_line); a++) { //foreach char in line
+			if(check_line[a] == '\t') { //checking for tabs
+				//get the number of spaces this tabs is showing
+				//i dont like doing it this way but will look into it later
+				new_line[j] = '\0';
+				number = render_string(new_line);
+				new_line[j] = '\t';
+				new_line[j+1] = '\0';
+				number = render_string(new_line)-number;
+				for(extra = 0; extra < number; extra++) {
+					new_line[j] = ' ';
+					j++;
+				}
+			} else {
+				new_line[j] = check_line[a];
+				++j;
+			}
+		}
+		new_line[j] = '\0';
+		// put new_line in the tmp->line spot still need to try and set the curc correctly
+		if (tmp->line) MEM_freeN(tmp->line);
+		if(tmp->format) MEM_freeN(tmp->format);
+		
+		tmp->line = new_line;
+		tmp->len = strlen(new_line);
+		tmp->format = format;
+		tmp = tmp->next;
+	}
+	
+	if (tab) // Converting to tabs
+	{	//start over from the begining
+		tmp = text->lines.first;
+		
+		while(tmp) {
+			check_line = tmp->line;
+			extra = 0;
+			for (a = 0; a < strlen(check_line); a++) {
+				number = 0;
+				for (j = 0; j < st->tabnumber; j++) {
+					if ((a+j) <= strlen(check_line)) { //check to make sure we are not pass the end of the line
+						if(check_line[a+j] != ' ') {
+							number = 1;
+						}
+					}
+				}
+				if (!number) { //found all number of space to equal a tab
+					a = a+(st->tabnumber-1);
+					extra = extra+1;
+				}
+			}
+			
+			if ( extra > 0 ) { //got tabs make malloc and do what you have to do
+				new_line = MEM_mallocN(strlen(check_line)-(((st->tabnumber*extra)-extra)-1), "Converted_Line");
+				format = MEM_mallocN(strlen(check_line)-(((st->tabnumber*extra)-extra)-1), "Converted_Syntax_format");
+				extra = 0; //reuse vars
+				for (a = 0; a < strlen(check_line); a++) {
+					number = 0;
+					for (j = 0; j < st->tabnumber; j++) {
+						if ((a+j) <= strlen(check_line)) { //check to make sure we are not pass the end of the line
+							if(check_line[a+j] != ' ') {
+								number = 1;
+							}
+						}
+					}
+					if (!number) { //found all number of space to equal a tab
+						new_line[extra] = '\t';
+						a = a+(st->tabnumber-1);
+						++extra;
+						
+					} else { //not adding a tab
+						new_line[extra] = check_line[a];
+						++extra;
+					}
+				}
+				new_line[extra] = '\0';
+				// put new_line in the tmp->line spot still need to try and set the curc correctly
+				if (tmp->line) MEM_freeN(tmp->line);
+				if(tmp->format) MEM_freeN(tmp->format);
+				
+				tmp->line = new_line;
+				tmp->len = strlen(new_line);
+				tmp->format = format;
+			}
+			tmp = tmp->next;
+		}
+	}
+}
