@@ -1,5 +1,5 @@
 /**
- * $Id: edit.c,v 1.66 2005/05/07 15:50:08 ton Exp $
+ * $Id: edit.c,v 1.90 2006/07/06 17:14:03 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -56,25 +56,29 @@
 #include "DNA_lattice_types.h"
 #include "DNA_meta_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_space_types.h"
 #include "DNA_view3d_types.h"
-
+#include "DNA_userdef_types.h"	/* for U.dupflag */
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 #include "BLI_editVert.h"
+#include "BLI_linklist.h"
 
 #include "BKE_action.h"
 #include "BKE_armature.h"
 #include "BKE_anim.h"
 #include "BKE_curve.h"
+#include "BKE_depsgraph.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_ipo.h"
 #include "BKE_lattice.h"
 #include "BKE_mesh.h"
+#include "BKE_modifier.h"
 #include "BKE_object.h"
 #include "BKE_utildefines.h"
 
@@ -126,12 +130,12 @@ void circle_selectCB(select_CBfunc func);
 /* local protos ---------------*/
 void snap_curs_to_firstsel(void);
 
-
-int get_border(rcti *rect, short col)
+/* flag==2 only border, flag==3 cross+border */
+int get_border(rcti *rect, short flag)
 {
 	float dvec[4], fac1, fac2;
 	int retval=1;
-	unsigned short event;
+	unsigned short event= 0;
 	short mval[2], mvalo[4], val, x1, y1;
 	char str[64];
 
@@ -150,56 +154,59 @@ int get_border(rcti *rect, short col)
 	persp(PERSP_WIN);
 	initgrabz(0.0, 0.0, 0.0);
 	
-	getmouseco_areawin(mvalo);
+	if(flag & 1) {
+		getmouseco_areawin(mvalo);
 
-	/* draws the selection initial cross */
-	sdrawXORline4(0, 0,  mvalo[1],  curarea->winx,  mvalo[1]);
-	sdrawXORline4(1, mvalo[0],  0,  mvalo[0],  curarea->winy); 
-	glFlush();
-	
-	while(TRUE) {
-	
-		/* selection loop while mouse pressed */
-		getmouseco_areawin(mval);
-
-		if(mvalo[0]!=mval[0] || mvalo[1]!=mval[1]) {
-
-			/* aiming cross */
-  			sdrawXORline4(0, 0,  mval[1],  curarea->winx,  mval[1]);
-  			sdrawXORline4(1, mval[0],  0,  mval[0],  curarea->winy);
-			glFlush();
-
-			mvalo[0]= mval[0];
-			mvalo[1]= mval[1];
-		}
-		event= extern_qread(&val);
-
-		if(event && val) {
-
-			/* for when a renderwindow is open, and a mouse cursor activates it */
-			persp(PERSP_VIEW);
-			mywinset(curarea->win);
-			persp(PERSP_WIN);
-			
-			if(event==ESCKEY) {
-				retval= 0;
-				break;
-			}
-			else if(event==BKEY) {
-				/* b has been pressed twice: proceed with circle select */
-				retval= 0;
-				break;
-			}
-			else if(event==LEFTMOUSE) break;
-			else if(event==MIDDLEMOUSE) break;
-			else if(event==RIGHTMOUSE) break;
-		}
-		else PIL_sleep_ms(10);
+		/* draws the selection initial cross */
+		sdrawXORline4(0, 0,  mvalo[1],  curarea->winx,  mvalo[1]);
+		sdrawXORline4(1, mvalo[0],  0,  mvalo[0],  curarea->winy); 
+		glFlush();
 		
-	} /* end while (TRUE) */
+		while(TRUE) {
+		
+			/* selection loop while mouse pressed */
+			getmouseco_areawin(mval);
+			
+			if(mvalo[0]!=mval[0] || mvalo[1]!=mval[1]) {
 
-	/* erase XORed lines */
-	sdrawXORline4(-1, 0, 0, 0, 0);
+				/* aiming cross */
+				sdrawXORline4(0, 0,  mval[1],  curarea->winx,  mval[1]);
+				sdrawXORline4(1, mval[0],  0,  mval[0],  curarea->winy);
+				glFlush();
+
+				mvalo[0]= mval[0];
+				mvalo[1]= mval[1];
+			}
+			event= extern_qread(&val);
+
+			if(event && val) {
+
+				/* for when a renderwindow is open, and a mouse cursor activates it */
+				persp(PERSP_VIEW);
+				mywinset(curarea->win);
+				persp(PERSP_WIN);
+				
+				if(event==ESCKEY) {
+					retval= 0;
+					break;
+				}
+				else if(event==BKEY) {
+					/* b has been pressed twice: proceed with circle select */
+					retval= 0;
+					break;
+				}
+				else if(event==LEFTMOUSE) break;
+				else if(event==MIDDLEMOUSE) break;
+				else if(event==RIGHTMOUSE) break;
+			}
+			else PIL_sleep_ms(10);
+			
+		} /* end while (TRUE) */
+
+		/* erase XORed lines */
+		sdrawXORline4(-1, 0, 0, 0, 0);
+	}
+	else getmouseco_areawin(mval);
 	
 	if(retval) {
 		/* box select */
@@ -343,9 +350,11 @@ int get_border(rcti *rect, short col)
 			if (G.obedit) {
 				if ELEM4(G.obedit->type, OB_MESH, OB_CURVE, OB_SURF, OB_LATTICE) {
 					circle_selectCB(&obedit_selectionCB);
-					return 0;
 				}
 			}
+			else if (G.f&G_FACESELECT) {
+				circle_selectCB(&obedit_selectionCB);
+			}
 			return 0;
 			
 		case SPACE_IMAGE: // brush select in UV editor
@@ -453,11 +462,13 @@ void circle_selectCB(select_CBfunc callback)
 					firsttime= 1;
 					
 					break;
+				case PAGEUPKEY:
 				case WHEELDOWNMOUSE:
 				case PADPLUSKEY:
 				case EQUALKEY:
 					if(val) if(rad<200.0) rad*= 1.2;
 					break;
+				case PAGEDOWNKEY:
 				case WHEELUPMOUSE:
 				case PADMINUS:
 				case MINUSKEY:
@@ -486,27 +497,29 @@ void circle_selectCB(select_CBfunc callback)
 	allqueue(REDRAWINFO, 0);
 }
 
-void count_object(Object *ob, int sel)
+static void count_object(Object *ob, int sel, int totob)
 {
 	Mesh *me;
 	Curve *cu;
-	int tot=0, totf=0;
+	int tot=0, totf=0, subsurf;
 	
 	switch(ob->type) {
 	case OB_MESH:
-		G.totmesh++;
+		G.totmesh+=totob;
 		me= get_mesh(ob);
 		if(me) {
+			ModifierData *md = modifiers_findByType(ob, eModifierType_Subsurf);
 			int totvert, totface;
 			
-			/* note; do not make disp or get derived mesh here. spoils dependency order */
-			if (me->flag & ME_SUBSURF) {
-				totvert= me->totvert<<me->subdiv;
-				totface= me->totface<<me->subdiv;
-			} else {
-				totvert= me->totvert;
-				totface= me->totface;
+			if (md) {
+				SubsurfModifierData *smd = (SubsurfModifierData*) md;
+				
+				subsurf= 1<<(2*smd->levels);
 			}
+			else subsurf= 1;
+			
+			totvert= subsurf*me->totvert*totob;
+			totface= subsurf*me->totface*totob;
 			
 			G.totvert+= totvert;
 			G.totface+= totface;
@@ -518,16 +531,18 @@ void count_object(Object *ob, int sel)
 		break;
 
 	case OB_LAMP:
-		G.totlamp++;
+		G.totlamp+=totob;
 		break;
 	case OB_SURF:
 	case OB_CURVE:
 	case OB_FONT:
-		G.totcurve++;
+		G.totcurve+=totob;
 		tot=totf= 0;
 		cu= ob->data;
-		if(cu->disp.first==0) makeDispList(ob);
-		count_displist( &cu->disp, &tot, &totf);
+		if(cu->disp.first)
+			count_displist( &cu->disp, &tot, &totf);
+		tot*= totob;
+		totf*= totob;
 		G.totvert+= tot;
 		G.totface+= totf;
 		if(sel) {
@@ -537,13 +552,14 @@ void count_object(Object *ob, int sel)
 		break;
 	case OB_MBALL:
 		count_displist( &ob->disp, &tot, &totf);
+		tot*= totob;
+		totf*= totob;
 		G.totvert+= tot;
 		G.totface+= totf;
 		if(sel) {
 			G.totvertsel+= tot;
 			G.totfacesel+= totf;
 		}
-		
 		break;
 	}
 	
@@ -555,7 +571,7 @@ void countall()
 {
 	extern ListBase editNurb;
 	Base *base;
-	Object *ob;
+	Object *ob= OBACT;
 	Mesh *me;
 	Nurb *nu;
 	BezTriple *bezt;
@@ -565,7 +581,7 @@ void countall()
 	int a;
 
 	G.totvert= G.totvertsel= G.totedge= G.totedgesel= G.totfacesel= G.totface= G.totobj= 
-	    G.totmesh= G.totlamp= G.totcurve= G.totobjsel=  0;
+	    G.totmesh= G.totlamp= G.totcurve= G.totobjsel= G.totbone= G.totbonesel=  0;
 
 	if(G.obedit) {
 		
@@ -587,12 +603,15 @@ void countall()
 				G.totface++;
 				if(efa->f & SELECT) G.totfacesel++;
 			}
+			
+			/*add code to strip editselections*/
 		}
 		else if (G.obedit->type==OB_ARMATURE){
 			for (ebo=G.edbo.first;ebo;ebo=ebo->next){
+				G.totbone++;
 				
-				/* Sync selection to parent for ik children */
-				if ((ebo->flag & BONE_IK_TOPARENT) && ebo->parent){
+				/* Sync selection to parent for connected children */
+				if ((ebo->flag & BONE_CONNECTED) && ebo->parent){
 					G.totvert--;
 					if (ebo->parent->flag & BONE_TIPSEL)
 						ebo->flag |= BONE_ROOTSEL;
@@ -610,15 +629,14 @@ void countall()
 				else
 					ebo->flag &= ~BONE_SELECTED;
 				
-				//	If this is an IK child and it's parent is being moved, remove our root
-				if ((ebo->flag & BONE_IK_TOPARENT)&& (ebo->flag & BONE_ROOTSEL) && ebo->parent && (ebo->parent->flag & BONE_TIPSEL)){
+				if(ebo->flag & BONE_SELECTED) G.totbonesel++;
+
+				//	If this is a connected child and it's parent is being moved, remove our root
+				if ((ebo->flag & BONE_CONNECTED)&& (ebo->flag & BONE_ROOTSEL) && ebo->parent && (ebo->parent->flag & BONE_TIPSEL)){
 					G.totvertsel--;
 				}
 
 				G.totvert+=2;
-				G.totface++;
-				
-
 			}
 		}
 		else if ELEM3(G.obedit->type, OB_CURVE, OB_SURF, OB_FONT) {
@@ -672,6 +690,20 @@ void countall()
 		allqueue(REDRAWINFO, 1);	/* 1, because header->win==0! */
 		return;
 	}
+	else if(ob && (ob->flag & OB_POSEMODE)) {
+		if(ob->pose) {
+			bArmature *arm= ob->data;
+			bPoseChannel *pchan;
+			for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+				G.totbone++;
+				if(pchan->bone && (pchan->bone->flag & BONE_SELECTED))
+					if(pchan->bone->layer & arm->layer)
+						G.totbonesel++;
+			}
+		}
+		allqueue(REDRAWINFO, 1);	/* 1, because header->win==0! */
+		return;
+	}
 	else if(G.f & (G_FACESELECT + G_VERTEXPAINT + G_TEXTUREPAINT +G_WEIGHTPAINT)) {
 		me= get_mesh((G.scene->basact) ? (G.scene->basact->object) : 0);
 		if(me) {
@@ -687,23 +719,23 @@ void countall()
 	base= (G.scene->base.first);
 	while(base) {
 		if(G.scene->lay & base->lay) {
+			ob= base->object;	/* warning, ob not is obact anymore */
 			
-			G.totobj++;
 			if(base->flag & SELECT) G.totobjsel++;
 			
-			count_object(base->object, base->flag & SELECT);
-			
-			if(base->object->transflag & OB_DUPLI) {
-				extern ListBase duplilist;
-
-				make_duplilist(G.scene, base->object);
-				ob= duplilist.first;
-				while(ob) {
-					G.totobj++;
-					count_object(ob, base->flag & SELECT);
-					ob= ob->id.next;
-				}
-				free_duplilist();
+			if(ob->parent && (ob->parent->transflag & OB_DUPLIVERTS)) {
+				int tot= count_duplilist(ob->parent);
+				G.totobj+=tot;
+				count_object(ob, base->flag & SELECT, tot);
+			}
+			else if(ob->transflag & OB_DUPLIFRAMES) {
+				int tot= count_duplilist(ob);
+				G.totobj+=tot;
+				count_object(ob, base->flag & SELECT, tot);
+			}
+			else {
+				count_object(ob, base->flag & SELECT, 1);
+				G.totobj++;
 			}
 		}
 		base= base->next;
@@ -718,99 +750,34 @@ void countall()
 static TransVert *transvmain=NULL;
 static int tottrans= 0;
 
-/* selected things moved, and might need update in displists */
-static void update_select_dependency(void)
+/* copied from editobject.c, now uses (almost) proper depgraph */
+static void special_transvert_update(void)
 {
-	Base *base;
-	Object *ob;
-	for(base= (G.scene->base.first); base; base= base->next) {
-		ob= base->object;
-		if(ob->hooks.first) {
-			ObHook *hook= ob->hooks.first;
-			while(hook) {
-				if(hook->parent && (hook->parent->flag & SELECT)) freedisplist(&ob->disp);
-				hook= hook->next;
-			}
-		}
-	}
-	
-}
-
-static void calc_trans_verts(void)
-{
-	extern ListBase editNurb;
-	
-	if (ELEM(G.obedit->type, OB_MESH, OB_MBALL))
-		makeDispList(G.obedit);
-	else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
-		Nurb *nu= editNurb.first;
-		while(nu) {
-			test2DNurb(nu);
-			testhandlesNurb(nu); /* test for bezier too */
-			nu= nu->next;
-		}
-		makeDispList(G.obedit);
-	}
-}
-
-static int pose_flags_reset_done(Object *ob) {
-	/* Clear the constraint done status for every pose channe;
-	* that has been flagged as needing constant updating
-	*/
-	bPoseChannel *chan;
-	int numreset = 0;
-	
-	if (ob->pose) {
-		for (chan = ob->pose->chanbase.first; chan; chan=chan->next){
-			if (chan->flag & PCHAN_TRANS_UPDATE) {
-				chan->flag &= ~PCHAN_DONE;
-				numreset++;
-			}
-			
-		}
-	}
-	return numreset;
-}
-
-
-/* copied from editobject.c, should be replaced with proper depgraph */
-static void special_trans_update(void)
-{
-	Base *base;
-	Curve *cu;
-	IpoCurve *icu;
 	
 	if(G.obedit) {
+		
+		DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
+		
 		if(G.obedit->type==OB_MESH) {
 			recalc_editnormals();	// does face centers too
 		}
-		if(G.obedit->type==OB_CURVE) {
-			cu= G.obedit->data;
-			
-			makeBevelList(G.obedit); // might be needed for deform
-			calc_curvepath(G.obedit);
-			
-			base= FIRSTBASE;
-			while(base) {
-				if(base->lay & G.vd->lay) {
-					if(base->object->parent==G.obedit && base->object->partype==PARSKEL)
-						makeDispList(base->object);
-					else if(base->object->type==OB_CURVE) {
-						Curve *cu= base->object->data;
-						if(G.obedit==cu->bevobj || G.obedit==cu->taperobj)
-							makeDispList(base->object);
-					}
-				}
-				base= base->next;
+		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
+			extern ListBase editNurb;
+			Nurb *nu= editNurb.first;
+			while(nu) {
+				test2DNurb(nu);
+				testhandlesNurb(nu); /* test for bezier too */
+				nu= nu->next;
 			}
 		}
 		else if(G.obedit->type==OB_ARMATURE){
+			bArmature *arm= G.obedit->data;
 			EditBone *ebo;
 			
 			/* Ensure all bones are correctly adjusted */
 			for (ebo=G.edbo.first; ebo; ebo=ebo->next){
 				
-				if ((ebo->flag & BONE_IK_TOPARENT) && ebo->parent){
+				if ((ebo->flag & BONE_CONNECTED) && ebo->parent){
 					/* If this bone has a parent tip that has been moved */
 					if (ebo->parent->flag & BONE_TIPSEL){
 						VECCOPY (ebo->head, ebo->parent->tail);
@@ -821,77 +788,13 @@ static void special_trans_update(void)
 					}
 				}
 			}
+			if(arm->flag & ARM_MIRROR_EDIT) 
+				transform_armature_mirror_update();
 		}
 		else if(G.obedit->type==OB_LATTICE) {
-			
 			if(editLatt->flag & LT_OUTSIDE) outside_lattice(editLatt);
-			
-			base= FIRSTBASE;
-			while(base) {
-				if(base->lay & G.vd->lay) {
-					if(base->object->parent==G.obedit) {
-						makeDispList(base->object);
-					}
-				}
-				base= base->next;
-			}
 		}
 	}
-	else {
-		base= FIRSTBASE;
-		while(base) {
-			if(base->flag & BA_DO_IPO) {
-				
-				base->object->ctime= -1234567.0;
-				
-				icu= base->object->ipo->curve.first;
-				while(icu) {
-					calchandles_ipocurve(icu);
-					icu= icu->next;
-				}
-				
-			}
-			if(base->object->partype & PARSLOW) {
-				base->object->partype -= PARSLOW;
-				where_is_object(base->object);
-				base->object->partype |= PARSLOW;
-			}
-			else if(base->flag & BA_WHERE_UPDATE) {
-				where_is_object(base->object);
-			}
-			
-			base= base->next;
-		} 
-		
-		base= FIRSTBASE;
-		while(base) {
-			
-			if(base->flag & BA_DISP_UPDATE) makeDispList(base->object);
-			
-			base= base->next;
-		}
-		
-	}
-	
-	base= FIRSTBASE;
-	while(base) {
-		if (pose_flags_reset_done(base->object)) {
-			if (!is_delay_deform())
-				make_displists_by_armature(base->object);
-		}
-		
-		base= base->next;
-	}
-	
-#if 1
-	if (G.obpose && G.obpose->type == OB_ARMATURE)
-		clear_pose_constraint_status(G.obpose);
-	
-	if (!is_delay_deform()) make_displists_by_armature(G.obpose);
-#endif
-	
-	if(G.vd->drawtype == OB_SHADED) reshadeall_displist();
-	
 }
 
 /* copied from editobject.c, needs to be replaced with new transform code still */
@@ -994,28 +897,31 @@ static void make_trans_verts(float *min, float *max, int mode)
 		}
 	}
 	else if (G.obedit->type==OB_ARMATURE){
+		bArmature *arm= G.obedit->data;
+		
 		for (ebo=G.edbo.first;ebo;ebo=ebo->next){
-			if (ebo->flag & BONE_TIPSEL){
-				VECCOPY (tv->oldloc, ebo->tail);
-				tv->loc= ebo->tail;
-				tv->nor= NULL;
-				tv->flag= 1;
-				tv++;
-				tottrans++;
-			}
-
-			/*  Only add the root if there is no selected IK parent */
-			if (ebo->flag & BONE_ROOTSEL){
-				if (!(ebo->parent && (ebo->flag & BONE_IK_TOPARENT) && ebo->parent->flag & BONE_TIPSEL)){
-					VECCOPY (tv->oldloc, ebo->head);
-					tv->loc= ebo->head;
+			if(ebo->layer & arm->layer) {
+				if (ebo->flag & BONE_TIPSEL){
+					VECCOPY (tv->oldloc, ebo->tail);
+					tv->loc= ebo->tail;
 					tv->nor= NULL;
 					tv->flag= 1;
 					tv++;
 					tottrans++;
-				}		
-			}
-			
+				}
+
+				/*  Only add the root if there is no connection */
+				if (ebo->flag & BONE_ROOTSEL){
+					if (!(ebo->parent && (ebo->flag & BONE_CONNECTED) && ebo->parent->flag & BONE_TIPSEL)){
+						VECCOPY (tv->oldloc, ebo->head);
+						tv->loc= ebo->head;
+						tv->nor= NULL;
+						tv->flag= 1;
+						tv++;
+						tottrans++;
+					}		
+				}
+			}			
 		}
 	}
 	else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
@@ -1133,8 +1039,6 @@ static void make_trans_verts(float *min, float *max, int mode)
 	
 }
 
-
-
 void snap_sel_to_grid()
 {
 	extern float originmat[3][3];	/* object.c */
@@ -1147,56 +1051,51 @@ void snap_sel_to_grid()
 	gridf= G.vd->gridview;
 
 
-		if(G.obedit) {
-			tottrans= 0;
-			
-#ifdef __NLA
-			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans= 0;
+		
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
 
-				VECCOPY(vec, tv->loc);
-				Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
-				vec[0]= G.vd->gridview*floor(.5+ vec[0]/gridf);
-				vec[1]= G.vd->gridview*floor(.5+ vec[1]/gridf);
-				vec[2]= G.vd->gridview*floor(.5+ vec[2]/gridf);
-				VecSubf(vec, vec, G.obedit->obmat[3]);
+			VECCOPY(vec, tv->loc);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			vec[0]= G.vd->gridview*floor(.5+ vec[0]/gridf);
+			vec[1]= G.vd->gridview*floor(.5+ vec[1]/gridf);
+			vec[2]= G.vd->gridview*floor(.5+ vec[2]/gridf);
+			VecSubf(vec, vec, G.obedit->obmat[3]);
 
-				Mat3MulVecfl(imat, vec);
-				VECCOPY(tv->loc, vec);
+			Mat3MulVecfl(imat, vec);
+			VECCOPY(tv->loc, vec);
 
+		}
+
+		MEM_freeN(transvmain);
+		transvmain= 0;
+		
+		special_transvert_update();
+
+		allqueue(REDRAWVIEW3D, 0);
+		return;
+	}
+
+	base= (G.scene->base.first);
+	while(base) {
+		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+			ob= base->object;
+			if(ob->flag & OB_POSEMODE) {
+				;	// todo
 			}
-
-			MEM_freeN(transvmain);
-			transvmain= 0;
-			
-			calc_trans_verts(); // does test2d, makedisplist too */
-
-			special_trans_update();
-
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#ifdef __NLA
-		if (G.obpose){
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#endif
-		base= (G.scene->base.first);
-		while(base) {
-			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
-				ob= base->object;
-
+			else {
+				ob->recalc |= OB_RECALC_OB;
+				
 				vec[0]= -ob->obmat[3][0]+G.vd->gridview*floor(.5+ ob->obmat[3][0]/gridf);
 				vec[1]= -ob->obmat[3][1]+G.vd->gridview*floor(.5+ ob->obmat[3][1]/gridf);
 				vec[2]= -ob->obmat[3][2]+G.vd->gridview*floor(.5+ ob->obmat[3][2]/gridf);
@@ -1216,11 +1115,12 @@ void snap_sel_to_grid()
 					ob->loc[2]+= vec[2];
 				}
 			}
-
-			base= base->next;
 		}
-		update_select_dependency();
-		allqueue(REDRAWVIEW3D, 0);
+
+		base= base->next;
+	}
+	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_sel_to_curs()
@@ -1234,57 +1134,70 @@ void snap_sel_to_curs()
 
 	curs= give_cursor();
 
-		if(G.obedit) {
-			tottrans= 0;
-#ifdef __NLA
-			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans= 0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
 
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
 
-				vec[0]= curs[0]-G.obedit->obmat[3][0];
-				vec[1]= curs[1]-G.obedit->obmat[3][1];
-				vec[2]= curs[2]-G.obedit->obmat[3][2];
+			vec[0]= curs[0]-G.obedit->obmat[3][0];
+			vec[1]= curs[1]-G.obedit->obmat[3][1];
+			vec[2]= curs[2]-G.obedit->obmat[3][2];
 
+			Mat3MulVecfl(imat, vec);
+			VECCOPY(tv->loc, vec);
 
-				Mat3MulVecfl(imat, vec);
-				VECCOPY(tv->loc, vec);
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
 
+		special_transvert_update();
+
+		allqueue(REDRAWVIEW3D, 0);
+		return;
+	}
+
+	base= (G.scene->base.first);
+	while(base) {
+		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+			ob= base->object;
+			if(ob->flag & OB_POSEMODE) {
+				bPoseChannel *pchan;
+				bArmature *arm= ob->data;
+				float cursp[3];
+				
+				Mat4Invert(ob->imat, ob->obmat);
+				VECCOPY(cursp, curs);
+				Mat4MulVecfl(ob->imat, cursp);
+				
+				for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
+					if(pchan->bone->flag & BONE_SELECTED) {
+						if(pchan->bone->layer & arm->layer) {
+							if(pchan->parent==NULL) {
+								/* this is wrong... lazy! */
+								VECCOPY(pchan->loc, cursp);
+							}
+							/* else todo... */
+						}
+					}
+				}
+				ob->pose->flag |= (POSE_LOCKED|POSE_DO_UNLOCK);
+				ob->recalc |= OB_RECALC_DATA;
 			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
-
-			calc_trans_verts(); // does test2d, makedisplist too */
-
-			special_trans_update();
-
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#ifdef __NLA
-		if (G.obpose){
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#endif
-		base= (G.scene->base.first);
-		while(base) {
-			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
-				ob= base->object;
-
+			else {
+				ob->recalc |= OB_RECALC_OB;
+				
 				vec[0]= -ob->obmat[3][0] + curs[0];
 				vec[1]= -ob->obmat[3][1] + curs[1];
 				vec[2]= -ob->obmat[3][2] + curs[2];
 
-
 				if(ob->parent) {
 					where_is_object(ob);
 
@@ -1300,11 +1213,12 @@ void snap_sel_to_curs()
 					ob->loc[2]+= vec[2];
 				}
 			}
-
-			base= base->next;
 		}
-		update_select_dependency();
-		allqueue(REDRAWVIEW3D, 0);
+
+		base= base->next;
+	}
+	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_curs_to_grid()
@@ -1335,28 +1249,67 @@ void snap_curs_to_sel()
 	INIT_MINMAX(min, max);
 	centroid[0]= centroid[1]= centroid[2]= 0.0;
 
-		if(G.obedit) {
-			tottrans=0;
-#ifdef __NLA
-			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans=0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
-				VECCOPY(vec, tv->loc);
-				Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
-				VecAddf(centroid, centroid, vec);
-				DO_MINMAX(vec, min, max);
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
+			VECCOPY(vec, tv->loc);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			VecAddf(centroid, centroid, vec);
+			DO_MINMAX(vec, min, max);
+		}
+
+		if(G.vd->around==V3D_CENTROID) {
+			VecMulf(centroid, 1.0/(float)tottrans);
+			VECCOPY(curs, centroid);
+		}
+		else {
+			curs[0]= (min[0]+max[0])/2;
+			curs[1]= (min[1]+max[1])/2;
+			curs[2]= (min[2]+max[2])/2;
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
+	}
+	else {
+		Object *ob= OBACT;
+		
+		if(ob && (ob->flag & OB_POSEMODE)) {
+			bArmature *arm= ob->data;
+			bPoseChannel *pchan;
+			for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
+				if(arm->layer & pchan->bone->layer) {
+					if(pchan->bone->flag & BONE_SELECTED) {
+						VECCOPY(vec, pchan->pose_head);
+						Mat4MulVecfl(ob->obmat, vec);
+						VecAddf(centroid, centroid, vec);
+						DO_MINMAX(vec, min, max);
+						count++;
+					}
+				}
 			}
-
+		}
+		else {
+			for(base= G.scene->base.first; base; base= base->next) {
+				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+					VECCOPY(vec, base->object->obmat[3]);
+					VecAddf(centroid, centroid, vec);
+					DO_MINMAX(vec, min, max);
+					count++;
+				}
+			}
+		}
+		if(count) {
 			if(G.vd->around==V3D_CENTROID) {
-				VecMulf(centroid, 1.0/(float)tottrans);
+				VecMulf(centroid, 1.0/(float)count);
 				VECCOPY(curs, centroid);
 			}
 			else {
@@ -1364,33 +1317,9 @@ void snap_curs_to_sel()
 				curs[1]= (min[1]+max[1])/2;
 				curs[2]= (min[2]+max[2])/2;
 			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
 		}
-		else {
-			base= (G.scene->base.first);
-			while(base) {
-				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
-					VECCOPY(vec, base->object->obmat[3]);
-					VecAddf(centroid, centroid, vec);
-					DO_MINMAX(vec, min, max);
-					count++;
-				}
-				base= base->next;
-			}
-			if(count) {
-				if(G.vd->around==V3D_CENTROID) {
-					VecMulf(centroid, 1.0/(float)count);
-					VECCOPY(curs, centroid);
-				}
-				else {
-					curs[0]= (min[0]+max[0])/2;
-					curs[1]= (min[1]+max[1])/2;
-					curs[2]= (min[2]+max[2])/2;
-				}
-			}
-		}
-		allqueue(REDRAWVIEW3D, 0);
+	}
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_curs_to_firstsel()
@@ -1406,60 +1335,58 @@ void snap_curs_to_firstsel()
 	INIT_MINMAX(min, max);
 	centroid[0]= centroid[1]= centroid[2]= 0.0;
 
-		if(G.obedit) {
-			tottrans=0;
-#ifdef __NLA
-			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans=0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			VECCOPY(vec, tv->loc);
-				/*Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
-				VecAddf(centroid, centroid, vec);
-				DO_MINMAX(vec, min, max);*/
+		Mat3CpyMat4(bmat, G.obedit->obmat);
 
-			if(G.vd->around==V3D_CENTROID) {
-				VecMulf(vec, 1.0/(float)tottrans);
-				VECCOPY(curs, vec);
-			}
-			else {
-				curs[0]= vec[0];
-				curs[1]= vec[1];
-				curs[2]= vec[2];
-			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
+		tv= transvmain;
+		VECCOPY(vec, tv->loc);
+			/*Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			VecAddf(centroid, centroid, vec);
+			DO_MINMAX(vec, min, max);*/
+
+		if(G.vd->around==V3D_CENTROID) {
+			VecMulf(vec, 1.0/(float)tottrans);
+			VECCOPY(curs, vec);
 		}
 		else {
-			base= (G.scene->base.first);
-			while(base) {
-				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
-					VECCOPY(vec, base->object->obmat[3]);
-					VecAddf(centroid, centroid, vec);
-					DO_MINMAX(vec, min, max);
-					count++;
-				}
-				base= base->next;
+			curs[0]= vec[0];
+			curs[1]= vec[1];
+			curs[2]= vec[2];
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
+	}
+	else {
+		base= (G.scene->base.first);
+		while(base) {
+			if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+				VECCOPY(vec, base->object->obmat[3]);
+				VecAddf(centroid, centroid, vec);
+				DO_MINMAX(vec, min, max);
+				count++;
 			}
-			if(count) {
-				if(G.vd->around==V3D_CENTROID) {
-					VecMulf(centroid, 1.0/(float)count);
-					VECCOPY(curs, centroid);
-				}
-				else {
-					curs[0]= (min[0]+max[0])/2;
-					curs[1]= (min[1]+max[1])/2;
-					curs[2]= (min[2]+max[2])/2;
-				}
+			base= base->next;
+		}
+		if(count) {
+			if(G.vd->around==V3D_CENTROID) {
+				VecMulf(centroid, 1.0/(float)count);
+				VECCOPY(curs, centroid);
+			}
+			else {
+				curs[0]= (min[0]+max[0])/2;
+				curs[1]= (min[1]+max[1])/2;
+				curs[2]= (min[2]+max[2])/2;
 			}
 		}
-		allqueue(REDRAWVIEW3D, 0);
+	}
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_to_center()
@@ -1471,35 +1398,59 @@ void snap_to_center()
 	float snaploc[3], imat[3][3], bmat[3][3], vec[3], min[3], max[3], centroid[3];
 	int count, a;
 
-
-/*calculate the snaplocation (centerpoint) */
+	/*calculate the snaplocation (centerpoint) */
 	count= 0;
 	INIT_MINMAX(min, max);
-	centroid[0]= centroid[1]= centroid[2]= 0.0;
+	centroid[0]= centroid[1]= centroid[2]= 0.0f;
+	snaploc[0]= snaploc[1]= snaploc[2]= 0.0f;
 
-		if(G.obedit) {
-			tottrans= 0;
-#ifdef __NLA
-			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans= 0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
-				VECCOPY(vec, tv->loc);
-				Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
+
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
+			VECCOPY(vec, tv->loc);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			VecAddf(centroid, centroid, vec);
+			DO_MINMAX(vec, min, max);
+		}
+
+		if(G.vd->around==V3D_CENTROID) {
+			VecMulf(centroid, 1.0/(float)tottrans);
+			VECCOPY(snaploc, centroid);
+		}
+		else {
+			snaploc[0]= (min[0]+max[0])/2;
+			snaploc[1]= (min[1]+max[1])/2;
+			snaploc[2]= (min[2]+max[2])/2;
+		}
+		
+		MEM_freeN(transvmain);
+		transvmain= 0;
+
+	}
+	else {
+		base= (G.scene->base.first);
+		while(base) {
+			if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+				VECCOPY(vec, base->object->obmat[3]);
 				VecAddf(centroid, centroid, vec);
 				DO_MINMAX(vec, min, max);
+				count++;
 			}
-
+			base= base->next;
+		}
+		if(count) {
 			if(G.vd->around==V3D_CENTROID) {
-				VecMulf(centroid, 1.0/(float)tottrans);
+				VecMulf(centroid, 1.0/(float)count);
 				VECCOPY(snaploc, centroid);
 			}
 			else {
@@ -1507,89 +1458,53 @@ void snap_to_center()
 				snaploc[1]= (min[1]+max[1])/2;
 				snaploc[2]= (min[2]+max[2])/2;
 			}
-			
-			MEM_freeN(transvmain);
-			transvmain= 0;
-
 		}
-		else {
-			base= (G.scene->base.first);
-			while(base) {
-				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
-					VECCOPY(vec, base->object->obmat[3]);
-					VecAddf(centroid, centroid, vec);
-					DO_MINMAX(vec, min, max);
-					count++;
-				}
-				base= base->next;
+	}
+
+	/* Snap the selection to the snaplocation (duh!) */
+	if(G.obedit) {
+		tottrans= 0;
+
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
+
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
+
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
+
+			vec[0]= snaploc[0]-G.obedit->obmat[3][0];
+			vec[1]= snaploc[1]-G.obedit->obmat[3][1];
+			vec[2]= snaploc[2]-G.obedit->obmat[3][2];
+
+			Mat3MulVecfl(imat, vec);
+			VECCOPY(tv->loc, vec);
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
+
+		special_transvert_update();
+
+		allqueue(REDRAWVIEW3D, 0);
+		return;
+	}
+
+	base= (G.scene->base.first);
+	while(base) {
+		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+			ob= base->object;
+			if(ob->flag & OB_POSEMODE) {
+				; // todo
 			}
-			if(count) {
-				if(G.vd->around==V3D_CENTROID) {
-					VecMulf(centroid, 1.0/(float)count);
-					VECCOPY(snaploc, centroid);
-				}
-				else {
-					snaploc[0]= (min[0]+max[0])/2;
-					snaploc[1]= (min[1]+max[1])/2;
-					snaploc[2]= (min[2]+max[2])/2;
-				}
-			}
-		}
-
-
-/* Snap the selection to the snaplocation (duh!) */
-
-		if(G.obedit) {
-			tottrans= 0;
-#ifdef __NLA
-			if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM5(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
-
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
-
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
-
-
-				vec[0]= snaploc[0]-G.obedit->obmat[3][0];
-				vec[1]= snaploc[1]-G.obedit->obmat[3][1];
-				vec[2]= snaploc[2]-G.obedit->obmat[3][2];
-
-
-				Mat3MulVecfl(imat, vec);
-				VECCOPY(tv->loc, vec);
-
-			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
-
-			if ELEM4(G.obedit->type, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) makeDispList(G.obedit);
-
-			special_trans_update();
-
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#ifdef __NLA
-		if (G.obpose){
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#endif
-		base= (G.scene->base.first);
-		while(base) {
-			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
-				ob= base->object;
-
+			else {
+				ob->recalc |= OB_RECALC_OB;
+				
 				vec[0]= -ob->obmat[3][0] + snaploc[0];
 				vec[1]= -ob->obmat[3][1] + snaploc[1];
 				vec[2]= -ob->obmat[3][2] + snaploc[2];
 
-
 				if(ob->parent) {
 					where_is_object(ob);
 
@@ -1605,11 +1520,12 @@ void snap_to_center()
 					ob->loc[2]+= vec[2];
 				}
 			}
-
-			base= base->next;
 		}
-		update_select_dependency();
-		allqueue(REDRAWVIEW3D, 0);
+
+		base= base->next;
+	}
+	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 
@@ -1642,28 +1558,61 @@ void snapmenu()
 }
 
 
+#define MERGELIMIT 0.001
 void mergemenu(void)
-{
-	extern float doublimit;
+{	
+
 	short event;
-
-	event = pupmenu("Merge %t|At Center%x1|At Cursor%x2");
-
-	if (event==-1) return; /* Return if the menu is closed without any choices */
-
-	if (event==1) 
-		snap_to_center(); /*Merge at Center*/
-	else
-		snap_sel_to_curs(); /*Merge at Cursor*/
-
-	notice("Removed %d Vertices", removedoublesflag(1, doublimit));
+	int remCount = 0;
+	
+	if(G.scene->selectmode & SCE_SELECT_VERTEX)
+		if(G.editMesh->selected.first && G.editMesh->selected.last && 
+			((EditSelection*)G.editMesh->selected.first)->type == EDITVERT && ((EditSelection*)G.editMesh->selected.last)->type == EDITVERT) 
+				event = pupmenu("Merge %t|At First %x6|At Last%x1|At Center%x3|At Cursor%x4|Collapse%x2");
+		else if (G.editMesh->selected.first && ((EditSelection*)G.editMesh->selected.first)->type == EDITVERT) 
+			event = pupmenu("Merge %t|At First %x6|At Center%x3|At Cursor%x4|Collapse%x2");
+		else if (G.editMesh->selected.last && ((EditSelection*)G.editMesh->selected.last)->type == EDITVERT) 
+			event = pupmenu("Merge %t|At Last %x1|At Center%x3|At Cursor%x4|Collapse%x2");
+		else event = pupmenu("Merge %t|At Center%x3|At Cursor%x4|Collapse%x2");
+	else event = pupmenu("Merge %t|At Center%x3|At Cursor%x4|Collapse%x2");
+	switch (event)
+	{
+		case -1:
+			return;
+		case 3:
+			if(G.qual & LR_CTRLKEY) remCount = merge_target(0,1);
+			else remCount = merge_target(0,0);
+			BIF_undo_push("Merge at center");
+			break;
+		case 4:
+			if(G.qual & LR_CTRLKEY) remCount = merge_target(1,1);
+			else remCount = merge_target(1,0);
+			BIF_undo_push("Merge at cursor");
+			break;
+		case 1:
+			if(G.qual & LR_CTRLKEY) remCount = merge_firstlast(0,1);
+			else remCount = merge_firstlast(0,0);
+			BIF_undo_push("Merge at last selected");
+			break;
+		case 6:
+			if(G.qual & LR_CTRLKEY) remCount = merge_firstlast(1,1);
+			else remCount = merge_firstlast(1,0);
+			BIF_undo_push("Merge at first selected");
+			break;
+		case 2:
+			remCount = collapseEdges();
+			BIF_undo_push("Collapse");
+			break;
+	}
+	notice("Removed %d Vertices", remCount);
 	allqueue(REDRAWVIEW3D, 0);
 	countall();
-	BIF_undo_push("Merge"); /* push the mesh down the undo pipe */
-
 }
+#undef MERGELIMIT
 
-void delete_context_selected(void) {
+
+void delete_context_selected(void) 
+{
 	if(G.obedit) {
 		if(G.obedit->type==OB_MESH) delete_mesh();
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) delNurb();
@@ -1673,15 +1622,16 @@ void delete_context_selected(void) {
 	else delete_obj(0);
 }
 
-void duplicate_context_selected(void) {
+void duplicate_context_selected(void) 
+{
 	if(G.obedit) {
 		if(G.obedit->type==OB_MESH) adduplicate_mesh();
 		else if(G.obedit->type==OB_ARMATURE) adduplicate_armature();
 		else if(G.obedit->type==OB_MBALL) adduplicate_mball();
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) adduplicate_nurb();
 	}
-	else if(!(G.obpose)){
-		adduplicate(0);
+	else {
+		adduplicate(0, U.dupflag);
 	}
 }
 
@@ -1714,11 +1664,8 @@ void minmax_verts(float *min, float *max)
 	int a;
 
 	tottrans=0;
-#ifdef __NLA
-	if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-	if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
+	if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) 
+		make_trans_verts(bmat[0], bmat[1], 0);
 	if(tottrans==0) return;
 
 	Mat3CpyMat4(bmat, G.obedit->obmat);
