@@ -1,5 +1,5 @@
 /**
- * $Id: editcurve.c,v 1.30 2005/06/13 19:15:02 stiv Exp $
+ * $Id: editcurve.c,v 1.49 2006/06/25 08:20:45 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -64,11 +64,12 @@
 #include "BKE_utildefines.h"
 #include "BKE_library.h"
 #include "BKE_ipo.h"
-#include "BKE_displist.h"
+#include "BKE_depsgraph.h"
 #include "BKE_curve.h"
 #include "BKE_global.h"
 #include "BKE_object.h"
 #include "BKE_main.h"
+#include "BKE_key.h"
 
 #include "BIF_editmode_undo.h"
 #include "BIF_gl.h"
@@ -184,48 +185,35 @@ void printknots()
 }
 
 #if 0
-static void printweightsNurb(void)
+static void printweightsNurb__doPrint(void *userData, Nurb *nurb, BPoint *bp, BezTriple *bezt, int beztindex, int x, int y)
 {
-	Nurb *nu;
-	BPoint *bp;
-	int a;
 	char str[30];
 
+	if (bp && (bp->f1&1)) {
+		sprintf(str,"%2.2f", bp->vec[3]);
+
+		cpack(0x737373);
+		glRasterPos2i(x-1, y-1);
+		BMF_DrawString(G.font, str);
+
+		glRasterPos2i(x+1, y+1);
+		BMF_DrawString(G.font, str);
+
+		cpack(0xFFFFFF);
+		glRasterPos2i(x, y);
+		BMF_DrawString(G.font, str);
+	}
+}
+static void printweightsNurb(void)
+{
 	if(G.obedit==0) return;
 
 	persp(PERSP_WIN);
 
 	glDrawBuffer(GL_FRONT); 
+	nurbs_foreachScreenVert(printweightsNurb__doPrint, NULL);
+	glDrawBuffer(GL_BACK);
 
-	nu= editNurb.first;
-	while(nu) {
-		if((nu->type & 7)==CU_NURBS) {
-			a= nu->pntsu*nu->pntsv;
-			bp= nu->bp;
-			while(a--) {
-				if(bp->f1 & 1) {
-					if(bp->s[0]!= 3200) {
-						sprintf(str,"%2.2f", bp->vec[3]);
-
-						cpack(0x737373);
-						glRasterPos2i(bp->s[0]-1, bp->s[1]-1);
-						BMF_DrawString(G.font, str);
-
-						glRasterPos2i(bp->s[0]+1, bp->s[1]+1);
-						BMF_DrawString(G.font, str);
-
-						cpack(0xFFFFFF);
-						glRasterPos2i(bp->s[0], bp->s[1]);
-						BMF_DrawString(G.font, str);
-					}
-				}
-				bp++;
-			}
-		}
-		nu= nu->next;
-	}
-
-	glDrawBuffer(GL_BACK); 
 	persp(PERSP_VIEW);
 }
 #endif
@@ -238,7 +226,7 @@ void load_editNurb()
 	/* load editNurb in object */
 	Curve *cu= 0;
 	Nurb *nu, *newnu;
-	KeyBlock *actkey=0;
+	KeyBlock *actkey;
 
 	if(G.obedit==0) return;
 
@@ -249,33 +237,22 @@ void load_editNurb()
 		cu= G.obedit->data;
 
 		/* are there keys? */
-		if(cu->key) {
-			actkey= cu->key->block.first;
-			while(actkey) {
-				if(actkey->flag & SELECT) break;
-				actkey= actkey->next;
-			}
-
-			if(actkey) {
-				/* active key: the vertices */
-				
-				if(G.totvert) {
-					if(actkey->data) MEM_freeN(actkey->data);
-				
-					actkey->data= MEM_callocN(cu->key->elemsize*G.totvert, "actkey->data");
-					actkey->totelem= G.totvert;
+		actkey = ob_get_keyblock(G.obedit);
+		if(actkey) {
+			/* active key: the vertices */
 			
-					curve_to_key(cu, actkey, &editNurb);
-				}
+			if(G.totvert) {
+				if(actkey->data) MEM_freeN(actkey->data);
+			
+				actkey->data= MEM_callocN(cu->key->elemsize*G.totvert, "actkey->data");
+				actkey->totelem= G.totvert;
+		
+				curve_to_key(cu, actkey, &editNurb);
 			}
 		}
 		
 		if(cu->key && actkey!=cu->key->refkey) {
-			/* there are keys, only write changes in vertices */
-			/* when amount of vertices differs, becomes unpredictable a bit */
-				
-			/* vertex -> vertex copy! */
-			if(actkey) key_to_curve(actkey, cu, &cu->nurb);		
+			;
 		}
 		else {
 			freeNurblist(&(cu->nurb));
@@ -296,7 +273,6 @@ void load_editNurb()
 	}
 	
 	lastnu= NULL;	/* for selected */
-	
 }
 
 void make_editNurb()
@@ -304,7 +280,7 @@ void make_editNurb()
 	/* make copy of baseNurb in editNurb */
 	Curve *cu=0;
 	Nurb *nu, *newnu;
-	KeyBlock *actkey=0;
+	KeyBlock *actkey;
 
 	if(G.obedit==NULL) return;
 
@@ -323,18 +299,11 @@ void make_editNurb()
 			nu= nu->next;
 		}
 		
-		if(cu->key) {
-			actkey= cu->key->block.first;
-			while(actkey) {
-				if(actkey->flag & SELECT) break;
-				actkey= actkey->next;
-			}
-		
-			if(actkey) {
-				key_to_curve(actkey, cu, &editNurb);
-			}
+		actkey = ob_get_keyblock(G.obedit);
+		if(actkey) {
+			strcpy(G.editModeTitleExtra, "(Key) ");
+			key_to_curve(actkey, cu, &editNurb);
 		}
-		makeDispList(G.obedit);
 	}
 	else G.obedit= NULL;
 	
@@ -361,7 +330,6 @@ void separate_nurb()
 	Base *base, *oldbase;
 	Curve *cu;
 	ListBase editnurbo;
-	float trans[9];
 
 	if( (G.vd->lay & G.obedit->lay)==0 ) return;
 
@@ -407,11 +375,7 @@ void separate_nurb()
 	oldob= G.obedit;
 	oldbase= BASACT;
 
-	trans[0]=trans[1]=trans[2]=trans[3]=trans[4]=trans[5]= 0.0;
-	trans[6]=trans[7]=trans[8]= 1.0;
-	G.qual |= LR_ALTKEY;	/* patch to make sure we get a linked dupli */
-	adduplicate(trans);
-	G.qual &= ~LR_ALTKEY;
+	adduplicate(1, 0); /* no transform and zero so do get a linked dupli */
 	
 	G.obedit= BASACT->object;	/* basact is set in adduplicate() */
 	
@@ -428,9 +392,8 @@ void separate_nurb()
 	editNurb= editnurbo;
 	
 	G.obedit= 0;	/* displists behave different in edit mode */
-	makeDispList(OBACT);	/* this is the separated one */
-
-	curve_changes_other_objects(oldob);
+	DAG_object_flush_update(G.scene, OBACT, OB_RECALC_DATA);	/* this is the separated one */
+	DAG_object_flush_update(G.scene, oldob, OB_RECALC_DATA);	/* this is the separated one */
 	
 	G.obedit= oldob;
 	BASACT= oldbase;
@@ -1031,8 +994,7 @@ void switchdirectionNurb2(void)
 		nu= nu->next;
 	}
 	
-	makeDispList(G.obedit);
-	curve_changes_other_objects(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 
 	allqueue(REDRAWVIEW3D, 0);
 	BIF_undo_push("Switch direction");
@@ -1223,7 +1185,7 @@ void hideNurb(int swap)
 		nu= nu->next;
 	}
 
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
@@ -1268,7 +1230,7 @@ void revealNurb()
 		nu= nu->next;
 	}
 
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
 	BIF_undo_push("Reveal");
@@ -1693,8 +1655,7 @@ void subdivideNurb()
 	}
 
 
-	makeDispList(G.obedit);
-	curve_changes_other_objects(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
@@ -1703,93 +1664,62 @@ void subdivideNurb()
 
 }
 
-
-short findnearestNurbvert(short sel, Nurb **nurb, BezTriple **bezt, BPoint **bp)
+static void findnearestNurbvert__doClosest(void *userData, Nurb *nu, BPoint *bp, BezTriple *bezt, int beztindex, int x, int y)
 {
-	/* sel==1: selected gets a disadvantage */
-	/* in nurb and bezt or bp the nearest is written */
-	/* return 0 1 2: handlepunt */
-	Nurb *nu;
-	BezTriple *bezt1;
-	BPoint *bp1;
-	short dist= 100, temp, mval[2], a, hpoint=0;
+	struct { BPoint *bp; BezTriple *bezt; Nurb *nurb; short dist, hpoint, select, mval[2]; } *data = userData;
 
-	*nurb= 0;
-	*bezt= 0;
-	*bp= 0;
+	short flag;
+	short temp;
 
-	/* do projection */
-	calc_nurbverts_ext();	/* drawobject.c */
-	
-	getmouseco_areawin(mval);
-
-	nu= editNurb.first;
-	while(nu) {
-		if((nu->type & 7)==CU_BEZIER) {
-			bezt1= nu->bezt;
-			a= nu->pntsu;
-			while(a--) {
-				if(bezt1->hide==0) {
-					temp= abs(mval[0]- bezt1->s[0][0])+ abs(mval[1]- bezt1->s[0][1]);
-					if( (bezt1->f1 & 1)==sel) temp+=5;
-					if(temp<dist) { 
-						hpoint=0; 
-						*bezt=bezt1; 
-						dist= temp; 
-						*nurb= nu; 
-						*bp= 0; 
-					}
-
-					/* middle points get a small disadvantage */
-					temp= 3+abs(mval[0]- bezt1->s[1][0])+ abs(mval[1]- bezt1->s[1][1]);
-					if( (bezt1->f2 & 1)==sel) temp+=5;
-					if(temp<dist) { 
-						hpoint=1; 
-						*bezt=bezt1; 
-						dist= temp; 
-						*nurb= nu; 
-						*bp= 0; 
-					}
-
-					temp= abs(mval[0]- bezt1->s[2][0])+ abs(mval[1]- bezt1->s[2][1]);
-					if( (bezt1->f3 & 1)==sel) temp+=5;
-					if(temp<dist) { 
-						hpoint=2; 
-						*bezt=bezt1; 
-						dist= temp; 
-						*nurb= nu; 
-						*bp= 0; 
-					}
-				}
-				bezt1++;
-			}
+	if (bp) {
+		flag = bp->f1;
+	} else {
+		if (beztindex==0) {
+			flag = bezt->f1;
+		} else if (beztindex==1) {
+			flag = bezt->f2;
+		} else {
+			flag = bezt->f3;
 		}
-		else {
-			bp1= nu->bp;
-			a= nu->pntsu*nu->pntsv;
-			while(a--) {
-				if(bp1->hide==0) {
-					temp= abs(mval[0]- bp1->s[0])+ abs(mval[1]- bp1->s[1]);
-					if( (bp1->f1 & 1)==sel) temp+=5;
-					if(temp<dist) { 
-						hpoint=0; 
-						*bp=bp1; 
-						dist= temp; 
-						*nurb= nu; 
-						*bezt= 0; 
-					}
-				}
-				bp1++;
-			}
-		}
-		nu= nu->next;
 	}
 
-	return hpoint;
+	temp = abs(data->mval[0]-x) + abs(data->mval[1]-y);
+	if ((flag&1)==data->select) temp += 5;
+	if (bezt && beztindex==1) temp += 3; /* middle points get a small disadvantage */
+
+	if (temp<data->dist) {
+		data->dist = temp;
+
+		data->bp = bp;
+		data->bezt = bezt;
+		data->nurb = nu;
+		data->hpoint = bezt?beztindex:0;
+	}
+}
+
+static short findnearestNurbvert(short sel, Nurb **nurb, BezTriple **bezt, BPoint **bp)
+{
+		/* sel==1: selected gets a disadvantage */
+		/* in nurb and bezt or bp the nearest is written */
+		/* return 0 1 2: handlepunt */
+	struct { BPoint *bp; BezTriple *bezt; Nurb *nurb; short dist, hpoint, select, mval[2]; } data = {0};
+
+	data.dist = 100;
+	data.hpoint = 0;
+	data.select = sel;
+	getmouseco_areawin(data.mval);
+
+	nurbs_foreachScreenVert(findnearestNurbvert__doClosest, &data);
+
+	*nurb = data.nurb;
+	*bezt = data.bezt;
+	*bp = data.bp;
+
+	return data.hpoint;
 }
 
 
-void findselectedNurbvert(Nurb **nu, BezTriple **bezt, BPoint **bp)
+static void findselectedNurbvert(Nurb **nu, BezTriple **bezt, BPoint **bp)
 {
 	/* in nu and (bezt or bp) selected are written if there's 1 sel.  */
 	/* if more points selected in 1 spline: return only nu, bezt and bp are 0 */
@@ -1944,7 +1874,7 @@ int convertspline(short type, Nurb *nu)
 	else if( (nu->type & 7)==CU_NURBS) {
 		if(type==0) {			/* to Poly */
 			nu->type &= ~7;
-			MEM_freeN(nu->knotsu);
+			if(nu->knotsu) MEM_freeN(nu->knotsu); /* python created nurbs have a knotsu of zero */
 			nu->knotsu= 0;
 			if(nu->knotsv) MEM_freeN(nu->knotsv);
 			nu->knotsv= 0;
@@ -2291,8 +2221,7 @@ void merge_nurb()
 	countall();
 	lastnu= NULL;
 
-	makeDispList(G.obedit);
-	curve_changes_other_objects(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
@@ -2449,8 +2378,7 @@ void addsegment_nurb()
 		
 		lastnu= NULL;	/* for selected */
 
-		makeDispList(G.obedit);
-		curve_changes_other_objects(G.obedit);
+		DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
 
 		countall();
 		allqueue(REDRAWVIEW3D, 0);
@@ -2656,24 +2584,6 @@ void spinNurb(float *dvec, short mode)
 	BIF_undo_push("Spin");
 }
 
-void curve_changes_other_objects(Object *ob)
-{
-	Base *base= FIRSTBASE;
-	while(base) {
-		if(base->lay & G.vd->lay) {
-			if(base->object->parent==ob && base->object->partype==PARSKEL)
-				freedisplist(&base->object->disp);
-			
-			if(base->object->type==OB_CURVE) {
-				Curve *cu= base->object->data;
-				if(ob==cu->bevobj || ob==cu->taperobj)
-					makeDispList(base->object);
-			}
-		}
-		base= base->next;
-	}
-}
-
 void addvert_Nurb(int mode)
 {
 	Nurb *nu;
@@ -2684,8 +2594,6 @@ void addvert_Nurb(int mode)
 	if(G.obedit==0) return;
 	if( (G.vd->lay & G.obedit->lay)==0 ) return;
 
-	if(mode=='e' && okee("Extrude")==0) return;
-
 	Mat3CpyMat4(mat, G.obedit->obmat);
 	Mat3Inv(imat,mat);
 
@@ -2701,8 +2609,8 @@ void addvert_Nurb(int mode)
 			memcpy(newbezt+1, bezt, nu->pntsu*sizeof(BezTriple));
 			*newbezt= *bezt;
 			newbezt->f1= newbezt->f2= newbezt->f3= 1;
-			if(bezt->h1 & 1) newbezt->h1= newbezt->h2= HD_AUTO;
-			else newbezt->h1= newbezt->h2= HD_VECT;
+			if(newbezt->h1 >= 0) newbezt->h2= newbezt->h1;
+			else newbezt->h2= newbezt->h1= HD_ALIGN; /* does this ever happen? */
 			VECCOPY(temp, bezt->vec[1]);
 			MEM_freeN(nu->bezt);
 			nu->bezt= newbezt;
@@ -2719,16 +2627,14 @@ void addvert_Nurb(int mode)
 			nu->bezt= newbezt;
 			newbezt+= nu->pntsu;
 			newbezt->f1= newbezt->f2= newbezt->f3= 1;
-			if(newbezt->h2 & 1) newbezt->h1= newbezt->h2= HD_AUTO;
-			else newbezt->h1= newbezt->h2= HD_VECT;
+			if(newbezt->h1 >= 0) newbezt->h2= newbezt->h1;
+			else newbezt->h2= newbezt->h1= HD_ALIGN; /* does this ever happen? */
 			bezt= nu->bezt+nu->pntsu-1;
 		}
 		else bezt= 0;
 
 		if(bezt) {
 			nu->pntsu++;
-			newbezt->s[1][0]= G.vd->mx;
-			newbezt->s[1][1]= G.vd->my;
 			
 			if(mode=='e') {
 				VECCOPY(newbezt->vec[0], bezt->vec[0]);
@@ -2775,8 +2681,6 @@ void addvert_Nurb(int mode)
 
 		if(bp) {
 			nu->pntsu++;
-			newbp->s[0]= G.vd->mx;
-			newbp->s[1]= G.vd->my;
 
 			if(nu->resolu<3) nu->resolu++;
 			makeknots(nu, 1, nu->flagu>>1);
@@ -2796,7 +2700,7 @@ void addvert_Nurb(int mode)
 	}
 
 	test2DNurb(nu);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
@@ -2810,7 +2714,6 @@ void addvert_Nurb(int mode)
 
 	if(mode!='e') {
 		/* dependencies with other objects, should become event */
-		curve_changes_other_objects(G.obedit);
 		BIF_undo_push("Add vertex");
 	
 	}
@@ -2834,12 +2737,11 @@ void extrude_nurb()
 		}
 		else {
 
-			if(okee("Extrude")==0) return;
-	
+
 			ok= extrudeflagNurb(1); /* '1'= flag */
 		
 			if(ok) {
-				makeDispList(G.obedit);
+				DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 				countall();
 				BIF_TransformSetUndo("Extrude");
 				initTransform(TFM_TRANSLATION, CTX_NO_PET);
@@ -2895,6 +2797,7 @@ void makecyclicNurb()
 						if(nu->flagu & 1) nu->flagu--;
 						else {
 							nu->flagu++;
+							nu->flagu &= ~2;	/* endpoint flag, fixme */
 							fp= MEM_mallocN(sizeof(float)*KNOTSU(nu), "makecyclicN");
 							b= (nu->orderu+nu->pntsu);
 							memcpy(fp, nu->knotsu, sizeof(float)*b);
@@ -2954,8 +2857,7 @@ void makecyclicNurb()
 		}
 		nu= nu->next;
 	}
-	makeDispList(G.obedit);
-	curve_changes_other_objects(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	BIF_undo_push("Cyclic");
 }
 
@@ -3097,7 +2999,7 @@ void delNurb()
 		else freeNurblist(&editNurb);
 
 		countall();
-		makeDispList(G.obedit);
+		DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWBUTSEDIT, 0);
 		BIF_undo_push("Delete");
@@ -3214,7 +3116,7 @@ void delNurb()
 								bezt2= bezt+(nu->pntsu-1);
 								if( (bezt2->f1 & 1) || (bezt2->f2 & 1) || (bezt2->f3 & 1) ) {
 									nu->flagu--;
-									makeDispList(G.obedit);
+									DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 									allqueue(REDRAWVIEW3D, 0);
 									allqueue(REDRAWBUTSEDIT, 0);
 									BIF_undo_push("Delete");
@@ -3241,7 +3143,7 @@ void delNurb()
 								bp2= bp+(nu->pntsu-1);
 								if( bp2->f1 & 1 ) {
 									nu->flagu--;
-									makeDispList(G.obedit);
+									DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 									allqueue(REDRAWVIEW3D, 0);
 									allqueue(REDRAWBUTSEDIT, 0);
 									BIF_undo_push("Delete");
@@ -3342,8 +3244,7 @@ void delNurb()
 	}
 
 	countall();
-	makeDispList(G.obedit);
-	curve_changes_other_objects(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
@@ -3351,8 +3252,31 @@ void delNurb()
 	
 }
 
+void nurb_set_smooth(short event)
+{
+	Nurb *nu;
+	
+	if(G.obedit==0) return;
+	
+	if(G.obedit->type != OB_CURVE) return;
+	
+	nu= editNurb.first;
+	while(nu) {
+		if(isNurbsel(nu)) {
+			if(event==1) nu->flag |= CU_SMOOTH;
+			else if(event==0) nu->flag &= ~CU_SMOOTH;
+		}
+		nu= nu->next;
+	}
+	
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
+	allqueue(REDRAWVIEW3D, 0);
+	
+	if(event==1) BIF_undo_push("Set Smooth");
+	else if(event==0) BIF_undo_push("Set Solid");
+}
 
-void join_curve(int type)
+int join_curve(int type)
 {
 	Base *base, *nextb;
 	Object *ob;
@@ -3364,18 +3288,11 @@ void join_curve(int type)
 	float imat[4][4], cmat[4][4];
 	int a;
 	
-	if(G.obedit) return;
-	
 	ob= OBACT;
-	if(ob->type!=type) return;
-	if(ob->lay & G.vd->lay); else return;
+	if(ob->type!=type) return 0;
+	if(ob->lay & G.vd->lay); else return 0;
 	tempbase.first= tempbase.last= 0;
 	
-	if(type==OB_SURF) {
-		if(okee("Join selected NURBS")==0) return;
-	}
-	else if(okee("Join selected curves")==0) return;
-	
 	/* trasnform all selected curves inverse in obact */
 	Mat4Invert(imat, ob->obmat);
 	
@@ -3427,13 +3344,15 @@ void join_curve(int type)
 	cu= ob->data;
 	addlisttolist(&cu->nurb, &tempbase);
 	
+	DAG_scene_sort(G.scene);	// because we removed object(s), call before editmode!
+	
 	enter_editmode();
 	exit_editmode(1);
 	
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
 	BIF_undo_push("Join");
-	
+	return 1;
 }
 
 
@@ -3826,7 +3745,7 @@ void default_curve_ipo(Curve *cu)
 			
 	icu->blocktype= ID_CU;
 	icu->adrcode= CU_SPEED;
-	icu->flag= IPO_VISIBLE+IPO_SELECT;
+	icu->flag= IPO_VISIBLE|IPO_SELECT|IPO_AUTO_HORIZ;
 	set_icu_vars(icu);
 	
 	BLI_addtail( &(cu->ipo->curve), icu);
@@ -3900,8 +3819,7 @@ void add_primitiveCurve(int stype)
 	nu= addNurbprim(type, stype, newname);   /* 2D */
 	
 	BLI_addtail(&editNurb, nu);
-	makeDispList(G.obedit);
-	curve_changes_other_objects(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 
 	countall();
 	allqueue(REDRAWALL, 0);
@@ -3936,7 +3854,7 @@ void add_primitiveNurb(int type)
 
 	nu= addNurbprim(4, type, newname);
 	BLI_addtail(&editNurb,nu);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 
 	countall();
 	allqueue(REDRAWALL, 0);
@@ -3978,7 +3896,7 @@ void clear_tilt()
 		nu= nu->next;
 	}
 
-	makeBevelList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	allqueue(REDRAWVIEW3D, 0);
 	BIF_undo_push("Clear tilt");
 	
