@@ -1,5 +1,5 @@
 /**
- * $Id: editdeform.c,v 1.16 2005/03/28 19:55:04 zuster Exp $
+ * $Id: editdeform.c,v 1.26 2005/12/19 11:55:31 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,8 @@
 #include "BLI_blenlib.h"
 #include "BLI_editVert.h"
 
+#include "BKE_DerivedMesh.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
 #include "BKE_deform.h"
 #include "BKE_displist.h"
@@ -100,14 +102,15 @@ void sel_verts_defgroup (int select)
 	else EM_deselect_flush();
 }
 
-MDeformWeight *verify_defweight (MDeformVert *dv, int defgroup)
 /* Ensures that mv has a deform weight entry for
-the specified defweight group */
+   the specified defweight group */
+/* Note this function is mirrored in editmesh_tools.c, for use for editvertices */
+MDeformWeight *verify_defweight (MDeformVert *dv, int defgroup)
 {
-	int	i;
 	MDeformWeight *newdw;
+	int	i;
 
-	if (!dv)
+	if (!dv || defgroup<0)
 		return NULL;
 
 	for (i=0; i<dv->totweight; i++){
@@ -131,7 +134,8 @@ the specified defweight group */
 	return dv->dw+(dv->totweight-1);
 }
 
-void add_defgroup (Object *ob) {
+void add_defgroup (Object *ob) 
+{
 	add_defgroup_name (ob, "Group");
 }
 
@@ -142,14 +146,9 @@ bDeformGroup *add_defgroup_name (Object *ob, char *name)
 	if (!ob)
 		return NULL;
 	
-	defgroup = MEM_callocN (sizeof(bDeformGroup), "deformGroup");
+	defgroup = MEM_callocN (sizeof(bDeformGroup), "add deformGroup");
 
-	/* I think there should be some length
-	 * checking here -- don't know why NaN
-	 * never checks name lengths (see
-	 * unique_vertexgroup_name, for example).
-	 */
-	strcpy (defgroup->name, name);
+	BLI_strncpy (defgroup->name, name, 32);
 
 	BLI_addtail(&ob->defbase, defgroup);
 	unique_vertexgroup_name(defgroup, ob);
@@ -191,9 +190,16 @@ void del_defgroup (Object *ob)
 	/* Update the active material index if necessary */
 	if (ob->actdef==BLI_countlist(&ob->defbase))
 		ob->actdef--;
-
+	
 	/* Remove the group */
 	BLI_freelinkN (&ob->defbase, defgroup);
+	
+	/* remove all dverts */
+	if(ob->actdef==0) {
+		Mesh *me= ob->data;
+		free_dverts(me->dvert, me->totvert);
+		me->dvert= NULL;
+	}
 }
 
 void create_dverts(Mesh *me)
@@ -286,8 +292,11 @@ void add_vert_defnr (Object *ob, int def_nr, int vertnum,
 
 	/* get the vert
 	 */
-	dv = ((Mesh*)ob->data)->dvert + vertnum;
-
+	if(((Mesh*)ob->data)->dvert)
+		dv = ((Mesh*)ob->data)->dvert + vertnum;
+	else 
+		return;
+	
 	/* Lets first check to see if this vert is
 	 * already in the weight group -- if so
 	 * lets update it
@@ -594,40 +603,32 @@ void unique_vertexgroup_name (bDeformGroup *dg, Object *ob)
 			}
 		}
 		if (!exists){
-			strcpy (dg->name, tempname);
+			BLI_strncpy (dg->name, tempname, 32);
 			return;
 		}
 	}	
 }
 
+void vertexgroup_select_by_name(Object *ob, char *name)
+{
+	bDeformGroup *curdef;
+	int actdef= 1;
+	
+	if(ob==NULL || ob->type!=OB_MESH) return;
+	for (curdef = ob->defbase.first; curdef; curdef=curdef->next, actdef++){
+		if (!strcmp(curdef->name, name)) {
+			ob->actdef= actdef;
+			return;
+		}
+	}
+	ob->actdef=0;	// this signals on painting to create a new one, if a bone in posemode is selected */
+}
+
+
 /* ******************* other deform edit stuff ********** */
 
 void object_apply_deform(Object *ob)
 {
-	char *err= NULL;
-	
-	if(ob->type==OB_MESH) {
-		Mesh *me= ob->data;
-		if(me->id.us>1) {
-			err= "Can't apply deformation to Mesh with other users";
-		} else {
-			mesh_modifier(ob, 's'); // start
-			mesh_modifier(ob, 'a'); // apply and end
-			freedisplist(&ob->disp);
-		}
-	}
-	else if (ob->type==OB_CURVE || ob->type==OB_SURF) {
-		Curve *cu= ob->data;
-		if(cu->id.us>1) {
-			err= "Can't apply deformation to Curve with other users";
-		} else {
-			curve_modifier(ob, 's'); // start
-			curve_modifier(ob, 'a'); // apply and end
-			freedisplist(&ob->disp);
-		}
-	}
-
-	if(err) error(err);
-
+	notice("Apply Deformation now only availble in Modifier buttons");
 }
 
