@@ -1,5 +1,5 @@
 /**
- * $Id: editface.c,v 1.25 2004/05/03 13:23:19 blendix Exp $
+ * $Id: editface.c,v 1.63 2006/06/27 04:30:06 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,17 +34,12 @@
 #include <math.h>
 #include <string.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
+#include "BLI_heap.h"
+#include "BLI_edgehash.h"
 
 #include "MTC_matrixops.h"
 
@@ -57,18 +52,23 @@
 #include "DNA_object_types.h"
 #include "DNA_space_types.h"
 #include "DNA_screen_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
+#include "DNA_userdef_types.h"
 
 #include "BKE_utildefines.h"
+#include "BKE_depsgraph.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_mesh.h"
 #include "BKE_texture.h"
+#include "BKE_object.h"
 
 #include "BSE_view.h"
 #include "BSE_edit.h"
 #include "BSE_drawview.h"	/* for backdrawview3d */
 
+#include "BIF_editsima.h"
 #include "BIF_interface.h"
 #include "BIF_mywindow.h"
 #include "BIF_toolbox.h"
@@ -86,17 +86,93 @@
 
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
 #include "butspace.h"
 
-#include "TPT_DependKludge.h"
-
-#ifdef NAN_TPT
 #include "../img/IMG_Api.h"
 #include "BSE_trans_types.h"
-#endif /* NAN_TPT */
 
-TFace *lasttface=0;
+#include "BDR_unwrapper.h"
+
+/* Pupmenu codes: */
+#define UV_CUBE_MAPPING 2
+#define UV_CYL_MAPPING 3
+#define UV_SPHERE_MAPPING 4
+#define UV_BOUNDS8_MAPPING 68
+#define UV_BOUNDS4_MAPPING 65
+#define UV_BOUNDS2_MAPPING 66
+#define UV_BOUNDS1_MAPPING 67
+#define UV_STD8_MAPPING 131
+#define UV_STD4_MAPPING 130
+#define UV_STD2_MAPPING 129
+#define UV_STD1_MAPPING 128
+#define UV_WINDOW_MAPPING 5
+#define UV_UNWRAP_MAPPING 6
+#define UV_CYL_EX 32
+#define UV_SPHERE_EX 34
+
+/* Some macro tricks to make pupmenu construction look nicer :-)
+   Sorry, just did it for fun. */
+
+#define _STR(x) " " #x
+#define STRING(x) _STR(x)
+
+#define MENUSTRING(string, code) string " %x" STRING(code)
+#define MENUTITLE(string) string " %t|" 
+
+
+/* returns 0 if not found, otherwise 1 */
+static int facesel_face_pick(Mesh *me, short *mval, unsigned int *index, short rect)
+{
+	if (!me->tface || me->totface==0)
+		return 0;
+
+	if (G.vd->flag & V3D_NEEDBACKBUFDRAW) {
+		check_backbuf();
+		persp(PERSP_VIEW);
+	}
+
+	if (rect) {
+		/* sample rect to increase changes of selecting, so that when clicking
+		   on an edge in the backbuf, we can still select a face */
+		short dist;
+		*index = sample_backbuf_rect(mval, 3, 1, me->totface+1, &dist);
+	}
+	else
+		/* sample only on the exact position */
+		*index = sample_backbuf(mval[0], mval[1]);
+
+	if ((*index)<=0 || (*index)>(unsigned int)me->totface)
+		return 0;
+
+	(*index)--;
+	
+	return 1;
+}
+
+/* returns 0 if not found, otherwise 1 */
+static int facesel_edge_pick(Mesh *me, short *mval, unsigned int *index)
+{
+	short dist;
+	unsigned int min = me->totface + 1;
+	unsigned int max = me->totface + me->totedge + 1;
+
+	if (me->totedge == 0)
+		return 0;
+
+	if (G.vd->flag & V3D_NEEDBACKBUFDRAW) {
+		check_backbuf();
+		persp(PERSP_VIEW);
+	}
+
+	*index = sample_backbuf_rect(mval, 50, min, max, &dist);
+
+	if (*index == 0)
+		return 0;
+
+	(*index)--;
+	
+	return 1;
+}
 
 static void uv_calc_center_vector(float *result, Object *ob, Mesh *me)
 {
@@ -114,7 +190,7 @@ static void uv_calc_center_vector(float *result, Object *ob, Mesh *me)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				DO_MINMAX((me->mvert+mface->v1)->co, min, max);
 				DO_MINMAX((me->mvert+mface->v2)->co, min, max);
 				DO_MINMAX((me->mvert+mface->v3)->co, min, max);
@@ -270,17 +346,14 @@ void calculate_uv_map(unsigned short mapmode)
 	float dx, dy, rotatematrix[4][4], radius= 1.0, min[3], cent[3], max[3];
 	float fac= 1.0, upangledeg= 0.0, sideangledeg= 90.0;
 	int i, b, mi, a, n;
-	/* settings from buttonswindow */
-	extern float uv_calc_radius, uv_calc_cubesize;
-	extern short uv_calc_mapdir, uv_calc_mapalign;
 
-	if(uv_calc_mapdir==1)  {
+	if(G.scene->toolsettings->uvcalc_mapdir==1)  {
 		upangledeg= 90.0;
 		sideangledeg= 0.0;
 	}
 	else {
 		upangledeg= 0.0;
-		if(uv_calc_mapalign==1) sideangledeg= 0.0;
+		if(G.scene->toolsettings->uvcalc_mapalign==1) sideangledeg= 0.0;
 		else sideangledeg= 90.0;
 	}
 
@@ -308,7 +381,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				uv_calc_shift_project(tface->uv[0],cent,rotatematrix,3,(me->mvert+mface->v1)->co,min,max);
 				uv_calc_shift_project(tface->uv[1],cent,rotatematrix,3,(me->mvert+mface->v2)->co,min,max);
 				uv_calc_shift_project(tface->uv[2],cent,rotatematrix,3,(me->mvert+mface->v3)->co,min,max);
@@ -324,7 +397,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				if(mface->v4) b= 3; else b= 2;
 				for(; b>=0; b--) {
 					tface->uv[b][0]= ((tface->uv[b][0]-min[0])*fac)/dx;
@@ -341,7 +414,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				uv_calc_shift_project(tface->uv[0],cent,rotatematrix,4,(me->mvert+mface->v1)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[1],cent,rotatematrix,4,(me->mvert+mface->v2)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[2],cent,rotatematrix,4,(me->mvert+mface->v3)->co,NULL,NULL);
@@ -371,10 +444,10 @@ void calculate_uv_map(unsigned short mapmode)
 	case B_UVAUTO_SPHERE:
 		uv_calc_center_vector(cent, ob, me);
 			
-		if(mapmode==B_UVAUTO_CYLINDER) radius = uv_calc_radius;
+		if(mapmode==B_UVAUTO_CYLINDER) radius = G.scene->toolsettings->uvcalc_radius;
 
 		/* be compatible to the "old" sphere/cylinder mode */
-		if (uv_calc_mapdir== 2)
+		if (G.scene->toolsettings->uvcalc_mapdir== 2)
 			Mat4One(rotatematrix);
 		else 
 			uv_calc_map_matrix(rotatematrix,ob,upangledeg,sideangledeg,radius);
@@ -382,7 +455,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				uv_calc_shift_project(tface->uv[0],cent,rotatematrix,mapmode,(me->mvert+mface->v1)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[1],cent,rotatematrix,mapmode,(me->mvert+mface->v2)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[2],cent,rotatematrix,mapmode,(me->mvert+mface->v3)->co,NULL,NULL);
@@ -415,11 +488,12 @@ void calculate_uv_map(unsigned short mapmode)
 		short cox, coy;
 		float *loc= ob->obmat[3];
 		MVert *mv= me->mvert;
+		float cubesize = G.scene->toolsettings->uvcalc_cubesize;
 
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				CalcNormFloat((mv+mface->v1)->co, (mv+mface->v2)->co, (mv+mface->v3)->co, no);
 					
 				no[0]= fabs(no[0]);
@@ -431,23 +505,23 @@ void calculate_uv_map(unsigned short mapmode)
 				else if(no[1]>=no[0] && no[1]>=no[2]) coy= 2;
 				else { cox= 1; coy= 2; }
 				
-				tface->uv[0][0]= 0.5+0.5*uv_calc_cubesize*(loc[cox] + (mv+mface->v1)->co[cox]);
-				tface->uv[0][1]= 0.5+0.5*uv_calc_cubesize*(loc[coy] + (mv+mface->v1)->co[coy]);
+				tface->uv[0][0]= 0.5+0.5*cubesize*(loc[cox] + (mv+mface->v1)->co[cox]);
+				tface->uv[0][1]= 0.5+0.5*cubesize*(loc[coy] + (mv+mface->v1)->co[coy]);
 				dx = floor(tface->uv[0][0]);
 				dy = floor(tface->uv[0][1]);
 				tface->uv[0][0] -= dx;
 				tface->uv[0][1] -= dy;
-				tface->uv[1][0]= 0.5+0.5*uv_calc_cubesize*(loc[cox] + (mv+mface->v2)->co[cox]);
-				tface->uv[1][1]= 0.5+0.5*uv_calc_cubesize*(loc[coy] + (mv+mface->v2)->co[coy]);
+				tface->uv[1][0]= 0.5+0.5*cubesize*(loc[cox] + (mv+mface->v2)->co[cox]);
+				tface->uv[1][1]= 0.5+0.5*cubesize*(loc[coy] + (mv+mface->v2)->co[coy]);
 				tface->uv[1][0] -= dx;
 				tface->uv[1][1] -= dy;
-				tface->uv[2][0]= 0.5+0.5*uv_calc_cubesize*(loc[cox] + (mv+mface->v3)->co[cox]);
-				tface->uv[2][1]= 0.5+0.5*uv_calc_cubesize*(loc[coy] + (mv+mface->v3)->co[coy]);
+				tface->uv[2][0]= 0.5+0.5*cubesize*(loc[cox] + (mv+mface->v3)->co[cox]);
+				tface->uv[2][1]= 0.5+0.5*cubesize*(loc[coy] + (mv+mface->v3)->co[coy]);
 				tface->uv[2][0] -= dx;
 				tface->uv[2][1] -= dy;
 				if(mface->v4) {
-					tface->uv[3][0]= 0.5+0.5*uv_calc_cubesize*(loc[cox] + (mv+mface->v4)->co[cox]);
-					tface->uv[3][1]= 0.5+0.5*uv_calc_cubesize*(loc[coy] + (mv+mface->v4)->co[coy]);
+					tface->uv[3][0]= 0.5+0.5*cubesize*(loc[cox] + (mv+mface->v4)->co[cox]);
+					tface->uv[3][1]= 0.5+0.5*cubesize*(loc[coy] + (mv+mface->v4)->co[coy]);
 					tface->uv[3][0] -= dx;
 					tface->uv[3][1] -= dy;
 				}
@@ -464,7 +538,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(!(tface->flag & TF_SELECT && mface->v3)) return;
+			if(!(tface->flag & TF_SELECT)) continue;
 				
 			dx= dy= 0;
 			if(mface->v4) b= 3; else b= 2;
@@ -486,51 +560,40 @@ void calculate_uv_map(unsigned short mapmode)
 		}
 	}
 
+	BIF_undo_push("UV calculation");
+
+	object_uvs_changed(OBACT);
+
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWIMAGE, 0);
 }
 
-void set_lasttface()
+TFace *get_active_tface()
 {
 	Mesh *me;
-	TFace *tface;
+	TFace *tf;
 	int a;
 	
-	lasttface= 0;
-	if(OBACT==NULL || OBACT->type!=OB_MESH) return;
+	if(OBACT==NULL || OBACT->type!=OB_MESH)
+		return NULL;
 	
 	me= get_mesh(OBACT);
-	if(me==0 || me->tface==0) return;
+	if(me==0 || me->tface==0)
+		return NULL;
 	
-	tface= me->tface;
-	a= me->totface;
-	while(a--) {
-		if(tface->flag & TF_ACTIVE) {
-			lasttface= tface;
-			return;
-		}
-		tface++;
-	}
+	for(a=0, tf=me->tface; a < me->totface; a++, tf++)
+		if(tf->flag & TF_ACTIVE)
+			return tf;
 
-	tface= me->tface;
-	a= me->totface;
-	while(a--) {
-		if(tface->flag & TF_SELECT) {
-			lasttface= tface;
-			return;
-		}
-		tface++;
-	}
+	for(a=0, tf=me->tface; a < me->totface; a++, tf++)
+		if(tf->flag & TF_SELECT)
+			return tf;
 
-	tface= me->tface;
-	a= me->totface;
-	while(a--) {
-		if((tface->flag & TF_HIDE)==0) {
-			lasttface= tface;
-			return;
-		}
-		tface++;
-	}
+	for(a=0, tf=me->tface; a < me->totface; a++, tf++)
+		if((tf->flag & TF_HIDE)==0)
+			return tf;
+	
+	return NULL;
 }
 
 void default_uv(float uv[][2], float size)
@@ -586,6 +649,7 @@ void make_tfaces(Mesh *me)
 	}
 }
 
+
 void reveal_tface()
 {
 	Mesh *me;
@@ -605,12 +669,11 @@ void reveal_tface()
 		tface++;
 	}
 
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+	BIF_undo_push("Reveal UV face");
+
+	object_tface_flags_changed(OBACT, 0);
 }
 
-
-
 void hide_tface()
 {
 	Mesh *me;
@@ -641,74 +704,32 @@ void hide_tface()
 		
 		tface++;
 	}
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
-	
+
+	BIF_undo_push("Hide UV face");
+
+	object_tface_flags_changed(OBACT, 0);
 }
 
-void select_linked_tfaces()
+void select_linked_tfaces(int mode)
 {
+	Object *ob;
 	Mesh *me;
-	TFace *tface;
-	MFace *mface;
-	int a, doit=1;
-	char *cpmain;
-	
-	me= get_mesh(OBACT);
-	if(me==0 || me->tface==0 || me->totface==0) return;
-	
-	cpmain= MEM_callocN(me->totvert, "cpmain");
-	
-	while(doit) {
-		doit= 0;
-		
-		/* select connected: fill array */
-		tface= me->tface;
-		mface= me->mface;
-		a= me->totface;
-		while(a--) {
-			if(tface->flag & TF_HIDE);
-			else if(tface->flag & TF_SELECT) {
-				if( mface->v3) {
-					cpmain[mface->v1]= 1;
-					cpmain[mface->v2]= 1;
-					cpmain[mface->v3]= 1;
-					if(mface->v4) cpmain[mface->v4]= 1;
-				}
-			}
-			tface++; mface++;
-		}
-		
-		/* reverse: using array select the faces */
+	short mval[2];
+	unsigned int index=0;
 
-		tface= me->tface;
-		mface= me->mface;
-		a= me->totface;
-		while(a--) {
-			if(tface->flag & TF_HIDE);
-			else if((tface->flag & TF_SELECT)==0) {
-				if( mface->v3) {
-					if(mface->v4) {
-						if(cpmain[mface->v4]) {
-							tface->flag |= TF_SELECT;
-							doit= 1;
-						}
-					}
-					if( cpmain[mface->v1] || cpmain[mface->v2] || cpmain[mface->v3] ) {
-						tface->flag |= TF_SELECT;
-						doit= 1;
-					}
-				}
-			}
-			tface++; mface++;
-		}
-		
+	ob = OBACT;
+	me = get_mesh(ob);
+	if(me==0 || me->tface==0 || me->totface==0) return;
+
+	if (mode==0 || mode==1) {
+		if (!(ob->lay & G.vd->lay))
+			error("The active object is not in this layer");
+			
+		getmouseco_areawin(mval);
+		if (!facesel_face_pick(me, mval, &index, 1)) return;
 	}
-	MEM_freeN(cpmain);
-	
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
-	
+
+	select_linked_tfaces_with_seams(mode, me, index);
 }
 
 void deselectall_tface()
@@ -739,8 +760,10 @@ void deselectall_tface()
 		}
 		tface++;
 	}
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+
+	BIF_undo_push("(De)select all UV face");
+
+	object_tface_flags_changed(OBACT, 0);
 }
 
 void selectswap_tface(void)
@@ -763,8 +786,9 @@ void selectswap_tface(void)
 		tface++;
 	}
 
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+	BIF_undo_push("Select inverse UV face");
+
+	object_tface_flags_changed(OBACT, 0);
 }
 
 void rotate_uv_tface()
@@ -778,7 +802,7 @@ void rotate_uv_tface()
 	me= get_mesh(OBACT);
 	if(me==0 || me->tface==0) return;
 	
-	mode= pupmenu("OK? %t|Rot UV %x1|Rot VertexCol %x2");
+	mode= pupmenu("Rotate %t|UV Co-ordinates %x1|Vertex Colors %x2");
 	
 	if(mode<1) return;
 	
@@ -828,57 +852,398 @@ void rotate_uv_tface()
 		mface++;
 	}
 	
-	makeDispList(OBACT);
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+	BIF_undo_push("Rotate UV face");
+
+	object_uvs_changed(OBACT);
 }
 
-/**
- * Returns the face under the give position in screen coordinates.
- * Code extracted from face_select routine.
- * Question: why is all of the backbuffer drawn?
- * We're only interested in one pixel!
- * @author	Maarten Gribnau
- * @param	me	the mesh with the faces to be picked
- * @param	x	the x-coordinate to pick at
- * @param	y	the y-coordinate to pick at
- * @return the face under the cursor (-1 if there was no face found)
- */
-int face_pick(Mesh *me, short x, short y)
+void mirror_uv_tface()
 {
-	unsigned int col;
-	int index;
+	Mesh *me;
+	TFace *tface;
+	MFace *mface;
+	short mode;
+	int a;
+	
+	me= get_mesh(OBACT);
+	if(me==0 || me->tface==0) return;
+	
+	mode= pupmenu("Mirror %t|UV Co-ordinates %x1|Vertex Colors %x2");
+	
+	if(mode<1) return;
+	
+	tface= me->tface;
+	mface= me->mface;
+	a= me->totface;
+	while(a--) {
+		if(tface->flag & TF_SELECT) {
+			if(mode==1) {
+				float u1= tface->uv[0][0];
+				float v1= tface->uv[0][1];
+				if(mface->v4) {
+					tface->uv[0][0]= tface->uv[3][0];
+					tface->uv[0][1]= tface->uv[3][1];
+				
+					tface->uv[3][0]= u1;
+					tface->uv[3][1]= v1;
 
-	if (me==0 || me->tface==0) {
-		return -1;
+					u1= tface->uv[1][0];
+					v1= tface->uv[1][1];
+
+					tface->uv[1][0]= tface->uv[2][0];
+					tface->uv[1][1]= tface->uv[2][1];
+				
+					tface->uv[2][0]= u1;
+					tface->uv[2][1]= v1;
+				}
+				else {
+					tface->uv[0][0]= tface->uv[2][0];
+					tface->uv[0][1]= tface->uv[2][1];
+					tface->uv[2][0]= u1;
+					tface->uv[2][1]= v1;
+				}
+			}
+			else if(mode==2) {
+				unsigned int tcol= tface->col[0];
+				if(mface->v4) {
+					tface->col[0]= tface->col[3];
+					tface->col[3]= tcol;
+
+					tcol = tface->col[1];
+					tface->col[1]= tface->col[2];
+					tface->col[2]= tcol;
+				}
+				else {
+					tface->col[0]= tface->col[2];
+					tface->col[2]= tcol;
+				}
+			}
+		}
+		tface++;
+		mface++;
+	}
+	
+	BIF_undo_push("Mirror UV face");
+
+	object_uvs_changed(OBACT);
+}
+
+void minmax_tface(float *min, float *max)
+{
+	Object *ob;
+	Mesh *me;
+	MFace *mf;
+	TFace *tf;
+	MVert *mv;
+	int a;
+	float vec[3], bmat[3][3];
+	
+	ob = OBACT;
+	if (ob==0) return;
+	me= get_mesh(ob);
+	if(me==0 || me->tface==0) return;
+	
+	Mat3CpyMat4(bmat, ob->obmat);
+
+	mv= me->mvert;
+	mf= me->mface;
+	tf= me->tface;
+	for (a=me->totface; a>0; a--, mf++, tf++) {
+		if (tf->flag & TF_HIDE || !(tf->flag & TF_SELECT))
+			continue;
+
+		VECCOPY(vec, (mv+mf->v1)->co);
+		Mat3MulVecfl(bmat, vec);
+		VecAddf(vec, vec, ob->obmat[3]);
+		DO_MINMAX(vec, min, max);		
+
+		VECCOPY(vec, (mv+mf->v2)->co);
+		Mat3MulVecfl(bmat, vec);
+		VecAddf(vec, vec, ob->obmat[3]);
+		DO_MINMAX(vec, min, max);		
+
+		VECCOPY(vec, (mv+mf->v3)->co);
+		Mat3MulVecfl(bmat, vec);
+		VecAddf(vec, vec, ob->obmat[3]);
+		DO_MINMAX(vec, min, max);		
+
+		if (mf->v4) {
+			VECCOPY(vec, (mv+mf->v4)->co);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, ob->obmat[3]);
+			DO_MINMAX(vec, min, max);
+		}
+	}
+}
+
+#define ME_SEAM_DONE ME_SEAM_LAST		/* reuse this flag */
+
+static float seam_cut_cost(Mesh *me, int e1, int e2, int vert)
+{
+	MVert *v = me->mvert + vert;
+	MEdge *med1 = me->medge + e1, *med2 = me->medge + e2;
+	MVert *v1 = me->mvert + ((med1->v1 == vert)? med1->v2: med1->v1);
+	MVert *v2 = me->mvert + ((med2->v1 == vert)? med2->v2: med2->v1);
+	float cost, d1[3], d2[3];
+
+	cost = VecLenf(v1->co, v->co);
+	cost += VecLenf(v->co, v2->co);
+
+	VecSubf(d1, v->co, v1->co);
+	VecSubf(d2, v2->co, v->co);
+
+	cost = cost + 0.5f*cost*(2.0f - fabs(d1[0]*d2[0] + d1[1]*d2[1] + d1[2]*d2[2]));
+
+	return cost;
+}
+
+static void seam_add_adjacent(Mesh *me, Heap *heap, int mednum, int vertnum, int *nedges, int *edges, int *prevedge, float *cost)
+{
+	int startadj, endadj = nedges[vertnum+1];
+
+	for (startadj = nedges[vertnum]; startadj < endadj; startadj++) {
+		int adjnum = edges[startadj];
+		MEdge *medadj = me->medge + adjnum;
+		float newcost;
+
+		if (medadj->flag & ME_SEAM_DONE)
+			continue;
+
+		newcost = cost[mednum] + seam_cut_cost(me, mednum, adjnum, vertnum);
+
+		if (cost[adjnum] > newcost) {
+			cost[adjnum] = newcost;
+			prevedge[adjnum] = mednum;
+			BLI_heap_insert(heap, newcost, (void*)adjnum);
+		}
+	}
+}
+
+static int seam_shortest_path(Mesh *me, int source, int target)
+{
+	Heap *heap;
+	EdgeHash *ehash;
+	float *cost;
+	MEdge *med;
+	int a, *nedges, *edges, *prevedge, mednum = -1, nedgeswap = 0;
+	TFace *tf;
+	MFace *mf;
+
+	/* mark hidden edges as done, so we don't use them */
+	ehash = BLI_edgehash_new();
+
+	for (a=0, mf=me->mface, tf=me->tface; a<me->totface; a++, tf++, mf++) {
+		if (!(tf->flag & TF_HIDE)) {
+			BLI_edgehash_insert(ehash, mf->v1, mf->v2, NULL);
+			BLI_edgehash_insert(ehash, mf->v2, mf->v3, NULL);
+			if (mf->v4) {
+				BLI_edgehash_insert(ehash, mf->v3, mf->v4, NULL);
+				BLI_edgehash_insert(ehash, mf->v4, mf->v1, NULL);
+			}
+			else
+				BLI_edgehash_insert(ehash, mf->v3, mf->v1, NULL);
+		}
 	}
 
-	/* Have OpenGL draw in the back buffer with color coded face indices */
-	if (curarea->win_swap==WIN_EQUAL) {
-		G.vd->flag |= V3D_NEEDBACKBUFDRAW;
-	}
-	if (G.vd->flag & V3D_NEEDBACKBUFDRAW) {
-		backdrawview3d(0);
-		persp(PERSP_VIEW);
-	}
-	/* Read the pixel under the cursor */
-#ifdef __APPLE__
-	glReadBuffer(GL_AUX0);
-#endif
-	glReadPixels(x+curarea->winrct.xmin, y+curarea->winrct.ymin, 1, 1,
-		GL_RGBA, GL_UNSIGNED_BYTE, &col);
-	glReadBuffer(GL_BACK);
+	for (a=0, med=me->medge; a<me->totedge; a++, med++)
+		if (!BLI_edgehash_haskey(ehash, med->v1, med->v2))
+			med->flag |= ME_SEAM_DONE;
 
-	/* Unbelievable! */
-	if (G.order==B_ENDIAN) {
-		SWITCH_INT(col);
+	BLI_edgehash_free(ehash, NULL);
+
+	/* alloc */
+	nedges = MEM_callocN(sizeof(*nedges)*me->totvert+1, "SeamPathNEdges");
+	edges = MEM_mallocN(sizeof(*edges)*me->totedge*2, "SeamPathEdges");
+	prevedge = MEM_mallocN(sizeof(*prevedge)*me->totedge, "SeamPathPrevious");
+	cost = MEM_mallocN(sizeof(*cost)*me->totedge, "SeamPathCost");
+
+	/* count edges, compute adjacent edges offsets and fill adjacent edges */
+	for (a=0, med=me->medge; a<me->totedge; a++, med++) {
+		nedges[med->v1+1]++;
+		nedges[med->v2+1]++;
 	}
-	/* Convert the color back to a face index */
-	index = framebuffer_to_index(col);
-	if (col==0 || index<=0 || index>me->totface) {
-		return -1;
+
+	for (a=1; a<me->totvert; a++) {
+		int newswap = nedges[a+1];
+		nedges[a+1] = nedgeswap + nedges[a];
+		nedgeswap = newswap;
 	}
-	return (index-1);
+	nedges[0] = nedges[1] = 0;
+
+	for (a=0, med=me->medge; a<me->totedge; a++, med++) {
+		edges[nedges[med->v1+1]++] = a;
+		edges[nedges[med->v2+1]++] = a;
+
+		cost[a] = 1e20f;
+		prevedge[a] = -1;
+	}
+
+	/* regular dijkstra shortest path, but over edges instead of vertices */
+	heap = BLI_heap_new();
+	BLI_heap_insert(heap, 0.0f, (void*)source);
+	cost[source] = 0.0f;
+
+	while (!BLI_heap_empty(heap)) {
+		mednum = (int)BLI_heap_popmin(heap);
+		med = me->medge + mednum;
+
+		if (mednum == target)
+			break;
+
+		if (med->flag & ME_SEAM_DONE)
+			continue;
+
+		med->flag |= ME_SEAM_DONE;
+
+		seam_add_adjacent(me, heap, mednum, med->v1, nedges, edges, prevedge, cost);
+		seam_add_adjacent(me, heap, mednum, med->v2, nedges, edges, prevedge, cost);
+	}
+	
+	MEM_freeN(nedges);
+	MEM_freeN(edges);
+	MEM_freeN(cost);
+	BLI_heap_free(heap, NULL);
+
+	for (a=0, med=me->medge; a<me->totedge; a++, med++)
+		med->flag &= ~ME_SEAM_DONE;
+
+	if (mednum != target) {
+		MEM_freeN(prevedge);
+		return 0;
+	}
+
+	/* follow path back to source and mark as seam */
+	if (mednum == target) {
+		short allseams = 1;
+
+		mednum = target;
+		do {
+			med = me->medge + mednum;
+			if (!(med->flag & ME_SEAM)) {
+				allseams = 0;
+				break;
+			}
+			mednum = prevedge[mednum];
+		} while (mednum != source);
+
+		mednum = target;
+		do {
+			med = me->medge + mednum;
+			if (allseams)
+				med->flag &= ~ME_SEAM;
+			else
+				med->flag |= ME_SEAM;
+			mednum = prevedge[mednum];
+		} while (mednum != -1);
+	}
+
+	MEM_freeN(prevedge);
+	return 1;
+}
+
+static void seam_select(Mesh *me, short *mval, short path)
+{
+	unsigned int index = 0;
+	MEdge *medge, *med;
+	int a, lastindex = -1;
+
+	if (!facesel_edge_pick(me, mval, &index))
+		return;
+
+	for (a=0, med=me->medge; a<me->totedge; a++, med++) {
+		if (med->flag & ME_SEAM_LAST) {
+			lastindex = a;
+			med->flag &= ~ME_SEAM_LAST;
+			break;
+		}
+	}
+
+	medge = me->medge + index;
+	if (!path || (lastindex == -1) || (index == lastindex) ||
+	    !seam_shortest_path(me, lastindex, index))
+		medge->flag ^= ME_SEAM;
+	medge->flag |= ME_SEAM_LAST;
+
+	G.f |= G_DRAWSEAMS;
+
+	if (G.rt == 8)
+		unwrap_lscm(1);
+
+	BIF_undo_push("Mark Seam");
+
+	object_tface_flags_changed(OBACT, 1);
+}
+
+void seam_edgehash_insert_face(EdgeHash *ehash, MFace *mf)
+{
+	BLI_edgehash_insert(ehash, mf->v1, mf->v2, NULL);
+	BLI_edgehash_insert(ehash, mf->v2, mf->v3, NULL);
+	if (mf->v4) {
+		BLI_edgehash_insert(ehash, mf->v3, mf->v4, NULL);
+		BLI_edgehash_insert(ehash, mf->v4, mf->v1, NULL);
+	}
+	else
+		BLI_edgehash_insert(ehash, mf->v3, mf->v1, NULL);
+}
+
+void seam_mark_clear_tface(short mode)
+{
+	Mesh *me;
+	TFace *tf;
+	MFace *mf;
+	MEdge *med;
+	int a;
+	
+	me= get_mesh(OBACT);
+	if(me==0 || me->tface==0 || me->totface==0) return;
+
+	if (mode == 0)
+		mode = pupmenu("Seams%t|Mark Border Seam %x1|Clear Seam %x2");
+
+	if (mode != 1 && mode != 2)
+		return;
+
+	if (mode == 2) {
+		EdgeHash *ehash = BLI_edgehash_new();
+
+		for (a=0, mf=me->mface, tf=me->tface; a<me->totface; a++, tf++, mf++)
+			if (!(tf->flag & TF_HIDE) && (tf->flag & TF_SELECT))
+				seam_edgehash_insert_face(ehash, mf);
+
+		for (a=0, med=me->medge; a<me->totedge; a++, med++)
+			if (BLI_edgehash_haskey(ehash, med->v1, med->v2))
+				med->flag &= ~ME_SEAM;
+
+		BLI_edgehash_free(ehash, NULL);
+	}
+	else {
+		/* mark edges that are on both selected and deselected faces */
+		EdgeHash *ehash1 = BLI_edgehash_new();
+		EdgeHash *ehash2 = BLI_edgehash_new();
+
+		for (a=0, mf=me->mface, tf=me->tface; a<me->totface; a++, tf++, mf++) {
+			if ((tf->flag & TF_HIDE) || !(tf->flag & TF_SELECT))
+				seam_edgehash_insert_face(ehash1, mf);
+			else
+				seam_edgehash_insert_face(ehash2, mf);
+		}
+
+		for (a=0, med=me->medge; a<me->totedge; a++, med++)
+			if (BLI_edgehash_haskey(ehash1, med->v1, med->v2) &&
+			    BLI_edgehash_haskey(ehash2, med->v1, med->v2))
+				med->flag |= ME_SEAM;
+
+		BLI_edgehash_free(ehash1, NULL);
+		BLI_edgehash_free(ehash2, NULL);
+	}
+
+	if (G.rt == 8)
+		unwrap_lscm(1);
+
+	G.f |= G_DRAWSEAMS;
+	BIF_undo_push("Mark Seam");
+
+	object_tface_flags_changed(OBACT, 1);
 }
 
 void face_select()
@@ -888,17 +1253,22 @@ void face_select()
 	TFace *tface, *tsel;
 	MFace *msel;
 	short mval[2];
-	int a, index;
+	unsigned int a, index;
 
 	/* Get the face under the cursor */
 	ob = OBACT;
 	if (!(ob->lay & G.vd->lay)) {
-		error("Active object not in this layer!");
+		error("The active object is not in this layer");
 	}
 	me = get_mesh(ob);
 	getmouseco_areawin(mval);
-	index = face_pick(me, mval[0], mval[1]);
-	if (index==-1) return;
+
+	if (G.qual & LR_ALTKEY) {
+		seam_select(me, mval, (G.qual & LR_SHIFTKEY) != 0);
+		return;
+	}
+
+	if (!facesel_face_pick(me, mval, &index, 1)) return;
 	
 	tsel= (((TFace*)me->tface)+index);
 	msel= (((MFace*)me->mface)+index);
@@ -926,12 +1296,11 @@ void face_select()
 	}
 	else tsel->flag |= TF_SELECT;
 	
-	lasttface = tsel;
-	
 	/* image window redraw */
-	allqueue(REDRAWIMAGE, 0);
-	allqueue(REDRAWBUTSEDIT, 0);
-	allqueue(REDRAWVIEW3D, 0);
+	
+	BIF_undo_push("Select UV face");
+
+	object_tface_flags_changed(OBACT, 1);
 }
 
 void face_borderselect()
@@ -939,7 +1308,8 @@ void face_borderselect()
 	Mesh *me;
 	TFace *tface;
 	rcti rect;
-	unsigned int *rectm, *rt;
+	struct ImBuf *ibuf;
+	unsigned int *rt;
 	int a, sx, sy, index, val;
 	char *selar;
 	
@@ -962,9 +1332,10 @@ void face_borderselect()
 		sy= (rect.ymax-rect.ymin+1);
 		if(sx*sy<=0) return;
 
-		rt=rectm= MEM_mallocN(sizeof(int)*sx*sy, "selrect");
-		glReadPixels(rect.xmin+curarea->winrct.xmin,  rect.ymin+curarea->winrct.ymin, sx, sy, GL_RGBA, GL_UNSIGNED_BYTE,  rectm);
-		if(G.order==B_ENDIAN) IMB_convert_rgba_to_abgr(sx*sy, rectm);
+		ibuf = IMB_allocImBuf(sx,sy,32,IB_rect,0);
+		rt = ibuf->rect;
+		glReadPixels(rect.xmin+curarea->winrct.xmin,  rect.ymin+curarea->winrct.ymin, sx, sy, GL_RGBA, GL_UNSIGNED_BYTE,  ibuf->rect);
+		if(G.order==B_ENDIAN) IMB_convert_rgba_to_abgr(ibuf);
 
 		a= sx*sy;
 		while(a--) {
@@ -986,72 +1357,36 @@ void face_borderselect()
 			}
 		}
 		
-		MEM_freeN(rectm);
+		IMB_freeImBuf(ibuf);
 		MEM_freeN(selar);
-		allqueue(REDRAWVIEW3D, 0);
-		allqueue(REDRAWIMAGE, 0);
+
+		BIF_undo_push("Border Select UV face");
+
+		object_tface_flags_changed(OBACT, 0);
 	}
 #ifdef __APPLE__	
 	glReadBuffer(GL_BACK);
 #endif
 }
 
-#define TEST_STRUBI 1				
-#ifdef TEST_STRUBI
-float CalcNormUV(float *a, float *b, float *c)
-{
-	float d1[3], d2[3];
-
-	d1[0] = a[0] - b[0];
-	d1[1] = a[1] - b[1];
-	d2[0] = b[0] - c[0];
-	d2[1] = b[1] - c[1];
-	return (d1[0] * d2[1] - d1[1] * d2[0]);
-}
-#endif
-
-
-/* Pupmenu codes: */
-#define UV_CUBE_MAPPING 2
-#define UV_CYL_MAPPING 3
-#define UV_SPHERE_MAPPING 4
-#define UV_BOUNDS8_MAPPING 68
-#define UV_BOUNDS4_MAPPING 65
-#define UV_BOUNDS2_MAPPING 66
-#define UV_BOUNDS1_MAPPING 67
-#define UV_STD8_MAPPING 131
-#define UV_STD4_MAPPING 130
-#define UV_STD2_MAPPING 129
-#define UV_STD1_MAPPING 128
-#define UV_WINDOW_MAPPING 5
-#define UV_CYL_EX 32
-#define UV_SPHERE_EX 34
-
-/* Some macro tricks to make pupmenu construction look nicer :-)
-   Sorry, just did it for fun. */
-
-#define _STR(x) " " #x
-#define STRING(x) _STR(x)
-
-#define MENUSTRING(string, code) string " %x" STRING(code)
-#define MENUTITLE(string) string " %t|" 
-
 void uv_autocalc_tface()
 {
 	short mode;
 	mode= pupmenu(MENUTITLE("UV Calculation")
-	              MENUSTRING("Cube",          UV_CUBE_MAPPING) "|"
-	              MENUSTRING("Cylinder",      UV_CYL_MAPPING) "|"
-	              MENUSTRING("Sphere",        UV_SPHERE_MAPPING) "|"
-	              MENUSTRING("Bounds to 1/8", UV_BOUNDS8_MAPPING) "|"
-	              MENUSTRING("Bounds to 1/4", UV_BOUNDS4_MAPPING) "|"
-	              MENUSTRING("Bounds to 1/2", UV_BOUNDS2_MAPPING) "|"
-	              MENUSTRING("Bounds to 1/1", UV_BOUNDS1_MAPPING) "|"
-	              MENUSTRING("Standard 1/8",  UV_STD8_MAPPING) "|"
-	              MENUSTRING("Standard 1/4",  UV_STD4_MAPPING) "|"
-	              MENUSTRING("Standard 1/2",  UV_STD2_MAPPING) "|"
-	              MENUSTRING("Standard 1/1",  UV_STD1_MAPPING) "|"
-	              MENUSTRING("From Window",   UV_WINDOW_MAPPING) );
+	              MENUSTRING("Cube Projection",          UV_CUBE_MAPPING) "|"
+	              MENUSTRING("Cylinder from View",      UV_CYL_MAPPING) "|"
+	              MENUSTRING("Sphere from View",        UV_SPHERE_MAPPING) "|"
+	              MENUSTRING("Unwrap",          UV_UNWRAP_MAPPING) "|"
+	              MENUSTRING("Project From View",   UV_WINDOW_MAPPING) "|"
+	              MENUSTRING("Project from View 1/1", UV_BOUNDS1_MAPPING) "|"
+	              MENUSTRING("Project from View 1/2", UV_BOUNDS2_MAPPING) "|"
+	              MENUSTRING("Project from View 1/4", UV_BOUNDS4_MAPPING) "|"
+	              MENUSTRING("Project from View 1/8", UV_BOUNDS8_MAPPING) "|"
+	              MENUSTRING("Reset 1/1",  UV_STD1_MAPPING) "|"
+	              MENUSTRING("Reset 1/2",  UV_STD2_MAPPING) "|"
+	              MENUSTRING("Reset 1/4",  UV_STD4_MAPPING) "|"
+	              MENUSTRING("Reset 1/8",  UV_STD8_MAPPING) );
+	              
 	
 	
 	switch(mode) {
@@ -1079,6 +1414,8 @@ void uv_autocalc_tface()
 		calculate_uv_map(B_UVAUTO_STD1); break;
 	case UV_WINDOW_MAPPING:
 		calculate_uv_map(B_UVAUTO_WINDOW); break;
+	case UV_UNWRAP_MAPPING:
+		unwrap_lscm(0); break;
 	}
 }
 
@@ -1087,35 +1424,78 @@ void set_faceselect()	/* toggle */
 	Object *ob = OBACT;
 	Mesh *me = 0;
 	
-	scrarea_queue_headredraw(curarea);
-
-	if(G.f & G_FACESELECT) G.f &= ~G_FACESELECT;
-	else {
-		if (ob && ob->type == OB_MESH) G.f |= G_FACESELECT;
+	if(ob==NULL) return;
+	if(ob->id.lib) {
+		error("Can't edit library data");
+		return;
 	}
+	
+	me= get_mesh(ob);
+	if(me && me->id.lib) {
+		error("Can't edit library data");
+		return;
+	}
+	
+	scrarea_queue_headredraw(curarea);
+	
+	if(me) /* make sure modifiers are updated for mapping requirements */
+		DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+
+	if(G.f & G_FACESELECT) {
+		G.f &= ~G_FACESELECT;
+
+		if((G.f & (G_WEIGHTPAINT|G_VERTEXPAINT|G_TEXTUREPAINT))==0) {
+			if(me)
+				reveal_tface();
+			setcursor_space(SPACE_VIEW3D, CURSOR_STD);
+			BIF_undo_push("End UV Faceselect");
+		}
+	}
+	else if (me && (ob->lay & G.vd->lay)) {
+		G.f |= G_FACESELECT;
+		if(me->tface==NULL)
+			make_tfaces(me);
+
+		setcursor_space(SPACE_VIEW3D, CURSOR_FACESEL);
+		BIF_undo_push("Set UV Faceselect");
+	}
+
+	countall();
 
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWIMAGE, 0);
-	
-	ob= OBACT;
-	me= get_mesh(ob);
-	if(me && me->tface==NULL) make_tfaces(me);
-
-	if(G.f & G_FACESELECT) {
-		setcursor_space(SPACE_VIEW3D, CURSOR_FACESEL);
-		if(me) set_lasttface();
-	}
-	else if((G.f & (G_WEIGHTPAINT|G_VERTEXPAINT|G_TEXTUREPAINT))==0) {
-		if(me) reveal_tface();
-		setcursor_space(SPACE_VIEW3D, CURSOR_STD);
-		makeDispList(ob);
-	}
-	countall();
 }
 
+void set_texturepaint() /* toggle */
+{
+	Object *ob = OBACT;
+	Mesh *me = 0;
+	
+	scrarea_queue_headredraw(curarea);
+	if(ob==NULL) return;
+	if(ob->id.lib) {
+		error("Can't edit library data");
+		return;
+	}
+
+	me= get_mesh(ob);
+	if(me && me->id.lib) {
+		error("Can't edit library data");
+		return;
+	}
+	
+	if(me)
+		DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+
+	if(G.f & G_TEXTUREPAINT)
+		G.f &= ~G_TEXTUREPAINT;
+	else if (me)
+		G.f |= G_TEXTUREPAINT;
+
+	allqueue(REDRAWVIEW3D, 0);
+}
 
-#ifdef NAN_TPT
 /**
  * Get the view ray through the screen point.
  * Uses the OpenGL settings of the active view port.
@@ -1126,7 +1506,7 @@ void set_faceselect()	/* toggle */
  * @param	org		origin of the view ray.
  * @param	dir		direction of the view ray.
  */
-void get_pick_ray(short x, short y, float org[3], float dir[3])
+static void get_pick_ray(short x, short y, float org[3], float dir[3])
 {
 	double mvmatrix[16];
 	double projmatrix[16];
@@ -1162,7 +1542,7 @@ void get_pick_ray(short x, short y, float org[3], float dir[3])
 }
 
 
-int triangle_ray_intersect(float tv0[3], float tv1[3], float tv2[3], float org[3], float dir[3], float uv[2])
+static int triangle_ray_intersect(float tv0[3], float tv1[3], float tv2[3], float org[3], float dir[3], float uv[2])
 {
 	float v1v0[3];
 	float v2v0[3];
@@ -1255,7 +1635,7 @@ int triangle_ray_intersect(float tv0[3], float tv1[3], float tv2[3], float org[3
  * @param	v4		vertex 4 coordinates.
  * @return	number of vertices of this face
  */
-int face_get_vertex_coordinates(Mesh* mesh, TFace* face, float v1[3], float v2[3], float v3[3], float v4[3])
+static int face_get_vertex_coordinates(Mesh* mesh, TFace* face, float v1[3], float v2[3], float v3[3], float v4[3])
 {
 	int num_vertices;
 	MVert *mv;
@@ -1288,7 +1668,7 @@ int face_get_vertex_coordinates(Mesh* mesh, TFace* face, float v1[3], float v2[3
  * @param	u		(u,v) coordinate.
  * @param	v		(u,v) coordinate.
  */
-void face_get_uv(TFace* face, int v1, int v2, int v3, float a, float b, float* u, float* v)
+static void face_get_uv(TFace* face, int v1, int v2, int v3, float a, float b, float* u, float* v)
 {
 	float uv01[2], uv21[2];
 
@@ -1331,7 +1711,7 @@ void face_get_uv(TFace* face, int v1, int v2, int v3, float a, float b, float* u
  *			0 == no intersection, (u,v) invalid
  *			1 == intersection, (u,v) valid
  */
-int face_pick_uv(Object* object, Mesh* mesh, TFace* face, short x, short y, float* u, float* v)
+static int face_pick_uv(Object* object, Mesh* mesh, TFace* face, short x, short y, float* u, float* v)
 {
 	float org[3], dir[3];
 	float ab[2];
@@ -1384,39 +1764,42 @@ void face_draw()
 	Image *img=NULL, *img_old = NULL;
 	IMG_BrushPtr brush;
 	IMG_CanvasPtr canvas = 0;
-	int rowBytes, face_index;
+	unsigned int rowBytes, face_index;
 	char *warn_packed_file = 0;
 	float uv[2], uv_old[2];
 	extern VPaint Gvp;
+	short mousebut;
 
 	ob = OBACT;
 	if (!ob) {
 		error("No active object"); return;
 	}
 	if (!(ob->lay & G.vd->lay)) {
-		error("Active object not in this layer"); return;
+		error("The active object is not in this layer"); return;
 	}
 	me = get_mesh(ob);
 	if (!me) {
-		error("Active object does not have a mesh"); return;
+		error("The active object does not have a mesh obData"); return;
 	}
 
-	brush = IMG_BrushCreate(Gvp.size, Gvp.size, Gvp.r, Gvp.g, Gvp.b, Gvp.a);
+	brush = IMG_BrushCreate(Gvp.size, Gvp.size, &Gvp.r);
 	if (!brush) {
-		error("Can not create brush"); return;
+		error("Can't create brush"); return;
 	}
 
+	if (U.flag & USER_LMOUSESELECT) mousebut = R_MOUSE;
+	else mousebut = L_MOUSE;
+
 	persp(PERSP_VIEW);
 
 	getmouseco_areawin(xy_old);
-	while (get_mbut() & L_MOUSE) {
+	while (get_mbut() & mousebut) {
 		getmouseco_areawin(xy);
 		/* Check if cursor has moved */
 		if ((xy[0] != xy_old[0]) || (xy[1] != xy_old[1])) {
 
 			/* Get face to draw on */
-			face_index = face_pick(me, xy[0], xy[1]);
-			if (face_index == -1) face = NULL;
+			if (!facesel_face_pick(me, xy, &face_index, 0)) face = NULL;
 			else face = (((TFace*)me->tface)+face_index);
 
 			/* Check if this is another face. */
@@ -1516,11 +1899,12 @@ void face_draw()
 	}
 
 	if (warn_packed_file) {
-		error("Painting in packed images not supported: %s", warn_packed_file);
+		error("Painting in packed images is not supported: %s", warn_packed_file);
 	}
 
 	persp(PERSP_WIN);
 
+	BIF_undo_push("UV face draw");
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWIMAGE, 0);
 	allqueue(REDRAWHEADERS, 0);
@@ -1543,7 +1927,7 @@ void get_same_uv(void)
 	
 	ob = OBACT;
 	if (!(ob->lay & G.vd->lay)) {
-		error("Active object not in this layer!");
+		error("The active object is not in this layer");
 		return;
 	}
 	me = get_mesh(ob);
@@ -1565,7 +1949,7 @@ void get_same_uv(void)
 	}		
 	
 	if(!foundtex) {
-		error("No active face or active face has no UV-texture");
+		error("No active face, or active face has no UV texture");
 		return;
 	}
 
@@ -1584,10 +1968,9 @@ void get_same_uv(void)
 		tface++;
 	}
 	
-
-	
 	/* image window redraw */
-	allqueue(REDRAWIMAGE, 0);
-	allqueue(REDRAWVIEW3D, 0);
+	BIF_undo_push("Get same UV");
+
+	object_tface_flags_changed(OBACT, 0);
 }
-#endif /* NAN_TPT */
+
