@@ -1,5 +1,5 @@
 /**
- * $Id: editlattice.c,v 1.11 2005/03/19 21:08:12 zuster Exp $
+ * $Id: editlattice.c,v 1.18 2006/06/27 05:56:56 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,24 +44,24 @@
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
-#include "DNA_object_types.h"
-#include "DNA_scene_types.h"
-#include "DNA_lattice_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_key_types.h"
+#include "DNA_lattice_types.h"
+#include "DNA_object_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 
-#include "BKE_key.h"
-#include "BKE_displist.h"
-#include "BKE_lattice.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
+#include "BKE_key.h"
+#include "BKE_lattice.h"
 #include "BKE_utildefines.h"
 
-#include "BIF_space.h"
-#include "BIF_screen.h"
 #include "BIF_editlattice.h"
 #include "BIF_editmode_undo.h"
 #include "BIF_editkey.h"
+#include "BIF_space.h"
+#include "BIF_screen.h"
 #include "BIF_toolbox.h"
 
 #include "BSE_edit.h"
@@ -108,22 +108,15 @@ static void setflagsLatt(int flag)
 void make_editLatt(void)
 {
 	Lattice *lt;
-	KeyBlock *actkey=0;
+	KeyBlock *actkey;
 	
 	free_editLatt();
 	
 	lt= G.obedit->data;
 
-	/* keys? */
-	if(lt->key) {
-		actkey= lt->key->block.first;
-		while(actkey) {
-			if(actkey->flag & SELECT) break;
-			actkey= actkey->next;
-		}
-	}
-
+	actkey = ob_get_keyblock(G.obedit);
 	if(actkey) {
+		strcpy(G.editModeTitleExtra, "(Key) ");
 		key_to_latt(actkey, lt);
 	}
 
@@ -138,22 +131,14 @@ void make_editLatt(void)
 void load_editLatt(void)
 {
 	Lattice *lt;
-	KeyBlock *actkey=0;
+	KeyBlock *actkey;
 	BPoint *bp;
 	float *fp;
 	int tot;
 	
 	lt= G.obedit->data;
 	
-	/* are there keys? */
-	if(lt->key) {
-		actkey= lt->key->block.first;
-		while(actkey) {
-			if(actkey->flag & SELECT) break;
-			actkey= actkey->next;
-		}
-	}
-
+	actkey = ob_get_keyblock(G.obedit);
 	if(actkey) {
 		/* active key: vertices */
 		tot= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
@@ -169,8 +154,6 @@ void load_editLatt(void)
 			fp+= 3;
 			bp++;
 		}
-		
-		if(actkey) do_spec_key(lt->key);
 	}
 	else {
 
@@ -217,6 +200,7 @@ void deselectall_Latt(void)
 		if(bp->hide==0) {
 			if(bp->f1) {
 				setflagsLatt(0);
+				countall();
 				BIF_undo_push("(De)select all");
 				return;
 			}
@@ -224,43 +208,36 @@ void deselectall_Latt(void)
 		bp++;
 	}
 	setflagsLatt(1);
-
+	countall();
 	BIF_undo_push("(De)select all");
 }
 
+static void findnearestLattvert__doClosest(void *userData, BPoint *bp, int x, int y)
+{
+	struct { BPoint *bp; short dist, select, mval[2]; } *data = userData;
+	float temp = abs(data->mval[0]-x) + abs(data->mval[1]-y);
+	
+	if ((bp->f1&1)==data->select) temp += 5;
+	if (temp<data->dist) {
+		data->dist = temp;
 
+		data->bp = bp;
+	}
+}
 static BPoint *findnearestLattvert(int sel)
 {
-	/* sel==1: selected get a disadvantage */
-	/* in bp nearest is written */
-	BPoint *bp1, *bp;
-	short dist= 100, temp, mval[2], a;
+		/* sel==1: selected gets a disadvantage */
+		/* in nurb and bezt or bp the nearest is written */
+		/* return 0 1 2: handlepunt */
+	struct { BPoint *bp; short dist, select, mval[2]; } data = {0};
 
-	bp= 0;
+	data.dist = 100;
+	data.select = sel;
+	getmouseco_areawin(data.mval);
 
-	/* do projection */
-	calc_lattverts_ext();	/* drawobject.c */
-	
-	getmouseco_areawin(mval);
+	lattice_foreachScreenVert(findnearestLattvert__doClosest, &data);
 
-			
-	bp1= editLatt->def;
-	
-	a= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
-	
-	while(a--) {
-		if(bp1->hide==0) {
-			temp= abs(mval[0]- bp1->s[0])+ abs(mval[1]- bp1->s[1]);
-			if( (bp1->f1 & 1)==sel) temp+=5;
-			if(temp<dist) { 
-				bp= bp1; 
-				dist= temp; 
-			}
-		}
-		bp1++;
-	}
-
-	return bp;
+	return data.bp;
 }
 
 
@@ -300,21 +277,9 @@ void mouse_lattice(void)
 
 static void undoLatt_to_editLatt(void *defv)
 {
-	Base *base;
 	int a= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
 
 	memcpy(editLatt->def, defv, a*sizeof(BPoint));
-	
-	base= FIRSTBASE;
-	while(base) {
-		if(base->lay & G.vd->lay) {
-			if(base->object->parent==G.obedit) {
-				makeDispList(base->object);
-			}
-		}
-		base= base->next;
-	}
-	allqueue(REDRAWVIEW3D, 0);
 }
 
 static void *editLatt_to_undoLatt(void)
