@@ -1,5 +1,5 @@
 /**
- * $Id: editscreen.c,v 1.101 2005/05/24 19:33:04 ton Exp $
+ * $Id: editscreen.c,v 1.135 2006/07/06 12:50:48 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -58,6 +58,7 @@
 
 #include "DNA_action_types.h"
 #include "DNA_object_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_sound_types.h"
@@ -73,13 +74,17 @@
 #include "BKE_blender.h"
 #include "BKE_screen.h"
 
+#include "BIF_cursors.h"
+#include "BIF_drawscene.h"
 #include "BIF_editsound.h"
 #include "BIF_glutil.h"
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
 #include "BIF_interface.h"
+#include "BIF_interface_icons.h"
 #include "BIF_mainqueue.h"
 #include "BIF_mywindow.h"
+#include "BIF_previewrender.h"
 #include "BIF_renderwin.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
@@ -98,7 +103,6 @@
 #include "BPY_extern.h"
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"		/* R.flag */
 
 #include "winlay.h"
 
@@ -110,6 +114,8 @@
 	 always reset to zero.
  */
 
+/* comment added to test orange branch */
+
 static void testareas(void);
 static void area_autoplayscreen(void);
 static void wait_for_event(void);
@@ -133,6 +139,8 @@ static void drawscredge_area(ScrArea *sa);
 
 /**********************************************************************/
 
+extern int textediting;
+
 static void screen_set_cursor(bScreen *sc) 
 {
 	if (sc->winakt>3) {
@@ -146,10 +154,12 @@ static void screen_set_cursor(bScreen *sc)
 
 void waitcursor(int val)
 {
-	if(val) {
-		set_cursor(CURSOR_WAIT);
-	} else {
-		screen_set_cursor(G.curscreen);
+	if(G.curscreen) {
+		if(val) {
+			set_cursor(CURSOR_WAIT);
+		} else {
+			screen_set_cursor(G.curscreen);
+		}
 	}
 }
 
@@ -161,7 +171,11 @@ static int choose_cursor(ScrArea *sa)
 		else if(G.f & G_WEIGHTPAINT) return CURSOR_VPAINT;
 		else if(G.f & G_FACESELECT) return CURSOR_FACESEL;
 		else return CURSOR_STD;
-	} else {
+	}
+	else if (sa->spacetype==SPACE_TEXT) {
+		return CURSOR_TEXTEDIT;
+	}
+	else {
 		return CURSOR_STD;
 	}
 }
@@ -169,8 +183,10 @@ static int choose_cursor(ScrArea *sa)
 void wich_cursor(ScrArea *sa)
 {
 	sa->cursor= choose_cursor(sa);
-
-	screen_set_cursor(G.curscreen);
+	
+	/* well... the waitcursor() is not a state, so this call will cancel it out */
+	if(get_cursor()!=CURSOR_WAIT)
+		screen_set_cursor(G.curscreen);
 }
 
 
@@ -228,7 +244,8 @@ void headerprint(char *str)
 		areawinset(curarea->headwin);
 		
 		headerbox(curarea);
-		cpack(0x0);
+		
+		BIF_ThemeColor(TH_MENU_TEXT); /* better than cpack(0x0) color no? (desoto) */
 		glRasterPos2i(20+curarea->headbutofs,  6);
 		BMF_DrawString(G.font, str);
 		
@@ -322,10 +339,16 @@ void areawinset(short win)
 			G.v2d= &G.snla->v2d;
 			break;
 		case SPACE_TIME:
-			{
+		{
 			SpaceTime *stime= curarea->spacedata.first;
 			G.v2d= &stime->v2d;
-			}
+		}
+			break;
+		case SPACE_NODE:
+		{
+			SpaceNode *snode= curarea->spacedata.first;
+			G.v2d= &snode->v2d;
+		}
 			break;
 		default:
 			break;
@@ -395,7 +418,9 @@ void scrarea_do_headdraw(ScrArea *area)
 		case SPACE_ACTION:	action_buttons();	break;
 		case SPACE_NLA:		nla_buttons();		break;
 		case SPACE_TIME:	time_buttons(area);	break;
+		case SPACE_NODE:	node_buttons(area);	break;
 		}
+		uiClearButLock();
 
 		//glScissor(area->winrct.xmin, area->winrct.xmax, area->winx, area->winy);
 		area->head_swap= WIN_BACK_OK;
@@ -641,7 +666,6 @@ int is_allowed_to_change_screen(bScreen *new)
 
 void splash(void *data, int datasize, char *string)
 {
-	extern void uiDrawBoxShadow(unsigned char alpha, float minx, float miny, float maxx, float maxy);
 	ImBuf *bbuf;
 	int oldwin;
 	short val;
@@ -712,7 +736,7 @@ void splash(void *data, int datasize, char *string)
 }
 
 static void moveareas(ScrEdge *edge);
-static void joinarea(ScrArea *sa, ScrEdge *onedge);
+static void joinarea_interactive(ScrArea *area, ScrEdge *onedge);
 static void splitarea_interactive(ScrArea *area, ScrEdge *onedge);
 
 static void screen_edge_edit_event(ScrArea *actarea, ScrEdge *actedge, short evt, short val) 
@@ -737,7 +761,7 @@ static void screen_edge_edit_event(ScrArea *actarea, ScrEdge *actedge, short evt
 			if (edgeop==1) {
 				splitarea_interactive(actarea, actedge);
 			} else if (edgeop==2) {
-				joinarea(actarea, actedge);
+				joinarea_interactive(actarea, actedge);
 			} else if (edgeop==3) {
 				scrarea_change_headertype(actarea, actarea->headertype?0:HEADERDOWN);
 			}
@@ -787,6 +811,15 @@ static int nafterqitems= 0;
 void addafterqueue(short win, unsigned short evt, short val)
 {
 	if (nafterqitems<MAXQUEUE) {
+		int a;
+		
+		/* only one afterqueue event of each type */
+		for(a=0; a<nafterqitems; a++) {
+			if(afterqueue[a][0]==win && afterqueue[a][1]==evt) {
+				afterqueue[a][2]= val;
+				return;
+			}
+		}
 		afterqueue[nafterqitems][0]= win;
 		afterqueue[nafterqitems][1]= evt;
 		afterqueue[nafterqitems][2]= val;
@@ -827,7 +860,7 @@ void add_readfile_event(char *filename)
 	BLI_convertstringcode(ext_load_str, G.sce, G.scene->r.cfra);
 }
 
-static short ext_reshape= 0, ext_redraw=0, ext_inputchange=0, ext_mousemove=0;
+static short ext_reshape= 0, ext_redraw=0, ext_inputchange=0, ext_mousemove=0, ext_undopush=0;
 
 static void flush_extqd_events(void) {
 	if (ext_inputchange) {
@@ -836,6 +869,8 @@ static void flush_extqd_events(void) {
 		mainqenter(RESHAPE, ext_redraw);
 	} else if (ext_redraw) {
 		mainqenter(REDRAW, ext_redraw);
+	} else if (ext_undopush) {
+		mainqenter(UNDOPUSH, ext_undopush);
 	} else if (ext_mousemove) {
 		short mouse[2];
 		
@@ -845,10 +880,10 @@ static void flush_extqd_events(void) {
 		mainqenter(MOUSEY, mouse[1]);
 	}
 	
-	ext_inputchange= ext_reshape= ext_redraw= ext_mousemove= 0;
+	ext_inputchange= ext_reshape= ext_redraw= ext_mousemove= ext_undopush= 0;
 }
 
-unsigned short qtest(void)
+int qtest(void)
 {
 	if (!mainqtest()) {
 		winlay_process_events(0);
@@ -915,10 +950,14 @@ unsigned short extern_qread_ext(short *val, char *ascii)
 	event= screen_qread(val, ascii);
 	if(event==RESHAPE) ext_reshape= *val;
 	else if(event==REDRAW) ext_redraw= *val;
+	else if(event==UNDOPUSH) ext_undopush= *val;
 	else if(event==INPUTCHANGE) ext_inputchange= *val;
 	else if(event==MOUSEY || event==MOUSEX) ext_mousemove= 1;
 	else if((G.qual & (LR_CTRLKEY|LR_ALTKEY)) && event==F3KEY) {
-		if(*val) BIF_screendump(0);
+		if(*val) {
+			BIF_screendump(0);
+			return ESCKEY;	/* go out of menu, if that was set */
+		}
 	}
 
 	return event;
@@ -1043,6 +1082,7 @@ static void animated_screen(bScreen *sc, short val)
 	}
 	if(val & TIME_ALL_ANIM_WIN) allqueue(REDRAWANIM, 0);
 	if(val & TIME_ALL_BUTS_WIN) allqueue(REDRAWBUTSALL, 0);
+	if(val & TIME_SEQ) allqueue(REDRAWSEQ, 0);
 	
 	allqueue(REDRAWTIME, 0);
 }
@@ -1098,6 +1138,33 @@ int do_screenhandlers(bScreen *sc)
 
 /* ****** end screen handlers ************ */
 
+static void drawscreen(void)
+{
+	ScrArea *sa;
+	
+	mywinset(G.curscreen->mainwin);
+	myortho2(-0.375, (float)G.curscreen->sizex-0.375, -0.375, (float)G.curscreen->sizey-0.375);
+	
+	sa= G.curscreen->areabase.first;
+	while(sa) {
+		drawscredge_area(sa);
+		sa= sa->next;
+	}
+	
+	/* this double draw patch seems to be needed for certain sgi's (octane, indigo2) */
+#if defined(__sgi) || defined(__sun__) || defined( __sun ) || defined (__sparc) || defined (__sparc__)
+	glDrawBuffer(GL_FRONT);
+	
+	sa= G.curscreen->areabase.first;
+	while(sa) {
+		drawscredge_area(sa);
+		sa= sa->next;
+	}
+	
+	glDrawBuffer(GL_BACK);
+#endif
+}
+
 static void screen_dispatch_events(void) {
 	int events_remaining= 1;
 	ScrArea *sa;
@@ -1126,13 +1193,17 @@ static void screen_dispatch_events(void) {
 		if (winqueue_break) break;
 	}
 
-	if (dodrawscreen) {
-		drawscreen();
-		dodrawscreen= 0;
+	/* winqueue_break isnt the best of all solutions... but it is called on switching screens,
+		so drawing should wait for all redraw/init events to be handled */
+	if (winqueue_break==0) {
+		if (dodrawscreen) {
+			drawscreen();
+			dodrawscreen= 0;
+		}
+		
+		screen_swapbuffers();
+		do_screenhandlers(G.curscreen);
 	}
-	
-	screen_swapbuffers();
-	do_screenhandlers(G.curscreen);
 }
 
 static ScrArea *screen_find_area_for_pt(bScreen *sc, short *mval) 
@@ -1206,6 +1277,7 @@ void screenmain(void)
 			if (newactarea) {
 				if (BLI_in_rcti(&newactarea->headrct, mval[0], mval[1])) {
 					newactwin= newactarea->headwin;
+					set_cursor(CURSOR_STD);
 				} else {
 					newactwin= newactarea->win;
 				}
@@ -1216,7 +1288,8 @@ void screenmain(void)
 			if (newactarea && (newactarea != g_activearea)) {
 				if (g_activearea) scrarea_queue_headredraw(g_activearea);
 				scrarea_queue_headredraw(newactarea);
-				set_cursor(newactarea->cursor);
+				if (!(BLI_in_rcti(&newactarea->headrct, mval[0], mval[1]))) /* header always gets std cursor */
+					set_cursor(newactarea->cursor);
 				g_activearea= newactarea;
 			}
 			/* when you move mouse from header to window, buttons can remain hilited otherwise */
@@ -1227,7 +1300,8 @@ void screenmain(void)
 			
 			if (G.curscreen->winakt) {
 				areawinset(G.curscreen->winakt);
-				set_cursor(choose_cursor(g_activearea));
+				if (!(BLI_in_rcti(&newactarea->headrct, mval[0], mval[1]))) /* header always gets std cursor */
+					set_cursor(choose_cursor(g_activearea));
 			}
 		} 
 		else {
@@ -1286,7 +1360,7 @@ void screenmain(void)
 			}
 		}
 		else if(ELEM(event, LEFTARROWKEY, RIGHTARROWKEY)) {
-			if(val && (G.qual & LR_CTRLKEY)) {
+			if(textediting==0 && val && (G.qual & LR_CTRLKEY)) {
 				bScreen *sc= (event==LEFTARROWKEY)?G.curscreen->id.prev:G.curscreen->id.next;
 				if(is_allowed_to_change_screen(sc)) setscreen(sc);
 				g_activearea= NULL;
@@ -1337,10 +1411,20 @@ void screenmain(void)
 			}
 		}
 		else if (event==SPACEKEY) {
-			if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT||g_activearea->spacetype==SPACE_SCRIPT);
-			else if(G.qual==0) {
-				if(val) toolbox_n();
-				towin= 0;
+			if ((g_activearea->spacetype!=SPACE_TEXT) &&
+				( !((g_activearea->spacetype==SPACE_VIEW3D) && ((G.obedit) && G.obedit->type==OB_FONT)) ) &&
+				val && 
+				(G.qual & LR_SHIFTKEY)) {
+					area_fullscreen();
+					g_activearea= NULL;
+					towin= 0;
+			}
+			else {
+				if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT||g_activearea->spacetype==SPACE_SCRIPT||g_activearea->spacetype==SPACE_SEQ);
+				else if(G.qual==0) {
+					if(val) toolbox_n();
+					towin= 0;
+				}
 			}
 		}
 		else if(ELEM(event, UPARROWKEY, DOWNARROWKEY)) {
@@ -1381,7 +1465,7 @@ void screenmain(void)
 				// SET AUTOPLAY in G.flags for
 				// other fileloads
 
-				G.flags |= G_FLAGS_AUTOPLAY;
+				G.flags |= G_FILE_AUTOPLAY;
 				area_autoplayscreen();
 
 				// Let The Games Begin
@@ -1405,7 +1489,8 @@ void screenmain(void)
 
 #if 0
 //#ifdef _WIN32	// FULLSCREEN
-void mainwindow_toggle_fullscreen(int fullscreen){
+void mainwindow_toggle_fullscreen(int fullscreen)
+{
 	if (fullscreen) U.uiflag |= USER_FLIPFULLSCREEN;
 	else U.uiflag &= ~USER_FLIPFULLSCREEN;
 
@@ -1413,20 +1498,27 @@ void mainwindow_toggle_fullscreen(int fullscreen){
 }
 #endif
 
-void mainwindow_raise(void) {
-	window_raise(mainwin);
+void mainwindow_raise(void) 
+{
+	if(mainwin)
+		window_raise(mainwin);
 }
 
-void mainwindow_make_active(void) {
-	window_make_active(mainwin);
+void mainwindow_make_active(void) 
+{
+	if(mainwin)
+		window_make_active(mainwin);
 }
 
-void mainwindow_close(void) {
-	window_destroy(mainwin);
+void mainwindow_close(void) 
+{
+	if(mainwin)
+		window_destroy(mainwin);
 	mainwin= NULL;
 }
 
-void mainwindow_set_filename_to_title(char *filename) {
+void mainwindow_set_filename_to_title(char *filename)
+{
 	char str[FILE_MAXDIR + FILE_MAXFILE];
 	char dir[FILE_MAXDIR];
 	char file[FILE_MAXFILE];
@@ -1449,24 +1541,13 @@ void setprefsize(int stax, int stay, int sizx, int sizy)
 	
 	winlay_get_screensize(&scrwidth, &scrheight);
 		
-	if(stax<0) stax= 0;
-	if(stay<0) stay= 0;
 	if(sizx<320) sizx= 320;
 	if(sizy<256) sizy= 256;
 
-	if(stax+sizx>scrwidth) {
-		fprintf(stderr," width prob \n");
-		sizx= scrwidth-stax;
-		}
-
 	if(stay+sizy>scrheight) {
 		fprintf(stderr," height prob \n");
 		sizy= scrheight-stay;
-		}
-
-/* #ifdef __APPLE__ */
-/*	if(stay+sizy>scrheight-22) sizy= scrheight-22-stay;	// 22 = topbar  */
-/* #endif*/
+	}
 
 	if(sizx<320 || sizy<256) {
 		printf("ERROR: illegal prefsize\n");
@@ -1496,7 +1577,7 @@ static void sortscrvert(ScrVert **v1, ScrVert **v2)
 {
 	ScrVert *tmp;
 	
-	if ((long)*v1 > (long)*v2) {
+	if (*v1 > *v2) {
 		tmp= *v1;
 		*v1= *v2;
 		*v2= tmp;	
@@ -1705,7 +1786,9 @@ static void openheadwin(ScrArea *sa)
 
 	glMatrixMode(GL_MODELVIEW);
 	
-	areawinar[sa->headwin]= sa;	/* oterwise addqueue does not work */
+	areawinar[sa->headwin]= sa;	/* otherwise addqueue does not work */
+	
+	scrarea_do_headchange(sa);	/* headchange is no callback, apply right away. this is for render-to-imagewindow... this can be called on startup by sequencer, which invokes redraw before all events are handled. bad stuff... */
 	addqueue(sa->headwin, CHANGED, 1);
 }
 
@@ -1737,7 +1820,7 @@ static void del_area(ScrArea *sa)
 	closeareawin(sa);
 	closeheadwin(sa);
 
-	freespacelist(&sa->spacedata);
+	freespacelist(sa);
 	
 	uiFreeBlocks(&sa->uiblocks);
 	uiFreePanels(&sa->panels);
@@ -1756,7 +1839,7 @@ static void copy_areadata(ScrArea *sa1, ScrArea *sa2)
 	sa1->spacetype= sa2->spacetype;
 	Mat4CpyMat4(sa1->winmat, sa2->winmat);
 
-	freespacelist(&sa1->spacedata);
+	freespacelist(sa1);
 	duplicatespacelist(sa1, &sa1->spacedata, &sa2->spacedata);
 
 	BLI_freelistN(&sa1->panels);
@@ -1824,8 +1907,7 @@ static void testareas(void)
 	/* test for header, if removed, or moved */
 	/* test for window, if removed, or moved */
 	
-	sa= G.curscreen->areabase.first;
-	while(sa) {
+	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
 		rcti oldhr= sa->headrct;
 		rcti oldwr= sa->winrct;
 		
@@ -1861,9 +1943,12 @@ static void testareas(void)
 			if (!rcti_eq(&oldwr, &sa->winrct)) {
 				mywinposition(sa->win, sa->winrct.xmin, sa->winrct.xmax, sa->winrct.ymin, sa->winrct.ymax);
 				addqueue(sa->win, CHANGED, 1);
+				
+				/* exception handling... probably we need generic event */
+				if(sa->spacetype==SPACE_VIEW3D)
+					BIF_view3d_previewrender_free(sa->spacedata.first);
 			}
 		}
-		sa= next;
 	}
 	
 		/* remake global windowarray */
@@ -2153,7 +2238,7 @@ void setscreen(bScreen *sc)
 	
 	for(sa= sc->areabase.first; sa; sa= sa->next) {
 		SpaceLink *sl;
-
+		
 		for(sl= sa->spacedata.first; sl; sl= sl->next) {
 			sl->area= sa;
 
@@ -2174,7 +2259,9 @@ void setscreen(bScreen *sc)
 		sa->cursor= CURSOR_STD;
 	}
 	
-	G.scene= sc->scene;
+	if(G.scene!=sc->scene)
+		set_scene(sc->scene);
+
 	countall();
 	
 	G.curscreen->winakt= 0;
@@ -2192,7 +2279,7 @@ void area_fullscreen(void)	/* with curarea */
 {
 	/* this function toggles: if area is full then the parent will be restored */
 	bScreen *sc, *oldscreen;
-	ScrArea *newa, *old;
+	ScrArea *sa, *newa, *old;
 	short headertype, fulltype;
 	
 	if(curarea->full) {
@@ -2202,7 +2289,7 @@ void area_fullscreen(void)	/* with curarea */
 		// refuse to go out of SCREENAUTOPLAY as long as G_FLAGS_AUTOPLAY
 		// is set
 
-		if (fulltype != SCREENAUTOPLAY || (G.flags & G_FLAGS_AUTOPLAY) == 0) {
+		if (fulltype != SCREENAUTOPLAY || (G.flags & G_FILE_AUTOPLAY) == 0) {
 			sc->full= 0;
 		
 			/* find old area */
@@ -2263,6 +2350,15 @@ void area_fullscreen(void)	/* with curarea */
 		setscreen(sc);
 		wich_cursor(newa);
 	}
+
+	/* there's also events in queue for this, but we call fullscreen for render output
+	now, and that doesn't go back to queue. Bad code, but doesn't hurt... (ton) */
+	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+		scrarea_do_headchange(sa);
+		scrarea_do_winchange(sa);
+	}
+	/* bad code #2: setscreen() ends with first area active. fullscreen render assumes this too */
+	curarea= sc->areabase.first;
 }
 
 static void area_autoplayscreen(void)
@@ -2386,36 +2482,213 @@ void duplicate_screen(void)
 /* ************ END SCREEN MANAGEMENT ************** */
 /* ************  JOIN/SPLIT/MOVE ************** */
 
-static void joinarea(ScrArea *sa, ScrEdge *onedge)
+typedef struct point{
+	float x,y;
+}_point;
+
+/* draw vertical shape visualising future joining (left as well
+ * right direction of future joining) */
+static void draw_horizontal_join_shape(ScrArea *sa, char dir)
 {
-	ScrArea *sa2;
-	ScrArea *up=0, *down=0, *right=0, *left=0;
-	ScrEdge *setest;
-	short val=0;
+	_point points[10];
+	short i;
+	float w, h;
+	float width = sa->v3->vec.x - sa->v1->vec.x;
+	float height = sa->v3->vec.y - sa->v1->vec.y;
+
+	if(height<width) {
+		h = height/8;
+		w = height/4;
+	}
+	else {
+		h = width/8;
+		w = width/4;
+	}
+
+	points[0].x = sa->v1->vec.x;
+	points[0].y = sa->v1->vec.y + height/2;
+
+	points[1].x = sa->v1->vec.x;
+	points[1].y = sa->v1->vec.y;
+
+	points[2].x = sa->v4->vec.x - w;
+	points[2].y = sa->v4->vec.y;
+
+	points[3].x = sa->v4->vec.x - w;
+	points[3].y = sa->v4->vec.y + height/2 - 2*h;
+
+	points[4].x = sa->v4->vec.x - 2*w;
+	points[4].y = sa->v4->vec.y + height/2;
+
+	points[5].x = sa->v4->vec.x - w;
+	points[5].y = sa->v4->vec.y + height/2 + 2*h;
+
+	points[6].x = sa->v3->vec.x - w;
+	points[6].y = sa->v3->vec.y;
+
+	points[7].x = sa->v2->vec.x;
+	points[7].y = sa->v2->vec.y;
+
+	points[8].x = sa->v4->vec.x;
+	points[8].y = sa->v4->vec.y + height/2 - h;
+
+	points[9].x = sa->v4->vec.x;
+	points[9].y = sa->v4->vec.y + height/2 + h;
+
+	if(dir=='l') {
+		/* when direction is left, then we flip direction of arrow */
+		float cx = sa->v1->vec.x + width;
+		for(i=0;i<10;i++) {
+			points[i].x -= cx;
+			points[i].x = -points[i].x;
+			points[i].x += sa->v1->vec.x;
+		}
+	}
+
+	glBegin(GL_POLYGON);
+	for(i=0;i<5;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glEnd();
+	glBegin(GL_POLYGON);
+	for(i=4;i<8;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glVertex2f(points[0].x, points[0].y);
+	glEnd();
+
+	glRectf(points[2].x, points[2].y, points[8].x, points[8].y);
+	glRectf(points[6].x, points[6].y, points[9].x, points[9].y);
+}
+
+/* draw vertical shape visualising future joining (up/down direction) */
+static void draw_vertical_join_shape(ScrArea *sa, char dir)
+{
+	_point points[10];
+	short i;
+	float w, h;
+	float width = sa->v3->vec.x - sa->v1->vec.x;
+	float height = sa->v3->vec.y - sa->v1->vec.y;
+
+	if(height<width) {
+		h = height/4;
+		w = height/8;
+	}
+	else {
+		h = width/4;
+		w = width/8;
+	}
+
+	points[0].x = sa->v1->vec.x + width/2;
+	points[0].y = sa->v3->vec.y;
+
+	points[1].x = sa->v2->vec.x;
+	points[1].y = sa->v2->vec.y;
+
+	points[2].x = sa->v1->vec.x;
+	points[2].y = sa->v1->vec.y + h;
+
+	points[3].x = sa->v1->vec.x + width/2 - 2*w;
+	points[3].y = sa->v1->vec.y + h;
+
+	points[4].x = sa->v1->vec.x + width/2;
+	points[4].y = sa->v1->vec.y + 2*h;
+
+	points[5].x = sa->v1->vec.x + width/2 + 2*w;
+	points[5].y = sa->v1->vec.y + h;
+
+	points[6].x = sa->v4->vec.x;
+	points[6].y = sa->v4->vec.y + h;
 	
-	sa= test_edge_area(sa, onedge);
-	if (sa==0) return;
+	points[7].x = sa->v3->vec.x;
+	points[7].y = sa->v3->vec.y;
+
+	points[8].x = sa->v1->vec.x + width/2 - w;
+	points[8].y = sa->v1->vec.y;
+
+	points[9].x = sa->v1->vec.x + width/2 + w;
+	points[9].y = sa->v1->vec.y;
+
+	if(dir=='u') {
+		/* when direction is up, then we flip direction of arrow */
+		float cy = sa->v1->vec.y + height;
+		for(i=0;i<10;i++) {
+			points[i].y -= cy;
+			points[i].y = -points[i].y;
+			points[i].y += sa->v1->vec.y;
+		}
+	}
+
+	glBegin(GL_POLYGON);
+	for(i=0;i<5;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glEnd();
+	glBegin(GL_POLYGON);
+	for(i=4;i<8;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glVertex2f(points[0].x, points[0].y);
+	glEnd();
+
+	glRectf(points[2].x, points[2].y, points[8].x, points[8].y);
+	glRectf(points[6].x, points[6].y, points[9].x, points[9].y);
+}
+
+/* draw join shape due to direction of joining */
+static void draw_join_shape(ScrArea *sa, char dir)
+{
+	if(dir=='u' || dir=='d')
+		draw_vertical_join_shape(sa, dir);
+	else
+		draw_horizontal_join_shape(sa, dir);
+}
+
+/* draw screen area darker with arrow (visualisation of future joining) */
+static void scrarea_draw_shape_dark(ScrArea *sa, char dir)
+{
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	glColor4ub(0, 0, 0, 105);
+	draw_join_shape(sa, dir);
+	glDisable(GL_BLEND);
+}
+
+/* draw screen area ligher with arrow shape ("eraser" of previous dark shape) */
+static void scrarea_draw_shape_light(ScrArea *sa, char dir)
+{
+	glBlendFunc(GL_DST_COLOR, GL_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	/* value 181 was hardly computed: 181~105 */
+	glColor4ub(255, 255, 255, 181);		
+	draw_join_shape(sa, dir);
+	glDisable(GL_BLEND);
+}
+
+static void joinarea_interactive(ScrArea *area, ScrEdge *onedge)
+{
+	struct ScrArea *sa1 = area, *sa2, *scr;
+	struct ScrArea *up=0, *down=0, *right=0, *left=0;
+	struct ScrEdge *se;
+	unsigned short event;
+	short ok=0, val=0, mval[2];
+	char dir=0;
+
+	sa1 = test_edge_area(sa1, onedge);
+	if(sa1==0) return;
 
-	/* which edges? */
 	/* find directions with same edge */
 	sa2= G.curscreen->areabase.first;
 	while(sa2) {
-		if(sa2 != sa) {
-			setest= screen_findedge(G.curscreen, sa2->v1, sa2->v2);
-			if(onedge==setest) right= sa2;
-			setest= screen_findedge(G.curscreen, sa2->v2, sa2->v3);
-			if(onedge==setest) down= sa2;
-			setest= screen_findedge(G.curscreen, sa2->v3, sa2->v4);
-			if(onedge==setest) left= sa2;
-			setest= screen_findedge(G.curscreen, sa2->v4, sa2->v1);
-			if(onedge==setest) up= sa2;
+		if(sa2 != sa1) {
+			se= screen_findedge(G.curscreen, sa2->v1, sa2->v2);
+			if(onedge==se) right= sa2;
+			se= screen_findedge(G.curscreen, sa2->v2, sa2->v3);
+			if(onedge==se) down= sa2;
+			se= screen_findedge(G.curscreen, sa2->v3, sa2->v4);
+			if(onedge==se) left= sa2;
+			se= screen_findedge(G.curscreen, sa2->v4, sa2->v1);
+			if(onedge==se) up= sa2;
 		}
 		sa2= sa2->next;
 	}
 	
-	sa2= 0;
-	setest= 0;
-	
 	if(left) val++;
 	if(up) val++;
 	if(right) val++;
@@ -2423,55 +2696,154 @@ static void joinarea(ScrArea *sa, ScrEdge *onedge)
 	
 	if(val==0) return;
 	else if(val==1) {
-		if(left) sa2= left;
-		else if(up) sa2= up;
-		else if(right) sa2= right;
-		else if(down) sa2= down;
+		if(left) {
+			right = sa1;
+			sa2 = left;
+			dir = 'h';
+		}
+		else if(right) {
+			left = sa1;
+			sa2 = right;
+			dir = 'h';
+		}
+		else if(up) {
+			down = sa1;
+			sa2= up;
+			dir = 'v';
+		}
+		else if(down) {
+			up = sa1;
+			sa2 = down;
+			dir = 'v';
+		}
 	}
-	
-	
-	if(sa2) {
-		/* new area is old sa */
+
+	mywinset(G.curscreen->mainwin);
+
+	/* initial set up screen area asigned for destroying */
+	scr = sa2;
+
+	/* set up standard cursor */
+	set_cursor(CURSOR_STD);
+
+	/* should already have a good matrix */
+	glReadBuffer(GL_FRONT);
+	glDrawBuffer(GL_FRONT);
+
+	/* to prevent flickering after clicking at "Join Areas " */
+	getmouseco_sc(mval);
+	if(dir=='h') {
+		if(scr==left && mval[0]>=onedge->v1->vec.x) scr = right;
+		else if(scr==right && mval[0]<onedge->v1->vec.x) scr = left;
+	}
+	else if(dir=='v') {
+		if(scr==down && mval[1]>=onedge->v1->vec.y) scr = up;
+		else if(scr==up && mval[1]<onedge->v1->vec.y) scr = down;
+	}
+
+	/* draw scr screen area with dark shape */
+	if(scr==left)
+		scrarea_draw_shape_dark(scr,'r');
+	else if(scr==right)
+		scrarea_draw_shape_dark(scr,'l');
+	else if(scr==up)
+		scrarea_draw_shape_dark(scr,'d');
+	else if(scr==down)
+		scrarea_draw_shape_dark(scr,'u');
+	glFlush();
+
+	/* "never ending loop" of interactive selection */
+	while(!ok) {
+		getmouseco_sc(mval);
+
+		/* test if position of mouse is on the "different side" of
+		 * "joining edge" */
+		if(dir=='h') {
+			if(scr==left && mval[0]>=onedge->v1->vec.x) {
+				scrarea_draw_shape_light(scr,'r');
+				scr = right;
+				scrarea_draw_shape_dark(scr,'l');
+			}
+			else if(scr==right && mval[0]<onedge->v1->vec.x) {
+				scrarea_draw_shape_light(scr,'l');
+				scr = left;
+				scrarea_draw_shape_dark(scr,'r');
+			}
+		}
+		else if(dir=='v') {
+			if(scr==down && mval[1]>=onedge->v1->vec.y) {
+				scrarea_draw_shape_light(scr,'u');
+				scr = up;
+				scrarea_draw_shape_dark(scr,'d');
+			}
+			else if(scr==up && mval[1]<onedge->v1->vec.y){
+				scrarea_draw_shape_light(scr,'d');
+				scr = down;
+				scrarea_draw_shape_dark(scr,'u');
+			}
+		}
+
+
+		/* get pressed keys and mouse buttons */
+		event = extern_qread(&val);
+
+		/* confirm joining of two screen areas */
+		if(val && event==LEFTMOUSE) ok= 1;
+
+		/* cancel joining of joining */
+		if(val && (event==ESCKEY || event==RIGHTMOUSE)) ok= -1;
+
+		glFlush();
+	}
+
+	glReadBuffer(GL_BACK);
+	glDrawBuffer(GL_BACK);
+
+	/* joining af screen areas was confirmed ... proceed joining */
+	if(ok==1) {
+		if(sa2!=scr) {
+			sa1 = sa2;
+			sa2 = scr;
+		}
+
 		if(sa2==left) {
-			sa->v1= sa2->v1;
-			sa->v2= sa2->v2;
-			screen_addedge(G.curscreen, sa->v2, sa->v3);
-			screen_addedge(G.curscreen, sa->v1, sa->v4);
+			sa1->v1= sa2->v1;
+			sa1->v2= sa2->v2;
+			screen_addedge(G.curscreen, sa1->v2, sa1->v3);
+			screen_addedge(G.curscreen, sa1->v1, sa1->v4);
 		}
 		else if(sa2==up) {
-			sa->v2= sa2->v2;
-			sa->v3= sa2->v3;
-			screen_addedge(G.curscreen, sa->v1, sa->v2);
-			screen_addedge(G.curscreen, sa->v3, sa->v4);
+			sa1->v2= sa2->v2;
+			sa1->v3= sa2->v3;
+			screen_addedge(G.curscreen, sa1->v1, sa1->v2);
+			screen_addedge(G.curscreen, sa1->v3, sa1->v4);
 		}
 		else if(sa2==right) {
-			sa->v3= sa2->v3;
-			sa->v4= sa2->v4;
-			screen_addedge(G.curscreen, sa->v2, sa->v3);
-			screen_addedge(G.curscreen, sa->v1, sa->v4);
+			sa1->v3= sa2->v3;
+			sa1->v4= sa2->v4;
+			screen_addedge(G.curscreen, sa1->v2, sa1->v3);
+			screen_addedge(G.curscreen, sa1->v1, sa1->v4);
 		}
 		else if(sa2==down) {
-			sa->v1= sa2->v1;
-			sa->v4= sa2->v4;
-			screen_addedge(G.curscreen, sa->v1, sa->v2);
-			screen_addedge(G.curscreen, sa->v3, sa->v4);
+			sa1->v1= sa2->v1;
+			sa1->v4= sa2->v4;
+			screen_addedge(G.curscreen, sa1->v1, sa1->v2);
+			screen_addedge(G.curscreen, sa1->v3, sa1->v4);
 		}
 	
-		/* remove edge and area */
-		/* remlink(&G.curscreen->edgebase, setest); */
-		/* freeN(setest); */
 		del_area(sa2);
 		BLI_remlink(&G.curscreen->areabase, sa2);
 		MEM_freeN(sa2);
 		
 		removedouble_scredges();
 		removenotused_scredges();
-		removenotused_scrverts();	/* as last */
+		removenotused_scrverts();
 		
 		testareas();
 		mainqenter(DRAWEDGES, 1);
-			/* test cursor en inputwindow */
-		mainqenter(MOUSEY, -1);		
+
+		/* test cursor en inputwindow */
+		mainqenter(MOUSEY, -1);
 	}
 }
 
@@ -2604,8 +2976,7 @@ static void scrarea_draw_splitpoint(ScrArea *sa, char dir, float fac)
 
 static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 {
-	ScrArea *sa= area;
-	ScrArea *scr;
+	ScrArea *scr, *sa= area;
 	float fac= 0.0;
 	unsigned short event;
 	short ok= 0, val, split = 0, mval[2], mvalo[2], first= 1;
@@ -2634,8 +3005,9 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 				if((mval[0] > scr->v1->vec.x) && (mval[0] < scr->v4->vec.x) &&
 				(mval[1] < scr->v2->vec.y) && (mval[1] > scr->v1->vec.y)){
 					/* test: is ScreenArea enough big for splitting */
-					split= testsplitpoint(scr, dir, fac);
-					if(split){
+					short tsplit= testsplitpoint(scr, dir, fac);
+					if(tsplit){
+						split = tsplit;
 						/* delete old line from previous ScreenArea */
 						if(!first) scrarea_draw_splitpoint(sa, dir, fac);
 						sa= scr;
@@ -3017,7 +3389,6 @@ static void moveareas(ScrEdge *edge)
 
 			delta= (dir=='h')?(mval[1]-mvalo[1]):(mval[0]-mvalo[0]);
 			delta= CLAMPIS(delta, -smaller, bigger);
-			
 			draw_front_xor_dirdist_line(dir, edge_position+delta, edge_start, edge_end);
 			glFlush();
 		} 
@@ -3196,29 +3567,6 @@ void draw_area_emboss(ScrArea *sa)
 	sdrawline(0, 0, 0, sa->winy);
 
 	glDisable( GL_BLEND );
-	
-	
-	/* for test */
-	if(FALSE && sa->spacetype==SPACE_VIEW3D) {
-	cpack(0xA0A0A0);
-	uiSetRoundBox(31);
-	uiRoundBoxEmboss(5.0, 5.0, 25.0, 100.0, 8.0, 0);
-
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA); 
-	
-	glRasterPos2f(8.0, 10.0);
-	BIF_draw_icon(ICON_MATERIAL_HLT);
-	glRasterPos2f(8.0, 30.0);
-	BIF_draw_icon(ICON_IPO_HLT);
-	glRasterPos2f(8.0, 50.0);
-	BIF_draw_icon(ICON_HOME);
-	glRasterPos2f(8.0, 70.0);
-	BIF_draw_icon(ICON_BORDERMOVE);
-	
-	glBlendFunc(GL_ONE,  GL_ZERO); 
-	glDisable(GL_BLEND);
-	}
 }
 
 
@@ -3257,33 +3605,21 @@ void drawscredge_area(ScrArea *sa)
 	
 }
 
-void drawscreen(void)
+/* ********************************* */
+
+/* for depgraph updating, all layers visible in a screen */
+unsigned int screen_view3d_layers(void)
 {
 	ScrArea *sa;
-
-	mywinset(G.curscreen->mainwin);
-	myortho2(-0.375, (float)G.curscreen->sizex-0.375, -0.375, (float)G.curscreen->sizey-0.375);
-
-	sa= G.curscreen->areabase.first;
-	while(sa) {
-		drawscredge_area(sa);
-		sa= sa->next;
+	int layer= 0;
+	
+	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+		if(sa->spacetype==SPACE_VIEW3D)
+			layer |= ((View3D *)sa->spacedata.first)->lay;
 	}
-
-	/* this double draw patch seems to be needed for certain sgi's (octane, indigo2) */
-	glDrawBuffer(GL_FRONT);
-
-	sa= G.curscreen->areabase.first;
-	while(sa) {
-		drawscredge_area(sa);
-		sa= sa->next;
-	}
-
-	glDrawBuffer(GL_BACK);
+	return layer;
 }
 
-/* ********************************* */
-
 bScreen *default_twosplit() 
 {
 	bScreen *sc= addscreen("screen");
@@ -3350,7 +3686,7 @@ int get_cursor(void) {
 }
 
 void set_cursor(int curs) {
-	if (!(R.flag & R_RENDERING) && G.background == 0) {
+	if (G.background == 0) {
 		if (curs!=curcursor) {
 			curcursor= curs;
 			window_set_cursor(mainwin, curs);
