@@ -1,5 +1,5 @@
 /**
- * $Id: editsound.c,v 1.25 2005/03/09 19:45:56 lukep Exp $
+ * $Id: editsound.c,v 1.27 2006/06/14 12:42:23 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -175,7 +175,7 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			G.v2d->cur.xmin+= dx;
 			G.v2d->cur.xmax-= dx;
 			test_view2d(G.v2d, curarea->winx, curarea->winy);
-
+			view2d_do_locks(curarea, V2D_LOCK_COPY);
 			doredraw= 1;
 			break;
 		case PADMINUS:
@@ -183,7 +183,7 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			G.v2d->cur.xmin-= dx;
 			G.v2d->cur.xmax+= dx;
 			test_view2d(G.v2d, curarea->winx, curarea->winy);
-
+			view2d_do_locks(curarea, V2D_LOCK_COPY);
 			doredraw= 1;
 			break;
 		case HOMEKEY:
@@ -314,12 +314,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 		readPackedFile(pf, &shortbuf, 2);
 		if(G.order==B_ENDIAN)
 		{
-			/* was SWITCH_SHORT before */
-			char s_i, *p_i;
-			p_i= (char *)&(shortbuf);
-			s_i= p_i[0];
-			p_i[0]= p_i[1];
-			p_i[1]= s_i;
+			SWITCH_SHORT(shortbuf);
 		}
 
 		/* read the number of channels */
@@ -327,12 +322,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 
 		if(G.order==B_ENDIAN)
 		{
-			/* was SWITCH_SHORT before */
-			char s_i, *p_i;
-			p_i= (char *)&(shortbuf);
-			s_i= p_i[0];
-			p_i[0]= p_i[1];
-			p_i[1]= s_i;
+			SWITCH_SHORT(shortbuf);
 		}
 
 		/* check the number of channels */
@@ -367,12 +357,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 		readPackedFile(pf, &shortbuf, 2);
 		if(G.order==B_ENDIAN)
 		{
-			/* was SWITCH_SHORT before */
-			char s_i, *p_i;
-			p_i= (char *)&(shortbuf);
-			s_i= p_i[0];
-			p_i[0]= p_i[1];
-			p_i[1]= s_i;
+			SWITCH_SHORT(shortbuf);
 		}
 		bits = shortbuf;
 		
