@@ -1,5 +1,5 @@
 /**
- * $Id: filesel.c,v 1.54 2005/06/04 16:22:50 lukep Exp $
+ * $Id: filesel.c,v 1.76 2006/06/16 12:17:35 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,10 +34,6 @@
 #include <string.h>
 #include <math.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 #include <io.h>
 #include <direct.h>
@@ -61,10 +57,11 @@
 
 #include "IMB_imbuf.h"
 
+#include "DNA_armature_types.h"
+#include "DNA_action_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_image_types.h"
 #include "DNA_ipo_types.h"
-#include "DNA_vfont_types.h"
 #include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_object_types.h"
@@ -72,17 +69,20 @@
 #include "DNA_space_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
-#include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_vfont_types.h"
+#include "DNA_view3d_types.h"
 
-#include "BKE_utildefines.h"
-#include "BKE_global.h"
-#include "BKE_main.h"
-#include "BKE_displist.h"
-#include "BKE_library.h"
+#include "BKE_action.h"
+#include "BKE_constraint.h"
 #include "BKE_curve.h"
+#include "BKE_depsgraph.h"
 #include "BKE_font.h"
+#include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_main.h"
 #include "BKE_material.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_gl.h"
 #include "BIF_interface.h"
@@ -96,6 +96,8 @@
 #include "BLO_readfile.h"
 
 #include "BDR_editcurve.h"
+#include "BDR_editobject.h"
+
 #include "BPI_script.h"
 #include "BSE_filesel.h"
 #include "BSE_view.h"
@@ -107,7 +109,10 @@
 #include "BIF_fsmenu.h"  /* include ourselves */
 
 #if defined WIN32 || defined __BeOS
-	int fnmatch(){return 0;}
+int fnmatch(const char *pattern, const char *string, int flags)
+{
+	return 0;
+}
 #else
 	#include <fnmatch.h>
 #endif
@@ -143,7 +148,7 @@ static rcti scrollrct, textrct, bar;
 static int filebuty1, filebuty2, page_ofs, collumwidth, selecting=0;
 static int filetoname= 0;
 static float pixels_to_ofs;
-static char otherdir[FILE_MAXDIR];
+static char otherdir[FILE_MAXDIR+FILE_MAXFILE];
 static ScrArea *otherarea;
 
 /* FSMENU HANDLING */
@@ -383,6 +388,41 @@ static int compare_size(const void *a1, const void *a2)
 	else return BLI_strcasecmp(entry1->relname,entry2->relname);
 }
 
+static int compare_extension(const void *a1, const void *a2) {
+	const struct direntry *entry1=a1, *entry2=a2;
+	char *sufix1, *sufix2;
+	char *nil="";
+
+	if (!(sufix1= strstr (entry1->relname, ".blend.gz"))) 
+		sufix1= strrchr (entry1->relname, '.');
+	if (!(sufix2= strstr (entry2->relname, ".blend.gz")))
+		sufix2= strrchr (entry2->relname, '.');
+	if (!sufix1) sufix1= nil;
+	if (!sufix2) sufix2= nil;
+
+	/* type is is equal to stat.st_mode */
+
+	if (S_ISDIR(entry1->type)){
+		if (S_ISDIR(entry2->type)==0) return (-1);
+	} else{
+		if (S_ISDIR(entry2->type)) return (1);
+	}
+	if (S_ISREG(entry1->type)){
+		if (S_ISREG(entry2->type)==0) return (-1);
+	} else{
+		if (S_ISREG(entry2->type)) return (1);
+	}
+	if ((entry1->type & S_IFMT) < (entry2->type & S_IFMT)) return (-1);
+	if ((entry1->type & S_IFMT) > (entry2->type & S_IFMT)) return (1);
+	
+	/* make sure "." and ".." are always first */
+	if( strcmp(entry1->relname, ".")==0 ) return (-1);
+	if( strcmp(entry2->relname, ".")==0 ) return (1);
+	if( strcmp(entry1->relname, "..")==0 ) return (-1);
+	if( strcmp(entry2->relname, "..")==0 ) return (-1);
+	
+	return (BLI_strcasecmp(sufix1, sufix2));
+}
 
 /* **************************************** */
 
@@ -407,11 +447,11 @@ void filesel_statistics(SpaceFile *sfile, int *totfile, int *selfile, float *tot
 			(*totfile) ++;
 
 			len = sfile->filelist[a].s.st_size;
-			(*totlen) += (len/1048576.0); 		
+			(*totlen) += (float)(len/1048576.0); 		
 
 			if(sfile->filelist[a].flags & ACTIVE) {
 				(*selfile) ++;
-				(*sellen) += (len/1048576.0);
+				(*sellen) += (float)(len/1048576.0);
 			}
 		}
 	}
@@ -419,93 +459,6 @@ void filesel_statistics(SpaceFile *sfile, int *totfile, int *selfile, float *tot
 
 /* *************** HELP FUNCTIONS ******************* */
 
-/* This is a really ugly function... its purpose is to
- * take the space file name and clean it up, replacing
- * excess file entry stuff (like /tmp/../tmp/../)
- */
-
-void checkdir(char *dir)
-{
-	short a;
-	char *start, *eind;
-	char tmp[FILE_MAXDIR+FILE_MAXFILE];
-
-	BLI_make_file_string(G.sce, tmp, dir, "");
-	strcpy(dir, tmp);
-	
-#ifdef WIN32
-	if(dir[0]=='.') {	/* happens for example in FILE_MAIN */
-		dir[0]= '\\';
-		dir[1]= 0;
-		return;
-	}	
-
-	while (start = strstr(dir, "\\..\\")) {
-		eind = start + strlen("\\..\\") - 1;
-		a = start-dir-1;
-		while (a>0) {
-			if (dir[a] == '\\') break;
-			a--;
-		}
-		strcpy(dir+a,eind);
-	}
-
-	while (start = strstr(dir,"\\.\\")){
-		eind = start + strlen("\\.\\") - 1;
-		strcpy(start,eind);
-	}
-
-	while (start = strstr(dir,"\\\\" )){
-		eind = start + strlen("\\\\") - 1;
-		strcpy(start,eind);
-	}
-
-	if(a = strlen(dir)){				/* remove the '\\' at the end */
-		while(a>0 && dir[a-1] == '\\'){
-			a--;
-			dir[a] = 0;
-		}
-	}
-
-	strcat(dir, "\\");
-#else	
-	if(dir[0]=='.') {	/* happens, for example in FILE_MAIN */
-		dir[0]= '/';
-		dir[1]= 0;
-		return;
-	}	
-	
-	while ( (start = strstr(dir, "/../")) ) {
-		eind = start + strlen("/../") - 1;
-		a = start-dir-1;
-		while (a>0) {
-			if (dir[a] == '/') break;
-			a--;
-		}
-		strcpy(dir+a,eind);
-	}
-
-	while ( (start = strstr(dir,"/./")) ){
-		eind = start + strlen("/./") - 1;
-		strcpy(start,eind);
-	}
-
-	while ( (start = strstr(dir,"//" )) ){
-		eind = start + strlen("//") - 1;
-		strcpy(start,eind);
-	}
-
-	if( (a = strlen(dir)) ){				/* remove all '/' at the end */
-		while(dir[a-1] == '/'){
-			a--;
-			dir[a] = 0;
-			if (a<=0) break;
-		}
-	}
-
-	strcat(dir, "/");
-#endif
-}
 
 /* not called when browsing .blend itself */
 void test_flags_file(SpaceFile *sfile)
@@ -529,7 +482,7 @@ void test_flags_file(SpaceFile *sfile)
 				
 				if(sfile->type==FILE_LOADLIB) {
 					char name[FILE_MAXDIR+FILE_MAXFILE];
-					strcpy(name, sfile->dir);
+					BLI_strncpy(name, sfile->dir, sizeof(name));
 					strcat(name, file->relname);
 					
 					/* prevent current file being used as acceptable dir */
@@ -548,9 +501,17 @@ void test_flags_file(SpaceFile *sfile)
 					|| BLI_testextensie(file->relname, ".otf")
 					|| BLI_testextensie(file->relname, ".otc")) {
 				file->flags |= FTFONTFILE;			
+			} else if (G.have_libtiff &&
+					(BLI_testextensie(file->relname, ".tif")
+					||	BLI_testextensie(file->relname, ".tiff"))) {
+					file->flags |= IMAGEFILE;			
+			} else if (BLI_testextensie(file->relname, ".exr")) {
+					file->flags |= IMAGEFILE;			
 			} else if (G.have_quicktime){
 				if(		BLI_testextensie(file->relname, ".jpg")
 					||	BLI_testextensie(file->relname, ".jpeg")
+					||	BLI_testextensie(file->relname, ".hdr")
+					||	BLI_testextensie(file->relname, ".exr")
 					||	BLI_testextensie(file->relname, ".tga")
 					||	BLI_testextensie(file->relname, ".rgb")
 					||	BLI_testextensie(file->relname, ".bmp")
@@ -565,11 +526,14 @@ void test_flags_file(SpaceFile *sfile)
 					||	BLI_testextensie(file->relname, ".pict")
 					||	BLI_testextensie(file->relname, ".pntg") //macpaint
 					||	BLI_testextensie(file->relname, ".qtif")
+					||  BLI_testextensie(file->relname, ".cin")
+					||  BLI_testextensie(file->relname, ".dpx")
 					||	BLI_testextensie(file->relname, ".sgi")) {
 					file->flags |= IMAGEFILE;			
 				}
 				else if(BLI_testextensie(file->relname, ".avi")
 					||	BLI_testextensie(file->relname, ".flc")
+					||	BLI_testextensie(file->relname, ".dv")
 					||	BLI_testextensie(file->relname, ".mov")
 					||	BLI_testextensie(file->relname, ".movie")
 					||	BLI_testextensie(file->relname, ".mv")) {
@@ -577,12 +541,16 @@ void test_flags_file(SpaceFile *sfile)
 				}
 			} else { // no quicktime
 				if(BLI_testextensie(file->relname, ".jpg")
+				   ||	BLI_testextensie(file->relname, ".hdr")
+				   ||	BLI_testextensie(file->relname, ".exr")
 					||	BLI_testextensie(file->relname, ".tga")
 					||	BLI_testextensie(file->relname, ".rgb")
 					||	BLI_testextensie(file->relname, ".bmp")
 					||	BLI_testextensie(file->relname, ".png")
 					||	BLI_testextensie(file->relname, ".iff")
 					||	BLI_testextensie(file->relname, ".lbm")
+					||  BLI_testextensie(file->relname, ".cin")
+					||  BLI_testextensie(file->relname, ".dpx")
 					||	BLI_testextensie(file->relname, ".sgi")) {
 					file->flags |= IMAGEFILE;			
 				}
@@ -615,8 +583,7 @@ void sort_filelist(SpaceFile *sfile)
 		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_size);	
 		break;
 	case FILE_SORTEXTENS:
-		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_name);	
-		break;
+		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_extension);	
 	}
 	
 	sfile->act= -1;
@@ -631,7 +598,7 @@ void sort_filelist(SpaceFile *sfile)
 void read_dir(SpaceFile *sfile)
 {
 	int num, len;
-	char wdir[FILE_MAXDIR];
+	char wdir[FILE_MAXDIR+FILE_MAXFILE];
 
 	/* sfile->act is used for example in databrowse: double names of library objects */
 	sfile->act= -1;
@@ -692,9 +659,9 @@ void freefilelist(SpaceFile *sfile)
 
 static void split_sfile(SpaceFile *sfile, char *s1)
 {
-	char string[FILE_MAXDIR+FILE_MAXFILE], dir[FILE_MAXDIR], file[FILE_MAXFILE];
+	char string[FILE_MAXDIR+FILE_MAXFILE], dir[FILE_MAXDIR+FILE_MAXFILE], file[FILE_MAXDIR+FILE_MAXFILE];
 
-	strcpy(string, s1);
+	BLI_strncpy(string, s1, sizeof(string));
 
 	BLI_split_dirfile(string, dir, file);
 	
@@ -704,7 +671,8 @@ static void split_sfile(SpaceFile *sfile, char *s1)
 		}
 		else test_flags_file(sfile);
 	}
-	strcpy(sfile->file, file);
+	BLI_strncpy(sfile->file, file, sizeof(sfile->file));
+		
 	BLI_make_file_string(G.sce, sfile->dir, dir, "");
 }
 
@@ -720,21 +688,21 @@ void parent(SpaceFile *sfile)
 	dir= sfile->dir;
 	
 #ifdef WIN32
-	if(a = strlen(dir)) {				/* remove all '/' at the end */
+	if( (a = strlen(dir)) ) {				/* remove all '/' at the end */
 		while(dir[a-1] == '\\') {
 			a--;
 			dir[a] = 0;
 			if (a<=0) break;
 		}
 	}
-	if(a = strlen(dir)) {				/* then remove all until '/' */
+	if( (a = strlen(dir)) ) {				/* then remove all until '/' */
 		while(dir[a-1] != '\\') {
 			a--;
 			dir[a] = 0;
 			if (a<=0) break;
 		}
 	}
-	if (a = strlen(dir)) {
+	if( (a = strlen(dir)) ) {
 		if (dir[a-1] != '\\') strcat(dir,"\\");
 	}
 	else if(sfile->type!=FILE_MAIN) strcpy(dir,"\\");
@@ -847,9 +815,9 @@ static void calc_file_rcts(SpaceFile *sfile)
 	collumwidth= (textrct.xmax-textrct.xmin)/sfile->collums;
 	
 
-	totfile= sfile->totfile + 0.5;
+	totfile= sfile->totfile + 0.5f;
 
-	tot= FILESEL_DY*totfile;
+	tot= (int)(FILESEL_DY*totfile);
 	if(tot) fac= ((float)sfile->collums*(scrollrct.ymax-scrollrct.ymin))/( (float)tot);
 	else fac= 1.0;
 	
@@ -857,22 +825,22 @@ static void calc_file_rcts(SpaceFile *sfile)
 	
 	if(tot) start= ( (float)sfile->ofs)/(totfile);
 	else start= 0.0;
-	if(fac>1.0) fac= 1.0;
+	if(fac>1.0) fac= 1.0f;
 
 	if(start+fac>1.0) {
-		sfile->ofs= ceil((1.0-fac)*totfile);
+		sfile->ofs= (short)ceil((1.0-fac)*totfile);
 		start= ( (float)sfile->ofs)/(totfile);
-		fac= 1.0-start;
+		fac= 1.0f-start;
 	}
 
 	bar.xmin= scrollrct.xmin+2;
 	bar.xmax= scrollrct.xmax-2;
 	h= (scrollrct.ymax-scrollrct.ymin)-4;
-	bar.ymax= scrollrct.ymax-2- start*h;
-	bar.ymin= bar.ymax- fac*h;
+	bar.ymax= (int)(scrollrct.ymax-2- start*h);
+	bar.ymin= (int)(bar.ymax- fac*h);
 
 	pixels_to_ofs= (totfile)/(float)(h+3);
-	page_ofs= fac*totfile;
+	page_ofs= (int)(fac*totfile);
 }
 
 int filescrollselect= 0;
@@ -918,6 +886,7 @@ static void print_line(SpaceFile *sfile, struct direntry *files, int x, int y)
 		linerect(boxcol, x, y);
 	}
 
+	// this is where the little boxes in the file view are being drawn according to the file type
 	if(files->flags & BLENDERFILE) {
 		cpack(0xA0A0);
 		glRects(x-14,  y,  x-8,  y+7);
@@ -1098,7 +1067,6 @@ static void draw_filetext(SpaceFile *sfile)
 	for(a= sfile->ofs; a<sfile->totfile; a++, files++) {
 	
 		if( calc_filesel_line(sfile, a, &x, &y)==0 ) break;
-		
 		print_line(sfile, files, x, y);
 	}
 
@@ -1151,14 +1119,22 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	}
 	else loadbutton= 0;
 
-	uiDefBut(block, TEX,	    1,"",	textrct.xmin, filebuty1, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
-	uiDefBut(block, TEX,	    2,"",	textrct.xmin, filebuty2, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
+	uiBlockBeginAlign(block);
+	uiDefBut(block, TEX,2,"",	textrct.xmin, filebuty2, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "Directory, enter a directory and press enter to create it"); /* Directory input */
 	if(loadbutton) {
 		uiSetCurFont(block, UI_HELV);
 		uiDefBut(block, BUT,	    5, sfile->title,	textrct.xmax-loadbutton, filebuty2, loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
+	}
+	uiBlockEndAlign(block);
+	
+	uiBlockBeginAlign(block);
+	uiDefBut(block, TEX,1,"",	textrct.xmin, filebuty1, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "File, increment version number with (+/-)"); /* File input */
+	if(loadbutton) {
+		uiSetCurFont(block, UI_HELV);
 		uiDefBut(block, BUT,	    6, "Cancel",	textrct.xmax-loadbutton, filebuty1, loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
 	}
-
+	uiBlockEndAlign(block);
+	
 	menu= fsmenu_build_menu();
 	if(menu[0])	// happens when no .Bfs is there, and first time browse
 		uiDefButS(block, MENU,	3, menu, scrollrct.xmin, filebuty1, scrollrct.xmax-scrollrct.xmin, 21, &sfile->menu, 0, 0, 0, 0, "");
@@ -1250,7 +1226,7 @@ static void do_filescrollwheel(SpaceFile *sfile, int move)
 void activate_fileselect(int type, char *title, char *file, void (*func)(char *))
 {
 	SpaceFile *sfile;
-	char group[24], name[FILE_MAXDIR], temp[FILE_MAXDIR];
+	char group[24], name[FILE_MAXDIR+FILE_MAXFILE], temp[FILE_MAXDIR+FILE_MAXFILE];
 	
 	if(curarea==0) return;
 	if(curarea->win==0) return;
@@ -1259,11 +1235,10 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	scrarea_queue_winredraw(curarea);
 	
 	/* sometime double, when area already is SPACE_FILE with a different file name */
-	addqueue(curarea->headwin, CHANGED, 1);
-	
+	if(curarea->headwin) addqueue(curarea->headwin, CHANGED, 1);
 
 	name[2]= 0;
-	strcpy(name, file);
+	BLI_strncpy(name, file, sizeof(name));
 	
 	sfile= curarea->spacedata.first;
 	/* sfile wants a (*)(short), but get (*)(char*) */
@@ -1276,14 +1251,17 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	if(BLI_convertstringcode(name, G.sce, G.scene->r.cfra)) sfile->flag |= FILE_STRINGCODE;
 	else sfile->flag &= ~FILE_STRINGCODE;
 
+	if (U.uiflag & USER_HIDE_DOT)
+		sfile->flag |= FILE_HIDE_DOT;
+
 	if(type==FILE_MAIN) {
 		char *groupname;
 		
-		strcpy(sfile->file, name+2);
+		BLI_strncpy(sfile->file, name+2, sizeof(sfile->file));
 
 		groupname = BLO_idcode_to_name( GS(name) );
 		if (groupname) {
-			strcpy(sfile->dir, groupname);
+			BLI_strncpy(sfile->dir, groupname, sizeof(sfile->dir) - 1);
 			strcat(sfile->dir, "/");
 		}
 
@@ -1294,7 +1272,7 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 		freefilelist(sfile);
 	}
 	else if(type==FILE_LOADLIB) {
-		strcpy(sfile->dir, name);
+		BLI_strncpy(sfile->dir, name, sizeof(sfile->dir));
 		if( is_a_library(sfile, temp, group) ) {
 			/* force a reload of the library-filelist */
 			freefilelist(sfile);
@@ -1307,7 +1285,8 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	}
 	else {	/* FILE_BLENDER */
 		split_sfile(sfile, name);	/* test filelist too */
-		
+		BLI_cleanup_dir(G.sce, sfile->dir);
+
 		/* free: filelist and libfiledata became incorrect */
 		if(sfile->libfiledata) BLO_blendhandle_close(sfile->libfiledata);
 		sfile->libfiledata= 0;
@@ -1319,7 +1298,7 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 void activate_imageselect(int type, char *title, char *file, void (*func)(char *))
 {
 	SpaceImaSel *simasel;
-	char dir[FILE_MAXDIR], name[FILE_MAXFILE];
+	char dir[FILE_MAXDIR+FILE_MAXFILE], name[FILE_MAXDIR+FILE_MAXFILE];
 	
 	if(curarea==0) return;
 	if(curarea->win==0) return;
@@ -1331,7 +1310,7 @@ void activate_imageselect(int type, char *title, char *file, void (*func)(char *
 	addqueue(curarea->win, CHANGED, 1);
 
 	name[2]= 0;
-	strcpy(name, file);
+	BLI_strncpy(name, file, sizeof(name));
 
 	simasel= curarea->spacedata.first;
 	simasel->returnfunc= func;
@@ -1340,12 +1319,11 @@ void activate_imageselect(int type, char *title, char *file, void (*func)(char *
 	else simasel->mode &= ~IMS_STRINGCODE;
 	
 	BLI_split_dirfile(name, dir, simasel->file);
+	BLI_cleanup_dir(G.sce, simasel->dir);
 	if(strcmp(dir, simasel->dir)!=0) simasel->fase= 0;
-	strcpy(simasel->dir, dir);
-	
-	BLI_strncpy(simasel->title, title, sizeof(simasel->title));
 
-	
+	BLI_strncpy(simasel->dir, dir, sizeof(simasel->dir));
+	BLI_strncpy(simasel->title, title, sizeof(simasel->title));
 	
 	/* filetoname= 1; */
 }
@@ -1357,12 +1335,12 @@ void activate_databrowse(ID *id, int idcode, int fromcode, int retval, short *me
 	SpaceFile *sfile;
 	char str[32];
 	
-	if(id==0) {
+	if(id==NULL) {
 		lb= wich_libbase(G.main, idcode);
-		id= lb->last;
+		id= lb->first;
 	}
 	
-	if(id) strcpy(str, id->name);
+	if(id) BLI_strncpy(str, id->name, sizeof(str));
 	else return;
 	
 	activate_fileselect(FILE_MAIN, "SELECT DATABLOCK", str, (void (*) (char*))func);
@@ -1476,7 +1454,7 @@ void free_filesel_spec(char *dir)
 static void filesel_execute(SpaceFile *sfile)
 {
 	struct direntry *files;
-	char name[FILE_MAXDIR];
+	char name[FILE_MAXDIR+FILE_MAXFILE];
 	int a;
 	
 	filesel_prevspace();
@@ -1489,20 +1467,43 @@ static void filesel_execute(SpaceFile *sfile)
 	else if(sfile->returnfunc) {
 		fsmenu_insert_entry(sfile->dir, 1);
 	
-		if(sfile->type==FILE_MAIN) {
-			if (sfile->menup) {
+		if(sfile->type==FILE_MAIN) { /* DATABROWSE */
+			if (sfile->menup) {	/* with value pointing to ID block index */
+				int notfound = 1;
+
+				/*	Need special handling since hiding .* datablocks means that
+					sfile->act is no longer the same as files->nr.
+
+					Also, toggle HIDE_DOT on and off can make sfile->act not longer
+					correct (meaning it doesn't point to the correct item in the filelist.
+					
+					sfile->file is always correct, so first with check if, for the item
+					corresponding to sfile->act, the name is the same.
+
+					If it isn't (or if sfile->act is not good), go over filelist and take
+					the correct one.
+
+					This means that selecting a datablock than hiding it makes it
+					unselectable. Not really a problem.
+
+					- theeth
+				 */
+
+				*sfile->menup= -1;
+
 				if(sfile->act>=0) {
 					if(sfile->filelist) {
 						files= sfile->filelist+sfile->act;
-						*sfile->menup= files->nr;
+						if ( strcmp(files->relname, sfile->file)==0) {
+							notfound = 0;
+							*sfile->menup= files->nr;
+						}
 					}	
-					else *sfile->menup= sfile->act+1;
 				}
-				else {
-					*sfile->menup= -1;
+				if (notfound) {
 					for(a=0; a<sfile->totfile; a++) {
 						if( strcmp(sfile->filelist[a].relname, sfile->file)==0) {
-							*sfile->menup= a+1;
+							*sfile->menup= sfile->filelist[a].nr;
 							break;
 						}
 					}
@@ -1514,7 +1515,7 @@ static void filesel_execute(SpaceFile *sfile)
 			if(strncmp(sfile->title, "Save", 4)==0) free_filesel_spec(sfile->dir);
 			if(strncmp(sfile->title, "Export", 6)==0) free_filesel_spec(sfile->dir);
 			
-			strcpy(name, sfile->dir);
+			BLI_strncpy(name, sfile->dir, sizeof(name));
 			strcat(name, sfile->file);
 			
 			if(sfile->flag & FILE_STRINGCODE) BLI_makestringcode(G.sce, name);
@@ -1526,7 +1527,7 @@ static void filesel_execute(SpaceFile *sfile)
 
 static void do_filesel_buttons(short event, SpaceFile *sfile)
 {
-	char butname[FILE_MAXDIR];
+	char butname[FILE_MAXDIR+FILE_MAXFILE];
 	
 	if (event == 1) {
 		if (strchr(sfile->file, '*') || strchr(sfile->file, '?') || strchr(sfile->file, '[')) {
@@ -1538,14 +1539,14 @@ static void do_filesel_buttons(short event, SpaceFile *sfile)
 					match = TRUE;
 				}
 			}
-			if (match) strcpy(sfile->file, "");
+			if (match) sfile->file[0] = '\0';
 			if(sfile->type==FILE_MAIN) filesel_select_objects(sfile);
 			scrarea_queue_winredraw(curarea);
 		}
 	}
 	else if(event== 2) {
 		/* reuse the butname variable */
-		checkdir(sfile->dir);
+		BLI_cleanup_dir(G.sce, sfile->dir);
 
 		BLI_make_file_string(G.sce, butname, sfile->dir, "");
 		/* strip the trailing slash if its a real dir */
@@ -1569,9 +1570,9 @@ static void do_filesel_buttons(short event, SpaceFile *sfile)
 		
 		/* which string */
 		if (selected) {
-			strcpy(sfile->dir, selected);
+			BLI_strncpy(sfile->dir, selected, sizeof(sfile->dir));
 			BLI_make_exist(sfile->dir);
-			checkdir(sfile->dir);
+			BLI_cleanup_dir(G.sce, sfile->dir);
 			freefilelist(sfile);
 			sfile->ofs= 0;
 			scrarea_queue_winredraw(curarea);
@@ -1850,7 +1851,7 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 					if(S_ISDIR(sfile->filelist[act].type)) {
 						strcat(sfile->dir, sfile->filelist[act].relname);
 						strcat(sfile->dir,"/");
-						checkdir(sfile->dir);
+						BLI_cleanup_dir(G.sce, sfile->dir);
 						freefilelist(sfile);
 						sfile->ofs= 0;
 						do_draw= 1;
@@ -1858,7 +1859,7 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 					else {
 						if( strcmp(sfile->file, sfile->filelist[act].relname)) {
 							do_draw= 1;
-							strcpy(sfile->file, sfile->filelist[act].relname);
+							BLI_strncpy(sfile->file, sfile->filelist[act].relname, sizeof(sfile->file));
 						}
 						if(event==MIDDLEMOUSE && sfile->type) filesel_execute(sfile);
 					}
@@ -2096,9 +2097,9 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			if(sfile->type==FILE_MAIN) break;
 
 #ifdef WIN32
-			strcpy(sfile->dir, "\\");
+			BLI_strncpy(sfile->dir, "\\", sizeof(sfile->dir));
 #else
-			strcpy(sfile->dir, "/");
+			BLI_strncpy(sfile->dir, "/", sizeof(sfile->dir));
 #endif
 			freefilelist(sfile);
 			sfile->ofs= 0;
@@ -2183,10 +2184,77 @@ static int is_a_library(SpaceFile *sfile, char *dir, char *group)
 	return 1;
 }
 
+/* orange hack... :) */
+static void do_sync_pose(Library *lib)
+{
+	Object *ob, *obt;
+	Base *base;
+	bArmature *arm;
+	
+	/* find the armature and the pose from library */
+	for(ob= G.main->object.first; ob; ob= ob->id.next)
+		if(ob->type==OB_ARMATURE && ob->id.lib==lib)
+			break;
+	
+	if(ob==NULL || ob->pose==NULL) {
+		error("No pose appended");
+		return;
+	}
+	
+	arm= ob->data;
+
+	/* for all visible objects in this scene */
+	for(base= G.scene->base.first; base; base= base->next) {
+		if((base->flag & SELECT)) {
+			obt= base->object;
+			if(obt->type==OB_ARMATURE && obt->pose && ob!=obt) {
+				char str[128];
+				
+				sprintf(str, "Replace Object %s", obt->id.name);
+				if(okee(str)) {
+					bPoseChannel *chan;
+					bArmature *oldarm= obt->data;
+					
+					/* link armature */
+					oldarm->id.us--;
+					obt->data= arm;
+					arm->id.us++;
+					
+					/* link pose */
+					free_pose_channels(obt->pose);
+					MEM_freeN(obt->pose);
+					copy_pose(&obt->pose, ob->pose, 1);
+					
+					/* relink */
+					ob->id.newid= &obt->id;
+					for (chan = obt->pose->chanbase.first; chan; chan=chan->next){
+						relink_constraints(&chan->constraints);
+					}
+					
+					obt->pose->flag |= POSE_RECALC;
+					obt->recalc |= OB_RECALC_DATA;
+				}
+			}
+		}
+	}
+
+	/* prevent saving in file, unlink from scene */
+	for(base= G.scene->base.first; base; base= base->next) {
+		if(base->object==ob)
+			break;
+	}
+	
+	if(base) {
+		free_and_unlink_base(base);
+	}
+
+	DAG_scene_sort(G.scene);	// for accidentally appended other objects
+}
+
 static void do_library_append(SpaceFile *sfile)
 {
 	Library *lib;
-	char dir[FILE_MAXDIR], group[32];
+	char dir[FILE_MAXDIR+FILE_MAXFILE], group[32];
 	
 	if ( is_a_library(sfile, dir, group)==0 ) {
 		error("Not a library");
@@ -2202,36 +2270,32 @@ static void do_library_append(SpaceFile *sfile)
 		
 		BLO_library_append(sfile, dir, idcode);
 
-		/* DISPLISTS */
+		/* DISPLISTS? */
 		ob= G.main->object.first;
-		set_displist_onlyzero(1);
 		while(ob) {
 			if(ob->id.lib) {
-				if(ob->type==OB_FONT) {
-					Curve *cu= ob->data;
-					if(cu->nurb.first==0) text_to_curve(ob, 0);
-				}
-				makeDispList(ob);
+				ob->recalc |= OB_RECALC;
 			}
-			else if(ob->type==OB_MESH && ob->parent && ob->parent->type==OB_LATTICE ) {
-				makeDispList(ob);
-			}
-			
 			ob= ob->id.next;
 		}
-		set_displist_onlyzero(0);
 	
-		/* in sfile->dir is the whole lib name */
-		strcpy(G.lib, sfile->dir);
-		
 		/* and now find the latest append lib file */
 		lib= G.main->library.first;
 		while(lib) {
-			if (BLI_streq(dir, lib->name)) break;
+			if (BLI_streq(dir, lib->filename)) break;
 			lib= lib->id.next;
 		}
 		
-		if((sfile->flag & FILE_LINK)==0) all_local(lib);
+		if(lib) {
+			if(sfile->flag & FILE_SYNCPOSE)
+				do_sync_pose(lib);
+			if((sfile->flag & FILE_LINK)==0) 
+				all_local(lib);
+		}
+		
+		/* in sfile->dir is the whole lib name */
+		BLI_strncpy(G.lib, sfile->dir, sizeof(G.lib) );
+		
 	}
 }
 
@@ -2240,7 +2304,7 @@ static void library_to_filelist(SpaceFile *sfile)
 	LinkNode *l, *names;
 	int ok, i, nnames, idcode;
 	char filename[FILE_MAXDIR+FILE_MAXFILE];
-	char dir[FILE_MAXDIR], group[24];
+	char dir[FILE_MAXDIR+FILE_MAXFILE], group[24];
 	
 	/* name test */
 	ok= is_a_library(sfile, dir, group);
@@ -2335,6 +2399,7 @@ static void filesel_select_objects(SpaceFile *sfile)
 		}
 		countall();
 		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWOOPS, 0);
 	}
 	else if( strcmp(sfile->dir, "Scene/")==0 ) {
 		
@@ -2369,6 +2434,7 @@ static void active_file_object(SpaceFile *sfile)
 					BASACT->flag |= SELECT;
 					sfile->filelist[sfile->act].flags |= ACTIVE;
 					allqueue(REDRAWVIEW3D, 0);
+					allqueue(REDRAWOOPS, 0);
 					scrarea_queue_winredraw(curarea);
 				}
 			}
@@ -2383,6 +2449,10 @@ void main_to_filelist(SpaceFile *sfile)
 	struct direntry *files, *firstlib = NULL;
 	ListBase *lb;
 	int a, fake, idcode, len, ok, totlib, totbl;
+	short hide = 0;
+
+	if (sfile->flag & FILE_HIDE_DOT)
+		hide = 1;
 
 	if(sfile->dir[0]=='/') sfile->dir[0]= 0;
 	
@@ -2394,7 +2464,7 @@ void main_to_filelist(SpaceFile *sfile)
 	if( sfile->dir[0]==0) {
 		
 		/* make directories */
-		sfile->totfile= 22;
+		sfile->totfile= 23;
 		sfile->filelist= (struct direntry *)malloc(sfile->totfile * sizeof(struct direntry));
 		
 		for(a=0; a<sfile->totfile; a++) {
@@ -2405,14 +2475,14 @@ void main_to_filelist(SpaceFile *sfile)
 		sfile->filelist[0].relname= BLI_strdup("..");
 		sfile->filelist[1].relname= BLI_strdup(".");
 		sfile->filelist[2].relname= BLI_strdup("Scene");
-		sfile->filelist[3].relname= BLI_strdup("Object");
-		sfile->filelist[4].relname= BLI_strdup("Mesh");
-		sfile->filelist[5].relname= BLI_strdup("Curve");
-		sfile->filelist[6].relname= BLI_strdup("Metaball");
-		sfile->filelist[7].relname= BLI_strdup("Material");
-		sfile->filelist[8].relname= BLI_strdup("Texture");
-		sfile->filelist[9].relname= BLI_strdup("Image");
-		sfile->filelist[10].relname= BLI_strdup("Ika");
+		sfile->filelist[3].relname= BLI_strdup("Group");
+		sfile->filelist[4].relname= BLI_strdup("Object");
+		sfile->filelist[5].relname= BLI_strdup("Mesh");
+		sfile->filelist[6].relname= BLI_strdup("Curve");
+		sfile->filelist[7].relname= BLI_strdup("Metaball");
+		sfile->filelist[8].relname= BLI_strdup("Material");
+		sfile->filelist[9].relname= BLI_strdup("Texture");
+		sfile->filelist[10].relname= BLI_strdup("Image");
 		sfile->filelist[11].relname= BLI_strdup("Wave");
 		sfile->filelist[12].relname= BLI_strdup("Lattice");
 		sfile->filelist[13].relname= BLI_strdup("Lamp");
@@ -2424,6 +2494,8 @@ void main_to_filelist(SpaceFile *sfile)
 		sfile->filelist[19].relname= BLI_strdup("Text");
 		sfile->filelist[20].relname= BLI_strdup("Armature");
 		sfile->filelist[21].relname= BLI_strdup("Action");
+		sfile->filelist[22].relname= BLI_strdup("NodeTree");
+		
 		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_name);
 	}
 	else {
@@ -2437,12 +2509,12 @@ void main_to_filelist(SpaceFile *sfile)
 		id= lb->first;
 		sfile->totfile= 0;
 		while(id) {
-			
 			if(sfile->returnfunc && idcode==ID_IP) {
 				if(sfile->ipotype== ((Ipo *)id)->blocktype) sfile->totfile++;
 			}
-			else sfile->totfile++;
-			
+			else if (hide==0 || id->name[2] != '.')
+				sfile->totfile++;
+
 			id= id->next;
 		}
 		
@@ -2475,32 +2547,42 @@ void main_to_filelist(SpaceFile *sfile)
 			
 			if(ok) {
 		
-				memset( files, 0 , sizeof(struct direntry));
-				files->relname= BLI_strdup(id->name+2);
-				
-				if(sfile->returnfunc==0) { /* F4 DATA BROWSE */
-					if(idcode==ID_OB) {
-						if( ((Object *)id)->flag & SELECT) files->flags |= ACTIVE;
+				if (hide==0 || id->name[2] != '.') {
+					memset( files, 0 , sizeof(struct direntry));
+					if(id->lib==NULL)
+						files->relname= BLI_strdup(id->name+2);
+					else {
+						char tmp[FILE_MAXDIR+FILE_MAXFILE], fi[FILE_MAXFILE];
+						BLI_strncpy(tmp, id->lib->name, FILE_MAXDIR+FILE_MAXFILE);
+						BLI_splitdirstring(tmp, fi);
+						files->relname= MEM_mallocN(FILE_MAXFILE+32, "filename for lib");
+						sprintf(files->relname, "%s / %s", fi, id->name+2);
 					}
-					else if(idcode==ID_SCE) {
-						if( ((Scene *)id)->r.scemode & R_BG_RENDER) files->flags |= ACTIVE;
+					
+					if(sfile->returnfunc==0) { /* F4 DATA BROWSE */
+						if(idcode==ID_OB) {
+							if( ((Object *)id)->flag & SELECT) files->flags |= ACTIVE;
+						}
+						else if(idcode==ID_SCE) {
+							if( ((Scene *)id)->r.scemode & R_BG_RENDER) files->flags |= ACTIVE;
+						}
 					}
+					files->nr= totbl+1;
+					files->poin= id;
+					fake= id->flag & LIB_FAKEUSER;
+					
+					if(id->lib && fake) sprintf(files->extra, "LF %d", id->us);
+					else if(id->lib) sprintf(files->extra, "L    %d", id->us);
+					else if(fake) sprintf(files->extra, "F    %d", id->us);
+					else sprintf(files->extra, "      %d", id->us);
+					
+					if(id->lib) {
+						if(totlib==0) firstlib= files;
+						totlib++;
+					}
+					
+					files++;
 				}
-				files->nr= totbl+1;
-				files->poin= id;
-				fake= id->flag & LIB_FAKEUSER;
-				
-				if(id->lib && fake) sprintf(files->extra, "LF %d", id->us);
-				else if(id->lib) sprintf(files->extra, "L    %d", id->us);
-				else if(fake) sprintf(files->extra, "F    %d", id->us);
-				else sprintf(files->extra, "      %d", id->us);
-				
-				if(id->lib) {
-					if(totlib==0) firstlib= files;
-					totlib++;
-				}
-				
-				files++;
 				totbl++;
 			}
 			
@@ -2547,7 +2629,7 @@ void clever_numbuts_filesel()
 
 	if (test != -1 && !(S_ISDIR(sfile->filelist[test].type))){
 		BLI_make_file_string(G.sce, orgname, sfile->dir, sfile->filelist[test].relname);
-		strcpy(filename, sfile->filelist[test].relname);
+		BLI_strncpy(filename, sfile->filelist[test].relname, sizeof(filename));
 		
 		add_numbut(0, TEX, "", 0, len, filename, "Rename File");
 
