@@ -4,7 +4,7 @@
  * Functions to draw the "UV/Image Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_image.c,v 1.36 2005/05/03 10:54:42 ton Exp $
+ * $Id: header_image.c,v 1.60 2006/07/03 11:21:38 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -45,6 +45,7 @@
 #include "DNA_ID.h"
 #include "DNA_image_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_object_types.h"
 #include "DNA_packedFile_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
@@ -55,19 +56,23 @@
 #include "BDR_unwrapper.h"
 #include "BKE_global.h"
 #include "BKE_image.h"
+#include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_packedFile.h"
 #include "BKE_utildefines.h"
+#include "BKE_depsgraph.h"
 
 #include "BLI_blenlib.h"
 #include "BIF_drawimage.h"
 #include "BIF_editsima.h"
 #include "BIF_interface.h"
+#include "BIF_previewrender.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_toets.h"
 #include "BIF_toolbox.h"
+#include "BIF_transform.h"
 #include "BIF_writeimage.h"
 
 #include "BSE_filesel.h"
@@ -78,12 +83,11 @@
 
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
+#include "BSE_trans_types.h"
 
 #include "blendef.h"
 #include "mydevice.h"
 
-#include "TPT_DependKludge.h"
-
 static void load_space_image(char *str)	/* called from fileselect */
 {
 	Image *ima=0;
@@ -172,30 +176,42 @@ static void replace_space_image(char *str)		/* called from fileselect */
 
 static void save_paint(char *name)
 {
-	char str[FILE_MAXDIR+FILE_MAXFILE];
 	Image *ima = G.sima->image;
-	ImBuf *ibuf;
+	int len;
+	char str[FILE_MAXDIR+FILE_MAXFILE];
 
 	if (ima  && ima->ibuf) {
 		BLI_strncpy(str, name, sizeof(str));
 
 		BLI_convertstringcode(str, G.sce, G.scene->r.cfra);
+		
+		if(G.scene->r.scemode & R_EXTENSION) 
+			BKE_add_image_extension(str, G.scene->r.imtype);
 
 		if (saveover(str)) {
-			ibuf = IMB_dupImBuf(ima->ibuf);
-
-			if (ibuf) {
-				if (BIF_write_ibuf(ibuf, str)) {
-					BLI_strncpy(ima->name, name, sizeof(ima->name));
-					ima->ibuf->userflags &= ~IB_BITMAPDIRTY;
-					allqueue(REDRAWHEADERS, 0);
-					allqueue(REDRAWBUTSSHADING, 0);
-				} else {
-					error("Couldn't write image: %s", str);
-				}
-
-				IMB_freeImBuf(ibuf);
+			
+			/* enforce user setting for RGB or RGBA, but skip BW */
+			if(G.scene->r.planes==32)
+				ima->ibuf->depth= 32;
+			else if(G.scene->r.planes==24)
+				ima->ibuf->depth= 24;
+			
+			waitcursor(1);
+			if (BKE_write_ibuf(ima->ibuf, str, G.scene->r.imtype, G.scene->r.subimtype, G.scene->r.quality)) {
+				BLI_strncpy(ima->name, name, sizeof(ima->name));
+				ima->ibuf->userflags &= ~IB_BITMAPDIRTY;
+				allqueue(REDRAWHEADERS, 0);
+				allqueue(REDRAWBUTSSHADING, 0);
+			} else {
+				error("Couldn't write image: %s", str);
 			}
+			
+			/* name image as how we saved it */
+			len= strlen(str);
+			while (len > 0 && str[len - 1] != '/' && str[len - 1] != '\\') len--;
+			rename_id(&ima->id, str+len);
+
+			waitcursor(0);
 		}
 	}
 }
@@ -298,12 +314,6 @@ void do_image_buttons(unsigned short event)
 		}
 		break;
 
-	case B_CLIP_UV:
-		tface_do_clip();
-		allqueue(REDRAWIMAGE, 0);
-		allqueue(REDRAWVIEW3D, 0);
-		break;
-
 	case B_SIMAGEPAINTTOOL:
 		// check for packed file here
 		allqueue(REDRAWIMAGE, 0);
@@ -338,12 +348,32 @@ void do_image_buttons(unsigned short event)
 		if (ima) {
 			strcpy(name, ima->name);
 			if (ima->ibuf) {
-				char str[32];	// sufficient for message
+				char str[64];
 				save_image_filesel_str(str);
+				
+				/* so it shows extension in file window */
+				if(G.scene->r.scemode & R_EXTENSION) 
+					BKE_add_image_extension(name, G.scene->r.imtype);
+				
 				activate_fileselect(FILE_SPECIAL, str, name, save_paint);
 			}
 		}
 		break;
+	case B_SIMA_USE_ALPHA:
+		G.sima->flag &= ~(SI_SHOW_ALPHA|SI_SHOW_ZBUF);
+		scrarea_queue_winredraw(curarea);
+		scrarea_queue_headredraw(curarea);
+		break;
+	case B_SIMA_SHOW_ALPHA:
+		G.sima->flag &= ~(SI_USE_ALPHA|SI_SHOW_ZBUF);
+		scrarea_queue_winredraw(curarea);
+		scrarea_queue_headredraw(curarea);
+		break;
+	case B_SIMA_SHOW_ZBUF:
+		G.sima->flag &= ~(SI_SHOW_ALPHA|SI_USE_ALPHA);
+		scrarea_queue_winredraw(curarea);
+		scrarea_queue_headredraw(curarea);
+		break;
 	}
 }
 
@@ -395,13 +425,13 @@ static uiBlock *image_view_viewnavmenu(void *arg_unused)
 
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:8", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:4", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 2:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 4:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 8:1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:8|Shift+NumPad 8", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:4|Shift+NumPad 4", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:2|Shift+NumPad 2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 1:1|NumPad 1", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 2:1|NumPad 2", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 4:1|NumPad 4", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Zoom 8:1|NumPad 8", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
 
 	uiBlockSetDirection(block, UI_RIGHT);
 	uiTextBoundsBlock(block, 50);
@@ -444,6 +474,17 @@ static void do_image_viewmenu(void *arg, int event)
 		break;
 	case 9:
 		image_viewcentre();
+	case 10: /* Display Normalized Coordinates */
+		G.sima->flag ^= SI_COORDFLOATS;
+		allqueue(REDRAWIMAGE, 0);
+		break;
+	case 11: /* Curves Panel... */
+		add_blockhandler(curarea, IMAGE_HANDLER_CURVES, UI_PNL_UNSTOW);
+		break;
+	case 12: /* composite preview */
+		toggle_blockhandler(curarea, IMAGE_HANDLER_PREVIEW, 0);
+		scrarea_queue_winredraw(curarea);
+
 	}
 	allqueue(REDRAWVIEW3D, 0);
 }
@@ -457,15 +498,18 @@ static uiBlock *image_viewmenu(void *arg_unused)
 	block= uiNewBlock(&curarea->uiblocks, "image_viewmenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_viewmenu, NULL);
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "View Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "View Paint Tool...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Properties...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Paint Tool...|C",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Curves Tool...",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 11, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_MENU_PANEL, "Composite Preview...|Shift P",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 12, "");
 
-	
+	if(G.sima->flag & SI_COORDFLOATS) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Display Normalized Coordinates|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Display Normalized Coordinates|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
 	if(G.f & G_DRAWFACES) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Draw Faces", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Draw Faces|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
 	if(G.sima->flag & SI_DRAWSHADOW) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Draw Shadow Mesh", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Draw Shadow Mesh|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
-	
+
 	uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	uiDefIconTextBlockBut(block, image_view_viewnavmenu, NULL, ICON_RIGHTARROW_THIN, "View Navigation", 0, yco-=20, 120, 19, "");
 
@@ -501,7 +545,10 @@ static void do_image_selectmenu(void *arg, int event)
 	switch(event)
 	{
 	case 0: /* Border Select */
-		borderselect_sima();
+		borderselect_sima(UV_SELECT_ALL);
+		break;
+	case 8: /* Border Select Pinned */
+		borderselect_sima(UV_SELECT_PINNED);
 		break;
 	case 1: /* Select/Deselect All */
 		select_swap_tface_uv();
@@ -509,7 +556,7 @@ static void do_image_selectmenu(void *arg, int event)
 	case 2: /* Unlink Selection */
 		unlink_selection();
 		break;
-	case 3: /* Select Linked UVs */
+	case 3: /* Linked UVs */
 		select_linked_tface_uv(2);
 		break;
 	case 4: /* Toggle Local UVs Stick to Vertex in Mesh */
@@ -522,8 +569,10 @@ static void do_image_selectmenu(void *arg, int event)
 		allqueue(REDRAWIMAGE, 0);
 		break;  
 	case 5: /* Toggle UVs Stick to Vertex in Mesh */
-		if(G.sima->flag & SI_STICKYUVS)
+		if(G.sima->flag & SI_STICKYUVS) {
 			G.sima->flag &= ~SI_STICKYUVS;
+			G.sima->flag |= SI_LOCALSTICKY;
+		}
 		else {
 			G.sima->flag |= SI_STICKYUVS;
 			G.sima->flag &= ~SI_LOCALSTICKY;
@@ -537,6 +586,9 @@ static void do_image_selectmenu(void *arg, int event)
 			G.sima->flag |= SI_SELACTFACE;
 		allqueue(REDRAWIMAGE, 0);
 		break;
+	case 7: /* Pinned UVs */
+		select_pinned_tface_uv();
+		break;
 	}
 }
 
@@ -559,15 +611,21 @@ static uiBlock *image_selectmenu(void *arg_unused)
 	if(G.sima->flag & SI_STICKYUVS) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Stick UVs to Mesh Vertex|Ctrl C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Stick UVs to Mesh Vertex|Ctrl C", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
 
-	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Border Select|B", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Border Select Pinned|Shift B", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
+
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select/Deselect All|A", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unlink Selection|Alt L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 2, "");
+	
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Select Linked UVs|Ctrl L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Pinned UVs|Shift P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Linked UVs|Ctrl L", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
 
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
@@ -688,8 +746,13 @@ static void do_image_imagemenu(void *arg, int event)
 		if (ima) {
 			strcpy(name, ima->name);
 			if (ima->ibuf) {
-				char str[32];	// sufficient for message
+				char str[64];
 				save_image_filesel_str(str);
+				
+				/* so it shows an extension in filewindow */
+				if(G.scene->r.scemode & R_EXTENSION) 
+					BKE_add_image_extension(name, G.scene->r.imtype);
+				
 				activate_fileselect(FILE_SPECIAL, str, name, save_paint);
 			}
 		}
@@ -707,13 +770,40 @@ static void do_image_imagemenu(void *arg, int event)
 				else
 					error("Image not available. Keeping packed image.");
 			}
+			if (ima->preview) {
+				free_image_preview(ima);
+			}
 			free_image_buffers(ima);	/* force read again */
 			ima->ok= 1;
 			image_changed(G.sima, 0);
 		}
 		allqueue(REDRAWIMAGE, 0);
 		allqueue(REDRAWVIEW3D, 0);
+		BIF_preview_changed(ID_TE);
 		break;
+	case 7: /* New Image */
+		{
+			static int width= 256, height= 256;
+			static short uvtestgrid=0;
+			char name[256];
+
+			strcpy(name, "Image");
+
+			add_numbut(0, TEX, "Name:", 0, 255, name, NULL);
+			add_numbut(1, NUM|INT, "Width:", 1, 5000, &width, NULL);
+			add_numbut(2, NUM|INT, "Height:", 1, 5000, &height, NULL);
+			add_numbut(3, TOG|SHO, "UV Test Grid", 0, 0, &uvtestgrid, NULL);
+			if (!do_clever_numbuts("New Image", 4, REDRAW))
+				return;
+
+			G.sima->image= new_image(width, height, name, uvtestgrid);
+			image_changed(G.sima, 0);
+
+			allqueue(REDRAWIMAGE, 0);
+			allqueue(REDRAWVIEW3D, 0);
+
+			break;
+		}
 	}
 }
 
@@ -725,10 +815,11 @@ static uiBlock *image_imagemenu(void *arg_unused)
 	block= uiNewBlock(&curarea->uiblocks, "image_imagemenu", UI_EMBOSSP, UI_HELV, curarea->headwin);
 	uiBlockSetButmFunc(block, do_image_imagemenu, NULL);
 
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "New...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Open...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
 	
 	if (G.sima->image) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save As...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 5, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Replace...", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Reload", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 6, "");
 
@@ -802,15 +893,8 @@ static uiBlock *image_uvs_showhidemenu(void *arg_unused)
 
 static void do_image_uvs_propfalloffmenu(void *arg, int event)
 {
-	
-	switch(event) {
-	case 0: /* proportional edit - sharp*/
-		G.scene->prop_mode = 0;
-		break;
-	case 1: /* proportional edit - smooth*/
-		G.scene->prop_mode = 1;
-		break;
-	}
+	G.scene->prop_mode= event;
+	allqueue(REDRAWVIEW3D, 1);
 }
 
 static uiBlock *image_uvs_propfalloffmenu(void *arg_unused)
@@ -821,10 +905,18 @@ static uiBlock *image_uvs_propfalloffmenu(void *arg_unused)
 	block= uiNewBlock(&curarea->uiblocks, "image_uvs_propfalloffmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetButmFunc(block, do_image_uvs_propfalloffmenu, NULL);
 	
-	if (G.scene->prop_mode==0) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Sharp|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Sharp|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
-	if (G.scene->prop_mode==1) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Smooth|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Smooth|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	if (G.scene->prop_mode==PROP_SMOOTH) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Smooth|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_SMOOTH, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Smooth|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_SMOOTH, "");
+	if (G.scene->prop_mode==PROP_SPHERE) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Sphere|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_SPHERE, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Sphere|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_SPHERE, "");
+	if (G.scene->prop_mode==PROP_ROOT) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Root|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_ROOT, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Root|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_ROOT, "");
+	if (G.scene->prop_mode==PROP_SHARP) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Sharp|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_SHARP, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Sharp|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_SHARP, "");
+	if (G.scene->prop_mode==PROP_LIN) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Linear|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_LIN, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Linear|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_LIN, "");
+	if (G.scene->prop_mode==PROP_CONST) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Constant|Shift O", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_CONST, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Constant|Shift O",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, PROP_CONST, "");
 		
 	uiBlockSetDirection(block, UI_RIGHT);
 	uiTextBoundsBlock(block, 60);
@@ -835,13 +927,16 @@ static void do_image_uvs_transformmenu(void *arg, int event)
 {
 	switch(event) {
 	case 0: /* Grab */
-		transform_tface_uv('g', 0);
+		initTransform(TFM_TRANSLATION, CTX_NONE);
+		Transform();
 		break;
 	case 1: /* Rotate */
-		transform_tface_uv('r', 0);
+		initTransform(TFM_ROTATION, CTX_NONE);
+		Transform();
 		break;
 	case 2: /* Scale */
-		transform_tface_uv('s', 0);
+		initTransform(TFM_RESIZE, CTX_NONE);
+		Transform();
 		break;
 	}
 }
@@ -956,8 +1051,7 @@ static void do_image_uvsmenu(void *arg, int event)
 			G.scene->proportional= 1;
 		break;
 	case 7: /* UVs Snap to Pixel */
-		if(G.sima->flag & SI_NOPIXELSNAP) G.sima->flag &= ~SI_NOPIXELSNAP;
-		else G.sima->flag |= SI_NOPIXELSNAP;
+		G.sima->flag ^= SI_PIXELSNAP;
 		break;
     case 8:
 		pin_tface_uv(1);
@@ -966,7 +1060,14 @@ static void do_image_uvsmenu(void *arg, int event)
 		pin_tface_uv(0);
 		break;
     case 10:
-		unwrap_lscm();
+		unwrap_lscm(0);
+		break;
+	case 11:
+		if(G.sima->flag & SI_LIVE_UNWRAP) G.sima->flag &= ~SI_LIVE_UNWRAP;
+		else G.sima->flag |= SI_LIVE_UNWRAP;
+		break;
+	case 12:
+		minimize_stretch_tface_uv();
 		break;
 	}
 }
@@ -983,8 +1084,8 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 	// uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Transform Properties...|N", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 0, "");
 	// uiDefBut(block, SEPR, 0, "",        0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
-	if(G.sima->flag & SI_NOPIXELSNAP) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Snap to Pixels|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
-	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Snap to Pixels|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	if(G.sima->flag & SI_PIXELSNAP) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Snap to Pixels|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Snap to Pixels|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 7, "");
 
 	if(G.sima->flag & SI_BE_SQUARE) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Quads Constrained Rectangular|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
 	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Quads Constrained Rectangular|", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 1, "");
@@ -994,12 +1095,15 @@ static uiBlock *image_uvsmenu(void *arg_unused)
 
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
+	if(G.sima->flag & SI_LIVE_UNWRAP) uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Live Unwrap Transform", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 11, "");
+	else uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Live Unwrap Transform", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 11, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unwrap|E", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Unpin|Alt P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 9, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Pin|P", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 8, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "LSCM Unwrap|E", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 10, "");
 
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");	
 
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Minimize Stretch|Ctrl V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 12, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Limit Stitch...|Shift V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 3, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Stitch|V", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 4, "");
 	uiDefIconTextBlockBut(block, image_uvs_transformmenu, NULL, ICON_RIGHTARROW_THIN, "Transform", 0, yco-=20, 120, 19, "");
@@ -1071,11 +1175,11 @@ void image_buttons(void)
 
 	uiBlockSetEmboss(block, UI_EMBOSSN);
 	if(curarea->flag & HEADER_NO_PULLDOWN) {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Show pulldown menus");
 	} else {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Hide pulldown menus");
 	}
@@ -1104,96 +1208,42 @@ void image_buttons(void)
 		xco+= xmax;
 	}
 	
-	
 	/* other buttons: */
 	uiBlockSetEmboss(block, UI_EMBOSS);
 
-	// xco+=XIC;
+	xco= std_libbuttons(block, xco, 0, 0, NULL, B_SIMABROWSE, ID_IM, 0, (ID *)G.sima->image, 0, &(G.sima->imanr), 0, 0, B_IMAGEDELETE, 0, 0);
 
-	/*xco = 8;
-	
-	uiDefIconTextButC(block, ICONTEXTROW,B_NEWSPACE, ICON_VIEW3D, windowtype_pup(), xco,0,XIC+10,YIC, &(curarea->butspacetype), 1.0, SPACEICONMAX, 0, 0, "Displays Current Window Type. Click for menu of available types.");
-*/
-	// xco+= XIC+22;
-	
-	/* FULL WINDOW */
-	/* if(curarea->full) uiDefIconBut(block, BUT,B_FULL, ICON_SPLITSCREEN,	xco,0,XIC,YIC, 0, 0, 0, 0, 0, "Returns to multiple views window (CTRL+Up arrow)");
-	else uiDefIconBut(block, BUT,B_FULL, ICON_FULLSCREEN,	xco,0,XIC,YIC, 0, 0, 0, 0, 0, "Makes current window full screen (CTRL+Down arrow)");
-	*/
-	
-	/* HOME*/
-	/*	uiDefIconBut(block, BUT, B_SIMAGEHOME, ICON_HOME,	xco+=XIC,0,XIC,YIC, 0, 0, 0, 0, 0, "Zooms window to home view showing all items (HOMEKEY)"); 
-	
-	
-	uiDefIconButS(block, TOG|BIT|0, B_BE_SQUARE, ICON_KEEPRECT,	xco+=XIC,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Toggles constraining UV polygons to squares while editing");
-	uiDefIconButS(block, ICONTOG|BIT|2, B_CLIP_UV, ICON_CLIPUV_DEHLT,xco+=XIC,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Toggles clipping UV with image size");
-	*/
-    
-	xco= std_libbuttons(block, xco, 0, 0, NULL, B_SIMABROWSE, (ID *)G.sima->image, 0, &(G.sima->imanr), 0, 0, B_IMAGEDELETE, 0, 0);
-
-	
 	if (G.sima->image) {
-	
 		xco+= 8;
 	
 		if (G.sima->image->packedfile) {
 			headerbuttons_packdummy = 1;
 		}
-		uiDefIconButI(block, TOG|BIT|0, B_SIMAPACKIMA, ICON_PACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Pack/Unpack this image");
-		
-		xco+= XIC;
-	}
-	
-	
-	/*
-	uiBlockSetCol(block, TH_AUTO);
-	uiDefBut(block, BUT, B_SIMAGELOAD, "Load",		xco, 0, 2*XIC, YIC, 0, 0, 0, 0, 0, "Loads image - file select");
-	xco+= 2*XIC;
-	
-	if (G.sima->image) {
-		uiBlockSetCol(block, TH_AUTO);
-		uiDefBut(block, BUT, B_SIMAGEREPLACE, "Replace",xco,0,(short)(3*XIC),YIC, 0, 0, 0, 0, 0, "Replaces current image - file select");
-		xco+= 3.5*XIC;
-		uiDefIconButS(block, TOG|BIT|4, 0, ICON_ENVMAP, xco,0,XIC,YIC, &G.sima->image->flag, 0, 0, 0, 0, "Uses this image as a reflection map (Ignores UV Coordinates)");
-
-		uiDefIconButS(block, TOG|BIT|0, B_SIMAGEDRAW1, ICON_GRID, xco+=XIC,0,XIC,YIC, &G.sima->image->flag, 0, 0, 0, 0, "");
-		uiDefButS(block, NUM, B_SIMAGEDRAW, "",	xco+=XIC,0,XIC,YIC, &G.sima->image->xrep, 1.0, 16.0, 0, 0, "Sets the degree of repetition in the X direction");
-		uiDefButS(block, NUM, B_SIMAGEDRAW, "",	xco+=XIC,0,XIC,YIC, &G.sima->image->yrep, 1.0, 16.0, 0, 0, "Sets the degree of repetition in the Y direction");
-	
-		uiDefButS(block, TOG|BIT|1, B_TWINANIM, "Anim", xco,0,(2*XIC),YIC, &G.sima->image->tpageflag, 0, 0, 0, 0, "Toggles use of animated texture");
-		xco+= XIC;
-		uiDefButS(block, NUM, B_TWINANIM, "",	xco+=XIC,0,XIC,YIC, &G.sima->image->twsta, 0.0, 128.0, 0, 0, "Displays the start frame of an animated texture. Click to change.");
-		uiDefButS(block, NUM, B_TWINANIM, "",	xco+=XIC,0,XIC,YIC, &G.sima->image->twend, 0.0, 128.0, 0, 0, "Displays the end frame of an animated texture. Click to change.");
-//		uiDefButS(block, TOG|BIT|2, 0, "Cycle", xco+=XIC,0,2*XIC,YIC, &G.sima->image->tpageflag, 0, 0, 0, 0, "");
-		
-		uiDefButS(block, NUM, 0, "Speed", xco,0,4*XIC,YIC, &G.sima->image->animspeed, 1.0, 100.0, 0, 0, "Displays Speed of the animation in frames per second. Click to change.");
-		xco+= 4.5*XIC;
-
-		// texture paint button used to be here
-		
-		xco+= XIC;
-		if (G.sima->image && G.sima->image->ibuf && (G.sima->image->ibuf->userflags & IB_BITMAPDIRTY)) {
-			uiDefBut(block, BUT, B_SIMAGESAVE, "Save",		xco,0,2*XIC,YIC, 0, 0, 0, 0, 0, "Saves image");
-			xco += 2*XIC;
-		}
-	}
-*/
-
-	xco+= 8;
-
-	if (G.sima->image) {
-
-#ifdef NAN_TPT
-		
-		uiDefIconButS(block, TOG|BIT|3, B_SIMAGEPAINTTOOL, ICON_TPAINT_HLT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Enables painting textures on the image with left mouse button");
+		uiDefIconButBitI(block, TOG, 1, B_SIMAPACKIMA, ICON_PACKAGE,	xco,0,XIC,YIC, &headerbuttons_packdummy, 0, 0, 0, 0, "Pack/Unpack this image");
 		xco+= XIC+8;
-#endif /* NAN_TPT */
 
+		uiDefIconButBitI(block, TOG, SI_DRAWTOOL, B_SIMAGEPAINTTOOL, ICON_TPAINT_HLT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Enables painting textures on the image with left mouse button");
+		xco+= XIC+8;
+
+		uiBlockBeginAlign(block);
+		uiDefIconButBitI(block, TOG, SI_USE_ALPHA, B_SIMA_USE_ALPHA, ICON_TRANSP_HLT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws image with alpha");
+		xco+= XIC;
+		uiDefIconButBitI(block, TOG, SI_SHOW_ALPHA, B_SIMA_SHOW_ALPHA, ICON_DOT, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws only alpha");
+		xco+= XIC;
+		if(G.sima->image->ibuf) {
+			if(G.sima->image->ibuf->zbuf || G.sima->image->ibuf->zbuf_float) {
+				uiDefIconButBitI(block, TOG, SI_SHOW_ZBUF, B_SIMA_SHOW_ZBUF, ICON_SOLID, xco,0,XIC,YIC, &G.sima->flag, 0, 0, 0, 0, "Draws zbuffer values");
+				xco+= XIC;
+			}
+			else G.sima->flag &= ~SI_SHOW_ZBUF;	/* no confusing display for non-zbuf images */
+		}		
+		uiBlockEndAlign(block);
+		xco+= 8;
 	}
 
 	/* draw LOCK */
 	uiDefIconButS(block, ICONTOG, 0, ICON_UNLOCKED,	xco,0,XIC,YIC, &(G.sima->lock), 0, 0, 0, 0, "Updates other affected window spaces automatically to reflect changes in real time");
-	
+
 	/* Always do this last */
 	curarea->headbutlen= xco+2*XIC;
 	
