@@ -4,7 +4,7 @@
  * Functions to draw the "User Preferences" window header
  * and handle user events sent to it.
  * 
- * $Id: header_info.c,v 1.60 2005/05/23 18:20:20 lukep Exp $
+ * $Id: header_info.c,v 1.90 2006/07/11 09:07:18 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,6 +43,7 @@
 #include <config.h>
 #endif
 
+#include "DNA_group_types.h"
 #include "DNA_ID.h"
 #include "DNA_image_types.h"
 #include "DNA_lamp_types.h"
@@ -57,6 +58,8 @@
 #include "BDR_editcurve.h"
 #include "BDR_editmball.h"
 #include "BDR_editobject.h"
+#include "BDR_editface.h"
+#include "BDR_vpaint.h"
 
 #include "BIF_editarmature.h"
 #include "BIF_editfont.h"
@@ -73,14 +76,17 @@
 #include "BIF_toets.h"
 #include "BIF_toolbox.h"
 #include "BIF_usiblender.h"
+#include "BIF_writeimage.h"
 #include "BIF_drawscene.h"
 
 #include "BKE_blender.h"
+#include "BKE_depsgraph.h"
 #include "BKE_exotic.h"
 #include "BKE_global.h"
 #include "BKE_image.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
+#include "BKE_node.h"
 #include "BKE_packedFile.h"
 #include "BKE_scene.h"
 #include "BKE_world.h"
@@ -239,51 +245,37 @@ int buttons_do_unpack()
 {
 	int how;
 	char menu[2048];
-	char line[128];
+	char *line = menu;
 	int ret_value = RET_OK, count = 0;
 
 	count = countPackedFiles();
 
-	if (count) {
-		if (count == 1) {
-			sprintf(menu, "Unpack 1 file%%t");
-		} else {
-			sprintf(menu, "Unpack %d files%%t", count);
-		}
-		
-		sprintf(line, "|Use files in current directory (create when necessary)%%x%d", PF_USE_LOCAL);
-		strcat(menu, line);
-	
-		sprintf(line, "|Write files to current directory (overwrite existing files)%%x%d", PF_WRITE_LOCAL);
-		strcat(menu, line);
-	
-		sprintf(line, "|%%l|Use files in original location (create when necessary)%%x%d", PF_USE_ORIGINAL);
-		strcat(menu, line);
-	
-		sprintf(line, "|Write files to original location (overwrite existing files)%%x%d", PF_WRITE_ORIGINAL);
-		strcat(menu, line);
-	
-		sprintf(line, "|%%l|Disable AutoPack, keep all packed files %%x%d", PF_KEEP);
-		strcat(menu, line);
-	
-		sprintf(line, "|Ask for each file %%x%d", PF_ASK);
-		strcat(menu, line);
-		
-		how = pupmenu(menu);
-		
-		if(how != -1) {
-			if (how != PF_KEEP) {
-				unpackAll(how);
-			}
-			G.fileflags &= ~G_AUTOPACK;
-		} else {
-			ret_value = RET_CANCEL;
-		}
-	} else {
+	if(!count) {
 		pupmenu("No packed files. Autopack disabled");
+		return ret_value;
 	}
+	if (count == 1)
+		line += sprintf(line, "Unpack 1 file%%t");
+	else
+		line += sprintf(line, "Unpack %d files%%t", count);
 	
-	return (ret_value);
+	line += sprintf(line, "|Use files in current directory (create when necessary)%%x%d", PF_USE_LOCAL);
+	line += sprintf(line, "|Write files to current directory (overwrite existing files)%%x%d", PF_WRITE_LOCAL);
+	line += sprintf(line, "|%%l|Use files in original location (create when necessary)%%x%d", PF_USE_ORIGINAL);
+	line += sprintf(line, "|Write files to original location (overwrite existing files)%%x%d", PF_WRITE_ORIGINAL);
+	line += sprintf(line, "|%%l|Disable AutoPack, keep all packed files %%x%d", PF_KEEP);
+	line += sprintf(line, "|Ask for each file %%x%d", PF_ASK);
+
+	how = pupmenu(menu);
+
+	if(how == -1)
+		ret_value = RET_CANCEL;
+	else {
+		if (how != PF_KEEP) unpackAll(how);
+		G.fileflags &= ~G_AUTOPACK;
+ 	}
+ 	
+	return ret_value;
 }
 
 /* here, because of all creator stuff */
@@ -306,11 +298,16 @@ Scene *copy_scene(Scene *sce, int level)
 	id_us_plus((ID *)scen->world);
 	id_us_plus((ID *)scen->set);
 
-	scen->ed= 0;
-	scen->radio= 0;
+	scen->ed= NULL;
+	scen->radio= NULL;
 	scen->theDag= NULL;
-	scen->dagisvalid= 0;
+	scen->toolsettings= MEM_dupallocN(sce->toolsettings);
 
+	duplicatelist(&(scen->markers), &(sce->markers));
+	duplicatelist(&(scen->r.layers), &(sce->r.layers));
+	
+	scen->nodetree= ntreeCopyTree(sce->nodetree, 0);
+	
 	obase= sce->base.first;
 	base= scen->base.first;
 	while(base) {
@@ -325,6 +322,7 @@ Scene *copy_scene(Scene *sce, int level)
 
 	/* level 1 */
 	G.scene= scen;
+	
 	single_object_users(0);
 
 	/*	camera */
@@ -339,12 +337,14 @@ Scene *copy_scene(Scene *sce, int level)
 		single_obdata_users(0);
 		single_mat_users_expand();
 		single_tex_users_expand();
+		
+		scen->radio= MEM_dupallocN(sce->radio);
+		
 	}
 
 	clear_id_newpoins();
 
 	BPY_copy_scriptlink(&sce->scriptlink);
-
 	// make a private copy of the avicodecdata
 
 	if (sce->r.avicodecdata) {
@@ -446,8 +446,13 @@ void do_info_buttons(unsigned short event)
 			nr= pupmenu("Add scene%t|Empty|Link Objects|Link ObData|Full Copy");
 			if(nr<= 0) return;
 			if(nr==1) {
+				ListBase lb;
+				
 				sce= add_scene(G.scene->id.name+2);
+				/* pretty bad ass copying here. we should use copy_scene to do all (ton) */
+				lb= sce->r.layers;
 				sce->r= G.scene->r;
+				sce->r.layers= lb;
 #ifdef _WIN32
 				if (sce->r.avicodecdata) {
 					sce->r.avicodecdata = MEM_dupallocN(G.scene->r.avicodecdata);
@@ -466,7 +471,7 @@ void do_info_buttons(unsigned short event)
 
 			set_scene(sce);
 		}
-		BIF_preview_changed(G.buts);
+		BIF_preview_changed(ID_TE);
 
 		break;
 	case B_INFODELSCE:
@@ -475,7 +480,14 @@ void do_info_buttons(unsigned short event)
 		else if(G.scene->id.next) sce= G.scene->id.next;
 		else return;
 		if(okee("Delete current scene")) {
-
+			
+			/* exit modes... could become single call once */
+			exit_editmode(1);
+			if(G.f & G_VERTEXPAINT) set_vpaint(); /* Switch off vertex paint */
+			if(G.f & G_TEXTUREPAINT) set_texturepaint(); /* Switch off tex paint */
+			if(G.f & G_WEIGHTPAINT) set_wpaint();		/* Switch off weight paint */
+			if(G.f & G_FACESELECT) set_faceselect(); /* Switch off face select */
+			
 			/* check all sets */
 			sce1= G.main->scene.first;
 			while(sce1) {
@@ -533,8 +545,7 @@ static int write_runtime(char *str, char *exename)
 #endif
 	if (ext && (!BLI_testextensie(str, ext))) {
 		freestr= MEM_mallocN(strlen(str) + strlen(ext) + 1, "write_runtime_check");
-		strcpy(freestr, str);
-		strcat(freestr, ext);
+		sprintf(freestr,"%s%s", str, ext);
 		str= freestr;
 	}
 
@@ -573,95 +584,6 @@ static void write_runtime_check(char *str)
 /************************** MAIN MENU *****************************/
 /************************** FILE *****************************/
 
-#if 0
-static void do_info_file_optionsmenu(void *arg, int event)
-{
-	G.fileflags ^= (1 << event);
-
-	// allqueue(REDRAWINFO, 0);
-}
-#endif
-
-#if GAMEBLENDER == 1
-
-#if 0
-static uiBlock *info_file_optionsmenu(void *arg_unused)
-{
-	uiBlock *block;
-	short yco= 0, xco = 20;
-
-	block= uiNewBlock(&curarea->uiblocks, "runtime_options", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
-	uiBlockSetButmFunc(block, do_info_file_optionsmenu, NULL);
-	uiBlockSetXOfs(block,-40);	// offset to parent button
-	
-	/* flags are case-values */
-	uiDefBut(block, BUTM, 1, "Compress File",	xco, yco-=20, 100, 19, NULL, 0.0, 0.0, 0, G_FILE_COMPRESS_BIT, "Enables file compression");
-/*
-	uiDefBut(block, BUTM, 1, "Sign File",	xco, yco-=20, 100, 19, NULL, 0.0, 0.0, 0, G_FILE_SIGN_BIT, "Add signature to file");
-	uiDefBut(block, BUTM, 1, "Lock File",	xco, yco-=20, 100, 19, NULL, 0.0, 0.0, 0, G_FILE_LOCK_BIT, "Protect the file from editing by others");
-*/
-	uiTextBoundsBlock(block, 50);
-
-	/* Toggle buttons */
-	
-	yco= 0;
-	xco -= 20;
-	uiBlockSetEmboss(block, UI_EMBOSSM);
-	uiBlockSetButmFunc(block, NULL, NULL);
-	/* flags are defines */
-	uiDefIconButI(block, ICONTOG|BIT|G_FILE_COMPRESS_BIT, 0, ICON_CHECKBOX_DEHLT, xco, yco-=20, 19, 19, &G.fileflags, 0.0, 0.0, 0, 0, "");
-/*
-	uiDefIconButI(block, ICONTOG|BIT|G_FILE_SIGN_BIT, 0, ICON_CHECKBOX_DEHLT, xco, yco-=20, 19, 19, &G.fileflags, 0.0, 0.0, 0, 0, "");
-	uiDefIconButI(block, ICONTOG|BIT|G_FILE_LOCK_BIT, 0, ICON_CHECKBOX_DEHLT, xco, yco-=20, 19, 19, &G.fileflags, 0.0, 0.0, 0, 0, "");
-*/
-	uiBlockSetDirection(block, UI_RIGHT);
-		
-	return block;
-}
-
-static uiBlock *info_runtime_optionsmenu(void *arg_unused)
-{
-	uiBlock *block;
-	short yco= 0, xco = 20;
-
-	block= uiNewBlock(&curarea->uiblocks, "add_surfacemenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
-	uiBlockSetXOfs(block, -40);  // offset to parent button
-	uiBlockSetEmboss(block, UI_EMBOSSM);
-
-	uiDefBut(block, LABEL, 0, "Size options:",		xco, yco-=20, 114, 19, 0, 0.0, 0.0, 0, 0, "");
-	uiDefButS(block, NUM, 0, "X:",		xco+19, yco-=20, 95, 19,		&G.scene->r.xplay, 10.0, 2000.0, 0, 0, "Displays current X screen/window resolution. Click to change.");
-	uiDefButS(block, NUM, 0, "Y:",		xco+19, yco-=20, 95, 19, &G.scene->r.yplay, 10.0, 2000.0, 0, 0, "Displays current Y screen/window resolution. Click to change.");
-
-	uiDefBut(block, SEPR, 0, "",		xco, yco-=4, 114, 4, NULL, 0.0, 0.0, 0, 0, "");
-
-	uiDefBut(block, LABEL, 0, "Fullscreen options:",		xco, yco-=20, 114, 19, 0, 0.0, 0.0, 0, 0, "");
-	uiDefButS(block, TOG, 0, "Fullscreen", xco + 19, yco-=20, 95, 19, &G.scene->r.fullscreen, 0.0, 0.0, 0, 0, "Starts player in a new fullscreen display");
-	uiDefButS(block, NUM, 0, "Freq:",	xco+19, yco-=20, 95, 19, &G.scene->r.freqplay, 10.0, 120.0, 0, 0, "Displays clock frequency of fullscreen display. Click to change.");
-	uiDefButS(block, NUM, 0, "Bits:",	xco+19, yco-=20, 95, 19, &G.scene->r.depth, 1.0, 32.0, 0, 0, "Displays bit depth of full screen display. Click to change.");
-
-	uiDefBut(block, SEPR, 0, "",		xco, yco-=4, 114, 4, NULL, 0.0, 0.0, 0, 0, "");
-
-	/* stereo settings */
-	/* can't use any definition from the game engine here so hardcode it. Change it here when it changes there!
-	 * RAS_IRasterizer has definitions:
-	 * RAS_STEREO_NOSTEREO		 1
-	 * RAS_STEREO_QUADBUFFERED 2
-	 * RAS_STEREO_ABOVEBELOW	 3
-	 * RAS_STEREO_INTERLACED	 4	 future
-	 */
-	uiDefBut(block, LABEL, 0, "Stereo options", xco, yco-=20, 114, 19, 0, 0.0, 0.0, 0, 0, "");
-	uiDefButS(block, ROW, 0, "no stereo", xco+19, yco-=20, 95, 19, &(G.scene->r.stereomode), 6.0, 1.0, 0, 0, "Disables stereo");
-	uiDefButS(block, ROW, 0, "h/w pageflip", xco+19, yco-=20, 95, 19, &(G.scene->r.stereomode), 6.0, 2.0, 0, 0, "Enables hardware pageflip stereo method");
-	uiDefButS(block, ROW, 0, "syncdoubling", xco+19, yco-=20, 95, 19, &(G.scene->r.stereomode), 6.0, 3.0, 0, 0, "Enables syncdoubling stereo method");
-
-	uiBlockSetDirection(block, UI_RIGHT);
-	uiTextBoundsBlock(block, 50);
-		
-	return block;
-}
-#endif
-
-#endif
 
 static void do_info_file_importmenu(void *arg, int event)
 {
@@ -819,15 +741,6 @@ static void do_info_filemenu(void *arg, int event)
 	case 1: /* open */
 		activate_fileselect(FILE_BLENDER, "Open", G.sce, BIF_read_file);
 		break;
-	case 2: /* reopen last */
-		{
-			char *s= MEM_mallocN(strlen(G.sce) + 11 + 1, "okee_reload");
-			strcpy(s, "Open file: ");
-			strcat(s, G.sce);
-			if (okee(s)) BIF_read_file(G.sce);
-			MEM_freeN(s);
-		}
-		break;
 	case 3: /* append */
 		activate_fileselect(FILE_LOADLIB, "Load Library", G.lib, 0);
 		break;
@@ -846,7 +759,7 @@ static void do_info_filemenu(void *arg, int event)
 		}
 		break;
 	case 6: /* save image */
-		BIF_save_rendered_image();
+		BIF_save_rendered_image_fs(0);
 		break;
 	case 22: /* save runtime */
 		activate_fileselect(FILE_SPECIAL, "Save Runtime", "", write_runtime_check);
@@ -900,10 +813,53 @@ static void do_info_filemenu(void *arg, int event)
 	case 31: /* save default settings */
 		BIF_write_homefile();
 		break;
+
+	case 35: /* compress toggle */
+		U.flag ^= (USER_FILECOMPRESS);
+		break;
 	}
 	allqueue(REDRAWINFO, 0);
 }
 
+static void do_info_operecentmenu(void *arg, int event)
+{
+	struct RecentFile *recent;
+
+	if(event==0 && G.sce[0]) {
+		BIF_read_file(G.sce);
+	}
+	else {	/* Global */
+		recent = BLI_findlink(&(G.recent_files), event-1);
+		BIF_read_file(recent->filename);
+	}
+}
+
+static uiBlock *info_openrecentmenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco = 20, menuwidth = 120, i;
+	struct RecentFile *recent;
+
+	block= uiNewBlock(&curarea->uiblocks, "info_openrecentmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_info_operecentmenu, NULL);
+
+	if (G.sce[0]) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, G.sce, 0, yco-=20,
+				menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
+	}
+
+	for (recent = G.recent_files.first, i=0; i<10 && recent; recent = recent->next, i++) {
+		if (strcmp(recent->filename, G.sce)!=0) {
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, recent->filename, 0, yco-=20,
+					menuwidth, 19, NULL, 0.0, 0.0, 1, i+1, "");
+		}
+	}
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+	return block;
+}
+
 static uiBlock *info_filemenu(void *arg_unused)
 {
 	uiBlock *block;
@@ -915,18 +871,24 @@ static uiBlock *info_filemenu(void *arg_unused)
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "New|Ctrl X",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Open...|F1",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Reopen Last|Ctrl O",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 2, "");
+	uiDefIconTextBlockBut(block, info_openrecentmenu, NULL, ICON_RIGHTARROW_THIN, "Open Recent",0, yco-=20, 120, 19, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Recover Last Session",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 15, "");
 
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save|Ctrl W",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save As...|F2",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 4, "");
+	
+	if(U.flag & USER_FILECOMPRESS) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Compress File",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 35, "Enable file compression");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Compress File",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 35, "Enable file compression");
+	}
 
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save Image...|F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 6, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Dump 3DView|Ctrl F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 24, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Dump Subwindow|Ctrl F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 24, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Dump Screen|Ctrl Shift F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 25, "");
 #if GAMEBLENDER == 1
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save Runtime...",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 22, "");
@@ -1251,6 +1213,42 @@ static uiBlock *info_add_lampmenu(void *arg_unused)
 	return block;
 }
 
+static void do_info_add_groupmenu(void *arg, int event)
+{
+	Object *ob;
+	
+	add_object_draw(OB_EMPTY);
+	ob= OBACT;
+	
+	ob->dup_group= BLI_findlink(&G.main->group, event);
+	if(ob->dup_group) {
+		id_us_plus((ID *)ob->dup_group);
+		ob->transflag |= OB_DUPLIGROUP;
+		DAG_scene_sort(G.scene);
+	}
+}
+
+
+static uiBlock *info_add_groupmenu(void *arg_unused)
+{
+	Group *group;
+	uiBlock *block;
+	short yco= 0;
+	int a;
+	
+	block= uiNewBlock(&curarea->uiblocks, "add_groupmenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_info_add_groupmenu, NULL);
+	
+	for(a=0, group= G.main->group.first; group; group= group->id.next, a++) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, group->id.name+2,				0, yco-=20, 160, 19, NULL, 0.0, 0.0, 1, a, "");
+	}
+	
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 50);
+		
+	return block;
+}
+
 void do_info_addmenu(void *arg, int event)
 {
 	switch(event) {		
@@ -1278,10 +1276,6 @@ void do_info_addmenu(void *arg, int event)
 			/* Camera */
 			add_object_draw(OB_CAMERA);
 			break;
-		case 7:
-			/* Lamp */
-		//	add_object_draw(OB_LAMP);
-			break;
 		case 8:
 			/* Armature */
 			add_primitiveArmature(OB_ARMATURE);
@@ -1290,6 +1284,9 @@ void do_info_addmenu(void *arg, int event)
 			/* Lattice */
 			add_object_draw(OB_LATTICE);
 			break;
+		case 10:
+			/* group instance not yet */
+			break;
 		default:
 			break;
 	}
@@ -1315,14 +1312,16 @@ static uiBlock *info_addmenu(void *arg_unused)
 	
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, 120, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
+	uiDefIconTextBlockBut(block, info_add_groupmenu, NULL, ICON_RIGHTARROW_THIN, "Group", 0, yco-=20, 120, 19, "");
+	
+	uiDefBut(block, SEPR, 0, "",					0, yco-=6, 120, 6, NULL, 0.0, 0.0, 0, 0, "");
+	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Camera",				0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 6, "");
-//	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Lamp",				0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 7, "");
 	uiDefIconTextBlockBut(block, info_add_lampmenu, NULL, ICON_RIGHTARROW_THIN, "Lamp", 0, yco-=20, 120, 19, "");
 	
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, 120, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
-	/* armature needs 3d window to draw */
-	//uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Armature",			0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Armature",			0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 8, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Lattice",			0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 9, "");
 
 	uiBlockSetDirection(block, UI_DOWN);
@@ -1333,14 +1332,18 @@ static uiBlock *info_addmenu(void *arg_unused)
 
 /************************** GAME *****************************/
 
+	
 static void do_info_gamemenu(void *arg, int event)
 {
 	switch (event) {
-	case G_FILE_ENABLE_ALL_FRAMES_BIT:
-	case G_FILE_SHOW_FRAMERATE_BIT:
-	case G_FILE_SHOW_DEBUG_PROPS_BIT:
-	case G_FILE_AUTOPLAY_BIT:
-		G.fileflags ^= (1 << event);
+	case G_FILE_ENABLE_ALL_FRAMES:
+	case G_FILE_DIAPLAY_LISTS:
+	case G_FILE_SHOW_FRAMERATE:
+	case G_FILE_SHOW_DEBUG_PROPS:
+	case G_FILE_AUTOPLAY:
+	case G_FILE_GAME_TO_IPO:
+	case G_FILE_GAME_MAT:
+		G.fileflags ^= event;
 		break;
 	default:
 		; /* ignore the rest */
@@ -1362,34 +1365,54 @@ static uiBlock *info_gamemenu(void *arg_unused)
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 1, 0, "");
 
 
-	if(G.fileflags & (1 << G_FILE_ENABLE_ALL_FRAMES_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES_BIT, "");
+	if(G.fileflags & G_FILE_ENABLE_ALL_FRAMES) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES, "");
 	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES_BIT, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES, "");
+	}
+	
+	if(G.fileflags & G_FILE_GAME_TO_IPO) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Record Game Physics to IPO",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_TO_IPO, "");
+	} else {
+
+	if(G.fileflags & G_FILE_DIAPLAY_LISTS) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Generate Display Lists",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_DIAPLAY_LISTS, "");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Generate Display Lists",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_DIAPLAY_LISTS, "");
+	}	
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Record Game Physics to IPO",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_TO_IPO, "");
+	}
+	
+	if(G.fileflags & G_FILE_GAME_MAT) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Use Blender Materials",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_MAT, "");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Use Blender Materials",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_MAT, "");
+	}	
+
+
+
+	if(G.fileflags & G_FILE_SHOW_FRAMERATE) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE, "");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE, "");
 	}
 
-	if(G.fileflags & (1 << G_FILE_SHOW_FRAMERATE_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE_BIT, "");
+	if(G.fileflags & G_FILE_SHOW_DEBUG_PROPS) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS, "");
 	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE_BIT, "");
-	}
-
-	if(G.fileflags & (1 << G_FILE_SHOW_DEBUG_PROPS_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS_BIT, "");
-	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS_BIT, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS, "");
 	}
 	
 	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 1, 0, "");
 
-	if(G.fileflags & (1 << G_FILE_AUTOPLAY_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY_BIT, "");
+	if(G.fileflags & G_FILE_AUTOPLAY) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY, "");
 	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY_BIT, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY, "");
 	}
 
 	uiBlockSetDirection(block, UI_DOWN);
-	uiTextBoundsBlock(block, 50);
+	uiTextBoundsBlock(block, 70);
 	
 	return block;
 }
@@ -1512,25 +1535,10 @@ static uiBlock *info_timelinemenu(void *arg_unused)
 
 /************************** RENDER *****************************/
 
-/* copied from buttons.c. .. probably not such a good idea!? */
-static void run_playanim(char *file)
-{
-	extern char bprogname[];	/* usiblender.c */
-	char str[FILE_MAXDIR+FILE_MAXFILE];
-	int pos[2], size[2];
-
-	calc_renderwin_rectangle(G.winpos, pos, size);
-
-	sprintf(str, "%s -a -p %d %d \"%s\"", bprogname, pos[0], pos[1], file);
-	system(str);
-}
-
 static void do_info_rendermenu(void *arg, int event)
 {
-	char file[FILE_MAXDIR+FILE_MAXFILE];
 
-	extern void makeavistring(char *string);
-	extern void makeqtstring (char *string);
+	extern void playback_anim();
 
 	switch(event) {
 		
@@ -1548,22 +1556,7 @@ static void do_info_rendermenu(void *arg, int event)
 		BIF_toggle_render_display();
 		break;
 	case 5:
-#ifdef WITH_QUICKTIME
-		if(G.scene->r.imtype == R_QUICKTIME)
-			makeqtstring(file);
-		else
-#endif
-			makeavistring(file);
-		if(BLI_exist(file)) {
-			run_playanim(file);
-		}
-		else {
-			makepicstring(file, G.scene->r.sfra);
-			if(BLI_exist(file)) {
-				run_playanim(file);
-			}
-			else error("Can't find image: %s", file);
-		}
+		playback_anim();
 		break;
 	case 6:
 		/* dodgy hack turning on SHIFT key to do a proper render border select
@@ -1581,8 +1574,8 @@ static void do_info_rendermenu(void *arg, int event)
 
 		break;
 
-		case 7:
-		extern_set_butspace(F10KEY);
+	case 7:
+		extern_set_butspace(F10KEY, 0);
 		break;
 	}
 	allqueue(REDRAWINFO, 0);
@@ -1599,12 +1592,12 @@ static uiBlock *info_rendermenu(void *arg_unused)
 	uiBlockSetButmFunc(block, do_info_rendermenu, NULL);
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Render Current Frame|F12",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 0, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Render Animation",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Render Animation|Ctrl F12",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 1, "");
 
 	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Show Render Buffer|F11",		0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 4, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Play Back Rendered Animation",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Play Back Rendered Animation|Ctrl F11",	0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
 
 	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
@@ -1766,37 +1759,51 @@ char info_time_str[32]="";
 
 static void info_text(int x, int y)
 {
-	Object *ob;
+	Object *ob= OBACT;
 	extern float hashvectf[];
-	extern int mem_in_use;
+	extern unsigned long mem_in_use, mmap_in_use;
 	unsigned int swatch_color;
 	float fac1, fac2, fac3;
-	char infostr[300];
-	char *headerstr;
+	char infostr[300], memstr[64];
+	char *headerstr, *s;
 	int hsize;
 
+	s= memstr + sprintf(memstr," | Mem:%.2fM ", ((mem_in_use-mmap_in_use)>>10)/1024.0);
+	if(mmap_in_use)
+		sprintf(s,"(%.2fM) ", ((mmap_in_use)>>10)/1024.0);
+
+	
 	if(G.obedit) {
+		s = infostr;
+
+		s+= sprintf(s, "%s", G.editModeTitleExtra);
 		if(G.obedit->type==OB_MESH) {
 			if(G.scene->selectmode & SCE_SELECT_VERTEX)
-				sprintf(infostr,"Ve:%d-%d | Ed:%d-%d | Fa:%d-%d | Mem:%.2fM ",
-						G.totvertsel, G.totvert, G.totedgesel, G.totedge, G.totfacesel, G.totface, (mem_in_use>>10)/1024.0);
+				s+= sprintf(s,"Ve:%d-%d | Ed:%d-%d | Fa:%d-%d",
+						G.totvertsel, G.totvert, G.totedgesel, G.totedge, G.totfacesel, G.totface);
 			else if(G.scene->selectmode & SCE_SELECT_EDGE)
-				sprintf(infostr,"Ed:%d-%d | Fa:%d-%d | Mem:%.2fM ",
-						G.totedgesel, G.totedge, G.totfacesel, G.totface, (mem_in_use>>10)/1024.0);
+				s+= sprintf(s,"Ed:%d-%d | Fa:%d-%d",
+						G.totedgesel, G.totedge, G.totfacesel, G.totface);
 			else 
-				sprintf(infostr,"Fa:%d-%d | Mem:%.2fM ",
-						G.totfacesel, G.totface, (mem_in_use>>10)/1024.0);
+				s+= sprintf(s,"Fa:%d-%d", G.totfacesel, G.totface);
+		}
+		else if(G.obedit->type==OB_ARMATURE) {
+			s+= sprintf(s,"Ve:%d-%d | Bo:%d-%d", G.totvertsel, G.totvert, G.totbonesel, G.totbone);
 		}
 		else {
-			sprintf(infostr,"Ve:%d-%d | Mem:%.2fM ",
-					G.totvertsel, G.totvert, (mem_in_use>>10)/1024.0);
+			s+= sprintf(s,"Ve:%d-%d", G.totvertsel, G.totvert);
 		}
+
+		strcat(s, memstr);
+	}
+	else if(ob && (ob->flag & OB_POSEMODE)) {
+		sprintf(infostr,"Bo:%d-%d %s",
+					G.totbonesel, G.totbone, memstr);
 	}
 	else {
-		sprintf(infostr,"Ve:%d | Fa:%d | Ob:%d-%d | La:%d | Mem:%.2fM | Time:%s | ",
-			G.totvert, G.totface, G.totobj, G.totobjsel, G.totlamp,  (mem_in_use>>10)/1024.0, info_time_str);
+		sprintf(infostr,"Ve:%d | Fa:%d | Ob:%d-%d | La:%d %s | Time:%s | ",
+			G.totvert, G.totface, G.totobj, G.totobjsel, G.totlamp, memstr, info_time_str);
 	}
-	ob= OBACT;
 	if(ob) {
 		strcat(infostr, ob->id.name+2);
 	}
@@ -1826,17 +1833,19 @@ static void info_text(int x, int y)
 	cpack( swatch_color );
 	glRecti(x-24,  y-6,  x-30+hsize,	y+14);
 
-	glColor3ub(0, 0, 0);
-
+	glColor3ub(0, 0, 0); /* makes text black colored rect */
+	
 	glRasterPos2i(x, y);
 	BIF_RasterPos(x, y);
 
 	BIF_DrawString(G.font, headerstr, (U.transopts & USER_TR_MENUS));
 	hsize= BIF_GetStringWidth(G.font, headerstr, (U.transopts & USER_TR_BUTTONS));
 	
+	BIF_ThemeColor(TH_MENU_TEXT); /* makes text readable on dark theme */
+	
 	glRasterPos2i(x+hsize+10,	y);
 	BIF_RasterPos(x+hsize+10,	y);
-
+	
 	BIF_DrawString(G.font, infostr, (U.transopts & USER_TR_MENUS));
 }
 
@@ -1852,11 +1861,11 @@ void info_buttons(void)
 	else uiBlockSetCol(block, TH_HEADERDESEL);
 	
 	if(curarea->flag & HEADER_NO_PULLDOWN) {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Enables display of pulldown menus");
 	} else {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Hides pulldown menus");
 	}
@@ -1909,12 +1918,12 @@ void info_buttons(void)
 		uiDefIconTextButC(block, ICONTEXTROW,B_NEWSPACE, ICON_VIEW3D, windowtype_pup(), 8,0,XIC+10,YIC, &(curarea->butspacetype), 1.0, SPACEICONMAX, 0, 0, "Displays Current Window Type. Click for menu of available types.");
 		
 		/* STD SCREEN BUTTONS */
-		xco= std_libbuttons(block, xco, 0, 0, NULL, B_INFOSCR, (ID *)G.curscreen, 0, &G.curscreen->screennr, 1, 1, B_INFODELSCR, 0, 0);
+		xco= std_libbuttons(block, xco, 0, 0, NULL, B_INFOSCR, ID_SCR, 0, (ID *)G.curscreen, 0, &G.curscreen->screennr, 1, 1, B_INFODELSCR, 0, 0);
 		
 		xco +=8;
 	
 		/* STD SCENE BUTTONS */
-		xco= std_libbuttons(block, xco, 0, 0, NULL, B_INFOSCE, (ID *)G.scene, 0, &G.curscreen->scenenr, 1, 1, B_INFODELSCE, 0, 0);
+		xco= std_libbuttons(block, xco, 0, 0, NULL, B_INFOSCE, ID_SCE, 0, (ID *)G.scene, 0, &G.curscreen->scenenr, 1, 1, B_INFODELSCE, 0, 0);
 	}
 	else xco= 430;
 	
