@@ -4,7 +4,7 @@
  * Functions to draw the "Audio Timeline" window header
  * and handle user events sent to it.
  *
- * $Id: header_sound.c,v 1.16 2005/03/23 21:10:02 phase Exp $
+ * $Id: header_sound.c,v 1.20 2006/01/28 18:33:18 hos Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -163,6 +163,7 @@ void do_sound_buttons(unsigned short event)
 		}
 		G.v2d->cur= G.v2d->tot;
 		test_view2d(G.v2d, curarea->winx, curarea->winy);
+		view2d_do_locks(curarea, V2D_LOCK_COPY);
 		scrarea_queue_winredraw(curarea);
 		break;
 	}
@@ -252,14 +253,14 @@ void sound_buttons(void)
 
 	uiBlockSetEmboss(block, UI_EMBOSSN);
 	if (curarea->flag & HEADER_NO_PULLDOWN) {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, 
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, 
 					  ICON_DISCLOSURE_TRI_RIGHT,
 					  xco,2,XIC,YIC-2,
 					  &(curarea->flag), 0, 0, 0, 0, 
 					  "Show pulldown menus");
 	}
 	else {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, 
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, 
 					  ICON_DISCLOSURE_TRI_DOWN,
 					  xco,2,XIC,YIC-2,
 					  &(curarea->flag), 0, 0, 0, 0, 
@@ -279,8 +280,8 @@ void sound_buttons(void)
 
 	}
 
-	uiBlockSetEmboss(block, UI_EMBOSSX);
-	xco= std_libbuttons(block, xco+8, 0, 0, NULL, B_SOUNDBROWSE, (ID *)G.ssound->sound, 0, &(G.ssound->sndnr), 1, 0, 0, 0, 0);	
+	uiBlockSetEmboss(block, UI_EMBOSS);
+	xco= std_libbuttons(block, xco+8, 0, 0, NULL, B_SOUNDBROWSE, ID_SO, 0, (ID *)G.ssound->sound, 0, &(G.ssound->sndnr), 1, 0, 0, 0, 0);	
 
 	if(G.ssound->sound) {
 		bSound *sound= G.ssound->sound;
