@@ -1,5 +1,6 @@
+
 /**
- * $Id: interface_draw.c,v 1.38 2005/05/29 10:59:50 ton Exp $
+ * $Id: interface_draw.c,v 1.58 2006/06/29 09:44:08 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,10 +42,6 @@
 #include <string.h>
 #include <ctype.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifndef WIN32
 #include <unistd.h>
 #else
@@ -58,14 +55,24 @@
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
+#include "DNA_color_types.h"
+#include "DNA_key_types.h"
+#include "DNA_packedFile_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
+#include "DNA_texture_types.h"
 #include "DNA_userdef_types.h"
 #include "DNA_vec_types.h"
+#include "DNA_vfont_types.h"
 
 #include "BKE_blender.h"
-#include "BKE_utildefines.h"
+#include "BKE_colortools.h"
+#include "BKE_font.h"
 #include "BKE_global.h"
+#include "BKE_key.h"
+#include "BKE_utildefines.h"
+
+#include "datatoc.h"            /* std font */
 
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
@@ -78,11 +85,16 @@
 #include "BIF_space.h"
 #include "BIF_glutil.h"
 #include "BIF_interface.h"
+#include "BIF_interface_icons.h"
 #include "BIF_butspace.h"
 #include "BIF_language.h"
 
 #include "BSE_view.h"
 
+#ifdef INTERNATIONAL
+#include "FTF_Api.h"
+#endif
+
 #include "mydevice.h"
 #include "interface.h"
 #include "blendef.h"
@@ -90,9 +102,6 @@
 // globals
 extern float UIwinmat[4][4];
 
-// local prototypes 
-void uiDrawBoxShadow(unsigned char alpha, float minx, float miny, float maxx, float maxy);
-
 
 /* ************** safe rasterpos for pixmap alignment with pixels ************* */
 
@@ -148,44 +157,54 @@ void uiEmboss(float x1, float y1, float x2, float y2, int sel)
 
 /* ************** GENERIC ICON DRAW, NO THEME HERE ************* */
 
+/* icons have been standardized... and this call draws in untransformed coordinates */
+#define ICON_HEIGHT		16.0f
+
 static void ui_draw_icon(uiBut *but, BIFIconID icon)
 {
-	// void BIF_icon_pos(float xs, float ys);
+	float xs=0, ys=0, aspect, height;
 	int blend= 0;
-	float xs=0, ys=0;
 
-	// this icon doesn't need draw...
+	/* this icon doesn't need draw... */
 	if(icon==ICON_BLANK1) return;
-
+	
+	/* we need aspect from block, for menus... these buttons are scaled in uiPositionBlock() */
+	aspect= but->block->aspect;
+	if(aspect != but->aspect) {
+		/* prevent scaling up icon in pupmenu */
+		if (aspect < 1.0f) {			
+			height= ICON_HEIGHT;
+			aspect = 1.0f;
+			
+		}
+		else 
+			height= ICON_HEIGHT/aspect;
+	}
+	else
+		height= ICON_HEIGHT;
+	
 	if(but->flag & UI_ICON_LEFT) {
 		if (but->type==BUTM) {
 			xs= but->x1+1.0;
 		}
 		else if ((but->type==ICONROW) || (but->type==ICONTEXTROW)) {
-			xs= but->x1+4.0;
+			xs= but->x1+3.0;
 		}
 		else {
-			xs= but->x1+6.0;
+			xs= but->x1+4.0;
 		}
-		ys= (but->y1+but->y2- BIF_get_icon_height(icon))/2.0;
+		ys= (but->y1+but->y2- height)/2.0;
 	}
 	if(but->flag & UI_ICON_RIGHT) {
 		xs= but->x2-17.0;
-		ys= (but->y1+but->y2- BIF_get_icon_height(icon))/2.0;
+		ys= (but->y1+but->y2- height)/2.0;
 	}
 	if (!((but->flag & UI_ICON_RIGHT) || (but->flag & UI_ICON_LEFT))) {
-		xs= (but->x1+but->x2- BIF_get_icon_width(icon))/2.0;
-		ys= (but->y1+but->y2- BIF_get_icon_height(icon))/2.0;
+		xs= (but->x1+but->x2- height)/2.0;
+		ys= (but->y1+but->y2- height)/2.0;
 	}
 
-	glRasterPos2f(xs, ys);
-	// BIF_icon_pos(xs, ys);
-
-	if(but->aspect>1.1) glPixelZoom(1.0/but->aspect, 1.0/but->aspect);
-	else if(but->aspect<0.9) glPixelZoom(1.0/but->aspect, 1.0/but->aspect);
-
 	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	/* calculate blend color */
 	if ELEM4(but->type, ICONTOG, TOG, ROW, TOGN) {
@@ -193,12 +212,10 @@ static void ui_draw_icon(uiBut *but, BIFIconID icon)
 		else if(but->flag & UI_ACTIVE);
 		else blend= -60;
 	}
-	BIF_draw_icon_blended(icon, but->themecol, blend);
+	BIF_icon_draw_aspect_blended(xs, ys, icon, aspect, blend);
 	
-	glBlendFunc(GL_ONE, GL_ZERO);
 	glDisable(GL_BLEND);
 
-	glPixelZoom(1.0, 1.0);
 }
 
 
@@ -406,7 +423,6 @@ static void ui_default_iconrow_arrows(float x1, float y1, float x2, float y2)
 {
 	glEnable( GL_POLYGON_SMOOTH );
 	glEnable( GL_BLEND );
-	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
 	
 	glShadeModel(GL_FLAT);
 	glBegin(GL_TRIANGLES);
@@ -430,7 +446,6 @@ static void ui_default_menu_arrows(float x1, float y1, float x2, float y2)
 {
 	glEnable( GL_POLYGON_SMOOTH );
 	glEnable( GL_BLEND );
-	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
 	
 	glShadeModel(GL_FLAT);
 	glBegin(GL_TRIANGLES);
@@ -456,7 +471,6 @@ static void ui_default_num_arrows(float x1, float y1, float x2, float y2)
 
 		glEnable( GL_POLYGON_SMOOTH );
 		glEnable( GL_BLEND );
-		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
 		
 		glShadeModel(GL_FLAT);
 		glBegin(GL_TRIANGLES);
@@ -486,7 +500,6 @@ static void ui_tog3_invert(float x1, float y1, float x2, float y2, int seltype)
 	short alpha = 30;
 	
 	if (seltype == 0) {
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 		
 		glColor4ub(0, 0, 0, alpha);
@@ -497,7 +510,6 @@ static void ui_tog3_invert(float x1, float y1, float x2, float y2, int seltype)
 		
 		glDisable(GL_BLEND);
 	} else {
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 		
 		glColor4ub(255, 255, 255, alpha);
@@ -521,7 +533,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 		if (!((align == UI_BUT_ALIGN_DOWN) ||
 			(align == (UI_BUT_ALIGN_DOWN|UI_BUT_ALIGN_RIGHT)) ||
 			(align == (UI_BUT_ALIGN_DOWN|UI_BUT_ALIGN_LEFT)))) {
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			MM_WHITE_OP;
 			fdrawline(x1, y1-1, x2, y1-1);	
@@ -547,7 +558,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 		case UI_BUT_ALIGN_LEFT:
 			
 			/* RIGHT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -564,7 +574,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 		case UI_BUT_ALIGN_RIGHT:
 		
 			/* LEFT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -590,7 +599,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 		case UI_BUT_ALIGN_TOP|UI_BUT_ALIGN_RIGHT:
 		
 			/* LEFT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -607,7 +615,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 		case UI_BUT_ALIGN_TOP|UI_BUT_ALIGN_LEFT:
 		
 			/* RIGHT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -628,7 +635,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 		}
 	} 
 	else {	
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 		glShadeModel(GL_SMOOTH);
 		
@@ -663,7 +669,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 	case ICONROW:
 	case ICONTEXTROW:
 		/* DARKENED AREA */
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 		
 		glColor4ub(0, 0, 0, 30);
@@ -679,7 +684,6 @@ static void ui_default_button(int type, int colorid, float asp, float x1, float
 		break;
 	case MENU:
 		/* DARKENED AREA */
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 		
 		glColor4ub(0, 0, 0, 30);
@@ -708,7 +712,6 @@ static void ui_default_flat(int type, int colorid, float asp, float x1, float y1
 		if (!((align == UI_BUT_ALIGN_DOWN) ||
 			(align == (UI_BUT_ALIGN_DOWN|UI_BUT_ALIGN_RIGHT)) ||
 			(align == (UI_BUT_ALIGN_DOWN|UI_BUT_ALIGN_LEFT)))) {
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			MM_WHITE_OP;
 			fdrawline(x1, y1-1, x2, y1-1);	
@@ -734,7 +737,6 @@ static void ui_default_flat(int type, int colorid, float asp, float x1, float y1
 		case UI_BUT_ALIGN_LEFT:
 			
 			/* RIGHT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -751,7 +753,6 @@ static void ui_default_flat(int type, int colorid, float asp, float x1, float y1
 		case UI_BUT_ALIGN_RIGHT:
 		
 			/* LEFT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -777,7 +778,6 @@ static void ui_default_flat(int type, int colorid, float asp, float x1, float y1
 		case UI_BUT_ALIGN_TOP|UI_BUT_ALIGN_RIGHT:
 		
 			/* LEFT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -794,7 +794,6 @@ static void ui_default_flat(int type, int colorid, float asp, float x1, float y1
 		case UI_BUT_ALIGN_TOP|UI_BUT_ALIGN_LEFT:
 		
 			/* RIGHT OUTER SUNKEN EFFECT */
-			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
 			glShadeModel(GL_SMOOTH);
 			glBegin(GL_LINES);
@@ -816,7 +815,6 @@ static void ui_default_flat(int type, int colorid, float asp, float x1, float y1
 	} 
 	else {
 	
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		glEnable(GL_BLEND);
 		glShadeModel(GL_SMOOTH);
 		
@@ -1079,7 +1077,6 @@ static void round_button(float x1, float y1, float x2, float y2, float asp,
 	/* fake AA */
 	uiSetRoundBox(round);
 	glEnable( GL_BLEND );
-	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
 
 	BIF_GetThemeColor3ubv(colorid, col);
 		
@@ -1207,12 +1204,18 @@ static void ui_draw_round(int type, int colorid, float asp, float x1, float y1,
 /* super minimal button as used in logic menu */
 static void ui_draw_minimal(int type, int colorid, float asp, float x1, float y1, float x2, float y2, int flag)
 {
-	
+	/* too much space between buttons */
+	/* 
 	x1+= asp;
 	x2-= asp;
 	y1+= asp;
 	y2-= asp;
-
+	*/
+	
+	/* Less space between buttons looks nicer */
+	y2-= asp;
+	x2-= asp;
+	
 	/* paper */
 	if(flag & UI_SELECT) {
 		if(flag & UI_ACTIVE) BIF_ThemeColorShade(colorid, -40);
@@ -1318,7 +1321,6 @@ static void ui_draw_slider(int colorid, float fac, float aspect, float x1, float
 
 static void ui_shadowbox(float minx, float miny, float maxx, float maxy, float shadsize, unsigned char alpha)
 {
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	glEnable(GL_BLEND);
 	glShadeModel(GL_SMOOTH);
 	
@@ -1399,7 +1401,6 @@ static void ui_draw_pulldown_item(int type, int colorid, float asp, float x1, fl
 	BIF_GetThemeColor4ubv(TH_MENU_BACK, col);
 	if(col[3]!=255) {
 		glEnable(GL_BLEND);
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	}
 	
 	if((flag & UI_ACTIVE) && type!=LABEL) {
@@ -1427,7 +1428,6 @@ static void ui_draw_pulldown_round(int type, int colorid, float asp, float x1, f
 
 		glEnable( GL_LINE_SMOOTH );
 		glEnable( GL_BLEND );
-		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		gl_round_box(GL_LINE_LOOP, x1, y1+3, x2, y2-3, 7.0);
 		glDisable( GL_LINE_SMOOTH );
 		glDisable( GL_BLEND );
@@ -1450,6 +1450,8 @@ static void ui_draw_text_icon(uiBut *but)
 	float x;
 	int len;
 	char *cpoin;
+	short t, pos, ch;
+	short selsta_tmp, selend_tmp, selsta_draw, selwidth_draw;
 	
 	/* check for button text label */
 	if (but->type == ICONTEXTROW) {
@@ -1457,25 +1459,53 @@ static void ui_draw_text_icon(uiBut *but)
 	}
 	else {
 
-		// text button cursor
+		/* text button selection and cursor */
 		if(but->pos != -1) {
-			short t, pos, ch;
-			
-			pos= but->pos+strlen(but->str);
-			if(pos >= but->ofs) {
+		
+			if (SELWIDTH > 0) {
+				/* text button selection */
+				selsta_tmp = but->selsta + strlen(but->str);
+				selend_tmp = but->selend + strlen(but->str);
+				if (but->ofs >= strlen(but->str))
+					selsta_tmp += (but->ofs - strlen(but->str));
+					
 				if(but->drawstr[0]!=0) {
-					ch= but->drawstr[pos];
-					but->drawstr[pos]= 0;
-		
-					t= but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
-		
-					but->drawstr[pos]= ch;
+					ch= but->drawstr[selsta_tmp];
+					but->drawstr[selsta_tmp]= 0;
+					
+					selsta_draw = but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
+					
+					but->drawstr[selsta_tmp]= ch;
+					
+					
+					ch= but->drawstr[selend_tmp];
+					but->drawstr[selend_tmp]= 0;
+					
+					selwidth_draw = but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
+					
+					but->drawstr[selend_tmp]= ch;
+					
+					BIF_ThemeColor(TH_BUT_TEXTFIELD_HI);
+					glRects(but->x1+selsta_draw+1, but->y1+2, but->x1+selwidth_draw+1, but->y2-2);
 				}
-				else t= 3;
-				
-				glColor3ub(255,0,0);
-				glRects(but->x1+t, but->y1+2, but->x1+t+3, but->y2-2);
-			}	
+			} else {
+				/* text cursor */
+				pos= but->pos+strlen(but->str);
+				if(pos >= but->ofs) {
+					if(but->drawstr[0]!=0) {
+						ch= but->drawstr[pos];
+						but->drawstr[pos]= 0;
+			
+						t= but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
+						
+						but->drawstr[pos]= ch;
+					}
+					else t= 3;
+					
+					glColor3ub(255,0,0);
+					glRects(but->x1+t, but->y1+2, but->x1+t+2, but->y2-2);
+				}
+			}
 		}
 		if(but->drawstr[0]!=0) {
 			int transopts;
@@ -1491,7 +1521,7 @@ static void ui_draw_text_icon(uiBut *but)
 			if ( (but->flag & UI_HAS_ICON) && (but->flag & UI_ICON_LEFT) ) {
 				ui_draw_icon(but, but->icon);
 
-				if(but->flag & UI_TEXT_LEFT) x= but->x1+24.0;
+				if(but->flag & UI_TEXT_LEFT) x= but->x1 + but->aspect*BIF_icon_get_width(but->icon)+5.0;
 				else x= (but->x1+but->x2-but->strwidth+1)/2.0;
 			}
 			else {
@@ -1705,6 +1735,505 @@ static void ui_draw_but_HSVCUBE(uiBut *but)
 	fdrawbox((but->x1), (but->y1), (but->x2), (but->y2));
 }
 
+#ifdef INTERNATIONAL
+static void ui_draw_but_CHARTAB(uiBut *but)
+{
+	/* Some local variables */
+	float sx, sy, ex, ey;
+	float width, height;
+	float butw, buth;
+	int x, y, cs;
+	wchar_t wstr[2];
+	unsigned char ustr[16];
+	PackedFile *pf;
+	int result = 0;
+	int charmax = G.charmax;
+
+	/* <builtin> font in use. There are TTF <builtin> and non-TTF <builtin> fonts */
+	if(!strcmp(G.selfont->name, "<builtin>"))
+	{
+		if(G.ui_international == TRUE)
+		{
+			charmax = 0xff;
+		}
+		else
+		{
+			charmax = 0xff;
+		}
+	}
+
+	/* Category list exited without selecting the area */
+	if(G.charmax == 0)
+		charmax = G.charmax = 0xffff;
+
+	/* Calculate the size of the button */
+	width = abs(but->x2 - but->x1);
+	height = abs(but->y2 - but->y1);
+	
+	butw = floor(width / 12);
+	buth = floor(height / 6);
+	
+	/* Initialize variables */
+	sx = but->x1;
+	ex = but->x1 + butw;
+	sy = but->y1 + height - buth;
+	ey = but->y1 + height;
+
+	cs = G.charstart;
+
+	/* Set the font, in case it is not <builtin> font */
+	if(G.selfont && strcmp(G.selfont->name, "<builtin>"))
+	{
+		char tmpStr[256];
+
+		// Is the font file packed, if so then use the packed file
+		if(G.selfont->packedfile)
+		{
+			pf = G.selfont->packedfile;		
+			FTF_SetFont(pf->data, pf->size, 14.0);
+		}
+		else
+		{
+			int err;
+
+			strcpy(tmpStr, G.selfont->name);
+			BLI_convertstringcode(tmpStr, G.sce, 0);
+			err = FTF_SetFont((unsigned char *)tmpStr, 0, 14.0);
+		}
+	}
+	else
+	{
+		if(G.ui_international == TRUE)
+		{
+			FTF_SetFont((unsigned char *) datatoc_bfont_ttf, datatoc_bfont_ttf_size, 14.0);
+		}
+	}
+
+	/* Start drawing the button itself */
+	glShadeModel(GL_SMOOTH);
+
+	glColor3ub(200,  200,  200);
+	glRectf((but->x1), (but->y1), (but->x2), (but->y2));
+
+	glColor3ub(0,  0,  0);
+	for(y = 0; y < 6; y++)
+	{
+		// Do not draw more than the category allows
+		if(cs > charmax) break;
+
+		for(x = 0; x < 12; x++)
+		{
+			// Do not draw more than the category allows
+			if(cs > charmax) break;
+
+			// Draw one grid cell
+			glBegin(GL_LINE_LOOP);
+				glVertex2f(sx, sy);
+				glVertex2f(ex, sy);
+				glVertex2f(ex, ey);
+				glVertex2f(sx, ey);				
+			glEnd();	
+
+			// Draw character inside the cell
+			memset(wstr, 0, sizeof(wchar_t)*2);
+			memset(ustr, 0, 16);
+
+			// Set the font to be either unicode or <builtin>				
+			wstr[0] = cs;
+			if(strcmp(G.selfont->name, "<builtin>"))
+			{
+				wcs2utf8s((char *)ustr, (wchar_t *)wstr);
+			}
+			else
+			{
+				if(G.ui_international == TRUE)
+				{
+					wcs2utf8s((char *)ustr, (wchar_t *)wstr);
+				}
+				else
+				{
+					ustr[0] = cs;
+					ustr[1] = 0;
+				}
+			}
+
+			if((G.selfont && strcmp(G.selfont->name, "<builtin>")) || (G.selfont && !strcmp(G.selfont->name, "<builtin>") && G.ui_international == TRUE))
+			{
+				float wid;
+				float llx, lly, llz, urx, ury, urz;
+				float dx, dy;
+				float px, py;
+	
+				// Calculate the position
+				wid = FTF_GetStringWidth((char *) ustr, FTF_USE_GETTEXT | FTF_INPUT_UTF8);
+				FTF_GetBoundingBox((char *) ustr, &llx,&lly,&llz,&urx,&ury,&urz, FTF_USE_GETTEXT | FTF_INPUT_UTF8);
+				dx = urx-llx;
+				dy = ury-lly;
+
+				// This isn't fully functional since the but->aspect isn't working like I suspected
+				px = sx + ((butw/but->aspect)-dx)/2;
+				py = sy + ((buth/but->aspect)-dy)/2;
+
+				// Set the position and draw the character
+				ui_rasterpos_safe(px, py, but->aspect);
+				FTF_DrawString((char *) ustr, FTF_USE_GETTEXT | FTF_INPUT_UTF8);
+			}
+			else
+			{
+				ui_rasterpos_safe(sx + butw/2, sy + buth/2, but->aspect);
+				BIF_DrawString(but->font, (char *) ustr, 0);
+			}
+	
+			// Calculate the next position and character
+			sx += butw; ex +=butw;
+			cs++;
+		}
+		/* Add the y position and reset x position */
+		sy -= buth; 
+		ey -= buth;
+		sx = but->x1;
+		ex = but->x1 + butw;
+	}	
+	glShadeModel(GL_FLAT);
+
+	/* Return Font Settings to original */
+	if(U.fontsize && U.fontname[0])
+	{
+		result = FTF_SetFont((unsigned char *)U.fontname, 0, U.fontsize);
+	}
+	else if (U.fontsize)
+	{
+		result = FTF_SetFont((unsigned char *) datatoc_bfont_ttf, datatoc_bfont_ttf_size, U.fontsize);
+	}
+
+	if (result == 0)
+	{
+		result = FTF_SetFont((unsigned char *) datatoc_bfont_ttf, datatoc_bfont_ttf_size, 11);
+	}
+}
+
+#endif // INTERNATIONAL
+
+static void ui_draw_but_COLORBAND(uiBut *but)
+{
+	ColorBand *coba= (ColorBand *)but->poin;
+	CBData *cbd;
+	float x1, y1, sizex, sizey;
+	float dx, v3[2], v1[2], v2[2];
+	int a;
+		
+	if(coba==NULL) return;
+	
+	x1= but->x1;
+	y1= but->y1;
+	sizex= but->x2-x1;
+	sizey= but->y2-y1;
+	
+	/* first background, to show tranparency */
+	dx= sizex/12.0;
+	v1[0]= x1;
+	for(a=0; a<12; a++) {
+		if(a & 1) glColor3f(0.3, 0.3, 0.3); else glColor3f(0.8, 0.8, 0.8);
+		glRectf(v1[0], y1, v1[0]+dx, y1+0.5*sizey);
+		if(a & 1) glColor3f(0.8, 0.8, 0.8); else glColor3f(0.3, 0.3, 0.3);
+		glRectf(v1[0], y1+0.5*sizey, v1[0]+dx, y1+sizey);
+		v1[0]+= dx;
+	}
+	
+	glShadeModel(GL_SMOOTH);
+	glEnable(GL_BLEND);
+	
+	cbd= coba->data;
+	
+	v1[0]= v2[0]= x1;
+	v1[1]= y1;
+	v2[1]= y1+sizey;
+	
+	glBegin(GL_QUAD_STRIP);
+	
+	glColor4fv( &cbd->r );
+	glVertex2fv(v1); glVertex2fv(v2);
+	
+	for(a=0; a<coba->tot; a++, cbd++) {
+		
+		v1[0]=v2[0]= x1+ cbd->pos*sizex;
+		
+		glColor4fv( &cbd->r );
+		glVertex2fv(v1); glVertex2fv(v2);
+	}
+	
+	v1[0]=v2[0]= x1+ sizex;
+	glVertex2fv(v1); glVertex2fv(v2);
+	
+	glEnd();
+	glShadeModel(GL_FLAT);
+	glDisable(GL_BLEND);
+	
+	/* outline */
+	v1[0]= x1; v1[1]= y1;
+	
+	cpack(0x0);
+	glBegin(GL_LINE_LOOP);
+	glVertex2fv(v1);
+	v1[0]+= sizex;
+	glVertex2fv(v1);
+	v1[1]+= sizey;
+	glVertex2fv(v1);
+	v1[0]-= sizex;
+	glVertex2fv(v1);
+	glEnd();
+	
+	
+	/* help lines */
+	v1[0]= v2[0]=v3[0]= x1;
+	v1[1]= y1;
+	v2[1]= y1+0.5*sizey;
+	v3[1]= y1+sizey;
+	
+	cbd= coba->data;
+	glBegin(GL_LINES);
+	for(a=0; a<coba->tot; a++, cbd++) {
+		v1[0]=v2[0]=v3[0]= x1+ cbd->pos*sizex;
+		
+		glColor3ub(0, 0, 0);
+		glVertex2fv(v1);
+		glVertex2fv(v2);
+		
+		if(a==coba->cur) {
+			glVertex2f(v1[0]-1, v1[1]);
+			glVertex2f(v2[0]-1, v2[1]);
+			glVertex2f(v1[0]+1, v1[1]);
+			glVertex2f(v2[0]+1, v2[1]);
+		}
+		
+		glColor3ub(255, 255, 255);
+		glVertex2fv(v2);
+		glVertex2fv(v3);
+		
+		if(a==coba->cur) {
+			if(cbd->pos>0.01) {
+				glVertex2f(v2[0]-1, v2[1]);
+				glVertex2f(v3[0]-1, v3[1]);
+			}
+			if(cbd->pos<0.99) {
+				glVertex2f(v2[0]+1, v2[1]);
+				glVertex2f(v3[0]+1, v3[1]);
+			}
+		}
+	}
+	glEnd();
+}
+
+static void ui_draw_but_NORMAL(uiBut *but)
+{
+	static GLuint displist=0;
+	int a, old[8];
+	GLfloat diff[4], diffn[4]={1.0f, 1.0f, 1.0f, 1.0f};
+	float vec0[4]={0.0f, 0.0f, 0.0f, 0.0f};
+	float dir[4], size;
+	
+	/* store stuff */
+	glGetMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
+		
+	/* backdrop */
+	BIF_ThemeColor(TH_BUT_NEUTRAL);
+	uiSetRoundBox(15);
+	gl_round_box(GL_POLYGON, but->x1, but->y1, but->x2, but->y2, 5.0f);
+	
+	/* sphere color */
+	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffn);
+	glCullFace(GL_BACK); glEnable(GL_CULL_FACE);
+	
+	/* disable blender light */
+	for(a=0; a<8; a++) {
+		old[a]= glIsEnabled(GL_LIGHT0+a);
+		glDisable(GL_LIGHT0+a);
+	}
+	
+	/* own light */
+	glEnable(GL_LIGHT7);
+	glEnable(GL_LIGHTING);
+	
+	VECCOPY(dir, (float *)but->poin);
+	dir[3]= 0.0f;	/* glLight needs 4 args, 0.0 is sun */
+	glLightfv(GL_LIGHT7, GL_POSITION, dir); 
+	glLightfv(GL_LIGHT7, GL_DIFFUSE, diffn); 
+	glLightfv(GL_LIGHT7, GL_SPECULAR, vec0); 
+	glLightf(GL_LIGHT7, GL_CONSTANT_ATTENUATION, 1.0f);
+	glLightf(GL_LIGHT7, GL_LINEAR_ATTENUATION, 0.0f);
+	
+	/* transform to button */
+	glPushMatrix();
+	glTranslatef(but->x1 + 0.5f*(but->x2-but->x1), but->y1+ 0.5f*(but->y2-but->y1), 0.0f);
+	size= (but->x2-but->x1)/200.f;
+	glScalef(size, size, size);
+			 
+	if(displist==0) {
+		GLUquadricObj	*qobj;
+		
+		displist= glGenLists(1);
+		glNewList(displist, GL_COMPILE_AND_EXECUTE);
+		
+		qobj= gluNewQuadric();
+		gluQuadricDrawStyle(qobj, GLU_FILL); 
+		glShadeModel(GL_SMOOTH);
+		gluSphere( qobj, 100.0, 32, 24);
+		glShadeModel(GL_FLAT);
+		gluDeleteQuadric(qobj);  
+		
+		glEndList();
+	}
+	else glCallList(displist);
+	
+	/* restore */
+	glPopMatrix();
+	glDisable(GL_LIGHTING);
+	glDisable(GL_CULL_FACE);
+	glMaterialfv(GL_FRONT, GL_DIFFUSE, diff); 
+	
+	glDisable(GL_LIGHT7);
+	
+	/* enable blender light */
+	for(a=0; a<8; a++) {
+		if(old[a])
+			glEnable(GL_LIGHT0+a);
+	}
+}
+
+static void ui_draw_but_curve_grid(uiBut *but, float zoomx, float zoomy, float offsx, float offsy, float step)
+{
+	float dx, dy, fx, fy;
+	
+	glBegin(GL_LINES);
+	dx= step*zoomx;
+	fx= but->x1 + zoomx*(-offsx);
+	if(fx > but->x1) fx -= dx*( floor(fx-but->x1));
+	while(fx < but->x2) {
+		glVertex2f(fx, but->y1); 
+		glVertex2f(fx, but->y2);
+		fx+= dx;
+	}
+	
+	dy= step*zoomy;
+	fy= but->y1 + zoomy*(-offsy);
+	if(fy > but->y1) fy -= dy*( floor(fy-but->y1));
+	while(fy < but->y2) {
+		glVertex2f(but->x1, fy); 
+		glVertex2f(but->x2, fy);
+		fy+= dy;
+	}
+	glEnd();
+	
+}
+
+static void ui_draw_but_CURVE(uiBut *but)
+{
+	CurveMapping *cumap= (CurveMapping *)but->poin;
+	CurveMap *cuma= cumap->cm+cumap->cur;
+	CurveMapPoint *cmp;
+	float fx, fy, dx, dy, fac[2], zoomx, zoomy, offsx, offsy;
+	GLint scissor[4];
+	int a;
+	
+	/* need scissor test, curve can draw outside of boundary */
+	glGetIntegerv(GL_VIEWPORT, scissor);
+	fx= but->x1; fy= but->y1;
+	ui_graphics_to_window(but->win, &fx, &fy);
+	dx= but->x2; dy= but->y2;
+	ui_graphics_to_window(but->win, &dx, &dy);
+	glScissor((int)floor(fx), (int)floor(fy), (int)ceil(dx-fx), (int)ceil(dy-fy));
+	
+	/* calculate offset and zoom */
+	zoomx= (but->x2-but->x1-2.0*but->aspect)/(cumap->curr.xmax - cumap->curr.xmin);
+	zoomy= (but->y2-but->y1-2.0*but->aspect)/(cumap->curr.ymax - cumap->curr.ymin);
+	offsx= cumap->curr.xmin-but->aspect/zoomx;
+	offsy= cumap->curr.ymin-but->aspect/zoomy;
+	
+	/* backdrop */
+	if(cumap->flag & CUMA_DO_CLIP) {
+		BIF_ThemeColorShade(TH_BUT_NEUTRAL, -20);
+		glRectf(but->x1, but->y1, but->x2, but->y2);
+		BIF_ThemeColor(TH_BUT_NEUTRAL);
+		glRectf(but->x1 + zoomx*(cumap->clipr.xmin-offsx),
+				but->y1 + zoomy*(cumap->clipr.ymin-offsy),
+				but->x1 + zoomx*(cumap->clipr.xmax-offsx),
+				but->y1 + zoomy*(cumap->clipr.ymax-offsy));
+	}
+	else {
+		BIF_ThemeColor(TH_BUT_NEUTRAL);
+		glRectf(but->x1, but->y1, but->x2, but->y2);
+	}
+	
+	/* grid, every .25 step */
+	BIF_ThemeColorShade(TH_BUT_NEUTRAL, -16);
+	ui_draw_but_curve_grid(but, zoomx, zoomy, offsx, offsy, 0.25f);
+	/* grid, every 1.0 step */
+	BIF_ThemeColorShade(TH_BUT_NEUTRAL, -24);
+	ui_draw_but_curve_grid(but, zoomx, zoomy, offsx, offsy, 1.0f);
+	/* axes */
+	BIF_ThemeColorShade(TH_BUT_NEUTRAL, -50);
+	glBegin(GL_LINES);
+	glVertex2f(but->x1, but->y1 + zoomy*(-offsy));
+	glVertex2f(but->x2, but->y1 + zoomy*(-offsy));
+	glVertex2f(but->x1 + zoomx*(-offsx), but->y1);
+	glVertex2f(but->x1 + zoomx*(-offsx), but->y2);
+	glEnd();
+	
+	/* the curve */
+	BIF_ThemeColor(TH_TEXT);
+	glBegin(GL_LINE_STRIP);
+	
+	if(cuma->table==NULL)
+		curvemapping_changed(cumap, 0);	/* 0 = no remove doubles */
+	cmp= cuma->table;
+	
+	glVertex2f(but->x1, but->y1 + zoomy*(cmp[0].y-offsy));	/* first point */
+	for(a=0; a<=CM_TABLE; a++) {
+		fx= but->x1 + zoomx*(cmp[a].x-offsx);
+		fy= but->y1 + zoomy*(cmp[a].y-offsy);
+		glVertex2f(fx, fy);
+	}
+	glVertex2f(but->x2, but->y1 + zoomy*(cmp[CM_TABLE].y-offsy));	/* last point */
+	glEnd();
+
+	/* the points, use aspect to make them visible on edges */
+	cmp= cuma->curve;
+	glPointSize(3.0f);
+	bglBegin(GL_POINTS);
+	for(a=0; a<cuma->totpoint; a++) {
+		if(cmp[a].flag & SELECT)
+			BIF_ThemeColor(TH_TEXT_HI);
+		else
+			BIF_ThemeColor(TH_TEXT);
+		fac[0]= but->x1 + zoomx*(cmp[a].x-offsx);
+		fac[1]= but->y1 + zoomy*(cmp[a].y-offsy);
+		bglVertex2fv(fac);
+	}
+	bglEnd();
+	glPointSize(1.0f);
+	
+	/* restore scissortest */
+	glScissor(scissor[0], scissor[1], scissor[2], scissor[3]);
+
+	/* outline */
+	BIF_ThemeColor(TH_BUT_OUTLINE);
+	fdrawbox(but->x1, but->y1, but->x2, but->y2);
+
+}
+
+static void ui_draw_roundbox(uiBut *but)
+{
+	glEnable(GL_BLEND);
+	
+	BIF_ThemeColorShadeAlpha(TH_PANEL, but->a2, but->a2);
+
+	uiSetRoundBox(but->a1);
+	gl_round_box(GL_POLYGON, but->x1, but->y1, but->x2, but->y2, but->min);
+
+	glDisable(GL_BLEND);
+}
+
+
 /* nothing! */
 static void ui_draw_nothing(int type, int colorid, float asp, float x1, float y1, float x2, float y2, int flag)
 {
@@ -1723,11 +2252,12 @@ void ui_set_embossfunc(uiBut *but, int drawtype)
 	but->sliderfunc= ui_draw_slider;
 
 	// standard builtin first:
-	if(but->type==LABEL) but->embossfunc= ui_draw_nothing;
+	if(but->type==LABEL || but->type==ROUNDBOX) but->embossfunc= ui_draw_nothing;
 	else if(but->type==PULLDOWN) but->embossfunc= ui_draw_pulldown_round;
 	else if(drawtype==UI_EMBOSSM) but->embossfunc= ui_draw_minimal;
 	else if(drawtype==UI_EMBOSSN) but->embossfunc= ui_draw_nothing;
 	else if(drawtype==UI_EMBOSSP) but->embossfunc= ui_draw_pulldown_item;
+	else if(drawtype==UI_EMBOSSR) but->embossfunc= ui_draw_round;
 	else {
 		int theme= BIF_GetThemeValue(TH_BUT_DRAWTYPE);
 		
@@ -1750,7 +2280,6 @@ void ui_set_embossfunc(uiBut *but, int drawtype)
 	// note: if you want aligning, adapt the call uiBlockEndAlign in interface.c 
 }
 
-
 void ui_draw_but(uiBut *but)
 {
 	double value;
@@ -1792,11 +2321,32 @@ void ui_draw_but(uiBut *but)
 		ui_draw_but_HSVCUBE(but);  // box for colorpicker, three types
 		break;
 
+#ifdef INTERNATIONAL
+	case CHARTAB:
+		value= ui_get_but_val(but);
+		ui_draw_but_CHARTAB(but);
+		break;
+#endif
+
 	case LINK:
 	case INLINK:
 		ui_draw_icon(but, but->icon);
 		break;
-
+		
+	case ROUNDBOX:
+		ui_draw_roundbox(but);
+		break;
+		
+	case BUT_COLORBAND:
+		ui_draw_but_COLORBAND(but);
+		break;
+	case BUT_NORMAL:
+		ui_draw_but_NORMAL(but);
+		break;
+	case BUT_CURVE:
+		ui_draw_but_CURVE(but);
+		break;
+		
 	default:
 		but->embossfunc(but->type, but->themecol, but->aspect, but->x1, but->y1, but->x2, but->y2, but->flag);
 		ui_draw_text_icon(but);
