@@ -1,5 +1,5 @@
 /**
- * $Id: transform_constraints.c,v 1.54 2005/06/05 13:50:21 theeth Exp $
+ * $Id: transform_constraints.c,v 1.68 2006/06/04 18:38:58 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <math.h>
 
@@ -51,7 +52,6 @@
 #include "DNA_camera_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_effect_types.h"
-#include "DNA_ika_types.h"
 #include "DNA_image_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_key_types.h"
@@ -63,39 +63,22 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
-#include "DNA_texture_types.h"
+#include "DNA_space_types.h"
 #include "DNA_view3d_types.h"
-#include "DNA_world_types.h"
-#include "DNA_userdef_types.h"
-#include "DNA_property_types.h"
-#include "DNA_vfont_types.h"
-#include "DNA_constraint_types.h"
 
 #include "BIF_screen.h"
-#include "BIF_space.h"
-#include "BIF_editview.h"
 #include "BIF_resources.h"
 #include "BIF_mywindow.h"
 #include "BIF_gl.h"
-#include "BIF_editlattice.h"
-#include "BIF_editarmature.h"
-#include "BIF_editmesh.h"
 
 #include "BKE_global.h"
-#include "BKE_object.h"
 #include "BKE_utildefines.h"
-#include "BKE_lattice.h"
-#include "BKE_armature.h"
-#include "BKE_curve.h"
-#include "BKE_displist.h"
 
 #include "BSE_view.h"
-#include "BSE_edit.h"
 
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 
-#include "BDR_drawobject.h"
+#include "BDR_drawobject.h"	/* drawcircball	*/
 
 #include "blendef.h"
 
@@ -103,14 +86,9 @@
 
 #include "transform.h"
 
-extern ListBase editNurb;
-extern ListBase editelems;
-
-void recalcData();
+static void drawObjectConstraint(TransInfo *t);
 
 /* ************************** CONSTRAINTS ************************* */
-void getConstraintMatrix(TransInfo *t);
-
 void constraintNumInput(TransInfo *t, float vec[3])
 {
 	int mode = t->con.mode;
@@ -118,15 +96,16 @@ void constraintNumInput(TransInfo *t, float vec[3])
 		float nval = (t->flag & T_NULL_ONE)?1.0f:0.0f;
 
 		if (getConstraintSpaceDimension(t) == 2) {
-			if (mode & (CON_AXIS0|CON_AXIS1)) {
+			int axis = mode & (CON_AXIS0|CON_AXIS1|CON_AXIS2);
+			if (axis == (CON_AXIS0|CON_AXIS1)) {
 				vec[2] = nval;
 			}
-			else if (mode & (CON_AXIS1|CON_AXIS2)) {
+			else if (axis == (CON_AXIS1|CON_AXIS2)) {
 				vec[2] = vec[1];
 				vec[1] = vec[0];
 				vec[0] = nval;
 			}
-			else if (mode & (CON_AXIS0|CON_AXIS2)) {
+			else if (axis == (CON_AXIS0|CON_AXIS2)) {
 				vec[2] = vec[1];
 				vec[1] = nval;
 			}
@@ -186,24 +165,16 @@ static void postConstraintChecks(TransInfo *t, float vec[3], float pvec[3]) {
 	Mat3MulVecfl(t->con.mtx, vec);
 }
 
-
 static void axisProjection(TransInfo *t, float axis[3], float in[3], float out[3]) {
-	float norm[3], n[3], vec[3], factor;
-
-	VecAddf(vec, in, t->con.center);
-	getViewVector(vec, norm);
-
-	Normalise(axis);
-
-	VECCOPY(n, axis);
-	Mat4MulVecfl(t->viewmat, n);
-	n[2] = t->viewmat[3][2];
-	Mat4MulVecfl(t->viewinv, n);
-
+	float norm[3], vec[3], factor;
+	
+	if(in[0]==0.0f && in[1]==0.0f && in[2]==0.0f)
+		return;
+	
 	/* For when view is parallel to constraint... will cause NaNs otherwise
 	   So we take vertical motion in 3D space and apply it to the
 	   constraint axis. Nice for camera grab + MMB */
-	if(n[0]*n[0] + n[1]*n[1] + n[2]*n[2] < 0.000001f) {
+	if(1.0f - fabs(Inpf(axis, t->viewinv[2])) < 0.000001f) {
 		Projf(vec, in, t->viewinv[1]);
 		factor = Inpf(t->viewinv[1], vec) * 2.0f;
 		/* since camera distance is quite relative, use quadratic relationship. holding shift can compensate */
@@ -215,38 +186,44 @@ static void axisProjection(TransInfo *t, float axis[3], float in[3], float out[3
 		VecMulf(out, -factor);	/* -factor makes move down going backwards */
 	}
 	else {
-		// prevent division by zero, happens on constrainting without initial delta transform */
-		if(in[0]!=0.0f || in[1]!=0.0f || in[2]!=0.0) {
-			Projf(vec, in, n);
-			factor = Normalise(vec);
-			// prevent NaN for 0.0/0.0
-			if(factor!=0.0f)
-				factor /= Inpf(axis, vec);
-
-			VecMulf(axis, factor);
-			VECCOPY(out, axis);
-		}
+		float cb[3], ab[3];
+		
+		VECCOPY(out, axis);
+		
+		/* Get view vector on axis to define a plane */
+		VecAddf(vec, t->con.center, in);
+		getViewVector(vec, norm);
+		
+		Crossf(vec, norm, axis);
+		
+		/* Project input vector on the plane passing on axis */
+		Projf(vec, in, vec);
+		VecSubf(vec, in, vec);
+		
+		/* intersect the two lines: axis and norm */
+		Crossf(cb, vec, norm);
+		Crossf(ab, axis, norm);
+		
+		VecMulf(out, Inpf(cb, ab) / Inpf(ab, ab));
 	}
 }
 
 static void planeProjection(TransInfo *t, float in[3], float out[3]) {
-	float vec[3], factor, angle, norm[3];
+	float vec[3], factor, norm[3];
 
 	VecAddf(vec, in, t->con.center);
 	getViewVector(vec, norm);
 
 	VecSubf(vec, out, in);
-	factor = Normalise(vec);
-	angle = Inpf(vec, norm);
 
-	if (angle * angle >= 0.000001f) {
-		factor /= angle;
+	factor = Inpf(vec, norm);
+	if (factor == 0.0f) return; /* prevent divide by zero */
+	factor = Inpf(vec, vec) / factor;
 
-		VECCOPY(vec, norm);
-		VecMulf(vec, factor);
+	VECCOPY(vec, norm);
+	VecMulf(vec, factor);
 
-		VecAddf(out, in, vec);
-	}
+	VecAddf(out, in, vec);
 }
 
 /*
@@ -326,6 +303,18 @@ static void applyObjectConstraintVec(TransInfo *t, TransData *td, float in[3], f
 			VECCOPY(out, pvec);
 		}
 		else {
+			int i=0;
+
+			out[0] = out[1] = out[2] = 0.0f;
+			if (t->con.mode & CON_AXIS0) {
+				out[0] = in[i++];
+			}
+			if (t->con.mode & CON_AXIS1) {
+				out[1] = in[i++];
+			}
+			if (t->con.mode & CON_AXIS2) {
+				out[2] = in[i++];
+			}
 			Mat3MulVecfl(td->axismtx, out);
 		}
 	}
@@ -468,71 +457,7 @@ static void applyObjectConstraintRot(TransInfo *t, TransData *td, float vec[3])
 	}
 }
 
-static void drawObjectConstraint(TransInfo *t) {
-	int i;
-	TransData * td = t->data;
-
-	/* Draw the first one lighter because that's the one who controls the others.
-	   Meaning the transformation is projected on that one and just copied on the others
-	   constraint space.
-	   In a nutshell, the object with light axis is controlled by the user and the others follow.
-	   Without drawing the first light, users have little clue what they are doing.
-	 */
-	if (t->con.mode & CON_AXIS0) {
-		drawLine(td->ob->obmat[3], td->axismtx[0], 'x', DRAWLIGHT);
-	}
-	if (t->con.mode & CON_AXIS1) {
-		drawLine(td->ob->obmat[3], td->axismtx[1], 'y', DRAWLIGHT);
-	}
-	if (t->con.mode & CON_AXIS2) {
-		drawLine(td->ob->obmat[3], td->axismtx[2], 'z', DRAWLIGHT);
-	}
-	
-	td++;
-
-	for(i=1;i<t->total;i++,td++) {
-		if (t->con.mode & CON_AXIS0) {
-			drawLine(td->ob->obmat[3], td->axismtx[0], 'x', 0);
-		}
-		if (t->con.mode & CON_AXIS1) {
-			drawLine(td->ob->obmat[3], td->axismtx[1], 'y', 0);
-		}
-		if (t->con.mode & CON_AXIS2) {
-			drawLine(td->ob->obmat[3], td->axismtx[2], 'z', 0);
-		}
-	}
-}
-
-/*
- * Returns the dimension of the constraint space.
- * 
- * For that reason, the flags always needs to be set to properly evaluate here,
- * even if they aren't actually used in the callback function. (Which could happen
- * for weird constraints not yet designed. Along a path for example.)
- */
-
-int getConstraintSpaceDimension(TransInfo *t)
-{
-	int n = 0;
-
-	if (t->con.mode & CON_AXIS0)
-		n++;
-
-	if (t->con.mode & CON_AXIS1)
-		n++;
-
-	if (t->con.mode & CON_AXIS2)
-		n++;
-
-	return n;
-/*
-  Someone willing to do it criptically could do the following instead:
-
-  return t->con & (CON_AXIS0|CON_AXIS1|CON_AXIS2);
-	
-  Based on the assumptions that the axis flags are one after the other and start at 1
-*/
-}
+/*--------------------- INTERNAL SETUP CALLS ------------------*/
 
 void setConstraint(TransInfo *t, float space[3][3], int mode, const char text[]) {
 	strncpy(t->con.text + 1, text, 48);
@@ -549,38 +474,6 @@ void setConstraint(TransInfo *t, float space[3][3], int mode, const char text[])
 	t->redraw = 1;
 }
 
-void BIF_setLocalAxisConstraint(char axis, char *text) {
-	TransInfo *t = BIF_GetTransInfo();
-
-	switch (axis) {
-	case 'X':
-		setLocalConstraint(t, CON_AXIS0, text);
-		break;
-	case 'Y':
-		setLocalConstraint(t, CON_AXIS1, text);
-		break;
-	case 'Z':
-		setLocalConstraint(t, CON_AXIS2, text);
-		break;
-	}
-}
-
-void BIF_setLocalLockConstraint(char axis, char *text) {
-	TransInfo *t = BIF_GetTransInfo();
-
-	switch (axis) {
-	case 'x':
-		setLocalConstraint(t, (CON_AXIS1|CON_AXIS2), text);
-		break;
-	case 'y':
-		setLocalConstraint(t, (CON_AXIS0|CON_AXIS2), text);
-		break;
-	case 'z':
-		setLocalConstraint(t, (CON_AXIS0|CON_AXIS1), text);
-		break;
-	}
-}
-
 void setLocalConstraint(TransInfo *t, int mode, const char text[]) {
 	if (t->flag & T_EDIT) {
 		float obmat[3][3];
@@ -608,6 +501,75 @@ void setLocalConstraint(TransInfo *t, int mode, const char text[]) {
 	}
 }
 
+/*
+	Set the constraint according to the user defined orientation
+
+	ftext is a format string passed to sprintf. It will add the name of
+	the orientation where %s is (logically).
+*/
+void setUserConstraint(TransInfo *t, int mode, const char ftext[]) {
+	char text[40];
+	short twmode= (t->spacetype==SPACE_VIEW3D)? G.vd->twmode: V3D_MANIP_GLOBAL;
+
+	switch(twmode) {
+	case V3D_MANIP_GLOBAL:
+	/*
+		sprintf(text, ftext, "global");
+		Mat3One(mtx);
+		setConstraint(t, mtx, mode, text);
+		break;
+	*/
+	case V3D_MANIP_LOCAL:
+		sprintf(text, ftext, "local");
+		setLocalConstraint(t, mode, text);
+		break;
+	case V3D_MANIP_NORMAL:
+		sprintf(text, ftext, "normal");
+		setConstraint(t, t->spacemtx, mode, text);
+		break;
+	case V3D_MANIP_VIEW:
+		sprintf(text, ftext, "view");
+		setConstraint(t, t->spacemtx, mode, text);
+		break;
+	}
+
+	t->con.mode |= CON_USER;
+}
+
+/*--------------------- EXTERNAL SETUP CALLS ------------------*/
+
+void BIF_setLocalLockConstraint(char axis, char *text) {
+	TransInfo *t = BIF_GetTransInfo();
+
+	switch (axis) {
+	case 'x':
+		setLocalConstraint(t, (CON_AXIS1|CON_AXIS2), text);
+		break;
+	case 'y':
+		setLocalConstraint(t, (CON_AXIS0|CON_AXIS2), text);
+		break;
+	case 'z':
+		setLocalConstraint(t, (CON_AXIS0|CON_AXIS1), text);
+		break;
+	}
+}
+
+void BIF_setLocalAxisConstraint(char axis, char *text) {
+	TransInfo *t = BIF_GetTransInfo();
+
+	switch (axis) {
+	case 'X':
+		setLocalConstraint(t, CON_AXIS0, text);
+		break;
+	case 'Y':
+		setLocalConstraint(t, CON_AXIS1, text);
+		break;
+	case 'Z':
+		setLocalConstraint(t, CON_AXIS2, text);
+		break;
+	}
+}
+
 /* text is optional, for header print */
 void BIF_setSingleAxisConstraint(float vec[3], char *text) {
 	TransInfo *t = BIF_GetTransInfo();
@@ -661,19 +623,22 @@ void BIF_setDualAxisConstraint(float vec1[3], float vec2[3], char *text) {
 	t->redraw = 1;
 }
 
+/*----------------- DRAWING CONSTRAINTS -------------------*/
 
 void BIF_drawConstraint(void)
 {
 	TransInfo *t = BIF_GetTransInfo();
 	TransCon *tc = &(t->con);
 
+	if (t->spacetype!=SPACE_VIEW3D)
+		return;
 	if (!(tc->mode & CON_APPLY))
 		return;
 	if (t->flag & T_USES_MANIPULATOR)
 		return;
 	
 	/* nasty exception for Z constraint in camera view */
-	if( (t->flag & T_OBJECT) && G.vd->camera==OBACT && G.vd->persp>1) 
+	if((t->flag & T_OBJECT) && G.vd->camera==OBACT && G.vd->persp>1) 
 		return;
 
 	if (tc->drawExtra) {
@@ -685,7 +650,7 @@ void BIF_drawConstraint(void)
 			short mval[2];
 			char col2[3] = {255,255,255};
 			getmouseco_areawin(mval);
-			window_to_3d(vec, (short)(mval[0] - t->con.imval[0]), (short)(mval[1] - t->con.imval[1]));
+			convertViewVec(t, vec, (short)(mval[0] - t->con.imval[0]), (short)(mval[1] - t->con.imval[1]));
 			VecAddf(vec, vec, tc->center);
 
 			drawLine(tc->center, tc->mtx[0], 'x', 0);
@@ -701,7 +666,7 @@ void BIF_drawConstraint(void)
 				glVertex3fv(vec); 
 			glEnd();
 			setlinestyle(0);
-			if(G.zbuf) glEnable(GL_DEPTH_TEST);	// warning for global!
+			if(G.vd->zbuf) glEnable(GL_DEPTH_TEST);	
 		}
 
 		if (tc->mode & CON_AXIS0) {
@@ -731,7 +696,7 @@ void BIF_drawPropCircle()
 		
 		mygetmatrix(tmat);
 		Mat4Invert(imat, tmat);
-		
+
  		drawcircball(GL_LINE_LOOP, t->center, t->propsize, imat);
 		
 		/* if editmode we restore */
@@ -739,27 +704,54 @@ void BIF_drawPropCircle()
 	}
 }
 
-int isLockConstraint(TransInfo *t) {
-	int mode = t->con.mode;
+void BIF_getPropCenter(float *center)
+{
+	TransInfo *t = BIF_GetTransInfo();
 
-	if ( (mode & (CON_AXIS0|CON_AXIS1)) == (CON_AXIS0|CON_AXIS1))
-		return 1;
-
-	if ( (mode & (CON_AXIS1|CON_AXIS2)) == (CON_AXIS1|CON_AXIS2))
-		return 1;
-
-	if ( (mode & (CON_AXIS0|CON_AXIS2)) == (CON_AXIS0|CON_AXIS2))
-		return 1;
-
-	return 0;
+	if (t && t->flag & T_PROP_EDIT) {
+		VECCOPY(center, t->center);
+	}
+	else
+		center[0] = center[1] = center[2] = 0.0f;
 }
 
-void initConstraint(TransInfo *t) {
-	if (t->con.mode & CON_APPLY) {
-		startConstraint(t);
+static void drawObjectConstraint(TransInfo *t) {
+	int i;
+	TransData * td = t->data;
+
+	/* Draw the first one lighter because that's the one who controls the others.
+	   Meaning the transformation is projected on that one and just copied on the others
+	   constraint space.
+	   In a nutshell, the object with light axis is controlled by the user and the others follow.
+	   Without drawing the first light, users have little clue what they are doing.
+	 */
+	if (t->con.mode & CON_AXIS0) {
+		drawLine(td->ob->obmat[3], td->axismtx[0], 'x', DRAWLIGHT);
+	}
+	if (t->con.mode & CON_AXIS1) {
+		drawLine(td->ob->obmat[3], td->axismtx[1], 'y', DRAWLIGHT);
+	}
+	if (t->con.mode & CON_AXIS2) {
+		drawLine(td->ob->obmat[3], td->axismtx[2], 'z', DRAWLIGHT);
+	}
+	
+	td++;
+
+	for(i=1;i<t->total;i++,td++) {
+		if (t->con.mode & CON_AXIS0) {
+			drawLine(td->ob->obmat[3], td->axismtx[0], 'x', 0);
+		}
+		if (t->con.mode & CON_AXIS1) {
+			drawLine(td->ob->obmat[3], td->axismtx[1], 'y', 0);
+		}
+		if (t->con.mode & CON_AXIS2) {
+			drawLine(td->ob->obmat[3], td->axismtx[2], 'z', 0);
+		}
 	}
 }
 
+/*--------------------- START / STOP CONSTRAINTS ---------------------- */
+
 void startConstraint(TransInfo *t) {
 	t->con.mode |= CON_APPLY;
 	*t->con.text = ' ';
@@ -800,6 +792,8 @@ void getConstraintMatrix(TransInfo *t)
 	Mat3MulMat3(t->con.pmtx, t->con.mtx, mat);
 }
 
+/*------------------------- MMB Select -------------------------------*/
+
 void initSelectConstraint(TransInfo *t, float mtx[3][3])
 {
 	Mat3CpyMat3(t->con.mtx, mtx);
@@ -862,8 +856,8 @@ void setNearestAxis(TransInfo *t)
 	   of two 2D points 30 pixels apart (that's the last factor in the formula) after
 	   projecting them with window_to_3d and then get the length of that vector.
 	*/
-	zfac= G.vd->persmat[0][3]*t->center[0]+ G.vd->persmat[1][3]*t->center[1]+ G.vd->persmat[2][3]*t->center[2]+ G.vd->persmat[3][3];
-	zfac = VecLength(G.vd->persinv[0]) * 2.0f/curarea->winx * zfac * 30.0f;
+	zfac= t->persmat[0][3]*t->center[0]+ t->persmat[1][3]*t->center[1]+ t->persmat[2][3]*t->center[2]+ t->persmat[3][3];
+	zfac = VecLength(t->persinv[0]) * 2.0f/curarea->winx * zfac * 30.0f;
 
 	for (i = 0; i<3; i++) {
 		VECCOPY(axis, t->con.mtx[i]);
@@ -871,7 +865,7 @@ void setNearestAxis(TransInfo *t)
 		VecMulf(axis, zfac);
 		/* now we can project to get window coordinate */
 		VecAddf(axis, axis, t->con.center);
-		project_int(axis, icoord);
+		projectIntView(t, axis, icoord);
 		
 		axis[0] = (float)(icoord[0] - t->center2d[0]);
 		axis[1] = (float)(icoord[1] - t->center2d[1]);
@@ -890,36 +884,38 @@ void setNearestAxis(TransInfo *t)
 	if (len[0] <= len[1] && len[0] <= len[2]) {
 		if (G.qual & LR_SHIFTKEY) {
 			t->con.mode |= (CON_AXIS1|CON_AXIS2);
-			strcpy(t->con.text, " locking global X");
+			sprintf(t->con.text, " locking %s X axis", t->spacename);
 		}
 		else {
 			t->con.mode |= CON_AXIS0;
-			strcpy(t->con.text, " along global X");
+			sprintf(t->con.text, " along %s X axis", t->spacename);
 		}
 	}
 	else if (len[1] <= len[0] && len[1] <= len[2]) {
 		if (G.qual & LR_SHIFTKEY) {
 			t->con.mode |= (CON_AXIS0|CON_AXIS2);
-			strcpy(t->con.text, " locking global Y");
+			sprintf(t->con.text, " locking %s Y axis", t->spacename);
 		}
 		else {
 			t->con.mode |= CON_AXIS1;
-			strcpy(t->con.text, " along global Y");
+			sprintf(t->con.text, " along %s Y axis", t->spacename);
 		}
 	}
 	else if (len[2] <= len[1] && len[2] <= len[0]) {
 		if (G.qual & LR_SHIFTKEY) {
 			t->con.mode |= (CON_AXIS0|CON_AXIS1);
-			strcpy(t->con.text, " locking global Z");
+			sprintf(t->con.text, " locking %s Z axis", t->spacename);
 		}
 		else {
 			t->con.mode |= CON_AXIS2;
-			strcpy(t->con.text, " along global Z");
+			sprintf(t->con.text, " along %s Z axis", t->spacename);
 		}
 	}
 	getConstraintMatrix(t);
 }
 
+/*-------------- HELPER FUNCTIONS ----------------*/
+
 char constraintModeToChar(TransInfo *t) {
 	if ((t->con.mode & CON_APPLY)==0) {
 		return '\0';
@@ -938,3 +934,50 @@ char constraintModeToChar(TransInfo *t) {
 		return '\0';
 	}
 }
+
+
+int isLockConstraint(TransInfo *t) {
+	int mode = t->con.mode;
+
+	if ( (mode & (CON_AXIS0|CON_AXIS1)) == (CON_AXIS0|CON_AXIS1))
+		return 1;
+
+	if ( (mode & (CON_AXIS1|CON_AXIS2)) == (CON_AXIS1|CON_AXIS2))
+		return 1;
+
+	if ( (mode & (CON_AXIS0|CON_AXIS2)) == (CON_AXIS0|CON_AXIS2))
+		return 1;
+
+	return 0;
+}
+
+/*
+ * Returns the dimension of the constraint space.
+ * 
+ * For that reason, the flags always needs to be set to properly evaluate here,
+ * even if they aren't actually used in the callback function. (Which could happen
+ * for weird constraints not yet designed. Along a path for example.)
+ */
+
+int getConstraintSpaceDimension(TransInfo *t)
+{
+	int n = 0;
+
+	if (t->con.mode & CON_AXIS0)
+		n++;
+
+	if (t->con.mode & CON_AXIS1)
+		n++;
+
+	if (t->con.mode & CON_AXIS2)
+		n++;
+
+	return n;
+/*
+  Someone willing to do it criptically could do the following instead:
+
+  return t->con & (CON_AXIS0|CON_AXIS1|CON_AXIS2);
+	
+  Based on the assumptions that the axis flags are one after the other and start at 1
+*/
+}
