@@ -47,6 +47,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "DNA_armature_types.h"
+#include "DNA_action_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
@@ -59,7 +60,6 @@
 #include "DNA_view3d_types.h"
 
 #include "BKE_armature.h"
-#include "BKE_DerivedMesh.h"
 #include "BKE_global.h"
 #include "BKE_lattice.h"
 #include "BKE_object.h"
@@ -75,6 +75,7 @@
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_transform.h"
+#include "BIF_editmesh.h"
 
 #include "BSE_edit.h"
 #include "BSE_view.h"
@@ -132,50 +133,92 @@ static void calc_tw_center(float *co)
 	VecAddf(twcent, twcent, co);
 }
 
-/* callback */
-static void stats_pose(ListBase *lb, float *normal, float *plane)
+static void protectflag_to_drawflags(short protectflag, short *drawflags)
 {
-	Bone *bone;
-	float vec[3], mat[4][4];
+	if(protectflag & OB_LOCK_LOCX)
+		*drawflags &= ~MAN_TRANS_X;
+	if(protectflag & OB_LOCK_LOCY)
+		*drawflags &= ~MAN_TRANS_Y;
+	if(protectflag & OB_LOCK_LOCZ)
+		*drawflags &= ~MAN_TRANS_Z;
 	
-	for(bone= lb->first; bone; bone= bone->next) {
-		if (bone->flag & BONE_SELECTED) {
-			/* We don't let IK children get "grabbed" */
-			/* ALERT! abusive global Trans here */
-			if ( (Trans.mode!=TFM_TRANSLATION) || (bone->flag & BONE_IK_TOPARENT)==0 ) {
-				
-				get_bone_root_pos (bone, vec, 1);
-				
-				calc_tw_center(vec);
-				where_is_bone(G.obpose, bone);
-				get_objectspace_bone_matrix(bone, mat, 1, 1);	// points in negative Y o_O
-					
-				VecAddf(normal, normal, mat[2]);
-				VecAddf(plane, plane, mat[1]);
-				
-				return;	// see above function
-			}
+	if(protectflag & OB_LOCK_ROTX)
+		*drawflags &= ~MAN_ROT_X;
+	if(protectflag & OB_LOCK_ROTY)
+		*drawflags &= ~MAN_ROT_Y;
+	if(protectflag & OB_LOCK_ROTZ)
+		*drawflags &= ~MAN_ROT_Z;
+
+	if(protectflag & OB_LOCK_SCALEX)
+		*drawflags &= ~MAN_SCALE_X;
+	if(protectflag & OB_LOCK_SCALEY)
+		*drawflags &= ~MAN_SCALE_Y;
+	if(protectflag & OB_LOCK_SCALEZ)
+		*drawflags &= ~MAN_SCALE_Z;
+}
+
+/* for pose mode */
+static void stats_pose(View3D *v3d, bPoseChannel *pchan, float *normal, float *plane)
+{
+	Bone *bone= pchan->bone;
+	
+	if(bone) {
+		if (bone->flag & BONE_TRANSFORM) {
+			calc_tw_center(pchan->pose_head);
+			protectflag_to_drawflags(pchan->protectflag, &v3d->twdrawflag);
+
+			VecAddf(normal, normal, pchan->pose_mat[2]);
+			VecAddf(plane, plane, pchan->pose_mat[1]);
 		}
-		stats_pose(&bone->childbase, normal, plane);
 	}
 }
 
+/* only counts the parent selection, and tags transform flag */
+/* bad call... should re-use method from transform_conversion once */
+static void count_bone_select(TransInfo *t, bArmature *arm, ListBase *lb, int do_it) 
+{
+	Bone *bone;
+	int do_next;
+	
+	for(bone= lb->first; bone; bone= bone->next) {
+		bone->flag &= ~BONE_TRANSFORM;
+		do_next= do_it;
+		if(do_it) {
+			if(bone->layer & arm->layer) {
+				if (bone->flag & BONE_SELECTED) {
+					/* We don't let connected children get "grabbed" */
+					if ( (t->mode!=TFM_TRANSLATION) || (bone->flag & BONE_CONNECTED)==0 ) {
+						bone->flag |= BONE_TRANSFORM;
+						t->total++;
+						do_next= 0;	// no transform on children if one parent bone is selected
+					}
+				}
+			}
+		}
+		count_bone_select(t, arm, &bone->childbase, do_next);
+	}
+}
 
 /* centroid, boundbox, of selection */
 /* returns total items selected */
 int calc_manipulator_stats(ScrArea *sa)
 {
 	extern ListBase editNurb;
+	TransInfo *t;
 	View3D *v3d= sa->spacedata.first;
 	Base *base;
-	Object *ob=NULL;
+	Object *ob= OBACT;
 	float normal[3]={0.0, 0.0, 0.0};
 	float plane[3]={0.0, 0.0, 0.0};
 	int a, totsel=0;
+
+	t = BIF_GetTransInfo();
 	
 	/* transform widget matrix */
 	Mat4One(v3d->twmat);
 	
+	v3d->twdrawflag= 0xFFFF;
+	
 	/* transform widget centroid/center */
 	G.scene->twcent[0]= G.scene->twcent[1]= G.scene->twcent[2]= 0.0f;
 	INIT_MINMAX(G.scene->twmin, G.scene->twmax);
@@ -185,60 +228,63 @@ int calc_manipulator_stats(ScrArea *sa)
 		if((ob->lay & G.vd->lay)==0) return 0;
 
 		if(G.obedit->type==OB_MESH) {
-			int dmNeedsFree;
-			DerivedMesh *dm = mesh_get_cage_derived(G.obedit, &dmNeedsFree);
 			EditMesh *em = G.editMesh;
 			EditVert *eve;
-			float vec[3];
+			float vec[3]= {0,0,0};
 			int no_faces= 1;
 			
-			if(v3d->twmode == V3D_MANIP_NORMAL) {
-				EditFace *efa;
-				
-				for(efa= em->faces.first; efa; efa= efa->next) {
-					if(efa->f & SELECT) {
-						no_faces= 0;
-						VECADD(normal, normal, efa->n);
-						VecSubf(vec, efa->v2->co, efa->v1->co);
-						VECADD(plane, plane, vec);
-					}
-				}
-			}
-			
-			/* do vertices for center, and if still no normal found, use vertex normals */
-			for(eve= em->verts.first; eve; eve= eve->next) {
-				if(eve->f & SELECT) {
-					if(no_faces) VECADD(normal, normal, eve->no);
+			/* USE LAST SELECTE WITH ACTIVE */
+			if (G.vd->around==V3D_ACTIVE && em->selected.last) {
+				EM_editselection_center(vec, em->selected.last);
+				calc_tw_center(vec);
+				totsel= 1;
+				if (v3d->twmode == V3D_MANIP_NORMAL) {
+					EM_editselection_normal(normal, em->selected.last);
+					EM_editselection_plane(plane, em->selected.last);
+				} /* NORMAL OPERATION */
+			} else {
+				if(v3d->twmode == V3D_MANIP_NORMAL) {
+					EditFace *efa;
 					
-					totsel++;
-					dm->getMappedVertCoEM(dm, eve, vec);
-					calc_tw_center(vec);
+					for(efa= em->faces.first; efa; efa= efa->next) {
+						if(efa->f & SELECT) {
+							no_faces= 0;
+							VECADD(normal, normal, efa->n);
+							VecSubf(vec, efa->v2->co, efa->v1->co);
+							VECADD(plane, plane, vec);
+						}
+					}
 				}
-			}
-			/* the edge case... */
-			if(no_faces && v3d->twmode == V3D_MANIP_NORMAL) {
-				EditEdge *eed;
 				
-				for(eed= em->edges.first; eed; eed= eed->next) {
-					if(eed->f & SELECT) {
-						/* ok we got an edge, only use one, and as normal */
-						VECCOPY(plane, normal);
-						VecSubf(normal, eed->v2->co, eed->v1->co);
-						break;
+				/* do vertices for center, and if still no normal found, use vertex normals */
+				for(eve= em->verts.first; eve; eve= eve->next) {
+					if(eve->f & SELECT) {
+						if(no_faces) VECADD(normal, normal, eve->no);
+						
+						totsel++;
+						calc_tw_center(eve->co);
+					}
+				}
+				/* the edge case... */
+				if(no_faces && v3d->twmode == V3D_MANIP_NORMAL) {
+					EditEdge *eed;
+					
+					for(eed= em->edges.first; eed; eed= eed->next) {
+						if(eed->f & SELECT) {
+							/* ok we got an edge, only use one, and as normal */
+							VECCOPY(plane, normal);
+							VecSubf(normal, eed->v2->co, eed->v1->co);
+							break;
+						}
 					}
 				}
 			}
-			if (dmNeedsFree) {
-				dm->release(dm);
-			}
-		}
+		} /* end editmesh */
 		else if (G.obedit->type==OB_ARMATURE){
+			bArmature *arm= G.obedit->data;
 			EditBone *ebo;
 			for (ebo=G.edbo.first;ebo;ebo=ebo->next){
-				
-				//	If this is an IK child and it's parent is being moved, don't count as selected
-				if ((ebo->flag & BONE_IK_TOPARENT)&& (ebo->flag & BONE_ROOTSEL) && ebo->parent && (ebo->parent->flag & BONE_TIPSEL));
-				else {
+				if(ebo->layer & arm->layer) {
 					if (ebo->flag & BONE_TIPSEL) {
 						calc_tw_center(ebo->tail);
 						totsel++;
@@ -261,17 +307,24 @@ int calc_manipulator_stats(ScrArea *sa)
 					bezt= nu->bezt;
 					a= nu->pntsu;
 					while(a--) {
-						if(bezt->f1) {
-							calc_tw_center(bezt->vec[0]);
-							totsel++;
-						}
-						if(bezt->f2) {
+						/* exception */
+						if( (bezt->f1 & SELECT) + (bezt->f2 & SELECT) + (bezt->f3 & SELECT) > SELECT ) {
 							calc_tw_center(bezt->vec[1]);
 							totsel++;
 						}
-						if(bezt->f3) {
-							calc_tw_center(bezt->vec[2]);
-							totsel++;
+						else {
+							if(bezt->f1) {
+								calc_tw_center(bezt->vec[0]);
+								totsel++;
+							}
+							if(bezt->f2) {
+								calc_tw_center(bezt->vec[1]);
+								totsel++;
+							}
+							if(bezt->f3) {
+								calc_tw_center(bezt->vec[2]);
+								totsel++;
+							}
 						}
 						bezt++;
 					}
@@ -293,16 +346,34 @@ int calc_manipulator_stats(ScrArea *sa)
 		else if(G.obedit->type==OB_MBALL) {
 			/* editmball.c */
 			extern ListBase editelems;  /* go away ! */
-			MetaElem *ml;
+			MetaElem *ml, *ml_sel=NULL;
 		
 			ml= editelems.first;
 			while(ml) {
 				if(ml->flag & SELECT) {
 					calc_tw_center(&ml->x);
+					ml_sel = ml;
 					totsel++;
 				}
 				ml= ml->next;
 			}
+			/* normal manipulator */
+			if(totsel==1){	
+				float mat1[4][4];
+
+				/* Rotation of MetaElem is stored in quat */
+ 				QuatToMat4(ml_sel->quat, mat1);
+
+				/* Translation of MetaElem */
+				mat1[3][0]= ml_sel->x;
+				mat1[3][1]= ml_sel->y;
+				mat1[3][2]= ml_sel->z;
+
+				VECCOPY(normal, mat1[2]);
+				VECCOPY(plane, mat1[1]);
+
+				VecMulf(plane, -1.0);
+			}
 		}
 		else if(G.obedit->type==OB_LATTICE) {
 			BPoint *bp;
@@ -326,27 +397,30 @@ int calc_manipulator_stats(ScrArea *sa)
 			Mat4MulVecfl(G.obedit->obmat, G.scene->twmax);
 		}
 	}
-	else if(G.obpose) {
-		bArmature *arm= G.obpose->data;
+	else if(ob && (ob->flag & OB_POSEMODE)) {
+		bArmature *arm= ob->data;
+		bPoseChannel *pchan;
 		
-		ob= G.obpose;
 		if((ob->lay & G.vd->lay)==0) return 0;
 		
 		Trans.mode= TFM_ROTATION;	// mislead counting bones... bah
 		
-		/* count total */
-		count_bone_select(&Trans, &arm->bonebase, &totsel);
+		/* count total, we use same method as transform will do */
+		Trans.total= 0;
+		count_bone_select(&Trans, arm, &arm->bonebase, 1);
+		totsel= Trans.total;
 		if(totsel) {
-			/* recursive get stats */
-			stats_pose(&arm->bonebase, normal, plane);
-			
+			/* use channels to get stats */
+			for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+				stats_pose(v3d, pchan, normal, plane);
+			}
 			//VecMulf(normal, -1.0);
 			VecMulf(plane, -1.0);
 			
 			VecMulf(G.scene->twcent, 1.0f/(float)totsel);	// centroid!
-			Mat4MulVecfl(G.obpose->obmat, G.scene->twcent);
-			Mat4MulVecfl(G.obpose->obmat, G.scene->twmin);
-			Mat4MulVecfl(G.obpose->obmat, G.scene->twmax);
+			Mat4MulVecfl(ob->obmat, G.scene->twcent);
+			Mat4MulVecfl(ob->obmat, G.scene->twmin);
+			Mat4MulVecfl(ob->obmat, G.scene->twmax);
 		}
 	}
 	else if(G.f & (G_FACESELECT + G_VERTEXPAINT + G_TEXTUREPAINT +G_WEIGHTPAINT)) {
@@ -358,14 +432,14 @@ int calc_manipulator_stats(ScrArea *sa)
 		ob= OBACT;
 		if(ob && !(ob->flag & SELECT)) ob= NULL;
 		
-		base= (G.scene->base.first);
-		while(base) {
+		for(base= G.scene->base.first; base; base= base->next) {
 			if TESTBASELIB(base) {
-				if(ob==NULL) ob= base->object;
+				if(ob==NULL) 
+					ob= base->object;
 				calc_tw_center(base->object->obmat[3]);
+				protectflag_to_drawflags(base->object->protectflag, &v3d->twdrawflag);
 				totsel++;
 			}
-			base= base->next;
 		}
 		
 		/* selection center */
@@ -379,10 +453,12 @@ int calc_manipulator_stats(ScrArea *sa)
 		
 		switch(v3d->twmode) {
 		case V3D_MANIP_GLOBAL:
+			strcpy(t->spacename, "global");
 			break;
 			
 		case V3D_MANIP_NORMAL:
-			if(G.obedit || G.obpose) {
+			if(G.obedit || (ob->flag & OB_POSEMODE)) {
+				strcpy(t->spacename, "normal");
 				if(normal[0]!=0.0 || normal[1]!=0.0 || normal[2]!=0.0) {
 					float imat[3][3], mat[3][3];
 					
@@ -403,18 +479,29 @@ int calc_manipulator_stats(ScrArea *sa)
 					
 					Mat4CpyMat3(v3d->twmat, mat);
 					Mat4Ortho(v3d->twmat);
-					
+
 					break;
 				}
 			}
 			/* no break we define 'normal' as 'local' in Object mode */
 		case V3D_MANIP_LOCAL:
-			if(totsel==1 || v3d->around==V3D_LOCAL || G.obedit || G.obpose) {
+			if(totsel==1 || v3d->around==V3D_LOCAL || G.obedit || (ob->flag & OB_POSEMODE)) {
+				strcpy(t->spacename, "local");
 				Mat4CpyMat4(v3d->twmat, ob->obmat);
 				Mat4Ortho(v3d->twmat);
 			}
 			break;
-		}		
+		case V3D_MANIP_VIEW:
+			{
+				float mat[3][3];
+				strcpy(t->spacename, "view");
+				Mat3CpyMat4(mat, v3d->viewinv);
+				Mat3Ortho(mat);
+				Mat4CpyMat3(v3d->twmat, mat);
+			}
+			break;
+		}
+		
 	}
 	   
 	return totsel;
@@ -580,8 +667,8 @@ static void draw_manipulator_axes(int colcode, int flagx, int flagy, int flagz)
 		if(flagx & MAN_SCALE_X) glLoadName(MAN_SCALE_X);
 		else if(flagx & MAN_TRANS_X) glLoadName(MAN_TRANS_X);
 		glBegin(GL_LINES);
-		glVertex3f(0.2, 0.0, 0.0);
-		glVertex3f(1.0, 0.0, 0.0);
+		glVertex3f(0.2f, 0.0f, 0.0f);
+		glVertex3f(1.0f, 0.0f, 0.0f);
 		glEnd();
 	}		
 	if(flagy) {
@@ -589,8 +676,8 @@ static void draw_manipulator_axes(int colcode, int flagx, int flagy, int flagz)
 		else if(flagy & MAN_TRANS_Y) glLoadName(MAN_TRANS_Y);
 		manipulator_setcolor('y', colcode);
 		glBegin(GL_LINES);
-		glVertex3f(0.0, 0.2, 0.0);
-		glVertex3f(0.0, 1.0, 0.0);
+		glVertex3f(0.0f, 0.2f, 0.0f);
+		glVertex3f(0.0f, 1.0f, 0.0f);
 		glEnd();
 	}		
 	if(flagz) {
@@ -598,8 +685,8 @@ static void draw_manipulator_axes(int colcode, int flagx, int flagy, int flagz)
 		else if(flagz & MAN_TRANS_Z) glLoadName(MAN_TRANS_Z);
 		manipulator_setcolor('z', colcode);
 		glBegin(GL_LINES);
-		glVertex3f(0.0, 0.0, 0.2);
-		glVertex3f(0.0, 0.0, 1.0);
+		glVertex3f(0.0f, 0.0f, 0.2f);
+		glVertex3f(0.0f, 0.0f, 1.0f);
 		glEnd();
 	}
 }
@@ -673,9 +760,9 @@ static void draw_manipulator_rotate_ghost(float mat[][4], int drawflags)
 			svec[1]+= tmat[2][1];
 			Normalise(svec);
 			
-			startphi= atan2(svec[0], svec[1]);
+			startphi= (float)atan2(svec[0], svec[1]);
 		}
-		else startphi= 0.5*M_PI;
+		else startphi= 0.5f*(float)M_PI;
 		
 		VECCOPY(vec, mat[0]);	// use x axis to detect rotation
 		Normalise(vec);
@@ -695,9 +782,9 @@ static void draw_manipulator_rotate_ghost(float mat[][4], int drawflags)
 			svec[2]+= tmat[2][2];
 			Normalise(svec);
 			
-			startphi= M_PI + atan2(svec[2], -svec[1]);
+			startphi= (float)(M_PI + atan2(svec[2], -svec[1]));
 		}
-		else startphi= 0.0;
+		else startphi= 0.0f;
 		
 		VECCOPY(vec, mat[1]);	// use y axis to detect rotation
 		Normalise(vec);
@@ -719,9 +806,9 @@ static void draw_manipulator_rotate_ghost(float mat[][4], int drawflags)
 			svec[2]+= tmat[2][2];
 			Normalise(svec);
 			
-			startphi= M_PI + atan2(-svec[0], svec[2]);
+			startphi= (float)(M_PI + atan2(-svec[0], svec[2]));
 		}
-		else startphi= M_PI;
+		else startphi= (float)M_PI;
 		
 		VECCOPY(vec, mat[2]);	// use z axis to detect rotation
 		Normalise(vec);
@@ -825,18 +912,18 @@ static void draw_manipulator_rotate(float mat[][4], int moving, int drawflags, i
 				glBegin(GL_LINES);
 				if( (drawflags & MAN_ROT_X) || (moving && (drawflags & MAN_ROT_Z)) ) {
 					manipulator_setcolor('x', colcode);
-					glVertex3f(0.2, 0.0, 0.0);
-					glVertex3f(1.0, 0.0, 0.0);
+					glVertex3f(0.2f, 0.0f, 0.0f);
+					glVertex3f(1.0f, 0.0f, 0.0f);
 				}		
 				if( (drawflags & MAN_ROT_Y) || (moving && (drawflags & MAN_ROT_X)) ) {
 					manipulator_setcolor('y', colcode);
-					glVertex3f(0.0, 0.2, 0.0);
-					glVertex3f(0.0, 1.0, 0.0);
+					glVertex3f(0.0f, 0.2f, 0.0f);
+					glVertex3f(0.0f, 1.0f, 0.0f);
 				}		
 				if( (drawflags & MAN_ROT_Z) || (moving && (drawflags & MAN_ROT_Y)) ) {
 					manipulator_setcolor('z', colcode);
-					glVertex3f(0.0, 0.0, 0.2);
-					glVertex3f(0.0, 0.0, 1.0);
+					glVertex3f(0.0f, 0.0f, 0.2f);
+					glVertex3f(0.0f, 0.0f, 1.0f);
 				}
 				glEnd();
 			}
@@ -910,7 +997,7 @@ static void draw_manipulator_rotate(float mat[][4], int moving, int drawflags, i
 			if(G.f & G_PICKSEL) glLoadName(MAN_ROT_Z);
 			manipulator_setcolor('z', colcode);
 
-			partial_donut(0.7*cusize, 1.0, 31, 33, 8, 64);
+			partial_donut(0.7f*cusize, 1.0f, 31, 33, 8, 64);
 
 			glPopMatrix();
 		}	
@@ -923,7 +1010,7 @@ static void draw_manipulator_rotate(float mat[][4], int moving, int drawflags, i
 			
 			glRotatef(90.0, 1.0, 0.0, 0.0);
 			glRotatef(90.0, 0.0, 0.0, 1.0);
-			partial_donut(0.7*cusize, 1.0, 31, 33, 8, 64);
+			partial_donut(0.7f*cusize, 1.0f, 31, 33, 8, 64);
 			
 			glPopMatrix();
 		}
@@ -936,7 +1023,7 @@ static void draw_manipulator_rotate(float mat[][4], int moving, int drawflags, i
 			
 			glRotatef(-90.0, 0.0, 1.0, 0.0);
 			glRotatef(90.0, 0.0, 0.0, 1.0);
-			partial_donut(0.7*cusize, 1.0, 31, 33, 8, 64);
+			partial_donut(0.7f*cusize, 1.0f, 31, 33, 8, 64);
 
 			glPopMatrix();
 		}
@@ -946,7 +1033,7 @@ static void draw_manipulator_rotate(float mat[][4], int moving, int drawflags, i
 	/* restore */
 	myloadmatrix(G.vd->viewmat);
 	gluDeleteQuadric(qobj);
-	if(G.zbuf) glEnable(GL_DEPTH_TEST);		// shouldn't be global, tsk!
+	if(G.vd->zbuf) glEnable(GL_DEPTH_TEST);
 	
 }
 
@@ -971,7 +1058,7 @@ static void draw_manipulator_scale(float mat[][4], int moving, int drawflags, in
 		glPushMatrix();
 		size= screen_aligned(mat);
 		Mat4One(unitmat);
-		drawcircball(GL_LINE_LOOP, unitmat[3], 0.2*size, unitmat);
+		drawcircball(GL_LINE_LOOP, unitmat[3], 0.2f*size, unitmat);
 		glPopMatrix();
 		
 		dz= 1.0;
@@ -1032,7 +1119,7 @@ static void draw_manipulator_scale(float mat[][4], int moving, int drawflags, in
 	/* restore */
 	myloadmatrix(G.vd->viewmat);
 	
-	if(G.zbuf) glEnable(GL_DEPTH_TEST);		// shouldn't be global, tsk!
+	if(G.vd->zbuf) glEnable(GL_DEPTH_TEST);	
 	glFrontFace(GL_CCW);
 }
 
@@ -1084,7 +1171,7 @@ static void draw_manipulator_translate(float mat[][4], int moving, int drawflags
 	glPushMatrix();
 	size= screen_aligned(mat);
 	Mat4One(unitmat);
-	drawcircball(GL_LINE_LOOP, unitmat[3], 0.2*size, unitmat);
+	drawcircball(GL_LINE_LOOP, unitmat[3], 0.2f*size, unitmat);
 	glPopMatrix();
 	
 	/* and now apply matrix, we move to local matrix drawing */
@@ -1099,8 +1186,8 @@ static void draw_manipulator_translate(float mat[][4], int moving, int drawflags
 
 	
 	/* offset in combo mode, for rotate a bit more */
-	if(combo & (V3D_MANIP_ROTATE)) dz= 1.0f+2.0*cylen;
-	else if(combo & (V3D_MANIP_SCALE)) dz= 1.0f+0.5*cylen;
+	if(combo & (V3D_MANIP_ROTATE)) dz= 1.0f+2.0f*cylen;
+	else if(combo & (V3D_MANIP_SCALE)) dz= 1.0f+0.5f*cylen;
 	else dz= 1.0f;
 	
 	/* Z Cone */
@@ -1131,7 +1218,7 @@ static void draw_manipulator_translate(float mat[][4], int moving, int drawflags
 	gluDeleteQuadric(qobj);
 	myloadmatrix(G.vd->viewmat);
 	
-	if(G.zbuf) glEnable(GL_DEPTH_TEST);		// shouldn't be global, tsk!
+	if(G.vd->zbuf) glEnable(GL_DEPTH_TEST);
 	
 }
 
@@ -1231,7 +1318,7 @@ static void draw_manipulator_rotate_cyl(float mat[][4], int moving, int drawflag
 	gluDeleteQuadric(qobj);
 	myloadmatrix(G.vd->viewmat);
 	
-	if(G.zbuf) glEnable(GL_DEPTH_TEST);		// shouldn't be global, tsk!
+	if(G.vd->zbuf) glEnable(GL_DEPTH_TEST);
 	
 }
 
@@ -1327,7 +1414,8 @@ void BIF_draw_manipulator(ScrArea *sa)
 		
 		totsel= calc_manipulator_stats(sa);
 		if(totsel==0) return;
-		
+		drawflags= v3d->twdrawflag;	/* set in calc_manipulator_stats */
+
 		v3d->twflag |= V3D_DRAW_MANIPULATOR;
 
 		/* now we can define centre */
@@ -1337,9 +1425,10 @@ void BIF_draw_manipulator(ScrArea *sa)
 			v3d->twmat[3][0]= (G.scene->twmin[0] + G.scene->twmax[0])/2.0f;
 			v3d->twmat[3][1]= (G.scene->twmin[1] + G.scene->twmax[1])/2.0f;
 			v3d->twmat[3][2]= (G.scene->twmin[2] + G.scene->twmax[2])/2.0f;
-			if(v3d->around==V3D_ACTIVE && G.obedit==NULL && G.obpose==NULL) {
+			if(v3d->around==V3D_ACTIVE && G.obedit==NULL) {
 				Object *ob= OBACT;
-				if(ob) VECCOPY(v3d->twmat[3], ob->obmat[3]);
+				if(ob && !(ob->flag & OB_POSEMODE)) 
+					VECCOPY(v3d->twmat[3], ob->obmat[3]);
 			}
 			break;
 		case V3D_LOCAL:
@@ -1357,6 +1446,7 @@ void BIF_draw_manipulator(ScrArea *sa)
 	if(v3d->twflag & V3D_DRAW_MANIPULATOR) {
 		
 		if(v3d->twtype & V3D_MANIP_ROTATE) {
+			
 			/* rotate has special ghosting draw, for pie chart */
 			if(G.moving) draw_manipulator_rotate_ghost(v3d->twmat, drawflags);
 			
@@ -1410,7 +1500,7 @@ static int manipulator_selectbuf(ScrArea *sa, float hotspot)
 	/* get rid of overlay button matrix */
 	persp(PERSP_VIEW);
 	
-	setwinmatrixview3d(&rect);
+	setwinmatrixview3d(sa->winx, sa->winy, &rect);
 	Mat4MulMat4(v3d->persmat, v3d->viewmat, sa->winmat);
 	
 	glSelectBuffer( 64, buffer);
@@ -1420,41 +1510,57 @@ static int manipulator_selectbuf(ScrArea *sa, float hotspot)
 	
 	/* do the drawing */
 	if(v3d->twtype & V3D_MANIP_ROTATE) {
-		if(G.rt==3) draw_manipulator_rotate_cyl(v3d->twmat, 0, MAN_ROT_C, v3d->twtype, MAN_RGB);
-		else draw_manipulator_rotate(v3d->twmat, 0, MAN_ROT_C, v3d->twtype);
+		if(G.rt==3) draw_manipulator_rotate_cyl(v3d->twmat, 0, MAN_ROT_C & v3d->twdrawflag, v3d->twtype, MAN_RGB);
+		else draw_manipulator_rotate(v3d->twmat, 0, MAN_ROT_C & v3d->twdrawflag, v3d->twtype);
 	}
 	if(v3d->twtype & V3D_MANIP_SCALE)
-		draw_manipulator_scale(v3d->twmat, 0, MAN_SCALE_C, v3d->twtype, MAN_RGB);
+		draw_manipulator_scale(v3d->twmat, 0, MAN_SCALE_C & v3d->twdrawflag, v3d->twtype, MAN_RGB);
 	if(v3d->twtype & V3D_MANIP_TRANSLATE)
-		draw_manipulator_translate(v3d->twmat, 0, MAN_TRANS_C, v3d->twtype, MAN_RGB);
+		draw_manipulator_translate(v3d->twmat, 0, MAN_TRANS_C & v3d->twdrawflag, v3d->twtype, MAN_RGB);
 	
 	glPopName();
 	hits= glRenderMode(GL_RENDER);
 	
 	G.f &= ~G_PICKSEL;
-	setwinmatrixview3d(0);
+	setwinmatrixview3d(sa->winx, sa->winy, NULL);
 	Mat4MulMat4(v3d->persmat, v3d->viewmat, sa->winmat);
 	
 	persp(PERSP_WIN);
 	
 	if(hits==1) return buffer[3];
 	else if(hits>1) {
-		GLuint mindep, minval;
+		GLuint val, dep, mindep=0, mindeprot=0, minval=0, minvalrot=0;
 		int a;
 		
 		/* we compare the hits in buffer, but value centers highest */
-		mindep= buffer[1];
-		minval= buffer[3];
+		/* we also store the rotation hits separate (because of arcs) and return hits on other widgets if there are */
 
-		for(a=1; a<hits; a++) {
-			if(minval==MAN_TRANS_C || minval==MAN_SCALE_C) break;
+		for(a=0; a<hits; a++) {
+			dep= buffer[4*a + 1];
+			val= buffer[4*a + 3];
 			
-			if(buffer[4*a + 3]==MAN_TRANS_C || buffer[4*a + 3]==MAN_SCALE_C || buffer[4*a + 1] < mindep) {
-				mindep= buffer[4*a + 1];
-				minval= buffer[4*a + 3];
+			if(val==MAN_TRANS_C) return MAN_TRANS_C;
+			else if(val==MAN_SCALE_C) return MAN_SCALE_C;
+			else {
+				if(val & MAN_ROT_C) {
+					if(minvalrot==0 || dep<mindeprot) {
+						mindeprot= dep;
+						minvalrot= val;
+					}
+				}
+				else {
+					if(minval==0 || dep<mindep) {
+						mindep= dep;
+						minval= val;
+					}
+				}
 			}
 		}
-		return minval;
+		
+		if(minval)
+			return minval;
+		else
+			return minvalrot;
 	}
 	return 0;
 }
@@ -1475,90 +1581,88 @@ int BIF_do_manipulator(ScrArea *sa)
 		// drawflags still global, for drawing call above
 		drawflags= manipulator_selectbuf(sa, 0.2f*(float)U.tw_hotspot);
 		if(drawflags==0) drawflags= val;
-		
-		switch(drawflags) {
-		case MAN_TRANS_C:
-			ManipulatorTransform(TFM_TRANSLATION);
-			break;
-		case MAN_TRANS_X:
-			if(G.qual & LR_SHIFTKEY) {
-				drawflags= MAN_TRANS_Y|MAN_TRANS_Z;
-				BIF_setDualAxisConstraint(v3d->twmat[1], v3d->twmat[2], " Y+Z");
+
+		if (drawflags & MAN_TRANS_C) {
+			initManipulator(TFM_TRANSLATION);
+			switch(drawflags) {
+			case MAN_TRANS_C:
+				break;
+			case MAN_TRANS_X:
+				if(G.qual & LR_SHIFTKEY) {
+					drawflags= MAN_TRANS_Y|MAN_TRANS_Z;
+					BIF_setDualAxisConstraint(v3d->twmat[1], v3d->twmat[2], " Y+Z");
+				}
+				else
+					BIF_setSingleAxisConstraint(v3d->twmat[0], " X");
+				break;
+			case MAN_TRANS_Y:
+				if(G.qual & LR_SHIFTKEY) {
+					drawflags= MAN_TRANS_X|MAN_TRANS_Z;
+					BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[2], " X+Z");
+				}
+				else
+					BIF_setSingleAxisConstraint(v3d->twmat[1], " Y");
+				break;
+			case MAN_TRANS_Z:
+				if(G.qual & LR_SHIFTKEY) {
+					drawflags= MAN_TRANS_X|MAN_TRANS_Y;
+					BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[1], " X+Y");
+				}
+				else
+					BIF_setSingleAxisConstraint(v3d->twmat[2], " Z");
+				break;
 			}
-			else
+			ManipulatorTransform();
+		}
+		else if (drawflags & MAN_SCALE_C) {
+			initManipulator(TFM_RESIZE);
+			switch(drawflags) {
+			case MAN_SCALE_X:
+				if(G.qual & LR_SHIFTKEY) {
+					drawflags= MAN_SCALE_Y|MAN_SCALE_Z;
+					BIF_setDualAxisConstraint(v3d->twmat[1], v3d->twmat[2], " Y+Z");
+				}
+				else
+					BIF_setSingleAxisConstraint(v3d->twmat[0], " X");
+				break;
+			case MAN_SCALE_Y:
+				if(G.qual & LR_SHIFTKEY) {
+					drawflags= MAN_SCALE_X|MAN_SCALE_Z;
+					BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[2], " X+Z");
+				}
+				else
+					BIF_setSingleAxisConstraint(v3d->twmat[1], " Y");
+				break;
+			case MAN_SCALE_Z:
+				if(G.qual & LR_SHIFTKEY) {
+					drawflags= MAN_SCALE_X|MAN_SCALE_Y;
+					BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[1], " X+Y");
+				}
+				else
+					BIF_setSingleAxisConstraint(v3d->twmat[2], " Z");
+				break;
+			}
+			ManipulatorTransform();
+		}
+		else if (drawflags == MAN_ROT_T) { /* trackbal need special case, init is different */
+			initManipulator(TFM_TRACKBALL);
+			ManipulatorTransform();
+		}
+		else if (drawflags & MAN_ROT_C) {
+			initManipulator(TFM_ROTATION);
+			switch(drawflags) {
+			case MAN_ROT_X:
 				BIF_setSingleAxisConstraint(v3d->twmat[0], " X");
-			ManipulatorTransform(TFM_TRANSLATION);
-			break;
-		case MAN_TRANS_Y:
-			if(G.qual & LR_SHIFTKEY) {
-				drawflags= MAN_TRANS_X|MAN_TRANS_Z;
-				BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[2], " X+Z");
-			}
-			else
+				break;
+			case MAN_ROT_Y:
 				BIF_setSingleAxisConstraint(v3d->twmat[1], " Y");
-			ManipulatorTransform(TFM_TRANSLATION);
-			break;
-		case MAN_TRANS_Z:
-			if(G.qual & LR_SHIFTKEY) {
-				drawflags= MAN_TRANS_X|MAN_TRANS_Y;
-				BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[1], " X+Y");
-			}
-			else
+				break;
+			case MAN_ROT_Z:
 				BIF_setSingleAxisConstraint(v3d->twmat[2], " Z");
-			ManipulatorTransform(TFM_TRANSLATION);
-			break;
-			
-		case MAN_SCALE_C:
-			ManipulatorTransform(TFM_RESIZE);
-			break;
-		case MAN_SCALE_X:
-			if(G.qual & LR_SHIFTKEY) {
-				drawflags= MAN_SCALE_Y|MAN_SCALE_Z;
-				BIF_setDualAxisConstraint(v3d->twmat[1], v3d->twmat[2], " Y+Z");
-			}
-			else
-				BIF_setSingleAxisConstraint(v3d->twmat[0], " X");
-			ManipulatorTransform(TFM_RESIZE);
-			break;
-		case MAN_SCALE_Y:
-			if(G.qual & LR_SHIFTKEY) {
-				drawflags= MAN_SCALE_X|MAN_SCALE_Z;
-				BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[2], " X+Z");
-			}
-			else
-				BIF_setSingleAxisConstraint(v3d->twmat[1], " Y");
-			ManipulatorTransform(TFM_RESIZE);
-			break;
-		case MAN_SCALE_Z:
-			if(G.qual & LR_SHIFTKEY) {
-				drawflags= MAN_SCALE_X|MAN_SCALE_Y;
-				BIF_setDualAxisConstraint(v3d->twmat[0], v3d->twmat[1], " X+Y");
+				break;
 			}
-			else
-				BIF_setSingleAxisConstraint(v3d->twmat[2], " Z");
-			ManipulatorTransform(TFM_RESIZE);
-			break;
-		
-		case MAN_ROT_X:
-			BIF_setSingleAxisConstraint(v3d->twmat[0], " X");
-			ManipulatorTransform(TFM_ROTATION);
-			break;
-		case MAN_ROT_Y:
-			BIF_setSingleAxisConstraint(v3d->twmat[1], " Y");
-			ManipulatorTransform(TFM_ROTATION);
-			break;
-		case MAN_ROT_Z:
-			BIF_setSingleAxisConstraint(v3d->twmat[2], " Z");
-			ManipulatorTransform(TFM_ROTATION);
-			break;
-		case MAN_ROT_T:
-			ManipulatorTransform(TFM_TRACKBALL);
-			break;			
-		case MAN_ROT_V:
-			ManipulatorTransform(TFM_ROTATION);
-			break;
+			ManipulatorTransform();
 		}
-		
 	}
 	/* after transform, restore drawflags */
 	drawflags= 0xFFFF;
