@@ -1,5 +1,5 @@
 /**
- * $Id: view.c,v 1.46 2005/06/05 13:50:21 theeth Exp $
+ * $Id: view.c,v 1.69 2006/07/19 11:28:48 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -50,23 +50,27 @@
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
+#include "DNA_action_types.h"
+#include "DNA_armature_types.h"
+#include "DNA_camera_types.h"
+#include "DNA_lamp_types.h"
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_space_types.h"
-#include "DNA_view3d_types.h"
-#include "DNA_camera_types.h"
-#include "DNA_lamp_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_view3d_types.h"
 
-#include "BKE_utildefines.h"
-#include "BKE_object.h"
+#include "BKE_anim.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_object.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_gl.h"
 #include "BIF_space.h"
 #include "BIF_mywindow.h"
+#include "BIF_previewrender.h"
 #include "BIF_screen.h"
 #include "BIF_toolbox.h"
 
@@ -80,9 +84,6 @@
 #include "mydevice.h"
 #include "blendef.h"
 
-/* Modules used */
-#include "render.h"		/* R. stuff for ogl view render */
-
 #define TRACKBALLSIZE  (1.1)
 #define BL_NEAR_CLIP 0.001
 
@@ -137,31 +138,25 @@ void persp(int a)
 }
 
 
-float zfac=1.0;
-
 void initgrabz(float x, float y, float z)
 {
-	if(G.vd==0) return;
-	zfac= G.vd->persmat[0][3]*x+ G.vd->persmat[1][3]*y+ G.vd->persmat[2][3]*z+ G.vd->persmat[3][3];
+	if(G.vd==NULL) return;
+	G.vd->zfac= G.vd->persmat[0][3]*x+ G.vd->persmat[1][3]*y+ G.vd->persmat[2][3]*z+ G.vd->persmat[3][3];
+
+	/* if x,y,z is exactly the viewport offset, zfac is 0 and we don't want that 
+	 * (accounting for near zero values)
+	 * */
+	if (G.vd->zfac < 1.e-6f && G.vd->zfac > -1.e-6f) G.vd->zfac = 1.0f;
 }
 
 void window_to_3d(float *vec, short mx, short my)
 {
-	/* always call initzgrab */
+	/* always call initgrabz */
 	float dx, dy;
-	float fmx, fmy, winx, winy;
 	
-	/* stupid! */
-	winx= curarea->winx;
-	winy= curarea->winy;
-	fmx= mx;
-	fmy= my;
+	dx= 2.0f*mx*G.vd->zfac/curarea->winx;
+	dy= 2.0f*my*G.vd->zfac/curarea->winy;
 	
-	dx= (2.0*fmx)/winx;
-	dx*= zfac;
-	dy= (2.0*fmy)/winy;
-	dy*= zfac;
-
 	vec[0]= (G.vd->persinv[0][0]*dx + G.vd->persinv[1][0]*dy);
 	vec[1]= (G.vd->persinv[0][1]*dx + G.vd->persinv[1][1]*dy);
 	vec[2]= (G.vd->persinv[0][2]*dx + G.vd->persinv[1][2]*dy);
@@ -171,12 +166,17 @@ void project_short(float *vec, short *adr)	/* clips */
 {
 	float fx, fy, vec4[4];
 
-	adr[0]= 3200;
+	adr[0]= IS_CLIPPED;
+	
+	if(G.vd->flag & V3D_CLIPPING) {
+		if(view3d_test_clipping(G.vd, vec))
+			return;
+	}
+
 	VECCOPY(vec4, vec);
 	vec4[3]= 1.0;
-	
 	Mat4MulVec4fl(G.vd->persmat, vec4);
-
+	
 	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
 		fx= (curarea->winx/2)*(1 + vec4[0]/vec4[3]);
 		
@@ -220,7 +220,7 @@ void project_short_noclip(float *vec, short *adr)
 {
 	float fx, fy, vec4[4];
 
-	adr[0]= 3200;
+	adr[0]= IS_CLIPPED;
 	VECCOPY(vec4, vec);
 	vec4[3]= 1.0;
 	
@@ -245,7 +245,7 @@ void project_float(float *vec, float *adr)
 {
 	float vec4[4];
 	
-	adr[0]= 3200.0;
+	adr[0]= IS_CLIPPED;
 	VECCOPY(vec4, vec);
 	vec4[3]= 1.0;
 	
@@ -257,6 +257,101 @@ void project_float(float *vec, float *adr)
 	}
 }
 
+void view3d_get_object_project_mat(ScrArea *area, Object *ob, float pmat[4][4], float vmat[4][4])
+{
+	if (area->spacetype!=SPACE_VIEW3D || !area->spacedata.first) {
+		Mat4One(pmat);
+		Mat4One(vmat);
+	} else {
+		View3D *vd = area->spacedata.first;
+
+		Mat4MulMat4(vmat, ob->obmat, vd->viewmat);
+		Mat4MulMat4(pmat, vmat, vd->winmat1);
+		Mat4CpyMat4(vmat, ob->obmat);
+	}
+}
+
+/* projectmat brings it to window coords, wmat to rotated world space */
+void view3d_project_short_clip(ScrArea *area, float *vec, short *adr, float projmat[4][4], float wmat[4][4])
+{
+	View3D *v3d= area->spacedata.first;
+	float fx, fy, vec4[4];
+
+	adr[0]= IS_CLIPPED;
+	
+	/* clipplanes in eye space */
+	if(v3d->flag & V3D_CLIPPING) {
+		VECCOPY(vec4, vec);
+		Mat4MulVecfl(wmat, vec4);
+		if(view3d_test_clipping(v3d, vec4))
+			return;
+	}
+	
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(projmat, vec4);
+	
+	/* clipplanes in window space */
+	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
+		fx= (area->winx/2)*(1 + vec4[0]/vec4[3]);
+		
+		if( fx>0 && fx<area->winx) {
+		
+			fy= (area->winy/2)*(1 + vec4[1]/vec4[3]);
+			
+			if(fy>0.0 && fy< (float)area->winy) {
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
+			}
+		}
+	}
+}
+
+void view3d_project_short_noclip(ScrArea *area, float *vec, short *adr, float mat[4][4])
+{
+	float fx, fy, vec4[4];
+
+	adr[0]= IS_CLIPPED;
+	
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(mat, vec4);
+
+	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
+		fx= (area->winx/2)*(1 + vec4[0]/vec4[3]);
+		
+		if( fx>-32700 && fx<32700) {
+		
+			fy= (area->winy/2)*(1 + vec4[1]/vec4[3]);
+			
+			if(fy>-32700.0 && fy<32700.0) {
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
+			}
+		}
+	}
+}
+
+void view3d_project_float(ScrArea *area, float *vec, float *adr, float mat[4][4])
+{
+	float vec4[4];
+	
+	adr[0]= IS_CLIPPED;
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(mat, vec4);
+
+	if( vec4[3]>FLT_EPSILON ) {
+		adr[0] = (area->winx/2.0)+(area->winx/2.0)*vec4[0]/vec4[3];	
+		adr[1] = (area->winy/2.0)+(area->winy/2.0)*vec4[1]/vec4[3];
+	} else {
+		adr[0] = adr[1] = 0.0;
+	}
+}
+
 int boundbox_clip(float obmat[][4], BoundBox *bb)
 {
 	/* return 1: draw */
@@ -432,12 +527,23 @@ void calctrackballvec(rcti *area, short *mval, float *vec)
 
 void viewmove(int mode)
 {
+	Object *ob = OBACT;
 	float firstvec[3], newvec[3], dvec[3];
-	float oldquat[4], q1[4], si, phi, dist0;
+	float reverse, oldquat[4], q1[4], si, phi, dist0;
+	float ofs[3], obofs[3];
 	int firsttime=1;
 	short mvalball[2], mval[2], mvalo[2];
+	short use_sel = 0;
+	short preview3d_event= 1;
 	
+	/* 3D window may not be defined */
+	if( !G.vd ) {
+		fprintf( stderr, "G.vd == NULL in viewmove()\n" );
+		return;
+	}
+
 	/* sometimes this routine is called from headerbuttons */
+
 	areawinset(curarea->win);
 	
 	initgrabz(-G.vd->ofs[0], -G.vd->ofs[1], -G.vd->ofs[2]);
@@ -453,6 +559,21 @@ void viewmove(int mode)
 
 	/* cumultime(0); */
 
+	if (ob && (U.uiflag & USER_ORBIT_SELECTION)) {
+		use_sel = 1;
+		VECCOPY(ofs, G.vd->ofs);
+
+		obofs[0] = -ob->obmat[3][0];
+		obofs[1] = -ob->obmat[3][1];
+		obofs[2] = -ob->obmat[3][2];
+	}
+	else
+		ofs[0] = ofs[1] = ofs[2] = 0.0f;
+
+	reverse= 1.0f;
+	if (G.vd->persmat[2][1] < 0.0f)
+		reverse= -1.0f;
+
 	while(TRUE) {
 		getmouseco_sc(mval);
 		
@@ -467,15 +588,24 @@ void viewmove(int mode)
 					if(G.vd->view!=0) scrarea_queue_headredraw(curarea);	/*for button */
 					G.vd->view= 0;
 				}
-						
-				if(G.vd->persp==2 || (G.vd->persp==3 && mode!=1)) {
+				if(G.vd->persp==2 && mode!=1) {
+					float upvec[3]={0,0,0}; /* the view vector */
+					float mat[3][3]; /* view matrix 3x3 to rotate the upvec */
+					where_is_object(G.vd->camera);
+					VecCopyf(G.vd->ofs, G.vd->camera->obmat[3]);
+					VecMulf(G.vd->ofs, -1.0f); /*flip the vector*/
+					
+					upvec[2]= G.vd->dist;
+					Mat3CpyMat4(mat, G.vd->viewinv);
+					Mat3MulVecfl(mat, upvec);
+					VecAddf(G.vd->ofs, G.vd->ofs, upvec);
+					
 					G.vd->persp= 1;
 					scrarea_do_windraw(curarea);
 					scrarea_queue_headredraw(curarea);
 				}
 			}
 
-
 			if(mode==0) {	/* view rotate */
 				if (U.uiflag & USER_AUTOPERSP) G.vd->persp= 1;
 
@@ -510,8 +640,21 @@ void viewmove(int mode)
 					q1[0]= cos(phi);
 					q1[1]*= si;
 					q1[2]*= si;
-					q1[3]*= si;						
+					q1[3]*= si;	
 					QuatMul(G.vd->viewquat, q1, oldquat);
+
+					if (use_sel) {
+						/* compute the post multiplication quat, to rotate the offset correctly */
+						QUATCOPY(q1, oldquat);
+						QuatConj(q1);
+						QuatMul(q1, q1, G.vd->viewquat);
+
+						QuatConj(q1); /* conj == inv for unit quat */
+						VECCOPY(G.vd->ofs, ofs);
+						VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+						QuatMulVecf(q1, G.vd->ofs);
+						VecAddf(G.vd->ofs, G.vd->ofs, obofs);
+					}
 				} else {
 					/* New turntable view code by John Aughey */
 
@@ -541,25 +684,37 @@ void viewmove(int mode)
 					q1[3] = si * xvec[2];
 					QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
 
+					if (use_sel) {
+						QuatConj(q1); /* conj == inv for unit quat */
+						VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+						QuatMulVecf(q1, G.vd->ofs);
+						VecAddf(G.vd->ofs, G.vd->ofs, obofs);
+					}
+
 					/* Perform the orbital rotation */
-					phi = sensitivity * (mval[0] - mvalo[0]);
+					phi = sensitivity * reverse * (mval[0] - mvalo[0]);
 					q1[0] = cos(phi);
 					q1[1] = q1[2] = 0.0;
 					q1[3] = sin(phi);
 					QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+
+					if (use_sel) {
+						QuatConj(q1);
+						VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+						QuatMulVecf(q1, G.vd->ofs);
+						VecAddf(G.vd->ofs, G.vd->ofs, obofs);
+					}
 				}
 			}
 			else if(mode==1) {	/* translate */
-				if(G.vd->persp==3) {
-					/* zoom= 0.5+0.5*(float)(2<<G.vd->rt1); */
-					/* dx-= (mval[0]-mvalo[0])/zoom; */
-					/* dy-= (mval[1]-mvalo[1])/zoom; */
-					/* G.vd->rt2= dx; */
-					/* G.vd->rt3= dy; */
-					/* if(G.vd->rt2<-320) G.vd->rt2= -320; */
-					/* if(G.vd->rt2> 320) G.vd->rt2=  320; */
-					/* if(G.vd->rt3<-250) G.vd->rt3= -250; */
-					/* if(G.vd->rt3> 250) G.vd->rt3=  250; */
+				if(G.vd->persp==2) {
+					float max= (float)MAX2(curarea->winx, curarea->winy);
+
+					G.vd->camdx += (mvalo[0]-mval[0])/(max);
+					G.vd->camdy += (mvalo[1]-mval[1])/(max);
+					CLAMP(G.vd->camdx, -1.0f, 1.0f);
+					CLAMP(G.vd->camdy, -1.0f, 1.0f);
+					preview3d_event= 0;
 				}
 				else {
 					window_to_3d(dvec, mval[0]-mvalo[0], mval[1]-mvalo[1]);
@@ -596,6 +751,8 @@ void viewmove(int mode)
 				
 				mval[1]= mvalo[1]; /* preserve first value */
 				mval[0]= mvalo[0];
+				
+				if(G.vd->persp==0 || G.vd->persp==2) preview3d_event= 0;
 			}
 			
 			mvalo[0]= mval[0];
@@ -619,26 +776,29 @@ void viewmove(int mode)
 		/* this in the end, otherwise get_mbut does not work on a PC... */
 		if( !(get_mbut() & (L_MOUSE|M_MOUSE))) break;
 	}
+
+	if(preview3d_event) 
+		BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT);
+	else
+		BIF_view3d_previewrender_signal(curarea, PR_PROJECTED);
+
 }
 
-short v3d_windowmode=0;
-
-/* important to not set windows active in here, can be renderwin for example */
-void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
+int get_view3d_viewplane(int winxi, int winyi, rctf *viewplane, float *clipsta, float *clipend)
 {
-	Camera *cam=0;
-	float near, far, winx = 0.0, winy = 0.0;
+	Camera *cam=NULL;
 	float lens, fac, x1, y1, x2, y2;
-	short orth;
+	float winx= (float)winxi, winy= (float)winyi;
+	int orth= 0;
 	
 	lens= G.vd->lens;	
 	
-	near= G.vd->near;
-	far= G.vd->far;
+	*clipsta= G.vd->near;
+	*clipend= G.vd->far;
 
 	if(G.vd->persp==2) {
-		near= G.vd->near;
-		far= G.vd->far;
+		*clipsta= G.vd->near;
+		*clipend= G.vd->far;
 		if(G.vd->camera) {
 			if(G.vd->camera->type==OB_LAMP ) {
 				Lamp *la;
@@ -649,27 +809,18 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 				x1= saacos(fac);
 				lens= 16.0*fac/sin(x1);
 		
-				near= la->clipsta;
-				far= la->clipend;
+				*clipsta= la->clipsta;
+				*clipend= la->clipend;
 			}
 			else if(G.vd->camera->type==OB_CAMERA) {
 				cam= G.vd->camera->data;
 				lens= cam->lens;
-				near= cam->clipsta;
-				far= cam->clipend;
+				*clipsta= cam->clipsta;
+				*clipend= cam->clipend;
 			}
 		}
 	}
 	
-	if(v3d_windowmode) { // hackish
-		winx= R.rectx;
-		winy= R.recty;
-	}
-	else {
-		winx= curarea->winx;
-		winy= curarea->winy;
-	}
-	
 	if(G.vd->persp==0) {
 		if(winx>winy) x1= -G.vd->dist;
 		else x1= -winx*G.vd->dist/winy;
@@ -679,7 +830,8 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 		else y1= -G.vd->dist;
 		y2= -y1;
 		
-		near= -far;
+		*clipend *= 0.5;	// otherwise too extreme low zbuffer quality
+		*clipsta= - *clipend;
 		orth= 1;
 	}
 	else {
@@ -709,46 +861,69 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 			if(winx>winy) dfac= 64.0/(fac*winx*lens);
 			else dfac= 64.0/(fac*winy*lens);
 			
-			x1= - near*winx*dfac;
+			x1= - *clipsta * winx*dfac;
 			x2= -x1;
-			y1= - near*winy*dfac;
+			y1= - *clipsta * winy*dfac;
 			y2= -y1;
 			orth= 0;
 		}
-	}
-
-	if(rect) {		/* picking */
-		rect->xmin/= winx;
-		rect->xmin= x1+rect->xmin*(x2-x1);
-		rect->ymin/= winy;
-		rect->ymin= y1+rect->ymin*(y2-y1);
-		rect->xmax/= winx;
-		rect->xmax= x1+rect->xmax*(x2-x1);
-		rect->ymax/= winy;
-		rect->ymax= y1+rect->ymax*(y2-y1);
-
-		if(orth) myortho(rect->xmin, rect->xmax, rect->ymin, rect->ymax, -far, far);
-		else mywindow(rect->xmin, rect->xmax, rect->ymin, rect->ymax, near, far);
-
-	}
-	else {
-		if(v3d_windowmode) {
-			if(orth) i_ortho(x1, x2, y1, y2, near, far, R.winmat);
-			else i_window(x1, x2, y1, y2, near, far, R.winmat);
-		}
-		else {
-			if(orth) myortho(x1, x2, y1, y2, near, far);
-			else mywindow(x1, x2, y1, y2, near, far);
+		/* cam view offset */
+		if(cam) {
+			float dx= G.vd->camdx*(x2-x1);
+			float dy= G.vd->camdy*(y2-y1);
+			x1+= dx;
+			x2+= dx;
+			y1+= dy;
+			y2+= dy;
 		}
 	}
-
-	if(v3d_windowmode==0) {
-		glMatrixMode(GL_PROJECTION);
-		mygetmatrix(curarea->winmat);
-		glMatrixMode(GL_MODELVIEW);
-	}
+	
+	viewplane->xmin= x1;
+	viewplane->ymin= y1;
+	viewplane->xmax= x2;
+	viewplane->ymax= y2;
+	
+	return orth;
 }
 
+/* important to not set windows active in here, can be renderwin for example */
+void setwinmatrixview3d(int winx, int winy, rctf *rect)		/* rect: for picking */
+{
+	rctf viewplane;
+	float clipsta, clipend, x1, y1, x2, y2;
+	int orth;
+	
+	orth= get_view3d_viewplane(winx, winy, &viewplane, &clipsta, &clipend);
+//	printf("%d %d %f %f %f %f %f %f\n", winx, winy, viewplane.xmin, viewplane.ymin, viewplane.xmax, viewplane.ymax, clipsta, clipend);
+	x1= viewplane.xmin;
+	y1= viewplane.ymin;
+	x2= viewplane.xmax;
+	y2= viewplane.ymax;
+	
+	if(rect) {		/* picking */
+		rect->xmin/= (float)curarea->winx;
+		rect->xmin= x1+rect->xmin*(x2-x1);
+		rect->ymin/= (float)curarea->winy;
+		rect->ymin= y1+rect->ymin*(y2-y1);
+		rect->xmax/= (float)curarea->winx;
+		rect->xmax= x1+rect->xmax*(x2-x1);
+		rect->ymax/= (float)curarea->winy;
+		rect->ymax= y1+rect->ymax*(y2-y1);
+		
+		if(orth) myortho(rect->xmin, rect->xmax, rect->ymin, rect->ymax, -clipend, clipend);
+		else mywindow(rect->xmin, rect->xmax, rect->ymin, rect->ymax, clipsta, clipend);
+		
+	}
+	else {
+		if(orth) myortho(x1, x2, y1, y2, clipsta, clipend);
+		else mywindow(x1, x2, y1, y2, clipsta, clipend);
+	}
+
+	/* not sure what this was for? (ton) */
+	glMatrixMode(GL_PROJECTION);
+	mygetmatrix(curarea->winmat);
+	glMatrixMode(GL_MODELVIEW);
+}
 
 void obmat_to_viewmat(Object *ob)
 {
@@ -814,10 +989,9 @@ void setcameratoview3d(void)
 }
 
 /* IGLuint-> GLuint*/
-short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short y2)
+short  view3d_opengl_select(unsigned int *buffer, unsigned int bufsize, short x1, short y1, short x2, short y2)
 {
 	rctf rect;
-	Base *base;
 	short mval[2], code, hits;
 
 	G.f |= G_PICKSEL;
@@ -837,51 +1011,81 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 	}
 	/* get rid of overlay button matrix */
 	persp(PERSP_VIEW);
-	setwinmatrixview3d(&rect);
+	setwinmatrixview3d(curarea->winx, curarea->winy, &rect);
 	Mat4MulMat4(G.vd->persmat, G.vd->viewmat, curarea->winmat);
 	
 	if(G.vd->drawtype > OB_WIRE) {
-		G.zbuf= TRUE;
+		G.vd->zbuf= TRUE;
 		glEnable(GL_DEPTH_TEST);
 	}
 
-	glSelectBuffer( MAXPICKBUF, (GLuint *)buffer);
+	if(G.vd->flag & V3D_CLIPPING)
+		view3d_set_clipping(G.vd);
+	
+	glSelectBuffer( bufsize, (GLuint *)buffer);
 	glRenderMode(GL_SELECT);
 	glInitNames();	/* these two calls whatfor? It doesnt work otherwise */
 	glPushName(-1);
 	code= 1;
 	
 	if(G.obedit && G.obedit->type==OB_MBALL) {
-		draw_object(BASACT);
+		draw_object(BASACT, DRAW_PICKING|DRAW_CONSTCOLOR);
 	}
-	else if ((G.obedit && G.obedit->type==OB_ARMATURE)||(G.obpose && G.obpose->type==OB_ARMATURE)) {
-		draw_object(BASACT);
+	else if ((G.obedit && G.obedit->type==OB_ARMATURE)) {
+		draw_object(BASACT, DRAW_PICKING|DRAW_CONSTCOLOR);
 	}
 	else {
-		base= G.scene->base.first;
-		while(base) {
+		Base *base;
+		
+		G.vd->xray= TRUE;	// otherwise it postpones drawing
+		for(base= G.scene->base.first; base; base= base->next) {
 			if(base->lay & G.vd->lay) {
 				base->selcol= code;
 				glLoadName(code);
-				draw_object(base);
+				draw_object(base, DRAW_PICKING|DRAW_CONSTCOLOR);
+				
+				/* we draw group-duplicators for selection too */
+				if((base->object->transflag & OB_DUPLI) && base->object->dup_group) {
+					ListBase *lb;
+					DupliObject *dob;
+					Base tbase;
+					
+					tbase.flag= OB_FROMDUPLI;
+					lb= object_duplilist(G.scene, base->object);
+					
+					for(dob= lb->first; dob; dob= dob->next) {
+						tbase.object= dob->ob;
+						Mat4CpyMat4(dob->ob->obmat, dob->mat);
+						
+						draw_object(&tbase, DRAW_PICKING|DRAW_CONSTCOLOR);
+						
+						Mat4CpyMat4(dob->ob->obmat, dob->omat);
+					}
+					free_object_duplilist(lb);
+				}
+				
 				code++;
 			}
-			base= base->next;
 		}
+		G.vd->xray= FALSE;	// restore
 	}
+	
 	glPopName();	/* see above (pushname) */
 	hits= glRenderMode(GL_RENDER);
 	if(hits<0) error("Too many objects in select buffer");
 
 	G.f &= ~G_PICKSEL;
-	setwinmatrixview3d(0);
+	setwinmatrixview3d(curarea->winx, curarea->winy, NULL);
 	Mat4MulMat4(G.vd->persmat, G.vd->viewmat, curarea->winmat);
 	
 	if(G.vd->drawtype > OB_WIRE) {
-		G.zbuf= 0;
+		G.vd->zbuf= 0;
 		glDisable(GL_DEPTH_TEST);
 	}
 	persp(PERSP_WIN);
+	
+	if(G.vd->flag & V3D_CLIPPING)
+		view3d_clr_clipping();
 
 	return hits;
 }
@@ -1026,29 +1230,65 @@ void initlocalview()
 		
 		G.vd->localview= 0;
 	}
+	BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT);
 }
 
 void centreview()	/* like a localview without local! */
 {
-	Base *base;
+	Object *ob= OBACT;
 	float size, min[3], max[3], afm[3];
 	int ok=0;
 
 	min[0]= min[1]= min[2]= 1.0e10;
 	max[0]= max[1]= max[2]= -1.0e10;
 
+	if (G.f & G_WEIGHTPAINT) {
+		/* hardcoded exception, we look for the one selected armature */
+		/* this is weak code this way, we should make a generic active/selection callback interface once... */
+		Base *base;
+		for(base=FIRSTBASE; base; base= base->next) {
+			if(TESTBASELIB(base)) {
+				if(base->object->type==OB_ARMATURE)
+					if(base->object->flag & OB_POSEMODE)
+						break;
+			}
+		}
+		if(base)
+			ob= base->object;
+	}
+	
+	
 	if(G.obedit) {
-		minmax_verts(min, max);
-		//minmax_object(G.obedit, min, max);
-		
+		minmax_verts(min, max);	// ony selected
 		ok= 1;
 	}
+	else if(ob && (ob->flag & OB_POSEMODE)) {
+		if(ob->pose) {
+			bArmature *arm= ob->data;
+			bPoseChannel *pchan;
+			float vec[3];
+			
+			for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+				if(pchan->bone->flag & BONE_SELECTED) {
+					if(pchan->bone->layer & arm->layer) {
+						ok= 1;
+						VECCOPY(vec, pchan->pose_head);
+						Mat4MulVecfl(ob->obmat, vec);
+						DO_MINMAX(vec, min, max);
+						VECCOPY(vec, pchan->pose_tail);
+						Mat4MulVecfl(ob->obmat, vec);
+						DO_MINMAX(vec, min, max);
+					}
+				}
+			}
+		}
+	}
 	else if (G.f & G_FACESELECT) {
 		minmax_tface(min, max);
 		ok= 1;
 	}
 	else {
-		base= FIRSTBASE;
+		Base *base= FIRSTBASE;
 		while(base) {
 			if TESTBASE(base)  {
 				minmax_object(base->object, min, max);
@@ -1089,6 +1329,7 @@ void centreview()	/* like a localview without local! */
 	G.vd->cursor[2]= -G.vd->ofs[2];
 
 	scrarea_queue_winredraw(curarea);
+	BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT);
 
 }
 
@@ -1138,7 +1379,10 @@ void endlocalview(ScrArea *sa)
 			if( base->lay & locallay ) {
 				base->lay-= locallay;
 				if(base->lay==0) base->lay= v3d->layact;
-				if(base->object != G.obedit) base->flag |= SELECT;
+				if(base->object != G.obedit) {
+					base->flag |= SELECT;
+					base->object->flag |= SELECT;
+				}
 				base->object->lay= base->lay;
 			}
 			base= base->next;
@@ -1146,7 +1390,8 @@ void endlocalview(ScrArea *sa)
 
 		countall();
 		allqueue(REDRAWVIEW3D, 0);	/* because of select */
-		
+		allqueue(REDRAWOOPS, 0);	/* because of select */
+		BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT);
 	}
 }
 
@@ -1201,6 +1446,8 @@ void view3d_home(int centre)
 		
 		scrarea_queue_winredraw(curarea);
 	}
+	BIF_view3d_previewrender_signal(curarea, PR_DBASE|PR_DISPRECT);
+
 }
 
 
