@@ -71,6 +71,8 @@ static int createDir(char* name)
 extern "C" { extern char bprogname[]; }
 
 // add drive character if not in path string, using blender executable location as reference
+// later added note: doesn't make much sense actually,
+// the intended file might not be located on the same drive as where blender is located...
 static void addDrive(string &path)
 {
 	int sp = path.find_first_of(":");
@@ -112,6 +114,24 @@ static string unixYafrayPath()
 
 #endif
 
+#ifdef WIN32
+#define MAXPATHLEN MAX_PATH
+#else
+#include <sys/param.h>
+#endif
+static void adjustPath(string &path)
+{
+	// if relative, expand to full path
+	char cpath[MAXPATHLEN];
+	strcpy(cpath, path.c_str());
+	BLI_convertstringcode(cpath, G.sce, 0);
+	path = cpath;
+#ifdef WIN32
+	// add drive char if not there
+	addDrive(path);
+#endif
+}
+
 bool yafrayFileRender_t::initExport()
 {
 	xmlpath = "";
@@ -128,22 +148,20 @@ bool yafrayFileRender_t::initExport()
 	}
 	else 
 	{
+		xmlpath = U.yfexportdir;
+		adjustPath(xmlpath);	// possibly relative
+		cout << "YFexport path is: " << xmlpath << endl;
 		// check if it exists
-		if (!BLI_exists(U.yfexportdir)) {
+		if (!BLI_exists(const_cast<char*>(xmlpath.c_str()))) {
 			cout << "YafRay temporary xml export directory:\n" << U.yfexportdir << "\ndoes not exist!\n";
 #ifdef WIN32
 			// try to create it
 			cout << "Trying to create...\n";
-			if (createDir(U.yfexportdir)==0) dir_failed=true; else dir_failed=false;
+			if (createDir(const_cast<char*>(xmlpath.c_str()))==0) dir_failed=true; else dir_failed=false;
 #else
 			dir_failed = true;
 #endif
 		}
-		xmlpath = U.yfexportdir;
-#ifdef WIN32
-		// have to add drive char here too, in case win user still wants to set path him/herself
-		addDrive(xmlpath);
-#endif
 	}
 
 #ifdef WIN32
@@ -206,29 +224,29 @@ bool yafrayFileRender_t::writeRender()
 	// finally export render block
 	ostr.str("");
 	ostr << "<render camera_name=\"MAINCAM\"\n";
-	ostr << "\traydepth=\"" << R.r.YF_raydepth << "\" gamma=\"" << R.r.YF_gamma << "\" exposure=\"" << R.r.YF_exposure << "\"\n";
+	ostr << "\traydepth=\"" << re->r.YF_raydepth << "\" gamma=\"" << re->r.YF_gamma << "\" exposure=\"" << re->r.YF_exposure << "\"\n";
 
-	if(R.r.YF_AA) {
-		ostr << "\tAA_passes=\"" << R.r.YF_AApasses << "\" AA_minsamples=\"" << R.r.YF_AAsamples << "\"\n";
-		ostr << "\tAA_pixelwidth=\"" << R.r.YF_AApixelsize << "\" AA_threshold=\"" << R.r.YF_AAthreshold << "\"\n";
+	if(re->r.YF_AA) {
+		ostr << "\tAA_passes=\"" << re->r.YF_AApasses << "\" AA_minsamples=\"" << re->r.YF_AAsamples << "\"\n";
+		ostr << "\tAA_pixelwidth=\"" << re->r.YF_AApixelsize << "\" AA_threshold=\"" << re->r.YF_AAthreshold << "\"\n";
 	}
 	else {
 		// removed the default AA settings for midquality GI, better leave it to user
-		if ((R.r.mode & R_OSA) && (R.r.osa)) {
-			int passes=(R.r.osa%4)==0 ? R.r.osa/4 : 1;
-			int minsamples=(R.r.osa%4)==0 ? 4 : R.r.osa;
+		if ((re->r.mode & R_OSA) && (re->r.osa)) {
+			int passes = (re->r.osa & 3)==0 ? (re->r.osa >> 2) : 1;
+			int minsamples=(re->r.osa & 3)==0 ? 4 : re->r.osa;
 			ostr << "\tAA_passes=\"" << passes << "\" AA_minsamples=\"" << minsamples << "\"\n";
 		}
 		else ostr << "\tAA_passes=\"0\" AA_minsamples=\"1\"\n";
-		ostr << "\tAA_pixelwidth=\"1.5\" AA_threshold=\"0.05\" bias=\"" << R.r.YF_raybias
-				 << "\" clamp_rgb=\"" << ((R.r.YF_clamprgb==0) ? "on" : "off") << "\"\n";
+		ostr << "\tAA_pixelwidth=\"1.5\" AA_threshold=\"0.05\" bias=\"" << re->r.YF_raybias
+				 << "\" clamp_rgb=\"" << ((re->r.YF_clamprgb==0) ? "on" : "off") << "\"\n";
 	}
 
 	World *world = G.scene->world;
 	if (world) ostr << "\tbackground_name=\"world_background\"\n";
  
 	// alpha channel render when RGBA button enabled
-	if (R.r.planes==R_PLANES32) ostr << "\n\tsave_alpha=\"on\"";
+	if (re->r.planes==R_PLANES32) ostr << "\n\tsave_alpha=\"on\"";
 	ostr << " >\n";
 
 	// basic fog
@@ -279,7 +297,9 @@ void yafrayFileRender_t::displayImage()
 	unsigned short width = (unsigned short)(header[12] + (header[13]<<8));
 	unsigned short height = (unsigned short)(header[14] + (header[15]<<8));
 	// don't do anything if resolution doesn't match that of rectot
-	if ((width!=R.rectx) || (height!=R.recty)) {
+	if ((width!=re->winx) || (height!=re->winy)) {
+		cout << "Wrong image width/height: " << width << "/" << height <<
+			" expected " << re->winx << "/" << re->winy << endl;
 		fclose(fp);
 		fp = NULL;
 		return;
@@ -289,45 +309,59 @@ void yafrayFileRender_t::displayImage()
 	unsigned int idlen = (unsigned int)header[0];
 	if (idlen) fseek(fp, idlen, SEEK_CUR);
 
+	/* XXX how to get the image from Blender and write to it. This call doesn't allow to change buffer rects */
+	RenderResult rres;
+	RE_GetResultImage(re, &rres);
+	// rres.rectx, rres.recty is width/height
+	// rres.rectf is float buffer, scanlines starting in bottom
+	// rres.rectz is zbuffer, available when associated pass is set
+	
 	// read data directly into buffer, picture is upside down
-	for (unsigned short y=0;y<height;y++) {
-		unsigned char* bpt = (unsigned char*)R.rectot + ((((height-1)-y)*width)<<2);
-		for (unsigned short x=0;x<width;x++) {
-			bpt[2] = (unsigned char)fgetc(fp);
-			bpt[1] = (unsigned char)fgetc(fp);
-			bpt[0] = (unsigned char)fgetc(fp);
-			if (byte_per_pix==4)
-				bpt[3] = (unsigned char)fgetc(fp);
-			else
-				bpt[3] = 255;
-			bpt += 4;
+	const float btf = 1.f/255.f;
+	if (re->r.mode & R_BORDER) {
+		// border render, yafray is full size render, blender at this point only wants the region
+		unsigned int xs = (unsigned int)(re->r.border.xmin * re->winx),
+		             ys = (unsigned int)((1.f-re->r.border.ymax) * re->winy);
+		for (unsigned short y=0; y<height; y++) {
+			for (unsigned short x=0; x<width; x++) {
+				char r = fgetc(fp);
+				char g = fgetc(fp);
+				char b = fgetc(fp);
+				char a = (byte_per_pix==4) ? fgetc(fp) : 255;
+				int bx = x-xs, by = y-ys;
+				if ((bx >= 0) && (bx < (int)re->rectx) && (by >= 0) && (by < re->recty)) {
+					float* bpt = (float*)rres.rectf + (bx + (((re->recty-1) - by)*re->rectx) << 2);
+					bpt[2] = (float)r * btf;
+					bpt[1] = (float)g * btf;
+					bpt[0] = (float)b * btf;
+					bpt[3] = (float)a * btf;
+					bpt += 4;
+				}
+			}
+		}
+	}
+	else {
+		for (unsigned short y=0; y<height; y++) {
+			float* bpt = (float*)rres.rectf + ((((height-1)-y)*width) << 2);
+			for (unsigned short x=0; x<width; x++) {
+				bpt[2] = ((float)fgetc(fp) * btf);
+				bpt[1] = ((float)fgetc(fp) * btf);
+				bpt[0] = ((float)fgetc(fp) * btf);
+				bpt[3] = (byte_per_pix==4) ? ((float)fgetc(fp) * btf) : 1.f;
+				bpt += 4;
+			}
 		}
 	}
 
 	fclose(fp);
 	fp = NULL;
+
+	// based on another assumption of ogl errors on my system, forgot to actually draw it...
+	re->result->renlay = render_get_active_layer(re, re->result);
+	re->display_draw(re->result, NULL);
+
 }
 
-
-#ifdef WIN32
-#define MAXPATHLEN MAX_PATH
-#else
-#include <sys/param.h>
-#endif
-static void adjustPath(string &path)
-{
-	// if relative, expand to full path
-	char cpath[MAXPATHLEN];
-	strcpy(cpath, path.c_str());
-	BLI_convertstringcode(cpath, G.sce, 0);
-	path = cpath;
-#ifdef WIN32
-	// add drive char if not there
-	addDrive(path);
-#endif
-}
-
-
 static string noise2string(short nbtype)
 {
 	switch (nbtype) {
@@ -680,7 +714,7 @@ void yafrayFileRender_t::writeShader(const string &shader_name, Material* matr,
 	ostr << "\t\t<specular_amount value=\"" << matr->spec << "\" />\n";
 	ostr << "\t\t<alpha value=\"" << matr->alpha << "\" />\n";
 	// if no GI used, the GIpower parameter is not always initialized, so in that case ignore it
-	float bg_mult = (R.r.GImethod==0) ? 1 : R.r.GIpower;
+	float bg_mult = (re->r.GImethod==0) ? 1 : re->r.GIpower;
 	ostr << "\t\t<emit value=\"" << (matr->emit * bg_mult) << "\" />\n";
 
 	// reflection/refraction
@@ -1040,7 +1074,8 @@ void yafrayFileRender_t::writeMaterialsAndModulators()
 				else if ((mtex->texco & TEXCO_GLOB) || (mtex->texco & TEXCO_OBJECT))
 					// object mode is also set as global, but the object matrix was specified above with <modulator..>
 					ostr << "\t\t<texco value=\"global\" />\n";
-				else if (mtex->texco & TEXCO_ORCO)
+				else if ((mtex->texco & TEXCO_ORCO) || (mtex->texco & TEXCO_STRAND))
+					// orco flag now used for 'strand'-mapping as well, see mesh code
 					ostr << "\t\t<texco value=\"orco\" />\n";
 				else if (mtex->texco & TEXCO_WINDOW)
 					ostr << "\t\t<texco value=\"window\" />\n";
@@ -1168,7 +1203,25 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 	xmlfile << ostr.str();
 
 	ostr.str("");
-	ostr << "<object name=\"" << obj->id.name << "\"";
+
+	// using the ObjectRen database, contruct a new name if object has a parent.
+	// This is done to prevent name clashes (group/library link related)
+	string obname(obj->id.name);
+	// previous implementation, keep around, in case this is still useful
+	//if (obj->id.flag & (LIB_EXTERN|LIB_INDIRECT))obname = "lib_" + obname;
+	ObjectRen *obren;
+	for (obren = static_cast<ObjectRen*>(re->objecttable.first);
+	     obren; obren=static_cast<ObjectRen*>(obren->next))
+	{
+		Object *db_ob = obren->ob, *db_par = obren->par;
+		if (db_ob==obj)
+			if ((db_ob!=NULL) && (db_par!=NULL)) {
+				obname += "_" + string(db_par->id.name);
+				break;
+			}
+	}
+
+	ostr << "<object name=\"" << obname << "\"";
 	// Yafray still needs default shader name in object def.,
 	// since we write a shader with every face, simply use the material of the first face.
 	// If this is an empty string, assume default material.
@@ -1207,10 +1260,15 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 	// Export orco coords test.
 	// Previously was done by checking orco pointer, however this can be non-null but still not initialized.
 	// Test the rendermaterial texco flag instead.
-	bool EXPORT_ORCO = ((face0mat->texco & TEXCO_ORCO)!=0);
+	// update2: bug #3193 it seems it has changed again with the introduction of static 'hair' particles,
+	// now it uses the vert pointer again as an extra test to make sure there are orco coords available
+	int has_orco = 0;
+	if (face0mat->texco & TEXCO_STRAND)
+		has_orco = 1;
+	else
+		has_orco = (((face0mat->texco & TEXCO_ORCO)!=0) && (face0->v1->orco!=NULL)) ? 2 : 0;
 
-	string has_orco = "off";
-	if (EXPORT_ORCO) has_orco = "on";
+	string has_orco_st = has_orco ? "on" : "off";
 
 	// smooth shading if enabled
 	bool no_auto = true;	//in case non-mesh, or mesh has no autosmooth
@@ -1219,7 +1277,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 		if (mesh->flag & ME_AUTOSMOOTH) {
 			no_auto = false;
 			ostr.str("");
-			ostr << "\t<mesh autosmooth=\"" << mesh->smoothresh << "\" has_orco=\"" << has_orco << "\" >\n";
+			ostr << "\t<mesh autosmooth=\"" << mesh->smoothresh << "\" has_orco=\"" << has_orco_st << "\" >\n";
 			xmlfile << ostr.str();
 		}
 	}
@@ -1229,9 +1287,9 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 		// or flat shaded, the smooth flag of the first face is used to determine
 		// the shading for the whole mesh
 		if (face0->flag & ME_SMOOTH)
-			xmlfile << "\t<mesh autosmooth=\"180\" has_orco=\"" << has_orco << "\" >\n";
+			xmlfile << "\t<mesh autosmooth=\"180\" has_orco=\"" << has_orco_st << "\" >\n";
 		else
-			xmlfile << "\t<mesh autosmooth=\"0.1\" has_orco=\"" << has_orco << "\" >\n";	//0 shows artefacts
+			xmlfile << "\t<mesh autosmooth=\"0.1\" has_orco=\"" << has_orco_st << "\" >\n";	//0 shows artefacts
 	}
 
 	// now all vertices
@@ -1244,7 +1302,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 	// for deformed objects, object->imat is no longer valid,
 	// so have to create inverse render matrix ourselves here
 	float mat[4][4], imat[4][4];
-	MTC_Mat4MulMat4(mat, obj->obmat, R.viewmat);
+	MTC_Mat4MulMat4(mat, obj->obmat, re->viewmat);
 	MTC_Mat4Invert(imat, mat);
 
 	for (vector<VlakRen*>::const_iterator fci=VLR_list.begin();
@@ -1261,13 +1319,19 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			MTC_cp3Float(ver->co, tvec);
 			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
-								 << "\" y=\"" << tvec[1]
-								 << "\" z=\"" << tvec[2] << "\" />\n";
-			if (EXPORT_ORCO) {
+			           << "\" y=\"" << tvec[1]
+			           << "\" z=\"" << tvec[2] << "\" />\n";
+			// has_orco now an int, if 1 -> strand mapping, if 2 -> normal orco mapping
+			if (has_orco==1) {
+				ostr << "\t\t\t<p x=\"" << ver->accum
+				     << "\" y=\"" << ver->accum
+				     << "\" z=\"" << ver->accum << "\" />\n";
+			}
+			else if (has_orco==2) {
 				orco = ver->orco;
 				ostr << "\t\t\t<p x=\"" << orco[0]
-									 << "\" y=\"" << orco[1]
-									 << "\" z=\"" << orco[2] << "\" />\n";
+				     << "\" y=\"" << orco[1]
+				     << "\" z=\"" << orco[2] << "\" />\n";
 			}
 		}
 		if (vert_idx.find(vlr->v2)==vert_idx.end()) {
@@ -1276,13 +1340,19 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			MTC_cp3Float(ver->co, tvec);
 			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
-								 << "\" y=\"" << tvec[1]
-								 << "\" z=\"" << tvec[2] << "\" />\n";
-			if (EXPORT_ORCO) {
+			           << "\" y=\"" << tvec[1]
+			           << "\" z=\"" << tvec[2] << "\" />\n";
+			// has_orco now an int, if 1 -> strand mapping, if 2 -> normal orco mapping
+			if (has_orco==1) {
+				ostr << "\t\t\t<p x=\"" << ver->accum
+				     << "\" y=\"" << ver->accum
+				     << "\" z=\"" << ver->accum << "\" />\n";
+			}
+			else if (has_orco==2) {
 				orco = ver->orco;
 				ostr << "\t\t\t<p x=\"" << orco[0]
-									 << "\" y=\"" << orco[1]
-									 << "\" z=\"" << orco[2] << "\" />\n";
+				     << "\" y=\"" << orco[1]
+				     << "\" z=\"" << orco[2] << "\" />\n";
 			}
 		}
 		if (vert_idx.find(vlr->v3)==vert_idx.end()) {
@@ -1291,13 +1361,19 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			MTC_cp3Float(ver->co, tvec);
 			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
-								 << "\" y=\"" << tvec[1]
-								 << "\" z=\"" << tvec[2] << "\" />\n";
-			if (EXPORT_ORCO) {
+			           << "\" y=\"" << tvec[1]
+			           << "\" z=\"" << tvec[2] << "\" />\n";
+			// has_orco now an int, if 1 -> strand mapping, if 2 -> normal orco mapping
+			if (has_orco==1) {
+				ostr << "\t\t\t<p x=\"" << ver->accum
+				     << "\" y=\"" << ver->accum
+				     << "\" z=\"" << ver->accum << "\" />\n";
+			}
+			else if (has_orco==2) {
 				orco = ver->orco;
 				ostr << "\t\t\t<p x=\"" << orco[0]
-									 << "\" y=\"" << orco[1]
-									 << "\" z=\"" << orco[2] << "\" />\n";
+				     << "\" y=\"" << orco[1]
+				     << "\" z=\"" << orco[2] << "\" />\n";
 			}
 		}
 		if ((vlr->v4) && (vert_idx.find(vlr->v4)==vert_idx.end())) {
@@ -1306,13 +1382,19 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			MTC_cp3Float(ver->co, tvec);
 			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
-								 << "\" y=\"" << tvec[1]
-								 << "\" z=\"" << tvec[2] << "\" />\n";
-			if (EXPORT_ORCO) {
+			           << "\" y=\"" << tvec[1]
+			           << "\" z=\"" << tvec[2] << "\" />\n";
+			// has_orco now an int, if 1 -> strand mapping, if 2 -> normal orco mapping
+			if (has_orco==1) {
+				ostr << "\t\t\t<p x=\"" << ver->accum
+				     << "\" y=\"" << ver->accum
+				     << "\" z=\"" << ver->accum << "\" />\n";
+			}
+			else if (has_orco==2) {
 				orco = ver->orco;
 				ostr << "\t\t\t<p x=\"" << orco[0]
-									 << "\" y=\"" << orco[1]
-									 << "\" z=\"" << orco[2] << "\" />\n";
+				     << "\" y=\"" << orco[1]
+				     << "\" z=\"" << orco[2] << "\" />\n";
 			}
 		}
 		xmlfile << ostr.str();
@@ -1342,7 +1424,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 		int idx2 = vert_idx.find(vlr->v2)->second;
 		int idx3 = vert_idx.find(vlr->v3)->second;
 		// make sure the indices point to the vertices when orco coords exported
-		if (EXPORT_ORCO) { idx1*=2;  idx2*=2;  idx3*=2; }
+		if (has_orco) { idx1*=2;  idx2*=2;  idx3*=2; }
 
 		ostr.str("");
 		ostr << "\t\t\t<f a=\"" << idx1 << "\" b=\"" << idx2 << "\" c=\"" << idx3 << "\"";
@@ -1358,8 +1440,8 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 		TFace* uvc = vlr->tface;	// possible uvcoords (v upside down)
 		if (uvc) {
 			ostr << " u_a=\"" << uvc->uv[ui1][0] << "\" v_a=\"" << 1-uvc->uv[ui1][1] << "\""
-					 << " u_b=\"" << uvc->uv[ui2][0] << "\" v_b=\"" << 1-uvc->uv[ui2][1] << "\""
-					 << " u_c=\"" << uvc->uv[ui3][0] << "\" v_c=\"" << 1-uvc->uv[ui3][1] << "\"";
+			     << " u_b=\"" << uvc->uv[ui2][0] << "\" v_b=\"" << 1-uvc->uv[ui2][1] << "\""
+			     << " u_c=\"" << uvc->uv[ui3][0] << "\" v_c=\"" << 1-uvc->uv[ui3][1] << "\"";
 		}
 
 		// since Blender seems to need vcols when uvs are used, for yafray only export when the material actually uses vcols
@@ -1367,13 +1449,13 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			// vertex colors
 			unsigned char* pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui1]);
 			ostr << " vcol_a_r=\"" << (float)pt[3]/255.f << "\" vcol_a_g=\"" << (float)pt[2]/255.f
-					 << "\" vcol_a_b=\"" << (float)pt[1]/255.f << "\"";
+			     << "\" vcol_a_b=\"" << (float)pt[1]/255.f << "\"";
 			pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui2]);
 			ostr << " vcol_b_r=\"" << (float)pt[3]/255.f << "\" vcol_b_g=\"" << (float)pt[2]/255.f
-					 << "\" vcol_b_b=\"" << (float)pt[1]/255.f << "\"";
+			     << "\" vcol_b_b=\"" << (float)pt[1]/255.f << "\"";
 			pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui3]);
 			ostr << " vcol_c_r=\"" << (float)pt[3]/255.f << "\" vcol_c_g=\"" << (float)pt[2]/255.f
-					 << "\" vcol_c_b=\"" << (float)pt[1]/255.f << "\"";
+			     << "\" vcol_c_b=\"" << (float)pt[1]/255.f << "\"";
 		}
 		ostr << " shader_name=\"" << fmatname << "\" />\n";
 
@@ -1384,7 +1466,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			idx3 = vert_idx.find(vlr->v1)->second;
 
 			// make sure the indices point to the vertices when orco coords exported
-			if (EXPORT_ORCO) { idx1*=2;  idx2*=2;  idx3*=2; }
+			if (has_orco) { idx1*=2;  idx2*=2;  idx3*=2; }
 
 			ostr << "\t\t\t<f a=\"" << idx1 << "\" b=\"" << idx2 << "\" c=\"" << idx3 << "\"";
 
@@ -1395,20 +1477,20 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 
 			if (uvc) {
 				ostr << " u_a=\"" << uvc->uv[ui1][0] << "\" v_a=\"" << 1-uvc->uv[ui1][1] << "\""
-						 << " u_b=\"" << uvc->uv[ui2][0] << "\" v_b=\"" << 1-uvc->uv[ui2][1] << "\""
-						 << " u_c=\"" << uvc->uv[ui3][0] << "\" v_c=\"" << 1-uvc->uv[ui3][1] << "\"";
+				     << " u_b=\"" << uvc->uv[ui2][0] << "\" v_b=\"" << 1-uvc->uv[ui2][1] << "\""
+				     << " u_c=\"" << uvc->uv[ui3][0] << "\" v_c=\"" << 1-uvc->uv[ui3][1] << "\"";
 			}
 			if ((EXPORT_VCOL) && (vlr->vcol)) {
 				// vertex colors
 				unsigned char* pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui1]);
 				ostr << " vcol_a_r=\"" << (float)pt[3]/255.f << "\" vcol_a_g=\"" << (float)pt[2]/255.f
-						 << "\" vcol_a_b=\"" << (float)pt[1]/255.f << "\"";
+				     << "\" vcol_a_b=\"" << (float)pt[1]/255.f << "\"";
 				pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui2]);
 				ostr << " vcol_b_r=\"" << (float)pt[3]/255.f << "\" vcol_b_g=\"" << (float)pt[2]/255.f
-						 << "\" vcol_b_b=\"" << (float)pt[1]/255.f << "\"";
+				     << "\" vcol_b_b=\"" << (float)pt[1]/255.f << "\"";
 				pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui3]);
 				ostr << " vcol_c_r=\"" << (float)pt[3]/255.f << "\" vcol_c_g=\"" << (float)pt[2]/255.f
-						 << "\" vcol_c_b=\"" << (float)pt[1]/255.f << "\"";
+				     << "\" vcol_c_b=\"" << (float)pt[1]/255.f << "\"";
 			}
 			ostr << " shader_name=\"" << fmatname << "\" />\n";
 
@@ -1495,7 +1577,7 @@ void yafrayFileRender_t::writeAreaLamp(LampRen* lamp, int num, float iview[4][4]
 	ostr.str("");
 	string md = "off";
 	// if no GI used, the GIphotons flag can still be set, so only use when 'full' selected
-	if ((R.r.GImethod==2) && (R.r.GIphotons)) { md="on";  power*=R.r.GIpower; }
+	if ((re->r.GImethod==2) && (re->r.GIphotons)) { md="on";  power*=re->r.GIpower; }
 	ostr << "<light type=\"arealight\" name=\"LAMP" << num+1 << "\" dummy=\""<< md << "\" power=\"" << power << "\" ";
 	// samples not used for GI with photons, can still be exported, is ignored
 	int psm=0, sm = lamp->ray_totsamp;
@@ -1525,17 +1607,20 @@ void yafrayFileRender_t::writeAreaLamp(LampRen* lamp, int num, float iview[4][4]
 
 void yafrayFileRender_t::writeLamps()
 {
+	GroupObject *go;
+	int i=0;
+	
 	// inverse viewmatrix needed for back2world transform
 	float iview[4][4];
-	// R.viewinv != inv.R.viewmat because of possible ortho mode (see convertBlenderScene.c)
+	// re->viewinv != inv.re->viewmat because of possible ortho mode (see convertBlenderScene.c)
 	// have to invert it here
-	MTC_Mat4Invert(iview, R.viewmat);
+	MTC_Mat4Invert(iview, re->viewmat);
 	
 	// all lamps
-	for (int i=0;i<R.totlamp;i++)
-	{
+	for(go=(GroupObject *)re->lights.first; go; go= go->next, i++) {
+		LampRen* lamp = (LampRen *)go->lampren;
+
 		ostr.str("");
-		LampRen* lamp = R.la[i];
 		
 		if (lamp->type==LA_AREA) { writeAreaLamp(lamp, i, iview);  continue; }
 		
@@ -1589,7 +1674,7 @@ void yafrayFileRender_t::writeLamps()
 			// 'dummy' mode for spherelight when used with gpm
 			string md = "off";
 			// if no GI used, the GIphotons flag can still be set, so only use when 'full' selected
-			if ((R.r.GImethod==2) && (R.r.GIphotons)) { md="on";  pwr*=R.r.GIpower; }
+			if ((re->r.GImethod==2) && (re->r.GIphotons)) { md="on";  pwr*=re->r.GIpower; }
 			ostr << "\" power=\"" <<  pwr << "\" dummy=\"" << md << "\"";
 		}
 		else ostr << "\" power=\"" << pwr << "\"";
@@ -1597,12 +1682,14 @@ void yafrayFileRender_t::writeLamps()
 		// cast_shadows flag not used with softlight, spherelight or photonlight
 		if ((!is_softL) && (!is_sphereL) && (lamp->type!=LA_YF_PHOTON)) {
 			string lpmode="off";
-			// Shadows only when Blender has shadow button enabled, only spots use LA_SHAD flag.
-			// Also blender hemilights exported as sunlights which might have shadow flag set
+			// Blender hemilights exported as sunlights which might have shadow flag set
 			// should have cast_shadows set to off (reported by varuag)
 			if (lamp->type!=LA_HEMI) {
-				if (R.r.mode & R_SHADOW)
-					if (((lamp->type==LA_SPOT) && (lamp->mode & LA_SHAD)) || (lamp->mode & LA_SHAD_RAY)) lpmode="on";
+				if (re->r.mode & R_SHADOW) {
+					// old bug was here since the yafray lamp settings panel was added,
+					// blender spotlight shadbuf flag should be ignored, since it is not in the panel anymore
+					if (lamp->mode & LA_SHAD_RAY) lpmode="on";
+				}
 			}
 			ostr << " cast_shadows=\"" << lpmode << "\"";
 		}
@@ -1691,24 +1778,30 @@ void yafrayFileRender_t::writeCamera()
 	// here Global used again
 	ostr.str("");
 	ostr << "<camera name=\"MAINCAM\" ";
-	if (R.r.mode & R_ORTHO)
+	if (re->r.mode & R_ORTHO)
 		ostr << "type=\"ortho\"";
 	else	
 		ostr << "type=\"perspective\"";
 
-	// render resolution including the percentage buttons (aleady calculated in initrender for R renderdata)
-	ostr << " resx=\"" << R.r.xsch << "\" resy=\"" << R.r.ysch << "\"";
+	// render resolution including the percentage buttons
+	ostr << " resx=\"" << re->winx << "\" resy=\"" << re->winy << "\"";
 
 	float f_aspect = 1;
-	if ((R.r.xsch*R.r.xasp)<=(R.r.ysch*R.r.yasp)) f_aspect = float(R.r.xsch*R.r.xasp)/float(R.r.ysch*R.r.yasp);
+	if ((re->winx * re->r.xasp) <= (re->winy * re->r.yasp))
+		f_aspect = float(re->winx * re->r.xasp) / float(re->winy * re->r.yasp);
 	ostr << "\n\tfocal=\"" << mainCamLens/(f_aspect*32.f);
-	ostr << "\" aspect_ratio=\"" << R.ycor << "\"";
+	// bug #4532, when field rendering is enabled, ycor is doubled
+	if (re->r.mode & R_FIELDS)
+		ostr << "\" aspect_ratio=\"" << (re->ycor * 0.5f) << "\"";
+	else
+		ostr << "\" aspect_ratio=\"" << re->ycor << "\"";
+
 
 	// dof params, only valid for real camera
 	float fdist = 1;	// only changes for ortho
 	if (maincam_obj->type==OB_CAMERA) {
 		Camera* cam = (Camera*)maincam_obj->data;
-		if (R.r.mode & R_ORTHO) fdist = cam->ortho_scale*(mainCamLens/32.f);
+		if (re->r.mode & R_ORTHO) fdist = cam->ortho_scale*(mainCamLens/32.f);
 		ostr << "\n\tdof_distance=\"" << cam->YF_dofdist << "\"";
 		ostr << " aperture=\"" << cam->YF_aperture << "\"";
 		string st = "on";
@@ -1745,12 +1838,12 @@ void yafrayFileRender_t::writeCamera()
 	ostr << "\t<from x=\"" << maincam_obj->obmat[3][0] << "\""
 							<< " y=\"" << maincam_obj->obmat[3][1] << "\""
 							<< " z=\"" << maincam_obj->obmat[3][2] << "\" />\n";
-	ostr << "\t<to x=\"" << maincam_obj->obmat[3][0] - fdist * R.viewmat[0][2]
-					<< "\" y=\"" << maincam_obj->obmat[3][1] - fdist * R.viewmat[1][2]
-					<< "\" z=\"" << maincam_obj->obmat[3][2] - fdist * R.viewmat[2][2] << "\" />\n";
-	ostr << "\t<up x=\"" << maincam_obj->obmat[3][0] + R.viewmat[0][1]
-					<< "\" y=\"" << maincam_obj->obmat[3][1] + R.viewmat[1][1]
-					<< "\" z=\"" << maincam_obj->obmat[3][2] + R.viewmat[2][1] << "\" />\n";
+	ostr << "\t<to x=\"" << maincam_obj->obmat[3][0] - fdist * re->viewmat[0][2]
+					<< "\" y=\"" << maincam_obj->obmat[3][1] - fdist * re->viewmat[1][2]
+					<< "\" z=\"" << maincam_obj->obmat[3][2] - fdist * re->viewmat[2][2] << "\" />\n";
+	ostr << "\t<up x=\"" << maincam_obj->obmat[3][0] + re->viewmat[0][1]
+					<< "\" y=\"" << maincam_obj->obmat[3][1] + re->viewmat[1][1]
+					<< "\" z=\"" << maincam_obj->obmat[3][2] + re->viewmat[2][1] << "\" />\n";
 	xmlfile << ostr.str();
 
 	xmlfile << "</camera>\n\n";
@@ -1760,7 +1853,7 @@ void yafrayFileRender_t::writeHemilight()
 {
 	World *world = G.scene->world;
 	bool fromAO = false;
-	if (R.r.GIquality==6){
+	if (re->r.GIquality==6){
 		// use Blender AO params is possible
 		if (world==NULL) return;
 		if ((world->mode & WO_AMB_OCC)==0) {
@@ -1770,18 +1863,18 @@ void yafrayFileRender_t::writeHemilight()
 		else fromAO = true;
 	}
 	ostr.str("");
-	if (R.r.GIcache) {
-		ostr << "<light type=\"pathlight\" name=\"path_LT\" power=\"" << R.r.GIpower << "\" mode=\"occlusion\"";
-		ostr << "\n\tcache=\"on\" use_QMC=\"on\" threshold=\"" << R.r.GIrefinement << "\" "
-				 << "cache_size=\"" << ((2.0/float(R.r.xsch))*R.r.GIpixelspersample) << "\"";
-		ostr << "\n\tshadow_threshold=\"" << (1.0-R.r.GIshadowquality) << "\" grid=\"82\" search=\"35\"";
-		ostr << "\n\tignore_bumpnormals=\"" << (R.r.YF_nobump ? "on" : "off") << "\"";
+	if (re->r.GIcache) {
+		ostr << "<light type=\"pathlight\" name=\"path_LT\" power=\"" << re->r.GIpower << "\" mode=\"occlusion\"";
+		ostr << "\n\tcache=\"on\" use_QMC=\"on\" threshold=\"" << re->r.GIrefinement << "\" "
+				 << "cache_size=\"" << ((2.0/float(re->winx))*re->r.GIpixelspersample) << "\"";
+		ostr << "\n\tshadow_threshold=\"" << (1.0-re->r.GIshadowquality) << "\" grid=\"82\" search=\"35\"";
+		ostr << "\n\tignore_bumpnormals=\"" << (re->r.YF_nobump ? "on" : "off") << "\"";
 		if (fromAO) {
 			// for AO, with cache, using range of 32*1 to 32*16 seems good enough
 			ostr << "\n\tsamples=\"" << 32*world->aosamp << "\" maxdistance=\"" << world->aodist << "\" >\n";
 		}
 		else {
-			switch (R.r.GIquality)
+			switch (re->r.GIquality)
 			{
 				case 1 : ostr << " samples=\"128\" >\n";  break;
 				case 2 : ostr << " samples=\"256\" >\n";  break;
@@ -1793,7 +1886,7 @@ void yafrayFileRender_t::writeHemilight()
 		}
 	}
 	else {
-		ostr << "<light type=\"hemilight\" name=\"hemi_LT\" power=\"" << R.r.GIpower << "\"";
+		ostr << "<light type=\"hemilight\" name=\"hemi_LT\" power=\"" << re->r.GIpower << "\"";
 		if (fromAO) {
 			// use minimum of 4 samples for lowest sample setting, single sample way too noisy
 			ostr << "\n\tsamples=\"" << 3 + world->aosamp*world->aosamp
@@ -1801,7 +1894,7 @@ void yafrayFileRender_t::writeHemilight()
 					 << "\" use_QMC=\"" << ((world->aomode & WO_AORNDSMP) ? "off" : "on") << "\" >\n";
 		}
 		else {
-			switch (R.r.GIquality)
+			switch (re->r.GIquality)
 			{
 				case 1 :
 				case 2 : ostr << " samples=\"16\" >\n";  break;
@@ -1819,19 +1912,19 @@ void yafrayFileRender_t::writeHemilight()
 void yafrayFileRender_t::writePathlight()
 {
 	ostr.str("");
-	if (R.r.GIphotons)
+	if (re->r.GIphotons)
 	{
-		ostr << "<light type=\"globalphotonlight\" name=\"gpm\" photons=\"" << R.r.GIphotoncount << "\"" << endl;
-		ostr << "\tradius=\"" << R.r.GIphotonradius << "\" depth=\"" << ((R.r.GIdepth>2) ? (R.r.GIdepth-1) : 1)
-				 << "\" caus_depth=\"" << R.r.GIcausdepth << "\" search=\"" << R.r.GImixphotons << "\" >"<<endl;
+		ostr << "<light type=\"globalphotonlight\" name=\"gpm\" photons=\"" << re->r.GIphotoncount << "\"" << endl;
+		ostr << "\tradius=\"" << re->r.GIphotonradius << "\" depth=\"" << ((re->r.GIdepth>2) ? (re->r.GIdepth-1) : 1)
+				 << "\" caus_depth=\"" << re->r.GIcausdepth << "\" search=\"" << re->r.GImixphotons << "\" >"<<endl;
 		ostr << "</light>"<<endl;
 	}
-	ostr << "<light type=\"pathlight\" name=\"path_LT\" power=\"" << R.r.GIindirpower << "\"";
-	ostr << " depth=\"" << ((R.r.GIphotons) ? 1 : R.r.GIdepth) << "\" caus_depth=\"" << R.r.GIcausdepth <<"\"\n";
-	if (R.r.GIdirect && R.r.GIphotons) ostr << "direct=\"on\"" << endl;
-	if (R.r.GIcache && !(R.r.GIdirect && R.r.GIphotons))
+	ostr << "<light type=\"pathlight\" name=\"path_LT\" power=\"" << re->r.GIindirpower << "\"";
+	ostr << " depth=\"" << ((re->r.GIphotons) ? 1 : re->r.GIdepth) << "\" caus_depth=\"" << re->r.GIcausdepth <<"\"\n";
+	if (re->r.GIdirect && re->r.GIphotons) ostr << "direct=\"on\"" << endl;
+	if (re->r.GIcache && !(re->r.GIdirect && re->r.GIphotons))
 	{
-		switch (R.r.GIquality)
+		switch (re->r.GIquality)
 		{
 			case 1 : ostr << " samples=\"128\" \n";   break;
 			case 2 : ostr << " samples=\"256\" \n";   break;
@@ -1840,15 +1933,15 @@ void yafrayFileRender_t::writePathlight()
 			case 5 : ostr << " samples=\"2048\" \n";  break;
 			default: ostr << " samples=\"512\" \n";
 		}
-		ostr << " cache=\"on\" use_QMC=\"on\" threshold=\"" << R.r.GIrefinement << "\"" << endl;
-		ostr << "\tignore_bumpnormals=\"" << (R.r.YF_nobump ? "on" : "off") << "\"\n";
-		float sbase = 2.0/float(R.r.xsch);
-		ostr << "\tcache_size=\"" << sbase*R.r.GIpixelspersample << "\" shadow_threshold=\"" <<
-			1.0-R.r.GIshadowquality << "\" grid=\"82\" search=\"35\" >\n";
+		ostr << " cache=\"on\" use_QMC=\"on\" threshold=\"" << re->r.GIrefinement << "\"" << endl;
+		ostr << "\tignore_bumpnormals=\"" << (re->r.YF_nobump ? "on" : "off") << "\"\n";
+		float sbase = 2.0/float(re->winx);
+		ostr << "\tcache_size=\"" << sbase*re->r.GIpixelspersample << "\" shadow_threshold=\"" <<
+			1.0-re->r.GIshadowquality << "\" grid=\"82\" search=\"35\" >\n";
 	}
 	else
 	{
-		switch (R.r.GIquality)
+		switch (re->r.GIquality)
 		{
 			case 1 : ostr << " samples=\"16\" >\n";   break;
 			case 2 : ostr << " samples=\"36\" >\n";   break;
@@ -1865,12 +1958,12 @@ void yafrayFileRender_t::writePathlight()
 bool yafrayFileRender_t::writeWorld()
 {
 	World *world = G.scene->world;
-	if (R.r.GIquality!=0) {
-		if (R.r.GImethod==1) {
+	if (re->r.GIquality!=0) {
+		if (re->r.GImethod==1) {
 			if (world==NULL) cout << "WARNING: need world background for skydome!\n";
 			writeHemilight();
 		}
-		else if (R.r.GImethod==2) writePathlight();
+		else if (re->r.GImethod==2) writePathlight();
 	}
 
 	if (world==NULL) return false;
@@ -1906,7 +1999,7 @@ bool yafrayFileRender_t::writeWorld()
 	ostr << "<background type=\"constant\" name=\"world_background\" >\n";
 	// if no GI used, the GIpower parameter is not always initialized, so in that case ignore it
 	// (have to change method to init yafray vars in Blender)
-	float bg_mult = (R.r.GImethod==0) ? 1 : R.r.GIpower;
+	float bg_mult = (re->r.GImethod==0) ? 1 : re->r.GIpower;
 	ostr << "\t<color r=\"" << (world->horr * bg_mult) << 
 								"\" g=\"" << (world->horg * bg_mult) << 
 								"\" b=\"" << (world->horb * bg_mult) << "\" />\n";
@@ -1918,9 +2011,20 @@ bool yafrayFileRender_t::writeWorld()
 
 bool yafrayFileRender_t::executeYafray(const string &xmlpath)
 {
-	char yfr[8];
-	sprintf(yfr, "%d ", R.r.YF_numprocs);
-	string command = command_path + "yafray -c " + yfr + "\"" + xmlpath + "\"";
+	ostr.str("");
+	if (re->r.mode & R_BORDER) {
+		ostr << command_path << "yafray -c " << re->r.YF_numprocs
+		     << " -r " << (2.f*re->r.border.xmin - 1.f)
+		     << ":"    << (2.f*re->r.border.xmax - 1.f)
+		     << ":"    << (2.f*re->r.border.ymin - 1.f)
+		     << ":"    << (2.f*re->r.border.ymax - 1.f)
+		     << " \"" << xmlpath << "\"";
+	}
+	else
+		ostr << command_path << "yafray -c " << re->r.YF_numprocs << " \"" << xmlpath << "\"";
+	
+	string command = ostr.str();
+	cout << "COMMAND: " << command << endl;
 #ifndef WIN32
 	sigset_t yaf,old;
 	sigemptyset(&yaf);
