@@ -1,5 +1,5 @@
 /*
- * $Id: creator.c,v 1.51 2005/05/08 21:20:34 ianwill Exp $
+ * $Id: creator.c,v 1.61 2006/07/12 20:01:48 djcapelis Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,14 +43,18 @@
 #include "GEN_messaging.h"
 
 #include "DNA_ID.h"
+#include "DNA_scene_types.h"
 
 #include "BLI_blenlib.h"
+#include "blendef.h" /* for MAXFRAME */
+
 
 #include "BKE_utildefines.h"
 #include "BKE_blender.h"
 #include "BKE_font.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_material.h"
 #include "BKE_packedFile.h"
 #include "BKE_scene.h"
 
@@ -72,13 +76,12 @@
 
 #include "IMB_imbuf.h"	// for quicktime_init
 
-#include "RE_renderconverter.h"
-
 #include "BPY_extern.h"
 
+#include "RE_pipeline.h"
+
 #include "playanim_ext.h"
 #include "mydevice.h"
-#include "render.h"
 #include "nla.h"
 #include "datatoc.h"
 
@@ -94,10 +97,12 @@
 #endif
 
 // from buildinfo.c
+#ifdef BUILD_DATE
 extern char * build_date;
 extern char * build_time;
 extern char * build_platform;
 extern char * build_type;
+#endif
 
 /*	Local Function prototypes */
 static void print_help();
@@ -149,22 +154,39 @@ static void blender_esc(int sig)
 
 static void print_version(void)
 {
+#ifdef BUILD_DATE
+	printf ("Blender %d.%02d Build\n", G.version/100, G.version%100);
+	printf ("\tbuild date: %s\n", build_date);
+	printf ("\tbuild time: %s\n", build_time);
+	printf ("\tbuild platform: %s\n", build_platform);
+	printf ("\tbuild type: %s\n", build_type);
+#else
 	printf ("Blender %d.%02d\n", G.version/100, G.version%100);
+#endif
 }
 
 static void print_help(void)
 {
 	printf ("Blender V %d.%02d\n", G.version/100, G.version%100);
 	printf ("Usage: blender [options ...] [file]\n");
-				
+	
 	printf ("\nRender options:\n");
 	printf ("  -b <file>\tRender <file> in background\n");
 	printf ("    -S <name>\tSet scene <name>\n");
 	printf ("    -f <frame>\tRender frame <frame> and save it\n");				
 	printf ("    -s <frame>\tSet start to frame <frame> (use with -a)\n");
 	printf ("    -e <frame>\tSet end to frame (use with -a)<frame>\n");
-	printf ("    -a\t\tRender animation\n");
-				
+	printf ("    -o <path>\tSet the render path and file name.\n");
+	printf ("      Use // at the start of the path to\n");
+	printf ("        render relative to the blend file.\n");
+	printf ("      Use # in the filename to be replaced with the frame number\n");
+	printf ("      eg: blender -b foobar.blend -o //render_# -F PNG -x 1\n");
+	printf ("    -F <format>\tSet the render format, Valid options are..\n");
+	printf ("    \tTGA IRIS HAMX FTYPE JPEG MOVIE IRIZ RAWTGA\n");
+	printf ("    \tAVIRAW AVIJPEG PNG BMP FRAMESERVER\n");
+	printf ("    (formats that can be compiled into blender, not available on all systems)\n");
+	printf ("    \tHDR TIFF EXR MPEG AVICODEC QUICKTIME CINEON DPX\n");
+	printf ("    -x <bool>\tSet option to add the file extension to the end of the file.\n");
 	printf ("\nAnimation options:\n");
 	printf ("  -a <file(s)>\tPlayback <file(s)>\n");
 	printf ("    -p <sx> <sy>\tOpen with lower left corner at <sx>, <sy>\n");
@@ -212,7 +234,6 @@ int main(int argc, char **argv)
 	int audio = 0;
 #endif
 	setCallbacks();
-
 #ifdef __APPLE__
 		/* patch to ignore argument finder gives us (pid?) */
 	if (argc==2 && strncmp(argv[1], "-psn_", 5)==0) {
@@ -253,6 +274,10 @@ int main(int argc, char **argv)
 #ifdef __linux__
     #ifdef __alpha__
 	signal (SIGFPE, fpe_handler);
+    #else
+	if ( getenv("SDL_AUDIODRIVER") == NULL) {
+		setenv("SDL_AUDIODRIVER", "dma", 1);
+	}
     #endif
 #endif
 #if defined(__sgi)
@@ -348,8 +373,8 @@ int main(int argc, char **argv)
 
 		/* background render uses this font too */
 	BKE_font_register_builtin(datatoc_Bfont, datatoc_Bfont_size);
-		/* init struct R, Osa, defmaterial, filters */
-	RE_init_render_data();
+	
+	init_def_material();
 
 	if(G.background==0) {
 		for(a=1; a<argc; a++) {
@@ -463,6 +488,14 @@ int main(int argc, char **argv)
 
 #endif /* WITH_QUICKTIME */
 
+	/* dynamically load libtiff, if available */
+	libtiff_init();
+	if (!G.have_libtiff && (G.f & G_DEBUG)) {
+		printf("Unable to load: libtiff.\n");
+		printf("Try setting the BF_TIFF_LIB environment variable if you want this support.\n");
+		printf("Example: setenv BF_TIFF_LIB /usr/lib/libtiff.so\n");
+	}
+
 	/* OK we are ready for it */
 
 	for(a=1; a<argc; a++) {
@@ -505,7 +538,6 @@ int main(int argc, char **argv)
 
 						} else
 						{
-
 							SYS_WriteCommandLineInt(syshandle,argv[a],1);
 
 							/* doMipMap */
@@ -526,19 +558,22 @@ int main(int argc, char **argv)
 				}
 			case 'f':
 				a++;
-				if (G.scene && a < argc) {
-					G.real_sfra = (G.scene->r.sfra);
-					G.real_efra = (G.scene->r.efra);
-					(G.scene->r.sfra) = atoi(argv[a]);
-					(G.scene->r.efra) = (G.scene->r.sfra);
-					RE_animrender(NULL);
+				if (G.scene) {
+					if (a < argc) {
+						int frame= MIN2(MAXFRAME, MAX2(1, atoi(argv[a])));
+						Render *re= RE_NewRender(G.scene->id.name);
+						RE_BlenderAnim(re, G.scene, frame, frame);
+					}
+				} else {
+					printf("\nError: no blend loaded. cannot use '-f'.\n");
 				}
 				break;
 			case 'a':
 				if (G.scene) {
-					G.real_sfra = (G.scene->r.sfra);
-					G.real_efra = (G.scene->r.efra);
-					RE_animrender(NULL);
+					Render *re= RE_NewRender(G.scene->id.name);
+					RE_BlenderAnim(re, G.scene, G.scene->r.sfra, G.scene->r.efra);
+				} else {
+					printf("\nError: no blend loaded. cannot use '-a'.\n");
 				}
 				break;
 			case 'S':
@@ -549,13 +584,19 @@ int main(int argc, char **argv)
 			case 's':
 				a++;
 				if(G.scene) {
-					if (a < argc) (G.scene->r.sfra) = atoi(argv[a]);
+					int frame= MIN2(MAXFRAME, MAX2(1, atoi(argv[a])));
+					if (a < argc) (G.scene->r.sfra) = frame;
+				} else {
+					printf("\nError: no blend loaded. cannot use '-s'.\n");
 				}
 				break;
 			case 'e':
 				a++;
 				if(G.scene) {
-					if (a < argc) (G.scene->r.efra) = atoi(argv[a]);
+					int frame= MIN2(MAXFRAME, MAX2(1, atoi(argv[a])));
+					if (a < argc) (G.scene->r.efra) = frame;
+				} else {
+					printf("\nError: no blend loaded. cannot use '-e'.\n");
 				}
 				break;
 			case 'P':
@@ -563,6 +604,70 @@ int main(int argc, char **argv)
 				if (a < argc) BPY_run_python_script (argv[a]);
 				else printf("\nError: you must specify a Python script after '-P '.\n");
 				break;
+			case 'o':
+				a++;
+				if (a < argc){
+					if(G.scene) {
+						BLI_strncpy(G.scene->r.pic, argv[a], FILE_MAXDIR);
+					} else {
+						printf("\nError: no blend loaded. cannot use '-o'.\n");
+					}
+				} else {
+					printf("\nError: you must specify a path after '-o '.\n");
+				}
+				break;
+			case 'F':
+				a++;
+				if (a < argc){
+					if(!G.scene) {
+						printf("\nError: no blend loaded. order the arguments so '-F ' is after the blend is loaded.\n");
+					} else {
+						if      (!strcmp(argv[a],"TGA")) G.scene->r.imtype = R_TARGA;
+						else if (!strcmp(argv[a],"IRIS")) G.scene->r.imtype = R_IRIS;
+						else if (!strcmp(argv[a],"HAMX")) G.scene->r.imtype = R_HAMX;
+						else if (!strcmp(argv[a],"FTYPE")) G.scene->r.imtype = R_FTYPE;
+						else if (!strcmp(argv[a],"JPEG")) G.scene->r.imtype = R_JPEG90;
+						else if (!strcmp(argv[a],"MOVIE")) G.scene->r.imtype = R_MOVIE;
+						else if (!strcmp(argv[a],"IRIZ")) G.scene->r.imtype = R_IRIZ;
+						else if (!strcmp(argv[a],"RAWTGA")) G.scene->r.imtype = R_RAWTGA;
+						else if (!strcmp(argv[a],"AVIRAW")) G.scene->r.imtype = R_AVIRAW;
+						else if (!strcmp(argv[a],"AVIJPEG")) G.scene->r.imtype = R_AVIJPEG;
+						else if (!strcmp(argv[a],"PNG")) G.scene->r.imtype = R_PNG;
+						else if (!strcmp(argv[a],"AVICODEC")) G.scene->r.imtype = R_AVICODEC;
+						else if (!strcmp(argv[a],"QUICKTIME")) G.scene->r.imtype = R_QUICKTIME;
+						else if (!strcmp(argv[a],"BMP")) G.scene->r.imtype = R_BMP;
+						else if (!strcmp(argv[a],"HDR")) G.scene->r.imtype = R_RADHDR;
+						else if (!strcmp(argv[a],"TIFF")) G.scene->r.imtype = R_IRIS;
+						else if (!strcmp(argv[a],"EXR")) G.scene->r.imtype = R_OPENEXR;
+						else if (!strcmp(argv[a],"MPEG")) G.scene->r.imtype = R_FFMPEG;
+						else if (!strcmp(argv[a],"FRAMESERVER")) G.scene->r.imtype = R_FRAMESERVER;
+						else if (!strcmp(argv[a],"CINEON")) G.scene->r.imtype = R_CINEON;
+						else if (!strcmp(argv[a],"DPX")) G.scene->r.imtype = R_DPX;
+						else printf("\nError: Format from '-F ' not known.\n");
+					}
+				} else {
+					printf("\nError: no blend loaded. cannot use '-x'.\n");
+				}
+				break;
+				
+			case 'x': /* extension */
+				a++;
+				if (a < argc) {
+					if(G.scene) {
+						if (argv[a][0] == '0') {
+							G.scene->r.scemode &= ~R_EXTENSION;
+						} else if (argv[a][0] == '1') {
+							G.scene->r.scemode |= R_EXTENSION;
+						} else {
+							printf("\nError: Use '-x 1' or '-x 0' To set the extension option.\n");
+						}
+					} else {
+						printf("\nError: no blend loaded. order the arguments so '-o ' is after '-x '.\n");
+					}
+				} else {
+					printf("\nError: you must specify a path after '- '.\n");
+				}
+				break;
 			}
 		}
 		else {
@@ -571,8 +676,8 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if(G.background)
-	{
+	if(G.background) {
+		/* actually incorrect, but works for now (ton) */
 		exit_usiblender();
 	}
 
@@ -593,10 +698,15 @@ static void error_cb(char *err)
 	error("%s", err);
 }
 
+static void mem_error_cb(char *errorStr)
+{
+	fprintf(stderr, errorStr);
+}
+
 void setCallbacks(void)
 {
 	/* Error output from the alloc routines: */
-	MEM_set_error_stream(stderr);
+	MEM_set_error_callback(mem_error_cb);
 
 
 	/* BLI_blenlib: */
@@ -604,7 +714,4 @@ void setCallbacks(void)
 	BLI_setErrorCallBack(error_cb); /* */
 	BLI_setInterruptCallBack(blender_test_break);
 
-	/* render module: execution flow, timers, cursors and display. */
-	RE_set_getrenderdata_callback(RE_rotateBlenderScene);
-	RE_set_freerenderdata_callback(RE_freeRotateBlenderScene);
 }
