@@ -1,5 +1,5 @@
 /**
- * $Id: BL_KetsjiEmbedStart.cpp,v 1.7 2005/03/25 10:33:35 kester Exp $
+ * $Id: BL_KetsjiEmbedStart.cpp,v 1.14 2006/05/22 21:03:42 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,12 +42,17 @@
 #pragma warning (disable:4786)
 #endif
 
+#ifdef __APPLE__
+#define GL_GLEXT_LEGACY 1
+#endif 
+
 #include "KX_BlenderGL.h"
 #include "KX_BlenderCanvas.h"
 #include "KX_BlenderKeyboardDevice.h"
 #include "KX_BlenderMouseDevice.h"
 #include "KX_BlenderRenderTools.h"
 #include "KX_BlenderSystem.h"
+#include "BL_Material.h"
 
 #include "KX_KetsjiEngine.h"
 #include "KX_BlenderSceneConverter.h"
@@ -56,6 +61,7 @@
 
 #include "RAS_OpenGLRasterizer.h"
 #include "RAS_VAOpenGLRasterizer.h"
+#include "RAS_ListRasterizer.h"
 #include "RAS_GLExtensionManager.h"
 
 #include "NG_LoopBackNetworkDeviceInterface.h"
@@ -79,8 +85,24 @@
 
 static BlendFileData *load_game_data(char *filename) {
 	BlendReadError error;
-	BlendFileData *bfd= BLO_read_from_file(filename, &error);
-	
+	//this doesn't work anymore for relative paths, so use BLO_read_from_memory instead
+	//BlendFileData *bfd= BLO_read_from_file(filename, &error);
+	FILE* file = fopen(filename,"rb");
+	BlendFileData *bfd  = 0;
+	if (file)
+	{
+		fseek(file, 0L, SEEK_END);
+		int len= ftell(file);
+		fseek(file, 0L, SEEK_SET);	
+		char* filebuffer= new char[len];//MEM_mallocN(len, "text_buffer");
+		int sizeread = fread(filebuffer,len,1,file);
+		if (sizeread==1)
+		{
+			bfd = BLO_read_from_memory(filebuffer, len, &error);
+		}
+		fclose(file);
+	}
+
 	if (!bfd) {
 		printf("Loading %s failed: %s\n", filename, BLO_bre_as_string(error));
 	}
@@ -90,17 +112,20 @@ static BlendFileData *load_game_data(char *filename) {
 
 extern "C" void StartKetsjiShell(struct ScrArea *area,
 								 char* scenename,
-								 struct Main* maggie,
+								 struct Main* maggie1,
+								 struct SpaceIpo *sipo,
 								 int always_use_expand_framing)
 {
 	int exitrequested = KX_EXIT_REQUEST_NO_REQUEST;
-	Main* blenderdata = maggie;
+	
+	Main* blenderdata = maggie1;
+
 	char* startscenename = scenename;
 	char pathname[160];
-	strcpy (pathname, maggie->name);
+	strcpy (pathname, blenderdata->name);
 	STR_String exitstring = "";
 	BlendFileData *bfd= NULL;
-	
+
 	bgl::InitExtensions(1);
 	
 	do
@@ -113,7 +138,23 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 		bool usefixed = (SYS_GetCommandLineInt(syshandle, "fixedtime", 0) != 0);
 		bool profile = (SYS_GetCommandLineInt(syshandle, "show_profile", 0) != 0);
 		bool frameRate = (SYS_GetCommandLineInt(syshandle, "show_framerate", 0) != 0);
+		bool game2ipo = (SYS_GetCommandLineInt(syshandle, "game2ipo", 0) != 0);
+		bool displaylists = (SYS_GetCommandLineInt(syshandle, "displaylists", 0) != 0);
+		bool usemat = false;
 		
+		#ifdef GL_ARB_multitexture
+		if(bgl::RAS_EXT_support._ARB_multitexture && bgl::QueryVersion(1, 1)) {
+			usemat = (SYS_GetCommandLineInt(syshandle, "blender_material", 0) != 0);
+			int unitmax=0;
+			glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, (GLint*)&unitmax);
+			bgl::max_texture_units = MAXTEX>unitmax?unitmax:MAXTEX;
+			//std::cout << "using(" << bgl::max_texture_units << ") of(" << unitmax << ") texture units." << std::endl;
+		} else {
+			bgl::max_texture_units = 0;
+		}
+		#endif
+
+
 		// create the canvas, rasterizer and rendertools
 		RAS_ICanvas* canvas = new KX_BlenderCanvas(area);
 		canvas->SetMouseState(RAS_ICanvas::MOUSE_INVISIBLE);
@@ -124,8 +165,12 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 		int usevta = SYS_GetCommandLineInt(syshandle,"vertexarrays",1);
 		bool useVertexArrays = (usevta > 0);
 		
-		if (useVertexArrays && bgl::QueryVersion(1, 1))
-			rasterizer = new RAS_VAOpenGLRasterizer(canvas);
+		bool lock_arrays = (displaylists && useVertexArrays);
+
+		if(displaylists && !useVertexArrays)
+			rasterizer = new RAS_ListRasterizer(canvas);
+		else if (useVertexArrays && bgl::QueryVersion(1, 1))
+			rasterizer = new RAS_VAOpenGLRasterizer(canvas, lock_arrays);
 		else
 			rasterizer = new RAS_OpenGLRasterizer(canvas);
 		
@@ -159,7 +204,9 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 		ketsjiengine->SetAudioDevice(audiodevice);
 		ketsjiengine->SetUseFixedTime(usefixed);
 		ketsjiengine->SetTimingDisplay(frameRate, profile, properties);
+
 		
+	
 		// some blender stuff
 		MT_CmMatrix4x4 projmat;
 		MT_CmMatrix4x4 viewmat;
@@ -226,8 +273,8 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 		Scene *blscene = NULL;
 		if (!bfd)
 		{
-			blscene = (Scene*) maggie->scene.first;
-			for (Scene *sce= (Scene*) maggie->scene.first; sce; sce= (Scene*) sce->id.next)
+			blscene = (Scene*) blenderdata->scene.first;
+			for (Scene *sce= (Scene*) blenderdata->scene.first; sce; sce= (Scene*) sce->id.next)
 			{
 				if (startscenename == (sce->id.name+2))
 				{
@@ -239,6 +286,13 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 			blscene = bfd->curscene;
 		}
 
+		if (blscene)
+		{
+			int startFrame = blscene->r.cfra;
+			ketsjiengine->SetGame2IpoMode(game2ipo,startFrame);
+		}
+
+
 		// Quad buffered needs a special window.
 		if (blscene->r.stereomode != RAS_IRasterizer::RAS_STEREO_QUADBUFFERED)
 			rasterizer->SetStereoMode((RAS_IRasterizer::StereoMode) blscene->r.stereomode);
@@ -254,13 +308,15 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 			}
 			
 			// create a scene converter, create and convert the startingscene
-			KX_ISceneConverter* sceneconverter = new KX_BlenderSceneConverter(blenderdata, ketsjiengine);
+			KX_ISceneConverter* sceneconverter = new KX_BlenderSceneConverter(blenderdata,sipo, ketsjiengine);
 			ketsjiengine->SetSceneConverter(sceneconverter);
 			
 			if (always_use_expand_framing)
 				sceneconverter->SetAlwaysUseExpandFraming(true);
 			
-			
+			if(usemat)
+				sceneconverter->SetMaterials(true);
+					
 			KX_Scene* startscene = new KX_Scene(keyboarddevice,
 				mousedevice,
 				networkdevice,
@@ -271,7 +327,7 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 			PyObject* dictionaryobject = initGamePythonScripting("Ketsji", psl_Lowest);
 			ketsjiengine->SetPythonDictionary(dictionaryobject);
 			initRasterizer(rasterizer, canvas);
-			initGameLogic(startscene);
+			PyObject *gameLogic = initGameLogic(startscene);
 			initGameKeys();
 			initPythonConstraintBinding();
 
@@ -330,11 +386,11 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 				exitstring = ketsjiengine->GetExitString();
 				
 				// when exiting the mainloop
-				exitGamePythonScripting();
+				dictionaryClearByHand(gameLogic);
 				ketsjiengine->StopEngine();
+				exitGamePythonScripting();
 				networkdevice->Disconnect();
 			}
-
 			if (sceneconverter)
 			{
 				delete sceneconverter;
@@ -387,7 +443,6 @@ extern "C" void StartKetsjiShell(struct ScrArea *area,
 			delete canvas;
 			canvas = NULL;
 		}
-		
 		SND_DeviceManager::Unsubscribe();
 	
 	} while (exitrequested == KX_EXIT_REQUEST_RESTART_GAME || exitrequested == KX_EXIT_REQUEST_START_OTHER_GAME);
