@@ -1,5 +1,5 @@
 /**
- * $Id: KX_BlenderRenderTools.cpp,v 1.4 2003/01/07 00:18:59 sgefant Exp $
+ * $Id: KX_BlenderRenderTools.cpp,v 1.18 2006/04/02 21:04:19 erwin Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,15 +31,12 @@
 
 #include "KX_BlenderRenderTools.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 // OpenGL gl.h needs 'windows.h' on windows platforms 
 #include <windows.h>
 #endif //WIN32
 #ifdef __APPLE__
+#define GL_GLEXT_LEGACY 1
 #include <OpenGL/gl.h>
 #else
 #include <GL/gl.h>
@@ -49,21 +46,32 @@
 #include "RAS_IRasterizer.h"
 #include "RAS_LightObject.h"
 #include "RAS_ICanvas.h"
-
+#include "RAS_GLExtensionManager.h"
 
 // next two includes/dependencies come from the shadow feature
 // it needs the gameobject and the sumo physics scene for a raycast
 #include "KX_GameObject.h"
 
 #include "KX_BlenderPolyMaterial.h"
+#include "KX_PolygonMaterial.h"
+#include "KX_BlenderMaterial.h"
+
 #include "Value.h"
 
 #include "KX_BlenderGL.h" // for text printing
 #include "STR_String.h"
 #include "RAS_BucketManager.h" // for polymaterial (needed for textprinting)
 
+#include "KX_RayCast.h"
+#include "KX_IPhysicsController.h"
+#include "PHY_IPhysicsEnvironment.h"
+#include "KX_Scene.h"
+
 KX_BlenderRenderTools::KX_BlenderRenderTools()
 {
+	glGetIntegerv(GL_MAX_LIGHTS, (GLint*) &m_numgllights);
+	if (m_numgllights < 8)
+		m_numgllights = 8;
 }
 
 /**
@@ -86,8 +94,6 @@ int	KX_BlenderRenderTools::ProcessLighting(int layer)
 	{
 		if (m_clientobject)
 		{
-
-			
 			if (applyLights(layer))
 			{
 				EnableOpenGLLights();
@@ -96,9 +102,7 @@ int	KX_BlenderRenderTools::ProcessLighting(int layer)
 			{
 				DisableOpenGLLights();
 				result = false;
-			}
-
-			
+			}			
 		}
 	}
 	return result;
@@ -119,10 +123,42 @@ void KX_BlenderRenderTools::BeginFrame(RAS_IRasterizer* rasty)
 
 
 }
+
+bool KX_BlenderRenderTools::RayHit(KX_ClientObjectInfo* client, MT_Point3& hit_point, MT_Vector3& hit_normal, void * const data)
+{
+	double* const oglmatrix = (double* const) data;
+	MT_Point3 resultpoint(hit_point);
+	MT_Vector3 resultnormal(hit_normal);
+	MT_Vector3 left(oglmatrix[0],oglmatrix[1],oglmatrix[2]);
+	MT_Vector3 dir = -(left.cross(resultnormal)).safe_normalized();
+	left = (dir.cross(resultnormal)).safe_normalized();
+	// for the up vector, we take the 'resultnormal' returned by the physics
 	
+	double maat[16]={
+			left[0],        left[1],        left[2], 0,
+				dir[0],         dir[1],         dir[2], 0,
+		resultnormal[0],resultnormal[1],resultnormal[2], 0,
+				0,              0,              0, 1};
+	glTranslated(resultpoint[0],resultpoint[1],resultpoint[2]);
+	//glMultMatrixd(oglmatrix);
+	glMultMatrixd(maat);
+	return true;
+}
 
 void KX_BlenderRenderTools::applyTransform(RAS_IRasterizer* rasty,double* oglmatrix,int objectdrawmode )
 {
+	/* FIXME:
+	blender: intern/moto/include/MT_Vector3.inl:42: MT_Vector3 operator/(const
+	MT_Vector3&, double): Assertion `!MT_fuzzyZero(s)' failed. 
+	
+	Program received signal SIGABRT, Aborted. 
+	[Switching to Thread 16384 (LWP 1519)] 
+	0x40477571 in kill () from /lib/libc.so.6 
+	(gdb) bt 
+	#7  0x08334368 in MT_Vector3::normalized() const () 
+	#8  0x0833e6ec in KX_BlenderRenderTools::applyTransform(RAS_IRasterizer*, double*, int) () 
+	*/
+
 	if (objectdrawmode & RAS_IPolyMaterial::BILLBOARD_SCREENALIGNED ||
 		objectdrawmode & RAS_IPolyMaterial::BILLBOARD_AXISALIGNED)
 	{
@@ -174,49 +210,32 @@ void KX_BlenderRenderTools::applyTransform(RAS_IRasterizer* rasty,double* oglmat
 		if (objectdrawmode & RAS_IPolyMaterial::SHADOW)
 		{
 			// shadow must be cast to the ground, physics system needed here!
-			KX_GameObject* gameobj = (KX_GameObject*) this->m_clientobject;
 			MT_Point3 frompoint(oglmatrix[12],oglmatrix[13],oglmatrix[14]);
+			KX_GameObject *gameobj = (KX_GameObject*) this->m_clientobject;
 			MT_Vector3 direction = MT_Vector3(0,0,-1);
 
-			
 			direction.normalize();
 			direction *= 100000;
 
 			MT_Point3 topoint = frompoint + direction;
+
+			KX_Scene* kxscene = (KX_Scene*) m_auxilaryClientInfo;
+			PHY_IPhysicsEnvironment* physics_environment = kxscene->GetPhysicsEnvironment();
+			KX_IPhysicsController* physics_controller = gameobj->GetPhysicsController();
+			
+			KX_GameObject *parent = gameobj->GetParent();
+			if (!physics_controller && parent)
+				physics_controller = parent->GetPhysicsController();
+			if (parent)
+				parent->Release();
+				
 			MT_Point3 resultpoint;
 			MT_Vector3 resultnormal;
-
-			//todo:
-			//use physics abstraction
-
-			
-			//SM_Scene* scene = (SM_Scene*) m_auxilaryClientInfo;
-
-			//SM_Object* hitObj = scene->rayTest(gameobj->GetSumoObject(),frompoint,topoint,
-			//									 resultpoint, resultnormal);
-
-			
-			if (0) //hitObj)
-			{
-				MT_Vector3 left(oglmatrix[0],oglmatrix[1],oglmatrix[2]);
-				MT_Vector3 dir = -(left.cross(resultnormal)).normalized();
-				left = (dir.cross(resultnormal)).normalized();
-				// for the up vector, we take the 'resultnormal' returned by the physics
-				
-				double maat[16]={
-					left[0], left[1],left[2], 0,
-					dir[0], dir[1],dir[2],0,
-					resultnormal[0],resultnormal[1],resultnormal[2],0,
-					0,0,0,1};
-				glTranslated(resultpoint[0],resultpoint[1],resultpoint[2]);
-				glMultMatrixd(maat);
-					//	glMultMatrixd(oglmatrix);
-			} else
+			if (!KX_RayCast::RayTest(physics_controller, physics_environment, frompoint, topoint, resultpoint, resultnormal, KX_RayCast::Callback<KX_BlenderRenderTools>(this, oglmatrix)))
 			{
+				// couldn't find something to cast the shadow on...
 				glMultMatrixd(oglmatrix);
 			}
-
-			
 		} else
 		{
 
@@ -236,8 +255,16 @@ void	KX_BlenderRenderTools::RenderText(int mode,RAS_IPolyMaterial* polymat,float
 		
 	STR_String mytext = ((CValue*)m_clientobject)->GetPropertyText("Text");
 	
-	KX_BlenderPolyMaterial* blenderpoly = (KX_BlenderPolyMaterial*)polymat;
-	struct TFace* tface = blenderpoly->GetTFace();
+	const unsigned int flag = polymat->GetFlag();
+	struct TFace* tface = 0;
+
+	if(flag & RAS_BLENDERMAT) {
+		KX_BlenderMaterial *bl_mat = static_cast<KX_BlenderMaterial*>(polymat);
+		tface = bl_mat->GetTFace();
+	} else {
+		KX_PolygonMaterial* blenderpoly = static_cast<KX_PolygonMaterial*>(polymat);
+		tface = blenderpoly->GetTFace();
+	}
 	
 	BL_RenderText( mode,mytext,mytext.Length(),tface,v1,v2,v3,v4);
 	
@@ -260,9 +287,6 @@ void KX_BlenderRenderTools::DisableOpenGLLights()
 {
 	glDisable(GL_LIGHTING);
 	glDisable(GL_COLOR_MATERIAL);
-#ifndef SLOWPAINT
-	glDisableClientState(GL_NORMAL_ARRAY);
-#endif //SLOWPAINT
 }
 
 	
@@ -270,12 +294,12 @@ void KX_BlenderRenderTools::EnableOpenGLLights()
 {
 	glEnable(GL_LIGHTING);
 	
-	glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);
 	glEnable(GL_COLOR_MATERIAL);
-#ifndef SLOWPAINT
-	glEnableClientState(GL_NORMAL_ARRAY);
-#endif //SLOWPAINT
-	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, false);
+	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
+	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, true);
+	if (bgl::QueryExtension(bgl::_GL_EXT_separate_specular_color) || bgl::QueryVersion(1, 2))
+		glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
+
 }
 	
 
@@ -320,36 +344,29 @@ int	KX_BlenderRenderTools::applyLights(int objectlayer)
 {
 // taken from blender source, incompatibility between Blender Object / GameObject	
 	
-	int count;
+	unsigned int count;
 	float vec[4];
 		
 	vec[3]= 1.0;
 	
-	for(count=0; count<8; count++)
+	for(count=0; count<m_numgllights; count++)
 		glDisable((GLenum)(GL_LIGHT0+count));
 	
-	count= 0;
-
 	//std::vector<struct	RAS_LightObject*> m_lights;
 	std::vector<struct	RAS_LightObject*>::iterator lit = m_lights.begin();
-
 	
-	for (lit = m_lights.begin(); !(lit==m_lights.end()); ++lit)
+	glPushMatrix();
+	glLoadMatrixf(m_viewmat);
+	for (lit = m_lights.begin(), count = 0; !(lit==m_lights.end()) && count < m_numgllights; ++lit)
 	{
 		RAS_LightObject* lightdata = (*lit);
 		if (lightdata->m_layer & objectlayer)
 		{
-
-			glPushMatrix();
-			glLoadMatrixf(m_viewmat);
-			
-			
 			vec[0] = (*(lightdata->m_worldmatrix))(0,3);
 			vec[1] = (*(lightdata->m_worldmatrix))(1,3);
 			vec[2] = (*(lightdata->m_worldmatrix))(2,3);
 			vec[3] = 1;
 
-
 			if(lightdata->m_type==RAS_LightObject::LIGHT_SUN) {
 				
 				vec[0] = (*(lightdata->m_worldmatrix))(0,2);
@@ -366,9 +383,9 @@ int	KX_BlenderRenderTools::applyLights(int objectlayer)
 				glLightfv((GLenum)(GL_LIGHT0+count), GL_POSITION, vec); 
 				glLightf((GLenum)(GL_LIGHT0+count), GL_CONSTANT_ATTENUATION, 1.0);
 				glLightf((GLenum)(GL_LIGHT0+count), GL_LINEAR_ATTENUATION, lightdata->m_att1/lightdata->m_distance);
-						// without this next line it looks backward compatible.
-						//attennuation still is acceptable 
-						// glLightf((GLenum)(GL_LIGHT0+count), GL_QUADRATIC_ATTENUATION, la->att2/(la->dist*la->dist)); 
+				// without this next line it looks backward compatible.
+				//attennuation still is acceptable 
+				glLightf((GLenum)(GL_LIGHT0+count), GL_QUADRATIC_ATTENUATION, lightdata->m_att2/(lightdata->m_distance*lightdata->m_distance)); 
 				
 				if(lightdata->m_type==RAS_LightObject::LIGHT_SPOT) {
 					vec[0] = -(*(lightdata->m_worldmatrix))(0,2);
@@ -384,21 +401,33 @@ int	KX_BlenderRenderTools::applyLights(int objectlayer)
 				else glLightf((GLenum)(GL_LIGHT0+count), GL_SPOT_CUTOFF, 180.0);
 			}
 			
-			vec[0]= lightdata->m_energy*lightdata->m_red;
-			vec[1]= lightdata->m_energy*lightdata->m_green;
-			vec[2]= lightdata->m_energy*lightdata->m_blue;
-			vec[3]= 1.0;
-			glLightfv((GLenum)(GL_LIGHT0+count), GL_DIFFUSE, vec); 
+			if (lightdata->m_nodiffuse)
+			{
+				vec[0] = vec[1] = vec[2] = vec[3] = 0.0;
+			} else {
+				vec[0]= lightdata->m_energy*lightdata->m_red;
+				vec[1]= lightdata->m_energy*lightdata->m_green;
+				vec[2]= lightdata->m_energy*lightdata->m_blue;
+				vec[3]= 1.0;
+			}
+			glLightfv((GLenum)(GL_LIGHT0+count), GL_DIFFUSE, vec);
+			if (lightdata->m_nospecular)
+			{
+				vec[0] = vec[1] = vec[2] = vec[3] = 0.0;
+			} else if (lightdata->m_nodiffuse) {
+				vec[0]= lightdata->m_energy*lightdata->m_red;
+				vec[1]= lightdata->m_energy*lightdata->m_green;
+				vec[2]= lightdata->m_energy*lightdata->m_blue;
+				vec[3]= 1.0;
+			}
 			glLightfv((GLenum)(GL_LIGHT0+count), GL_SPECULAR, vec);
 			glEnable((GLenum)(GL_LIGHT0+count));
 
-			glPopMatrix();
-
 			count++;
-			if(count>7) 
-				break;
+			
 		}
 	}
+	glPopMatrix();
 
 	return count;
 
@@ -408,12 +437,16 @@ int	KX_BlenderRenderTools::applyLights(int objectlayer)
 
 RAS_IPolyMaterial* KX_BlenderRenderTools::CreateBlenderPolyMaterial(
 		const STR_String &texname,
-		bool ba,const STR_String& matname,int tile,int tilexrep,int tileyrep,int mode,int transparant,int lightlayer
+		bool ba,const STR_String& matname,int tile,int tilexrep,int tileyrep,int mode,bool transparant,bool zsort, int lightlayer
 		,bool bIsTriangle,void* clientobject,void* tface)
 {
-	return new KX_BlenderPolyMaterial(
+	assert(!"Deprecated");
+/*	return new KX_BlenderPolyMaterial(
 
 		texname,
-		ba,matname,tile,tilexrep,tileyrep,mode,transparant,lightlayer
-		,bIsTriangle,clientobject,(struct TFace*)tface);
+		ba,matname,tile,tilexrep,tileyrep,mode,transparant,zsort, lightlayer
+		,bIsTriangle,clientobject,(struct TFace*)tface);*/
+	return NULL;
 }
+
+unsigned int KX_BlenderRenderTools::m_numgllights;
