@@ -1,5 +1,5 @@
 /**
- * $Id: BL_ArmatureObject.cpp,v 1.5 2005/04/23 11:36:44 kester Exp $
+ * $Id: BL_ArmatureObject.cpp,v 1.11 2006/04/28 17:35:03 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,6 +48,24 @@
 #include <config.h>
 #endif
 
+
+BL_ArmatureObject::BL_ArmatureObject(
+				void* sgReplicationInfo, 
+				SG_Callbacks callbacks, 
+				Object *armature )
+
+:	KX_GameObject(sgReplicationInfo,callbacks),
+	m_objArma(armature),
+	m_mrdPose(NULL),
+	m_lastframe(0.),
+	m_activeAct(NULL),
+	m_activePriority(999)
+{
+	m_armature = get_armature(m_objArma);
+	m_pose = m_objArma->pose;
+}
+
+
 CValue* BL_ArmatureObject::GetReplica()
 {
 	BL_ArmatureObject* replica = new BL_ArmatureObject(*this);
@@ -68,19 +86,24 @@ void BL_ArmatureObject::ProcessReplica(BL_ArmatureObject *replica)
 BL_ArmatureObject::~BL_ArmatureObject()
 {
 	if (m_mrdPose){
-		clear_pose(m_mrdPose);
+		free_pose_channels(m_mrdPose);
 		MEM_freeN(m_mrdPose);
 	}
 }
 
+/* note, you can only call this for exisiting Armature objects, and not mix it with other Armatures */
+/* there is only 1 unique Pose per Armature */
 void BL_ArmatureObject::ApplyPose()
 {
 	if (m_pose){
-		apply_pose_armature(GetArmature(), m_pose, 1);
-		if (!m_mrdPose)
-			copy_pose (&m_mrdPose, m_pose, 0);
-		else
-			get_pose_from_pose(&m_mrdPose, m_pose);
+		// copy to armature object
+		extract_pose_from_pose(m_objArma->pose, m_pose);
+		
+		// is this needed anymore?
+		//if (!m_mrdPose)
+		//	copy_pose (&m_mrdPose, m_pose, 0);
+		//else
+		//	extract_pose_from_pose(m_mrdPose, m_pose);
 	}
 }
 
@@ -123,10 +146,18 @@ void BL_ArmatureObject::GetPose(bPose **pose)
 {
 	/* If the caller supplies a null pose, create a new one. */
 	/* Otherwise, copy the armature's pose channels into the caller-supplied pose */
-	if (!*pose)
-		copy_pose(pose, m_pose, 0);
+
+	if (!*pose) {
+		/*	probably not to good of an idea to
+			duplicate everying, but it clears up 
+			a crash and memory leakage when 
+			&BL_ActionActuator::m_pose is freed
+		*/
+		int copy_constraint_channels_hack = 1;
+		copy_pose(pose, m_pose, copy_constraint_channels_hack);
+	}
 	else
-		get_pose_from_pose(pose, m_pose);
+		extract_pose_from_pose(*pose, m_pose);
 
 }
 
@@ -135,14 +166,15 @@ void BL_ArmatureObject::GetMRDPose(bPose **pose)
 	/* If the caller supplies a null pose, create a new one. */
 	/* Otherwise, copy the armature's pose channels into the caller-supplied pose */
 
-	if (!m_mrdPose){
-		copy_pose (&m_mrdPose, m_pose, 0);
-	}
+	// is this needed anymore?
+	//if (!m_mrdPose){
+	//	copy_pose (&m_mrdPose, m_pose, 0);
+	//}
 
 	if (!*pose)
-		copy_pose(pose, m_mrdPose, 0);
+		copy_pose(pose, m_objArma->pose, 0);
 	else
-		get_pose_from_pose(pose, m_mrdPose);
+		extract_pose_from_pose(*pose, m_objArma->pose);
 
 }
 
@@ -158,18 +190,17 @@ double BL_ArmatureObject::GetLastFrame()
 
 bool BL_ArmatureObject::GetBoneMatrix(Bone* bone, MT_Matrix4x4& matrix) const
 {
-	MT_assert(verify_boneptr((bArmature*) GetArmature(), bone) && "Bone is not part of this armature.");
-	
-	matrix.setValue(&bone->posemat[0][0]);
-	
-	return true;
+	// ton hack
+	bPoseChannel *pchan= get_pose_channel(m_pose, bone->name);
+
+	if(pchan) {
+		matrix.setValue(&pchan->pose_mat[0][0]);
+		return true;
+	}
+	return false;
 }
 
 float BL_ArmatureObject::GetBoneLength(Bone* bone) const
 {
-	MT_assert(verify_boneptr((bArmature*) GetArmature(), bone) && "Bone is not part of this armature.");
-	
-	return (MT_Point3(bone->head) - MT_Point3(bone->tail)).length();
+	return (float)(MT_Point3(bone->head) - MT_Point3(bone->tail)).length();
 }
-
-
