@@ -1,5 +1,5 @@
 /**
- * $Id: BL_BlenderDataConversion.cpp,v 1.27 2005/04/23 11:36:44 kester Exp $
+ * $Id: BL_BlenderDataConversion.cpp,v 1.41 2006/04/17 01:33:10 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -83,6 +83,9 @@
 #include "RAS_TexVert.h"
 #include "RAS_BucketManager.h"
 #include "RAS_IRenderTools.h"
+#include "BL_Material.h"
+#include "KX_BlenderMaterial.h"
+#include "BL_Texture.h"
 
 #include "DNA_action_types.h"
 #include "BKE_main.h"
@@ -108,6 +111,7 @@
 /* This list includes only data type definitions */
 #include "DNA_object_types.h"
 #include "DNA_material_types.h"
+#include "DNA_texture_types.h"
 #include "DNA_image_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_group_types.h"
@@ -217,6 +221,473 @@ static void SetDefaultFaceType(Scene* scene)
 	}
 }
 
+
+// --
+static void GetRGB(short type,
+	MFace* mface,
+	MCol* mmcol,
+	Material *mat,
+	TFace *tface,
+	unsigned int &c0, 
+	unsigned int &c1, 
+	unsigned int &c2, 
+	unsigned int &c3)
+{
+	unsigned int colour = 0xFFFFFFFFL;
+	switch(type)
+	{
+		case 0:	// vertex colors?
+		{
+			if(mmcol) {
+				c0 = KX_Mcol2uint_new(mmcol[0]);
+				c1 = KX_Mcol2uint_new(mmcol[1]);
+				c2 = KX_Mcol2uint_new(mmcol[2]);
+				if (mface->v4)
+					c3 = KX_Mcol2uint_new(mmcol[3]);
+				mmcol += 4;
+			}else // backup white
+			{
+				c0 = KX_rgbaint2uint_new(colour);
+				c1 = KX_rgbaint2uint_new(colour);
+				c2 = KX_rgbaint2uint_new(colour);	
+				if (mface->v4)
+					c3 = KX_rgbaint2uint_new( colour );
+			}
+		} break;
+		
+	
+		case 1: // material rgba
+		{
+			if (mat) {
+				union {
+					unsigned char cp[4];
+					unsigned int integer;
+				} col_converter;
+				col_converter.cp[3] = (unsigned char) (mat->r*255.0);
+				col_converter.cp[2] = (unsigned char) (mat->g*255.0);
+				col_converter.cp[1] = (unsigned char) (mat->b*255.0);
+				col_converter.cp[0] = (unsigned char) (mat->alpha*255.0);
+				colour = col_converter.integer;
+			}
+			c0 = KX_rgbaint2uint_new(colour);
+			c1 = KX_rgbaint2uint_new(colour);
+			c2 = KX_rgbaint2uint_new(colour);	
+			if (mface->v4)
+				c3 = KX_rgbaint2uint_new(colour);
+		} break;
+		
+		case 2: // vertex colors
+		{
+			if(tface ) {
+				c0 = KX_rgbaint2uint_new(tface->col[0]);
+				c1 = KX_rgbaint2uint_new(tface->col[1]);
+				c2 = KX_rgbaint2uint_new(tface->col[2]);
+			
+				if (mface->v4) {
+					c3 = KX_rgbaint2uint_new(tface->col[3]);
+				}
+			}else {
+				c0 = KX_rgbaint2uint_new(colour);
+				c1 = KX_rgbaint2uint_new(colour);
+				c2 = KX_rgbaint2uint_new(colour);	
+				if (mface->v4)
+					c3 = KX_rgbaint2uint_new( colour );
+			}
+		} break;
+		
+		case 3: // white
+		{
+			c0 = KX_rgbaint2uint_new(colour);
+			c1 = KX_rgbaint2uint_new(colour);
+			c2 = KX_rgbaint2uint_new(colour);	
+			if (mface->v4)
+				c3 = KX_rgbaint2uint_new(colour);
+		} break;
+	}
+}
+
+// ------------------------------------
+BL_Material* ConvertMaterial(  Mesh* mesh, Material *mat, TFace* tface,  MFace* mface, MCol* mmcol, int lightlayer, Object* blenderobj )
+{
+	//this needs some type of manager
+	BL_Material *material = new BL_Material();
+
+	int numchan =	-1;
+	bool validmat	= (mat!=0);
+	bool validface	= (mesh->tface && tface);
+	
+	short type = 0;
+	if( validmat )
+		type = 1; // material color 
+	else if(mesh->tface && tface)
+		type = 2; // vertex colors
+	
+	material->IdMode = DEFAULT_BLENDER;
+
+	// --------------------------------
+	if(validmat) {
+
+		// use vertex colors by explicitly setting
+		if(mat->mode &MA_VERTEXCOLP) 
+			type = 2;
+
+		// use lighting?
+		material->ras_mode |= ( mat->mode & MA_SHLESS )?0:USE_LIGHT;
+		MTex *mttmp = 0;
+		numchan = getNumTexChannels(mat);
+		int valid_index = 0;
+		
+		// use the face texture if
+		// 1) it is set in the buttons
+		// 2) we have a face texture and a material but no valid texture in slot 1
+		bool facetex = false;
+		if(validface && mat->mode &MA_FACETEXTURE) 
+			facetex = true;
+		if(validface && !mat->mtex[0])
+			facetex = true;
+		if(validface && mat->mtex[0]) {
+			MTex *tmp = mat->mtex[0];
+			if(!tmp->tex || tmp->tex && !tmp->tex->ima )
+				facetex = true;
+		}
+		numchan = numchan>MAXTEX?MAXTEX:numchan;
+	
+		// foreach MTex
+		for(int i=0; i<numchan; i++) {
+			// use face tex
+			if(i==0 && facetex ) {
+				Image*tmp = (Image*)(tface->tpage);
+				if(tmp) {
+					material->img[i] = tmp;
+					material->texname[i] = material->img[i]->id.name;
+					material->flag[i] |= ( tface->transp  &TF_ALPHA	)?USEALPHA:0;
+					material->flag[i] |= ( tface->transp  &TF_ADD	)?CALCALPHA:0;
+					material->ras_mode|= ( tface->transp  &(TF_ADD | TF_ALPHA))?TRANSP:0;
+					if(material->img[i]->flag & IMA_REFLECT)
+						material->mapping[i].mapping |= USEREFL;
+					else
+						material->mapping[i].mapping |= USEUV;
+					if(material->ras_mode & USE_LIGHT)
+						material->ras_mode &= ~USE_LIGHT;
+					if(tface->mode & TF_LIGHT)
+						material->ras_mode |= USE_LIGHT;
+
+					valid_index++;
+				}
+				else {
+					material->img[i] = 0;
+					material->texname[i] = "";
+				}
+				continue;
+			}
+
+			mttmp = getImageFromMaterial( mat, i );
+			if( mttmp ) {
+				if( mttmp->tex ) {
+					if( mttmp->tex->type == TEX_IMAGE ) {
+						material->mtexname[i] = mttmp->tex->id.name;
+						material->img[i] = mttmp->tex->ima;
+						if( material->img[i] ) {
+
+							material->texname[i] = material->img[i]->id.name;
+							material->flag[i] |= ( mttmp->tex->imaflag &TEX_MIPMAP )?MIPMAP:0;
+							// -----------------------
+							if( mttmp->tex->imaflag &TEX_USEALPHA ) {
+								material->flag[i]	|= USEALPHA;
+							}
+							// -----------------------
+							else if( mttmp->tex->imaflag &TEX_CALCALPHA ) {
+								material->flag[i]	|= CALCALPHA;
+							}
+							else if(mttmp->tex->flag &TEX_NEGALPHA) {
+								material->flag[i]	|= USENEGALPHA;
+							}
+
+							material->color_blend[i] = mttmp->colfac;
+							material->flag[i] |= ( mttmp->mapto  & MAP_ALPHA		)?TEXALPHA:0;
+							material->flag[i] |= ( mttmp->texflag& MTEX_NEGATIVE	)?TEXNEG:0;
+
+						}
+					}
+					else if(mttmp->tex->type == TEX_ENVMAP) {
+						if( mttmp->tex->env->stype == ENV_LOAD ) {
+					
+							material->mtexname[i]= mttmp->tex->id.name;
+							material->cubemap[i] = mttmp->tex->env;
+							if(material->cubemap[i]->ima) {
+								material->texname[i] = material->cubemap[i]->ima->id.name;
+								material->mapping[i].mapping |= USEENV;
+							}
+							else {
+								// in the player, we need to split it our self
+								material->cubemap[i]->ima = mttmp->tex->ima;
+								if(material->cubemap[i]->ima) {
+									material->texname[i] = material->cubemap[i]->ima->id.name;
+									material->mapping[i].mapping |= USEENV;
+								}
+								//
+							}
+						}
+					}
+					material->flag[i] |= (mat->ipo!=0)?HASIPO:0;
+					/// --------------------------------
+					// mapping methods
+					material->mapping[i].mapping |= ( mttmp->texco  & TEXCO_REFL	)?USEREFL:0;
+					
+					if(mttmp->texco & TEXCO_OBJECT) {
+						material->mapping[i].mapping |= USEOBJ;
+						if(mttmp->object)
+							material->mapping[i].objconame = mttmp->object->id.name;
+					}
+					else if(mttmp->texco &TEXCO_REFL)
+						material->mapping[i].mapping |= USEREFL;
+					else if(mttmp->texco &(TEXCO_ORCO|TEXCO_GLOB))
+						material->mapping[i].mapping |= USEORCO;
+					else if(mttmp->texco &TEXCO_UV)
+						material->mapping[i].mapping |= USEUV;
+					else if(mttmp->texco &TEXCO_NORM)
+						material->mapping[i].mapping |= USENORM;
+					else if(mttmp->texco &TEXCO_TANGENT)
+						material->mapping[i].mapping |= USETANG;
+					else
+						material->mapping[i].mapping |= DISABLE;
+					
+					material->mapping[i].scale[0] = mttmp->size[0];
+					material->mapping[i].scale[1] = mttmp->size[1];
+					material->mapping[i].scale[2] = mttmp->size[2];
+					material->mapping[i].offsets[0] = mttmp->ofs[0];
+					material->mapping[i].offsets[1] = mttmp->ofs[1];
+					material->mapping[i].offsets[2] = mttmp->ofs[2];
+
+					material->mapping[i].projplane[0] = mttmp->projx;
+					material->mapping[i].projplane[1] = mttmp->projy;
+					material->mapping[i].projplane[2] = mttmp->projz;
+					/// --------------------------------
+					
+					switch( mttmp->blendtype ) {
+					case MTEX_BLEND:
+						material->blend_mode[i] = BLEND_MIX;
+						break;
+					case MTEX_MUL:
+						material->blend_mode[i] = BLEND_MUL;
+						break;
+					case MTEX_ADD:
+						material->blend_mode[i] = BLEND_ADD;
+						break;
+					case MTEX_SUB:
+						material->blend_mode[i] = BLEND_SUB;
+						break;
+					case MTEX_SCREEN:
+						material->blend_mode[i] = BLEND_SCR;
+						break;
+					}
+					valid_index++;
+				}
+			}
+		}
+		// above one tex the switches here
+		// are not used
+		switch(valid_index) {
+		case 0:
+			material->IdMode = DEFAULT_BLENDER;
+			break;
+		case 1:
+			material->IdMode = ONETEX;
+			break;
+		default:
+			material->IdMode = GREATERTHAN2;
+			break;
+		}
+		material->SetUsers(mat->id.us);
+
+		material->num_enabled = valid_index;
+
+		material->speccolor[0]	= mat->specr;
+		material->speccolor[1]	= mat->specg;
+		material->speccolor[2]	= mat->specb;
+		material->hard			= (float)mat->har/4.0f;
+		material->matcolor[0]	= mat->r;
+		material->matcolor[1]	= mat->g;
+		material->matcolor[2]	= mat->b;
+		material->matcolor[3]	= mat->alpha;
+		material->alpha			= mat->alpha;
+		material->emit			= mat->emit;
+		material->spec_f		= mat->spec;
+		material->ref			= mat->ref;
+		material->amb			= mat->amb;
+
+		// set alpha testing without z-sorting
+		if( ( validface && (!tface->transp)) && mat->mode & MA_ZTRA) {
+			// sets the RAS_IPolyMaterial::m_flag |RAS_FORCEALPHA
+			// this is so we don't have the overhead of the z-sorting code
+			material->ras_mode|=ALPHA_TEST;
+		}
+		else{
+			// use regular z-sorting
+			material->ras_mode |= ((mat->mode & MA_ZTRA) != 0)?ZSORT:0;
+		}
+		material->ras_mode |= ((mat->mode & MA_WIRE) != 0)?WIRE:0;
+	}
+	else {
+		int valid = 0;
+		// check for tface tex to fallback on
+		if( validface ){
+
+			// no light bugfix
+			if(tface->mode) material->ras_mode |= USE_LIGHT;
+
+			material->img[0] = (Image*)(tface->tpage);
+			// ------------------------
+			if(material->img[0]) {
+				material->texname[0] = material->img[0]->id.name;
+				material->mapping[0].mapping |= ( (material->img[0]->flag & IMA_REFLECT)!=0 )?USEREFL:0;
+				material->flag[0] |= ( tface->transp  &TF_ALPHA	)?USEALPHA:0;
+				material->flag[0] |= ( tface->transp  &TF_ADD	)?CALCALPHA:0;
+				material->ras_mode|= ( tface->transp  & (TF_ADD|TF_ALPHA))?TRANSP:0;
+				valid++;
+			}
+		}
+		material->SetUsers(-1);
+		material->num_enabled	= valid;
+		material->IdMode		= TEXFACE;
+		material->speccolor[0]	= 1.f;
+		material->speccolor[1]	= 1.f;
+		material->speccolor[2]	= 1.f;
+		material->hard			= 35.f;
+		material->matcolor[0]	= 0.5f;
+		material->matcolor[1]	= 0.5f;
+		material->matcolor[2]	= 0.5f;
+		material->spec_f		= 0.5f;
+		material->ref			= 0.8f;
+	}
+	MT_Point2 uv[4];
+
+	if( validface ) {
+
+		material->ras_mode |= !( 
+			(tface->flag & TF_HIDE)	||
+			(tface->mode & TF_INVISIBLE)
+			)?POLY_VIS:0;
+
+		material->ras_mode |= ( (tface->mode & TF_DYNAMIC)!= 0 )?COLLIDER:0;
+		material->transp = tface->transp;
+		
+		if(tface->transp)
+			material->ras_mode |= TRANSP;
+
+		material->tile	= tface->tile;
+		material->mode	= tface->mode;
+			
+		uv[0]	= MT_Point2(tface->uv[0]);
+		uv[1]	= MT_Point2(tface->uv[1]);
+		uv[2]	= MT_Point2(tface->uv[2]);
+
+		if (mface->v4) 
+			uv[3]	= MT_Point2(tface->uv[3]);
+	} 
+	else {
+		// nothing at all
+		material->ras_mode |= (COLLIDER|POLY_VIS| (validmat?0:USE_LIGHT));
+		material->mode		= default_face_mode;	
+		material->transp	= TF_SOLID;
+		material->tile		= 0;
+	}
+	unsigned int rgb[4];
+	GetRGB(type,mface,mmcol,mat,tface,rgb[0],rgb[1],rgb[2], rgb[3]);
+	material->SetConversionRGB(rgb);
+	material->SetConversionUV(uv);
+
+	material->ras_mode |= (mface->v4==0)?TRIANGLE:0;
+	if(validmat)
+		material->matname	=(mat->id.name);
+
+	material->tface		= tface;
+	material->material	= mat;
+	return material;
+}
+
+
+static void BL_ComputeTriTangentSpace(const MT_Vector3 &v1, const MT_Vector3 &v2, const MT_Vector3 &v3, 
+	const MT_Vector2 &uv1, const MT_Vector2 &uv2, const MT_Vector2 &uv3, 
+	MFace* mface, MT_Vector3 *tan1, MT_Vector3 *tan2)
+{
+		MT_Vector3 dx1(v2 - v1), dx2(v3 - v1);
+		MT_Vector2 duv1(uv2 - uv1), duv2(uv3 - uv1);
+		
+		MT_Scalar r = 1.0 / (duv1.x() * duv2.y() - duv2.x() * duv1.y());
+		duv1 *= r;
+		duv2 *= r;
+		MT_Vector3 sdir(duv2.y() * dx1 - duv1.y() * dx2);
+		MT_Vector3 tdir(duv1.x() * dx2 - duv2.x() * dx1);
+		
+		tan1[mface->v1] += sdir;
+		tan1[mface->v2] += sdir;
+		tan1[mface->v3] += sdir;
+		
+		tan2[mface->v1] += tdir;
+		tan2[mface->v2] += tdir;
+		tan2[mface->v3] += tdir;
+}
+
+static MT_Vector4*  BL_ComputeMeshTangentSpace(Mesh* mesh)
+{
+	MFace* mface = static_cast<MFace*>(mesh->mface);
+	TFace* tface = static_cast<TFace*>(mesh->tface);
+
+	MT_Vector3 *tan1 = new MT_Vector3[mesh->totvert];
+	MT_Vector3 *tan2 = new MT_Vector3[mesh->totvert];
+	
+	unsigned int v;
+	for (v = 0; v < mesh->totvert; v++)
+	{
+		tan1[v] = MT_Vector3(0.0, 0.0, 0.0);
+		tan2[v] = MT_Vector3(0.0, 0.0, 0.0);
+	}
+	
+	for (unsigned int p = 0; p < mesh->totface; p++, mface++, tface++)
+	{
+		MT_Vector3 	v1(mesh->mvert[mface->v1].co),
+				v2(mesh->mvert[mface->v2].co),
+				v3(mesh->mvert[mface->v3].co);
+				
+		MT_Vector2	uv1(tface->uv[0]),
+				uv2(tface->uv[1]),
+				uv3(tface->uv[2]);
+				
+		BL_ComputeTriTangentSpace(v1, v2, v3, uv1, uv2, uv3, mface, tan1, tan2);
+		if (mface->v4)
+		{
+			MT_Vector3 v4(mesh->mvert[mface->v4].co);
+			MT_Vector2 uv4(tface->uv[3]);
+			
+			BL_ComputeTriTangentSpace(v1, v3, v4, uv1, uv3, uv4, mface, tan1, tan2);
+		}
+	}
+	
+	MT_Vector4 *tangent = new MT_Vector4[mesh->totvert];
+	for (v = 0; v < mesh->totvert; v++)
+	{
+		const MT_Vector3 no(mesh->mvert[v].no[0]/32767.0, 
+					mesh->mvert[v].no[1]/32767.0, 
+					mesh->mvert[v].no[2]/32767.0);
+		// Gram-Schmidt orthogonalize
+		MT_Vector3 t(tan1[v] - no.cross(no.cross(tan1[v])));
+		if (!MT_fuzzyZero(t))
+			t /= t.length();
+
+		tangent[v].x() = t.x();
+		tangent[v].y() = t.y();
+		tangent[v].z() = t.z();
+		// Calculate handedness
+		tangent[v].w() = no.dot(tan1[v].cross(tan2[v])) < 0.0 ? -1.0 : 1.0;
+	}
+	
+	delete [] tan1;
+	delete [] tan2;
+	
+	return tangent;
+}
+
 RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools* rendertools, KX_Scene* scene, KX_BlenderSceneConverter *converter)
 {
 	RAS_MeshObject *meshobj;
@@ -237,6 +708,9 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 	else {
 		meshobj = new RAS_MeshObject(lightlayer);
 	}
+	MT_Vector4 *tangent = 0;
+	if (tface)
+		tangent = BL_ComputeMeshTangentSpace(mesh);
 	
 	meshobj->SetName(mesh->id.name);
 	
@@ -261,7 +735,11 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 					pt1(mesh->mvert[mface->v2].co),
 					pt2(mesh->mvert[mface->v3].co),
 					pt3(0.0, 0.0, 0.0);
-			
+			MT_Vector4	tan0(0.0, 0.0, 0.0, 0.0),
+					tan1(0.0, 0.0, 0.0, 0.0),
+					tan2(0.0, 0.0, 0.0, 0.0),
+					tan3(0.0, 0.0, 0.0, 0.0);
+
 			no0 /= 32767.0;
 			no1 /= 32767.0;
 			no2 /= 32767.0;
@@ -283,127 +761,170 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 			}
 		
 			{
-				Image* bima = ((mesh->tface && tface) ? (Image*) tface->tpage : NULL);
-	
-				STR_String imastr = 
-					((mesh->tface && tface) ? 
-					(bima? (bima)->id.name : "" ) : "" );
-		
-				char transp=0;
-				short mode=0, tile=0;
-				int	tilexrep=4,tileyrep = 4;
-				
-				if (bima)
-				{
-					tilexrep = bima->xrep;
-					tileyrep = bima->yrep;
-			
-				}
-	
-				Material* ma = give_current_material(blenderobj, 1 /* mface->mat_nr */);
-				//const char* matnameptr = (ma ? ma->id.name : ""); /*unused*/
-				
+				Material* ma = 0;
 				bool polyvisible = true;
-				if (mesh->tface && tface)
-				{
-					// Use texface colors if available
-					//TF_DYNAMIC means the polygon is a collision face
-					collider = (tface->mode & TF_DYNAMIC != 0);
-					transp = tface->transp;
-					tile = tface->tile;
-					mode = tface->mode;
+				RAS_IPolyMaterial* polymat = NULL;
+
+				if(converter->GetMaterials()) 
+				{	
+					if(mesh->totcol > 1)
+						ma = mesh->mat[mface->mat_nr];
+					else 
+						ma = give_current_material(blenderobj, 1);
+
+					BL_Material *bl_mat = ConvertMaterial(mesh, ma, tface, mface, mmcol, lightlayer, blenderobj);
+					// set the index were dealing with
+					bl_mat->material_index =  (int)mface->mat_nr;
+
+					polyvisible = ((bl_mat->ras_mode & POLY_VIS)!=0);
+					collider = ((bl_mat->ras_mode & COLLIDER)!=0);
 					
-					polyvisible = !((tface->flag & TF_HIDE)||(tface->mode & TF_INVISIBLE));
+					polymat = new KX_BlenderMaterial(scene, bl_mat, skinMesh, lightlayer, blenderobj );
+					converter->RegisterBlenderMaterial(bl_mat);
 					
-					uv0 = MT_Point2(tface->uv[0]);
-					uv1 = MT_Point2(tface->uv[1]);
-					uv2 = MT_Point2(tface->uv[2]);
-					rgb0 = KX_rgbaint2uint_new(tface->col[0]);
-					rgb1 = KX_rgbaint2uint_new(tface->col[1]);
-					rgb2 = KX_rgbaint2uint_new(tface->col[2]);
-	
-					if (mface->v4)
-					{
-						uv3 = MT_Point2(tface->uv[3]);
-						rgb3 = KX_rgbaint2uint_new(tface->col[3]);
-					} 
-				} 
+					unsigned int rgb[4];
+					bl_mat->GetConversionRGB(rgb);
+					rgb0 = rgb[0];
+					rgb1 = rgb[1];
+					rgb2 = rgb[2];
+					rgb3 = rgb[3];
+					MT_Point2 uv[4];
+					bl_mat->GetConversionUV(uv);
+					uv0 = uv[0];
+					uv1 = uv[1];
+					uv2 = uv[2];
+					uv3 = uv[3];
+					if(tangent){
+						tan0 = tangent[mface->v1];
+						tan1 = tangent[mface->v2];
+						tan2 = tangent[mface->v3];
+						if (mface->v4)
+							tan3 = tangent[mface->v4];
+					}
+					// this is needed to free up memory afterwards
+					converter->RegisterPolyMaterial(polymat);
+				}
 				else
 				{
-					//
-					if (mmcol)
+					ma = give_current_material(blenderobj, 1);
+
+					Image* bima = ((mesh->tface && tface) ? (Image*) tface->tpage : NULL);
+		
+					STR_String imastr = 
+						((mesh->tface && tface) ? 
+						(bima? (bima)->id.name : "" ) : "" );
+			
+					char transp=0;
+					short mode=0, tile=0;
+					int	tilexrep=4,tileyrep = 4;
+					
+					if (bima)
 					{
-						// Use vertex colours
-						rgb0 = KX_Mcol2uint_new(mmcol[0]);
-						rgb1 = KX_Mcol2uint_new(mmcol[1]);
-						rgb2 = KX_Mcol2uint_new(mmcol[2]);
+						tilexrep = bima->xrep;
+						tileyrep = bima->yrep;
+				
+					}
+
+					if (mesh->tface && tface)
+					{
+						// Use texface colors if available
+						//TF_DYNAMIC means the polygon is a collision face
+						collider = ((tface->mode & TF_DYNAMIC) != 0);
+						transp = tface->transp;
+						tile = tface->tile;
+						mode = tface->mode;
 						
+						polyvisible = !((tface->flag & TF_HIDE)||(tface->mode & TF_INVISIBLE));
 						
+						uv0 = MT_Point2(tface->uv[0]);
+						uv1 = MT_Point2(tface->uv[1]);
+						uv2 = MT_Point2(tface->uv[2]);
+						rgb0 = KX_rgbaint2uint_new(tface->col[0]);
+						rgb1 = KX_rgbaint2uint_new(tface->col[1]);
+						rgb2 = KX_rgbaint2uint_new(tface->col[2]);
+		
 						if (mface->v4)
 						{
-							rgb3 = KX_Mcol2uint_new(mmcol[3]);
-							
-						}
-					
-						mmcol += 4;
-					}
+							uv3 = MT_Point2(tface->uv[3]);
+							rgb3 = KX_rgbaint2uint_new(tface->col[3]);
+						} 
+					} 
 					else
 					{
-						// If there are no vertex colors OR texfaces,
-						// Initialize face to white and set COLLSION true and everything else FALSE
-						unsigned int colour = 0xFFFFFFFFL;
-						mode = default_face_mode;	
-						transp = TF_SOLID;
-						tile = 0;
-						if (ma)
+						//
+						if (mmcol)
 						{
-							// If we have a material, take the default colour from the material.
-							union
+							// Use vertex colours
+							rgb0 = KX_Mcol2uint_new(mmcol[0]);
+							rgb1 = KX_Mcol2uint_new(mmcol[1]);
+							rgb2 = KX_Mcol2uint_new(mmcol[2]);
+							
+							
+							if (mface->v4)
 							{
-								unsigned char cp[4];
-								unsigned int integer;
-							} col_converter;
+								rgb3 = KX_Mcol2uint_new(mmcol[3]);
+								
+							}
+						
+							mmcol += 4;
+						}
+						else
+						{
+							// If there are no vertex colors OR texfaces,
+							// Initialize face to white and set COLLSION true and everything else FALSE
+							unsigned int colour = 0xFFFFFFFFL;
+							mode = default_face_mode;	
+							transp = TF_SOLID;
+							tile = 0;
+							if (ma)
+							{
+								// If we have a material, take the default colour from the material.
+								union
+								{
+									unsigned char cp[4];
+									unsigned int integer;
+								} col_converter;
+								
+								col_converter.cp[3] = (unsigned char) (ma->r*255.0);
+								col_converter.cp[2] = (unsigned char) (ma->g*255.0);
+								col_converter.cp[1] = (unsigned char) (ma->b*255.0);
+								col_converter.cp[0] = (unsigned char) (ma->alpha*255.0);
+								
+								colour = col_converter.integer;
+							}
 							
-							col_converter.cp[3] = (unsigned char) (ma->r*255.0);
-							col_converter.cp[2] = (unsigned char) (ma->g*255.0);
-							col_converter.cp[1] = (unsigned char) (ma->b*255.0);
-							col_converter.cp[0] = (unsigned char) (ma->alpha*255.0);
+							rgb0 = KX_rgbaint2uint_new(colour);
+							rgb1 = KX_rgbaint2uint_new(colour);
+							rgb2 = KX_rgbaint2uint_new(colour);	
 							
-							colour = col_converter.integer;
+							if (mface->v4)
+								rgb3 = KX_rgbaint2uint_new(colour);
 						}
-						
-						rgb0 = KX_rgbaint2uint_new(colour);
-						rgb1 = KX_rgbaint2uint_new(colour);
-						rgb2 = KX_rgbaint2uint_new(colour);	
-						
-						if (mface->v4)
-							rgb3 = KX_rgbaint2uint_new(colour);
 					}
-				}
+						
 					
-				
-				bool istriangle = (mface->v4==0);
-				bool zsort = ma?(ma->mode & MA_ZTRA) != 0:false;
-				
-				RAS_IPolyMaterial* polymat = new KX_PolygonMaterial(imastr, ma,
-					tile, tilexrep, tileyrep, 
-					mode, transp, zsort, lightlayer, istriangle, blenderobj, tface);
-	
-				if (ma)
-				{
-					polymat->m_specular = MT_Vector3(ma->specr, ma->specg, ma->specb)*ma->spec;
-					polymat->m_shininess = (float)ma->har/4.0; // 0 < ma->har <= 512
-					polymat->m_diffuse = MT_Vector3(ma->r, ma->g, ma->b)*(ma->emit + ma->ref);
+					bool istriangle = (mface->v4==0);
+					bool zsort = ma?(ma->mode & MA_ZTRA) != 0:false;
+					
+					polymat = new KX_PolygonMaterial(imastr, ma,
+						tile, tilexrep, tileyrep, 
+						mode, transp, zsort, lightlayer, istriangle, blenderobj, tface);
+		
+					if (ma)
+					{
+						polymat->m_specular = MT_Vector3(ma->specr, ma->specg, ma->specb)*ma->spec;
+						polymat->m_shininess = (float)ma->har/4.0; // 0 < ma->har <= 512
+						polymat->m_diffuse = MT_Vector3(ma->r, ma->g, ma->b)*(ma->emit + ma->ref);
+
+					} else
+					{
+						polymat->m_specular = MT_Vector3(0.0f,0.0f,0.0f);
+						polymat->m_shininess = 35.0;
+					}
+					// this is needed to free up memory afterwards
+					converter->RegisterPolyMaterial(polymat);
 
-				} else
-				{
-					polymat->m_specular = MT_Vector3(0.0f,0.0f,0.0f);
-					polymat->m_shininess = 35.0;
 				}
-
-			
-				// this is needed to free up memory afterwards
-				converter->RegisterPolyMaterial(polymat);
 	
 				RAS_MaterialBucket* bucket = scene->FindBucket(polymat);
 							 
@@ -425,19 +946,19 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 					d3=((BL_SkinMeshObject*)meshobj)->FindOrAddDeform(vtxarray, mface->v3, &mesh->dvert[mface->v3], polymat);
 					if (nverts==4)
 						d4=((BL_SkinMeshObject*)meshobj)->FindOrAddDeform(vtxarray, mface->v4, &mesh->dvert[mface->v4], polymat);
-					poly->SetVertex(0,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt0,uv0,rgb0,no0,d1,flat, polymat));
-					poly->SetVertex(1,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt1,uv1,rgb1,no1,d2,flat, polymat));
-					poly->SetVertex(2,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt2,uv2,rgb2,no2,d3,flat, polymat));
+					poly->SetVertex(0,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt0,uv0,tan0,rgb0,no0,d1,flat, polymat));
+					poly->SetVertex(1,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt1,uv1,tan1,rgb1,no1,d2,flat, polymat));
+					poly->SetVertex(2,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt2,uv2,tan2,rgb2,no2,d3,flat, polymat));
 					if (nverts==4)
-						poly->SetVertex(3,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt3,uv3,rgb3,no3,d4, flat,polymat));
+						poly->SetVertex(3,((BL_SkinMeshObject*)meshobj)->FindOrAddVertex(vtxarray,pt3,uv3,tan3,rgb3,no3,d4, flat,polymat));
 				}
 				else
 				{
-					poly->SetVertex(0,meshobj->FindOrAddVertex(vtxarray,pt0,uv0,rgb0,no0,polymat,mface->v1));
-					poly->SetVertex(1,meshobj->FindOrAddVertex(vtxarray,pt1,uv1,rgb1,no1,polymat,mface->v2));
-					poly->SetVertex(2,meshobj->FindOrAddVertex(vtxarray,pt2,uv2,rgb2,no2,polymat,mface->v3));
+					poly->SetVertex(0,meshobj->FindOrAddVertex(vtxarray,pt0,uv0,tan0,rgb0,no0,polymat,mface->v1));
+					poly->SetVertex(1,meshobj->FindOrAddVertex(vtxarray,pt1,uv1,tan1,rgb1,no1,polymat,mface->v2));
+					poly->SetVertex(2,meshobj->FindOrAddVertex(vtxarray,pt2,uv2,tan2,rgb2,no2,polymat,mface->v3));
 					if (nverts==4)
-						poly->SetVertex(3,meshobj->FindOrAddVertex(vtxarray,pt3,uv3,rgb3,no3,polymat,mface->v4));
+						poly->SetVertex(3,meshobj->FindOrAddVertex(vtxarray,pt3,uv3,tan3,rgb3,no3,polymat,mface->v4));
 				}
 				meshobj->AddPolygon(poly);
 				if (poly->IsCollider())
@@ -458,7 +979,7 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 					}
 				}
 				
-				poly->SetVisibleWireframeEdges(mface->edcode);
+//				poly->SetVisibleWireframeEdges(mface->edcode);
 				poly->SetCollider(collider);
 			}
 		}
@@ -466,19 +987,23 @@ RAS_MeshObject* BL_ConvertMesh(Mesh* mesh, Object* blenderobj, RAS_IRenderTools*
 			tface++;
 	}
 	meshobj->UpdateMaterialList();
-	
+
+
+
+	// -----------------------------------
+	// pre calculate texture generation
+	for(RAS_MaterialBucket::Set::iterator mit = meshobj->GetFirstMaterial();
+		mit != meshobj->GetLastMaterial(); ++ mit) {
+		(*mit)->GetPolyMaterial()->OnConstruction();
+	}
+
+	if(tangent)
+		delete [] tangent;
+
+
 	return meshobj;
 }
 
-static PHY_MaterialProps g_materialProps = {
-	1.0,    // restitution
-	2.0,    // friction 
-	0.0,    // fh spring constant
-	0.0,    // fh damping
-	0.0,    // fh distance
-	false   // sliding material?
-};
-
 	
 	
 static PHY_MaterialProps *CreateMaterialFromBlenderObject(struct Object* blenderobject,
@@ -502,7 +1027,14 @@ static PHY_MaterialProps *CreateMaterialFromBlenderObject(struct Object* blender
 		materialProps->m_fh_normal = (blendermat->dynamode & MA_FH_NOR) != 0;
 	}
 	else {
-		*materialProps = g_materialProps;
+		//give some defaults
+		materialProps->m_restitution = 0.f;
+		materialProps->m_friction = 0.5;
+		materialProps->m_fh_spring = 0.f;
+		materialProps->m_fh_damping = 0.f;
+		materialProps->m_fh_distance = 0.f;
+		materialProps->m_fh_normal = false;
+
 	}
 	
 	return materialProps;
@@ -734,10 +1266,12 @@ void BL_CreatePhysicsObjectNew(KX_GameObject* gameobj,
 		objprop.m_dyna = (blenderobject->gameflag & OB_DYNAMIC) != 0;
 		objprop.m_angular_rigidbody = (blenderobject->gameflag & OB_RIGID_BODY) != 0;
 		objprop.m_ghost = (blenderobject->gameflag & OB_GHOST) != 0;
+		objprop.m_disableSleeping = (blenderobject->gameflag & OB_COLLISION_RESPONSE) != 0;//abuse the OB_COLLISION_RESPONSE flag
 	} else {
 		objprop.m_dyna = false;
 		objprop.m_angular_rigidbody = false;
 		objprop.m_ghost = false;
+		objprop.m_disableSleeping = false;
 	}
 	//mmm, for now, taks this for the size of the dynamicobject
 	// Blender uses inertia for radius of dynamic object
@@ -816,6 +1350,12 @@ void BL_CreatePhysicsObjectNew(KX_GameObject* gameobj,
 	
 	switch (physics_engine)
 	{
+#ifdef USE_BULLET
+		case UseBullet:
+			KX_ConvertBulletObject(gameobj, meshobj, kxscene, shapeprops, smmaterial, &objprop);
+			break;
+
+#endif
 #ifdef USE_SUMO_SOLID
 		case UseSumo:
 			KX_ConvertSumoObject(gameobj, meshobj, kxscene, shapeprops, smmaterial, &objprop);
@@ -955,11 +1495,11 @@ static KX_GameObject *gameobject_from_blenderobject(
 	
 		//	If this is a skin object, make Skin Controller
 		if (ob->parent && ob->parent->type == OB_ARMATURE && ob->partype==PARSKEL && ((Mesh*)ob->data)->dvert){
-			BL_SkinDeformer *dcont = new BL_SkinDeformer(ob, (BL_SkinMeshObject*)meshobj);				
+			BL_SkinDeformer *dcont = new BL_SkinDeformer(ob, (BL_SkinMeshObject*)meshobj );				
 			((BL_DeformableGameObject*)gameobj)->m_pDeformer = dcont;
 		}
 		else if (((Mesh*)ob->data)->dvert){
-			BL_MeshDeformer *dcont = new BL_MeshDeformer(ob, (BL_SkinMeshObject*)meshobj);
+			BL_MeshDeformer *dcont = new BL_MeshDeformer(ob, (BL_SkinMeshObject*)meshobj );
 			((BL_DeformableGameObject*)gameobj)->m_pDeformer = dcont;
 		}
 		
@@ -974,10 +1514,11 @@ static KX_GameObject *gameobject_from_blenderobject(
 	
 	case OB_ARMATURE:
 	{
-		gameobj = new BL_ArmatureObject (kxscene, KX_Scene::m_callbacks,
-			get_armature(ob),
-			ob->pose);
-	
+		gameobj = new BL_ArmatureObject(
+			kxscene,
+			KX_Scene::m_callbacks,
+			ob // handle
+		);
 		/* Get the current pose from the armature object and apply it as the rest pose */
 		break;
 	}
@@ -1012,6 +1553,7 @@ static struct Scene *GetSceneForName(struct Main *maggie, const STR_String& scen
 	return (Scene*) maggie->scene.first;
 }
 
+
 // convert blender objects into ketsji gameobjects
 void BL_ConvertBlenderObjects(struct Main* maggie,
 							  const STR_String& scenename,
@@ -1026,6 +1568,7 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 							  bool alwaysUseExpandFraming
 							  )
 {	
+
 	Scene *blenderscene = GetSceneForName(maggie, scenename);
 
 	// Get the frame settings of the canvas.
@@ -1096,7 +1639,7 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 	{
 		logicmgr->RegisterActionName(curAct->id.name, curAct);
 	}
-	
+
 	SetDefaultFaceType(blenderscene);
 	
 	Base *base = static_cast<Base*>(blenderscene->base.first);
@@ -1134,6 +1677,9 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 			gameobj->NodeUpdateGS(0,true);
 			
 			BL_ConvertIpos(blenderobject,gameobj,converter);
+			// TODO: expand to multiple ipos per mesh
+			Material *mat = give_current_material(blenderobject, 1);
+			if(mat) BL_ConvertMaterialIpos(mat, gameobj, converter);	
 	
 			bool isInActiveLayer = (blenderobject->lay & activeLayerBitInfo) !=0;
 	
@@ -1261,7 +1807,6 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 			case PAROBJECT:
 			case PARCURVE:
 			case PARKEY:
-			case PARLIMB:
 			case PARVERT3:
 			default:
 				// unhandled
@@ -1331,7 +1876,9 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 	KX_WorldInfo* worldinfo = new BlenderWorldInfo(blenderscene->world);
 	converter->RegisterWorldInfo(worldinfo);
 	kxscene->SetWorldInfo(worldinfo);
-	
+
+#define CONVERT_LOGIC
+#ifdef CONVERT_LOGIC
 	// convert logic bricks, sensors, controllers and actuators
 	for (i=0;i<logicbrick_conversionlist->GetCount();i++)
 	{
@@ -1354,6 +1901,8 @@ void BL_ConvertBlenderObjects(struct Main* maggie,
 		bool isInActiveLayer = (blenderobj->lay & activeLayerBitInfo)!=0;
 		BL_ConvertSensors(blenderobj,gameobj,logicmgr,kxscene,keydev,executePriority,activeLayerBitInfo,isInActiveLayer,canvas,converter);
 	}
+#endif //CONVERT_LOGIC
+
 	logicbrick_conversionlist->Release();
 	
 	// Calculate the scene btree -
