@@ -1,5 +1,5 @@
 /**
- * $Id: KX_BlenderSceneConverter.h,v 1.7 2005/03/22 13:34:30 sirdude Exp $
+ * $Id: KX_BlenderSceneConverter.h,v 1.11 2006/01/06 03:46:51 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,12 +44,17 @@ class SCA_IController;
 class RAS_MeshObject;
 class RAS_IPolyMaterial;
 class BL_InterpolatorList;
+class BL_Material;
+struct IpoCurve;
+struct Main;
+struct SpaceIpo;
 
 class KX_BlenderSceneConverter : public KX_ISceneConverter
 {
 	vector<KX_WorldInfo*>	m_worldinfos;
 	vector<RAS_IPolyMaterial*> m_polymaterials;
 	vector<RAS_MeshObject*> m_meshobjects;
+	vector<BL_Material *>	m_materials;
 
 	GEN_Map<CHashedPtr,struct Object*> m_map_gameobject_to_blender;
 	GEN_Map<CHashedPtr,KX_GameObject*> m_map_blender_to_gameobject;
@@ -62,14 +67,21 @@ class KX_BlenderSceneConverter : public KX_ISceneConverter
 	
 	GEN_Map<CHashedPtr,BL_InterpolatorList*> m_map_blender_to_gameipolist;
 	
-	struct Main*			m_maggie;
+	Main*					m_maggie;
+	SpaceIpo*				m_sipo;
+
 	STR_String				m_newfilename;
 	class KX_KetsjiEngine*	m_ketsjiEngine;
 	bool					m_alwaysUseExpandFraming;
-	
+	bool					m_usemat;
+
+	void localDel_ipoCurve ( IpoCurve * icu ,struct SpaceIpo*	sipo);
+	struct Ipo* findIpoForName(char* objName);
+
 public:
 	KX_BlenderSceneConverter(
-		struct Main* maggie,
+		Main* maggie,
+		SpaceIpo *sipo,
 		class KX_KetsjiEngine* engine
 	);
 
@@ -104,6 +116,8 @@ public:
 //	DT_ShapeHandle FindSumoShape(RAS_MeshObject *for_gamemesh);
 
 	void RegisterPolyMaterial(RAS_IPolyMaterial *polymat);
+
+	void RegisterBlenderMaterial(BL_Material *mat);
 	
 	void RegisterInterpolatorList(BL_InterpolatorList *ipoList, struct Ipo *for_ipo);
 	BL_InterpolatorList *FindInterpolatorList(struct Ipo *for_ipo);
@@ -115,6 +129,17 @@ public:
 	SCA_IController *FindGameController(struct bController *for_controller);
 
 	void RegisterWorldInfo(KX_WorldInfo *worldinfo);
+
+	virtual void	ResetPhysicsObjectsAnimationIpo();
+
+	///this generates ipo curves for position, rotation, allowing to use game physics in animation
+	virtual void	WritePhysicsObjectToAnimationIpo(int frameNumber);
+	virtual void	TestHandlesPhysicsObjectToAnimationIpo();
+
+	// use blender materials
+	virtual void SetMaterials(bool val);
+	virtual bool GetMaterials();
+
 };
 
 #endif //__KX_BLENDERSCENECONVERTER_H
