@@ -1,5 +1,5 @@
 /**
- * $Id: PyObjectPlus.cpp,v 1.4 2004/03/22 22:01:29 jesterking Exp $
+ * $Id: PyObjectPlus.cpp,v 1.10 2006/01/30 20:33:58 erwin Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -49,7 +49,7 @@
  * http://www.python.org/doc/PyCPP.html
  *
 ------------------------------*/
-#include <assert.h>
+#include <MT_assert.h>
 #include "stdlib.h"
 #include "PyObjectPlus.h"
 #include "STR_String.h"
@@ -77,9 +77,18 @@ PyTypeObject PyObjectPlus::Type = {
 	0,				/*tp_call */
 };
 
+PyObjectPlus::~PyObjectPlus()
+{
+	if (ob_refcnt)
+	{
+		_Py_ForgetReference(this);
+	}
+//	assert(ob_refcnt==0);
+}
+
 PyObjectPlus::PyObjectPlus(PyTypeObject *T) 				// constructor
 {
-	assert(T != NULL);
+	MT_assert(T != NULL);
 	this->ob_type = T; 
 	_Py_NewReference(this);
 };
@@ -100,15 +109,23 @@ PyParentObject PyObjectPlus::Parents[] = {&PyObjectPlus::Type, NULL};
 /*------------------------------
  * PyObjectPlus attributes	-- attributes
 ------------------------------*/
-PyObject *PyObjectPlus::_getattr(char *attr)
+PyObject *PyObjectPlus::_getattr(const STR_String& attr)
 {
+	if (attr == "__doc__" && GetType()->tp_doc)
+		return PyString_FromString(GetType()->tp_doc);
+
   //if (streq(attr, "type"))
   //  return Py_BuildValue("s", (*(GetParents()))->tp_name);
 
-  return Py_FindMethod(Methods, this, attr);    
+  return Py_FindMethod(Methods, this, const_cast<char *>(attr.ReadPtr()));
 }
 
-int PyObjectPlus::_setattr(char *attr, PyObject *value)
+int PyObjectPlus::_delattr(const STR_String& attr)
+{
+	return 1;
+}
+
+int PyObjectPlus::_setattr(const STR_String& attr, PyObject *value)
 {
 	//return PyObject::_setattr(attr,value);
 	//cerr << "Unknown attribute" << endl;
