@@ -1,7 +1,7 @@
 /**
  * Execute Python scripts
  *
- * $Id: SCA_PythonController.cpp,v 1.10 2005/03/14 20:10:22 lukep Exp $
+ * $Id: SCA_PythonController.cpp,v 1.14 2006/05/14 03:39:07 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -56,6 +56,20 @@ SCA_PythonController::SCA_PythonController(SCA_IObject* gameobj,
 {
 }
 
+/*
+//debugging
+CValue*		SCA_PythonController::AddRef()
+{
+	//printf("AddRef refcount = %i\n",GetRefCount());
+	return CValue::AddRef();
+}
+int			SCA_PythonController::Release()
+{
+	//printf("Release refcount = %i\n",GetRefCount());
+	return CValue::Release();
+}
+*/
+
 
 
 SCA_PythonController::~SCA_PythonController()
@@ -144,8 +158,7 @@ PyObject* SCA_PythonController::sPyGetCurrentController(PyObject* self,
 #if 0
 static char* sPyAddActiveActuator__doc__;
 #endif
-	
-  
+
 PyObject* SCA_PythonController::sPyAddActiveActuator(
 	  
 		PyObject* self, 
@@ -156,18 +169,27 @@ PyObject* SCA_PythonController::sPyAddActiveActuator(
 	PyObject* ob1;
 	int activate;
 	if (!PyArg_ParseTuple(args, "Oi", &ob1,&activate))
-	{
 		return NULL;
-		
-	}
+
 	// for safety, todo: only allow for registered actuators (pointertable)
 	// we don't want to crash gameengine/blender by python scripts
+	std::vector<SCA_IActuator*> lacts =  m_sCurrentController->GetLinkedActuators();
 	
-	CValue* ac = (CValue*)ob1;
-	CValue* boolval = new CBoolValue(activate!=0);
-	m_sCurrentLogicManager->AddActiveActuator((SCA_IActuator*)ac,boolval);
-	boolval->Release();
-	
+	std::vector<SCA_IActuator*>::iterator it;
+	bool found = false;
+	CValue* act = (CValue*)ob1;
+
+	for(it = lacts.begin(); it!= lacts.end(); it++) {
+		if( static_cast<SCA_IActuator*>(act) == (*it) ) {
+			found=true;
+			break;
+		}
+	}
+	if(found){
+		CValue* boolval = new CBoolValue(activate!=0);
+		m_sCurrentLogicManager->AddActiveActuator((SCA_IActuator*)act,boolval);
+		boolval->Release();
+	}
 	Py_INCREF(Py_None);
 	return Py_None;
 }
@@ -268,12 +290,25 @@ void SCA_PythonController::Trigger(SCA_LogicManager* logicmgr)
 		);
 	PyDict_Clear(excdict);
 	Py_DECREF(excdict);*/
-	
+
+
+#if 1
+	PyObject *excdict= PyDict_Copy(m_pythondictionary);
+	PyObject* resultobj = PyEval_EvalCode((PyCodeObject*)m_bytecode,
+		excdict, 
+		excdict
+		);
+	PyDict_Clear(excdict);
+	Py_DECREF(excdict);
+#else
+
 	PyObject* resultobj = PyEval_EvalCode((PyCodeObject*)m_bytecode,
 		m_pythondictionary, 
 		m_pythondictionary
 		);
-	
+
+#endif
+
 	if (resultobj)
 	{
 		Py_DECREF(resultobj);
