@@ -1,7 +1,7 @@
 /**
  * Execute Python scripts
  *
- * $Id: SCA_PythonController.h,v 1.5 2004/03/22 22:01:33 jesterking Exp $
+ * $Id: SCA_PythonController.h,v 1.7 2006/05/14 03:39:07 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -54,6 +54,10 @@ class SCA_PythonController : public SCA_IController
  public: 
 	static SCA_PythonController* m_sCurrentController; // protected !!!
 
+	//for debugging
+	//virtual	CValue*		AddRef();
+	//virtual int			Release();												// Release a reference to this value (when reference count reaches 0, the value is removed from the heap)
+
 	SCA_PythonController(SCA_IObject* gameobj,PyTypeObject* T = &Type);
 	virtual ~SCA_PythonController();
 
@@ -72,7 +76,7 @@ class SCA_PythonController : public SCA_IController
 	static PyObject* sPyAddActiveActuator(PyObject* self, 
 										  PyObject* args, 
 										  PyObject* kwds);
-	virtual PyObject*  _getattr(char *attr);
+	virtual PyObject* _getattr(const STR_String& attr);
 
 	KX_PYMETHOD_DOC(SCA_PythonController,GetSensors);
 	KX_PYMETHOD_DOC(SCA_PythonController,GetSensor);
