@@ -1,5 +1,5 @@
 /**
- * $Id: GPC_RenderTools.cpp,v 1.13 2005/03/25 10:33:36 kester Exp $
+ * $Id: GPC_RenderTools.cpp,v 1.16 2006/05/13 00:08:14 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,6 +38,7 @@
 #endif 
 
 #ifdef __APPLE__
+#define GL_GLEXT_LEGACY 1
 #include <OpenGL/gl.h>
 #else
 #include <GL/gl.h>
@@ -98,7 +99,7 @@
 #include "KX_RayCast.h"
 #include "KX_IPhysicsController.h"
 #include "PHY_IPhysicsEnvironment.h"
-
+#include "KX_BlenderMaterial.h"
 
 GPC_RenderTools::GPC_RenderTools()
 {
@@ -173,6 +174,7 @@ void GPC_RenderTools::RenderText2D(RAS_TEXT_RENDER_MODE mode,
 	int lines;
 	char* s = tmpstr.Ptr();
 	char* p;
+	
 
 	// Save and change OpenGL settings
 	int texture2D;
@@ -182,6 +184,11 @@ void GPC_RenderTools::RenderText2D(RAS_TEXT_RENDER_MODE mode,
 	glGetIntegerv(GL_FOG, (GLint*)&fog);
 	glDisable(GL_FOG);
 	
+	int light;
+	glGetIntegerv(GL_LIGHTING, (GLint*)&light);
+	glDisable(GL_LIGHTING);
+
+	
 	// Set up viewing settings
 	glMatrixMode(GL_PROJECTION);
 	glPushMatrix();
@@ -228,6 +235,10 @@ void GPC_RenderTools::RenderText2D(RAS_TEXT_RENDER_MODE mode,
 		glEnable(GL_TEXTURE_2D);
 	else
 		glDisable(GL_TEXTURE_2D);
+	if (light)
+		glEnable(GL_LIGHTING);
+	else
+		glDisable(GL_LIGHTING);
 }
 
 /**
@@ -242,9 +253,17 @@ void GPC_RenderTools::RenderText(
 {
 	STR_String mytext = ((CValue*)m_clientobject)->GetPropertyText("Text");
 	
-	KX_PolygonMaterial* blenderpoly = static_cast<KX_PolygonMaterial*>(polymat);
-	struct TFace* tface = blenderpoly->GetTFace();
-	
+	const unsigned int flag = polymat->GetFlag();
+	struct TFace* tface = 0;
+
+	if(flag & RAS_BLENDERMAT) {
+		KX_BlenderMaterial *bl_mat = static_cast<KX_BlenderMaterial*>(polymat);
+		tface = bl_mat->GetTFace();
+	} else {
+		KX_PolygonMaterial* blenderpoly = static_cast<KX_PolygonMaterial*>(polymat);
+		tface = blenderpoly->GetTFace();
+	}
+		
 	BL_RenderText(mode, mytext, mytext.Length(), tface, v1, v2, v3, v4);
 }
 
