@@ -1,5 +1,5 @@
 /**
- * $Id: GPG_Application.cpp,v 1.3 2002/11/25 15:29:47 mein Exp $
+ * $Id: GPG_Application.cpp,v 1.23 2006/07/19 06:39:11 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,12 +37,22 @@
 
 #ifdef WIN32
 	#pragma warning (disable:4786) // suppress stl-MSVC debug info warning
+	#include <windows.h>
+#endif
+
+#ifdef __APPLE__
+#define GL_GLEXT_LEGACY 1
+#include <OpenGL/gl.h>
+#include <OpenGL/glu.h>
+#else
+#include <GL/gl.h>
+#include <GL/glu.h>
 #endif
 
 #include "GPG_Application.h"
 
 #include <iostream>
-#include <assert.h>
+#include <MT_assert.h>
 
 /**********************************
  * Begin Blender include block
@@ -53,6 +63,8 @@ extern "C"
 #endif  // __cplusplus
 #include "BLI_blenlib.h"
 #include "BLO_readfile.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
 #ifdef __cplusplus
 }
 #endif // __cplusplus
@@ -69,8 +81,12 @@ extern "C"
 #include "SCA_IActuator.h"
 #include "RAS_MeshObject.h"
 #include "RAS_OpenGLRasterizer.h"
+#include "RAS_VAOpenGLRasterizer.h"
+#include "RAS_ListRasterizer.h"
+#include "RAS_GLExtensionManager.h"
 #include "KX_PythonInit.h"
 #include "KX_PyConstraintBinding.h"
+#include "BL_Material.h" // MAXTEX
 
 #include "KX_BlenderSceneConverter.h"
 #include "NG_LoopBackNetworkDeviceInterface.h"
@@ -96,11 +112,27 @@ static void frameTimerProc(GHOST_ITimerTask* task, GHOST_TUns64 time);
 static GHOST_ISystem* fSystem = 0;
 static const int kTimerFreq = 10;
 
-GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR_String startSceneName)
-	: m_maggie(maggie), m_startSceneName(startSceneName), m_exitRequested(0),
-	  m_system(system), m_mainWindow(0), m_frameTimer(0), m_cursor(GHOST_kStandardCursorFirstCursor),
-	  m_mouse(0), m_keyboard(0), m_rasterizer(0), m_canvas(0), m_rendertools(0), m_kxsystem(0), m_networkdevice(0), m_audiodevice(0), m_sceneconverter(0),
-	  m_engineInitialized(0), m_engineRunning(0), m_ketsjiengine(0)
+GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main* maggie, STR_String startSceneName)
+	: m_startSceneName(startSceneName), 
+	  m_maggie(maggie),
+	  m_exitRequested(0),
+	  m_system(system), 
+	  m_mainWindow(0), 
+	  m_frameTimer(0), 
+	  m_cursor(GHOST_kStandardCursorFirstCursor),
+	  m_engineInitialized(0), 
+	  m_engineRunning(0), 
+	  m_ketsjiengine(0),
+	  m_kxsystem(0), 
+	  m_keyboard(0), 
+	  m_mouse(0), 
+	  m_canvas(0), 
+	  m_rendertools(0), 
+	  m_rasterizer(0), 
+	  m_sceneconverter(0),
+	  m_networkdevice(0), 
+	  m_audiodevice(0),
+	  m_blendermat(0)
 {
 	fSystem = system;
 }
@@ -115,12 +147,13 @@ GPG_Application::~GPG_Application(void)
 
 
 
-bool GPG_Application::SetGameEngineData(struct Main *maggie, STR_String startSceneName)
+bool GPG_Application::SetGameEngineData(struct Main* maggie, STR_String startSceneName)
 {
 	bool result = false;
 
 	if (maggie != NULL && startSceneName != "")
 	{
+		G.scene = (Scene*)maggie->scene.first;
 		m_maggie = maggie;
 		m_startSceneName = startSceneName;
 		result = true;
@@ -130,6 +163,138 @@ bool GPG_Application::SetGameEngineData(struct Main *maggie, STR_String startSce
 }
 
 
+#ifdef WIN32
+#define SCR_SAVE_MOUSE_MOVE_THRESHOLD 15
+
+static HWND found_ghost_window_hwnd;
+static GHOST_IWindow* ghost_window_to_find;
+static WNDPROC ghost_wnd_proc;
+static POINT scr_save_mouse_pos;
+
+static LRESULT CALLBACK screenSaverWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
+{
+	BOOL close = FALSE;
+	switch (uMsg)
+	{
+		case WM_MOUSEMOVE:
+		{ 
+			POINT pt; 
+			GetCursorPos(&pt);
+			LONG dx = scr_save_mouse_pos.x - pt.x;
+			LONG dy = scr_save_mouse_pos.y - pt.y;
+			if (abs(dx) > SCR_SAVE_MOUSE_MOVE_THRESHOLD
+			    || abs(dy) > SCR_SAVE_MOUSE_MOVE_THRESHOLD)
+			{
+				close = TRUE;
+			}
+			scr_save_mouse_pos = pt;
+			break;
+		}
+		case WM_LBUTTONDOWN: 
+		case WM_MBUTTONDOWN: 
+		case WM_RBUTTONDOWN: 
+		case WM_KEYDOWN:
+			close = TRUE;
+	}
+	if (close)
+		PostMessage(hwnd,WM_CLOSE,0,0);
+	return CallWindowProc(ghost_wnd_proc, hwnd, uMsg, wParam, lParam);
+}
+
+BOOL CALLBACK findGhostWindowHWNDProc(HWND hwnd, LPARAM lParam)
+{
+	GHOST_IWindow *p = (GHOST_IWindow*) GetWindowLong(hwnd, GWL_USERDATA);
+	BOOL ret = TRUE;
+	if (p == ghost_window_to_find)
+	{
+		found_ghost_window_hwnd = hwnd;
+		ret = FALSE;
+	}
+	return ret;
+}
+
+static HWND findGhostWindowHWND(GHOST_IWindow* window)
+{
+	found_ghost_window_hwnd = NULL;
+	ghost_window_to_find = window;
+	EnumWindows(findGhostWindowHWNDProc, NULL);
+	return found_ghost_window_hwnd;
+}
+
+bool GPG_Application::startScreenSaverPreview(
+	HWND parentWindow,
+	const bool stereoVisual,
+	const int stereoMode)
+{
+	bool success = false;
+
+	RECT rc;
+	if (GetWindowRect(parentWindow, &rc))
+	{
+		int windowWidth = rc.right - rc.left;
+		int windowHeight = rc.bottom - rc.top;
+		STR_String title = "";
+							
+		m_mainWindow = fSystem->createWindow(title, 0, 0, windowWidth, windowHeight, GHOST_kWindowStateMinimized,
+			GHOST_kDrawingContextTypeOpenGL, stereoVisual);
+		if (!m_mainWindow) {
+			printf("error: could not create main window\n");
+			exit(-1);
+		}
+
+		HWND ghost_hwnd = findGhostWindowHWND(m_mainWindow);
+		if (!ghost_hwnd) {
+			printf("error: could find main window\n");
+			exit(-1);
+		}
+
+		SetParent(ghost_hwnd, parentWindow);
+		LONG style = GetWindowLong(ghost_hwnd, GWL_STYLE);
+		LONG exstyle = GetWindowLong(ghost_hwnd, GWL_EXSTYLE);
+
+		RECT adjrc = { 0, 0, windowWidth, windowHeight };
+		AdjustWindowRectEx(&adjrc, style, FALSE, exstyle);
+
+		style = (style & (~(WS_POPUP|WS_OVERLAPPEDWINDOW|WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_THICKFRAME|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|WS_TILEDWINDOW ))) | WS_CHILD;
+		SetWindowLong(ghost_hwnd, GWL_STYLE, style);
+		SetWindowPos(ghost_hwnd, NULL, adjrc.left, adjrc.top, 0, 0, SWP_NOZORDER|SWP_NOSIZE|SWP_NOACTIVATE);
+
+		/* Check the size of the client rectangle of the window and resize the window
+		 * so that the client rectangle has the size requested.
+		 */
+		m_mainWindow->setClientSize(windowWidth, windowHeight);
+
+		success = initEngine(m_mainWindow, stereoMode);
+		if (success) {
+			success = startEngine();
+		}
+
+	}
+	return success;
+}
+
+bool GPG_Application::startScreenSaverFullScreen(
+		int width,
+		int height,
+		int bpp,int frequency,
+		const bool stereoVisual,
+		const int stereoMode)
+{
+	bool ret = startFullScreen(width, height, bpp, frequency, stereoVisual, stereoMode);
+	if (ret)
+	{
+		HWND ghost_hwnd = findGhostWindowHWND(m_mainWindow);
+		if (ghost_hwnd != NULL)
+		{
+			GetCursorPos(&scr_save_mouse_pos);
+			ghost_wnd_proc = (WNDPROC) GetWindowLong(ghost_hwnd, GWL_WNDPROC);
+			SetWindowLong(ghost_hwnd,GWL_WNDPROC, (LONG) screenSaverWindowProc);
+		}
+	}
+	return ret;
+}
+
+#endif
 
 bool GPG_Application::startWindow(STR_String& title,
 	int windowLeft,
@@ -144,15 +309,16 @@ bool GPG_Application::startWindow(STR_String& title,
 	//STR_String title ("Blender Player - GHOST");
 	m_mainWindow = fSystem->createWindow(title, windowLeft, windowTop, windowWidth, windowHeight, GHOST_kWindowStateNormal,
 		GHOST_kDrawingContextTypeOpenGL, stereoVisual);
-    if (!m_mainWindow) {
+	if (!m_mainWindow) {
 		printf("error: could not create main window\n");
-        exit(-1);
-    }
+		exit(-1);
+	}
 
 	/* Check the size of the client rectangle of the window and resize the window
 	 * so that the client rectangle has the size requested.
 	 */
 	m_mainWindow->setClientSize(windowWidth, windowHeight);
+	m_mainWindow->setCursorVisibility(false);
 
 	success = initEngine(m_mainWindow, stereoMode);
 	if (success) {
@@ -179,6 +345,7 @@ bool GPG_Application::startFullScreen(
 	setting.frequency = frequency;
 
 	fSystem->beginFullScreen(setting, &m_mainWindow, stereoVisual);
+	m_mainWindow->setCursorVisibility(false);
 
 	success = initEngine(m_mainWindow, stereoMode);
 	if (success) {
@@ -189,6 +356,7 @@ bool GPG_Application::startFullScreen(
 
 
 
+
 bool GPG_Application::StartGameEngine(int stereoMode)
 {
 	bool success = initEngine(m_mainWindow, stereoMode);
@@ -224,6 +392,10 @@ bool GPG_Application::processEvent(GHOST_IEvent* event)
 		case GHOST_kEventButtonUp:
 			handled = handleButton(event, false);
 			break;
+			
+		case GHOST_kEventWheel:
+			handled = handleWheel(event);
+			break;
 
 		case GHOST_kEventCursorMove:
 			handled = handleCursorMove(event);
@@ -311,78 +483,125 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 {
 	if (!m_engineInitialized)
 	{
+		bgl::InitExtensions(1);
+
 		// get and set the preferences
 		SYS_SystemHandle syshandle = SYS_GetSystem();
-		if (syshandle)
-		{
-			// SYS_WriteCommandLineInt(syshandle, "fixedtime", 0);
-			SYS_WriteCommandLineInt(syshandle, "vertexarrays",1);		
-			//bool properties	= (SYS_GetCommandLineInt(syshandle, "show_properties", 0) != 0);
-			//bool profile = (SYS_GetCommandLineInt(syshandle, "show_profile", 0) != 0);
-			//bool frameRate = (SYS_GetCommandLineInt(syshandle, "show_framerate", 0) != 0);
-			
-			// create the canvas, rasterizer and rendertools
-			m_canvas = new GPG_Canvas(window);
-			if (m_canvas)
-			{
-				m_canvas->Init();				
-				m_rendertools = new GPC_RenderTools();
-				if (m_rendertools)
-				{
-					m_rasterizer = new RAS_OpenGLRasterizer(m_canvas);
-					m_rasterizer->SetStereoMode(stereoMode);
-					if (m_rasterizer)
-					{
-						// create the inputdevices
-						m_keyboard = new GPG_KeyboardDevice();
-						if (m_keyboard)
-						{
-							m_mouse = new GPC_MouseDevice();
-							if (m_mouse)
-							{
-								// create a networkdevice
-								m_networkdevice = new NG_LoopBackNetworkDeviceInterface();
-								if (m_networkdevice)
-								{
-									// get an audiodevice
-									SND_DeviceManager::Subscribe();
-									m_audiodevice = SND_DeviceManager::Instance();
-									if (m_audiodevice)
-									{
-										m_audiodevice->UseCD();
-										// create a ketsjisystem (only needed for timing and stuff)
-										m_kxsystem = new GPG_System (m_system);
-										if (m_kxsystem)
-										{
-											// create the ketsjiengine
-											m_ketsjiengine = new KX_KetsjiEngine(m_kxsystem);
-											
-											// set the devices
-											m_ketsjiengine->SetKeyboardDevice(m_keyboard);
-											m_ketsjiengine->SetMouseDevice(m_mouse);
-											m_ketsjiengine->SetNetworkDevice(m_networkdevice);
-											m_ketsjiengine->SetCanvas(m_canvas);
-											m_ketsjiengine->SetRenderTools(m_rendertools);
-											m_ketsjiengine->SetRasterizer(m_rasterizer);
-											m_ketsjiengine->SetNetworkDevice(m_networkdevice);
-											m_ketsjiengine->SetAudioDevice(m_audiodevice);
+		if (!syshandle)
+			return false;
+		
+		// SYS_WriteCommandLineInt(syshandle, "fixedtime", 0);
+		// SYS_WriteCommandLineInt(syshandle, "vertexarrays",1);		
+		bool properties	= (SYS_GetCommandLineInt(syshandle, "show_properties", 0) != 0);
+		bool profile = (SYS_GetCommandLineInt(syshandle, "show_profile", 0) != 0);
+		bool fixedFr = (G.fileflags & G_FILE_ENABLE_ALL_FRAMES);
 
-											m_ketsjiengine->SetUseFixedTime(false);
-											//m_ketsjiengine->SetTimingDisplay(frameRate, profile, properties);
-
-											m_engineInitialized = true;
-										}
-									}
-								}
-							}
-						} 
-					}
-				}
-			}
+		bool fixed_framerate= (SYS_GetCommandLineInt(syshandle, "fixed_framerate", fixedFr) != 0);
+		bool frameRate = (SYS_GetCommandLineInt(syshandle, "show_framerate", 0) != 0);
+		bool useVertexArrays = SYS_GetCommandLineInt(syshandle,"vertexarrays",1) != 0;
+		bool useLists = (SYS_GetCommandLineInt(syshandle, "displaylists", G.fileflags & G_FILE_DIAPLAY_LISTS) != 0);
+#ifdef GL_ARB_multitexture
+		int gameflag =(G.fileflags & G_FILE_GAME_MAT);
+		// ----------------------------------
+		if(bgl::RAS_EXT_support._ARB_multitexture && bgl::QueryVersion(1, 1)) {
+			m_blendermat = (SYS_GetCommandLineInt(syshandle, "blender_material", gameflag) != 0);
+			int unitmax=0;
+			glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, (GLint*)&unitmax);
+			bgl::max_texture_units = MAXTEX>unitmax?unitmax:MAXTEX;
+			//std::cout << "using(" << bgl::max_texture_units << ") of(" << unitmax << ") texture units." << std::endl;
+		} else {
+			bgl::max_texture_units = 0;
 		}
+#else
+		m_blendermat=0;
+#endif//GL_ARB_multitexture
+		// ----------------------------------
+	
+		// create the canvas, rasterizer and rendertools
+		m_canvas = new GPG_Canvas(window);
+		if (!m_canvas)
+			return false;
+				
+		m_canvas->Init();				
+		m_rendertools = new GPC_RenderTools();
+		if (!m_rendertools)
+			goto initFailed;
+		
+		if(useLists)
+			m_rasterizer = new RAS_ListRasterizer(m_canvas);
+		else if (useVertexArrays && bgl::QueryVersion(1, 1))
+			m_rasterizer = new RAS_VAOpenGLRasterizer(m_canvas);
+		else
+			m_rasterizer = new RAS_OpenGLRasterizer(m_canvas);
+		m_rasterizer->SetStereoMode((RAS_IRasterizer::StereoMode) stereoMode);
+		if (!m_rasterizer)
+			goto initFailed;
+						
+		// create the inputdevices
+		m_keyboard = new GPG_KeyboardDevice();
+		if (!m_keyboard)
+			goto initFailed;
+			
+		m_mouse = new GPC_MouseDevice();
+		if (!m_mouse)
+			goto initFailed;
+			
+		// create a networkdevice
+		m_networkdevice = new NG_LoopBackNetworkDeviceInterface();
+		if (!m_networkdevice)
+			goto initFailed;
+			
+		// get an audiodevice
+		SND_DeviceManager::Subscribe();
+		m_audiodevice = SND_DeviceManager::Instance();
+		if (!m_audiodevice)
+			goto initFailed;
+		m_audiodevice->UseCD();
+		
+		// create a ketsjisystem (only needed for timing and stuff)
+		m_kxsystem = new GPG_System (m_system);
+		if (!m_kxsystem)
+			goto initFailed;
+		
+		// create the ketsjiengine
+		m_ketsjiengine = new KX_KetsjiEngine(m_kxsystem);
+		
+		// set the devices
+		m_ketsjiengine->SetKeyboardDevice(m_keyboard);
+		m_ketsjiengine->SetMouseDevice(m_mouse);
+		m_ketsjiengine->SetNetworkDevice(m_networkdevice);
+		m_ketsjiengine->SetCanvas(m_canvas);
+		m_ketsjiengine->SetRenderTools(m_rendertools);
+		m_ketsjiengine->SetRasterizer(m_rasterizer);
+		m_ketsjiengine->SetNetworkDevice(m_networkdevice);
+		m_ketsjiengine->SetAudioDevice(m_audiodevice);
+		m_ketsjiengine->SetTimingDisplay(frameRate, false, false);
+
+		m_ketsjiengine->SetUseFixedTime(fixed_framerate);
+		m_ketsjiengine->SetTimingDisplay(frameRate, profile, properties);
+
+		m_engineInitialized = true;
 	}
 
 	return m_engineInitialized;
+initFailed:
+	delete m_kxsystem;
+	delete m_audiodevice;
+	delete m_networkdevice;
+	delete m_mouse;
+	delete m_keyboard;
+	delete m_rasterizer;
+	delete m_rendertools;
+	delete m_canvas;
+	m_canvas = NULL;
+	m_rendertools = NULL;
+	m_rasterizer = NULL;
+	m_keyboard = NULL;
+	m_mouse = NULL;
+	m_networkdevice = NULL;
+	m_audiodevice = NULL;
+	m_kxsystem = NULL;
+	return false;
 }
 
 
@@ -410,7 +629,7 @@ bool GPG_Application::startEngine(void)
 	*/
 	
 	// create a scene converter, create and convert the stratingscene
-	m_sceneconverter = new KX_BlenderSceneConverter(m_maggie, m_ketsjiengine);
+	m_sceneconverter = new KX_BlenderSceneConverter(m_maggie,0, m_ketsjiengine);
 	if (m_sceneconverter)
 	{
 		STR_String startscenename = m_startSceneName.Ptr();
@@ -418,7 +637,8 @@ bool GPG_Application::startEngine(void)
 		
 		//	if (always_use_expand_framing)
 		//		sceneconverter->SetAlwaysUseExpandFraming(true);
-		
+		if(m_blendermat)
+			m_sceneconverter->SetMaterials(true);
 
 		KX_Scene* startscene = new KX_Scene(m_keyboard,
 			m_mouse,
@@ -426,18 +646,23 @@ bool GPG_Application::startEngine(void)
 			m_audiodevice,
 			startscenename);
 		
+		
 		// some python things
-		PyObject* m_dictionaryobject = initGamePythonScripting("Ketsji", psl_Lowest);
-		m_ketsjiengine->SetPythonDictionary(m_dictionaryobject);
+		PyObject* dictionaryobject = initGamePlayerPythonScripting("Ketsji", psl_Lowest);
+		m_ketsjiengine->SetPythonDictionary(dictionaryobject);
 		initRasterizer(m_rasterizer, m_canvas);
-		initGameLogic(startscene);
+		PyObject *gameLogic = initGameLogic(startscene);
 		initGameKeys();
 		initPythonConstraintBinding();
-		
+
+
+
+
+
 		m_sceneconverter->ConvertScene(
 			startscenename,
 			startscene,
-			m_dictionaryobject,
+			dictionaryobject,
 			m_keyboard,
 			m_rendertools,
 			m_canvas);
@@ -534,11 +759,29 @@ void GPG_Application::exitEngine()
 	m_engineInitialized = false;
 }
 
+bool GPG_Application::handleWheel(GHOST_IEvent* event)
+{
+	bool handled = false;
+	MT_assert(event);
+	if (m_mouse) 
+	{
+		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
+		GHOST_TEventWheelData* wheelData = static_cast<GHOST_TEventWheelData*>(eventData);
+		GPC_MouseDevice::TButtonId button;
+		if (wheelData->z > 0)
+			button = GPC_MouseDevice::buttonWheelUp;
+		else
+			button = GPC_MouseDevice::buttonWheelDown;
+		m_mouse->ConvertButtonEvent(button, true);
+		handled = true;
+	}
+	return handled;
+}
 
 bool GPG_Application::handleButton(GHOST_IEvent* event, bool isDown)
 {
 	bool handled = false;
-	assert(event);
+	MT_assert(event);
 	if (m_mouse) 
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
@@ -567,7 +810,7 @@ bool GPG_Application::handleButton(GHOST_IEvent* event, bool isDown)
 bool GPG_Application::handleCursorMove(GHOST_IEvent* event)
 {
 	bool handled = false;
-	assert(event);
+	MT_assert(event);
 	if (m_mouse && m_mainWindow)
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
@@ -584,7 +827,7 @@ bool GPG_Application::handleCursorMove(GHOST_IEvent* event)
 bool GPG_Application::handleKey(GHOST_IEvent* event, bool isDown)
 {
 	bool handled = false;
-	assert(event);
+	MT_assert(event);
 	if (m_keyboard)
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
