@@ -1,5 +1,5 @@
 /**
- * $Id: KXH_ketsji_hooks.cpp,v 1.6 2005/01/18 14:17:44 sirdude Exp $
+ * $Id: KXH_ketsji_hooks.cpp,v 1.7 2005/08/28 10:55:07 intrr Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -524,7 +524,7 @@ initialize_gameengine(ketsji_engine_data* k, struct BlendFileData * active_file)
 	if (k->kx_engine)
 	{	
 		k->converter = new
-			KX_BlenderSceneConverter(active_file->main,
+			KX_BlenderSceneConverter(active_file->main, 0,
 						 k->kx_engine);
 		
 		PyObject* dictionaryobject 
