@@ -1,5 +1,5 @@
 /**
- * $Id: KX_GameObject.cpp,v 1.15 2005/04/18 11:44:21 kester Exp $
+ * $Id: KX_GameObject.cpp,v 1.22 2006/06/18 22:09:59 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,6 +35,12 @@
 #include <config.h>
 #endif
 
+#if defined(_WIN64)
+typedef unsigned __int64 uint_ptr;
+#else
+typedef unsigned long uint_ptr;
+#endif
+
 #ifdef WIN32
 // This warning tells us about truncation of __long__ stl-generated names.
 // It can occasionally cause DevStudio to have internal compiler warnings.
@@ -44,6 +50,7 @@
 
 #define KX_INERTIA_INFINITE 10000
 #include "RAS_IPolygonMaterial.h"
+#include "KX_BlenderMaterial.h"
 #include "KX_GameObject.h"
 #include "RAS_MeshObject.h"
 #include "KX_MeshProxy.h"
@@ -361,6 +368,34 @@ void KX_GameObject::UpdateIPO(float curframetime,
 	UpdateTransform();
 }
 
+// IPO update
+void 
+KX_GameObject::UpdateMaterialData(
+		MT_Vector4 rgba,
+		MT_Vector3 specrgb,
+		MT_Scalar hard,
+		MT_Scalar spec,
+		MT_Scalar ref,
+		MT_Scalar emit,
+		MT_Scalar alpha
+
+	)
+{
+	int mesh = 0;
+	if (((unsigned int)mesh < m_meshes.size()) && mesh >= 0) {
+		RAS_MaterialBucket::Set::iterator mit = m_meshes[mesh]->GetFirstMaterial();
+		for(; mit != m_meshes[mesh]->GetLastMaterial(); ++mit)
+		{
+			RAS_IPolyMaterial* poly = (*mit)->GetPolyMaterial();
+			if(poly->GetFlag() & RAS_BLENDERMAT )
+			{
+				SetObjectColor(rgba);
+				KX_BlenderMaterial *m =  static_cast<KX_BlenderMaterial*>(poly);
+				m->UpdateIPO(rgba, specrgb,hard,spec,ref,emit, alpha);
+			}
+		}
+	}
+}
 bool
 KX_GameObject::GetVisible(
 	void
@@ -414,6 +449,7 @@ KX_GameObject::MarkVisible(
 	}
 }
 
+
 void KX_GameObject::addLinearVelocity(const MT_Vector3& lin_vel,bool local)
 {
 	if (m_pPhysicsController1)
@@ -591,6 +627,7 @@ PyMethodDef KX_GameObject::Methods[] = {
 	{"getMass", (PyCFunction) KX_GameObject::sPyGetMass, METH_VARARGS},
 	{"getReactionForce", (PyCFunction) KX_GameObject::sPyGetReactionForce, METH_VARARGS},
 	{"applyImpulse", (PyCFunction) KX_GameObject::sPyApplyImpulse, METH_VARARGS},
+	{"setCollisionMargin", (PyCFunction) KX_GameObject::sPySetCollisionMargin, METH_VARARGS},
 	{"suspendDynamics", (PyCFunction)KX_GameObject::sPySuspendDynamics,METH_VARARGS},
 	{"restoreDynamics", (PyCFunction)KX_GameObject::sPyRestoreDynamics,METH_VARARGS},
 	{"enableRigidBody", (PyCFunction)KX_GameObject::sPyEnableRigidBody,METH_VARARGS},
@@ -599,7 +636,6 @@ PyMethodDef KX_GameObject::Methods[] = {
 	{"getMesh", (PyCFunction)KX_GameObject::sPyGetMesh,METH_VARARGS},
 	{"getPhysicsId", (PyCFunction)KX_GameObject::sPyGetPhysicsId,METH_VARARGS},
 	KX_PYMETHODTABLE(KX_GameObject, getDistanceTo),
-	
 	{NULL,NULL} //Sentinel
 };
 
@@ -931,7 +967,7 @@ PyObject* KX_GameObject::PyGetMesh(PyObject* self,
 								   PyObject* kwds)
 {
 	int mesh = 0;
-	
+
 	if (PyArg_ParseTuple(args, "|i", &mesh))
 	{
 		if (((unsigned int)mesh < m_meshes.size()) && mesh >= 0)
@@ -945,24 +981,45 @@ PyObject* KX_GameObject::PyGetMesh(PyObject* self,
 
 
 
+
+
+PyObject* KX_GameObject::PySetCollisionMargin(PyObject* self, 
+										PyObject* args, 
+										PyObject* kwds)
+{
+	float collisionMargin;
+	if (PyArg_ParseTuple(args, "f", &collisionMargin))
+	{
+		if (m_pPhysicsController1)
+		{
+			m_pPhysicsController1->SetMargin(collisionMargin);
+			Py_Return;
+		}
+
+	}
+	
+	return NULL;
+}
+
+
+
 PyObject* KX_GameObject::PyApplyImpulse(PyObject* self, 
 										PyObject* args, 
 										PyObject* kwds)
 {
-	
-	
 	PyObject* pyattach;
 	PyObject* pyimpulse;
-	
 	if (PyArg_ParseTuple(args, "OO", &pyattach, &pyimpulse))
 	{
 		MT_Point3  attach;
 		MT_Vector3 impulse;
-	
-		if (PyVecTo(pyattach, attach) && PyVecTo(pyimpulse, impulse) && m_pPhysicsController1)
+		if (m_pPhysicsController1)
 		{
-			m_pPhysicsController1->applyImpulse(attach, impulse);
-			Py_Return;
+			if (PyVecTo(pyattach, attach) && PyVecTo(pyimpulse, impulse))
+			{
+				m_pPhysicsController1->applyImpulse(attach, impulse);
+				Py_Return;
+			}
 		}
 
 	}
@@ -1058,6 +1115,7 @@ PyObject* KX_GameObject::PySetPosition(PyObject* self,
 	if (PyVecArgTo(args, pos))
 	{
 		NodeSetLocalPosition(pos);
+		NodeUpdateGS(0.f,true);
 		Py_Return;
 	}
 	
@@ -1069,12 +1127,12 @@ PyObject* KX_GameObject::PyGetPhysicsId(PyObject* self,
 											   PyObject* kwds)
 {
 	KX_IPhysicsController* ctrl = GetPhysicsController();
-	int physid=0;
+	uint_ptr physid=0;
 	if (ctrl)
 	{
-		physid= (int)ctrl->GetUserData();
+		physid= (uint_ptr)ctrl->GetUserData();
 	}
-	return PyInt_FromLong(physid);
+	return PyInt_FromLong((long)physid);
 }
 
 KX_PYMETHODDEF_DOC(KX_GameObject, getDistanceTo,
@@ -1097,7 +1155,6 @@ KX_PYMETHODDEF_DOC(KX_GameObject, getDistanceTo,
 	return NULL;
 }
 
-
 /* --------------------------------------------------------------------- 
  * Some stuff taken from the header
  * --------------------------------------------------------------------- */
