@@ -1,7 +1,7 @@
 /**
  * Sense if other objects are near
  *
- * $Id: KX_NearSensor.cpp,v 1.11 2005/06/04 16:22:50 lukep Exp $
+ * $Id: KX_NearSensor.cpp,v 1.13 2006/05/22 21:03:43 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -63,21 +63,27 @@ KX_NearSensor::KX_NearSensor(SCA_EventManager* eventmgr,
 			 m_ResetMargin(resetmargin)
 
 {
+
 	gameobj->getClientInfo()->m_sensors.remove(this);
 	m_client_info = new KX_ClientObjectInfo(gameobj, KX_ClientObjectInfo::NEAR);
 	m_client_info->m_sensors.push_back(this);
 	
 	//DT_ShapeHandle shape = (DT_ShapeHandle) vshape;
 	m_physCtrl = ctrl;
-	m_physCtrl->SetMargin(m_Margin);
-	m_physCtrl->setNewClientInfo(m_client_info);
-	
+	if (m_physCtrl)
+	{
+		m_physCtrl->SetMargin(m_Margin);
+		m_physCtrl->setNewClientInfo(m_client_info);
+	}
 	SynchronizeTransform();
 }
 
 void KX_NearSensor::RegisterSumo(KX_TouchEventManager *touchman)
 {
-	touchman->GetPhysicsEnvironment()->addSensor(m_physCtrl);
+	if (m_physCtrl)
+	{
+		touchman->GetPhysicsEnvironment()->addSensor(m_physCtrl);
+	}
 }
 
 CValue* KX_NearSensor::GetReplica()
@@ -93,7 +99,18 @@ CValue* KX_NearSensor::GetReplica()
 	
 	replica->m_client_info = new KX_ClientObjectInfo(m_client_info->m_gameobject, KX_ClientObjectInfo::NEAR);
 	
-	replica->m_physCtrl = replica->m_physCtrl->GetReplica();
+	if (replica->m_physCtrl)
+	{
+		replica->m_physCtrl = replica->m_physCtrl->GetReplica();
+		if (replica->m_physCtrl)
+		{
+			//static_cast<KX_TouchEventManager*>(m_eventmgr)->GetPhysicsEnvironment()->addSensor(replica->m_physCtrl);
+			replica->m_physCtrl->SetMargin(m_Margin);
+			replica->m_physCtrl->setNewClientInfo(replica->m_client_info);
+		}
+		
+	}
+	//static_cast<KX_TouchEventManager*>(m_eventmgr)->RegisterSensor(this);
 	//todo: make sure replication works fine
 	//>m_sumoObj = new SM_Object(DT_NewSphere(0.0),NULL,NULL,NULL);
 	//replica->m_sumoObj->setMargin(m_Margin);
@@ -108,11 +125,21 @@ CValue* KX_NearSensor::GetReplica()
 
 void KX_NearSensor::ReParent(SCA_IObject* parent)
 {
+
 	SCA_ISensor::ReParent(parent);
 	
 	m_client_info->m_gameobject = static_cast<KX_GameObject*>(parent); 
 	m_client_info->m_sensors.push_back(this);
 	
+
+/*	KX_ClientObjectInfo *client_info = gameobj->getClientInfo();
+	client_info->m_gameobject = gameobj;
+	client_info->m_auxilary_info = NULL;
+	
+	client_info->m_sensors.push_back(this);
+	SCA_ISensor::ReParent(parent);
+*/
+
 	SynchronizeTransform();
 }
 
@@ -124,10 +151,11 @@ KX_NearSensor::~KX_NearSensor()
 	// for touchsensor, it's the parent
 	if (m_physCtrl)
 	{
-		static_cast<KX_TouchEventManager*>(m_eventmgr)->GetPhysicsEnvironment()->removeSensor(m_physCtrl);
+		//static_cast<KX_TouchEventManager*>(m_eventmgr)->GetPhysicsEnvironment()->removeSensor(m_physCtrl);
 		delete m_physCtrl;
 		m_physCtrl = NULL;
 	}
+	
 		
 	if (m_client_info)
 		delete m_client_info;
