@@ -1,7 +1,7 @@
 //
 // Replace the mesh for this actuator's parent
 //
-// $Id: KX_SCA_ReplaceMeshActuator.cpp,v 1.3 2002/11/25 15:29:49 mein Exp $
+// $Id: KX_SCA_ReplaceMeshActuator.cpp,v 1.9 2005/03/09 19:45:59 lukep Exp $
 //
 // ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 //
@@ -84,12 +84,13 @@ PyParentObject KX_SCA_ReplaceMeshActuator::Parents[] = {
 
 PyMethodDef KX_SCA_ReplaceMeshActuator::Methods[] = {
   {"setMesh", (PyCFunction) KX_SCA_ReplaceMeshActuator::sPySetMesh, METH_VARARGS, SetMesh_doc},
+  KX_PYMETHODTABLE(KX_SCA_ReplaceMeshActuator, getMesh),
   {NULL,NULL} //Sentinel
 };
 
 
 
-PyObject* KX_SCA_ReplaceMeshActuator::_getattr(char* attr)
+PyObject* KX_SCA_ReplaceMeshActuator::_getattr(const STR_String& attr)
 {
   _getattr_up(SCA_IActuator);
 }
@@ -115,7 +116,7 @@ PyObject* KX_SCA_ReplaceMeshActuator::PySetMesh(PyObject* self,
 
 	void* mesh = SCA_ILogicBrick::m_sCurrentLogicManager->GetMeshByName(STR_String(meshname));
 	
-	if (m_mesh) {
+	if (mesh) {
 		m_mesh= (class RAS_MeshObject*)mesh;
 		Py_Return;
 	}
@@ -123,7 +124,16 @@ PyObject* KX_SCA_ReplaceMeshActuator::PySetMesh(PyObject* self,
 	return NULL;
 }
 
+KX_PYMETHODDEF_DOC(KX_SCA_ReplaceMeshActuator, getMesh,
+"getMesh() -> string\n"
+"Returns the name of the mesh to be substituted.\n"
+)
+{
+	if (!m_mesh)
+		Py_Return;
 
+	return PyString_FromString(const_cast<char *>(m_mesh->GetName().ReadPtr()));
+}
 
 /* ------------------------------------------------------------------------- */
 /* Native functions                                                          */
@@ -135,8 +145,8 @@ KX_SCA_ReplaceMeshActuator::KX_SCA_ReplaceMeshActuator(SCA_IObject *gameobj,
 													   PyTypeObject* T) : 
 
 	SCA_IActuator(gameobj, T),
-	m_scene(scene),
-	m_mesh(mesh)
+	m_mesh(mesh),
+	m_scene(scene)
 {
 } /* End of constructor */
 
@@ -149,10 +159,9 @@ KX_SCA_ReplaceMeshActuator::~KX_SCA_ReplaceMeshActuator()
 
 
 
-bool KX_SCA_ReplaceMeshActuator::Update(double curtime,
-										double deltatime)
+bool KX_SCA_ReplaceMeshActuator::Update()
 {
-	bool result = false;	
+	// bool result = false;	/*unused*/
 	bool bNegativeEvent = IsNegativeEvent();
 	RemoveAllEvents();
 
