@@ -1,5 +1,5 @@
 /*
- * $Id: KX_Scene.cpp,v 1.19 2005/06/04 16:22:50 lukep Exp $
+ * $Id: KX_Scene.cpp,v 1.26 2006/05/22 21:03:43 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -54,6 +54,8 @@
 #include "SCA_JoystickManager.h"
 
 #include "RAS_MeshObject.h"
+#include "BL_SkinMeshObject.h"
+
 #include "RAS_IRasterizer.h"
 #include "RAS_BucketManager.h"
 
@@ -112,7 +114,8 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	m_active_camera(NULL),
 	m_ueberExecutionPriority(0)
 {
-		
+	m_suspendedtime = 0.0;
+	m_suspendeddelta = 0.0;
 
 	m_activity_culling = false;
 	m_suspend = false;
@@ -122,6 +125,7 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	m_parentlist = new CListValue();
 	m_lightlist= new CListValue();
 	m_euthanasyobjects = new CListValue();
+	m_delayReleaseObjects = new CListValue();
 
 	m_logicmgr = new SCA_LogicManager();
 	
@@ -178,7 +182,7 @@ KX_Scene::~KX_Scene()
 
 	if(m_objectlist)
 		m_objectlist->Release();
-	
+
 	if (m_parentlist)
 		m_parentlist->Release();
 	
@@ -190,6 +194,8 @@ KX_Scene::~KX_Scene()
 
 	if (m_euthanasyobjects)
 		m_euthanasyobjects->Release();
+	if (m_delayReleaseObjects)
+		m_delayReleaseObjects->Release();
 
 	if (m_logicmgr)
 		delete m_logicmgr;
@@ -207,7 +213,8 @@ KX_Scene::~KX_Scene()
 	{
 		delete m_bucketmanager;
 	}
-	Py_DECREF(m_attrlist);
+
+	//Py_DECREF(m_attrlist);
 }
 
 
@@ -258,6 +265,13 @@ SCA_TimeEventManager* KX_Scene::GetTimeEventManager()
 
 
 
+ 
+set<class KX_Camera*>* KX_Scene::GetCameras()
+{
+	return &m_cameras;
+}
+
+
 
 void KX_Scene::SetFramingType(RAS_FrameSettings & frame_settings)
 {
@@ -643,8 +657,13 @@ void KX_Scene::RemoveObject(class CValue* gameobj)
 		// recursively destruct
 		node->Destruct();
 	}
+	newobj->SetSGNode(0);
 }
 
+void KX_Scene::DelayedReleaseObject(CValue* gameobj)
+{
+	m_delayReleaseObjects->Add(gameobj->AddRef());
+}
 
 
 void KX_Scene::DelayedRemoveObject(class CValue* gameobj)
@@ -713,6 +732,7 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 		
 	if (newobj == m_active_camera)
 	{
+		m_active_camera->Release();
 		m_active_camera = NULL;
 	}
 }
@@ -721,34 +741,92 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 
 void KX_Scene::ReplaceMesh(class CValue* gameobj,void* meshobj)
 {
-	KX_GameObject* newobj = (KX_GameObject*) gameobj;
-	RAS_MeshObject* mesh = (RAS_MeshObject*) meshobj;
+	KX_GameObject* newobj = static_cast<KX_GameObject*>(gameobj);
+	RAS_MeshObject* mesh = static_cast<RAS_MeshObject*>(meshobj);
+
+	const STR_String origMeshName = newobj->GetMesh(0)->GetName();
+
+	if( !newobj || !mesh ) 
+	{
+		std::cout << "warning: invalid object, mesh will not be replaced" << std::endl;
+		return;
+	}
 
 	newobj->RemoveMeshes();
 	newobj->AddMesh(mesh);
 
-	if (newobj->m_isDeformable && mesh->m_class == 1) {
-		Object* blendobj = (struct Object*)m_logicmgr->FindBlendObjByGameObj(newobj);
-		Object* oldblendobj = (struct Object*)m_logicmgr->FindBlendObjByGameMeshName(mesh->GetName());
-		if (blendobj->parent && blendobj->parent->type == OB_ARMATURE && blendobj->partype==PARSKEL && ((Mesh*)blendobj->data)->dvert) {
-			BL_SkinDeformer* skindeformer = new BL_SkinDeformer(oldblendobj, blendobj, (BL_SkinMeshObject*)mesh);
-			skindeformer->SetArmature((BL_ArmatureObject*) newobj->GetParent());
-			
-			// FIXME: should the old m_pDeformer be deleted?
-			// delete ((BL_DeformableGameObject*)newobj)->m_pDeformer
-			
-			((BL_DeformableGameObject*)newobj)->m_pDeformer = skindeformer;
-		}
-		else if (((Mesh*)blendobj->data)->dvert) {
-			BL_MeshDeformer* meshdeformer = new BL_MeshDeformer(oldblendobj, (BL_SkinMeshObject*)mesh);
-			
-			// FIXME: should the old m_pDeformer be deleted?
-			// delete ((BL_DeformableGameObject*)newobj)->m_pDeformer
-			
-			((BL_DeformableGameObject*)newobj)->m_pDeformer = meshdeformer;
-		}
-	}
+	bool isDeformer = (newobj->m_isDeformable && mesh->m_class == 1);
+	if(isDeformer)
+	{
+		/* FindBlendObjByGameObj() can return 0... 
+			In the case of 0 here,
+			the replicated object that is calling this function
+			is some how not in the map. (which is strange because it's added)
+			So we will search the map by the first mesh name
+			to try to locate it there. If its still not found 
+			spit some message rather than crash 
+		*/
+		Object* blendobj = static_cast<struct Object*>(m_logicmgr->FindBlendObjByGameObj(newobj));
+		Object* oldblendobj = static_cast<struct Object*>(m_logicmgr->FindBlendObjByGameMeshName(mesh->GetName()));
+	
+		bool parSkin = blendobj && blendobj->parent && blendobj->parent->type == OB_ARMATURE && blendobj->partype==PARSKEL;
+		bool releaseParent = true;
+		KX_GameObject* parentobj = newobj->GetParent();
 
+
+		// lookup by mesh name if blendobj is 0 
+		if( !blendobj &&  parentobj )
+		{
+			blendobj = static_cast<struct Object*>(m_logicmgr->FindBlendObjByGameMeshName(origMeshName));
+
+			// replace the mesh on the parent armature 
+			if( blendobj )
+				parSkin = parentobj->GetGameObjectType() == SCA_IObject::OBJ_ARMATURE;
+
+			// can't do it 
+			else 
+				std::cout << "warning: child object for " << parentobj->GetName().ReadPtr() 
+					<< " not found, and can't create!" << std::endl;
+		}
+
+		if( blendobj && oldblendobj )
+		{
+			isDeformer = (static_cast<Mesh*>(blendobj->data)->dvert != 0);
+			BL_DeformableGameObject* deformIter =0;
+
+			// armature parent
+			if( parSkin && isDeformer )
+			{
+				deformIter = static_cast<BL_DeformableGameObject*>( newobj );
+				delete deformIter->m_pDeformer;
+
+				BL_SkinDeformer* skinDeformer = new BL_SkinDeformer(
+					oldblendobj, blendobj,
+					static_cast<BL_SkinMeshObject*>(mesh),
+					true,
+					static_cast<BL_ArmatureObject*>( parentobj )
+				);
+				releaseParent= false;
+				deformIter->m_pDeformer = skinDeformer;
+			}
+
+			// normal deformer
+			if( !parSkin && isDeformer)
+			{
+				deformIter = static_cast<BL_DeformableGameObject*>( newobj );
+				delete deformIter->m_pDeformer;
+
+				BL_MeshDeformer* meshdeformer = new BL_MeshDeformer(
+					oldblendobj, static_cast<BL_SkinMeshObject*>(mesh)
+				);
+
+				deformIter->m_pDeformer = meshdeformer;
+			}
+		}
+		// release parent reference if its not being used 
+		if( releaseParent && parentobj)
+			parentobj->Release();
+	}
 	newobj->Bucketize();
 }
 
@@ -831,48 +909,48 @@ void KX_Scene::UpdateMeshTransformations()
 	}
 }
 
-void KX_Scene::MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty)
+void KX_Scene::MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty, KX_Camera* cam)
 {
 	int intersect = KX_Camera::INTERSECT;
 	KX_GameObject *gameobj = node->Client()?(KX_GameObject*) node->Client()->GetSGClientObject():NULL;
 	bool dotest = (gameobj && gameobj->GetVisible()) || node->Left() || node->Right();
 	
 	/* If the camera is inside the box, assume intersect. */
-	if (dotest && !node->inside(GetActiveCamera()->NodeGetWorldPosition()))
+	if (dotest && !node->inside( cam->NodeGetWorldPosition()))
 	{
 		MT_Scalar radius = node->Radius();
 		MT_Point3 centre = node->Centre();
 		
-		intersect = GetActiveCamera()->SphereInsideFrustum(centre, radius); 
+		intersect =  cam->SphereInsideFrustum(centre, radius); 
 		
 		if (intersect == KX_Camera::INTERSECT)
 		{
 			MT_Point3 box[8];
 			node->get(box);
-			intersect = GetActiveCamera()->BoxInsideFrustum(box);
+			intersect = cam->BoxInsideFrustum(box);
 		}
 	}
 
 	switch (intersect)
 	{
 		case KX_Camera::OUTSIDE:
-			MarkSubTreeVisible(node, rasty, false);
+			MarkSubTreeVisible(node, rasty, false, cam);
 			break;
 		case KX_Camera::INTERSECT:
 			if (gameobj)
-				MarkVisible(rasty, gameobj);
+				MarkVisible(rasty, gameobj,cam);
 			if (node->Left())
-				MarkVisible(node->Left(), rasty);
+				MarkVisible(node->Left(), rasty,cam);
 			if (node->Right())
-				MarkVisible(node->Right(), rasty);
+				MarkVisible(node->Right(), rasty,cam);
 			break;
 		case KX_Camera::INSIDE:
-			MarkSubTreeVisible(node, rasty, true);
+			MarkSubTreeVisible(node, rasty, true,cam);
 			break;
 	}
 }
 
-void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible)
+void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible,KX_Camera* cam)
 {
 	if (node->Client())
 	{
@@ -882,7 +960,7 @@ void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool vi
 			if (visible)
 			{
 				int nummeshes = gameobj->GetMeshCount();
-				MT_Transform t( GetActiveCamera()->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
+				MT_Transform t( cam->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
 	
 				
 				for (int m=0;m<nummeshes;m++)
@@ -895,18 +973,18 @@ void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool vi
 		}
 	}
 	if (node->Left())
-		MarkSubTreeVisible(node->Left(), rasty, visible);
+		MarkSubTreeVisible(node->Left(), rasty, visible,cam);
 	if (node->Right())
-		MarkSubTreeVisible(node->Right(), rasty, visible);
+		MarkSubTreeVisible(node->Right(), rasty, visible,cam);
 }
 
-void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
+void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj,KX_Camera*  cam)
 {
 	// User (Python/Actuator) has forced object invisible...
 	if (!gameobj->GetVisible())
 		return;
 	// If Frustum culling is off, the object is always visible.
-	bool vis = !GetActiveCamera()->GetFrustumCulling();
+	bool vis = !cam->GetFrustumCulling();
 	
 	// If the camera is inside this node, then the object is visible.
 	if (!vis)
@@ -919,7 +997,7 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 	{
 		MT_Vector3 scale = gameobj->GetSGNode()->GetWorldScaling();
 		MT_Scalar radius = fabs(scale[scale.closestAxis()] * gameobj->GetSGNode()->Radius());
-		switch (GetActiveCamera()->SphereInsideFrustum(gameobj->NodeGetWorldPosition(), radius))
+		switch (cam->SphereInsideFrustum(gameobj->NodeGetWorldPosition(), radius))
 		{
 			case KX_Camera::INSIDE:
 				vis = true;
@@ -931,7 +1009,7 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 				// Test the object's bound box against the view frustum.
 				MT_Point3 box[8];
 				gameobj->GetSGNode()->getBBox(box); 
-				vis = GetActiveCamera()->BoxInsideFrustum(box) != KX_Camera::OUTSIDE;
+				vis = cam->BoxInsideFrustum(box) != KX_Camera::OUTSIDE;
 				break;
 		}
 	}
@@ -939,7 +1017,7 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 	if (vis)
 	{
 		int nummeshes = gameobj->GetMeshCount();
-		MT_Transform t(GetActiveCamera()->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
+		MT_Transform t(cam->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
 		
 		for (int m=0;m<nummeshes;m++)
 		{
@@ -954,20 +1032,20 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 	}
 }
 
-void KX_Scene::CalculateVisibleMeshes(RAS_IRasterizer* rasty)
+void KX_Scene::CalculateVisibleMeshes(RAS_IRasterizer* rasty,KX_Camera* cam)
 {
 // FIXME: When tree is operational
 #if 1
 	// do this incrementally in the future
 	for (int i = 0; i < m_objectlist->GetCount(); i++)
 	{
-		MarkVisible(rasty, static_cast<KX_GameObject*>(m_objectlist->GetValue(i)));
+		MarkVisible(rasty, static_cast<KX_GameObject*>(m_objectlist->GetValue(i)), cam);
 	}
 #else
-	if (GetActiveCamera()->GetFrustumCulling())
-		MarkVisible(m_objecttree, rasty);
+	if (cam->GetFrustumCulling())
+		MarkVisible(m_objecttree, rasty, cam);
 	else
-		MarkSubTreeVisible(m_objecttree, rasty, true);
+		MarkSubTreeVisible(m_objecttree, rasty, true, cam);
 #endif
 }
 
@@ -1017,12 +1095,22 @@ void KX_Scene::LogicEndFrame()
 {
 	m_logicmgr->EndFrame();
 	int numobj = m_euthanasyobjects->GetCount();
-
-	for (int i = numobj - 1; i >= 0; i--)
+	int i;
+	for (i = numobj - 1; i >= 0; i--)
 	{
 		KX_GameObject* gameobj = (KX_GameObject*)m_euthanasyobjects->GetValue(i);
 		this->RemoveObject(gameobj);
 	}
+
+	numobj=	m_delayReleaseObjects->GetCount();
+	for (i = numobj-1;i>=0;i--)
+	{
+		KX_GameObject* gameobj = (KX_GameObject*)m_delayReleaseObjects->GetValue(i);
+		m_delayReleaseObjects->RemoveValue(gameobj);	
+		
+	}
+	
+
 }
 
 
@@ -1135,6 +1223,23 @@ void KX_Scene::SetPhysicsEnvironment(class PHY_IPhysicsEnvironment* physEnv)
 	m_logicmgr->RegisterEventManager(touchmgr);
 	return;
 }
+ 
+void KX_Scene::setSuspendedTime(double suspendedtime)
+{
+	m_suspendedtime = suspendedtime;
+}
+double KX_Scene::getSuspendedTime()
+{
+	return m_suspendedtime;
+}
+void KX_Scene::setSuspendedDelta(double suspendeddelta)
+{
+	m_suspendeddelta = suspendeddelta;
+}
+double KX_Scene::getSuspendedDelta()
+{
+	return m_suspendeddelta;
+}
 
 //----------------------------------------------------------------------------
 //Python
