@@ -1,7 +1,7 @@
 /**
  * Senses touch and collision events
  *
- * $Id: KX_TouchSensor.cpp,v 1.3 2002/11/25 15:29:49 mein Exp $
+ * $Id: KX_TouchSensor.cpp,v 1.12 2006/04/26 03:20:28 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,14 +37,14 @@
 #include "SCA_LogicManager.h"
 #include "KX_GameObject.h"
 #include "KX_TouchEventManager.h"
+#include "KX_SumoPhysicsController.h"
 #include <iostream>
+#include "PHY_IPhysicsEnvironment.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifdef PHYSICS_NOT_YET
-
 /* ------------------------------------------------------------------------- */
 /* Native functions                                                          */
 /* ------------------------------------------------------------------------- */
@@ -52,13 +52,13 @@
 void KX_TouchSensor::SynchronizeTransform()
 {
 
-	if (m_sumoObj)
+	if (m_physCtrl)
 	{
-		m_sumoObj->setPosition(((KX_GameObject*)GetParent())->NodeGetWorldPosition());
-		m_sumoObj->setOrientation(
-			((KX_GameObject*)GetParent())->NodeGetWorldOrientation().getRotation()
-			);
-		m_sumoObj->calcXform();
+		MT_Vector3 pos = ((KX_GameObject*)GetParent())->NodeGetWorldPosition();
+		MT_Quaternion orn = ((KX_GameObject*)GetParent())->NodeGetWorldOrientation().getRotation();
+		m_physCtrl->setPosition(pos.x(),pos.y(),pos.z());
+		m_physCtrl->setOrientation(orn.x(),orn.y(),orn.z(),orn.w());
+		m_physCtrl->calcXform();
 	}
 	
 }
@@ -84,79 +84,96 @@ bool KX_TouchSensor::Evaluate(CValue* event)
 	return result;
 }
 
-KX_TouchSensor::KX_TouchSensor(SCA_EventManager* eventmgr,KX_GameObject* gameobj,SM_Object* sumoObj,bool bFindMaterial,const STR_String& touchedpropname,PyTypeObject* T)
+KX_TouchSensor::KX_TouchSensor(SCA_EventManager* eventmgr,KX_GameObject* gameobj,bool bFindMaterial,const STR_String& touchedpropname,PyTypeObject* T)
 :SCA_ISensor(gameobj,eventmgr,T),
 m_touchedpropname(touchedpropname),
 m_bFindMaterial(bFindMaterial),
-m_sumoObj(sumoObj),
+m_eventmgr(eventmgr),
+/*m_sumoObj(sumoObj),*/
 m_bCollision(false),
 m_bTriggered(false),
 m_bLastTriggered(false)
 {
-	m_eventmgr = eventmgr;
-	KX_TouchEventManager* touchmgr = (KX_TouchEventManager*) eventmgr;
+//	KX_TouchEventManager* touchmgr = (KX_TouchEventManager*) eventmgr;
+//	m_resptable = touchmgr->GetResponseTable();
 	
-	m_resptable = touchmgr->GetResponseTable();
-	
-	m_solidHandle = m_sumoObj->getObjectHandle();
+//	m_solidHandle = m_sumoObj->getObjectHandle();
 
 	m_hitObject =  NULL;
 	m_colliders = new CListValue();
+	
+	KX_ClientObjectInfo *client_info = gameobj->getClientInfo();
+	client_info->m_gameobject = gameobj;
+	client_info->m_auxilary_info = NULL;
+	client_info->m_sensors.push_back(this);
+	
+	m_physCtrl = dynamic_cast<PHY_IPhysicsController*>(gameobj->GetPhysicsController());
+	MT_assert( !gameobj->GetPhysicsController() || m_physCtrl );
 }
 
 
 KX_TouchSensor::~KX_TouchSensor()
 {
-	DT_ClearObjectResponse(m_resptable,m_solidHandle);
+	//DT_ClearObjectResponse(m_resptable,m_solidHandle);
 	m_colliders->Release();
 }
 
+CValue* KX_TouchSensor::GetReplica() 
+{
+	KX_TouchSensor* replica = new KX_TouchSensor(*this);
+	replica->m_colliders = new CListValue();
+	replica->m_bCollision = false;
+	replica->m_bTriggered= false;
+	replica->m_hitObject = NULL;
+	replica->m_bLastTriggered = false;
+	// this will copy properties and so on...
+	CValue::AddDataToReplica(replica);
+	return replica;
+}
+
 void	KX_TouchSensor::ReParent(SCA_IObject* parent)
 {
-
-	m_sumoObj = ((KX_GameObject*)parent)->GetSumoObject();
-	m_solidHandle = m_sumoObj->getObjectHandle();
-
-	m_client_info.m_clientobject = NULL;//parent;
-	m_client_info.m_auxilary_info = NULL;
+	KX_GameObject *gameobj = static_cast<KX_GameObject *>(parent);
+	PHY_IPhysicsController *sphy = dynamic_cast<PHY_IPhysicsController*>(((KX_GameObject*)parent)->GetPhysicsController());
+	if (sphy)
+		m_physCtrl = sphy;
+	
+//	m_solidHandle = m_sumoObj->getObjectHandle();
+	KX_ClientObjectInfo *client_info = gameobj->getClientInfo();
+	client_info->m_gameobject = gameobj;
+	client_info->m_auxilary_info = NULL;
+	
+	client_info->m_sensors.push_back(this);
 	SCA_ISensor::ReParent(parent);
 }
 
-
-void KX_TouchSensor::RegisterSumo()
+void KX_TouchSensor::RegisterSumo(KX_TouchEventManager *touchman)
 {
+	if (m_physCtrl)
+	{
+		touchman->GetPhysicsEnvironment()->requestCollisionCallback(m_physCtrl);
+		// collision
+		// Deprecated	
 
-		if (m_sumoObj)
-		{
-			// collision
-			DT_SetObjectResponse(
-				m_resptable,
-				m_solidHandle,
-				collisionResponse,
-				DT_SIMPLE_RESPONSE,
-				this);
-
-		}
-
+	}
 }
 
-void    KX_TouchSensor::HandleCollision(void* obj1,void* obj2,const DT_CollData * coll_data)
+bool	KX_TouchSensor::NewHandleCollision(void*object1,void*object2,const PHY_CollData* colldata)
 {
-	KX_TouchEventManager* toucheventmgr = (KX_TouchEventManager*)m_eventmgr;
+//	KX_TouchEventManager* toucheventmgr = (KX_TouchEventManager*)m_eventmgr;
 	KX_GameObject* parent = (KX_GameObject*)GetParent();
 
-	// need the mapping from SM_Objects to gameobjects now
+	// need the mapping from PHY_IPhysicsController to gameobjects now
 	
-	SM_ClientObjectInfo* client_info =(SM_ClientObjectInfo*) (obj1 == m_sumoObj? 
-					((SM_Object*)obj2)->getClientObject() : 
-					((SM_Object*)obj1)->getClientObject());
-
+	KX_ClientObjectInfo* client_info = static_cast<KX_ClientObjectInfo*> (object1 == m_physCtrl? 
+					((PHY_IPhysicsController*)object2)->getNewClientInfo(): 
+					((PHY_IPhysicsController*)object1)->getNewClientInfo());
 
 	KX_GameObject* gameobj = ( client_info ? 
-			(KX_GameObject*)client_info->m_clientobject : 
+			client_info->m_gameobject : 
 			NULL);
-
-	if (gameobj && (gameobj != parent))
+	
+	if (gameobj && (gameobj != parent) && client_info->isActor())
 	{
 		if (!m_colliders->SearchValue(gameobj))
 			m_colliders->Add(gameobj->AddRef());
@@ -168,14 +185,8 @@ void    KX_TouchSensor::HandleCollision(void* obj1,void* obj2,const DT_CollData
 			{
 				if (client_info->m_auxilary_info)
 				{
-					found = (m_touchedpropname == ((char*)client_info->m_auxilary_info));
+					found = (m_touchedpropname == STR_String((char*)client_info->m_auxilary_info));
 				}
-
-				if (found)
-				{
-					int i=0;
-				}
-
 			} else
 			{
 				found = (gameobj->GetProperty(m_touchedpropname) != NULL);
@@ -185,12 +196,14 @@ void    KX_TouchSensor::HandleCollision(void* obj1,void* obj2,const DT_CollData
 		{
 			m_bTriggered = true;
 			m_hitObject = gameobj;
+			//printf("KX_TouchSensor::HandleCollision\n");
 		}
 		
 	} 
-	
+	return DT_CONTINUE;
 }
 
+
 /* ------------------------------------------------------------------------- */
 /* Python functions                                                          */
 /* ------------------------------------------------------------------------- */
@@ -234,7 +247,7 @@ PyMethodDef KX_TouchSensor::Methods[] = {
 	{NULL,NULL} //Sentinel
 };
 
-PyObject* KX_TouchSensor::_getattr(char* attr) {
+PyObject* KX_TouchSensor::_getattr(const STR_String& attr) {
 	_getattr_up(SCA_ISensor);
 }
 
@@ -322,10 +335,10 @@ PyObject* KX_TouchSensor::PyGetHitObjectList(PyObject* self,
 				 * - this also doesn't work (obviously) for multi-materials... 
 				 */
 				KX_GameObject* gameob = (KX_GameObject*) m_colliders->GetValue(i);
-				SM_Object* smob = (SM_Object*) gameob->GetSumoObject();
+				PHY_IPhysicsController* spc = dynamic_cast<PHY_IPhysicsController*>(gameob->GetPhysicsController());
 				
-				if (smob) {
-					SM_ClientObjectInfo* cl_inf = (SM_ClientObjectInfo*) smob->getClientObject();
+				if (spc) {
+					KX_ClientObjectInfo* cl_inf = static_cast<KX_ClientObjectInfo*>(spc->getNewClientInfo());
 					
 					if (m_touchedpropname == ((char*)cl_inf->m_auxilary_info)) {
 						newList->Add(m_colliders->GetValue(i)->AddRef());
@@ -356,15 +369,7 @@ PyObject* KX_TouchSensor::PyGetTouchMaterial(PyObject* self,
 											 PyObject* args, 
 											 PyObject* kwds)
 {
-	int retval = 0;
-	
-	if (m_bFindMaterial) {
-		retval = KX_TRUE;
-	} else {
-		retval = KX_FALSE;
-	}
-
-	return PyInt_FromLong(retval);
+	return PyInt_FromLong(m_bFindMaterial);
 }
 
 /* 6. setTouchMaterial */
@@ -381,17 +386,10 @@ PyObject* KX_TouchSensor::PySetTouchMaterial(PyObject* self, PyObject* args, PyO
 		return NULL;
 	}
 	
-	if (pulseArg == KX_TRUE) {
-		m_bFindMaterial = true;
-	} else if (pulseArg == KX_FALSE){
-		m_bFindMaterial = false;
-	} else {
-		; /* internal error */
-	}
+	m_bFindMaterial = pulseArg != 0;
 
 	Py_Return;
 }
 
-#endif //#ifdef PHYSICS_NOT_YET
 
 /* eof */
