@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_IRasterizer.h,v 1.8 2004/04/24 06:40:14 kester Exp $
+ * $Id: RAS_IRasterizer.h,v 1.18 2006/04/11 05:57:30 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -61,7 +61,7 @@ public:
 	/**
 	 * Drawing types
 	 */
-	enum	{
+	enum DrawType {
 			KX_BOUNDINGBOX = 1,
 			KX_WIREFRAME,
 			KX_SOLID,
@@ -72,7 +72,7 @@ public:
 	/**
 	 * Valid SetDepthMask parameters
 	 */
-	enum	{
+	enum DepthMask {
 			KX_DEPTHMASK_ENABLED =1,
 			KX_DEPTHMASK_DISABLED
 	};
@@ -87,16 +87,40 @@ public:
 	/**
 	 * Stereo mode types
 	 */
-	enum	{
+	enum StereoMode {
 			RAS_STEREO_NOSTEREO = 1,
 			RAS_STEREO_QUADBUFFERED,
 			RAS_STEREO_ABOVEBELOW,
-			RAS_STEREO_INTERLACED
+			RAS_STEREO_INTERLACED,
+			RAS_STEREO_ANAGLYPH,
+			RAS_STEREO_SIDEBYSIDE,
+			RAS_STEREO_VINTERLACE,
+			
+			RAS_STEREO_MAXSTEREO
 	};
+
+	/**
+	 * Texture gen modes.
+	 */
+	enum TexCoGen {
+		RAS_TEXCO_GEN,		//< GPU will generate texture coordinates
+		RAS_TEXCO_ORCO,		//< Vertex coordinates (object space)
+		RAS_TEXCO_GLOB,		//< Vertex coordinates (world space)
+		RAS_TEXCO_UV1,		//< UV coordinates
+		RAS_TEXCO_OBJECT,	//< Use another object's position as coordinates
+		RAS_TEXCO_LAVECTOR,	//< Light vector as coordinates
+		RAS_TEXCO_VIEW,		//< View vector as coordinates
+		RAS_TEXCO_STICKY,	//< Sticky coordinates
+		RAS_TEXCO_WINDOW,	//< Window coordinates
+		RAS_TEXCO_NORM,		//< Normal coordinates 
+		RAS_TEXTANGENT,		//<
+		RAS_TEXCO_DISABLE	//< Disable this texture unit (cached)
+	};
+
 	/**
 	 * Render pass identifiers for stereo.
 	 */
-	enum	{
+	enum StereoEye {
 			RAS_STEREO_LEFTEYE = 1,
 			RAS_STEREO_RIGHTEYE
 	};
@@ -105,13 +129,13 @@ public:
 	 * SetDepthMask enables or disables writing a fragment's depth value
 	 * to the Z buffer.
 	 */
-	virtual void	SetDepthMask(int depthmask)=0;
+	virtual void	SetDepthMask(DepthMask depthmask)=0;
 	/**
 	 * SetMaterial sets the material settings for subsequent primitives
 	 * to be rendered with.
 	 * The material will be cached.
 	 */
-	virtual void	SetMaterial(const RAS_IPolyMaterial& mat)=0;
+	virtual bool	SetMaterial(const RAS_IPolyMaterial& mat)=0;
 	/**
 	 * Init initialises the renderer.
 	 */
@@ -145,7 +169,7 @@ public:
 	/**
 	 * SetStereoMode will set the stereo mode
 	 */
-	virtual void	SetStereoMode(const int stereomode)=0;
+	virtual void	SetStereoMode(const StereoMode stereomode)=0;
 	/**
 	 * Stereo can be used to query if the rasterizer is in stereo mode.
 	 * @return true if stereo mode is enabled.
@@ -154,13 +178,18 @@ public:
 	/**
 	 * Sets which eye buffer subsequent primitives will be rendered to.
 	 */
-	virtual void	SetEye(const int eye)=0;
+	virtual void	SetEye(const StereoEye eye)=0;
+	virtual StereoEye	GetEye()=0;
 	/**
+	 * Sets the distance between eyes for stereo mode.
 	 */
 	virtual void	SetEyeSeparation(const float eyeseparation)=0;
+	virtual float	GetEyeSeparation() = 0;
 	/**
+	 * Sets the focal length for stereo mode.
 	 */
 	virtual void	SetFocalLength(const float focallength)=0;
+	virtual float	GetFocalLength() = 0;
 	/**
 	 * SwapBuffers swaps the back buffer with the front buffer.
 	 */
@@ -185,7 +214,8 @@ public:
 							class RAS_IPolyMaterial* polymat,
 							class RAS_IRenderTools* rendertools,
 							bool useObjectColor,
-							const MT_Vector4& rgbacolor)=0;
+							const MT_Vector4& rgbacolor,
+							class KX_ListSlot** slot)=0;
 	/**
 	 * @copydoc IndexPrimitives
 	 * IndexPrimitivesEx will renormalize faces if @param vertexarrays[i].getFlag() & TV_CALCFACENORMAL
@@ -209,6 +239,26 @@ public:
 							bool useObjectColor,
 							const MT_Vector4& rgbacolor)=0;
 
+	virtual void IndexPrimitivesMulti( 
+						const vecVertexArray& vertexarrays,
+						const vecIndexArrays & indexarrays,
+						int mode,
+						class RAS_IPolyMaterial* polymat,
+						class RAS_IRenderTools* rendertools,
+						bool useObjectColor,
+						const MT_Vector4& rgbacolor,
+						class KX_ListSlot** slot)=0;
+
+	virtual void IndexPrimitivesMulti_Ex( 
+						const vecVertexArray& vertexarrays,
+						const vecIndexArrays & indexarrays,
+						int mode,
+						class RAS_IPolyMaterial* polymat,
+						class RAS_IRenderTools* rendertools,
+						bool useObjectColor,
+						const MT_Vector4& rgbacolor)=0;
+
+
 	virtual void	SetProjectionMatrix(MT_CmMatrix4x4 & mat)=0;
 	/* This one should become our final version, methinks. */
 	/**
@@ -216,7 +266,7 @@ public:
 	 * from camera coordinates to window coordinates.
 	 * @param mat The projection matrix.
 	 */
-	virtual void	SetProjectionMatrix(MT_Matrix4x4 & mat)=0;
+	virtual void	SetProjectionMatrix(const MT_Matrix4x4 & mat)=0;
 	/**
 	 * Sets the modelview matrix.
 	 */
@@ -299,18 +349,54 @@ public:
 		float frustfar,
 		bool perspective = true
 	)=0;
-
+	/**
+	 * Sets the specular colour component of the lighting equation.
+	 */
 	virtual void	SetSpecularity(float specX,
 								   float specY,
 								   float specZ,
 								   float specval)=0;
-
+	
+	/**
+	 * Sets the specular exponent component of the lighting equation.
+	 */
 	virtual void	SetShinyness(float shiny)=0;
+	/**
+	 * Sets the diffuse colour component of the lighting equation.
+	 */
 	virtual void	SetDiffuse(float difX,
 							   float difY,
 							   float difZ,
 							   float diffuse)=0;
+	/**
+	 * Sets the emissive colour component of the lighting equation.
+	 */ 
+	virtual void	SetEmissive(float eX,
+								float eY,
+								float eZ,
+								float e
+							   )=0;
 	
+	virtual void	SetAmbientColor(float red, float green, float blue)=0;
+	virtual void	SetAmbient(float factor)=0;
+	/**
+	 * Sets alpha testing
+	 */
+	virtual void	SetAlphaTest(bool enable)=0;
+
+	/**
+	 * Sets a polygon offset.  z depth will be: z1 = mult*z0 + add
+	 */
+	virtual void	SetPolygonOffset(float mult, float add) = 0;
+	
+	virtual	void	DrawDebugLine(const MT_Vector3& from,const MT_Vector3& to,const MT_Vector3& color)=0;
+
+	virtual void	SetTexCoords(TexCoGen coords, int unit) = 0;
+	virtual void	SetAttrib(int type) = 0;
+	virtual void	GetViewMatrix(MT_Matrix4x4 &mat) const = 0;
+
+	virtual bool	QueryLists(){return false;}
+	virtual bool	QueryArrays(){return false;}
 };
 
 #endif //__RAS_IRASTERIZER
