@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_MeshObject.h,v 1.13 2004/11/22 10:19:19 kester Exp $
+ * $Id: RAS_MeshObject.h,v 1.15 2006/04/02 21:04:20 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -132,8 +132,9 @@ class RAS_MeshObject
 	vector<class RAS_Polygon*> 	m_Polygons;
 	STR_String					m_name;
 	static STR_String			s_emptyname;
-	bool					m_zsort;
-	
+	bool						m_zsort;
+	bool						m_MeshMod;
+
 	struct polygonSlot;
 	struct backtofront;
 	struct fronttoback;
@@ -236,6 +237,7 @@ public:
 							int vtxarray,
 							const MT_Point3& xyz,
 							const MT_Point2& uv,
+							const MT_Vector4& tangent,
 							const unsigned int rgbacolor,
 							const MT_Vector3& normal,
 							RAS_IPolyMaterial* mat,
@@ -254,6 +256,10 @@ public:
 	const vecIndexArrays& GetIndexCache (RAS_IPolyMaterial* mat);
 	void				SetName(STR_String name);
 	const STR_String&	GetName();
+
+	bool				MeshModified();
+	void				SetMeshModified(bool v){m_MeshMod = v;}
+
 };
 
 #endif //__RAS_MESHOBJECT
