@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_OpenGLRasterizer.cpp,v 1.16 2005/01/16 06:02:06 kester Exp $
+ * $Id: RAS_OpenGLRasterizer.cpp,v 1.26 2006/04/11 05:57:30 erwin Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -41,6 +41,7 @@
 #include <windows.h>
 #endif // WIN32
 #ifdef __APPLE__
+#define GL_GLEXT_LEGACY 1
 #include <OpenGL/gl.h>
 #include <OpenGL/glu.h>
 #else
@@ -80,6 +81,7 @@ RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas)
 	m_focallength(0.0),
 	m_setfocallength(false),
 	m_noOfScanlines(32),
+	m_useTang(false),
 	m_materialCachingInfo(0)
 {
 	m_viewmatrix.Identity();
@@ -177,6 +179,10 @@ bool RAS_OpenGLRasterizer::Init()
 	m_blueback = 0.4375;
 	m_alphaback = 0.0;
 	
+	m_ambr = 0.0f;
+	m_ambg = 0.0f;
+	m_ambb = 0.0f;
+
 	glClearColor(m_redback,m_greenback,m_blueback,m_alphaback);
 	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -187,6 +193,32 @@ bool RAS_OpenGLRasterizer::Init()
 }
 
 
+void RAS_OpenGLRasterizer::SetAmbientColor(float red, float green, float blue)
+{
+	m_ambr = red;
+	m_ambg = green;
+	m_ambb = blue;
+}
+
+
+void RAS_OpenGLRasterizer::SetAlphaTest(bool enable)
+{
+	if (enable)
+	{
+		glEnable(GL_ALPHA_TEST);
+		glAlphaFunc(GL_GREATER, 0.6f);
+	}
+	else glDisable(GL_ALPHA_TEST);
+}
+
+
+
+void RAS_OpenGLRasterizer::SetAmbient(float factor)
+{
+	float ambient[] = { m_ambr*factor, m_ambg*factor, m_ambb*factor, 1.0f };
+	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
+}
+
 
 void RAS_OpenGLRasterizer::SetBackColor(float red,
 										float green,
@@ -401,6 +433,26 @@ void RAS_OpenGLRasterizer::ClearCachingInfo(void)
 
 void RAS_OpenGLRasterizer::EndFrame()
 {
+	glDisable(GL_LIGHTING);
+	glDisable(GL_TEXTURE_2D);
+
+	//DrawDebugLines
+	glBegin(GL_LINES);
+	for (unsigned int i=0;i<m_debugLines.size();i++)
+	{
+		
+
+		glColor4f(m_debugLines[i].m_color[0],m_debugLines[i].m_color[1],m_debugLines[i].m_color[2],1.f);
+		const MT_Scalar* fromPtr = &m_debugLines[i].m_from.x();
+		const MT_Scalar* toPtr= &m_debugLines[i].m_to.x();
+
+		glVertex3dv(fromPtr);
+		glVertex3dv(toPtr);
+	}
+	glEnd();
+
+	m_debugLines.clear();
+
 	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
 	m_2DCanvas->EndFrame();
 }	
@@ -552,13 +604,23 @@ void RAS_OpenGLRasterizer::SwapBuffers()
 
 
 
+void RAS_OpenGLRasterizer::GetViewMatrix(MT_Matrix4x4 &mat) const
+{
+	float viewmat[16];
+	glGetFloatv(GL_MODELVIEW_MATRIX, viewmat);
+	mat.setValue(viewmat);
+}
+
+
+
 void RAS_OpenGLRasterizer::IndexPrimitives(const vecVertexArray & vertexarrays,
 									const vecIndexArrays & indexarrays,
 									int mode,
 									class RAS_IPolyMaterial* polymat,
 									class RAS_IRenderTools* rendertools,
 									bool useObjectColor,
-									const MT_Vector4& rgbacolor
+									const MT_Vector4& rgbacolor,
+									class KX_ListSlot** slot
 									)
 { 
 	GLenum drawmode;
@@ -1140,6 +1202,569 @@ void RAS_OpenGLRasterizer::IndexPrimitives_3DText(const vecVertexArray & vertexa
 	}	//for each vertexarray
 }
 
+void RAS_OpenGLRasterizer::SetTexCoords(TexCoGen coords,int unit)
+{
+	// this changes from material to material
+	if(unit <= RAS_MAX)
+		m_texco[unit] = coords;
+}
+
+void RAS_OpenGLRasterizer::SetAttrib(int type)
+{
+	if(type == RAS_TEXTANGENT) m_useTang=true;
+}
+
+void RAS_OpenGLRasterizer::TexCoord(const RAS_TexVert &tv, int enabled)
+{
+#ifdef GL_ARB_multitexture
+	if(bgl::RAS_EXT_support._ARB_multitexture)
+	{
+		for(int unit=0; unit<enabled; unit++)
+		{
+			if( tv.getFlag() & TV_2NDUV && tv.getUnit() == unit ) {
+				bgl::blMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV2());
+				continue;
+			}
+			switch(m_texco[unit])
+			{
+			case RAS_TEXCO_DISABLE:
+			case RAS_TEXCO_OBJECT:
+			case RAS_TEXCO_GEN:
+				break;
+			case RAS_TEXCO_ORCO:
+			case RAS_TEXCO_GLOB:
+				bgl::blMultiTexCoord3fvARB(GL_TEXTURE0_ARB+unit, tv.getLocalXYZ());
+				break;
+			case RAS_TEXCO_UV1:
+				bgl::blMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV1());
+				break;
+			case RAS_TEXCO_NORM:
+				bgl::blMultiTexCoord3fvARB(GL_TEXTURE0_ARB+unit, tv.getNormal());
+				break;
+			case RAS_TEXTANGENT:
+				bgl::blMultiTexCoord4fvARB(GL_TEXTURE0_ARB+unit, tv.getTangent());
+			}
+		}
+	}
+#endif
+
+#ifdef GL_ARB_vertex_program
+	if(m_useTang && bgl::RAS_EXT_support._ARB_vertex_program)
+		bgl::blVertexAttrib4fvARB(1/*tangent*/, tv.getTangent());
+#endif
+
+}
+void RAS_OpenGLRasterizer::Tangent(	const RAS_TexVert& v1,
+									const RAS_TexVert& v2,
+									const RAS_TexVert& v3,
+									const MT_Vector3 &no)
+{
+#ifdef GL_ARB_multitexture
+	// TODO: set for deformer... 
+	MT_Vector3 x1(v1.getLocalXYZ()), x2(v2.getLocalXYZ()), x3(v3.getLocalXYZ());
+	MT_Vector2 uv1(v1.getUV1()), uv2(v2.getUV1()), uv3(v3.getUV1());
+	MT_Vector3 dx1(x2 - x1), dx2(x3 - x1);
+	MT_Vector2 duv1(uv2 - uv1), duv2(uv3 - uv1);
+	
+	MT_Scalar r = 1.0 / (duv1.x() * duv2.y() - duv2.x() * duv1.y());
+	duv1 *= r;
+	duv2 *= r;
+	MT_Vector3 sdir(duv2.y() * dx1 - duv1.y() * dx2);
+	MT_Vector3 tdir(duv1.x() * dx2 - duv2.x() * dx1);
+	
+	// Gram-Schmidt orthogonalize
+	MT_Vector3 t(sdir - no.cross(no.cross(sdir)));
+	if (!MT_fuzzyZero(t))
+		t /= t.length();
+
+	float tangent[4];
+	t.getValue(tangent);
+	// Calculate handedness
+	tangent[3] = no.dot(sdir.cross(tdir)) < 0.0 ? -1.0 : 1.0;
+#endif
+}
+
+
+void RAS_OpenGLRasterizer::IndexPrimitivesMulti(
+		const vecVertexArray& vertexarrays,
+		const vecIndexArrays & indexarrays,
+		int mode,
+		class RAS_IPolyMaterial* polymat,
+		class RAS_IRenderTools* rendertools,
+		bool useObjectColor,
+		const MT_Vector4& rgbacolor,
+		class KX_ListSlot** slot
+		)
+{ 
+#ifdef GL_ARB_multitexture
+
+	GLenum drawmode;
+	switch (mode)
+	{
+	case 0:
+		drawmode = GL_TRIANGLES;
+		break;
+	case 1:
+		drawmode = GL_LINES;
+		break;
+	case 2:
+		drawmode = GL_QUADS;
+		break;
+	default:
+		drawmode = GL_LINES;
+		break;
+	}
+	
+	const RAS_TexVert* vertexarray ;
+	unsigned int numindices,vt;
+
+	for (vt=0;vt<vertexarrays.size();vt++)
+	{
+		vertexarray = &((*vertexarrays[vt]) [0]);
+		const KX_IndexArray & indexarray = (*indexarrays[vt]);
+		numindices = indexarray.size();
+		const unsigned int enabled = polymat->GetEnabled();
+
+		if (!numindices)
+			break;
+		
+		int vindex=0;
+		switch (mode)
+		{
+		case 1:
+			{
+				glBegin(GL_LINES);
+				vindex=0;
+				for (unsigned int i=0;i<numindices;i+=2)
+				{
+					glVertex3fv(vertexarray[(indexarray[vindex++])].getLocalXYZ());
+					glVertex3fv(vertexarray[(indexarray[vindex++])].getLocalXYZ());
+				}
+				glEnd();
+			}
+			break;
+		case 2:
+			{
+				glBegin(GL_QUADS);
+				vindex=0;
+				if (useObjectColor)
+				{
+					for (unsigned int i=0;i<numindices;i+=4)
+					{
+
+						glColor4d(rgbacolor[0], rgbacolor[1], rgbacolor[2], rgbacolor[3]);
+						
+						//
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				else
+				{
+					for (unsigned int i=0;i<numindices;i+=4)
+					{
+						// This looks curiously endian unsafe to me.
+						// However it depends on the way the colors are packed into 
+						// the m_rgba field of RAS_TexVert
+						
+						//
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+					
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				glEnd();	
+				break;
+			}
+		case 0:
+			{
+				glBegin(GL_TRIANGLES);
+				vindex=0;
+				if (useObjectColor)
+				{
+					for (unsigned int i=0;i<numindices;i+=3)
+					{
+
+						glColor4d(rgbacolor[0], rgbacolor[1], rgbacolor[2], rgbacolor[3]);
+						//
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				else 
+				{
+					for (unsigned int i=0;i<numindices;i+=3)
+					{
+						//
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						//
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				glEnd();	
+				break;
+			}
+		default:
+			{
+			}
+		} // switch
+	} // for each vertexarray
+#endif// GL_ARB_multitexture
+
+}
+
+void RAS_OpenGLRasterizer::IndexPrimitivesMulti_Ex(
+									const vecVertexArray & vertexarrays,
+									const vecIndexArrays & indexarrays,
+									int mode,
+									class RAS_IPolyMaterial* polymat,
+									class RAS_IRenderTools* rendertools,
+									bool useObjectColor,
+									const MT_Vector4& rgbacolor)
+{ 
+#ifdef GL_ARB_multitexture
+	bool	recalc;
+	GLenum drawmode;
+	switch (mode)
+	{
+	case 0:
+		drawmode = GL_TRIANGLES;
+		break;
+	case 1:
+		drawmode = GL_LINES;
+		break;
+	case 2:
+		drawmode = GL_QUADS;
+		break;
+	default:
+		drawmode = GL_LINES;
+		break;
+	}
+	
+	const RAS_TexVert* vertexarray ;
+	unsigned int numindices,vt;
+
+	for (vt=0;vt<vertexarrays.size();vt++)
+	{
+		vertexarray = &((*vertexarrays[vt]) [0]);
+		const KX_IndexArray & indexarray = (*indexarrays[vt]);
+		numindices = indexarray.size();
+		const unsigned int enabled = polymat->GetEnabled();
+		
+		if (!numindices)
+			continue;
+		
+		int vindex=0;
+		switch (mode)
+		{
+		case 1:
+			{
+				glBegin(GL_LINES);
+				vindex=0;
+				for (unsigned int i=0;i<numindices;i+=2)
+				{
+					glVertex3fv(vertexarray[(indexarray[vindex++])].getLocalXYZ());
+					glVertex3fv(vertexarray[(indexarray[vindex++])].getLocalXYZ());
+				}
+				glEnd();
+			}
+			break;
+		case 2:
+			{
+				glBegin(GL_QUADS);
+				vindex=0;
+				if (useObjectColor)
+				{
+					for (unsigned int i=0;i<numindices;i+=4)
+					{
+						MT_Point3 mv1, mv2, mv3, mv4, fnor;
+						/* Calc a new face normal */
+
+						if (vertexarray[(indexarray[vindex])].getFlag() & TV_CALCFACENORMAL)
+							recalc= true;
+						else
+							recalc=false;
+
+						if (recalc){
+							mv1 = MT_Point3(vertexarray[(indexarray[vindex])].getLocalXYZ());
+							mv2 = MT_Point3(vertexarray[(indexarray[vindex+1])].getLocalXYZ());
+							mv3 = MT_Point3(vertexarray[(indexarray[vindex+2])].getLocalXYZ());
+							mv4 = MT_Point3(vertexarray[(indexarray[vindex+2])].getLocalXYZ());
+							
+							fnor = (((mv2-mv1).cross(mv3-mv2))+((mv4-mv3).cross(mv1-mv4))).safe_normalized();
+
+							glNormal3f(fnor[0], fnor[1], fnor[2]);
+						}
+
+						glColor4d(rgbacolor[0], rgbacolor[1], rgbacolor[2], rgbacolor[3]);
+
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+					
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+					
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());						
+					
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				else
+				{
+					for (unsigned int i=0;i<numindices;i+=4)
+					{
+						// This looks curiously endian unsafe to me.
+						// However it depends on the way the colors are packed into 
+						// the m_rgba field of RAS_TexVert
+						MT_Point3 mv1, mv2, mv3, mv4, fnor;
+						/* Calc a new face normal */
+
+						if (vertexarray[(indexarray[vindex])].getFlag() & TV_CALCFACENORMAL)
+							recalc= true;
+						else
+							recalc=false;
+
+
+						if (recalc){
+							mv1 = MT_Point3(vertexarray[(indexarray[vindex])].getLocalXYZ());
+							mv2 = MT_Point3(vertexarray[(indexarray[vindex+1])].getLocalXYZ());
+							mv3 = MT_Point3(vertexarray[(indexarray[vindex+2])].getLocalXYZ());
+							mv4 = MT_Point3(vertexarray[(indexarray[vindex+3])].getLocalXYZ());
+							
+							fnor = (((mv2-mv1).cross(mv3-mv2))+((mv4-mv3).cross(mv1-mv4))).safe_normalized();
+
+							glNormal3f(fnor[0], fnor[1], fnor[2]);
+						}
+
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+					
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+					
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+					
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				glEnd();	
+				break;
+			}
+		case 0:
+			{
+				glBegin(GL_TRIANGLES);
+				vindex=0;
+				if (useObjectColor)
+				{
+					for (unsigned int i=0;i<numindices;i+=3)
+					{
+						MT_Point3 mv1, mv2, mv3, fnor;
+						/* Calc a new face normal */
+
+						if (vertexarray[(indexarray[vindex])].getFlag() & TV_CALCFACENORMAL)
+							recalc= true;
+						else
+							recalc=false;
+
+						if (recalc){
+							mv1 = MT_Point3(vertexarray[(indexarray[vindex])].getLocalXYZ());
+							mv2 = MT_Point3(vertexarray[(indexarray[vindex+1])].getLocalXYZ());
+							mv3 = MT_Point3(vertexarray[(indexarray[vindex+2])].getLocalXYZ());
+							
+							fnor = ((mv2-mv1).cross(mv3-mv2)).safe_normalized();
+							glNormal3f(fnor[0], fnor[1], fnor[2]);
+						}
+
+						glColor4d(rgbacolor[0], rgbacolor[1], rgbacolor[2], rgbacolor[3]);
+
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				else 
+				{
+					for (unsigned int i=0;i<numindices;i+=3)
+					{
+						MT_Point3 mv1, mv2, mv3, fnor;
+						/* Calc a new face normal */
+
+						if (vertexarray[(indexarray[vindex])].getFlag() & TV_CALCFACENORMAL)
+							recalc= true;
+						else
+							recalc=false;
+
+
+						if (recalc){
+							mv1 = MT_Point3(vertexarray[(indexarray[vindex])].getLocalXYZ());
+							mv2 = MT_Point3(vertexarray[(indexarray[vindex+1])].getLocalXYZ());
+							mv3 = MT_Point3(vertexarray[(indexarray[vindex+2])].getLocalXYZ());
+							
+							fnor = ((mv2-mv1).cross(mv3-mv2)).safe_normalized();
+							glNormal3f(fnor[0], fnor[1], fnor[2]);
+						}
+
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+						
+						glColor4ubv((const GLubyte *)(vertexarray[(indexarray[vindex])].getRGBA()));
+						if (!recalc)
+							glNormal3fv(vertexarray[(indexarray[vindex])].getNormal());
+						TexCoord(vertexarray[(indexarray[vindex])],enabled );
+						glVertex3fv(vertexarray[(indexarray[vindex])].getLocalXYZ());
+						vindex++;
+					}
+				}
+				glEnd();	
+				break;
+			}
+		default:
+			{
+			}
+			
+		} // switch
+	} // for each vertexarray
+#endif
+}
+
 
 
 void RAS_OpenGLRasterizer::SetProjectionMatrix(MT_CmMatrix4x4 &mat)
@@ -1327,6 +1952,13 @@ void RAS_OpenGLRasterizer::SetDiffuse(float difX,float difY,float difZ,float dif
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
 }
 
+void RAS_OpenGLRasterizer::SetEmissive(float eX, float eY, float eZ, float e)
+{
+	GLfloat mat_emit [] = {eX,eY,eZ,e};
+	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, mat_emit);
+}
+
+
 double RAS_OpenGLRasterizer::GetTime()
 {
 	return m_time;
