@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_OpenGLRasterizer.h,v 1.12 2005/01/16 06:02:06 kester Exp $
+ * $Id: RAS_OpenGLRasterizer.h,v 1.17 2006/04/11 05:57:30 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,6 +44,15 @@ using namespace std;
 #include "RAS_MaterialBucket.h"
 #include "RAS_ICanvas.h"
 
+#define RAS_MAX 3// match in BL_Material
+
+struct	OglDebugLine
+{
+	MT_Vector3	m_from;
+	MT_Vector3	m_to;
+	MT_Vector3	m_color;
+};
+
 /**
  * 3D rendering device context.
  */
@@ -64,6 +73,10 @@ class RAS_OpenGLRasterizer : public RAS_IRasterizer
 	float			m_greenback;
 	float			m_blueback;
 	float			m_alphaback;
+	
+	float			m_ambr;
+	float			m_ambg;
+	float			m_ambb;
 
 	double			m_time;
 	MT_CmMatrix4x4	m_viewmatrix;
@@ -76,11 +89,13 @@ class RAS_OpenGLRasterizer : public RAS_IRasterizer
 	float			m_focallength;
 	bool			m_setfocallength;
 	int				m_noOfScanlines;
-	
 	bool			InterlacedStereo() const;
 
 protected:
 	int				m_drawingmode;
+	TexCoGen		m_texco[RAS_MAX];
+	bool			m_useTang;
+
 	/** Stores the caching information for the last material activated. */
 	RAS_IPolyMaterial::TCachingInfo m_materialCachingInfo;
 
@@ -123,6 +138,8 @@ public:
 	virtual void	SetFocalLength(const float focallength);
 	virtual float	GetFocalLength();
 
+	virtual void	SetAlphaTest(bool enable);
+
 	virtual void	SwapBuffers();
 	virtual void	IndexPrimitives(
 						const vecVertexArray& vertexarrays,
@@ -131,7 +148,8 @@ public:
 						class RAS_IPolyMaterial* polymat,
 						class RAS_IRenderTools* rendertools,
 						bool useObjectColor,
-						const MT_Vector4& rgbacolor
+						const MT_Vector4& rgbacolor,
+						class KX_ListSlot** slot
 					);
 
 	virtual void	IndexPrimitives_Ex(
@@ -154,6 +172,26 @@ public:
 						const MT_Vector4& rgbacolor
 					);
 
+	virtual void IndexPrimitivesMulti( 
+						const vecVertexArray& vertexarrays,
+						const vecIndexArrays & indexarrays,
+						int mode,
+						class RAS_IPolyMaterial* polymat,
+						class RAS_IRenderTools* rendertools,
+						bool useObjectColor,
+						const MT_Vector4& rgbacolor,
+						class KX_ListSlot** slot);
+
+	virtual void IndexPrimitivesMulti_Ex( 
+						const vecVertexArray& vertexarrays,
+						const vecIndexArrays & indexarrays,
+						int mode,
+						class RAS_IPolyMaterial* polymat,
+						class RAS_IRenderTools* rendertools,
+						bool useObjectColor,
+						const MT_Vector4& rgbacolor);
+
+
 	virtual void	SetProjectionMatrix(MT_CmMatrix4x4 & mat);
 	virtual void	SetProjectionMatrix(const MT_Matrix4x4 & mat);
 	virtual void	SetViewMatrix(
@@ -224,8 +262,38 @@ public:
 						float difZ,
 						float diffuse
 					);
+	virtual void	SetEmissive(float eX,
+								float eY,
+								float eZ,
+								float e
+							   );
+
+	virtual void	SetAmbientColor(float red, float green, float blue);
+	virtual void	SetAmbient(float factor);
+
 	virtual void	SetPolygonOffset(float mult, float add);
 
+	virtual	void	DrawDebugLine(const MT_Vector3& from,const MT_Vector3& to,const MT_Vector3& color)
+	{
+		OglDebugLine line;
+		line.m_from = from;
+		line.m_to = to;
+		line.m_color = color;
+		m_debugLines.push_back(line);
+	}
+
+	std::vector <OglDebugLine>	m_debugLines;
+
+	virtual void	SetTexCoords(TexCoGen coords,int enabled);
+	virtual void	SetAttrib(int type);
+	void			TexCoord(const RAS_TexVert &tv, int unit);
+	virtual void	GetViewMatrix(MT_Matrix4x4 &mat) const;
+
+	void	Tangent(const RAS_TexVert& v1,
+					const RAS_TexVert& v2,
+					const RAS_TexVert& v3,
+					const MT_Vector3 &no);
+
 };
 
 #endif //__RAS_OPENGLRASTERIZER
