@@ -1,5 +1,5 @@
 /**
- * $Id: RAS_TexVert.h,v 1.6 2004/03/22 22:02:13 jesterking Exp $
+ * $Id: RAS_TexVert.h,v 1.12 2006/02/13 05:45:31 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,28 +40,40 @@
 static MT_Point3 g_pt3;
 static MT_Point2 g_pt2;
 
-#define TV_CALCFACENORMAL 0x0001
+#define TV_CALCFACENORMAL	0x0001
+#define TV_2NDUV			0x0002
+
+#define TV_MAX				3//match Def in BL_Material.h
 
 #define RAS_TexVert_INLINE 1
 
 class RAS_TexVert
 {
 	
-	float			m_localxyz[3];	// 3*4 = 12 = 24
-	float			m_uv1[2];	// 2*4 =  8 = 24 + 16 = 40
-	unsigned int	m_rgba;			// 4   = 40 + 4 = 44
-	short			m_normal[3];	// 3*2 =  6 = 50 
-	short			m_flag;		// 32 bytes total size, fits nice = 52 = not fit nice
-
+	float			m_localxyz[3];	// 3*4 = 12
+	float			m_uv1[2];		// 2*4 =  8
+	float			m_uv2[2];		// 2*4 =  8	... python access to this set
+	unsigned int	m_rgba;			//        4
+	float			m_tangent[4];   // 4*2 =  8
+	float			m_normal[3];	// 3*2 =  6 
+	short			m_flag;			//        2
+	unsigned int	m_unit;			//		  4
+									//---------
+									//       52
+	//32 bytes total size, fits nice = 52 = not fit nice.
+	// We'll go for 64 bytes total size - 24 bytes left.
 
 public:
 	short getFlag() const;
+	unsigned int getUnit() const;
+	
 	RAS_TexVert()// :m_xyz(0,0,0),m_uv(0,0),m_rgba(0)
 	{}
 	RAS_TexVert(const MT_Point3& xyz,
 				const MT_Point2& uv,
+				const MT_Vector4& tangent,
 				const unsigned int rgba,
-				const short *normal,
+				const MT_Vector3& normal,
 				const short flag);
 	~RAS_TexVert() {};
 
@@ -70,33 +82,45 @@ public:
 	const float* getUV1 () const { 
 		return m_uv1;
 	};
-	
+
+	const float* getUV2 () const { 
+		return m_uv2;
+	};
+
 	const float* getLocalXYZ() const { 
 		return m_localxyz;
 	};
 	
-	const short* getNormal() const {
+	const float* getNormal() const {
 		return m_normal;
 	}
 	
-	const unsigned int& getRGBA() const {
-		return m_rgba;
+	const float* getTangent() const {
+		return m_tangent;
+	}
+
+	const unsigned char* getRGBA() const {
+		return (unsigned char *) &m_rgba;
 	}
 #else
 	const float* getUV1 () const;
-	const short*		getNormal() const;
+	const float* getUV2 () const;
+	const float*		getNormal() const;
 	const float*		getLocalXYZ() const;
-	const unsigned int&	getRGBA() const;
+	const unsigned char*	getRGBA() const;
 #endif
-
-	const MT_Point3&	xyz();
-
 	void				SetXYZ(const MT_Point3& xyz);
 	void				SetUV(const MT_Point2& uv);
+	void				SetUV2(const MT_Point2& uv);
+
 	void				SetRGBA(const unsigned int rgba);
 	void				SetNormal(const MT_Vector3& normal);
 	void				SetFlag(const short flag);
+	void				SetUnit(const unsigned u);
 	
+	void				SetRGBA(const MT_Vector4& rgba);
+	const MT_Point3&	xyz();
+
 	// compare two vertices, and return TRUE if both are almost identical (they can be shared)
 	bool				closeTo(const RAS_TexVert* other);
 
